@@ -58,7 +58,6 @@ extern char *rl_gets (char **line_read, const char *prompt);
 extern void initialize_readline (void);
 #endif /* HAVE_READLINE */
 
-char prefix[MAXLEN];
 char runfile[MAXLEN];
 char cmdfile[MAXLEN];
 char datfile[MAXLEN];
@@ -73,12 +72,11 @@ PATHS paths;                  /* useful paths */
 PRN *cmdprn;
 MODELSPEC *modelspec;
 MODEL tmpmod;
-FILE *dat, *fb;
-int i, j, dot, opt, errfatal, batch;
-int runit, loopstack, looprun;
+FILE *fb;
+int errfatal, batch;
+int runit;
 int data_status;
 int plot_count;             /* graphs via gnuplot */
-int order;                  /* VAR lag order */
 int lines[1];               /* for gnuplot command */
 char *line;                 /* non-Readline command line */
 char texfile[MAXLEN];
@@ -89,7 +87,7 @@ char *line_read;
 gretl_equation_system *sys;
 gretl_restriction_set *rset;
 
-static int exec_line (char *line, LOOPSET **ploop, PRN *prn); 
+static int exec_line (char *line, PRN *prn); 
 static int push_input_file (FILE *fp);
 static FILE *pop_input_file (void);
 static int saved_object_action (const char *line, PRN *prn);
@@ -215,7 +213,7 @@ int fn_get_line (void)
     int err = 0;
 
     clear(line, MAXLINE);
-    gretl_function_get_line(line, MAXLINE, &Z, datainfo, &err);
+    gretl_function_get_line(line, MAXLINE, &Z, &datainfo, &err);
 
     if (*line != '\0') {
 	*linebak = 0;
@@ -297,7 +295,7 @@ static int clear_data (void)
 
     *paths.datfile = 0;
 
-    err = restore_full_sample(&Z, &datainfo, OPT_C); 
+    err = restore_full_sample(&Z, &datainfo); 
 
     if (Z != NULL) {
 	free_Z(Z, datainfo); 
@@ -323,8 +321,17 @@ static int clear_data (void)
     return err;
 }
 
+static void get_a_filename (char *fname)
+{
+    *fname = 0;
+
+    fgets(fname, MAXLEN - 1, stdin);
+}
+
 static int get_an_input_line (void)
 {
+    int compiling = gretl_compiling_function() || 
+	gretl_compiling_loop();
     int err = 0;
 
     if (gretl_executing_function()) {
@@ -336,18 +343,16 @@ static int get_an_input_line (void)
     } else {
 	/* normal interactive use */
 #ifdef HAVE_READLINE
-	rl_gets(&line_read, (loopstack || gretl_compiling_function())? 
-		"> " : "? ");
+	rl_gets(&line_read, (compiling)? "> " : "? ");
 	if (line_read == NULL) {
 	    strcpy(line, "quit");
 	} else {
 	    strcpy(line, line_read);
 	}
 #else
-	printf("%s", (loopstack || gretl_compiling_function())? 
-	       "> " : "? ");
+	printf("%s", (compiling)? "> " : "? ");
 	fflush(stdout);
-	file_get_line(); /* "file" = stdin here */
+	file_get_line(); /* note: "file" = stdin here */
 #endif
     }
 
@@ -367,7 +372,7 @@ static int maybe_get_input_line_continuation (char *tmp)
 	tmp[0] = '\0';
 
 	if (gretl_executing_function()) {
-	    gretl_function_get_line(tmp, MAXLINE - 1, &Z, datainfo, &err);
+	    gretl_function_get_line(tmp, MAXLINE - 1, &Z, &datainfo, &err);
 	} else if (batch || runit) {
 	    fgets(tmp, MAXLINE - 1, fb);
 	} else {
@@ -420,7 +425,6 @@ int main (int argc, char *argv[])
     int cmd_overflow = 0;
     char filearg[MAXLEN];
     char tmp[MAXLINE];
-    LOOPSET *loop = NULL;
     PRN *prn;
     int err = 0;
 
@@ -522,40 +526,58 @@ int main (int argc, char *argv[])
 
     if (!cli_get_data) {
 	char given_file[MAXLEN];
+	int ftype;
 
 	strcpy(given_file, filearg);
 	strcpy(paths.datfile, filearg);
 
-	err = detect_filetype(paths.datfile, &paths, prn);
+	ftype = detect_filetype(paths.datfile, &paths, prn);
 
-	if (err == GRETL_UNRECOGNIZED || err == GRETL_NATIVE_DB ||
-	    err == GRETL_RATS_DB) { 
+	switch (ftype) {
+	case GRETL_UNRECOGNIZED:
+	case GRETL_NATIVE_DB:
+	case GRETL_RATS_DB:
 	    exit(EXIT_FAILURE);
-	}
-
-	if (err == GRETL_NATIVE_DATA) {
+	    break;
+	case GRETL_NATIVE_DATA:
 	    err = gretl_get_data(&Z, &datainfo, paths.datfile, &paths, 
 				 DATA_NONE, prn);
-	} else if (err == GRETL_XML_DATA) {
+	    break;
+	case GRETL_XML_DATA:
 	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, 
 				 DATA_NONE, prn, 0);
-	} else if (err == GRETL_CSV_DATA) {
+	    break;
+	case GRETL_CSV_DATA:
 	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
-	} else if (err == GRETL_OCTAVE) {
+	    break;
+	case GRETL_OCTAVE:
 	    err = import_octave(&Z, &datainfo, paths.datfile, prn);
-	} else if (err == GRETL_BOX_DATA) {
+	    break;
+	case GRETL_BOX_DATA:
 	    err = import_box(&Z, &datainfo, paths.datfile, prn);
-	} else if (err == GRETL_SCRIPT) { /* maybe it's a script file? */
+	    break;
+	case GRETL_GNUMERIC:
+	case GRETL_EXCEL:
+	case GRETL_WF1:
+	case GRETL_DTA:
+	    err = import_other(&Z, &datainfo, ftype, paths.datfile, prn);
+	    break;
+	case GRETL_SCRIPT:
 	    runit = 1;
 	    strcpy(runfile, paths.datfile); 
 	    clear(paths.datfile, MAXLEN);
 	    cli_get_data = 1;
+	    break;
+	default:
+	    break;
 	}
 
 	if (!cli_get_data) {
 	    if (err) {
 		errmsg(err, prn);
-		if (err == E_FOPEN) show_paths(&paths);
+		if (err == E_FOPEN) {
+		    show_paths(&paths);
+		}
 		return EXIT_FAILURE;
 	    }
 	    data_status = 1;
@@ -585,10 +607,11 @@ int main (int argc, char *argv[])
 
     /* check for help file */
     if (!batch) {
-	dat = fopen(paths.helpfile, "r");
-	if (dat != NULL) { 
+	FILE *fp = fopen(paths.helpfile, "r");
+
+	if (fp != NULL) { 
 	    printf(_("\n\"help\" gives a list of commands\n"));
-	    fclose(dat);
+	    fclose(fp);
 	} else {
 	    printf(_("help file %s is not accessible\n"), 
 		   paths.helpfile);
@@ -613,7 +636,7 @@ int main (int argc, char *argv[])
 	/* re-initialize: will be incremented by "run" cmd */
 	runit = 0;
 	sprintf(line, "run %s", runfile);
-	exec_line(line, &loop, prn);
+	exec_line(line, prn);
     }
 
     /* should we stop immediately on error, in batch mode? */
@@ -627,13 +650,10 @@ int main (int argc, char *argv[])
 	    gretl_abort(linecopy);
 	}
 
-	if (looprun) { 
-	    if (loop_exec(loop, line, &Z, &datainfo, models, prn)) {
+	if (gretl_execute_loop()) { 
+	    if (gretl_loop_exec(line, &Z, &datainfo, models, prn)) {
 		return 1;
 	    }
-	    gretl_loop_destroy(loop);
-	    loop = NULL;
-	    looprun = 0;
 	    set_errfatal(ERRFATAL_AUTO);
 	} else {
 	    err = get_an_input_line();
@@ -650,7 +670,7 @@ int main (int argc, char *argv[])
 	    break;
 	} else {
 	    strcpy(linecopy, line);
-	    err = exec_line(line, &loop, prn);
+	    err = exec_line(line, prn);
 	}
     } /* end of get commands loop */
 
@@ -679,7 +699,7 @@ int main (int argc, char *argv[])
     return 0;
 }
 
-static void printf_strip (char *s, int loopstack)
+static void printf_strip (char *s)
 {
     int i, n;
 
@@ -692,47 +712,33 @@ static void printf_strip (char *s, int loopstack)
 	else break;
     }
 
-    if (loopstack) {
+    if (gretl_compiling_loop()) {
 	printf("> %s\n", s);
     } else {
 	printf("%s\n", s);
     }
 }
 
-static int handle_user_defined_function (char *line, int *fncall,
-					 PRN *prn)
-{
-    int ufunc = gretl_is_user_function(line);
-    int err = 0;
-
-    /* allow for nested function calls */
-    if (ufunc && gretl_compiling_function()) {
-	return 0;
-    }
-
-    /* an actual function call */
-    else if (ufunc) {
-	echo_function_call(line, CMD_ECHO_TO_STDOUT, prn);
-	err = gretl_function_start_exec(line, &Z, datainfo);
-	*fncall = 1;
-    } 
-
-    return err;
-}
-
 static int do_autofit_plot (PRN *prn)
 {
     int *plotlist;
-    int err = 0;
+    int pv, err = 0;
 
-    plotvar(&Z, datainfo, "time");
+    pv = plotvar(&Z, datainfo);
+    if (pv < 0) {
+	return 1;
+    }
 
     plotlist = gretl_list_new(3);
+    if (plotlist == NULL) {
+	return 1;
+    }
+
     plotlist[1] = gretl_model_get_depvar(models[0]);
     plotlist[2] = varindex(datainfo, "autofit");
-    plotlist[3] = varindex(datainfo, "time");
-    lines[0] = 1;
+    plotlist[3] = pv;
 
+    lines[0] = 1;
     err = gnuplot(plotlist, lines, NULL, &Z, datainfo,
 		  &plot_count, gp_flags(batch, 0));
 
@@ -745,45 +751,35 @@ static int do_autofit_plot (PRN *prn)
     return err;
 }
 
-static int exec_line (char *line, LOOPSET **ploop, PRN *prn) 
+static int exec_line (char *line, PRN *prn) 
 {
-    LOOPSET *loop = *ploop;
     GRETL_VAR *var = NULL;
-    int chk, nulldata_n, renumber;
-    int dbdata = 0, do_arch = 0, do_nls = 0;
+    int old_runit = runit;
+    int alt_model = 0;
+    int dbdata = 0;
     unsigned char echo_flags;
     char s1[12], s2[12];
-    int fncall = 0;
     double rho;
-    int err = 0;
+    int k, err = 0;
 
     if (string_is_blank(line)) {
 	return 0;
     }
 
-    /* catch any user-defined functions */
-    err = handle_user_defined_function(line, &fncall, prn);
-    if (err) {
-	errmsg(err, prn);
-	return err;
-    } else if (fncall) {
-	return 0;
-    }
-
     if (gretl_compiling_function()) {
 	err = gretl_function_append_line(line);
 	if (err) {
 	    errmsg(err, prn);
-	}
+	}    
 	return err;
-    }  
+    } 
 
     /* catch requests relating to saved objects, which are not
        really "commands" as such */
-    chk = saved_object_action(line, prn);
-    if (chk == 1) return 0;   /* action was OK */
-    if (chk == -1) return 1;  /* action was faulty */
-    
+    k = saved_object_action(line, prn);
+    if (k == 1) return 0;   /* action was OK */
+    if (k == -1) return 1;  /* action was faulty */
+
     /* are we ready for this? */
     if (!data_status && !cmd.ignore && 
 	!ready_for_command(line)) {
@@ -795,12 +791,12 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     gretl_set_batch_mode(batch);
 
     /* if we're stacking commands for a loop, parse "lightly" */
-    if (loopstack) {
-	err = get_command_index(line, &cmd);
+    if (gretl_compiling_loop()) {
+	err = get_command_index(line, &cmd, datainfo);
     } else {
 	err = parse_command_line(line, &cmd, &Z, datainfo);
 	if (err && gretl_executing_function()) {
-	    gretl_function_stop_on_error(datainfo, prn);
+	    gretl_function_stop_on_error(&Z, &datainfo, prn);
 	}	
     }
 
@@ -811,7 +807,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     /* if in batch mode, echo comments from input */
     if (batch && cmd.ci == CMD_COMMENT && gretl_echo_on()) {
-	printf_strip(linebak, loopstack);
+	printf_strip(linebak);
     }
 
     if (cmd.ci < 0) {
@@ -833,14 +829,14 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 		     "Type 'endloop' to get out\n"));
     }
    
-    if (loopstack || cmd.ci == LOOP) {  
+    if (cmd.ci == LOOP || gretl_compiling_loop()) {  
 	/* accumulating loop commands */
-	if (!ok_in_loop(cmd.ci, loop)) {
+	if (!ok_in_loop(cmd.ci)) {
 	    printf(_("Command '%s' ignored; not available in loop mode\n"), line);
 	} else {
 	    if (gretl_echo_on()) {
 		echo_flags = CMD_ECHO_TO_STDOUT;
-		if (loopstack) {
+		if (gretl_compiling_loop()) {
 		    echo_flags |= CMD_STACKING;
 		}
 		if (batch || runit) {
@@ -848,16 +844,11 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 		}
 		echo_cmd(&cmd, datainfo, line, echo_flags, cmdprn);
 	    }
-	    loop = add_to_loop(line, cmd.ci, cmd.opt,
-			       datainfo, &Z, loop, 
-			       &loopstack, &looprun);
-	    if (loop == NULL) {
-		loopstack = 0;
+	    err = gretl_loop_append_line(line, cmd.ci, cmd.opt, &Z, datainfo);
+	    if (err) {
 		set_errfatal(ERRFATAL_FORCE);
 		print_gretl_errmsg(prn);
-		err = 1;
 	    } 
-	    *ploop = loop;
 	}
 	return err;
     }
@@ -867,7 +858,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if (batch || runit) {
 	    echo_flags |= CMD_BATCH_MODE;
 	}
-	echo_cmd(&cmd, datainfo, line, echo_flags, cmdprn);
+	echo_cmd(&cmd, datainfo, line, echo_flags, NULL);
     }
 
     check_for_loop_only_options(cmd.ci, cmd.opt, prn);
@@ -884,6 +875,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case DATA:
     case DIFF: 
     case ESTIMATE:
+    case FNCALL:
     case FUNC:
     case FUNCERR:
     case GRAPH:
@@ -897,6 +889,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case LDIFF: 
     case LOGS:
     case MAHAL:
+    case MATRIX:
     case MEANTEST: 
     case MULTIPLY: 
     case OUTFILE: 
@@ -908,7 +901,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case RMPLOT: 
     case RUNS: 
     case SDIFF:
-    case SIM: 
+    case SHELL:
     case SPEARMAN:
     case SQUARE: 
     case STORE:
@@ -941,7 +934,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    /* for command-line use, we keep a "stack" of 
 	       two models, and recycle the places */
 	    if (!(cmd.opt & OPT_Q)) {
-		swap_models(&models[0], &models[1]);
+		swap_models(models[0], models[1]);
 	    } 
 	    clear_model(models[1]);
 	}
@@ -949,17 +942,17 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case ADDTO:
     case OMITFROM:
-	i = atoi(cmd.param);
-	if ((err = model_test_start(cmd.ci, i, prn))) break;
-	if (i == models[0]->ID) goto plain_add_omit;
-	err = re_estimate(modelspec_get_command_by_id(modelspec, i), 
+	k = atoi(cmd.param);
+	if ((err = model_test_start(cmd.ci, k, prn))) break;
+	if (k == models[0]->ID) goto plain_add_omit;
+	err = re_estimate(modelspec_get_command_by_id(modelspec, k), 
 			  &tmpmod, &Z, datainfo);
 	if (err) {
-	    pprintf(prn, _("Failed to reconstruct model %d\n"), i);
+	    pprintf(prn, _("Failed to reconstruct model %d\n"), k);
 	    break;
 	} 
 	clear_model(models[1]);
-	tmpmod.ID = i;
+	tmpmod.ID = k;
 	if (cmd.ci == ADDTO) {
 	    err = add_test(cmd.list, &tmpmod, models[1], 
 			   &Z, datainfo, cmd.opt, prn);
@@ -973,7 +966,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    break;
 	} else {
 	    if (!(cmd.opt & OPT_Q)) {
-		swap_models(&models[0], &models[1]);
+		swap_models(models[0], models[1]);
 	    }
 	    clear_model(models[1]);
 	}
@@ -982,24 +975,22 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case AR:
 	clear_model(models[0]);
-	*models[0] = ar_func(cmd.list, atoi(cmd.param), &Z, 
-			     datainfo, cmd.opt, prn);
+	*models[0] = ar_func(cmd.list, &Z, datainfo, cmd.opt, prn);
 	if ((err = models[0]->errcode)) { 
 	    errmsg(err, prn); 
 	}
 	break;
 
     case ARCH:
-	order = atoi(cmd.param);
 	clear_model(models[1]);
-	*models[1] = arch_model(cmd.list, order, &Z, datainfo, 
+	*models[1] = arch_model(cmd.list, cmd.order, &Z, datainfo, 
 				cmd.opt, prn);
 	if ((err = models[1]->errcode)) { 
 	    errmsg(err, prn);
 	}
 	if (models[1]->ci == ARCH) {
-	    do_arch = 1;
-	    swap_models(&models[0], &models[1]); 
+	    alt_model = 1;
+	    swap_models(models[0], models[1]); 
 	}
 	clear_model(models[1]);
 	break;
@@ -1047,7 +1038,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    break;
 	}
 	clear_model(models[0]);
-	*models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt, rho);
+	*models[0] = ar1_lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt, rho);
 	if ((err = (models[0])->errcode)) {
 	    errmsg(err, prn);
 	} else {
@@ -1056,8 +1047,8 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case CORRGM:
-	order = atoi(cmd.param);
-	err = corrgram(cmd.list[1], order, &Z, datainfo, batch, prn);
+	k = atoi(cmd.param);
+	err = corrgram(cmd.list[1], k, &Z, datainfo, prn, OPT_A);
 	if (err) {
 	    pputs(prn, _("Failed to generate correlogram\n"));
 	}
@@ -1069,12 +1060,11 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 			 " mode\n"));
 	    break;
 	}	
-	err = dataset_drop_listed_variables(cmd.list, &Z, datainfo, 
-					    &renumber);
+	err = dataset_drop_listed_variables(cmd.list, &Z, datainfo, &k);
 	if (err) {
 	    pputs(prn, _("Failed to shrink the data set"));
 	} else {
-	    if (renumber) {
+	    if (k) {
 		pputs(prn, _("Take note: variables have been renumbered"));
 		pputc(prn, '\n');
 	    }
@@ -1095,9 +1085,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    if ((err = models[0]->errcode)) {
 		errmsg(err, prn);
 	    } else {
-		if (!strcmp(cmd.param, "nls")) {
-		    do_nls = 1;
-		}
+		alt_model = 1;
 		printmodel(models[0], datainfo, cmd.opt, prn);
 	    }
 	} else if (!strcmp(cmd.param, "restrict")) {
@@ -1121,7 +1109,6 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case EQUATION:
 	err = gretl_equation_system_append(sys, cmd.list);
 	if (err) {
-	    gretl_equation_system_destroy(sys);
 	    sys = NULL;
 	    errmsg(err, prn);
 	}
@@ -1178,12 +1165,10 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case GENR:
-	err = generate(line, &Z, datainfo, cmd.opt);
+	err = generate(line, &Z, datainfo, cmd.opt, prn);
 	if (err) { 
 	    errmsg(err, prn);
-	} else {
-	    print_gretl_msg(prn);
-	}	
+	} 
 	break;
 
     case GNUPLOT:
@@ -1198,7 +1183,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    err = gnuplot(cmd.list, NULL, cmd.param, &Z, datainfo,
 			  &plot_count, gp_flags(batch, cmd.opt));
 	} else {
-	    lines[0] = (cmd.opt != 0);
+	    lines[0] = (cmd.opt & OPT_O)? 1 : 0;
 	    err = gnuplot(cmd.list, lines, cmd.param, 
 			  &Z, datainfo, &plot_count, 
 			  gp_flags(batch, 0));
@@ -1211,8 +1196,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case SCATTERS:
-	err = multi_scatters(cmd.list, atoi(cmd.param), &Z, 
-			     datainfo, &plot_count, 
+	err = multi_scatters(cmd.list, &Z, datainfo, &plot_count, 
 			     gp_flags(batch, cmd.opt));
 	if (err) {
 	    pputs(prn, _("scatters command failed\n"));
@@ -1267,8 +1251,8 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    break;
 	}
 
-	chk = detect_filetype(datfile, &paths, prn);
-	dbdata = (chk == GRETL_NATIVE_DB || chk == GRETL_RATS_DB);
+	k = detect_filetype(datfile, &paths, prn);
+	dbdata = (k == GRETL_NATIVE_DB || k == GRETL_RATS_DB);
 
 	if (data_status && !batch && !dbdata && cmd.ci != APPEND &&
 	    strcmp(datfile, paths.datfile)) {
@@ -1286,17 +1270,19 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    clear_data();
 	}
 
-	if (chk == GRETL_CSV_DATA) {
+	if (k == GRETL_CSV_DATA) {
 	    err = import_csv(&Z, &datainfo, datfile, prn);
-	} else if (chk == GRETL_OCTAVE) {
+	} else if (k == GRETL_OCTAVE) {
 	    err = import_octave(&Z, &datainfo, datfile, prn);
-	} else if (chk == GRETL_BOX_DATA) {
+	} else if (k == GRETL_BOX_DATA) {
 	    err = import_box(&Z, &datainfo, datfile, prn);
-	} else if (chk == GRETL_XML_DATA) {
+	} else if (WORKSHEET_IMPORT(k)) {
+	    err = import_other(&Z, &datainfo, k, datfile, prn);
+	} else if (k == GRETL_XML_DATA) {
 	    err = gretl_read_gdt(&Z, &datainfo, datfile, &paths, 
 				 data_status, prn, 0);
 	} else if (dbdata) {
-	    err = set_db_name(datfile, chk, &paths, prn);
+	    err = set_db_name(datfile, k, &paths, prn);
 	} else {
 	    err = gretl_get_data(&Z, &datainfo, datfile, &paths, 
 				 data_status, prn);
@@ -1415,13 +1401,13 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case NULLDATA:
-	nulldata_n = atoi(cmd.param);
-	if (nulldata_n < 2) {
+	k = atoi(cmd.param);
+	if (k < 2) {
 	    pputs(prn, _("Data series length count missing or invalid\n"));
 	    err = 1;
 	    break;
 	}
-	if (nulldata_n > 1000000) {
+	if (k > 1000000) {
 	    pputs(prn, _("Data series too long\n"));
 	    err = 1;
 	    break;
@@ -1429,8 +1415,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if (data_status) {
 	    clear_data();
 	}	
-	err = open_nulldata(&Z, datainfo, data_status, 
-			    nulldata_n, prn);
+	err = open_nulldata(&Z, datainfo, data_status, k, prn);
 	if (err) { 
 	    pputs(prn, _("Failed to create empty data set\n"));
 	} else {
@@ -1446,7 +1431,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if (cmd.ci == POOLED) {
 	    *models[0] = pooled(cmd.list, &Z, datainfo, cmd.opt, prn);
 	} else {
-	    *models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt, 0.0);
+	    *models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt);
 	}
 	if ((err = models[0]->errcode)) {
 	    errmsg(err, prn);
@@ -1470,7 +1455,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case PERGM:
-	err = periodogram(cmd.list[1], &Z, datainfo, batch, cmd.opt, prn);
+	err = periodogram(cmd.list[1], &Z, datainfo, prn, cmd.opt | OPT_N);
 	if (err) {
 	    pputs(prn, _("Failed to generate periodogram\n"));
 	}
@@ -1482,7 +1467,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case PVALUE:
 	if (batch || runit || (sscanf(line, "%s %s", s1, s2) == 2)) {
-	    batch_pvalue(line, (const double **) Z, datainfo, prn);
+	    batch_pvalue(line, (const double **) Z, datainfo, prn, OPT_NONE);
 	} else {
 	    interact_pvalue();
 	}
@@ -1503,14 +1488,16 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	printf(_("commands saved as %s\n"), cmdfile);
 	gretl_print_destroy(cmdprn);
 
-	if (cmd.opt & OPT_X) break;
+	if (cmd.opt & OPT_X) {
+	    break;
+	}
 
 	printf(_("type a filename to store output (enter to quit): "));
-	*outfile = '\0';
-	fgets(outfile, MAXLEN-1, stdin); 
+	get_a_filename(outfile);
 	top_n_tail(outfile);
 
-	if (*outfile != '\n' && *outfile != '\r' && strcmp(outfile, "q")) {
+	if (*outfile != 0 && *outfile != '\n' && *outfile != '\r' 
+	    && strcmp(outfile, "q")) {
 	    const char *udir = gretl_user_dir();
 
 	    printf(_("writing session output to %s%s\n"), udir, outfile);
@@ -1576,35 +1563,17 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	set_miss(cmd.list, cmd.param, Z, datainfo, prn);
 	break;
 
-    case SHELL:
-#ifdef WIN32
-	WinExec(line + 1, SW_SHOWNORMAL);
-#else		
-	shell(line + 1);
-#endif
-	break;
-
     case SMPL:
-	if (cmd.opt) {
-	    err = restore_full_sample(&Z, &datainfo, cmd.opt);
-	    if (err) {
-		errmsg(err, prn);
-		break;
-	    } else {
-		err = restrict_sample(line, &Z, &datainfo, 
-				      cmd.list, cmd.opt);
-	    }
-	} else if (!strcmp(line, "smpl full") ||
-		   !strcmp(line, "smpl --full")) {
-	    err = restore_full_sample(&Z, &datainfo, OPT_C);
+	if (cmd.opt == OPT_F) {
+	    err = restore_full_sample(&Z, &datainfo);
+	} else if (cmd.opt) {
+	    err = restrict_sample(line, cmd.list, &Z, &datainfo, 
+				  cmd.opt, prn);
 	} else { 
 	    err = set_sample(line, (const double **) Z, datainfo);
 	}
-
 	if (err) {
 	    errmsg(err, prn);
-	} else if (get_gretl_msg()) {
-	    print_gretl_msg(prn);
 	} else {
 	    print_smpl(datainfo, get_full_length_n(), prn);
 	}
@@ -1636,7 +1605,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case SYSTEM:
 	/* system of equations */
 	if (sys == NULL) {
-	    sys = system_start(line);
+	    sys = system_start(line, cmd.opt);
 	    if (sys == NULL) {
 		err = 1;
 		errmsg(err, prn);
@@ -1659,23 +1628,18 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case VAR:
-	order = atoi(cmd.param);
-	var = full_VAR(order, cmd.list, &Z, datainfo, cmd.opt, prn);
-	if (var == NULL) {
-	    err = 1;
-	} else {
+	var = gretl_VAR(cmd.order, cmd.list, &Z, datainfo, cmd.opt, 
+			prn, &err);
+	if (var != NULL) {
 	    err = maybe_stack_var(var, &cmd);
 	}
 	if (err) errmsg(err, prn);
 	break;
 
     case VECM:
-	order = atoi(cmd.param);
-	var = vecm(order, atoi(cmd.extra), cmd.list, &Z, datainfo, 
-		   cmd.opt, prn);
-	if (var == NULL) {
-	    err = 1;
-	} else {
+	var = vecm(cmd.order, atoi(cmd.extra), cmd.list, &Z, datainfo, 
+		   cmd.opt, prn, &err);
+	if (var != NULL) {
 	    err = maybe_stack_var(var, &cmd);
 	}
 	if (err) errmsg(err, prn);
@@ -1689,20 +1653,27 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     }
 
     if (err && gretl_executing_function()) {
-	gretl_function_stop_on_error(datainfo, prn);
+	gretl_function_stop_on_error(&Z, &datainfo, prn);
     }
 
-    if (!err && (is_model_cmd(cmd.word) || do_nls || do_arch)
+    if (!err && (is_model_cmd(cmd.word) || alt_model)
 	&& !is_quiet_model_test(cmd.ci, cmd.opt)) { 
 	attach_subsample_to_model(models[0], datainfo);
 #ifdef MSPEC_DEBUG
 	fprintf(stderr, "\ngretlcli: saving spec: model.ID = %d, model_count = %d\n",
 		(models[0])->ID, get_model_count());
 #endif
-	err = modelspec_save(models[0], &modelspec);
+	if (is_model_cmd(cmd.word)) {
+	    err = modelspec_save(models[0], &modelspec);
+	}
 	maybe_stack_model(models[0], &cmd, prn);
     }
 
+    if (!err && cmd.ci != QUIT && gretl_echo_on() && !batch && !old_runit) {
+	/* record a successful interactive command */
+	echo_cmd(&cmd, datainfo, line, 0, cmdprn);
+    }
+
     return err;
 }
 
