@@ -93,7 +93,6 @@ static const struct poptOption options[] = {
 
 windata_t *mdata;
 DATAINFO *datainfo;
-char *errtext;
 char cmdfile[MAXLEN], scriptfile[MAXLEN];
 char trydatfile[MAXLEN], tryscript[MAXLEN];
 PATHS paths;                /* useful paths */
@@ -170,7 +169,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/_Open data"), NULL, NULL, 0, "<Branch>" },
     { N_("/File/Open data/user file..."), NULL, open_data, OPEN_DATA, NULL },
     { N_("/File/Open data/sample file..."), NULL, display_files, TEXTBOOK_DATA, NULL },
-    { N_("/File/Open data/sep1"), NULL, NULL, 0, "<Separator>" },    
+    { "/File/Open data/sep1", NULL, NULL, 0, "<Separator>" },    
     { N_("/File/Open data/Import"), NULL, NULL, 0, "<Branch>" },
     { N_("/File/Open data/Import/CSV..."), NULL, open_data, OPEN_CSV, NULL },
     { N_("/File/Open data/Import/ASCII..."), NULL, open_data, OPEN_ASCII, NULL },
@@ -208,7 +207,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Send To..."), NULL, email_data, OPEN_DATA, NULL },
 #endif
     { N_("/File/C_lear data set"), NULL, verify_clear_data, 0, NULL },
-    { N_("/File/sep0"), NULL, NULL, 0, "<Separator>" },
+    { "/File/sep0", NULL, NULL, 0, "<Separator>" },
 
     /* File, Browse databases */
     { N_("/File/_Browse databases"), NULL, NULL, 0, "<Branch>" },
@@ -216,7 +215,7 @@ GtkItemFactoryEntry data_items[] = {
       NATIVE_DB, NULL },
     { N_("/File/Browse databases/_RATS 4"), NULL, display_files, 
       RATS_DB, NULL },
-    { N_("/File/Browse databases/sep1"), NULL, NULL, 0, "<Separator>" },
+    { "/File/Browse databases/sep1", NULL, NULL, 0, "<Separator>" },
     { N_("/File/Browse databases/on database _server"), NULL, display_files, 
       REMOTE_DB, NULL },
 
@@ -245,9 +244,9 @@ GtkItemFactoryEntry data_items[] = {
       NULL, newdata_callback, 0, NULL }, 
     { N_("/File/Create data set/simulation"), NULL, gretl_callback, 
       NULLDATA, NULL },
-    { N_("/File/sep1"), NULL, NULL, 0, "<Separator>" },
+    { "/File/sep1", NULL, NULL, 0, "<Separator>" },
     { N_("/File/_View command log"), NULL, view_command_log, 0, NULL },
-    { N_("/File/sep2a"), NULL, NULL, 0, "<Separator>" },
+    { "/File/sep2a", NULL, NULL, 0, "<Separator>" },
 
     /* File, command files */
     { N_("/File/Open command file"), NULL, NULL, 0, "<Branch>" },
@@ -259,13 +258,13 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/New command file/regular script"), NULL, do_new_script, 0, NULL },
     { N_("/File/New command file/Monte Carlo loop"), NULL, 
       do_new_script, 1, NULL },
-    { N_("/File/sep3"), NULL, NULL, 0, "<Separator>" },
+    { "/File/sep3", NULL, NULL, 0, "<Separator>" },
 
     /* File, preferences */
     { N_("/File/_Preferences"), NULL, NULL, 0, "<Branch>" },
     { N_("/File/_Preferences/_General..."), NULL, options_dialog, 0, NULL },
     { N_("/File/Preferences/_Fixed font..."), NULL, font_selector, 0, NULL },
-    { N_("/File/sep5"), NULL, NULL, 0, "<Separator>" },
+    { "/File/sep5", NULL, NULL, 0, "<Separator>" },
     { N_("/File/E_xit"), NULL, menu_exit_check, 0, NULL },
 
     /* Utilities menu */
@@ -273,9 +272,13 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Utilities/Statistical tables"), NULL, stats_calculator, CALC_DIST, NULL },
     { N_("/Utilities/p-value finder"), NULL, stats_calculator, CALC_PVAL, NULL },
     { N_("/Utilities/Test statistic calculator"), NULL, stats_calculator, CALC_TEST, NULL },
-    { N_("/Utilities/sep"), NULL, NULL, 0, "<Separator>" },
+    { "/Utilities/sep", NULL, NULL, 0, "<Separator>" },
     { N_("/Utilities/Gretl console"), NULL, show_gretl_console, 0, NULL },
-    { N_("/Utilities/sep2"), NULL, NULL, 0, "<Separator>" },
+    { "/Utilities/sep2", NULL, NULL, 0, "<Separator>" },
+#if 0
+    { N_("/Utilities/Additional functions..."), NULL, display_files, FUNC_FILES, NULL },
+    { "/Utilities/sep3", NULL, NULL, 0, "<Separator>" },
+#endif
     { N_("/Utilities/Start GNU R"), NULL, startRcallback, 0, NULL },
     { N_("/Utilities/NIST test suite"), NULL, NULL, 0, "<Branch>" },
     { N_("/Utilities/NIST test suite/basic"), NULL, do_nistcheck, 0, NULL },
@@ -285,9 +288,9 @@ GtkItemFactoryEntry data_items[] = {
     /* Session menu */
     { N_("/_Session"), NULL, NULL, 0, "<Branch>" },
     { N_("/Session/_Icon view"), NULL, view_session, 0, NULL },
-    { N_("/Session/sep0"), NULL, NULL, 0, "<Separator>" },
+    { "/Session/sep0", NULL, NULL, 0, "<Separator>" },
     { N_("/Session/_Open..."), NULL, open_script, OPEN_SESSION, NULL },
-    { N_("/Session/sep1"), NULL, NULL, 0, "<Separator>" },
+    { "/Session/sep1", NULL, NULL, 0, "<Separator>" },
     { N_("/Session/_Save"), NULL, save_session_callback, 0, NULL },
     { N_("/Session/Save _as..."), NULL, save_session_callback, 1, NULL },
 
@@ -302,7 +305,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/Sort variables"), NULL, NULL, 0, "<Branch>" },
     { N_("/Data/Sort variables/by ID number"), NULL, sort_varlist, 0, NULL },
     { N_("/Data/Sort variables/by name"), NULL, sort_varlist, 1, NULL },
-    { N_("/Data/sep1"), NULL, NULL, 0, "<Separator>" },
+    { "/Data/sep1", NULL, NULL, 0, "<Separator>" },
     { N_("/Data/_Graph specified vars"), NULL, NULL, 0, "<Branch>" },
     { N_("/Data/Graph specified vars/Time series plot..."), 
       NULL, selector_callback, GR_PLOT, NULL },
@@ -320,11 +323,11 @@ GtkItemFactoryEntry data_items[] = {
       NULL, selector_callback, GR_3D, NULL },
     { N_("/Data/_Multiple scatterplots..."), 
       NULL, selector_callback, SCATTERS, NULL},
-    { N_("/Data/sep2"), NULL, NULL, 0, "<Separator>" },
+    { "/Data/sep2", NULL, NULL, 0, "<Separator>" },
     { N_("/Data/_Read info"), NULL, open_info, 0, NULL },
     { N_("/Data/Edit _info"), NULL, edit_header, 0, NULL },
     { N_("/Data/Print description"), NULL, print_report, 0, NULL },
-    { N_("/Data/sep3"), NULL, NULL, 0, "<Separator>" },
+    { "/Data/sep3", NULL, NULL, 0, "<Separator>" },
 
     { N_("/Data/_Summary statistics"), NULL, NULL, 0, "<Branch>" },
     { N_("/Data/_Summary statistics/_all variables"), NULL, 
@@ -341,14 +344,14 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/_Principal components"), NULL, do_menu_op, PCA, NULL },
     { N_("/Data/_Mahalanobis distances"), NULL, do_menu_op, MAHAL, NULL },
 
-    { N_("/Data/sep4"), NULL, NULL, 0, "<Separator>" },
+    { "/Data/sep4", NULL, NULL, 0, "<Separator>" },
     { N_("/Data/Difference of means"), NULL, NULL, 0, "<Branch>" },
     { N_("/Data/Difference of means/assuming equal variances..."), NULL, 
       selector_callback, MEANTEST, NULL },
     { N_("/Data/Difference of means/assuming unequal variances..."), NULL, 
       selector_callback, MEANTEST2, NULL },
     { N_("/Data/Difference of variances..."), NULL, selector_callback, VARTEST, NULL },
-    { N_("/Data/sep5"), NULL, NULL, 0, "<Separator>" },
+    { "/Data/sep5", NULL, NULL, 0, "<Separator>" },
     { N_("/Data/Add variables"), NULL, NULL, 0, "<Branch>", },
     { N_("/Data/Add variables/time trend"), NULL, add_index, 1, NULL },
     { N_("/Data/Add variables/index variable"), NULL, add_index, 0, NULL },
@@ -367,14 +370,14 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/Add variables/periodic dummies"), NULL, add_dummies, 0, NULL },
     { N_("/Data/Add variables/unit dummies"), NULL, add_dummies, 1, NULL },
     { N_("/Data/Add variables/panel dummies"), NULL, add_dummies, 2, NULL },
-    { N_("/Data/Add variables/sep"), NULL, NULL, 0, "<Separator>" },
+    { "/Data/Add variables/sep", NULL, NULL, 0, "<Separator>" },
     { N_("/Data/Add variables/random normal..."), NULL, 
       add_random_callback, GENR_NORMAL, NULL },
     { N_("/Data/Add variables/random uniform..."), NULL, 
       add_random_callback, GENR_UNIFORM, NULL },
     { N_("/Data/Add variables/seed generator..."), NULL, gretl_callback, 
       SETSEED, NULL },
-    { N_("/Data/Add variables/sep2"), NULL, NULL, 0, "<Separator>" },
+    { "/Data/Add variables/sep2", NULL, NULL, 0, "<Separator>" },
     { N_("/Data/Add variables/Define _new variable..."), NULL, gretl_callback, GENR, NULL },
     { N_("/Data/Add observations..."), NULL, do_add_obs, 0, NULL },
     { N_("/Data/Remove extra observations"), NULL, do_remove_obs, 0, NULL },
@@ -384,26 +387,26 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/_Sample"), NULL, NULL, 0, "<Branch>" },
     { N_("/Sample/_Set range..."), NULL, sample_range_dialog, SMPL, NULL },
     { N_("/Sample/_Restore full range"), NULL, restore_sample_callback, 1, NULL },
-    { N_("/Sample/sep1"), NULL, NULL, 0, "<Separator>" },
+    { "/Sample/sep1", NULL, NULL, 0, "<Separator>" },
     { N_("/Sample/Dataset structure..."), NULL, data_structure_wizard, 0, NULL },
     { N_("/Sample/Compact data..."), NULL, do_compact_data_set, 0, NULL },
     { N_("/Sample/Expand data..."), NULL, do_expand_data_set, 0, NULL },
-    { N_("/Sample/sep2"), NULL, NULL, 0, "<Separator>" },   
+    { "/Sample/sep2", NULL, NULL, 0, "<Separator>" },   
     { N_("/Sample/_Define, based on dummy..."), NULL, sample_range_dialog, 
       SMPLDUM, NULL },
     { N_("/Sample/_Restrict, based on criterion..."), NULL, gretl_callback, 
       SMPLBOOL, NULL },
     { N_("/Sample/R_andom sub-sample..."), NULL, sample_range_dialog, SMPLRAND, NULL },
-    { N_("/Sample/sep3"), NULL, NULL, 0, "<Separator>" },  
+    { "/Sample/sep3", NULL, NULL, 0, "<Separator>" },  
     { N_("/Sample/Drop all obs with _missing values"), NULL, drop_all_missing, 
       0, NULL },
     { N_("/Sample/_Count missing values"), NULL, count_missing, 0, NULL },
     { N_("/Sample/Set missing _value code..."), NULL, gretl_callback, 
       GSETMISS, NULL },
-    { N_("/Sample/sep4"), NULL, NULL, 0, "<Separator>" },  
+    { "/Sample/sep4", NULL, NULL, 0, "<Separator>" },  
     { N_("/Sample/_Add case markers..."), NULL, open_data, OPEN_MARKERS, NULL },
     { N_("/Sample/Remove case _markers"), NULL, do_remove_markers, 0, NULL },
-    { N_("/Sample/sep5"), NULL, NULL, 0, "<Separator>" },
+    { "/Sample/sep5", NULL, NULL, 0, "<Separator>" },
     { N_("/Sample/Restructure panel..."), NULL, panel_restructure_dialog, 0, NULL },
     { N_("/Sample/Transpose data..."), NULL, gui_transpose_data, 0, NULL },
 
@@ -422,8 +425,9 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Variable/Frequency plot/against Gamma"), NULL, do_freqplot, 
       DIST_GAMMA, NULL },
     { N_("/Variable/Estimated density plot..."), NULL, do_kernel, 0, NULL },
+    { N_("/Variable/Gini coefficient"), NULL, do_gini, 0, NULL },
     { N_("/Variable/Range-mean graph"), NULL, do_range_mean, 0, NULL }, 
-    { N_("/Variable/sep1"), NULL, NULL, 0, "<Separator>" },
+    { "/Variable/sep1", NULL, NULL, 0, "<Separator>" },
     { N_("/Variable/_Time series plot"), NULL, ts_plot_var, 0, NULL },
     { N_("/Variable/Correlogram"), NULL, do_corrgm, CORRGM, NULL },
     { N_("/Variable/Spectrum"), NULL, NULL, 0, "<Branch>" },
@@ -439,24 +443,21 @@ GtkItemFactoryEntry data_items[] = {
 #endif
     { N_("/Variable/Hurst exponent"), NULL, do_hurst, 0, NULL }, 
     { N_("/Variable/Runs test"), NULL, do_menu_op, RUNS, NULL }, 
-    { N_("/Variable/sep2"), NULL, NULL, 0, "<Separator>" },
+    { "/Variable/sep2", NULL, NULL, 0, "<Separator>" },
     { N_("/Variable/Set missing value code..."), NULL, gretl_callback, 
       VSETMISS, NULL },
     { N_("/Variable/_Edit attributes"), NULL, varinfo_callback, 0, NULL },
-    { N_("/Variable/sep3"), NULL, NULL, 0, "<Separator>" },
+    { "/Variable/sep3", NULL, NULL, 0, "<Separator>" },
     { N_("/Variable/Define _new variable..."), NULL, gretl_callback, GENR, NULL },
     { N_("/_Model"), NULL, NULL, 0, "<Branch>" },
     { N_("/Model/_Ordinary Least Squares..."), NULL, model_callback, OLS, NULL },
     { N_("/Model/_Weighted Least Squares..."), NULL, model_callback, WLS, NULL },
-    { N_("/Model/sep1"),  NULL, NULL, 0, "<Separator>" },
-#if 0
-    { N_("/Model/HCC_M..."), NULL, model_callback, HCCM, NULL },
-#endif
+    { "/Model/sep1",  NULL, NULL, 0, "<Separator>" },
     { N_("/Model/H_eteroskedasticity corrected..."), NULL, model_callback, 
       HSK, NULL },
-    { N_("/Model/sep2"),  NULL, NULL, 0, "<Separator>" },
+    { "/Model/sep2",  NULL, NULL, 0, "<Separator>" },
     { N_("/Model/Time series"), NULL, NULL, 0, "<Branch>" },
-    { N_("/Model/sep3"),  NULL, NULL, 0, "<Separator>" },
+    { "/Model/sep3",  NULL, NULL, 0, "<Separator>" },
     { N_("/Model/_Two-Stage Least Squares..."), NULL, model_callback, TSLS, NULL },
     { "/Model/sep4",  NULL, NULL, 0, "<Separator>" },
     { N_("/Model/_Logit..."), NULL, model_callback, LOGIT, NULL },
@@ -479,9 +480,9 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Help/Command reference/plain text"), NULL, plain_text_cmdref, 0, NULL },
     { N_("/Help/Command reference/PDF"), NULL, display_pdf_help, 0, NULL },
     { N_("/Help/_User's guide"), NULL, display_pdf_help, 1, NULL },
-    { N_("/Help/sep1"), NULL, NULL, 0, "<Separator>" },
+    { "/Help/sep1", NULL, NULL, 0, "<Separator>" },
     { N_("/Help/Check for _updates"), NULL, manual_update_query, 0, NULL },
-    { N_("/Help/sep2"), NULL, NULL, 0, "<Separator>" },
+    { "/Help/sep2", NULL, NULL, 0, "<Separator>" },
     { N_("/Help/_About gretl"), NULL, about_dialog, 0, NULL }
 };
 
@@ -614,9 +615,6 @@ int main (int argc, char *argv[])
     nls_init();
 #endif       
 
-    if ((errtext = malloc(MAXLEN)) == NULL) 
-	noalloc(_("startup"));
-
     *tryscript = '\0';
     *scriptfile = '\0';
     *paths.datfile = '\0';
@@ -937,20 +935,33 @@ static void check_varmenu_state (GtkCList *list, gint i, gint j,
 
 static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 {
+    GdkModifierType mods;
+
+    gdk_window_get_pointer(w->window, NULL, NULL, &mods);
+    
 #ifdef HAVE_FLITE
     if (key->keyval == GDK_a) {
 	audio_render_window(vwin, AUDIO_LISTBOX);
-    } else if (key->keyval == GDK_x) {
+    } else if (key->keyval == GDK_x && !(mods & GDK_MOD1_MASK)) {
 	audio_render_window(NULL, AUDIO_LISTBOX);
     }
 #endif
 
-    if (key->keyval == GDK_e) {
+    if (key->keyval == GDK_x && (mods & GDK_MOD1_MASK)) {
+	gretl_callback(NULL, MINIBUF, NULL);
+    } else if (key->keyval == GDK_e || key->keyval == GDK_F2) {
 	varinfo_dialog(mdata->active_var, 1);
-    } 
-    else if (key->keyval == GDK_t) {
+    } else if (key->keyval == GDK_h || key->keyval == GDK_F1) {
+	plain_text_cmdref(NULL, 0, NULL);
+    } else if (key->keyval == GDK_g) {
+	gretl_callback(NULL, GENR, NULL);
+    } else if (key->keyval == GDK_t) {
 	do_graph_var(mdata->active_var);
-    } 
+    } else if (key->keyval == GDK_Delete) {
+	delete_selected_vars(mdata->active_var);
+    } else if (key->keyval == GDK_Return) {
+	display_selected(NULL, 0, NULL);
+    }
 
     return 1;
 }
@@ -1222,11 +1233,11 @@ static void set_up_main_menu (void)
     gtk_accel_group_attach(main_accel, GTK_OBJECT (mdata->w));
 }
 
-int restore_sample (gretlopt opt)
+int gui_restore_sample (void)
 {
     int err;
 
-    err = restore_full_sample(&Z, &datainfo, opt);
+    err = restore_full_sample(&Z, &datainfo);
     if (err) {
 	gui_errmsg(err);
     } else {
@@ -1238,7 +1249,7 @@ int restore_sample (gretlopt opt)
 
 static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
 {
-    int err = restore_sample(OPT_NONE); /* FIXME OPT_C? */
+    int err = gui_restore_sample();
 
     if (verbose && !err) {
 	set_sample_label(datainfo);    
@@ -1286,8 +1297,7 @@ int gretl_fork (const char *prog, const char *arg)
     sleep(1);
 
     if (fork_err) {
-	sprintf(errtext, "%s: %s", _("Command failed"), prog);
-	errbox(errtext);
+	errbox("%s: %s", _("Command failed"), prog);
     }
 
     return fork_err;
