@@ -104,16 +104,13 @@ static int printnum (char *dest, const char *s, int d)
 	if (data_status && (v = varindex(datainfo, s)) < datainfo->v) {
 	    xx = Z[v][0];
 	    if (na(xx)) {
-		sprintf(errtext, _("Data missing for variable '%s'"), s);
-		errbox(errtext);
+		errbox(_("Data missing for variable '%s'"), s);
 		return 1;
 	    }
 	    if (d) sprintf(numstr, "%f", xx);
 	    else sprintf(numstr, "%d", (int) xx);
 	} else {
-	    sprintf(errtext, 
-		    _("Unrecognized variable '%s' in p-value command"), s);
-	    errbox(errtext);
+	    errbox(_("Unrecognized variable '%s' in p-value command"), s);
 	    return 1;
 	}
     } else if (d != 0) {
@@ -431,7 +428,7 @@ static void get_pvalue (GtkWidget *w, gpointer data)
 	    return;
 	}
     } else {
-	batch_pvalue(cmd, (const double **) Z, datainfo, prn);
+	batch_pvalue(cmd, (const double **) Z, datainfo, prn, OPT_NONE);
     }
 
     view_buffer(prn, 78, 200, _("gretl: p-value"), PVALUE, NULL);
@@ -544,7 +541,7 @@ static void htest_graph (int dist, double x, int df1, int df2)
     fprintf(fp, "%g %g\n", x, spike);
     fputs("e\n", fp);
 
-    gretl_push_c_numeric_locale();
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
@@ -836,7 +833,7 @@ static void add_lookup_entry (GtkWidget *tbl, gint *tbl_len,
     /* label */
     gtk_table_resize(GTK_TABLE (tbl), *tbl_len, 2);
     tempwid = gtk_label_new(_(label));
-    gtk_misc_set_alignment (GTK_MISC (tempwid), 1, 0.5);
+    gtk_misc_set_alignment(GTK_MISC (tempwid), 1, 0.5);
     gtk_table_attach_defaults(GTK_TABLE (tbl), 
 			      tempwid, 0, 1, *tbl_len - 1, *tbl_len);
     gtk_widget_show(tempwid);
@@ -1025,7 +1022,7 @@ static void trash_test (GtkWidget *w, gpointer data)
 static int get_restriction_vxy (const char *s, int *vx, int *vy, 
 				GretlOp *yop, double *yval)
 {
-    char test[16];
+    char test[VNAMELEN];
     char *p, *q = NULL;
     char *str = g_strdup(s);
     int err = 0;
@@ -1038,7 +1035,7 @@ static int get_restriction_vxy (const char *s, int *vx, int *vy,
     *p = 0;
     p++;
 
-    if (sscanf(str, "%8s", test) != 1) {
+    if (sscanf(str, "%15s", test) != 1) {
 	err = 1;
     } else {
 	*vx = varindex(datainfo, test);
@@ -1081,7 +1078,7 @@ static int get_restriction_vxy (const char *s, int *vx, int *vy,
     }
 
     if (!err) {
-	if (sscanf(p, "%8s", test) != 1) {
+	if (sscanf(p, "%15s", test) != 1) {
 	    err = 1;
 	} else {
 	    *vy = varindex(datainfo, test);
@@ -1165,9 +1162,7 @@ static void populate_stats (GtkWidget *w, gpointer p)
     }
 
     if (n == 0) {		
-	sprintf(errtext, _("Data missing for variable '%s'"),
-		datainfo->varname[vx]);
-	errbox(errtext);
+	errbox(_("Data missing for variable '%s'"), datainfo->varname[vx]);
 	return;
     }
 
@@ -1708,7 +1703,7 @@ void stats_calculator (gpointer data, guint code, GtkWidget *widget)
 			 G_CALLBACK(context_help), 
 			 GINT_TO_POINTER(HTEST));
 	gtk_widget_show(tempwid);
-    }	
+    }
 
     gtk_widget_show(dialog->win);
 }
