@@ -296,7 +296,6 @@ void file_save (gpointer data, guint file_code, GtkWidget *widget)
 	delimiter_dialog(&opt);
 	p = GINT_TO_POINTER(opt);
     case EXPORT_R:
-    case EXPORT_R_ALT:
     case EXPORT_OCTAVE:
     case EXPORT_DAT:
 	data_save_selection_wrapper(file_code, p);
@@ -411,30 +410,34 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
     } else if (action == VAR || action == VECM) {
 	selection_dialog((action == VAR)? _("gretl: VAR") : _("gretl: VECM"),
 			 do_vector_model, action, 0);
+    } else if (action == VLAGSEL) {
+	selection_dialog(_("gretl: VAR lag selection"), do_vector_model, action, 0);
     } else if (action == GR_XY || action == GR_IMP || action == GR_DUMMY
 	       || action == SCATTERS || action == GR_3D) {
-	void (*okfunc)() = NULL;
+	int (*selfunc)() = NULL;
 
 	switch (action) {
 	case GR_XY:
 	case GR_IMP:
-	    okfunc = do_graph_from_selector;
+	    selfunc = do_graph_from_selector;
 	    break;
 	case GR_3D:
-	    okfunc = do_splot_from_selector;
+	    selfunc = do_splot_from_selector;
 	    break;
 	case GR_DUMMY:
-	    okfunc = do_dummy_graph;
+	    selfunc = do_dummy_graph;
 	    break;
 	case SCATTERS:
-	    okfunc = do_scatters;
+	    selfunc = do_scatters;
 	    break;
 	default:
 	    return;
 	}
-	selection_dialog(_("gretl: define graph"), okfunc, action, 0);
+	selection_dialog(_("gretl: define graph"), selfunc, action, 0);
     } else if (action == ADD || action == OMIT) {
 	simple_selection(_("gretl: model tests"), do_add_omit, action, vwin);
+    } else if (action == VAROMIT) {
+	simple_selection(_("gretl: model tests"), do_VAR_omit, action, vwin);
     } else if (action == COEFFSUM) {
 	simple_selection(_("gretl: model tests"), do_coeff_sum, action, vwin);
     } else if (action == ELLIPSE) {
@@ -529,6 +532,11 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	query = N_("Specify restrictions:");
 	okfunc = do_restrict;
 	break;	
+    case MINIBUF:
+	title = N_("gretl: command entry");
+	query = N_("Type a command:");
+	okfunc = do_minibuf;
+	break;	
     default:
 	errbox("Bug: unrecognized action code in gretl_callback");
 	return;
