@@ -35,9 +35,6 @@ static gchar *cbuf;
 static char **cmd_history;
 static int hl, hlmax, hlines;
 
-static LOOPSET *loop;
-static int loopstack, looprun;
-
 static int gretl_console_init (void)
 {
     char *hstr;
@@ -68,9 +65,6 @@ static int gretl_console_init (void)
 
     cbuf = NULL;
 
-    loop = NULL;
-    loopstack = looprun = 0;
-
     set_gretl_echo(1);
 
     return 0;
@@ -239,24 +233,23 @@ void console_record_sample (const DATAINFO *pdinfo)
     console_sample_handler(pdinfo, SAMPLE_RECORD);
 }
 
-static int sample_changed (const DATAINFO *pdinfo)
+int console_sample_changed (const DATAINFO *pdinfo)
 {
     return console_sample_handler(pdinfo, SAMPLE_CHECK);
 }
 
 static int console_function_exec (char *execline)
 {
-    char *gotline;
+    char *gotline = NULL;
     int err = 0;
 
-    while (!looprun) {
-	gotline = gretl_function_get_line(execline, MAXLINE, &Z, datainfo, &err);
+    while (!gretl_execute_loop()) {
+	gotline = gretl_function_get_line(execline, MAXLINE, &Z, &datainfo, &err);
 	if (gotline == NULL || *gotline == '\0') {
 	    break;
 	}
 	if (!err) {
-	    err = gui_exec_line(execline, &loop, &loopstack, &looprun, 
-				console_prn, SCRIPT_EXEC, NULL);
+	    err = gui_exec_line(execline, console_prn, SCRIPT_EXEC, NULL);
 	}
     }
 
@@ -313,27 +306,27 @@ static void console_exec (void)
     push_history_line(execline); 
 
     /* actually execute the command line */
-    gui_exec_line(execline, &loop, &loopstack, &looprun, console_prn, 
-		  CONSOLE_EXEC, NULL);
+    gui_exec_line(execline, console_prn, CONSOLE_EXEC, NULL);
 
     /* the control structure below is rather weird and needs more
        testing/thought.  The issue is the nesting of loops inside
-       functions or vice versa */
+       functions or vice versa. */
 
-    if (looprun) { 
+    if (gretl_execute_loop()) { 
     fn_run_loop:
-	loop_exec(loop, execline, &Z, &datainfo, models, console_prn);
-	gretl_loop_destroy(loop);
-	loop = NULL;
-	looprun = 0;
+	gretl_loop_exec(execline, &Z, &datainfo, models, console_prn);
 	if (gretl_executing_function()) {
 	    console_function_exec(execline);
 	}
     } else if (gretl_executing_function()) {
+    fn_run_fn:
 	console_function_exec(execline);
-	if (looprun) {
-	    /* the function we are exec'ing includes a loop */
+	if (gretl_execute_loop()) {
+	    /* the function we are exec'ing includes a loop */ 
 	    goto fn_run_loop;
+	} else if (gretl_executing_function()) {
+	    /* the function we are exec'ing includes a function */ 
+	    goto fn_run_fn;
 	}
     } 
 
@@ -354,8 +347,11 @@ static void console_exec (void)
         /* put results into console window */
 #ifndef OLD_GTK
 	if (!g_utf8_validate(cbuf, -1, NULL)) {
-	    fprintf(stderr, "text did not validate as utf8:\n'%s'\n", 
-		    cbuf);
+	    gchar *trbuf = my_locale_to_utf8(cbuf);
+
+	    fprintf(stderr, "console text did not validate as utf8\n");
+	    gtk_text_buffer_insert(buf, &start, trbuf, strlen(trbuf));
+	    g_free(trbuf);
 	} else {
 	    gtk_text_buffer_insert(buf, &start, cbuf, strlen(cbuf));
 	}
@@ -371,11 +367,11 @@ static void console_exec (void)
 
 #ifndef OLD_GTK
     gtk_text_buffer_insert_with_tags_by_name(buf, &start, 
-					     (loopstack)? "\n> " : "\n? ", 3,
-					     "redtext", NULL);
+					     (gretl_compiling_loop())? "\n> " : "\n? ", 
+					     3, "redtext", NULL);
 #else
     gtk_text_insert(GTK_TEXT(console_view), fixed_font,
-		    &red, NULL, (loopstack)? "\n> " : "\n? ", 3);
+		    &red, NULL, (gretl_compiling_loop())? "\n> " : "\n? ", 3);
     gtk_text_thaw(GTK_TEXT(console_view));
 #endif
 
@@ -392,7 +388,7 @@ static void console_exec (void)
     }
 
     /* update sample info and options if needed */
-    if (sample_changed(datainfo)) {
+    if (console_sample_changed(datainfo)) {
 	set_sample_label(datainfo);
     }
 }
