@@ -695,6 +695,37 @@ void browser_open_ps (GtkWidget *w, gpointer data)
     view_file(scriptfile, 0, 0, 78, 370, VIEW_SCRIPT);
 } 
 
+void browser_load_func (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+    char fnfile[FILENAME_MAX];
+    gchar *fname;
+    gchar *dir;
+
+#ifndef OLD_GTK
+    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
+			 0, &fname);
+    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
+			 2, &dir);
+#else
+    gtk_clist_get_text(GTK_CLIST(vwin->listbox), vwin->active_var, 
+		       0, &fname);
+    gtk_clist_get_text(GTK_CLIST(vwin->listbox), vwin->active_var, 
+		       2, &dir); /* wrong */
+#endif
+
+    build_path(dir, fname, fnfile, NULL);
+
+    fprintf(stderr, "fnfile: '%s'\n", fnfile);
+
+#ifndef OLD_GTK
+    g_free(fname);
+    g_free(dir);
+#endif
+
+    gtk_widget_destroy(GTK_WIDGET(vwin->w));
+} 
+
 static void set_browser_status (windata_t *vwin, int status)
 {
     if (status == BROWSER_BUSY) {
@@ -717,169 +748,6 @@ static gpointer get_browser_ptr (int role)
     return (gpointer) &(browsers[role - TEXTBOOK_DATA]);
 }
 
-static void get_local_status (char *fname, char *status, time_t remtime)
-{
-    char fullname[MAXLEN];
-    struct stat fbuf;
-    int err;
-
-    build_path(paths.binbase, fname, fullname, NULL);
-
-    if ((err = stat(fullname, &fbuf)) == -1) {
-	if (errno == ENOENT) {
-#ifdef G_OS_WIN32
-	    strcpy(status, _("Not installed"));
-#else
-	    /* try user dir too if not on Windows */
-	    build_path(paths.userdir, fname, fullname, NULL);
-	    if ((err = stat(fullname, &fbuf)) == -1) {
-		strcpy(status, _("Not installed"));
-	    } 
-#endif
-	} else {
-	    strcpy(status, _("Unknown: access error"));
-	}
-    }
-
-    if (!err) {
-	if (difftime(remtime, fbuf.st_ctime) > 360) {
-	    strcpy(status, _("Not up to date"));
-	} else {
-	    strcpy(status, _("Up to date"));
-	}
-    }
-}
-
-static int parse_db_list_line (char *line, char *fname, time_t *date)
-{
-    char mon[4], hrs[9];
-    int day, yr;
-    struct tm mytime;
-    const char *months[] = {
-	"Jan", "Feb", "Mar", "Apr",
-	"May", "Jun", "Jul", "Aug",
-	"Sep", "Oct", "Nov", "Dec"
-    };
-    int i;
-
-    if (sscanf(line, "%*s%*s%3s%2d%8s%4d%16s", 
-	       mon, &day, hrs, &yr, fname) != 5) {
-	return 1;
-    }
-
-    hrs[2] = 0;
-
-    mytime.tm_sec = 0;
-    mytime.tm_min = 0;   
-    mytime.tm_wday = 0;   
-    mytime.tm_yday = 0;   
-    mytime.tm_isdst = -1; 
-    mytime.tm_hour = atoi(hrs);
-    mytime.tm_year = yr - 1900;
-    mytime.tm_mday = day;
-    mytime.tm_mon = 0;
-
-    for (i=0; i<12; i++) {
-	if (strcmp(mon, months[i]) == 0) {
-	    mytime.tm_mon = i;
-	}
-    }
-
-    *date = mktime(&mytime);
-
-    return 0;
-}
-
-static gint populate_remote_db_list (windata_t *win)
-{
-#ifndef OLD_GTK
-    GtkListStore *store;
-    GtkTreeIter iter;  
-#endif  
-    int err;
-    char *buf;
-    char fname[16], line[128], errbuf[80], status[20];
-    gchar *row[3];
-    gint i;
-    time_t remtime;
-
-    buf = mymalloc(GRETL_BUFSIZE);
-    if (buf == NULL) return 1;
-
-    memset(buf, 0, GRETL_BUFSIZE);
-
-    *errbuf = '\0';
-
-    err = list_remote_dbs(&buf, errbuf);
-
-    if (err) {
-	show_db_error(NULL, errbuf);
-	free(buf);
-	return err;
-    }
-
-#ifndef OLD_GTK
-    store = GTK_LIST_STORE(gtk_tree_view_get_model 
-			   (GTK_TREE_VIEW(win->listbox)));
-    gtk_list_store_clear (store);
-    gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
-#else
-    gtk_clist_clear(GTK_CLIST(win->listbox));
-    gtk_clist_freeze(GTK_CLIST(win->listbox));
-#endif
-
-    bufgets_init(buf);
-    i = 0;
-
-    while (bufgets(line, sizeof line, buf)) {
-	if (strstr(line, "idx")) {
-	    continue;
-	}
-	if (parse_db_list_line(line, fname, &remtime)) {
-	    continue;
-	}
-	get_local_status(fname, status, remtime);
-	row[0] = strip_extension(fname);
-
-	if (bufgets(line, sizeof line, buf)) {
-#ifndef OLD_GTK
-	    my_utf_validate(line);
-#endif
-	    row[1] = line + 2;
-	} else {
-	    row[1] = NULL;
-	}
-
-	row[2] = status;
-#ifndef OLD_GTK
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set (store, &iter, 0, row[0], 1, row[1],
-			    2, row[2], -1);
-#else
-	gtk_clist_append(GTK_CLIST(win->listbox), row);
-	if (i % 2) {
-	    gtk_clist_set_background(GTK_CLIST(win->listbox), i, &gray);
-	}
-#endif
-	i++;
-    }
-
-#ifdef OLD_GTK
-    gtk_clist_thaw(GTK_CLIST(win->listbox));
-#endif
-
-    free(buf);
-
-    if (i == 0) errbox(_("No database files found"));
-#ifdef OLD_GTK
-    else {
-	gtk_clist_select_row(GTK_CLIST(win->listbox), 0, 0);
-    }
-#endif
-
-    return 0;
-}
-
 static void build_datafiles_popup (windata_t *vwin)
 {
     if (vwin->popup == NULL) {
@@ -946,6 +814,11 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 			     _("gretl: practice files"));
 	browse_func = browser_open_ps;
 	break;
+    case FUNC_FILES:
+	gtk_window_set_title(GTK_WINDOW(vwin->w), 
+			     _("gretl: function files"));
+	browse_func = browser_load_func;
+	break;
     case TEXTBOOK_DATA:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: data files"));
@@ -1043,19 +916,167 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     }
 
     if (err) {
-	errbox(_("Couldn't open database"));
+	if (0) {
+	    /* FIXME? */
+	    errbox(_("Couldn't open database"));
+	}
 	gtk_widget_destroy(vwin->w);
     } else {
 	gtk_widget_show_all(vwin->w); 
     }
 }
 
+static char *get_func_description (const char *fname, const char *fndir)
+{
+    char fullname[FILENAME_MAX];
+    char line[128];
+    FILE *fp;
+
+    build_path(fndir, fname, fullname, NULL);
+    
+    fp = fopen(fullname, "r");
+    if (fp == NULL) {
+	return NULL;
+    }
+    
+    while (fgets(line, sizeof line, fp)) {
+	if (*line != '#') {
+	    break;
+	}
+	printf("%s", line + 2);
+    }
+
+    fclose(fp);
+
+    return g_strdup("Not ready yet");
+}
+
+#ifndef OLD_GTK
+
+static int
+read_fn_files_in_dir (DIR *dir, const char *fndir, 
+		      GtkListStore *store, GtkTreeIter *iter)
+{
+    struct dirent *dirent;
+    const char *fname;
+    char *descrip;
+    int n, nfn = 0;
+
+    while ((dirent = readdir(dir)) != NULL) {
+	fname = dirent->d_name;
+	n = strlen(fname);
+	if (!g_ascii_strcasecmp(fname + n - 4, ".inp")) {
+	    descrip = get_func_description(fname, fndir);
+	    if (descrip != NULL) {
+		gtk_list_store_append(store, iter);
+		gtk_list_store_set(store, iter, 0, fname, 1, descrip,
+				   2, fndir, -1);
+		g_free(descrip);
+		nfn++;
+	    }
+	} 
+    }
+
+    return nfn;
+}
+
+#else
+
+static int
+read_fn_files_in_dir (DIR *dir, char *fndir, windata_t *vwin, int nfn)
+{
+    struct dirent *dirent;
+    char *fname;
+    char *descrip;
+    gchar *row[2];
+    int n, i;
+
+    while ((dirent = readdir(dir)) != NULL) {
+	fname = dirent->d_name;
+	n = strlen(fname);
+	if (!g_strcasecmp(fname + n - 4, ".inp")) {
+	    descrip = get_func_description(fname, fndir);
+	    if (descrip != NULL) {
+		row[0] = fname;
+		row[1] = descrip;
+		i = gtk_clist_append(GTK_CLIST(vwin->listbox), row);
+		gtk_clist_set_row_data(GTK_CLIST(vwin->listbox), i, fndir);
+		g_free(descrip);
+		nfn++;
+		if (nfn % 2) {
+		    gtk_clist_set_background(GTK_CLIST(vwin->listbox), i, &gray);
+		}
+	    } 
+	}
+    }
+
+    return nfn;
+}
+
+#endif
+
+gint populate_func_list (windata_t *vwin)
+{
+#ifndef OLD_GTK
+    GtkListStore *store;
+    GtkTreeIter iter;
+#endif
+    char fndir[FILENAME_MAX];
+    DIR *dir;
+    int nfn = 0;
+
+#ifndef OLD_GTK
+    store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+#endif
+
+    /* pick up any function files in system dir */
+    sprintf(fndir, "%sfunctions", paths.gretldir);
+    dir = opendir(fndir);
+
+    if (dir != NULL) {
+#ifndef OLD_GTK
+	nfn += read_fn_files_in_dir(dir, fndir, store, &iter);
+#else
+	nfn = read_fn_files_in_dir(dir, fndir, vwin, 0);
+#endif
+	closedir(dir);
+    }
+
+    /* pick up any function files in the user's personal dir */
+    sprintf(fndir, "%sfunctions", paths.userdir);
+    dir = opendir(fndir);
+
+    if (dir != NULL) {
+#ifndef OLD_GTK
+	nfn += read_fn_files_in_dir(dir, fndir, store, &iter);
+#else
+	nfn = read_fn_files_in_dir(dir, fndir, vwin, nfn);
+#endif
+	closedir(dir);
+    }
+
+    if (nfn == 0) {
+	errbox(_("No function files found"));
+	/* FIXME don't leak list store */
+	return 1;
+    }
+
+#ifdef OLD_GTK
+    gtk_clist_select_row(GTK_CLIST(vwin->listbox), 0, 0);
+#endif
+
+    return 0;
+}
+
 gint populate_filelist (windata_t *vwin, gpointer p)
 {
     if (vwin->role == NATIVE_DB || vwin->role == RATS_DB) {
 	return populate_dbfilelist(vwin);
     } else if (vwin->role == REMOTE_DB) {
 	return populate_remote_db_list(vwin);
+    } else if (vwin->role == FUNC_FILES) {
+	return populate_func_list(vwin);
     } else {
 	return read_file_descriptions(vwin, p);
     }
@@ -1113,6 +1134,9 @@ static GtkWidget *files_window (windata_t *vwin)
 	cols = 3;
 	full_width = 480;
 	break;
+    case FUNC_FILES:
+	hidden_col = 1;
+	break;
     default:
 	break;
     }
