@@ -26,10 +26,10 @@
 
 #include "system.h"
 
-/* Various buttons, usable in several sorts of dialogs */
-
 #ifdef OLD_GTK
 
+/* compatibility functions */
+
 GtkWidget *standard_button (int code)
 {
     const char *button_strings[] = {
@@ -47,8 +47,22 @@ GtkWidget *standard_button (int code)
     return gtk_button_new_with_label(_(button_strings[code]));
 }
 
+GtkWidget *
+gtk_spin_button_new_with_range (double lo, double hi, double step)
+{
+    GtkAdjustment *adj;
+    GtkWidget *sb;
+
+    adj = (GtkAdjustment *) gtk_adjustment_new(lo, lo, hi, step, 10 * step, 0);
+    sb = gtk_spin_button_new(adj, 0, 0);
+
+    return sb;
+}
+
 #endif
 
+/* Various buttons, usable in several sorts of dialogs */
+
 GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode)
 {
     GtkWidget *w;
@@ -84,19 +98,38 @@ static void opt_invalid (GtkWidget *w, int *opt)
     *opt = -1;
 }
 
+#ifndef OLD_GTK
+static void maybe_opt_invalid (GtkDialog *d, int resp, int *opt)
+{
+    if (resp == GTK_RESPONSE_NONE || resp == GTK_RESPONSE_DELETE_EVENT) {
+	*opt = -1;
+    }
+}
+#endif
+
 GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 				  int *opt)
 {
-    GtkWidget *w;
+    GtkWidget *w = standard_button(GTK_STOCK_CANCEL);
 
-    w = standard_button(GTK_STOCK_CANCEL);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+
     if (opt != NULL) {
 	g_signal_connect(G_OBJECT(w), "clicked", 
 			 G_CALLBACK(opt_invalid), 
 			 opt);
+#ifdef OLD_GTK
+	g_signal_connect(G_OBJECT(targ), "delete_event", 
+			 G_CALLBACK(opt_invalid), 
+			 opt);
+#else
+	g_signal_connect(GTK_DIALOG(targ), "response", 
+			 G_CALLBACK(maybe_opt_invalid), 
+			 opt);
+#endif
     }
+
     g_signal_connect(G_OBJECT(w), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     targ);
@@ -116,6 +149,17 @@ GtkWidget *ok_button (GtkWidget *hbox)
     return w;
 }
 
+GtkWidget *apply_button (GtkWidget *hbox)
+{
+    GtkWidget *w;
+
+    w = standard_button(GTK_STOCK_APPLY);
+    GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
+    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+
+    return w;
+}
+
 GtkWidget *next_button (GtkWidget *hbox)
 {
     GtkWidget *w;
@@ -649,7 +693,7 @@ static void system_estimator_list (GtkWidget *vbox, gpointer data)
     gchar **strs;
     int i;
 
-    strs = create_strings_array(SYS_MAX - 1);
+    strs = create_strings_array(SYS_MAX);
 
     for (i=SYS_SUR; i<SYS_MAX; i++) {
 	strs[i] = g_strdup_printf("%s (%s)", _(system_method_full_string(i)),
