@@ -20,6 +20,10 @@
 #include "gretl.h"
 #include "filelists.h"
 
+#ifdef G_OS_WIN32
+# include "gretlwin32.h"
+#endif
+
 /* lists of recently opened files */
 static char datalist[MAXRECENT][MAXSTR];
 static char sessionlist[MAXRECENT][MAXSTR];
@@ -68,20 +72,21 @@ static char **get_file_list (int filetype)
     }
 }
 
+static const char *file_sections[] = {
+    "recent_data_files",
+    "recent_session_files",
+    "recent_script_files"
+};
+
 #if defined(USE_GNOME) && !defined(OLD_GTK)
 
 static void printfilelist (int filetype, GConfClient *client)
 {
     GSList *flist = NULL;
-    gchar *key;
-    int i;
-    char **filep;
     GError *err = NULL;
-    const char *file_sections[] = {
-	"recent_data_files",
-	"recent_session_files",
-	"recent_script_files"
-    };
+    gchar *key;
+    char **filep;
+    int i;
 
     filep = get_file_list(filetype);
     if (filep == NULL) {
@@ -112,18 +117,36 @@ void save_file_lists (GConfClient *client)
     printfilelist(FILE_LIST_SCRIPT, client);
 }
 
+void read_file_lists (GConfClient *client)
+{
+    GSList *flist = NULL;
+    char key[MAXSTR];
+    int i, j;
+
+    initialize_file_lists();
+
+    for (i=0; i<3; i++) {
+	sprintf(key, "/apps/gretl/%s", file_sections[i]);
+	flist = gconf_client_get_list(client, key,
+				      GCONF_VALUE_STRING, NULL);
+	if (flist != NULL) {
+	    for (j=0; j<MAXRECENT; j++) {
+		write_filename_to_list(i, j, flist->data);
+		flist = flist->next;
+	    }
+	    g_slist_free(flist);
+	    flist = NULL;
+	}
+    }
+}
+
 #elif defined(USE_GNOME)
 
 static void printfilelist (int filetype)
 {
-    int i;
-    char **filep;
     char gpath[MAXLEN];
-    const char *file_sections[] = {
-	"recent data files",
-	"recent session files",
-	"recent script files"
-    };
+    char **filep;
+    int i;
 
     filep = get_file_list(filetype);
     if (filep == NULL) {
@@ -143,18 +166,35 @@ void save_file_lists (void)
     printfilelist(FILE_LIST_SCRIPT);
 }
 
+void read_file_lists (void)
+{
+    char key[MAXSTR];
+    gchar *value;
+    int i, j;
+
+    initialize_file_lists();
+
+    for (i=0; i<3; i++) {
+	for (j=0; j<MAXRECENT; j++) {
+	    sprintf(key, "/gretl/%s/%d", file_sections[i], j);
+	    value = gnome_config_get_string(key);
+	    if (value != NULL) {
+		write_filename_to_list(i, j, value);
+		g_free(value);
+	    } else {
+		break;
+	    }
+	}
+    }
+}  
+
 #elif defined(G_OS_WIN32)
 
 static void printfilelist (int filetype)
 {
-    int i;
-    char **filep;
     char rpath[MAXLEN];
-    const char *file_sections[] = {
-	"recent data files",
-	"recent session files",
-	"recent script files"
-    };
+    char **filep;
+    int i;
 
     filep = get_file_list(filetype);
     if (filep == NULL) {
@@ -176,31 +216,41 @@ void save_file_lists (void)
     printfilelist(FILE_LIST_SCRIPT);
 }
 
+void read_file_lists (void)
+{
+    char rpath[MAXSTR], value[MAXSTR];
+    int i, j;
+
+    initialize_file_lists();
+
+    for (i=0; i<3; i++) {
+	for (j=0; j<MAXRECENT; j++) {
+	    sprintf(rpath, "%s\\%d", file_sections[i], j);
+	    if (read_reg_val(HKEY_CURRENT_USER, "gretl", rpath, value) == 0) { 
+		write_filename_to_list(i, j, value);
+	    } else {
+		break;
+	    }
+	}
+    } 
+}
+
 #else /* "plain" version follows */
 
 static void printfilelist (int filetype, FILE *fp)
 {
-    int i;
     char **filep;
-    const char *file_sections[] = {
-	"recent data files",
-	"recent session files",
-	"recent script files"
-    };
+    int i;
 
     filep = get_file_list(filetype);
     if (filep == NULL) {
 	return;
     }
 
-    fprintf(fp, "%s:\n", file_sections[filetype]);
-
     for (i=0; i<MAXRECENT; i++) {
-	if (*filep[i] != '\0') {
-	    fprintf(fp, "%s\n", filep[i]);
-	} else {
-	    break;
-	}
+	if (filep[i] != NULL && *filep[i] != 0) {
+	    fprintf(fp, "%s%d %s\n", file_sections[filetype], i, filep[i]);
+	} 
     }
 }
 
@@ -211,6 +261,39 @@ void save_file_lists (FILE *fp)
     printfilelist(FILE_LIST_SCRIPT, fp);
 }    
 
+void read_file_lists (FILE *fp, char *prev)
+{
+    char line[MAXLEN];
+    int i, j, len;
+
+    initialize_file_lists();
+
+    strcpy(line, prev);
+    while (strncmp(line, "recent", 6)) {
+	if (fgets(line, sizeof line, fp) == NULL) {
+	    return;
+	}
+    }
+
+    j = 0;
+    for (i=0; i<3; ) {
+	len = strlen(file_sections[i]);
+	if (!strncmp(line, file_sections[i], len)) {
+	    chopstr(line);
+	    if (*line != '\0') {
+		write_filename_to_list(i, j++, line + len + 2);
+	    }
+	}
+	if (fgets(line, sizeof line, fp) == NULL) {
+	    break;
+	}
+	if (i < 2 && strstr(line, file_sections[i+1])) {
+	    i++;
+	    j = 0;
+	}
+    }	    
+}
+
 #endif 
 
 static char *endbit (char *dest, const char *src, int addscore)
@@ -300,21 +383,24 @@ static void add_files_to_menu (int ftype)
     real_add_files_to_menus(ftype);
 }
 
-void mkfilelist (int filetype, char *fname)
+void mkfilelist (int filetype, char *fname_in)
 {
     char *tmp[MAXRECENT-1];
     char **filep;
+    char *fname;
     int i, match = -1;
-#if defined(ENABLE_NLS) && !defined(OLD_GTK)
+#ifndef OLD_GTK
     char trfname[MAXLEN];
 #endif
 
-    cut_multiple_slashes(fname);
+    cut_multiple_slashes(fname_in);
 
-#if defined(ENABLE_NLS) && !defined(OLD_GTK)
-    strcpy(trfname, fname);
+#ifndef OLD_GTK
+    strcpy(trfname, fname_in);
     my_filename_to_utf8(trfname);
     fname = trfname;
+#else
+    fname = fname_in;
 #endif
 
     filep = get_file_list(filetype);
@@ -323,12 +409,22 @@ void mkfilelist (int filetype, char *fname)
     }
 
     /* see if this file is already on the list */
+#ifdef G_OS_WIN32
+    for (i=0; i<MAXRECENT; i++) {
+	/* ignore case */
+        if (fnamecmp_win32(filep[i], fname) == 0) {
+            match = i;
+            break;
+        }
+    }
+#else
     for (i=0; i<MAXRECENT; i++) {
         if (strcmp(filep[i], fname) == 0) {
             match = i;
             break;
         }
     }
+#endif
 
     if (match == 0) {
 	/* file is on top: no change in list */
@@ -392,12 +488,21 @@ void delete_from_filelist (int filetype, const char *fname)
     }
 
     /* save pointers to current order */
+#ifdef G_OS_WIN32
+    for (i=0; i<MAXRECENT; i++) {
+	tmp[i] = filep[i];
+        if (fnamecmp_win32(filep[i], fname) == 0) {
+            match = i;
+        }
+    }
+#else
     for (i=0; i<MAXRECENT; i++) {
 	tmp[i] = filep[i];
 	if (!strcmp(filep[i], fname)) {
 	    match = i;
 	}
     }
+#endif
 
     if (match == -1) {
 	return;
@@ -420,7 +525,8 @@ void delete_from_filelist (int filetype, const char *fname)
 static void copy_sys_filename (char *targ, const char *src)
 {
     strcpy(targ, src);
-#if defined(ENABLE_NLS) && !defined(OLD_GTK)
+    /* check this: very confusing! */
+#if !defined(OLD_GTK) && !defined(G_OS_WIN32)
     my_filename_from_utf8(targ);
 #endif
 }    
