@@ -74,7 +74,6 @@
 
 #define EXPORT_ACTION(a,s) ((a == EXPORT_OCTAVE || \
                              a == EXPORT_R || \
-                             a == EXPORT_R_ALT || \
                              a == EXPORT_CSV || \
                              a == EXPORT_DAT) && s != FSEL_DATA_PRN)
 
@@ -103,7 +102,6 @@ static struct extmap action_map[] = {
     { SAVE_BOXPLOT_XPM,  ".xpm" },
     { EXPORT_CSV,        ".csv" },
     { EXPORT_R,          ".R" },
-    { EXPORT_R_ALT,      ".R" },
     { OPEN_OCTAVE,       ".m" },
     { APPEND_OCTAVE,     ".m" },
     { EXPORT_OCTAVE,     ".m" },
@@ -142,7 +140,6 @@ static gretlopt save_action_to_opt (int action, gpointer p)
     case SAVE_DBDATA:   opt = OPT_D; break;
     case EXPORT_OCTAVE: opt = OPT_M; break;
     case EXPORT_R:      opt = OPT_R; break;
-    case EXPORT_R_ALT:  opt = OPT_A; break;
     case EXPORT_CSV:    opt = OPT_C; break;
     case EXPORT_DAT:    opt = OPT_G; break; /* PcGive */
     default: break;
@@ -349,8 +346,7 @@ static void filesel_save_prn_buffer (PRN *prn, const char *fname)
     FILE *fp = gretl_fopen(fname, "w");
 
     if (fp == NULL) {
-	sprintf(errtext, _("Couldn't write to %s"), fname);
-	errbox(errtext);
+	errbox(_("Couldn't write to %s"), fname);
     } else {
 	const char *buf = gretl_print_get_buffer(prn);
 
@@ -435,7 +431,6 @@ static char *suggested_exportname (const char *fname, int action)
 	    test = ".m";
 	    break;
 	case EXPORT_R:
-	case EXPORT_R_ALT:
 	    test = ".R";
 	    break;
 	case EXPORT_CSV:
@@ -482,8 +477,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	FILE *fp = gretl_fopen(fname, "r");
 
 	if (fp == NULL) {
-	    sprintf(errtext, _("Couldn't open %s"), fname);
-	    errbox(errtext);
+	    errbox(_("Couldn't open %s"), fname);
 	    return;
 	} else {
 	    fclose(fp);
@@ -524,12 +518,14 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	return;
     }
 
-    if (src == FSEL_DATA_PRN) {
-	if (action == SAVE_TEX) {
+    if (action == SAVE_TEX) {
+	if (src == FSEL_DATA_PRN) {
 	    save_latex((PRN *) data, fname);
 	} else {
-	    filesel_save_prn_buffer((PRN *) data, fname);
+	    save_latex(NULL, fname);
 	}
+    } else if (src == FSEL_DATA_PRN) {
+	filesel_save_prn_buffer((PRN *) data, fname);
     } else if (SAVE_DATA_ACTION(action)) {
 	int overwrite = 0;
 
@@ -638,7 +634,6 @@ static struct winfilter get_filter (int action, gpointer data)
 	{SAVE_GP_CMDS, { N_("gnuplot files (*.plt)"), "*.plt" }},
 	{EXPORT_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
 	{EXPORT_R,     { N_("GNU R files (*.R)"), "*.R" }},
-	{EXPORT_R_ALT, { N_("GNU R files (*.R)"), "*.R" }},
 	{EXPORT_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
 	{OPEN_OCTAVE,  { N_("GNU Octave files (*.m)"), "*.m" }},
 	{APPEND_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
@@ -811,9 +806,7 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	return;
     }
 
-#ifdef ENABLE_NLS
     my_filename_to_utf8(fname);
-#endif
 
     file_selector_process_result(fname, action, src, data);
 }
@@ -918,7 +911,20 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
     gtk_dialog_set_default_response(GTK_DIALOG(filesel), GTK_RESPONSE_ACCEPT);
 
     filter = get_file_filter(action, data);
+    if (action == OPEN_ASCII || action == APPEND_ASCII) {
+	gtk_file_filter_set_name(filter, _("ASCII files (*.txt)"));
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
+    }
+
     gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filesel), filter);
+
+    if (action == OPEN_ASCII || action == APPEND_ASCII) {
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name(filter, _("all files (*.*)"));
+	gtk_file_filter_add_pattern(filter, "*.*");
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
+    }
+
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
 
     /* special cases */
