@@ -26,11 +26,11 @@
 #include "fileselect.h"
 
 #define GPDEBUG 0
-#undef POINTS_DEBUG
+#define POINTS_DEBUG 0
 
 #ifdef G_OS_WIN32
-# include <windows.h>
 # include <io.h>
+# include "gretlwin32.h"
 #endif
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -66,8 +66,11 @@ enum {
     PLOT_Y2AXIS         = 1 << 3,
     PLOT_Y2LABEL        = 1 << 4,
     PLOT_MARKERS_UP     = 1 << 5,
+    PLOT_POLAR          = 1 << 6
 } plot_format_flags;
 
+#define MAX_MARKERS 100
+
 #define plot_is_saved(p)        (p->status & PLOT_SAVED)
 #define plot_has_controller(p)  (p->status & PLOT_HAS_CONTROLLER)
 #define plot_is_zoomed(p)       (p->status & PLOT_ZOOMED)
@@ -86,12 +89,14 @@ enum {
 #define plot_has_y2axis(p)       (p->format & PLOT_Y2AXIS)
 #define plot_has_y2label(p)      (p->format & PLOT_Y2LABEL)
 #define plot_has_data_markers(p) (p->format & PLOT_MARKERS_UP)
+#define plot_is_polar(p)         (p->format & PLOT_POLAR)
 
 #define plot_is_range_mean(p)   (p->spec->code == PLOT_RANGE_MEAN)
 #define plot_is_hurst(p)        (p->spec->code == PLOT_HURST)
+#define plot_is_roots(p)        (p->spec->code == PLOT_VAR_ROOTS)
 
 #define plot_has_regression_list(p) (p->spec->reglist != NULL)
-#define plot_show_all_markers(p) (p->spec->flags & GPTSPEC_ALL_MARKERS)
+#define plot_show_all_markers(p)    (p->spec->flags & GPTSPEC_ALL_MARKERS)
 
 enum {
     PNG_START,
@@ -132,13 +137,6 @@ static int zoom_unzoom_png (png_plot *plot, int view);
 static void create_selection_gc (png_plot *plot);
 static int get_plot_ranges (png_plot *plot);
 
-#ifdef G_OS_WIN32
-enum {
-    WIN32_TO_CLIPBOARD,
-    WIN32_TO_PRINTER
-};
-#endif /* G_OS_WIN32 */
-
 #ifdef PNG_COMMENTS
 
 enum {
@@ -247,11 +245,10 @@ static FILE *open_gp_file (const char *fname, const char *mode)
 
     if (fp == NULL) {
 	if (*mode == 'w') {
-	    sprintf(errtext, _("Couldn't write to %s"), fname);
+	    errbox(_("Couldn't write to %s"), fname);
 	} else {
-	    sprintf(errtext, _("Couldn't open %s"), fname);
+	    errbox(_("Couldn't open %s"), fname);
 	}
-        errbox(errtext);
     }
 
     return fp;
@@ -388,8 +385,7 @@ static int gnuplot_png_init (GPT_SPEC *spec, FILE **fpp)
     *fpp = gretl_fopen(spec->fname, "w");
 
     if (*fpp == NULL) {
-	sprintf(errtext, _("Couldn't write to %s"), spec->fname);
-	errbox(errtext);
+	errbox(_("Couldn't write to %s"), spec->fname);
 	return 1;
     }
 
@@ -413,11 +409,7 @@ void display_session_graph_png (char *fname)
     }
 
     plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, myfname);
-#ifdef G_OS_WIN32
-    err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
-#else
     err = gretl_spawn(plotcmd);
-#endif
     g_free(plotcmd);
 
     if (err) {
@@ -427,7 +419,7 @@ void display_session_graph_png (char *fname)
     }
 }
 
-static int maybe_switch_emf_point_style (char *s, PRN *prn)
+int maybe_switch_emf_point_style (char *s, PRN *prn)
 {
     char *p = strstr(s, "w points");
     int do_pt2 = 0;
@@ -453,10 +445,6 @@ static int maybe_switch_emf_point_style (char *s, PRN *prn)
     return do_pt2;
 }
 
-#ifdef G_OS_WIN32
-static void win32_process_graph (GPT_SPEC *spec, int color, int dest);
-#endif
-
 void save_this_graph (GPT_SPEC *plot, const char *fname)
 {
     FILE *fq;
@@ -488,7 +476,7 @@ void save_this_graph (GPT_SPEC *plot, const char *fname)
 
 #ifdef ENABLE_NLS
 	pprint_gnuplot_encoding(termstr, prn);
-#endif /* ENABLE_NLS */
+#endif
 	pprintf(prn, "set term %s\n", termstr);
 	pprintf(prn, "set output '%s'\n", fname);
 	while (fgets(plotline, MAXLEN-1, fq)) {
@@ -506,12 +494,7 @@ void save_this_graph (GPT_SPEC *plot, const char *fname)
 
     plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, 
 			      plottmp);
-
-#ifdef G_OS_WIN32
-    err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
-#else
     err = gretl_spawn(plotcmd);
-#endif /* G_OS_WIN32 */
 
     remove(plottmp);
     g_free(plotcmd);
@@ -551,6 +534,9 @@ static int get_gpt_marker (const char *line, char *label)
     if (p != NULL) {
 	sprintf(format, "%%%ds", OBSLEN - 1);
 	sscanf(p + 1, format, label);
+#if GPDEBUG > 1
+	fprintf(stderr, "read marker: '%s'\n", label);
+#endif
 	return 0;
     }
 
@@ -641,7 +627,7 @@ static GPT_SPEC *plotspec_new (void)
     return spec;
 }
 
-static int get_gpt_data (GPT_SPEC *spec, int have_markers, FILE *fp)
+static int get_gpt_data (GPT_SPEC *spec, int do_markers, FILE *fp)
 {
     char s[MAXLEN];
     char *got;
@@ -700,7 +686,7 @@ static int get_gpt_data (GPT_SPEC *spec, int have_markers, FILE *fp)
 		}
 	    }
 
-	    if (i == 0 && have_markers) {
+	    if (i == 0 && do_markers) {
 		get_gpt_marker(s, spec->markers[t]);
 	    }
 	}
@@ -932,14 +918,14 @@ static int allocate_plotspec_markers (GPT_SPEC *spec)
    the data).
 */
 
-static int get_plot_nobs (FILE *fp, PlotType *ptype, int *have_markers)
+static int get_plot_nobs (FILE *fp, PlotType *ptype, int *do_markers)
 {
     int n = 0, started = -1;
     char line[MAXLEN], label[9];
     char *p;
 
     *ptype = PLOT_REGULAR;
-    *have_markers = 0;
+    *do_markers = 0;
 
     while (fgets(line, MAXLEN - 1, fp)) {
 
@@ -958,9 +944,9 @@ static int get_plot_nobs (FILE *fp, PlotType *ptype, int *have_markers)
 	}
 
 	if (started == 1) {
-	    if (*have_markers == 0 && (p = strchr(line, '#')) != NULL) {
+	    if (*do_markers == 0 && (p = strchr(line, '#')) != NULL) {
 		if (sscanf(p + 1, "%8s", label) == 1) {
-		    *have_markers = 1;
+		    *do_markers = 1;
 		}
 	    }
 	    if (*line == 'e') {
@@ -1055,24 +1041,106 @@ static void maybe_set_all_markers_ok (GPT_SPEC *spec)
     if (spec->n_lines <= 2 &&
 	spec->lines[0].ncols == 2 &&
 	spec->lines[1].ncols == 0 &&
-	spec->markers != NULL &&
-	spec->n_markers > 0 &&
-	spec->n_markers < 55) {
+	spec->n_markers > 0) {
 	spec->flags |= GPTSPEC_ALL_MARKERS_OK;
+#if GPDEBUG > 1
+	fprintf(stderr, "set GPTSPEC_ALL_MARKERS_OK\n");
+#endif
     } else {
 	spec->flags &= ~GPTSPEC_ALL_MARKERS_OK;
+#if GPDEBUG > 1
+	fprintf(stderr, "unset GPTSPEC_ALL_MARKERS_OK\n");
+#endif
     }
 }
 
+static int 
+plot_get_data_and_markers (GPT_SPEC *spec, FILE *fp, int datacols, 
+			   int do_markers)
+{
+    int err = 0;
+
+#if GPDEBUG
+    fprintf(stderr, "allocating: nobs=%d, datacols=%d, size=%d\n", 
+	    spec->nobs, datacols, spec->nobs * datacols * sizeof *spec->data);
+#endif  
+
+    /* allocate for the plot data... */
+    spec->data = mymalloc(spec->nobs * datacols * sizeof *spec->data);
+    if (spec->data == NULL) {
+	err = 1;
+    }
+
+    /* and markers if any */
+    if (!err && do_markers) {
+	if (allocate_plotspec_markers(spec)) {
+	    free(spec->data);
+	    spec->data = NULL;
+	    err = 1;
+	}
+    }
+
+    /* Read the data (and perhaps markers) from the plot file */
+    if (!err) {
+	err = get_gpt_data(spec, do_markers, fp);
+    }
+
+#if GPDEBUG
+    fprintf(stderr, "plot_get_data_and_markers:\n"
+	    " spec->data = %p, spec->markers = %p, spec->n_markers = %d, err = %d\n",
+	    spec->data, (void *) spec->markers, spec->n_markers, err);
+#endif
+
+    return err;
+}
+
+static int uneditable_get_markers (GPT_SPEC *spec, FILE *fp, int *polar)
+{
+    char line[256];
+    long offset = 0;
+    int gotit = 0;
+    int err = 0;
+
+    rewind(fp);
+
+    /* advance to the right line (with data plus markers) */
+    while (fgets(line, sizeof line, fp)) {
+	if ((isdigit(*line) || *line == '-') && strchr(line, '#')) {
+	    gotit = 1;
+	    break;
+	} else if (strstr(line, "set polar")) {
+	    *polar = 1;
+	}
+	offset = ftell(fp);
+    }
+
+    if (!gotit) {
+	return 1;
+    } else {
+	fseek(fp, offset, SEEK_SET);
+    }
+
+    spec->n_lines = 1;
+    spec->lines[0].ncols = 2;
+
+    err = plot_get_data_and_markers(spec, fp, 2, 1);
+
+    if (!err) {
+	maybe_set_all_markers_ok(spec);
+    }
+
+    return err;
+}
+
 /* Read plotspec struct from gnuplot command file.  This is _not_ a
    general parser for gnuplot files; it is designed specifically for
    files auto-generated by gretl.
 */
 
-static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd)
+static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 {
     int i, done, labelno;
-    int have_markers = 0;
+    int do_markers = 0;
     int datacols = 0;
     int reglist[4] = {0};
     char gpline[MAXLEN];
@@ -1101,7 +1169,7 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd)
     }
 
     /* get the number of data-points, plot type, and check for markers */
-    spec->nobs = get_plot_nobs(fp, &spec->code, &have_markers);
+    spec->nobs = get_plot_nobs(fp, &spec->code, &do_markers);
     if (spec->nobs == 0 && spec->code != PLOT_ELLIPSE) {
 	/* failed reading plot data */
 #if GPDEBUG
@@ -1109,8 +1177,17 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd)
 #endif
 	fclose(fp);
 	return 1;
-    } else if (cant_edit(spec->code)) {
+    }
+
+    if (spec->nobs > MAX_MARKERS && do_markers) {
+	do_markers = 0;
+    }
+
+    if (cant_edit(spec->code)) {
 	fprintf(stderr, "read_plotspec_from_file: plot is not editable\n");
+	if (do_markers) {
+	    uneditable_get_markers(spec, fp, polar);
+	}
 	fclose(fp);
 	return 0;
     }
@@ -1257,37 +1334,9 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd)
 	}
     }
 
-#if GPDEBUG
-    fprintf(stderr, "allocating: nobs=%d, datacols=%d, size=%d\n", 
-	    spec->nobs, datacols, spec->nobs * datacols * sizeof *spec->data);
-#endif    
+    err = plot_get_data_and_markers(spec, fp, datacols, do_markers);
 
-    /* allocate for the plot data... */
-    spec->data = mymalloc(spec->nobs * datacols * sizeof *spec->data);
-    if (spec->data == NULL) {
-	err = 1;
-	goto plot_bailout;
-    }
-
-    /* and markers if any */
-    if (have_markers) {
-	if (allocate_plotspec_markers(spec)) {
-	    free(spec->data);
-	    spec->data = NULL;
-	    err = 1;
-	    goto plot_bailout;
-	}
-    }
-
-    /* Read the data (and markers) from the plot file */
-    err = get_gpt_data(spec, have_markers, fp);
-
-#if GPDEBUG
-    fprintf(stderr, "spec->markers = %p, spec->n_markers = %d\n",
-	    (void *) spec->markers, spec->n_markers);
-#endif
-
-    if (reglist[0] > 0) {
+    if (!err && reglist[0] > 0) {
 	spec->reglist = gretl_list_copy(reglist);
     }
 
@@ -1311,6 +1360,9 @@ static int get_data_xy (png_plot *plot, int x, int y,
 {
     double xmin, xmax;
     double ymin, ymax;
+    double dx = NADBL;
+    double dy = NADBL;
+    int ok = 1;
 
     if (plot_is_zoomed(plot)) {
 	xmin = plot->zoom_xmin;
@@ -1324,7 +1376,7 @@ static int get_data_xy (png_plot *plot, int x, int y,
 	ymax = plot->ymax;
     }
 
-#ifdef POINTS_DEBUG
+#if POINTS_DEBUG
     if (plot_doing_position(plot)) {
 	fprintf(stderr, "get_data_xy:\n"
 		" plot->xmin=%g, plot->xmax=%g, plot->ymin=%g, plot->ymax=%g\n",
@@ -1332,25 +1384,38 @@ static int get_data_xy (png_plot *plot, int x, int y,
     }
 #endif
 
-    if (xmin == 0.0 && xmax == 0.0) { /* unknown x range */
+    if (xmin == 0.0 && xmax == 0.0) { 
+	/* unknown x range */
 	fprintf(stderr, "get_data_xy: unknown x range\n");
-	*data_x = NADBL;
     } else {
-	*data_x = xmin + ((double) x - plot->pixel_xmin) / 
+	dx = xmin + ((double) x - plot->pixel_xmin) / 
 	    (plot->pixel_xmax - plot->pixel_xmin) * (xmax - xmin);
     }
 
-    if (!na(*data_x)) {
-	if (ymin == 0.0 && ymax == 0.0) { /* unknown y range */
+    if (!na(dx)) {
+	if (ymin == 0.0 && ymax == 0.0) { 
+	    /* unknown y range */
 	    fprintf(stderr, "get_data_xy: unknown y range\n");
-	    *data_y = NADBL;
 	} else {
-	    *data_y = ymax - ((double) y - plot->pixel_ymin) / 
+	    dy = ymax - ((double) y - plot->pixel_ymin) / 
 		(plot->pixel_ymax - plot->pixel_ymin) * (ymax - ymin);
 	}
     }
 
-    return (!na(*data_x) && !na(*data_y));
+    if (na(dx) || na(dx)) {
+	ok = 0;
+    } else if (plot_is_polar(plot)) {
+	double px = atan2(dy, dx);
+	double py = sqrt(dx * dx + dy * dy);
+
+	dx = px;
+	dy = py;
+    }
+
+    *data_x = dx;
+    *data_y = dy;
+
+    return ok;
 }
 
 static void x_to_date (double x, int pd, char *str)
@@ -1406,14 +1471,48 @@ static void draw_selection_rectangle (png_plot *plot,
 		       rx, ry, rw, rh);
 }
 
+static int make_alt_label (gchar *alt, const gchar *label)
+{
+    double x, y;
+    int err = 0;
+
+    gretl_push_c_numeric_locale();
+
+    if (sscanf(label, "%lf,%lf", &x, &y) != 2) {
+	err = 1;
+    }
+
+    gretl_pop_c_numeric_locale();
+
+    if (!err) {
+	if (get_local_decpoint() != '.') {
+	    sprintf(alt, "%.2f %.2f", x, y);
+	} else {
+	    sprintf(alt, "%.2f,%.2f", x, y);
+	}
+    }
+
+    return err;
+}
+
 #ifdef OLD_GTK
 
 static void
-write_label_to_plot (png_plot *plot, const gchar *label,
-		     gint x, gint y)
+write_label_to_plot (png_plot *plot, int i, gint x, gint y)
 {
+    const gchar *label = plot->spec->markers[i];
     static GdkFont *label_font;
 
+    if (plot_is_roots(plot)) {
+	gchar alt_label[12];
+	
+	if (make_alt_label(alt_label, label)) {
+	    return;
+	}
+
+	label = alt_label;
+    }
+
     if (plot->invert_gc == NULL) {
 	create_selection_gc(plot);
     }
@@ -1450,12 +1549,22 @@ write_label_to_plot (png_plot *plot, const gchar *label,
 #else
 
 static void
-write_label_to_plot (png_plot *plot, const gchar *label,
-		     gint x, gint y)
+write_label_to_plot (png_plot *plot, int i, gint x, gint y)
 {
+    const gchar *label = plot->spec->markers[i];
     PangoContext *context;
     PangoLayout *pl;
 
+    if (plot_is_roots(plot)) {
+	gchar alt_label[12];
+	
+	if (make_alt_label(alt_label, label)) {
+	    return;
+	}
+
+	label = alt_label;
+    }
+
     if (plot->invert_gc == NULL) {
 	create_selection_gc(plot);
     }
@@ -1490,12 +1599,18 @@ static gint
 identify_point (png_plot *plot, int pixel_x, int pixel_y,
 		double x, double y) 
 {
+    const double *data_x = NULL;
+    const double *data_y = NULL;
     double xrange, yrange;
     double xdiff, ydiff;
     double min_xdist, min_ydist;
     int best_match = -1;
     int t;
-    const double *data_x, *data_y = NULL;
+
+#if GPDEBUG > 2
+    fprintf(stderr, "identify_point: pixel_x = %d (x=%g), pixel_y = %d (y=%g)\n",
+	    pixel_x, x, pixel_y, y);
+#endif
 
     if (plot->err) {
 	return TRUE;
@@ -1510,11 +1625,10 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
 #ifndef OLD_GTK
     /* need array to keep track of which points are labeled */
     if (plot->spec->labeled == NULL) {
-	plot->spec->labeled = mymalloc(plot->spec->nobs);
+	plot->spec->labeled = calloc(plot->spec->nobs, 1);
 	if (plot->spec->labeled == NULL) {
 	    return TRUE;
 	}
-	memset(plot->spec->labeled, 0, plot->spec->nobs);
     }
 #endif
 
@@ -1554,6 +1668,9 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
 	if (na(data_x[t]) || na(data_y[t])) {
 	    continue;
 	}
+#if GPDEBUG > 2
+	fprintf(stderr, "considering t=%d: x=%g, y=%g\n", t, data_x[t], data_y[t]);
+#endif
 	xdiff = fabs(data_x[t] - x);
 	ydiff = fabs(data_y[t] - y);
 	if (xdiff <= min_xdist && ydiff <= min_ydist) {
@@ -1570,11 +1687,17 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
     }
 #endif
 
+#if GPDEBUG > 2
+    fprintf(stderr, " best_match=%d, with data_x[%d]=%g, data_y[%d]=%g\n", 
+	    best_match, best_match, data_x[best_match], 
+	    best_match, data_y[best_match]);
+#endif
+
     /* if the match is good enough, show the label */
-    if (best_match >= 0 && min_xdist < TOLDIST * xrange &&
+    if (best_match >= 0 && 
+	min_xdist < TOLDIST * xrange &&
 	min_ydist < TOLDIST * yrange) {
-	write_label_to_plot(plot, plot->spec->markers[best_match],
-			    pixel_x, pixel_y);
+	write_label_to_plot(plot, best_match, pixel_x, pixel_y);
 #ifndef OLD_GTK
 	/* flag the point as labeled already */
 	plot->spec->labeled[best_match] = 1;
@@ -1584,15 +1707,12 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
     return TRUE;
 }
 
-#define MAX_MARKERS_SHOWN 250
-
 static gint
-motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
-		     png_plot *plot)
+motion_notify_event (GtkWidget *widget, GdkEventMotion *event, png_plot *plot)
 {
-    int x, y;
     GdkModifierType state;
     gchar label[32], label_y[16];
+    int x, y;
 
     if (plot->err) {
 	return TRUE;
@@ -1616,7 +1736,6 @@ motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 	if (na(data_x)) return TRUE;
 
 	if (!plot_has_no_markers(plot) && !plot_show_all_markers(plot) &&
-	    datainfo->t2 - datainfo->t1 < MAX_MARKERS_SHOWN &&
 	    !plot_is_zooming(plot) &&
 	    !na(data_y)) {
 	    identify_point(plot, x, y, data_x, data_y);
@@ -1727,13 +1846,15 @@ static gint color_popup_activated (GtkWidget *w, gpointer data)
 	strcpy(plot->spec->termtype, "postscript");
 	if (color) {
 	    strcat(plot->spec->termtype, " color");
-	}
+	} 
 	file_selector(_("Save gnuplot graph"), SAVE_THIS_GRAPH, 
 		      FSEL_DATA_MISC, plot->spec);
     } else if (!strcmp(parent_item, _("Save as Windows metafile (EMF)..."))) {
 	strcpy(plot->spec->termtype, "emf");
 	if (color) {
 	    strcat(plot->spec->termtype, " color");
+	} else {
+	    strcat(plot->spec->termtype, " mono");
 	}
 	file_selector(_("Save gnuplot graph"), SAVE_THIS_GRAPH, 
 		      FSEL_DATA_MISC, plot->spec);
@@ -1749,6 +1870,61 @@ static gint color_popup_activated (GtkWidget *w, gpointer data)
     return TRUE;
 }
 
+static void show_numbers_from_markers (GPT_SPEC *spec)
+{
+    PRN *prn;
+    double x, y;
+    double mod, freq;
+    int i, err = 0;
+
+    if (bufopen(&prn)) {
+	return;
+    } 
+
+    pputs(prn, _("VAR roots (real, imaginary, modulus, frequency)"));
+    pputs(prn, "\n\n");
+
+    if (get_local_decpoint() != '.') {
+	gretl_push_c_numeric_locale();
+	for (i=0; i<spec->n_markers; i++) {
+	    if (sscanf(spec->markers[i], "%lf,%lf", &x, &y) == 2) {
+		freq = spec->data[i] / (2.0 * M_PI);
+		mod = spec->data[spec->nobs + i];
+		gretl_pop_c_numeric_locale();
+		pprintf(prn, "%2d: (%7.4f  %7.4f  %7.4f  %7.4f)\n", i+1, 
+			x, y, mod, freq);
+		gretl_push_c_numeric_locale();
+	    } else {
+		err = E_DATA;
+		break;
+	    }
+	}
+	gretl_pop_c_numeric_locale();
+    } else {
+	for (i=0; i<spec->n_markers; i++) {
+	    if (sscanf(spec->markers[i], "%lf,%lf", &x, &y) == 2) {
+		freq = spec->data[i] / (2.0 * M_PI);
+		mod = spec->data[spec->nobs + i];
+		pprintf(prn, "%2d: (%7.4f, %7.4f, %7.4f, %7.4f)\n", i+1, 
+			x, y, mod, freq);
+	    } else {
+		err = E_DATA;
+		break;
+	    }
+	}
+    }
+
+    if (err) {
+	gui_errmsg(err);
+	gretl_print_destroy(prn);
+    } else {
+	gchar *title = g_strdup_printf("gretl: %s", _("VAR roots"));
+
+	view_buffer(prn, 72, 340, title, PRINT, NULL);
+	g_free(title);	
+    }
+}
+
 static gint plot_popup_activated (GtkWidget *w, gpointer data)
 {
     gchar *item = (gchar *) data;
@@ -1802,6 +1978,8 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
 #endif 
     else if (!strcmp(item, _("OLS estimates"))) { 
 	do_graph_model(plot->spec);
+    } else if (!strcmp(item, _("Numerical values"))) {
+	show_numbers_from_markers(plot->spec);
     } else if (!strcmp(item, _("Edit"))) { 
 	start_editing_png_plot(plot);
     } else if (!strcmp(item, _("Close"))) { 
@@ -1868,6 +2046,7 @@ static void build_plot_menu (png_plot *plot)
 	N_("Print"),
 #endif
 	N_("OLS estimates"),
+	N_("Numerical values"),
 	N_("Edit"),
 	N_("Help"),
         N_("Close"),
@@ -1932,6 +2111,11 @@ static void build_plot_menu (png_plot *plot)
 	    !strcmp(plot_items[i], "OLS estimates")) {
 	    i++;
 	    continue;
+	}
+	if (!plot_is_roots(plot) && 
+	    !strcmp(plot_items[i], "Numerical values")) {
+	    i++;
+	    continue;
 	}	
 
         item = gtk_menu_item_new_with_label(_(plot_items[i]));
@@ -1983,13 +2167,7 @@ int redisplay_edited_png (png_plot *plot)
     /* get gnuplot to create a new PNG graph */
     plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, 
 			      plot->spec->fname);
-
-#ifdef G_OS_WIN32
-    err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
-#else    
     err = gretl_spawn(plotcmd);
-#endif
-
     g_free(plotcmd);
 
     if (err) {
@@ -2057,11 +2235,7 @@ static int zoom_unzoom_png (png_plot *plot, int view)
 				  plot->spec->fname);
     }
 
-#ifdef G_OS_WIN32
-    err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
-#else
     err = gretl_spawn(plotcmd);
-#endif
     g_free(plotcmd);  
 
     if (view == PNG_ZOOM) {
@@ -2218,12 +2392,7 @@ static int test_file_open (const char *fname)
 
     fp = fopen(fname, "r");
     if (fp == NULL) {
-	char *errstr = strerror(errno);
-
-	sprintf(errtext, _("Couldn't open %s"), fname);
-	strcat(errtext, ": ");
-	strcat(errtext, errstr);
-	errbox(errtext);
+	errbox(_("Couldn't open %s: %s"), fname, strerror(errno));
 	err = 1;
     } else {
 	fclose(fp);
@@ -2376,7 +2545,7 @@ static void set_approx_pixel_bounds (png_plot *plot,
 	plot->pixel_xmax -= 11;
     }
 
-#ifdef POINTS_DEBUG
+#if POINTS_DEBUG
     fprintf(stderr, "set_approx_pixel_bounds():\n"
 	    " xmin=%d xmax=%d ymin=%d ymax=%d\n", 
 	    plot->pixel_xmin, plot->pixel_xmax,
@@ -2433,18 +2602,13 @@ static int get_dumb_plot_yrange (png_plot *plot)
 
     plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot,
 			      dumbgp);
-
-#ifdef G_OS_WIN32
-    err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
-#else
     err = gretl_spawn(plotcmd);
-#endif
-    
     g_free(plotcmd);
+
     remove(dumbgp);
 
     if (err) {
-#ifdef POINTS_DEBUG
+#if POINTS_DEBUG
 	fputs("get_dumb_plot_yrange(): plot command failed\n", stderr);
 #endif
 	return 1;
@@ -2478,7 +2642,7 @@ static int get_dumb_plot_yrange (png_plot *plot)
 		continue; 
 	    }
 	    if (sscanf(s, "%lf", &y[i]) == 1) {
-#ifdef POINTS_DEBUG
+#if POINTS_DEBUG
 		fprintf(stderr, "from text plot: read y[%d]=%g\n",
 			i, y[i]);
 #endif
@@ -2499,7 +2663,7 @@ static int get_dumb_plot_yrange (png_plot *plot)
 	gretl_pop_c_numeric_locale();
 
 	fclose(fpin);
-#ifndef POINTS_DEBUG
+#if (POINTS_DEBUG == 0)
 	remove(dumbtxt);
 #endif
 
@@ -2515,7 +2679,7 @@ static int get_dumb_plot_yrange (png_plot *plot)
 	    }
 	}	    
 
-#ifdef POINTS_DEBUG
+#if POINTS_DEBUG
 	fprintf(stderr, "Reading y range from text plot: plot->ymin=%g, "
 		"plot->ymax=%g\n", plot->ymin, plot->ymax);
 #endif
@@ -2598,7 +2762,7 @@ static int get_plot_ranges (png_plot *plot)
 	plot->xmax = b.xmax;
 	plot->ymin = b.ymin;
 	plot->ymax = b.ymax;
-# ifdef POINTS_DEBUG
+# if POINTS_DEBUG
 	fprintf(stderr, "get_png_bounds_info():\n"
 		" xmin=%d xmax=%d ymin=%d ymax=%d\n", 
 		plot->pixel_xmin, plot->pixel_xmax,
@@ -2639,7 +2803,7 @@ static int get_plot_ranges (png_plot *plot)
 	}
     } else {
 	plot->status |= (PLOT_DONT_ZOOM | PLOT_DONT_MOUSE);
-#ifdef POINTS_DEBUG 
+#if POINTS_DEBUG 
 	fputs("get_plot_ranges: setting PLOT_DONT_ZOOM, PLOT_DONT_MOUSE\n", 
 	      stderr);
 #endif
@@ -2693,6 +2857,7 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
     GtkWidget *label_frame = NULL;
     GtkWidget *status_hbox = NULL;
     png_plot *plot;
+    int polar = 0;
     int err = 0;
 
 #if GPDEBUG
@@ -2727,12 +2892,21 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
        flag this, but it's not necessarily a show-stopper in
        terms of simply displaying the graph. 
     */
-    err = read_plotspec_from_file(plot->spec, &plot->pd);
+    err = read_plotspec_from_file(plot->spec, &plot->pd, &polar);
+
     if (err) {
 	plot->err = 1;
 	plot->status |= (PLOT_DONT_EDIT | PLOT_DONT_ZOOM | PLOT_DONT_MOUSE);
     } else if (cant_edit(plot->spec->code)) {
-	plot->status |= (PLOT_DONT_EDIT | PLOT_DONT_ZOOM | PLOT_DONT_MOUSE);
+	if (plot->spec->n_markers > 0) {
+	    plot->status |= (PLOT_DONT_EDIT | PLOT_DONT_ZOOM);
+	    get_plot_ranges(plot);
+	    if (polar) {
+		plot->format |= PLOT_POLAR;
+	    }
+	} else {
+	    plot->status |= (PLOT_DONT_EDIT | PLOT_DONT_ZOOM | PLOT_DONT_MOUSE);
+	}
     } else {
 	set_plot_format_flags(plot);
 	get_plot_ranges(plot);
@@ -2966,9 +3140,8 @@ static int get_png_bounds_info (png_bounds *bounds)
 
     if (png_sig_cmp(header, 0, PNG_CHECK_BYTES)) {
 	fclose(fp);
-	sprintf(errtext, "Bad PNG header: Got bytes %x %x %x %x", 
-		header[0],header[1],header[2],header[3]);
-	errbox(errtext);
+	errbox("Bad PNG header: Got bytes %x %x %x %x", 
+	       header[0],header[1],header[2],header[3]);
 	return GRETL_PNG_NOT_PNG;
     }
 
@@ -3036,126 +3209,3 @@ static int get_png_bounds_info (png_bounds *bounds)
 }
 
 #endif /* PNG_COMMENTS */
-
-#ifdef G_OS_WIN32
-
-/* win32: copy plot to clipboard by generating an EMF file (enhanced
-   metafile), reading it into a buffer, and putting it on the
-   clipboard.
-
-   Weirdness: when an emf is put on the clipboard as below, Word 2000
-   behaves thus: a straight "Paste" puts in a version of the graph
-   with squashed up numbers on the axes and no legend text; but a
-   "Paste special" (where one accepts the default of pasting it as an
-   enhanced metafile) puts in an accurate version with correct text.
-   Go figure.  (This is on win98)
-*/
-
-#include <gdk/gdkwin32.h>
-#include "guiprint.h"
-
-static int emf_to_clip (char *emfname)
-{
-    HWND mainw;
-    HENHMETAFILE hemf, hemfclip;
-    HANDLE htest;
-
-    mainw = GDK_WINDOW_HWND(mdata->w->window);
-    if (mainw == NULL) {
-	errbox("Got NULL HWND");
-	return 1;
-    }	
-
-    if (!OpenClipboard(mainw)) {
-	errbox(_("Cannot open the clipboard"));
-	return 1;
-    }
-
-    EmptyClipboard();
-
-    hemf = GetEnhMetaFile(emfname);
-    if (hemf == NULL) {
-	errbox("Couldn't get handle to graphic metafile");
-	return 1;
-    }
-
-    hemfclip = CopyEnhMetaFile(hemf, NULL);
-    if (hemfclip == NULL) {
-	errbox("Couldn't copy graphic metafile");
-	return 1;
-    }    
-
-    htest = SetClipboardData(CF_ENHMETAFILE, hemfclip);
-    if (htest == NULL) {
-	errbox("Failed to put data on clipboard");
-	return 1;
-    }  	
-
-    CloseClipboard();
-
-    DeleteEnhMetaFile(hemf);
-
-    return 0;
-}
-
-static void win32_process_graph (GPT_SPEC *spec, int color, int dest)
-{
-    FILE *fq;
-    PRN *prn;
-    char plottmp[MAXLEN], plotline[MAXLEN];
-    gchar *plotcmd = NULL;
-    gchar *emfname = NULL;
-    int err, done_pt2 = 0;
-
-    /* create temporary file to hold the special gnuplot commands */
-    if (user_fopen("gptout.tmp", plottmp, &prn)) return;
-
-    /* open the gnuplot source file for the graph */
-    fq = gretl_fopen(spec->fname, "r");
-    if (fq == NULL) {
-	errbox(_("Couldn't access graph info"));
-	gretl_print_destroy(prn);
-	return;
-    }
-
-    /* generate gnuplot source file to make emf */
-    pprintf(prn, "%s\n", get_gretl_emf_term_line(spec->code, color));
-    emfname = g_strdup_printf("%sgpttmp.emf", paths.userdir);
-    pprintf(prn, "set output '%s'\n", emfname);
-    pprintf(prn, "set size 0.8,0.8\n");
-    while (fgets(plotline, MAXLEN-1, fq)) {
-	if (!done_pt2 && strstr(plotline, "using 1:2")) {
-	    done_pt2 = maybe_switch_emf_point_style(plotline, prn);
-	} else if (strncmp(plotline, "set term", 8) && 
-	    strncmp(plotline, "set output", 10)) {
-	    pputs(prn, plotline);
-	}
-    }
-
-    gretl_print_destroy(prn);
-    fclose(fq);
-
-    /* get gnuplot to create the emf file */
-    plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, 
-			      plottmp);
-    err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
-    g_free(plotcmd);
-    remove(plottmp);
-    
-    if (err) {
-        errbox(_("Gnuplot error creating graph"));
-    } else if (dest == WIN32_TO_CLIPBOARD) {
-	err = emf_to_clip(emfname);
-	if (!err) {
-	    infobox(_("To paste, use Edit/Paste special.../Enhanced metafile"));
-	}
-    } else if (dest == WIN32_TO_PRINTER) {
-	err = winprint_graph(emfname);
-    }
-
-    remove(emfname);
-    g_free(emfname);
-}
-
-#endif /* G_OS_WIN32 */
-
