@@ -23,6 +23,7 @@
 #include "gpt_control.h"
 #include "session.h"
 #include "dlgutils.h"
+#include "fileselect.h"
 
 #include "../pixmaps/mouse.xpm"
 
@@ -53,6 +54,7 @@ static GtkWidget *keycombo;
 static GtkWidget *termcombo;
 static GtkWidget *fitline_check;
 static GtkWidget *border_check;
+static GtkWidget *markers_check;
 static GtkWidget *y2_check;
 static GtkWidget *ttfcombo;
 static GtkWidget *ttfspin;
@@ -75,11 +77,11 @@ struct gpt_titles_t gpt_titles[] = {
 
 #define frequency_plot(s)       (s->code == PLOT_FREQ_SIMPLE || \
 			         s->code == PLOT_FREQ_NORMAL || \
-			         s->code == PLOT_FREQ_GAMMA)
+			         s->code == PLOT_FREQ_GAMMA || \
+                                 s->code == PLOT_KERNEL)
 
 static const char *get_font_filename (const char *showname);
 
-
 static void close_plot_controller (GtkWidget *widget, gpointer data) 
 {
     GPT_SPEC *spec = (GPT_SPEC *) data;
@@ -87,7 +89,13 @@ static void close_plot_controller (GtkWidget *widget, gpointer data)
 
     gpt_control = NULL;
 
-    if (plot != NULL) { /* PNG plot window open */
+    if (plot != NULL) { 
+	/* PNG plot window is open */
+#ifdef G_OS_WIN32 /* z-order gets messed up */
+	GtkWidget *shell = plot_get_shell(plot);
+
+	gdk_window_raise(shell->window);
+#endif
 	plot_remove_controller(plot);
     } else {
 	free_plotspec(spec); 
@@ -117,6 +125,7 @@ static void entry_to_gp_string (GtkWidget *w, char *targ, size_t n)
     const gchar *wstr;
     
     *targ = '\0';
+
     g_return_if_fail(GTK_IS_ENTRY(w));
     wstr = gtk_entry_get_text(GTK_ENTRY(w));
 
@@ -166,7 +175,7 @@ static double entry_to_gp_double (GtkWidget *w)
 	    gchar *tmp = g_strdup(s);
 
 	    charsub(tmp, ',', '.');
-	    setlocale(LC_NUMERIC, "C");
+	    gretl_push_c_numeric_locale();
 #else
 	    const gchar *tmp = s;
 #endif
@@ -176,7 +185,7 @@ static double entry_to_gp_double (GtkWidget *w)
 		ret = atof(tmp);
 	    }
 #ifdef ENABLE_NLS
-	    setlocale(LC_NUMERIC, "");
+	    gretl_pop_c_numeric_locale();
 	    g_free(tmp);
 #endif
 	}
@@ -289,19 +298,18 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	} 
     } 
 
-    if (!frequency_plot(spec)) {    
-	for (i=0; i<spec->nlines; i++) {
-	    spec->lines[i].yaxis = 1;
-	    if (supress_y2) {
-		continue;
-	    }
-	    yaxis = 
-		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(yaxiscombo[i])->entry));
-	    if (yaxis != NULL && *yaxis && !strcmp(yaxis, "right"))
-		spec->lines[i].yaxis = 2;	
-	    if (spec->lines[i].yaxis == 2) {
-		spec->flags |= GPTSPEC_Y2AXIS;
-	    }
+    for (i=0; i<spec->n_lines; i++) {
+	spec->lines[i].yaxis = 1;
+	if (supress_y2 || yaxiscombo[i] == NULL) {
+	    continue;
+	}
+	yaxis = 
+	    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(yaxiscombo[i])->entry));
+	if (yaxis != NULL && *yaxis && !strcmp(yaxis, "right")) {
+	    spec->lines[i].yaxis = 2;	
+	}
+	if (spec->lines[i].yaxis == 2) {
+	    spec->flags |= GPTSPEC_Y2AXIS;
 	}
     }
 
@@ -321,17 +329,23 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	}
     }
 
-    if (!err && !frequency_plot(spec)) {   
-	for (i=0; i<spec->nlines; i++) {
-	    entry_to_gp_string(GTK_COMBO(stylecombo[i])->entry, 
-			       spec->lines[i].style, 
-			       sizeof spec->lines[0].style);
-	    entry_to_gp_string(linetitle[i], 
-			       spec->lines[i].title, 
-			       sizeof spec->lines[0].title);
-	    entry_to_gp_string(linescale[i], 
-			       spec->lines[i].scale, 
-			       sizeof spec->lines[0].scale);
+    if (!err) {   
+	for (i=0; i<spec->n_lines; i++) {
+	    if (stylecombo[i] != NULL) {
+		entry_to_gp_string(GTK_COMBO(stylecombo[i])->entry, 
+				   spec->lines[i].style, 
+				   sizeof spec->lines[0].style);
+	    }
+	    if (linetitle[i] != NULL) {
+		entry_to_gp_string(linetitle[i], 
+				   spec->lines[i].title, 
+				   sizeof spec->lines[0].title);
+	    }
+	    if (linescale[i] != NULL) {
+		entry_to_gp_string(linescale[i], 
+				   spec->lines[i].scale, 
+				   sizeof spec->lines[0].scale);
+	    }
 	}
     }
 
@@ -363,9 +377,9 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 
     if (!err && border_check != NULL) {
 	if (GTK_TOGGLE_BUTTON(border_check)->active) {
-	    spec->flags &= ~GPTSPEC_BORDER_HIDDEN;
+	    spec->flags &= ~GPTSPEC_MINIMAL_BORDER;
 	} else {
-	    spec->flags |= GPTSPEC_BORDER_HIDDEN;
+	    spec->flags |= GPTSPEC_MINIMAL_BORDER;
 	}
     } 
 
@@ -377,6 +391,16 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	}
     }
 
+    if (!err && markers_check != NULL) {
+	if (GTK_TOGGLE_BUTTON(markers_check)->active) {
+	    free(spec->labeled);
+	    spec->labeled = NULL;
+	    spec->flags |= GPTSPEC_ALL_MARKERS;
+	} else {
+	    spec->flags &= ~GPTSPEC_ALL_MARKERS;
+	}
+    }
+
     if (!err && ttfcombo != NULL && ttfspin != NULL) {
 	const gchar *tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(ttfcombo)->entry));
 #ifdef OLD_GTK
@@ -400,7 +424,8 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 
     if (!err) {
 	if (save) { /* do something other than a screen graph? */
-	    file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, spec);
+	    file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
+			  FSEL_DATA_MISC, spec);
 	} else { 
 	    png_plot *plot = (png_plot *) spec->ptr;
 
@@ -411,26 +436,22 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
     }
 }
 
-/* ........................................................... */
-
 static void set_keyspec_sensitivity (GPT_SPEC *spec)
 {
-    if (!frequency_plot(spec)) {
-	int i; 
-	const char *p;
+    gboolean state = FALSE;
+    const char *p;
+    int i;
 
-	for (i=0; i<spec->nlines; i++) {
-	    p = gtk_entry_get_text(GTK_ENTRY(linetitle[i]));
-	    if (p != NULL && *p != 0) {
-		gtk_widget_set_sensitive(keycombo, TRUE);
-		return;
-	    }
+    for (i=0; i<spec->n_lines; i++) {
+	p = gtk_entry_get_text(GTK_ENTRY(linetitle[i]));
+	if (p != NULL && *p != 0) {
+	    state = TRUE;
+	    break;
 	}
     }
-    gtk_widget_set_sensitive(keycombo, FALSE);
-}
 
-/* ........................................................... */
+    gtk_widget_set_sensitive(keycombo, state);
+}
 
 #define TAB_MAIN_COLS 3
 
@@ -548,7 +569,7 @@ static void toggle_axis_selection (GtkWidget *w, GPT_SPEC *spec)
     int no_y2 = GTK_TOGGLE_BUTTON(w)->active;
     int i;
 
-    for (i=0; i<spec->nlines; i++) {
+    for (i=0; i<spec->n_lines; i++) {
 	if (i >= MAX_PLOT_LINES) {
 	    break;
 	}
@@ -632,7 +653,7 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 			      keycombo, 1, TAB_MAIN_COLS, tbl_len-1, tbl_len);
     gtk_combo_set_popdown_strings(GTK_COMBO(keycombo), keypos_list); 
     gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(keycombo)->entry), spec->keyspec);
-    gtk_widget_show (keycombo);	
+    gtk_widget_show(keycombo);	
 
     /* give option of removing top & right border */
     if (!(spec->flags & GPTSPEC_Y2AXIS)) { 
@@ -642,7 +663,7 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  border_check, 0, TAB_MAIN_COLS, 
 				  tbl_len-1, tbl_len);
-	if (!(spec->flags & GPTSPEC_BORDER_HIDDEN)) {
+	if (!(spec->flags & GPTSPEC_MINIMAL_BORDER)) {
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(border_check),
 					 TRUE);
 	}	
@@ -677,6 +698,22 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	fitline_check = NULL;
     }
 
+    /* give option of showing all case markers */
+    if (spec->flags & GPTSPEC_ALL_MARKERS_OK) { 
+	tbl_len++;
+	markers_check = gtk_check_button_new_with_label(_("Show all data labels"));
+	gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				  markers_check, 0, TAB_MAIN_COLS, 
+				  tbl_len-1, tbl_len);
+	if (spec->flags & GPTSPEC_ALL_MARKERS) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(markers_check),
+					 TRUE);
+	}	
+	gtk_widget_show(markers_check);
+    } else {
+	markers_check = NULL;
+    }
+
     /* set TT font (if gnuplot uses libgd and freetype) */
     if (gnuplot_has_ttf()) {
 	GtkWidget *ebox, *hsep;
@@ -686,7 +723,7 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	GList *fontnames = NULL;
 	struct font_info *ttflist;
 	const char *default_font = NULL;
-	int nfonts;
+	int nfonts = 0;
 
 	ttflist = get_gnuplot_ttf_list(&nfonts);
 
@@ -806,25 +843,28 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
     }
 }
 
-/* ........................................................... */
-
 static void gpt_tab_output (GtkWidget *notebook, GPT_SPEC *spec) 
 {
     GtkWidget *label, *vbox, *tbl;
     int i, tbl_len;
     GList *termlist = NULL;
-
     gchar *termtypes[] = {
 	"postscript",
 	"postscript color",
+	"PDF",
 	"fig",
 	"latex",
 	"png",
 	"plot commands",
 	NULL
-    };  
+    }; 
+    int pdf_ok = gnuplot_has_pdf();
+    
 
     for (i=0; termtypes[i] != NULL; i++) {
+	if (!pdf_ok && !strcmp(termtypes[i], "PDF")) {
+	    continue;
+	}
 	termlist = g_list_append(termlist, termtypes[i]);
     }
    
@@ -856,7 +896,7 @@ static void gpt_tab_output (GtkWidget *notebook, GPT_SPEC *spec)
     gtk_widget_show(termcombo);
 
     /* button to generate output to file */
-    filesavebutton = gtk_button_new_with_label (_("Save to file..."));
+    filesavebutton = gtk_button_new_with_label(_("Save to file..."));
     GTK_WIDGET_SET_FLAGS(filesavebutton, GTK_CAN_DEFAULT);
     tbl_len++;
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
@@ -880,6 +920,11 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
     char label_text[32];
     GList *plot_types = NULL;
     GList *yaxis_loc = NULL;
+    int do_scale_axis = 0;
+
+    if (spec->code == PLOT_REGULAR && (spec->flags & GPTSPEC_TS)) {
+	do_scale_axis = 1;
+    }
 
     if (spec->flags & GPTSPEC_TS) {
 	plot_types = g_list_append(plot_types, "lines");
@@ -892,9 +937,12 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
     plot_types = g_list_append(plot_types, "linespoints"); 
     plot_types = g_list_append(plot_types, "impulses");
     plot_types = g_list_append(plot_types, "dots");
+    plot_types = g_list_append(plot_types, "steps");
 
-    yaxis_loc = g_list_append(yaxis_loc, "left");
-    yaxis_loc = g_list_append(yaxis_loc, "right");
+    if (do_scale_axis) {
+	yaxis_loc = g_list_append(yaxis_loc, "left");
+	yaxis_loc = g_list_append(yaxis_loc, "right");
+    }
 
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
@@ -903,7 +951,7 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
     label = gtk_label_new(_("Lines"));
     gtk_widget_show(label);
 
-    if (spec->nlines > 4) {
+    if (spec->n_lines > 4) {
 	GtkWidget *scroller;
 	
 	scroller = gtk_scrolled_window_new(NULL, NULL);
@@ -928,7 +976,7 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
    
     tbl_num = tbl_col = 0;
 
-    for (i=0; i<spec->nlines; i++) {
+    for (i=0; i<spec->n_lines; i++) {
 	/* identifier and key or legend text */
 	tbl_len++;
 	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
@@ -970,56 +1018,66 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	stylecombo[i] = gtk_combo_new();
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  stylecombo[i], 2, 3, tbl_len-1, tbl_len);
-	gtk_combo_set_popdown_strings(GTK_COMBO(stylecombo[i]), plot_types); 
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(stylecombo[i])->entry), 
-			   spec->lines[i].style);  
+	/* errorbars style is not exchangeable with the others */
+	if (!strcmp(spec->lines[i].style, "errorbars")) {
+	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(stylecombo[i])->entry), 
+			       spec->lines[i].style); 
+	    gtk_widget_set_sensitive(stylecombo[i], FALSE);
+	} else {
+	    gtk_combo_set_popdown_strings(GTK_COMBO(stylecombo[i]), plot_types); 
+	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(stylecombo[i])->entry), 
+			       spec->lines[i].style); 
+	} 
 	gtk_widget_show(stylecombo[i]);	
 
-	/* scale factor for data */
-	tbl_len++;
-	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
-	label = gtk_label_new(_("scale"));
-	gtk_table_attach_defaults(GTK_TABLE(tbl), 
-				  label, 1, 2, tbl_len-1, tbl_len);
-	gtk_widget_show(label);
+	if (!do_scale_axis) {
+	    linescale[i] = NULL;
+	    yaxiscombo[i] = NULL;
+	} else {
+	    /* scale factor for data? */
+	    tbl_len++;
+	    gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
+	    label = gtk_label_new(_("scale"));
+	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				      label, 1, 2, tbl_len-1, tbl_len);
+	    gtk_widget_show(label);
 
-	linescale[i] = gtk_entry_new();
-	gtk_entry_set_max_length(GTK_ENTRY(linescale[i]), 6);
-	gtk_entry_set_text(GTK_ENTRY(linescale[i]), spec->lines[i].scale);
+	    linescale[i] = gtk_entry_new();
+	    gtk_entry_set_max_length(GTK_ENTRY(linescale[i]), 6);
+	    gtk_entry_set_text(GTK_ENTRY(linescale[i]), spec->lines[i].scale);
 #ifdef OLD_GTK
-	gtk_signal_connect(GTK_OBJECT(linescale[i]), "activate", 
-			   GTK_SIGNAL_FUNC(apply_gpt_changes), 
-			   spec);
+	    gtk_signal_connect(GTK_OBJECT(linescale[i]), "activate", 
+			       GTK_SIGNAL_FUNC(apply_gpt_changes), 
+			       spec);
 #else
-	gtk_entry_set_width_chars(GTK_ENTRY(linescale[i]), 6);
-	g_signal_connect(G_OBJECT(linescale[i]), "activate", 
-			 G_CALLBACK(apply_gpt_changes), 
-			 spec);
+	    gtk_entry_set_width_chars(GTK_ENTRY(linescale[i]), 6);
+	    g_signal_connect(G_OBJECT(linescale[i]), "activate", 
+			     G_CALLBACK(apply_gpt_changes), 
+			     spec);
 #endif
-	gtk_table_attach_defaults(GTK_TABLE(tbl), 
-				  linescale[i], 2, 3, tbl_len-1, tbl_len);
-	gtk_widget_show(linescale[i]);
+	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				      linescale[i], 2, 3, tbl_len-1, tbl_len);
+	    gtk_widget_show(linescale[i]);
 
-	/* use left or right y axis? */
-	tbl_len++;
-	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
-	label = gtk_label_new(_("y axis"));
-	gtk_table_attach_defaults(GTK_TABLE(tbl), 
-				  label, 1, 2, tbl_len-1, tbl_len);
-	gtk_widget_show(label);
+	    /* use left or right y axis? */
+	    tbl_len++;
+	    gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
+	    label = gtk_label_new(_("y axis"));
+	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				      label, 1, 2, tbl_len-1, tbl_len);
+	    gtk_widget_show(label);
 
-	yaxiscombo[i] = gtk_combo_new();
-	gtk_table_attach_defaults(GTK_TABLE(tbl), 
-				  yaxiscombo[i], 2, 3, tbl_len-1, tbl_len);
-	gtk_combo_set_popdown_strings(GTK_COMBO(yaxiscombo[i]), yaxis_loc); 
-	gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(yaxiscombo[i])->entry), 
-			    (spec->lines[i].yaxis == 1)? "left" : "right");  
-	gtk_widget_show(yaxiscombo[i]);	
+	    yaxiscombo[i] = gtk_combo_new();
+	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				      yaxiscombo[i], 2, 3, tbl_len-1, tbl_len);
+	    gtk_combo_set_popdown_strings(GTK_COMBO(yaxiscombo[i]), yaxis_loc); 
+	    gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(yaxiscombo[i])->entry), 
+				(spec->lines[i].yaxis == 1)? "left" : "right");  
+	    gtk_widget_show(yaxiscombo[i]);	
+	} 
     }
 }
 
-/* ........................................................... */
-
 static void label_pos_to_entry (double *pos, GtkWidget *w)
 {
     if (!na(pos[0]) && !na(pos[1])) {
@@ -1179,8 +1237,6 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
     }
 }
 
-/* ........................................................... */
-
 static void gpt_tab_XY (GtkWidget *notebook, GPT_SPEC *spec, gint axis) 
 {
     GtkWidget *vbox, *manual, *tbl;
@@ -1325,10 +1381,14 @@ static void gpt_tab_XY (GtkWidget *notebook, GPT_SPEC *spec, gint axis)
     }
 }
 
-/* ........................................................... */
+void close_gnuplot_dialog (GtkWidget *w, gpointer p)
+{
+    gtk_widget_destroy(GTK_WIDGET(p));
+}
 
 int show_gnuplot_dialog (GPT_SPEC *spec) 
 {
+    png_plot *plot = (png_plot *) spec->ptr;
     GtkWidget *button, *notebook;
     int i;
 
@@ -1357,6 +1417,11 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
     gtk_box_set_homogeneous(GTK_BOX(GTK_DIALOG(gpt_control)->action_area), TRUE);
     gtk_window_set_position(GTK_WINDOW(gpt_control), GTK_WIN_POS_MOUSE);
 
+    if (plot != NULL) {
+	gtk_window_set_transient_for(GTK_WINDOW(gpt_control), 
+				     GTK_WINDOW(plot_get_shell(plot)));
+    }
+
     g_signal_connect(G_OBJECT(gpt_control), "destroy",
 		     G_CALLBACK(close_plot_controller), 
 		     (gpointer *) spec);
@@ -1374,10 +1439,7 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
 	gpt_tab_XY(notebook, spec, 2);
     }
 
-    if (!frequency_plot(spec)) {
-	gpt_tab_lines(notebook, spec);
-    }    
-
+    gpt_tab_lines(notebook, spec);
     gpt_tab_labels(notebook, spec); 
     gpt_tab_output(notebook, spec);
 
@@ -1398,16 +1460,16 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(apply_gpt_changes), spec);
     g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(delete_widget), gpt_control);
+		     G_CALLBACK(close_gnuplot_dialog), gpt_control);
     gtk_widget_show(button);
 
-    /* Close button (do not apply changes */
+    /* Close button (do not apply changes) */
     button = standard_button(GTK_STOCK_CLOSE);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(gpt_control)->action_area), 
 		       button, TRUE, TRUE, 0);
     g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(delete_widget), gpt_control);
+		     G_CALLBACK(close_gnuplot_dialog), gpt_control);
     gtk_widget_show(button);
 
     button = standard_button(GTK_STOCK_HELP);
