@@ -102,7 +102,6 @@ static const struct poptOption options[] = {
 
 windata_t *mdata;
 DATAINFO *datainfo;
-char *errtext;
 char cmdfile[MAXLEN], scriptfile[MAXLEN];
 char trydatfile[MAXLEN], tryscript[MAXLEN];
 PATHS paths;                /* useful paths */
@@ -209,7 +208,7 @@ GtkItemFactoryEntry data_items[] = {
       "<StockItem>", GTK_STOCK_OPEN },
     { N_("/File/Open data/sample file..."), "", display_files, TEXTBOOK_DATA, 
       "<StockItem>", GTK_STOCK_OPEN },
-    { N_("/File/Open data/sep1"), NULL, NULL, 0, "<Separator>", GNULL }, 
+    { "/File/Open data/sep1", NULL, NULL, 0, "<Separator>", GNULL }, 
     { N_("/File/Open data/Import"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/File/Open data/Import/CSV..."), NULL, open_data, OPEN_CSV, NULL, GNULL },
     { N_("/File/Open data/Import/ASCII..."), NULL, open_data, OPEN_ASCII, NULL, GNULL },
@@ -255,13 +254,13 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Send To..."), NULL, email_data, OPEN_DATA, "<StockItem>", GRETL_STOCK_MAIL },
 #endif
     { N_("/File/C_lear data set"), NULL, verify_clear_data, 0, "<StockItem>", GTK_STOCK_CLEAR },
-    { N_("/File/sep0"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/File/sep0", NULL, NULL, 0, "<Separator>", GNULL },
 
     /* File, Browse databases */
     { N_("/File/_Browse databases"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/File/Browse databases/_gretl native"), NULL, display_files, NATIVE_DB, NULL, GNULL },
     { N_("/File/Browse databases/_RATS 4"), NULL, display_files, RATS_DB, NULL, GNULL },
-    { N_("/File/Browse databases/sep1"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/File/Browse databases/sep1", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/File/Browse databases/on database _server"), NULL, display_files, REMOTE_DB, 
       "<StockItem>", GTK_STOCK_NETWORK },
 
@@ -290,9 +289,9 @@ GtkItemFactoryEntry data_items[] = {
       NULL, newdata_callback, 0, NULL, GNULL }, 
     { N_("/File/Create data set/simulation"), NULL, gretl_callback, 
       NULLDATA, NULL, GNULL },
-    { N_("/File/sep1"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/File/sep1", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/File/_View command log"), NULL, view_command_log, 0, NULL, GNULL },
-    { N_("/File/sep2a"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/File/sep2a", NULL, NULL, 0, "<Separator>", GNULL },
 
     /* File, command files */
     { N_("/File/Open command file"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -305,7 +304,7 @@ GtkItemFactoryEntry data_items[] = {
       "<StockItem>", GTK_STOCK_NEW },
     { N_("/File/New command file/Monte Carlo loop"), "", do_new_script, 1, 
       "<StockItem>", GTK_STOCK_NEW },
-    { N_("/File/sep3"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/File/sep3", NULL, NULL, 0, "<Separator>", GNULL },
 
     /* File, preferences */
     { N_("/File/_Preferences"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -317,7 +316,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Preferences/_Menu font..."), NULL, font_selector, 
       APP_FONT_SELECTION, "<StockItem>", GTK_STOCK_SELECT_FONT },
 #endif
-    { N_("/File/sep5"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/File/sep5", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/File/E_xit"), "<control>X", menu_exit_check, 0, "<StockItem>", GTK_STOCK_QUIT },
 
     /* Utilities menu */
@@ -326,11 +325,15 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Utilities/p-value finder"), NULL, stats_calculator, CALC_PVAL, NULL, GNULL },
     { N_("/Utilities/Test statistic calculator"), NULL, stats_calculator, CALC_TEST, 
       NULL, GNULL },
-    { N_("/Utilities/sep"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/Utilities/sep", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Utilities/Gretl console"), NULL, show_gretl_console, 0, NULL, GNULL },
-    { N_("/Utilities/sep2"), NULL, NULL, 0, "<Separator>", GNULL },
-    { N_("/Utilities/Start GNU R"), NULL, startRcallback, 0, NULL, GNULL },
+    { "/Utilities/sep2", NULL, NULL, 0, "<Separator>", GNULL },
+#if 0
+    { N_("/Utilities/Additional functions..."), NULL, display_files, FUNC_FILES, NULL, GNULL },
     { "/Utilities/sep3", NULL, NULL, 0, "<Separator>", GNULL },
+#endif
+    { N_("/Utilities/Start GNU R"), NULL, startRcallback, 0, NULL, GNULL },
+    { "/Utilities/sep4", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Utilities/NIST test suite"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Utilities/NIST test suite/basic"), NULL, do_nistcheck, 0, NULL, GNULL },
     { N_("/Utilities/NIST test suite/verbose"), NULL, do_nistcheck, 1, NULL, GNULL },
@@ -339,27 +342,27 @@ GtkItemFactoryEntry data_items[] = {
     /* Session menu */
     { N_("/_Session"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Session/_Icon view"), NULL, view_session, 0, NULL, GNULL },
-    { N_("/Session/sep0"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/Session/sep0", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Session/_Open..."), "", open_script, OPEN_SESSION, "<StockItem>", GTK_STOCK_OPEN },
-    { N_("/Session/sep1"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Session/sep1", NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Session/_Save"), "", save_session_callback, SAVE_AS_IS, 
       "<StockItem>", GTK_STOCK_SAVE },
     { N_("/Session/Save _as..."), "", save_session_callback, SAVE_RENAME, 
       "<StockItem>", GTK_STOCK_SAVE_AS },
 
     /* Data menu */
-    { N_("/_Data"), NULL, NULL, 0, "<Branch>", NULL },
-    { N_("/Data/_Display values"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/_Data"), NULL, NULL, 0, "<Branch>", GNULL },
+    { N_("/Data/_Display values"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Data/Display values/_all variables"), NULL, display_data, 0, NULL, GNULL },
     { N_("/Data/Display values/_selected variables..."), 
       NULL, display_selected, 0, NULL, GNULL },
     { N_("/Data/_Edit values"), NULL, spreadsheet_edit, 0, NULL, GNULL },
-    { "/Data/sepsort", NULL, NULL, 0, "<Separator>", NULL },
-    { N_("/Data/Sort variables"), NULL, NULL, 0, "<Branch>", NULL },
+    { "/Data/sepsort", NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Data/Sort variables"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Data/Sort variables/by ID number"), NULL, sort_varlist, 0, NULL, GNULL },
     { N_("/Data/Sort variables/by name"), NULL, sort_varlist, 1, NULL, GNULL },
-    { N_("/Data/sep1"), NULL, NULL, 0, "<Separator>", NULL },
-    { N_("/Data/_Graph specified vars"), NULL, NULL, 0, "<Branch>", NULL },
+    { "/Data/sep1", NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Data/_Graph specified vars"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Data/Graph specified vars/Time series plot..."), 
       NULL, selector_callback, GR_PLOT, NULL, GNULL },
     { N_("/Data/Graph specified vars/X-Y scatter..."), 
@@ -376,37 +379,33 @@ GtkItemFactoryEntry data_items[] = {
       NULL, selector_callback, GR_3D, NULL, GNULL },
     { N_("/Data/_Multiple scatterplots..."), 
       NULL, selector_callback, SCATTERS, NULL, GNULL },
-    { N_("/Data/sep2"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Data/sep2", NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Data/_Read info"), NULL, open_info, 0, NULL, GNULL },
     { N_("/Data/Edit _info"), NULL, edit_header, 0, NULL, GNULL },
     { N_("/Data/Print description"), NULL, print_report, 0, NULL, GNULL },
-    { N_("/Data/sep3"), NULL, NULL, 0, "<Separator>", NULL },
-
-    { N_("/Data/_Summary statistics"), NULL, NULL, 0, "<Branch>", NULL },
+    { "/Data/sep3", NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Data/_Summary statistics"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Data/_Summary statistics/_all variables"), NULL, 
       do_menu_op, SUMMARY, NULL, GNULL },
     { N_("/Data/_Summary statistics/_selected variables"), NULL, 
       do_menu_op, SUMMARY_SELECTED, NULL, GNULL },
-
-    { N_("/Data/_Correlation matrix"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/Data/_Correlation matrix"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Data/_Correlation matrix/_all variables"), 
       NULL, do_menu_op, CORR, NULL, GNULL },
     { N_("/Data/_Correlation matrix/_selected variables"), NULL, do_menu_op, 
       CORR_SELECTED, NULL, GNULL },
-
     { N_("/Data/_Principal components"), NULL, do_menu_op, PCA, NULL, GNULL },
     { N_("/Data/_Mahalanobis distances"), NULL, do_menu_op, MAHAL, NULL, GNULL },
-
-    { N_("/Data/sep4"), NULL, NULL, 0, "<Separator>", NULL },
-    { N_("/Data/Difference of means"), NULL, NULL, 0, "<Branch>", NULL },
+    { "/Data/sep4", NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Data/Difference of means"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Data/Difference of means/assuming equal variances..."), NULL, 
       selector_callback, MEANTEST, NULL, GNULL },
     { N_("/Data/Difference of means/assuming unequal variances..."), NULL, 
       selector_callback, MEANTEST2, NULL, GNULL },
     { N_("/Data/Difference of variances..."), NULL, selector_callback, VARTEST, NULL, 
       GNULL },
-    { N_("/Data/sep5"), NULL, NULL, 0, "<Separator>", NULL },
-    { N_("/Data/Add variables"), NULL, NULL, 0, "<Branch>", NULL },
+    { "/Data/sep5", NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Data/Add variables"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Data/Add variables/time trend"), NULL, add_index, 1, NULL, GNULL },
     { N_("/Data/Add variables/index variable"), NULL, add_index, 0, NULL, GNULL },
     { N_("/Data/Add variables/logs of selected variables"), NULL, 
@@ -424,14 +423,14 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/Add variables/periodic dummies"), NULL, add_dummies, 0, NULL, GNULL },
     { N_("/Data/Add variables/unit dummies"), NULL, add_dummies, 1, NULL, GNULL },
     { N_("/Data/Add variables/panel dummies"), NULL, add_dummies, 2, NULL, GNULL },
-    { N_("/Data/Add variables/sep"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Data/Add variables/sep", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Data/Add variables/random normal..."), NULL, 
       add_random_callback, GENR_NORMAL, NULL, GNULL },
     { N_("/Data/Add variables/random uniform..."), NULL, 
       add_random_callback, GENR_UNIFORM, NULL, GNULL },
     { N_("/Data/Add variables/seed generator..."), NULL, gretl_callback, 
       SETSEED, NULL, GNULL },
-    { N_("/Data/Add variables/sep2"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Data/Add variables/sep2", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Data/Add variables/Define _new variable..."), NULL, gretl_callback, 
       GENR, NULL, GNULL },
     { N_("/Data/Add observations..."), NULL, do_add_obs, 0, NULL, GNULL },
@@ -439,52 +438,53 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/Refresh window"), NULL, refresh_data, 0, NULL, GNULL },
 
     /* Sample menu */
-    { N_("/_Sample"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/_Sample"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Sample/_Set range..."), NULL, sample_range_dialog, SMPL, NULL, GNULL },
     { N_("/Sample/_Restore full range"), NULL, restore_sample_callback, 1, NULL, GNULL },
-    { N_("/Sample/sep1"), NULL, NULL, 0, "<Separator>", NULL }, 
+    { "/Sample/sep1", NULL, NULL, 0, "<Separator>", GNULL }, 
     { N_("/Sample/Dataset structure..."), NULL, data_structure_wizard, 0, NULL, GNULL },
     { N_("/Sample/Compact data..."), NULL, do_compact_data_set, 0, NULL, GNULL },
     { N_("/Sample/Expand data..."), NULL, do_expand_data_set, 0, NULL, GNULL },
-    { N_("/Sample/sep2"), NULL, NULL, 0, "<Separator>", NULL },   
+    { "/Sample/sep2", NULL, NULL, 0, "<Separator>", GNULL },   
     { N_("/Sample/_Define, based on dummy..."), NULL, sample_range_dialog, 
       SMPLDUM, NULL, GNULL },
     { N_("/Sample/_Restrict, based on criterion..."), NULL, gretl_callback, 
       SMPLBOOL, NULL, GNULL },
     { N_("/Sample/R_andom sub-sample..."), NULL, sample_range_dialog, SMPLRAND, NULL, GNULL },
-    { N_("/Sample/sep3"), NULL, NULL, 0, "<Separator>", NULL },  
+    { "/Sample/sep3", NULL, NULL, 0, "<Separator>", GNULL },  
     { N_("/Sample/Drop all obs with _missing values"), NULL, drop_all_missing, 
       0, NULL, GNULL },
     { N_("/Sample/_Count missing values"), NULL, count_missing, 0, NULL, GNULL },
     { N_("/Sample/Set missing _value code..."), NULL, gretl_callback, 
       GSETMISS, NULL, GNULL },
-    { N_("/Sample/sep4"), NULL, NULL, 0, "<Separator>", NULL },  
+    { "/Sample/sep4", NULL, NULL, 0, "<Separator>", GNULL },  
     { N_("/Sample/_Add case markers..."), NULL, open_data, OPEN_MARKERS, NULL, GNULL },
     { N_("/Sample/Remove case _markers"), NULL, do_remove_markers, 0, NULL, GNULL },
-    { N_("/Sample/sep5"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Sample/sep5", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Sample/Restructure panel..."), NULL, panel_restructure_dialog, 0, NULL, GNULL },
     { N_("/Sample/Transpose data..."), NULL, gui_transpose_data, 0, NULL, GNULL },
 
     /* Variable menu */
-    { N_("/_Variable"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/_Variable"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Variable/Find..."), NULL, find_var, 0, "<StockItem>", GTK_STOCK_FIND },
     { N_("/Variable/_Display values"), NULL, display_var, 0, NULL, GNULL },
     { N_("/Variable/_Summary statistics"), NULL, do_menu_op, 
       VAR_SUMMARY, NULL, GNULL },
     { N_("/Variable/_Frequency distribution"), NULL, do_menu_op, 
       FREQ, NULL, GNULL },
-    { N_("/Variable/Frequency plot"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/Variable/Frequency plot"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Variable/Frequency plot/simple"), NULL, do_freqplot, 0, NULL, GNULL },
     { N_("/Variable/Frequency plot/against Normal"), NULL, do_freqplot, 
       DIST_NORMAL, NULL, GNULL },
     { N_("/Variable/Frequency plot/against Gamma"), NULL, do_freqplot, 
       DIST_GAMMA, NULL, GNULL },
     { N_("/Variable/Estimated density plot..."), NULL, do_kernel, 0, NULL, GNULL },
+    { N_("/Variable/Gini coefficient"), NULL, do_gini, 0, NULL, GNULL },
     { N_("/Variable/Range-mean graph"), NULL, do_range_mean, 0, NULL, GNULL }, 
-    { N_("/Variable/sep1"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Variable/sep1", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Variable/_Time series plot"), NULL, ts_plot_var, 0, NULL, GNULL },
     { N_("/Variable/Correlogram"), NULL, do_corrgm, CORRGM, NULL, GNULL },
-    { N_("/Variable/Spectrum"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/Variable/Spectrum"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Variable/Spectrum/sample periodogram"), NULL, do_pergm, 0, NULL, GNULL }, 
     { N_("/Variable/Spectrum/Bartlett lag window"), NULL, do_pergm, 1, NULL, GNULL }, 
     { N_("/Variable/_Augmented Dickey-Fuller test"), NULL, unit_root_test, ADF, NULL, GNULL },
@@ -497,32 +497,29 @@ GtkItemFactoryEntry data_items[] = {
 #endif
     { N_("/Variable/Hurst exponent"), NULL, do_hurst, 0, NULL, GNULL }, 
     { N_("/Variable/Runs test"), NULL, do_menu_op, RUNS, NULL, GNULL }, 
-    { N_("/Variable/sep2"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Variable/sep2", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Variable/_Edit attributes"), NULL, varinfo_callback, 0, NULL, GNULL },
     { N_("/Variable/Set missing value code..."), NULL, gretl_callback, 
       VSETMISS, NULL, GNULL },
-    { N_("/Variable/sep3"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Variable/sep3", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Variable/Define _new variable..."), NULL, gretl_callback, GENR, NULL, GNULL },
 
     /* Model menu */
-    { N_("/_Model"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/_Model"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Model/_Ordinary Least Squares..."), NULL, model_callback, OLS, NULL, GNULL },
     { N_("/Model/_Weighted Least Squares..."), NULL, model_callback, WLS, NULL, GNULL },
-    { N_("/Model/sep1"),  NULL, NULL, 0, "<Separator>", GNULL },
-#if 0
-    { N_("/Model/HCC_M..."), NULL, model_callback, HCCM, NULL, GNULL },
-#endif
+    { "/Model/sep1",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/H_eteroskedasticity corrected..."), NULL, model_callback, 
       HSK, NULL, GNULL },
-    { N_("/Model/sep2"),  NULL, NULL, 0, "<Separator>", NULL },
-    { N_("/Model/Time series"), NULL, NULL, 0, "<Branch>", NULL },
-    { N_("/Model/sep3"),  NULL, NULL, 0, "<Separator>", NULL },
+    { "/Model/sep2",  NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Model/Time series"), NULL, NULL, 0, "<Branch>", GNULL },
+    { "/Model/sep3",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/_Two-Stage Least Squares..."), NULL, model_callback, TSLS, NULL, GNULL },
     { "/Model/sep4",  NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Model/_Logit..."), NULL, model_callback, LOGIT, NULL, GNULL },
     { N_("/Model/_Probit..."), NULL, model_callback, PROBIT, NULL, GNULL },
     { N_("/Model/To_bit..."), NULL, model_callback, TOBIT, NULL, GNULL },
-    { "/Model/sep5",  NULL, NULL, 0, "<Separator>", NULL },
+    { "/Model/sep5",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/Poi_sson..."), NULL, model_callback, POISSON, NULL, GNULL },
     { N_("/Model/Lo_gistic..."), NULL, model_callback, LOGISTIC, NULL, GNULL },
     { N_("/Model/Least _Absolute Deviation..."), NULL, model_callback, LAD, NULL, GNULL },
@@ -536,18 +533,18 @@ GtkItemFactoryEntry data_items[] = {
 #endif
 
     /* Help menu */
-    { N_("/_Help"), NULL, NULL, 0, "<LastBranch>", NULL },
-    { N_("/Help/_Command reference"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/_Help"), NULL, NULL, 0, "<LastBranch>", GNULL },
+    { N_("/Help/_Command reference"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Help/Command reference/plain text"), NULL, plain_text_cmdref, 0, "<StockItem>", 
       GRETL_STOCK_BOOK },
     { N_("/Help/Command reference/PDF"), NULL, display_pdf_help, 0, "<StockItem>", 
       GRETL_STOCK_PDF },
     { N_("/Help/_User's guide"), NULL, display_pdf_help, 1, "<StockItem>", 
       GRETL_STOCK_PDF },
-    { N_("/Help/sep1"), NULL, NULL, 0, "<Separator>", GNULL },
+    { "/Help/sep1", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Help/Check for _updates"), NULL, manual_update_query, 0, "<StockItem>", 
       GTK_STOCK_NETWORK },
-    { N_("/Help/sep2"), NULL, NULL, 0, "<Separator>", NULL },
+    { "/Help/sep2", NULL, NULL, 0, "<Separator>", GNULL },
 #if GTK_MINOR_VERSION >= 6
     { N_("/Help/_About gretl"), NULL, about_dialog, 0, "<StockItem>", GTK_STOCK_ABOUT }
 #else
@@ -757,10 +754,6 @@ int main (int argc, char *argv[])
     putenv("G_FILENAME_ENCODING=@locale");
 #endif     
 
-    if ((errtext = malloc(MAXLEN)) == NULL) {
-	noalloc(_("startup"));
-    }
-
     *tryscript = '\0';
     *scriptfile = '\0';
     *paths.datfile = '\0';
@@ -891,6 +884,7 @@ int main (int argc, char *argv[])
 	if (unmangle(filearg, paths.datfile)) {
 	    exit(EXIT_FAILURE);
 	}
+	my_filename_to_utf8(paths.datfile);
 #else
 	record_filearg(paths.datfile, filearg);
 #endif
@@ -943,7 +937,7 @@ int main (int argc, char *argv[])
 	    break;
 	}
 
-#ifdef notdef
+#if 0
 	if (paths.datfile[0] != 0) {
 	    my_filename_to_utf8(paths.datfile);
 	}
@@ -1012,8 +1006,7 @@ int main (int argc, char *argv[])
     /* try opening specified database */
     if (gui_get_data == OPT_DBOPEN) {
 	open_named_db_list(dbname);
-    }
-    else if (gui_get_data == OPT_WEBDB) {
+    } else if (gui_get_data == OPT_WEBDB) {
 	open_named_remote_db_list(dbname);
     }
 
@@ -1064,20 +1057,48 @@ static void check_varmenu_state (GtkTreeSelection *select, gpointer p)
 
 static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 {
+    GdkModifierType mods;
+
+    gdk_window_get_pointer(w->window, NULL, NULL, &mods);
+
 #if defined(HAVE_FLITE) || defined(G_OS_WIN32)
     if (key->keyval == GDK_a) {
 	audio_render_window(vwin, AUDIO_LISTBOX);
-    } else if (key->keyval == GDK_x) {
+    } else if (key->keyval == GDK_x && !(mods & GDK_MOD1_MASK)) {
 	audio_render_window(NULL, AUDIO_LISTBOX);
     }
 #endif
 
-    if (key->keyval == GDK_e || key->keyval == GDK_t) {
+    if (key->keyval == GDK_h || key->keyval == GDK_F1) {
+	/* invoke help */
+	plain_text_cmdref(NULL, 0, NULL);
+	return FALSE;
+    }
+
+    if (key->keyval == GDK_g) {
+	/* invoke genr */
+	gretl_callback(NULL, GENR, NULL);
+	return FALSE;
+    }
+
+    if (key->keyval == GDK_x && (mods & GDK_MOD1_MASK)) {
+	/* invoke minibuffer */
+	gretl_callback(NULL, MINIBUF, NULL);
+	return FALSE;
+    }
+
+    if (
+	key->keyval == GDK_Return                        /* display variable(s) */
+	|| key->keyval == GDK_Delete                     /* delete variable(s) */
+	|| key->keyval == GDK_e || key->keyval == GDK_F2 /* edit variable's info */
+	|| key->keyval == GDK_t                          /* graph variable */
+	) {
 	int selcount, row;
 
 	selcount = 
 	    selection_count(gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox)),
 			    &row);
+
 	if (selcount == 1 && row != 0) {
 	    gchar *varnum;
 
@@ -1085,13 +1106,23 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	    mdata->active_var = atoi(varnum);
 	    g_free(varnum);
 
-	    if (key->keyval == GDK_e) {
+	    if (key->keyval == GDK_e || key->keyval == GDK_F2) {
 		varinfo_dialog(mdata->active_var, 1);
-	    } 
-	    else if (key->keyval == GDK_t) {
+	    } else if (key->keyval == GDK_t) {
 		do_graph_var(mdata->active_var);
-	    } 
+	    } else if (key->keyval == GDK_Return) {
+		display_var();
+	    } else if (key->keyval == GDK_Delete) {
+		delete_selected_vars(mdata->active_var);
+	    }
+	} else if (selcount > 1) {
+	    if (key->keyval == GDK_Delete) {
+		delete_selected_vars(0);
+	    } else if (key->keyval == GDK_Return) {
+		display_selected(NULL, 0, NULL);
+	    }
 	}
+	    
     } 
 
     return FALSE;
@@ -1207,11 +1238,11 @@ static void sort_varlist (gpointer p, guint col, GtkWidget *w)
 
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(mdata->listbox));
 
-    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE(model), 0,
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), 0,
 				     compare_var_ids, NULL, NULL);
-    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE(model), 1,
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), 1,
 				     compare_varnames, NULL, NULL);
-    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(model), 
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 
 					  col, GTK_SORT_ASCENDING);
 }
 
@@ -1399,11 +1430,11 @@ static void set_up_main_menu (void)
     mdata->mbar = gtk_item_factory_get_widget(mdata->ifac, "<main>");
 }
 
-int restore_sample (gretlopt opt)
+int gui_restore_sample (void)
 {
     int err;
 
-    err = restore_full_sample(&Z, &datainfo, opt);
+    err = restore_full_sample(&Z, &datainfo);
     if (err) {
 	gui_errmsg(err);
     } else {
@@ -1415,11 +1446,11 @@ int restore_sample (gretlopt opt)
 
 static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
 {
-    int err = restore_sample(OPT_C); 
+    int err = gui_restore_sample(); 
 
     if (verbose && !err) {
 	set_sample_label(datainfo);    
-	gretl_command_strcpy("smpl full");
+	gretl_command_strcpy("smpl --full");
 	check_and_record_command();
     }
 }
@@ -1448,8 +1479,7 @@ int gretl_fork (const char *prog, const char *arg)
 			NULL, NULL, NULL, NULL);
 
     if (!run) {
-	sprintf(errtext, "%s: %s", _("Command failed"), prog);
-	errbox(errtext);
+	errbox("%s: %s", _("Command failed"), prog);
     }
 
     g_free(argv[0]);
@@ -1495,8 +1525,9 @@ drag_data_received  (GtkWidget *widget,
 
     /* ignore the wrong sort of data */
     if (data == NULL || (dfname = (gchar *) data->data) == NULL || 
-	strlen(dfname) <= 5 || strncmp(dfname, "file:", 5))
+	strlen(dfname) <= 5 || strncmp(dfname, "file:", 5)) {
 	return;
+    }
 
     if (strncmp(dfname, "file://", 7) == 0) skip = 7;
 #ifdef G_OS_WIN32
@@ -1517,6 +1548,8 @@ drag_data_received  (GtkWidget *widget,
 
 #ifdef G_OS_WIN32
     slash_convert(tmp, TO_BACKSLASH);
+    /* also convert to UTF-8 here? */
+    my_filename_to_utf8(tmp);
 #endif
 
     suff = strrchr(tmp, '.');
