@@ -24,6 +24,17 @@
 # include <gtksourceview/gtksourceview.h>
 #endif
 
+#define GRETL_STOCK_TEX    "gretl-tex"
+#define GRETL_STOCK_MAIL   "gretl-mail"
+#define GRETL_STOCK_TS     "gretl-tsplot"
+#define GRETL_STOCK_BOX    "gretl-boxplot"
+#define GRETL_STOCK_PDF    "gretl-pdf"
+#define GRETL_STOCK_BOOK   "gretl-book"
+
+#if (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION < 4)
+# define GTK_STOCK_NETWORK  "gretl-network"
+#endif
+
 enum windata_flags {
     VWIN_HELP_ACTIVE = 1 << 0,
     VWIN_BUSY        = 1 << 1
@@ -40,10 +51,13 @@ struct _windata_t {
     GtkWidget *status;
     GtkWidget *popup;
     GtkItemFactory *ifac; 
+    windata_t *gretl_parent;
+    windata_t **gretl_children;
     gpointer data;
     int active_var; 
     int role;
     int n_model_tests;
+    int n_gretl_children;
     unsigned char flags;
     char fname[MAXLEN];
 #ifdef USE_GTKSOURCEVIEW
