@@ -355,7 +355,7 @@ rtfprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 			"%s - %s"), date1, date2);
 	pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n", tmp);
 	if (vmat->missing) {
-	    pprintf(prn, "(%s)\\par\n", I_("(missing values were skipped)"));
+	    pprintf(prn, "%s\\par\n", I_("(missing values were skipped)"));
 	}
 	sprintf(tmp, I_("5%% critical value (two-tailed) = %.4f for n = %d"), 
 		rhocrit95(n), n);
@@ -547,8 +547,6 @@ void tex_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo,
     pputs(prn, I_("Model estimation range:"));
     pprintf(prn, " %s--%s", date1, date2);
 
-    pprintf(prn, " ($n$ = %d)\\\\\n", fr->real_nobs); 
-
     pprintf(prn, I_("Standard error of residuals = %g"), fr->sigma);
     pputs(prn, "\n\\end{raggedright}\n");
 }
@@ -565,7 +563,7 @@ void rtf_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo,
 
     pputs(prn, "{\\rtf1\\par\n\\qc ");
     pputs(prn, I_("Model estimation range:")); 
-    pprintf(prn, " %s - %s (n = %d)\\par\n", date1, date2, fr->real_nobs);
+    pprintf(prn, " %s - %s\\par\n", date1, date2);
 
     sprintf(tmp, I_("Standard error of residuals = %g"), 
 	    fr->sigma);
@@ -609,8 +607,8 @@ static void texprint_fit_resid (const FITRESID *fr,
 	    "   \\multicolumn{1}{c}{%s}\\\\\n",
 	    vname, I_("fitted"), I_("residual"));
 
-    for (t=0; t<fr->nobs; t++) {
-	tex_print_obs_marker(t + fr->t1, pdinfo, prn);
+    for (t=fr->t1; t<=fr->t2; t++) {
+	tex_print_obs_marker(t, pdinfo, prn);
 	pputs(prn, " & ");
 
 	if (na(fr->actual[t])) {
@@ -665,8 +663,8 @@ static void rtfprint_fit_resid (const FITRESID *fr,
 	    " \\intbl \\row\n",
 	    fr->depvar, I_("fitted"), I_("residual"));
 
-    for (t=0; t<fr->nobs; t++) {
-	rtf_print_obs_marker(t + fr->t1, pdinfo, prn);
+    for (t=fr->t1; t<=fr->t2; t++) {
+	rtf_print_obs_marker(t, pdinfo, prn);
 	if (na(fr->actual[t])) {
 	    pputs(prn, "\\qc \\cell \\qc \\cell \\qc \\cell \\ql \\cell"
 		  " \\intbl \\row\n"); 
@@ -744,10 +742,10 @@ static void texprint_fcast_without_errs (const FITRESID *fr,
     pprintf(prn, "%s & %s & \\multicolumn{1}{c}{%s} \\\\ [4pt] \n",
 	    I_("Obs"), vname, I_("prediction"));
 
-    for (t=fr->pre_n; t<fr->nobs; t++) {
+    for (t=fr->t1; t<=fr->t2; t++) {
 	texprint_fcast_x(fr->actual[t], fr->pmax, actual);
 	texprint_fcast_x(fr->fitted[t], fr->pmax, fitted);
-	tex_print_obs_marker(t + fr->t1, pdinfo, prn);
+	tex_print_obs_marker(t, pdinfo, prn);
 	pprintf(prn, " & %s & %s \\\\\n",
 		actual, fitted);
     }
@@ -807,7 +805,7 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
 	errpmax = pmax + 1;
     }
 
-    for (t=fr->pre_n; t<fr->nobs; t++) {
+    for (t=fr->t1; t<=fr->t2; t++) {
 	double xlo, xhi;
 
 	if (na(fr->sderr[t])) {
@@ -822,7 +820,7 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
 	texprint_fcast_x(fr->sderr[t], errpmax, sderr);
 	texprint_fcast_x(xlo, pmax, lo);
 	texprint_fcast_x(xhi, pmax, hi);
-	tex_print_obs_marker(t + fr->t1, pdinfo, prn);
+	tex_print_obs_marker(t, pdinfo, prn);
 	pprintf(prn, " & %s & %s & %s & %s & %s \\\\\n",
 		actual, fitted, sderr, lo, hi);
     }
@@ -854,8 +852,8 @@ static void rtfprint_fcast_without_errs (const FITRESID *fr,
 	    " \\intbl \\row\n", 
 	    I_("Obs"), fr->depvar, I_("prediction")); 
 
-    for (t=fr->pre_n; t<fr->nobs; t++) {
-	rtf_print_obs_marker(t + fr->t1, pdinfo, prn);
+    for (t=fr->t1; t<=fr->t2; t++) {
+	rtf_print_obs_marker(t, pdinfo, prn);
 	printfrtf(fr->actual[t], prn, 0);
 	printfrtf(fr->fitted[t], prn, 0);
     }
@@ -893,8 +891,8 @@ static void rtfprint_fcast_with_errs (const FITRESID *fr,
 	    /* xgettext:no-c-format */
 	    I_("95% confidence interval"));
 
-    for (t=fr->pre_n; t<fr->nobs; t++) {
-	rtf_print_obs_marker(t + fr->t1, pdinfo, prn);
+    for (t=fr->t1; t<=fr->t2; t++) {
+	rtf_print_obs_marker(t, pdinfo, prn);
 	maxerr = fr->tval * fr->sderr[t];
 	printfrtf(fr->actual[t], prn, 0);
 	printfrtf(fr->fitted[t], prn, 0);
@@ -1054,19 +1052,20 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
     int i, t, l0 = list[0];
     int *pmax = NULL;
     int tsamp = datainfo->t2 - datainfo->t1 + 1;
-    char delim = datainfo->delim;
     double xx;
 
     if (l0 == 0) return 1;
 
-    if (delim == ',' && ',' == datainfo->decpoint) {
+    if (datainfo->delim == ',' && ',' == datainfo->decpoint) {
 	errbox(_("You can't use the same character for "
 		 "the column delimiter and the decimal point"));
 	return 1;
     }
 
-    pmax = malloc(l0 * sizeof *pmax);
-    if (pmax == NULL) return 1;
+    pmax = mymalloc(l0 * sizeof *pmax);
+    if (pmax == NULL) {
+	return 1;
+    }
 
     for (i=1; i<=l0; i++) {
 	if (datainfo->vector[list[i]]) {
@@ -1081,12 +1080,16 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
 	gretl_push_c_numeric_locale();
     }
 
+    /* obs column heading? */
+    if (datainfo->S != NULL || datainfo->structure != CROSS_SECTION) {
+	pprintf(prn, "obs%c", datainfo->delim);
+    }
+
     /* variable names */
-    pprintf(prn, "obs%c", delim);
-    for (i=1; i<l0; i++) {
-	pprintf(prn, "%s%c", datainfo->varname[list[i]], delim);
+    for (i=1; i<=l0; i++) {
+	pprintf(prn, "%s", datainfo->varname[list[i]]);
+	pputc(prn, (i < l0)? datainfo->delim : '\n');
     }
-    pprintf(prn, "%s\n", datainfo->varname[list[l0]]);
 
     /* actual data values */
     for (t=datainfo->t1; t<=datainfo->t2; t++) {
@@ -1101,7 +1104,7 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
 	    } else {
 		pprintf(prn, "%.*f", pmax[i-1], xx);
 	    }
-	    pprintf(prn, "%c", (i < l0)? delim : '\n');
+	    pputc(prn, (i < l0)? datainfo->delim : '\n');
 	}
     }
 
@@ -1182,7 +1185,8 @@ int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
 {
     int *list = series_view_get_list(vwin);
     PRN *prn = NULL;
-    char delim = datainfo->delim;
+    char save_delim = datainfo->delim;
+    char save_decpoint = datainfo->decpoint;
     int i, err = 0;
 
     if (list != NULL) {
@@ -1194,7 +1198,8 @@ int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
 	}
 
 	if (fmt == GRETL_FORMAT_CSV) {
-	    delimiter_dialog(NULL);
+	    datainfo->delim = ',';
+	    datainfo->decpoint = '.';
 	} else {
 	    datainfo->delim = '\t';
 	}
@@ -1221,10 +1226,8 @@ int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
 	free(list);
     }
 
-    if (fmt == GRETL_FORMAT_TABLE) {
-	/* restore default after forcing to TAB */
-	datainfo->delim = delim;
-    }
+    datainfo->delim = save_delim;
+    datainfo->decpoint = save_decpoint;
 
     return err;
 }
