@@ -124,6 +124,11 @@ static struct gui_help_item gui_help_items[] = {
     { MODEL_RESTR,    "restrict-model" },
     { SYS_RESTR,      "restrict-system" },
     { VECM_RESTR,     "restrict-vecm" },
+    { LAGS_DIALOG,    "lags-dialog" },
+    { COPY_FORMATS,   "copy-formats" },
+    { MINIBUF,        "minibuffer" },
+    { VLAGSEL,        "VAR-lagselect" },
+    { VAROMIT,        "VAR-omit" },
     { -1,          NULL },
 };
 
@@ -465,6 +470,8 @@ static int add_topic_to_head (help_head *head, int j, const char *word,
 	    head->topicnames[j] = label;
 	}
     } else {
+	fprintf(stderr, "helpfile: add_topic_to_head: hnum=%d for word='%s'\n", 
+		hnum, word);
 	err = 1;
     }
 
@@ -499,6 +506,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
     }
     
     if (!sscanf(line, "headings %d", &nh) || nh <= 0) {
+	fprintf(stderr, "Couldn't reading number of headings in helpfile\n");
 	err = 1;
 	goto bailout;
     }
@@ -521,6 +529,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 	int nt;
 
 	if (fgets(line, sizeof line, fp) == NULL) {
+	    fprintf(stderr, "Couldn't read line from helpfile: i=%d\n", i);
 	    err = 1;
 	    goto bailout;
 	}
@@ -531,6 +540,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 
 	/* heading name plus number of following topics */
 	if (sscanf(line, "%31s %d", tmp, &nt) != 2) {
+	    fprintf(stderr, "Couldn't read heading and number of topics: i=%d\n", i);
 	    err = 1;
 	    goto bailout;
 	}	    
@@ -546,6 +556,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 	/* heading with at least one topic */
 	heads[nh2] = help_head_new(tmp, nt, gui);
 	if (heads[nh2] == NULL) {
+	    fprintf(stderr, "healp_head_new() failed\n");
 	    err = 1;
 	}
 
@@ -570,8 +581,10 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 
 	    if (!err) {
 		err = add_topic_to_head(heads[nh2], j, tmp, label, gui);
-		if (err && label != NULL) {
-		    free(label);
+		if (err) {
+		    if (label != NULL) {
+			free(label);
+		    }
 		}
 	    }
 	}
@@ -601,7 +614,8 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
     fclose(fp);
 
     if (err) {
-	fprintf(stderr, "*** get_helpfile_structure: err = %d\n", err);
+	fprintf(stderr, "*** get_helpfile_structure: err = %d for\n"
+		"filename = '%s'\n", err, fname);
 	free_help_heads(heads, nh);
     } else {
 	*pheads = heads;
@@ -631,8 +645,7 @@ static int real_helpfile_init (int gui, int en)
     err = get_helpfile_structure(&heads, gui, helpfile);
 
     if (err) {
-	sprintf(errtext, _("help file %s is not accessible\n"), helpfile);
-	errbox(errtext);
+	errbox(_("help file %s is not accessible\n"), helpfile);
     } else {
 	if (gui) {
 	    if (en) {
@@ -827,10 +840,10 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
 #ifndef OLD_GTK
     if (cli) {
 	/* Add general index as "topic" */
-	hitem.callback_action = 1; 
+	hitem.callback_action = 0; 
 	hitem.item_type = NULL;
 	hitem.path = g_strdup_printf("%s/%s", mpath, _("Index"));
-	hitem.callback = (en)? do_en_cli_help : do_cli_help;
+	hitem.callback = plain_text_cmdref;
 	gtk_item_factory_create_item(hwin->ifac, &hitem, 
 				     GINT_TO_POINTER(0), 
 				     1);
@@ -1055,6 +1068,11 @@ void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w)
 	en = GPOINTER_TO_INT(p);
     }
 
+#if HDEBUG
+    fprintf(stderr, "plain_text_cmdref: p=%p, cmdnum=%d, w=%p\n", 
+	    (void *) p, cmdnum, (void *) w);
+#endif
+
     if (cmdnum > 0) {
 	pos = cli_pos_from_cmd(cmdnum, en);
 	if (pos < 0 && !en && translated_helpfile == 1) {
@@ -1671,7 +1689,7 @@ enum {
 
 #ifdef G_OS_WIN32
 
-static char *full_doc_path (char *path, const char *fname, int *err)
+static char *full_doc_path (char *path, const char *fname)
 {
     strcpy(path, paths.gretldir);
     strcat(path, "doc");
@@ -1782,6 +1800,7 @@ static int maybe_grab_pdf (int uguide, int i, char *fullpath)
 	"gretl-ref-es.pdf"  
     };
     const char *fname;
+    char errtext[MAXLEN];
     FILE *fp;
     int err = 0;
 
