@@ -37,6 +37,8 @@ int winfork (char *cmdline, const char *dir, int wshow,
 	     DWORD flags);
 #endif
 
+typedef struct _selector selector;
+
 void library_command_init (void);
 
 void library_command_free (void);
@@ -59,7 +61,7 @@ gint bufopen (PRN **pprn);
 
 void do_menu_op (gpointer data, guint action, GtkWidget *widget);
 
-void do_two_var_test (GtkWidget *widget, gpointer p);
+int do_two_var_test (selector *sr);
 
 void do_run_script (gpointer data, guint code, GtkWidget *w);
 
@@ -101,15 +103,17 @@ int get_original_n (void);
 
 /* model-related functions */
 
-void do_coint (GtkWidget *widget, gpointer p);
+int do_coint (selector *sr);
 
 void do_forecast (gpointer data, guint u, GtkWidget *w);
 
-void do_coeff_sum (GtkWidget *widget, gpointer p);
+int do_coeff_sum (selector *sr);
 
-void do_add_omit (GtkWidget *widget, gpointer p);
+int do_add_omit (selector *sr);
 
-void do_confidence_region (GtkWidget *widget, gpointer p);
+int do_VAR_omit (selector *sr);
+
+int do_confidence_region (selector *sr);
 
 void do_lmtest (gpointer data, guint aux_code, GtkWidget *widget);
 
@@ -131,12 +135,14 @@ void do_mle_model (GtkWidget *widget, dialog_t *dlg);
 
 void do_eqn_system (GtkWidget *widget, dialog_t *dlg);
 
-void do_model (GtkWidget *widget, gpointer p);
+int do_model (selector *sr);
 
-void do_vector_model (GtkWidget *widget, gpointer p);
+int do_vector_model (selector *sr);
 
 void do_graph_model (GPT_SPEC *spec);
 
+void do_gini (gpointer data, guint u, GtkWidget *w);
+
 void do_kernel (gpointer data, guint u, GtkWidget *w);
 
 void do_vif (gpointer data, guint u, GtkWidget *w);
@@ -149,10 +155,10 @@ void do_coeff_intervals (gpointer data, guint i, GtkWidget *w);
 
 void do_panel_diagnostics (gpointer data, guint u, GtkWidget *w);
 
-void do_spearman (GtkWidget *widget, gpointer p);
+int do_spearman (selector *sr);
 
 #ifdef ENABLE_GMP
-void do_mp_ols (GtkWidget *widget, gpointer p);
+int do_mp_ols (selector *sr);
 #endif
 
 int out_of_sample_info (int add_ok, int *t2);
@@ -161,6 +167,8 @@ int out_of_sample_info (int add_ok, int *t2);
 
 void do_simdata (GtkWidget *widget, dialog_t *dlg);
 
+void do_minibuf (GtkWidget *widget, dialog_t *dlg);
+
 void do_genr (GtkWidget *widget, dialog_t *dlg);
 
 void do_model_genr (GtkWidget *widget, dialog_t *dlg);
@@ -185,8 +193,12 @@ void do_freqplot (gpointer data, guint gamma, GtkWidget *widget);
 
 void do_corrgm (gpointer data, guint u, GtkWidget *widget);
 
+void residual_correlogram (gpointer data, guint u, GtkWidget *widget);
+
 void do_pergm (gpointer data, guint opt, GtkWidget *widget);
 
+void residual_periodogram (gpointer data, guint opt, GtkWidget *widget);
+
 #if defined (HAVE_TRAMO) || defined (HAVE_X12A)
 void do_tramo_x12a (gpointer data, guint opt, GtkWidget *widget);
 #endif
@@ -231,17 +243,17 @@ void do_boxplot_var (int varnum);
 
 void ts_plot_var (gpointer data, guint opt, GtkWidget *widget);
 
-void do_scatters (GtkWidget *widget, gpointer p);
+int do_scatters (selector *sr);
 
-void do_graph_from_selector (GtkWidget *widget, gpointer p);
+int do_graph_from_selector (selector *sr);
 
-void do_splot_from_selector (GtkWidget *widget, gpointer p);
+int do_splot_from_selector (selector *sr);
 
 void plot_from_selection (gpointer data, guint action, GtkWidget *widget);
 
 void do_box_graph (GtkWidget *widget, dialog_t *dlg);
 
-void do_dummy_graph (GtkWidget *widget, gpointer p);
+int do_dummy_graph (selector *sr);
 
 void delete_selected_vars (int id);
 
@@ -270,9 +282,7 @@ void do_save_text (char *fname, MODEL *pmod);
 int execute_script (const char *runfile, const char *buf,
 		    PRN *prn, int exec_code);
 
-int gui_exec_line (char *line, 
-		   LOOPSET **plp, int *plstack, int *plrun, 
-		   PRN *prn, int exec_code, 
+int gui_exec_line (char *line, PRN *prn, int exec_code, 
 		   const char *myname); 
 
 int check_and_record_command (void);
