@@ -104,9 +104,10 @@ static GtkItemFactoryEntry time_series_model_items[] = {
     { N_("/Model/Time series/_Hildreth-Lu..."), NULL, model_callback, HILU, NULL, GNULL },
     { N_("/Model/Time series/_Prais-Winsten..."), NULL, model_callback, PWE, NULL, GNULL },
     { N_("/Model/Time series/_Autoregressive estimation..."), NULL, model_callback, AR, NULL, GNULL },
-    { N_("/Model/Time series/AR_MA..."), NULL, model_callback, ARMA, NULL, GNULL },
+    { N_("/Model/Time series/ARI_MA..."), NULL, model_callback, ARMA, NULL, GNULL },
     { N_("/Model/Time series/_GARCH..."), NULL, model_callback, GARCH, NULL, GNULL },
     { N_("/Model/Time series/_Vector Autoregression..."), NULL, selector_callback, VAR, NULL, GNULL },
+    { N_("/Model/Time series/VAR lag selection..."), NULL, selector_callback, VLAGSEL, NULL, GNULL },
     { N_("/Model/Time series/VECM..."), NULL, selector_callback, VECM, NULL, GNULL },
     { N_("/Model/Time series/Cointegration test"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Model/Time series/Cointegration test/Engle-Granger..."), NULL, 
@@ -122,9 +123,10 @@ static GtkItemFactoryEntry time_series_model_items[] = {
     { N_("/Model/Time series/_Hildreth-Lu..."), NULL, model_callback, HILU, NULL },
     { N_("/Model/Time series/_Prais-Winsten..."), NULL, model_callback, PWE, NULL },
     { N_("/Model/Time series/_Autoregressive estimation..."), NULL, model_callback, AR, NULL },
-    { N_("/Model/Time series/AR_MA..."), NULL, model_callback, ARMA, NULL },
+    { N_("/Model/Time series/ARI_MA..."), NULL, model_callback, ARMA, NULL },
     { N_("/Model/Time series/_GARCH..."), NULL, model_callback, GARCH, NULL },
     { N_("/Model/Time series/_Vector Autoregression..."), NULL, selector_callback, VAR, NULL },
+    { N_("/Model/Time series/VAR lag selection..."), NULL, selector_callback, VLAGSEL, NULL },
     { N_("/Model/Time series/VECM..."), NULL, selector_callback, VECM, NULL },
     { N_("/Model/Time series/Cointegration test"), NULL, NULL, 0, "<Branch>" },
     { N_("/Model/Time series/Cointegration test/Engle-Granger..."), NULL, 
@@ -283,13 +285,15 @@ static gint var_popup_click (GtkWidget *widget, gpointer data)
 	do_freqplot(NULL, 0, NULL);
     else if (!strcmp(item, _("Boxplot")))
 	do_boxplot_var(mdata->active_var);
+    else if (!strcmp(item, _("Gini coefficient")))
+	do_gini(NULL, 0, NULL);
     else if (!strcmp(item, _("Correlogram")))
 	do_corrgm(NULL, CORRGM, NULL);
     else if (!strcmp(item, _("Spectrum"))) 
 	do_pergm(NULL, 0, NULL);
     else if (!strcmp(item, _("Spectrum (Bartlett)"))) 
 	do_pergm(NULL, 1, NULL);
-    else if (!strcmp(item, _("ARMA model"))) 
+    else if (!strcmp(item, _("ARIMA model"))) 
 	model_callback(GINT_TO_POINTER(mdata->active_var), ARMA, NULL);
     else if (!strcmp(item, _("Dickey-Fuller test"))) 
 	unit_root_test(NULL, ADF, NULL);
@@ -322,10 +326,11 @@ GtkWidget *build_var_popup (void)
 	N_("Frequency distribution"),
 	N_("Frequency plot"),
 	N_("Boxplot"),
+	N_("Gini coefficient"),
 	N_("Correlogram"),
 	N_("Spectrum"),
 	N_("Spectrum (Bartlett)"),
-	N_("ARMA model"),
+	N_("ARIMA model"),
 	N_("Dickey-Fuller test"),
 	N_("KPSS test"),
 	N_("Runs test"),
@@ -343,7 +348,7 @@ GtkWidget *build_var_popup (void)
     var_menu = gtk_menu_new();
 
     for (i=0; i<n_items; i++) {
-	if (!dataset_is_time_series(datainfo) && (i >= 6 && i <= 11)) {
+	if (!dataset_is_time_series(datainfo) && (i >= 7 && i <= 12)) {
 	    continue;
 	}
 	if (i == 2 && !extended_ts(datainfo)) {
