@@ -47,7 +47,9 @@ static int real_table_n_models (void)
     int i, len = 0;
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] != NULL) len++;
+	if (table_models[i] != NULL) {
+	    len++;
+	}
     }
 
     return len;    
@@ -59,6 +61,7 @@ static int model_table_too_many (int gui)
 	mtable_errmsg(_("Model table is full"), gui);
 	return 1;
     }
+
     return 0;
 }
 
@@ -67,7 +70,7 @@ static int model_already_in_table (const MODEL *pmod)
     int i;
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] != NULL && pmod->ID == table_models[i]->ID) {
+	if (pmod == table_models[i]) {
 	    return 1;
 	}
     }
@@ -80,7 +83,10 @@ void clear_model_table (PRN *prn)
     int i;
 
     for (i=0; i<n_models; i++) {
-	gretl_model_free(table_models[i]);
+	/* reduce refcount on the model pointer */
+	if (table_models[i] != NULL) {
+	    gretl_object_unref(table_models[i], GRETL_OBJ_EQN);
+	}
     }
 
     free(table_models);
@@ -96,16 +102,30 @@ void clear_model_table (PRN *prn)
     }
 }
 
-int add_to_model_table (const MODEL *pmod, int add_mode, PRN *prn)
+static int model_table_depvar (void)
+{
+    int i;
+
+    for (i=0; i<n_models; i++) {
+	if (table_models[i] != NULL) {
+	    return table_models[i]->list[1];
+	}
+    }
+
+    return -1;
+}
+
+int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
 {
-    MODEL *mcpy;
     int gui = (add_mode != MODEL_ADD_BY_CMD);
 
-    /* FIXME update restrictions here (garch, mle?) */
+    if (pmod == NULL) {
+	return 1;
+    }
 
-    /* NLS models won't work */
-    if (pmod->ci == NLS) {
-	mtable_errmsg(_("Sorry, NLS models can't be put in the model table"),
+    /* NLS and MLE models won't work */
+    if (pmod->ci == NLS || pmod->ci == MLE) {
+	mtable_errmsg(_("Sorry, this model can't be put in the model table"),
 		      gui);
 	return 1;
     }
@@ -124,18 +144,19 @@ int add_to_model_table (const MODEL *pmod, int add_mode, PRN *prn)
 	return 1;
     }    
 
-    /* is the list is started or not? */
+    /* is the list started or not? */
     if (n_models == 0) {
-
 	table_models = mymalloc(sizeof *table_models);
-	if (table_models == NULL) return 1;
+	if (table_models == NULL) {
+	    return 1;
+	}
 	n_models = 1;
-
     } else {
+	int dv = model_table_depvar();
 	MODEL **mods;
 
 	/* check that the dependent variable is in common */
-	if (pmod->list[1] != table_models[0]->list[1]) {
+	if (pmod->list[1] != dv) {
 	    mtable_errmsg(_("Can't add model to table -- this model has a "
 			    "different dependent variable"), gui);
 	    return 1;
@@ -162,14 +183,11 @@ int add_to_model_table (const MODEL *pmod, int add_mode, PRN *prn)
 	table_models = mods;
     }
 
-    mcpy = gretl_model_copy(pmod);
-    if (mcpy == NULL) {
-	clear_model_table(NULL);
-	return E_ALLOC;
-    }
+    table_models[n_models - 1] = pmod;
+
+    /* augment refcount so model won't get deleted */
+    gretl_object_ref(pmod, GRETL_OBJ_EQN);
 
-    table_models[n_models - 1] = mcpy;
-    
     if (add_mode == MODEL_ADD_FROM_MENU) {
 	infobox(_("Model added to table"));
     } else if (add_mode == MODEL_ADD_BY_CMD) {
@@ -185,8 +203,12 @@ static int var_is_in_model (int v, const MODEL *pmod)
     int i;
 
     for (i=2; i<=pmod->list[0]; i++) {
-	if (pmod->list[i] == LISTSEP) break;
-	if (v == pmod->list[i]) return i;
+	if (pmod->list[i] == LISTSEP) {
+	    break;
+	}
+	if (v == pmod->list[i]) {
+	    return i;
+	}
     }
 
     return 0;    
@@ -197,7 +219,9 @@ static int on_grand_list (int v)
     int i;
 
     for (i=2; i<=grand_list[0]; i++) {
-	if (v == grand_list[i]) return 1;
+	if (v == grand_list[i]) {
+	    return 1;
+	}
     }
 
     return 0;
@@ -220,7 +244,9 @@ static int real_list_length (int *list)
     int i;
 
     for (i=1; i<=list[0]; i++) {
-	if (list[i] == LISTSEP) return i - 1;
+	if (list[i] == LISTSEP) {
+	    return i - 1;
+	}
     }
 
     return list[0];
@@ -228,31 +254,37 @@ static int real_list_length (int *list)
 
 static int make_grand_varlist (void)
 {
-    int i, j, f = 1;
+    int i, j, first = 1;
     int l0 = 0;
     const MODEL *pmod;
 
     free(grand_list);
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] == NULL) continue;
-	l0 += real_list_length(table_models[i]->list);
+	if (table_models[i] != NULL) {
+	    l0 += real_list_length(table_models[i]->list);
+	}
     }
 
-    grand_list = mymalloc((l0 + 1) * sizeof *grand_list);
-    if (grand_list == NULL) return 1;
+    grand_list = gretl_list_new(l0);
+    if (grand_list == NULL) {
+	return 1;
+    }
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] == NULL) continue;
-	pmod = table_models[i];
-	if (f == 1) {
-	    for (j=0; j<=pmod->list[0]; j++) {
-		if (pmod->list[j] == LISTSEP) break;
-		grand_list[j] = pmod->list[j];
+	if (table_models[i] != NULL) {
+	    pmod = table_models[i];
+	    if (first) {
+		for (j=0; j<=pmod->list[0]; j++) {
+		    if (pmod->list[j] == LISTSEP) {
+			break;
+		    }
+		    grand_list[j] = pmod->list[j];
+		}
+		first = 0;
+	    } else {
+		add_to_grand_list(pmod->list);
 	    }
-	    f = 0;
-	} else {
-	    add_to_grand_list(pmod->list);
 	}
     }
 
@@ -281,11 +313,12 @@ static int common_estimator (void)
     int i, ci0 = -1;
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] == NULL) continue;
-	if (ci0 == -1) {
-	    ci0 = table_models[i]->ci;
-	} else {
-	    if (table_models[i]->ci != ci0) return 0;
+	if (table_models[i] != NULL) {
+	    if (ci0 == -1) {
+		ci0 = table_models[i]->ci;
+	    } else if (table_models[i]->ci != ci0) {
+		return 0;
+	    }
 	}
     }  
 
@@ -297,41 +330,28 @@ static int common_df (void)
     int i, dfn0 = -1, dfd0 = -1;
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] == NULL) continue;
-	if (dfn0 == -1) {
-	    dfn0 = table_models[i]->dfn;
-	    dfd0 = table_models[i]->dfd;
-	} else {
-	    if (table_models[i]->dfn != dfn0) return 0;
-	    if (table_models[i]->dfd != dfd0) return 0;
+	if (table_models[i] != NULL) {
+	    if (dfn0 == -1) {
+		dfn0 = table_models[i]->dfn;
+		dfd0 = table_models[i]->dfd;
+	    } else {
+		if (table_models[i]->dfn != dfn0 ||
+		    table_models[i]->dfd != dfd0) {
+		    return 0;
+		}
+	    }
 	}
     }  
 
     return 1;
 }
 
-static void center_in_field (const char *s, int width, PRN *prn)
-{
-    int rem = width - strlen(s);
-
-    if (rem <= 1) {
-	pprintf(prn, "%s", s);
-    }
-    else {
-	int i, off = rem / 2;
-
-	for (i=0; i<off; i++) {
-	    pputs(prn, " ");
-	}
-	pprintf(prn, "%-*s", width - off, s);
-    }
-}
-
 static const char *short_estimator_string (int ci, PRN *prn)
 {
     if (ci == HSK) return N_("HSK");
     else if (ci == CORC) return N_("CORC");
     else if (ci == HILU) return N_("HILU");
+    else if (ci == PWE) return N_("PWE");
     else if (ci == ARCH) return N_("ARCH");
     else if (ci == POOLED) return N_("OLS");
     else return estimator_string(ci, prn);
@@ -352,11 +372,11 @@ static const char *get_pre_asts (double pval)
     return (pval >= 0.1)? "" : (pval >= 0.05)? "$\\,$" : "$\\,\\,$";
 }
 
-static void print_model_table_coeffs (PRN *prn)
+static void print_model_table_coeffs (int nwidth, PRN *prn)
 {
     int i, j, k;
     const MODEL *pmod;
-    char tmp[16];
+    char tmp[32];
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
 
@@ -372,13 +392,15 @@ static void print_model_table_coeffs (PRN *prn)
 	    print_rtf_row_spec(prn, 0);
 	    pprintf(prn, "\\intbl \\qc %s\\cell ", datainfo->varname[v]);
 	} else {
-	    pprintf(prn, "%8s ", datainfo->varname[v]);
+	    pprintf(prn, "%-*s ", nwidth, datainfo->varname[v]);
 	}
 
 	/* print the coefficient estimates across a row */
 	for (j=0; j<n_models; j++) {
 	    pmod = table_models[j];
-	    if (pmod == NULL) continue;
+	    if (pmod == NULL) {
+		continue;
+	    }
 	    if ((k = var_is_in_model(v, pmod))) {
 		double x = screen_zero(pmod->coeff[k-2]);
 		double s = screen_zero(pmod->sderr[k-2]);
@@ -409,22 +431,25 @@ static void print_model_table_coeffs (PRN *prn)
 		} else if (rtf) {
 		    pprintf(prn, "\\qc %s%s\\cell ", numstr, get_asts(pval));
 		} else {
+		    /* note: strlen(asts) = 2 */
 		    pprintf(prn, "%*s%s", (f == 1)? 12 : 10,
 			    numstr, get_asts(pval));
 		}
 		f = 0;
 	    } else {
+		/* variable not present in this column */
 		if (tex) {
 		    pputs(prn, "& ");
 		} else if (rtf) {
 		    pputs(prn, "\\qc \\cell ");
 		} else {
-		    pputs(prn, "            ");
+		    pputs(prn, "            "); /* 12 */
 		}
 	    }
 	}
 
-	/* terminate the coefficient row and start the next one */
+	/* terminate the coefficient row and start the next one,
+	   which holds standard errors */
 	if (tex) {
 	    pputs(prn, "\\\\\n");
 	} else if (rtf) {
@@ -432,7 +457,8 @@ static void print_model_table_coeffs (PRN *prn)
 	    print_rtf_row_spec(prn, 1);
 	    pputs(prn, "\\intbl ");
 	} else {
-	    pputs(prn, "\n          ");
+	    pputc(prn, '\n');
+	    bufspace(nwidth + 2, prn);
 	}
 
 	/* print the t-stats or standard errors across a row */
@@ -472,12 +498,13 @@ static void print_model_table_coeffs (PRN *prn)
 		    pprintf(prn, "%12s", tmp);
 		}
 	    } else {
+		/* variable not present in this column */
 		if (tex) {
 		    pputs(prn, "& ");
 		} else if (rtf) {
 		    pputs(prn, "\\qc \\cell ");
 		} else {
-		    pputs(prn, "            ");
+		    pputs(prn, "            "); /* 12 */
 		}
 	    }
 	}
@@ -497,8 +524,12 @@ static int any_log_lik (void)
     int i;
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] == NULL) continue;
-	if (!na(table_models[i]->lnL)) return 1;
+	if (table_models[i] == NULL) {
+	    continue;
+	}
+	if (!na(table_models[i]->lnL)) {
+	    return 1;
+	}
     }
 
     return 0;
@@ -509,38 +540,57 @@ static int any_r_squared (void)
     int i;
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] == NULL) continue;
-	if (!na(table_models[i]->rsq)) return 1;
+	if (table_models[i] == NULL) {
+	    continue;
+	}
+	if (!na(table_models[i]->rsq)) {
+	    return 1;
+	}
     }
 
     return 0;
 }
 
-static void print_n_r_squared (PRN *prn, int *binary)
+static void print_n_r_squared (int wid, PRN *prn, int *binary)
 {
-    int j;
     int same_df, any_R2, any_ll;
     const MODEL *pmod;
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
+    int j;
 
-    if (rtf) print_rtf_row_spec(prn, 0);
+    if (rtf) {
+	print_rtf_row_spec(prn, 0);
+    }
 
-    if (tex) pprintf(prn, "$%s$ ", _("n"));
-    else if (rtf) pprintf(prn, "\\intbl \\qc %s\\cell ", _("n"));
-    else pprintf(prn, "%8s ", _("n"));
+    if (tex) {
+	pprintf(prn, "$%s$ ", _("n"));
+    } else if (rtf) {
+	pprintf(prn, "\\intbl \\qc %s\\cell ", _("n"));
+    } else {
+	pprintf(prn, "%*s", wid, _("n"));
+    }
 
     for (j=0; j<n_models; j++) {
 	pmod = table_models[j];
-	if (pmod == NULL) continue;
-	if (tex) pprintf(prn, "& %d ", pmod->nobs);
-	else if (rtf) pprintf(prn, "\\qc %d\\cell ", pmod->nobs);
-	else pprintf(prn, "%12d", pmod->nobs);
+	if (pmod != NULL) {
+	    if (tex) {
+		pprintf(prn, "& %d ", pmod->nobs);
+	    } else if (rtf) {
+		pprintf(prn, "\\qc %d\\cell ", pmod->nobs);
+	    } else {
+		pprintf(prn, "%12d", pmod->nobs);
+	    }
+	}
     }
 
-    if (tex) pputs(prn, "\\\\\n");
-    else if (rtf) pputs(prn, "\\intbl \\row\n\\intbl ");
-    else pputc(prn, '\n');
+    if (tex) {
+	pputs(prn, "\\\\\n");
+    } else if (rtf) {
+	pputs(prn, "\\intbl \\row\n\\intbl ");
+    } else {
+	pputc(prn, '\n');
+    }
 
     same_df = common_df();
     any_R2 = any_r_squared();
@@ -548,14 +598,13 @@ static void print_n_r_squared (PRN *prn, int *binary)
 
     if (any_R2) {
 	/* print R^2 values */
-
 	if (tex) {
 	    pputs(prn, (same_df)? "$R^2$" : "$\\bar R^2$ ");
 	} else if (rtf) {
 	    pprintf(prn, "\\qc %s\\cell ", 
 		    (same_df)? "R{\\super 2}" : _("Adj. R{\\super 2}"));
 	} else {
-	    pprintf(prn, "%9s", (same_df)? _("R-squared") : _("Adj. R**2"));
+	    pprintf(prn, "%*s", wid, (same_df)? _("R-squared") : _("Adj. R**2"));
 	}
 
 	for (j=0; j<n_models; j++) {
@@ -569,8 +618,7 @@ static void print_n_r_squared (PRN *prn, int *binary)
 		} else {
 		    pputs(prn, "            ");
 		}		
-	    }
-	    else if (pmod->ci == LOGIT || pmod->ci == PROBIT) {
+	    } else if (pmod->ci == LOGIT || pmod->ci == PROBIT) {
 		*binary = 1;
 		/* McFadden */
 		if (tex) {
@@ -593,11 +641,15 @@ static void print_n_r_squared (PRN *prn, int *binary)
 	    }
 	}
 
-	if (tex) pputs(prn, "\\\\\n");
-	else if (rtf) pputs(prn, "\\intbl \\row\n");
-	else {
+	if (tex) {
+	    pputs(prn, "\\\\\n");
+	} else if (rtf) {
+	    pputs(prn, "\\intbl \\row\n");
+	} else {
 	    pputc(prn, '\n');
-	    if (!any_ll) pputc(prn, '\n');
+	    if (!any_ll) {
+		pputc(prn, '\n');
+	    }
 	}
     }
 
@@ -609,7 +661,7 @@ static void print_n_r_squared (PRN *prn, int *binary)
 	} else if (rtf) {
 	    pputs(prn, "\\qc lnL\\cell ");
 	} else {
-	    pprintf(prn, "%9s", "lnL");
+	    pprintf(prn, "%*s", wid, "lnL");
 	}
 
 	for (j=0; j<n_models; j++) {
@@ -634,17 +686,36 @@ static void print_n_r_squared (PRN *prn, int *binary)
 	    }
 	}
 
-	if (tex) pputs(prn, "\\\\\n");
-	else if (rtf) pputs(prn, "\\intbl \\row\n");
-	else pputs(prn, "\n\n");
+	if (tex) {
+	    pputs(prn, "\\\\\n");
+	} else if (rtf) {
+	    pputs(prn, "\\intbl \\row\n");
+	} else {
+	    pputs(prn, "\n\n");
+	}
     }
 }
 
+static int grand_list_namelen (void)
+{
+    int i, len, maxlen = 8;
+
+    for (i=2; i<=grand_list[0]; i++) {
+	len = strlen(datainfo->varname[grand_list[i]]);
+	if (len > maxlen) {
+	    maxlen = len;
+	}
+    }
+
+    return maxlen;
+}
+
 int display_model_table (int gui)
 {
     int j, ci;
     int binary = 0;
     int winwidth = 78;
+    int namelen;
     PRN *prn;
 
     if (model_table_is_empty()) {
@@ -652,7 +723,11 @@ int display_model_table (int gui)
 	return 1;
     }
 
-    if (make_grand_varlist()) return 1;
+    if (make_grand_varlist()) {
+	return 1;
+    }
+
+    namelen = grand_list_namelen();
 
     if (bufopen(&prn)) {
 	clear_model_table(NULL);
@@ -671,41 +746,45 @@ int display_model_table (int gui)
     pprintf(prn, _("Dependent variable: %s\n"),
 	    datainfo->varname[grand_list[1]]);
 
-    pputs(prn, "\n            ");
+    pputc(prn, '\n');
+    bufspace(namelen + 4, prn);
 
     for (j=0; j<n_models; j++) {
-	char modhd[16];
+	char modhd[32];
 
-	if (table_models[j] == NULL) continue;
-	sprintf(modhd, _("Model %d"), table_models[j]->ID);
-	center_in_field(modhd, 12, prn);
+	if (table_models[j] != NULL) {
+	    sprintf(modhd, _("Model %d"), table_models[j]->ID);
+	    print_centered(modhd, 12, prn);
+	}
     }
     pputc(prn, '\n');
     
     if (ci == 0) {
-	char est[12];	
+	char est[32];	
 
-	pputs(prn, "            ");
+	bufspace(namelen + 4, prn);
 	for (j=0; j<n_models; j++) {
-	    if (table_models[j] == NULL) continue;
-	    strcpy(est, 
-		   _(short_estimator_string(table_models[j]->ci,
-					    prn)));
-	    center_in_field(est, 12, prn);
+	    if (table_models[j] != NULL) {
+		strcpy(est, 
+		       _(short_estimator_string(table_models[j]->ci,
+						prn)));
+		print_centered(est, 12, prn);
+	    }
 	}
 	pputc(prn, '\n');
     }
 
     pputc(prn, '\n'); 
 
-    print_model_table_coeffs(prn);
-    print_n_r_squared(prn, &binary);
+    print_model_table_coeffs(namelen, prn);
+    print_n_r_squared(namelen + 1, prn, &binary);
 
     if (use_tstats) {
 	pprintf(prn, "%s\n", _("t-statistics in parentheses"));
     } else {
 	pprintf(prn, "%s\n", _("Standard errors in parentheses"));
     }
+
     pprintf(prn, "%s\n", _("* indicates significance at the 10 percent level"));
     pprintf(prn, "%s\n", _("** indicates significance at the 5 percent level"));
    
@@ -714,7 +793,9 @@ int display_model_table (int gui)
 				 "McFadden's pseudo-R-squared"));
     }
 
-    if (real_table_n_models() > 5) winwidth = 90;
+    if (real_table_n_models() > 5) {
+	winwidth = 90;
+    }
 
     view_buffer(prn, winwidth, 450, _("gretl: model table"), VIEW_MODELTABLE, 
 		NULL);
@@ -726,14 +807,16 @@ static int tex_print_model_table (PRN *prn)
 {
     int j, ci;
     int binary = 0;
-    char tmp[16];
+    char tmp[32];
 
     if (model_table_is_empty()) {
 	mtable_errmsg(_("The model table is empty"), 1);
 	return 1;
     }
 
-    if (make_grand_varlist()) return 1;
+    if (make_grand_varlist()) {
+	return 1;
+    }
 
     gretl_print_set_format(prn, GRETL_FORMAT_TEX);
 
@@ -759,21 +842,25 @@ static int tex_print_model_table (PRN *prn)
     pputs(prn, "}\n");
 
     for (j=0; j<n_models; j++) {
-	char modhd[16];
+	char modhd[32];
 
-	if (table_models[j] == NULL) continue;
+	if (table_models[j] == NULL) {
+	    continue;
+	}
 	sprintf(modhd, I_("Model %d"), table_models[j]->ID);
 	pprintf(prn, " & %s ", modhd);
     }
     pputs(prn, "\\\\ ");
     
     if (ci == 0) {
-	char est[12];
+	char est[32];
 
 	pputc(prn, '\n');
 
 	for (j=0; j<n_models; j++) {
-	    if (table_models[j] == NULL) continue;
+	    if (table_models[j] == NULL) {
+		continue;
+	    }
 	    strcpy(est, 
 		   I_(short_estimator_string(table_models[j]->ci,
 					     prn)));
@@ -784,8 +871,8 @@ static int tex_print_model_table (PRN *prn)
 
     pputs(prn, " [6pt] \n");   
 
-    print_model_table_coeffs(prn);
-    print_n_r_squared(prn, &binary);
+    print_model_table_coeffs(0, prn);
+    print_n_r_squared(0, prn, &binary);
 
     pputs(prn, "\\end{tabular}\n\n");
     pputs(prn, "\\vspace{1em}\n");
@@ -859,7 +946,7 @@ static int rtf_print_model_table (PRN *prn)
 
     pputs(prn, "\\intbl \\qc \\cell ");
     for (j=0; j<n_models; j++) {
-	char modhd[16];
+	char modhd[32];
 
 	if (table_models[j] == NULL) continue;
 	sprintf(modhd, I_("Model %d"), table_models[j]->ID);
@@ -868,7 +955,7 @@ static int rtf_print_model_table (PRN *prn)
     pputs(prn, "\\intbl \\row\n");
     
     if (ci == 0) {
-	char est[12];
+	char est[32];
 
 	pputs(prn, "\\intbl \\qc \\cell ");
 
@@ -881,8 +968,8 @@ static int rtf_print_model_table (PRN *prn)
 	pputs(prn, "\\intbl \\row\n");
     }
 
-    print_model_table_coeffs(prn);
-    print_n_r_squared(prn, &binary);
+    print_model_table_coeffs(0, prn);
+    print_n_r_squared(0, prn, &binary);
 
     pputs(prn, "}\n\n");
 
@@ -914,9 +1001,9 @@ int special_print_model_table (PRN *prn)
     }
 }
 
-int modeltab_parse_line (const char *line, const MODEL *pmod, PRN *prn)
+int modeltab_parse_line (const char *line, MODEL *pmod, PRN *prn)
 {
-    char cmdword[8];
+    char cmdword[9];
     int err = 0;
 
     if (sscanf(line, "%*s %8s", cmdword) != 1) {
