@@ -28,13 +28,13 @@ enum {
 
 void clear_model_table (PRN *prn);
 
-int add_to_model_table (const MODEL *pmod, int add_mode, PRN *prn);
+int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn);
 
 int display_model_table (int gui);
 
 int special_print_model_table (PRN *prn);
 
-int modeltab_parse_line (const char *line, const MODEL *pmod, PRN *prn);
+int modeltab_parse_line (const char *line, MODEL *pmod, PRN *prn);
 
 void model_table_dialog (void);
 
