@@ -22,7 +22,7 @@
 #ifndef SESSION_H
 #define SESSION_H
 
-#include "system.h"
+#include "objstack.h"
 
 enum {
     SCHEDULE_FOR_DELETION,
@@ -41,24 +41,6 @@ enum {
     ADD_OBJECT_FAIL
 };
 
-typedef enum {
-    OBJ_UNKNOWN,
-    OBJ_DATASET,
-    OBJ_INFO,
-    OBJ_STATS,
-    OBJ_CORR,
-    OBJ_SCRIPT,
-    OBJ_NOTES,
-    OBJ_MODTAB,
-    OBJ_GPAGE,
-    OBJ_MODEL,
-    OBJ_GRAPH,
-    OBJ_PLOT,
-    OBJ_VAR,
-    OBJ_SYS,
-    OBJ_TEXT
-} SessionObjType;
-
 int session_is_saved (void);
 
 void set_session_saved (int val);
@@ -72,35 +54,15 @@ int real_add_text_to_session (PRN *prn, const char *tname);
 
 void add_graph_to_session (gpointer data, guint code, GtkWidget *w);
 
-void remember_model (gpointer data, guint close, GtkWidget *widget);
+void model_add_as_icon (gpointer p, guint type, GtkWidget *w);
 
-void remember_var (gpointer data, guint close, GtkWidget *widget);
+void model_add_as_icon_and_close (gpointer p, guint type, GtkWidget *w);
 
-void remember_sys (gpointer data, guint close, GtkWidget *widget);
+int maybe_add_model_to_session (void *ptr, GretlObjType type);
 
-int model_already_saved (const char *modname);
+void session_model_callback (void *ptr, int action);
 
-int try_add_model_to_session (MODEL *pmod);
-
-int try_add_var_to_session (GRETL_VAR *var);
-
-int try_add_system_to_session (gretl_equation_system *sys);
-
-int display_saved_model (const char *modname);
-
-int display_saved_equation_system (const char *sysname);
-
-int display_saved_VAR (const char *varname);
-
-void session_VAR_do_irf (const char *varname, const char *line);
-
-int delete_model_from_session (const char *modname);
-
-int delete_system_from_session (const char *sysname);
-
-int delete_VAR_from_session (const char *varname);
-
-void *get_session_object_by_name (const char *name, int *which);
+void *get_session_object_by_name (const char *name, GretlObjType *type);
 
 void delete_text_from_session (void *p);
 
