@@ -113,6 +113,7 @@ GdkFont *fixed_font;
 static int usecwd;
 static int olddat;
 static int useqr;
+static int shellok;
 static int manpref;
 char gpcolors[32];
 static char datapage[24];
@@ -200,6 +201,8 @@ RCVAR rc_vars[] = {
     { "browser", N_("Web browser"), NULL, Browser, 
       ROOTSET, MAXSTR, 3, NULL },
 #endif
+    { "shellok", N_("Allow shell commands"), NULL, &shellok, 
+      BOOLSET, 0, 1, NULL },
     { "gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
       ROOTSET, MAXLEN, 3, NULL },
     { "Rcommand", N_("Command to launch GNU R"), NULL, Rcommand, 
@@ -345,6 +348,28 @@ void set_fixed_font (void)
 #endif
 }
 
+#ifndef G_OS_WIN32
+
+static void record_shell_opt (void)
+{
+    char shellstamp[FILENAME_MAX];
+
+    sprintf(shellstamp, "%s.gretl_shell_stamp", paths.userdir);
+
+    if (shellok) {
+	FILE *fp = fopen(shellstamp, "w");
+
+	if (fp != NULL) {
+	    fputs("ok\n", fp);
+	    fclose(fp);
+	}
+    } else {
+	remove(shellstamp);
+    }
+}
+
+#endif
+
 #if !defined(USE_GNOME) && !defined(OLD_GTK)
 
 const char *get_app_fontname (void)
@@ -1235,6 +1260,7 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
     /* register these for session using libset apparatus */
     set_use_qr(useqr);
+    set_shell_ok(shellok);
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
     set_garch_robust_vcv(hc_garch);
@@ -1286,6 +1312,7 @@ static void boolvar_to_str (void *b, char *s)
 static void common_read_rc_setup (void)
 {
     set_use_qr(useqr);
+    set_shell_ok(shellok);
     set_gp_colors();
     
     set_xsect_hccme(hc_xsect);
@@ -1336,26 +1363,19 @@ void write_rc (void)
     }
 
     save_file_lists(client);
-
     g_object_unref(G_OBJECT(client));
-
     set_paths(&paths, set_paths_opt);
+    record_shell_opt();
 }
 
 static void read_rc (void) 
 {
     GConfClient *client;
     GError *error = NULL;
-    GSList *flist = NULL;
     gboolean bval;
     int ival;
     gchar *strval;
     char key[MAXSTR];
-    const char *file_sections[] = {
-	"recent_data_files",
-	"recent_session_files",
-	"recent_script_files"
-    };	
     int i;
 
     client = gconf_client_get_default();
@@ -1395,23 +1415,7 @@ static void read_rc (void)
 	}
     }
 
-    initialize_file_lists();
-
-    for (i=0; i<3; i++) {
-	int j;
-
-	sprintf(key, "/apps/gretl/%s", file_sections[i]);
-	flist = gconf_client_get_list(client, key,
-				      GCONF_VALUE_STRING, NULL);
-	if (flist != NULL) {
-	    for (j=0; j<MAXRECENT; j++) {
-		write_filename_to_list(i, j, flist->data);
-		flist = flist->next;
-	    }
-	    g_slist_free(flist);
-	    flist = NULL;
-	}
-    }
+    read_file_lists(client);
 
     g_object_unref(G_OBJECT(client));
 
@@ -1443,28 +1447,22 @@ void write_rc (void)
     }
 
     save_file_lists();
-
     gnome_config_sync();
-
     set_paths(&paths, set_paths_opt);
+    record_shell_opt();
 }
 
 static void read_rc (void) 
 {
     gchar *value = NULL;
-    char gpath[MAXSTR];
+    char key[MAXSTR];
     char *strvar;
-    const char *file_sections[] = {
-	"recent data files",
-	"recent session files",
-	"recent script files"
-    };	
-    int i, j;
+    int i;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
-	sprintf(gpath, "/gretl/%s/%s", rc_vars[i].description, 
+	sprintf(key, "/gretl/%s/%s", rc_vars[i].description, 
 		rc_vars[i].key);
-	value = gnome_config_get_string(gpath);
+	value = gnome_config_get_string(key);
 	if (value != NULL && *value != '\0') {
 	    if (rc_vars[i].flags & BOOLSET) {
 		str_to_boolvar(value, rc_vars[i].var);
@@ -1479,20 +1477,7 @@ static void read_rc (void)
 	}
     }
 
-    initialize_file_lists();
-
-    /* get recent file lists */
-    for (j=0; j<3; j++) {
-	for (i=0; i<MAXRECENT; i++) {
-	    sprintf(gpath, "/gretl/%s/%d", file_sections[j], i);
-	    if ((value = gnome_config_get_string(gpath)) != NULL) { 
-		write_filename_to_list(j, i, value);
-		g_free(value);
-	    } else {
-		break;
-	    }
-	}
-    }  
+    read_file_lists();
 
     common_read_rc_setup();
 }
@@ -1540,7 +1525,6 @@ void write_rc (void)
     }
 
     save_file_lists();
-
     set_paths(&paths, set_paths_opt);
 }
 
@@ -1603,11 +1587,6 @@ void read_rc (void)
 {
     char rpath[MAXSTR], value[MAXSTR];
     char *strvar;
-    const char *file_sections[] = {
-	"recent data files",
-	"recent session files",
-	"recent script files"
-    };
     int i, j;
 
     if (get_network_settings() && *paths.userdir != '\0') {
@@ -1654,19 +1633,7 @@ void read_rc (void)
 	}
     }
 
-    initialize_file_lists();
-
-    /* get recent file lists */
-    for (j=0; j<3; j++) {
-	for (i=0; i<MAXRECENT; i++) {
-	    sprintf(rpath, "%s\\%d", file_sections[j], i);
-	    if (read_reg_val(HKEY_CURRENT_USER, "gretl", rpath, value) == 0) { 
-		write_filename_to_list(j, i, value);
-	    } else {
-		break;
-	    }
-	}
-    } 
+    read_file_lists();
 
     common_read_rc_setup();
 
@@ -1705,10 +1672,9 @@ void write_rc (void)
     }
 
     save_file_lists(rc);
-
     fclose(rc);
-
     set_paths(&paths, set_paths_opt);
+    record_shell_opt();
 }
 
 static void read_rc (void) 
@@ -1716,12 +1682,6 @@ static void read_rc (void)
     FILE *fp;
     char line[MAXLEN], key[32], linevar[MAXLEN];
     char *strvar;
-    const char *file_sections[] = {
-	"recent data files:",
-	"recent session files:",
-	"recent script files:"
-    };
-    int gotrecent = 0;
     int i, j;
 
     fp = fopen(rcfile, "r");
@@ -1735,8 +1695,7 @@ static void read_rc (void)
 	if (line[0] == '#') {
 	    continue;
 	}
-	if (!strncmp(line, "recent ", 7)) {
-	    gotrecent = 1;
+	if (!strncmp(line, "recent", 6)) {
 	    break;
 	}
 	if (sscanf(line, "%s", key) == 1) {
@@ -1760,44 +1719,7 @@ static void read_rc (void)
 	i++;
     }
 
-    initialize_file_lists();
-
-    if (gotrecent || (fgets(line, MAXLEN, fp) != NULL && 
-		      strncmp(line, file_sections[0], 18) == 0)) {
-	i = 0;
-	while (fgets(line, MAXLEN, fp) && i<MAXRECENT) {
-	    if (strncmp(line, file_sections[1], 21) == 0) {
-		break;
-	    }
-	    chopstr(line);
-	    if (*line != '\0') {
-		write_filename_to_list(FILE_LIST_DATA, i++, line);
-	    }
-	}
-    }
-
-    if (strncmp(line, file_sections[1], 21) == 0) {
-	i = 0;
-	while (fgets(line, MAXLEN, fp) && i<MAXRECENT) {
-	    if (strncmp(line, file_sections[2], 20) == 0) {
-		break;
-	    }
-	    chopstr(line);
-	    if (*line != '\0') {
-		write_filename_to_list(FILE_LIST_SESSION, i++, line);
-	    }
-	}
-    }
-
-    if (strncmp(line, file_sections[2], 20) == 0) {
-	i = 0;
-	while (fgets(line, MAXLEN, fp) && i<MAXRECENT) {
-	    chopstr(line);
-	    if (*line != '\0') {
-		write_filename_to_list(FILE_LIST_SCRIPT, i++, line);
-	    }
-	}
-    }
+    read_file_lists(fp, line);
 
     fclose(fp);
 
@@ -2374,8 +2296,7 @@ static int validate_dir (const char *dirname)
     if (!dir_exists(dirname, NULL)) {
 	err = mkdir(dirname, 0755);
 	if (err) {
-	    sprintf(errtext, _("Couldn't create directory '%s'"), dirname);
-	    errbox(errtext);
+	    errbox(_("Couldn't create directory '%s'"), dirname);
 	} else {
 	    infobox(_("Working directory created OK"));
 	}
