@@ -97,8 +97,6 @@ static int spreadsheet_hide (int i, const DATAINFO *pdinfo)
     return ret;
 }
 
-/* .................................................................. */
-
 static char *single_underscores (char *targ, const char *src)
 {
     char *p = targ;
@@ -118,8 +116,6 @@ static char *single_underscores (char *targ, const char *src)
     return targ;
 }
 
-/* .................................................................. */
-
 static void set_locator_label (Spreadsheet *sheet, GtkTreePath *path,
 			       GtkTreeViewColumn *column)
 {
@@ -140,8 +136,6 @@ static void set_locator_label (Spreadsheet *sheet, GtkTreePath *path,
     g_free(row_label);
 }
 
-/* .................................................................. */
-
 #ifndef OLD_SELECTION
 
 static void move_to_next_cell (Spreadsheet *sheet, GtkTreePath *path,
@@ -182,8 +176,6 @@ static void move_to_next_cell (Spreadsheet *sheet, GtkTreePath *path,
 
 #endif
 
-/* .................................................................. */
-
 static gint sheet_cell_edited (GtkCellRendererText *cell,
 			       const gchar *path_string,
 			       const gchar *user_text,
@@ -234,8 +226,6 @@ static gint sheet_cell_edited (GtkCellRendererText *cell,
     return FALSE;
 }
 
-/* .................................................................. */
-
 static void 
 spreadsheet_scroll_to_new_col (Spreadsheet *sheet, GtkTreeViewColumn *column)
 {
@@ -297,8 +287,6 @@ static int real_add_new_var (Spreadsheet *sheet, const char *varname)
     return 0;
 }
 
-/* .................................................................. */
-
 static void 
 spreadsheet_scroll_to_foot (Spreadsheet *sheet, int row, int col)
 {
@@ -408,8 +396,6 @@ real_add_new_obs (Spreadsheet *sheet, const char *obsname, int n)
     sheet->modified = 1;
 }
 
-/* ........................................................... */
-
 static void name_new_var (GtkWidget *widget, dialog_t *dlg) 
 {
     Spreadsheet *sheet = (Spreadsheet *) edit_dialog_get_data(dlg);
@@ -452,7 +438,7 @@ static void name_var_dialog (Spreadsheet *sheet)
 		 _("Enter name for new variable\n"
 		   "(max. 8 characters)"),
 		 NULL, name_new_var, sheet, 
-		 0, 0, 0);
+		 0, 0);
 }
 
 static void new_case_dialog (Spreadsheet *sheet) 
@@ -461,11 +447,9 @@ static void new_case_dialog (Spreadsheet *sheet)
 		 _("Enter case marker for new obs\n"
 		   "(max. 8 characters)"),
 		 NULL, name_new_obs, sheet, 
-		 0, 0, 0);
+		 0, 0);
 }
 
-/* ........................................................... */
-
 static int add_data_column (Spreadsheet *sheet)
 {
     GType *types;
@@ -535,8 +519,6 @@ static int add_data_column (Spreadsheet *sheet)
     return 0;
 }
 
-/* ........................................................... */
-
 static void sheet_add_var_callback (gpointer data, guint u, GtkWidget *w)
 {
     Spreadsheet *sheet = (Spreadsheet *) data;
@@ -595,8 +577,6 @@ static void build_sheet_popup (Spreadsheet *sheet)
 		   sheet);
 }
 
-/* ......................................................... */
-
 #ifndef OLD_SELECTION
 
 static gboolean update_cell_position (GtkTreeView *view, Spreadsheet *sheet)
@@ -767,15 +747,11 @@ static void get_data_from_sheet (GtkWidget *w, Spreadsheet *sheet)
 
     if (missobs) {
 	infobox(_("Warning: there were missing observations"));
-    } else {
-	infobox(_("Data updated OK"));
-    }
+    } 
 
     sheet->modified = 0;
 }
 
-/* ........................................................... */
-
 static void select_first_editable_cell (Spreadsheet *sheet)
 {
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
@@ -790,8 +766,6 @@ static void select_first_editable_cell (Spreadsheet *sheet)
     gtk_tree_path_free(path);
 }
 
-/* ........................................................... */
-
 static int add_data_to_sheet (Spreadsheet *sheet, SheetCode code)
 {
     gchar rowlabel[OBSLEN];
@@ -865,8 +839,6 @@ static int add_data_to_sheet (Spreadsheet *sheet, SheetCode code)
     return 0;
 }
 
-/* ........................................................... */
-
 static gint get_string_width (const gchar *str)
 {
     gint width;
@@ -908,8 +880,6 @@ static gint get_data_col_width (void)
     return width;
 }
 
-/* ........................................................... */
-
 static void 
 set_up_sheet_column (GtkTreeViewColumn *column, gint width, gboolean expand)
 {
@@ -922,8 +892,6 @@ set_up_sheet_column (GtkTreeViewColumn *column, gint width, gboolean expand)
 #endif
 }
 
-/* ........................................................... */
-
 static void create_sheet_cell_renderers (Spreadsheet *sheet)
 {
     GtkCellRenderer *r;
@@ -1251,8 +1219,6 @@ static int build_sheet_view (Spreadsheet *sheet)
     return 0;
 }
 
-/* ........................................................... */
-
 static void free_spreadsheet (GtkWidget *widget, Spreadsheet **psheet) 
 {
     Spreadsheet *sheet = *psheet;
@@ -1291,7 +1257,16 @@ static Spreadsheet *spreadsheet_new (void)
     return sheet;
 }
 
-/* ........................................................... */
+static int first_var_all_missing (void)
+{
+    int t;
+
+    for (t=0; t<datainfo->n; t++) {
+	if (!na(Z[1][t])) return 0;
+    }
+
+    return 1;
+}
 
 static gint maybe_exit_sheet (GtkWidget *w, Spreadsheet *sheet)
 {
@@ -1305,6 +1280,10 @@ static gint maybe_exit_sheet (GtkWidget *w, Spreadsheet *sheet)
 	    get_data_from_sheet(NULL, sheet);
 	}
 	else if (resp == GRETL_CANCEL || resp == -1) return FALSE;
+    } else if (datainfo->v == 2 && first_var_all_missing()) {
+	data_status |= (GUI_DATA|MODIFIED_DATA);
+	register_data(NULL, NULL, 0);
+	infobox(_("Warning: there were missing observations"));
     }
   
     gtk_widget_destroy(sheet->win);
@@ -1331,8 +1310,6 @@ static gint sheet_delete_event (GtkWidget *w, GdkEvent *event,
     return FALSE;
 }
 
-/* ........................................................... */
-
 void show_spreadsheet (SheetCode code) 
 {
     static Spreadsheet *sheet;    
