@@ -22,9 +22,11 @@
 #include "gretl.h"
 #include "console.h"
 #include "session.h"
+#include "datafiles.h"
 
 #ifdef G_OS_WIN32
 # include "webget.h"
+# include "gretlwin32.h"
 #endif
 
 /* pixmaps for gretl toolbar */
@@ -33,14 +35,18 @@
 #include "../pixmaps/mini.sh.xpm"
 #include "../pixmaps/mini.session.xpm"
 #include "../pixmaps/mini.manual.xpm"
-#include "../pixmaps/mini.netscape.xpm"
 #include "../pixmaps/mini.pdf.xpm"
 #include "../pixmaps/mini.plot.xpm"
 #include "../pixmaps/mini.model.xpm"
 #include "../pixmaps/mini.ofolder.xpm"
+#include "../pixmaps/mini.browser.xpm"
 
 static GtkWidget *toolbar_box;
 
+#ifndef G_OS_WIN32
+extern char Browser[MAXSTR];
+#endif
+
 /* callbacks for gretl toolbar icons */
 
 static void show_calc (void)
@@ -59,7 +65,7 @@ static void open_textbook_data (void)
 
 #ifndef G_OS_WIN32
 
-static void netscape_open (const char *url)
+static void browser_open (const char *url)
 {
 # ifdef USE_GNOME
 #  ifndef OLD_GTK
@@ -69,11 +75,11 @@ static void netscape_open (const char *url)
 #  endif  
 # else
     int err;
-    char ns_cmd[128];
+    char ns_cmd[256];
 
-    sprintf(ns_cmd, "netscape -remote \"openURLNewWindow(%s)\"", url);
+    sprintf(ns_cmd, "%s -remote \"openURLNewWindow(%s)\"", Browser, url);
     err = gretl_spawn(ns_cmd);
-    if (err) gretl_fork("netscape", url);
+    if (err) gretl_fork(Browser, url);
 # endif /* USE_GNOME */
 }
 
@@ -86,23 +92,19 @@ static void gretl_website (void)
 	errbox("Failed to open URL");
     }
 #else
-    netscape_open("http://gretl.sourceforge.net/");
+    browser_open("http://gretl.sourceforge.net/");
 #endif
 }
 
-static void gretl_pdf (void)
+void toolbar_users_guide (void)
 {
-    char manurl[64];
+    display_pdf_help(NULL, 1, NULL);
+}
 
-    sprintf(manurl, "http://gretl.sourceforge.net/%s", _("manual.pdf"));
-
-#ifdef G_OS_WIN32
-    if (goto_url(manurl)) {
-	errbox(_("Failed to open URL"));
-    }
-#else
-    netscape_open(manurl);
-#endif
+void toolbar_command_reference (void)
+{
+    plain_text_cmdref(NULL, 0, NULL);
+    /* display_pdf_help(NULL, 0, NULL); */
 }
 
 static void xy_graph (void)
@@ -189,8 +191,8 @@ static void make_toolbar (GtkWidget *w, GtkWidget *box)
 	N_("open gretl console"),
 	N_("session icon view"),
 	N_("gretl website"), 
-	N_("gretl manual (PDF)"),
-	N_("show help"), 
+	N_("user's guide"),
+	N_("command reference"), 
 	N_("X-Y graph"), 
 	N_("OLS model"),
 	N_("open dataset"),
@@ -244,16 +246,16 @@ static void make_toolbar (GtkWidget *w, GtkWidget *box)
 	    toolfunc = go_session;
 	    break;
 	case 4:
-	    toolxpm = mini_netscape_xpm;
+	    toolxpm = mini_browser_xpm;
 	    toolfunc = gretl_website;
 	    break;  
 	case 5:
 	    toolxpm = mini_pdf_xpm;
-	    toolfunc = gretl_pdf;
+	    toolfunc = toolbar_users_guide;
 	    break;    
 	case 6:
 	    toolxpm = mini_manual_xpm;
-	    toolfunc = do_gui_help;
+	    toolfunc = toolbar_command_reference;
 	    break;
 	case 7:
 	    toolxpm = mini_plot_xpm;
