@@ -24,10 +24,9 @@
    libgretl.
 */
 
-#undef WDEBUG
+#define WDEBUG 0
 
 #ifdef UPDATER
-# include "version.h"
 # define I_(String) String
 # define _(String) String
 # define MAXLEN         512
@@ -35,6 +34,8 @@
 # include "gretl.h"
 #endif
 
+#include "version.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -143,19 +144,65 @@ enum {
 };
 
 typedef enum {
-    NOCONERROR, HOSTERR, CONSOCKERR, CONERROR,
-    CONREFUSED, NEWLOCATION, NOTENOUGHMEM, CONPORTERR,
-    BINDERR, BINDOK, LISTENERR, ACCEPTERR, ACCEPTOK,
-    CONCLOSED, FTPOK, FTPLOGINC, FTPLOGREFUSED, FTPPORTERR,
-    FTPNSFOD, FTPRETROK, FTPUNKNOWNTYPE, FTPRERR,
-    FTPREXC, FTPSRVERR, FTPRETRINT, FTPRESTFAIL,
-    URLOK, URLHTTP, URLFTP, URLFILE, URLUNKNOWN, URLBADPORT,
-    URLBADHOST, FOPENERR, FWRITEERR, HOK, HLEXC, HEOF,
-    HERR, RETROK, RECLEVELEXC, FTPACCDENIED, WRONGCODE,
-    FTPINVPASV, FTPNOPASV,
-    RETRFINISHED, READERR, TRYLIMEXC, URLBADPATTERN,
-    FILEBADFILE, RANGEERR, RETRBADPATTERN, RETNOTSUP,
-    ROBOTSOK, NOROBOTS, PROXERR, AUTHFAILED, QUOTEXC, WRITEFAILED,
+    NOCONERROR, 
+    HOSTERR, 
+    CONSOCKERR, 
+    CONERROR,
+    CONREFUSED, 
+    NEWLOCATION, 
+    NOTENOUGHMEM, 
+    CONPORTERR,
+    BINDERR, 
+    BINDOK, 
+    LISTENERR, 
+    ACCEPTERR, 
+    ACCEPTOK,
+    CONCLOSED, 
+    FTPOK, 
+    FTPLOGINC, 
+    FTPLOGREFUSED, 
+    FTPPORTERR,
+    FTPNSFOD, 
+    FTPRETROK, 
+    FTPUNKNOWNTYPE, 
+    FTPRERR,
+    FTPREXC, 
+    FTPSRVERR, 
+    FTPRETRINT, 
+    FTPRESTFAIL,
+    URLOK, 
+    URLHTTP, 
+    URLFTP, 
+    URLFILE, 
+    URLUNKNOWN, 
+    URLBADPORT,
+    URLBADHOST, 
+    FOPENERR, 
+    FWRITEERR, 
+    HOK, 
+    HLEXC, 
+    HEOF,
+    HERR, 
+    RETROK, 
+    RECLEVELEXC, 
+    FTPACCDENIED, 
+    WRONGCODE,
+    FTPINVPASV, 
+    FTPNOPASV,
+    RETRFINISHED, 
+    READERR, 
+    TRYLIMEXC, 
+    URLBADPATTERN,
+    FILEBADFILE, 
+    RANGEERR, 
+    RETRBADPATTERN, 
+    RETNOTSUP,
+    ROBOTSOK, 
+    NOROBOTS, 
+    PROXERR, 
+    AUTHFAILED, 
+    QUOTEXC, 
+    WRITEFAILED,
     RETRCANCELED
 } uerr_t;
 
@@ -209,15 +256,16 @@ struct proto {
 
 struct urlinfo
 {
-    char *url;                    /* the URL */
-    uerr_t proto;                 /* URL protocol */
+    char *url;                   /* the URL */
+    uerr_t proto;                /* URL protocol */
     unsigned short port;
-    unsigned short saveopt;       /* save to buffer or file? */   
+    unsigned short saveopt;      /* save to buffer or file? */   
     char *path; 
     char *localfile;
     char **savebuf;
     char host[32];
     char errbuf[80];
+    FILE *fp;                   /* for saving content locally */
 };
 
 struct http_stat {
@@ -292,8 +340,6 @@ enum header_get_flags {
 
 /* end of header-type info, private to webget.c */
 
-extern const char *version_string;
-
 static struct urlinfo gretlproxy; 
 
 /* prototypes */
@@ -321,7 +367,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs, int *dt,
 		       struct urlinfo *proxy);
 static uerr_t http_loop (struct urlinfo *u, int *dt, struct urlinfo *proxy);
 static struct urlinfo *newurl (void);
-static void freeurl (struct urlinfo *u, int complete);
+static void freeurl (struct urlinfo *u, int delfile);
 static int get_contents (int fd, FILE *fp, char **getbuf, long *len, 
 			 long expected, struct rbuf *rbuf);
 static int store_hostaddress (unsigned char *where, const char *hostname);
@@ -362,7 +408,7 @@ int ws_startup (void)
 
 #endif /* WIN32 */
 
-static int get_db_host_ip (char *h_ip, const char *h_name)
+static int get_host_ip (char *h_ip, const char *h_name)
 {
     struct hostent *h_ent;
 
@@ -384,8 +430,6 @@ static int get_db_host_ip (char *h_ip, const char *h_name)
     return 0;
 }
 
-/* ........................................................... */
-
 static char *time_str (time_t *tm)
 {
   static char tms[15];
@@ -403,8 +447,6 @@ static char *time_str (time_t *tm)
 
 /* http header functions -- based on Wget */
 
-/* ........................................................... */
-
 static int rbuf_peek (struct rbuf *rbuf, char *store)
 {
     if (!rbuf->buffer_left) {
@@ -424,8 +466,6 @@ static int rbuf_peek (struct rbuf *rbuf, char *store)
     return 1;
 }
 
-/* ........................................................... */
-
 #ifdef UPDATER
 
 void clear (char *str, int len)
@@ -466,8 +506,6 @@ static char *g_strdup (const char *s)
 
 #endif /* UPDATER */
 
-/* ........................................................... */
-
 static int header_get (struct rbuf *rbuf, char **hdr, 
 		       enum header_get_flags flags)
 {
@@ -491,12 +529,12 @@ static int header_get (struct rbuf *rbuf, char **hdr,
 		if (!((flags & HG_NO_CONTINUATIONS)
 		      || i == 0
 		      || (i == 1 && (*hdr)[0] == '\r'))) {
-		    char next;
+		    char next = 0;
 
 		    /* If the header is non-empty, we need to check if
 		       it continues on to the other line.  We do that by
 		       peeking at the next character. */
-		    res = rbuf_peek (rbuf, &next);
+		    res = rbuf_peek(rbuf, &next);
 		    if (res == 0) {
 			return HG_EOF;
 		    } else if (res == -1) {
@@ -526,8 +564,6 @@ static int header_get (struct rbuf *rbuf, char **hdr,
     return HG_OK;
 }
 
-/* ........................................................... */
-
 static int header_extract_number (const char *header, void *closure)
 {
     const char *p = header;
@@ -545,16 +581,12 @@ static int header_extract_number (const char *header, void *closure)
     return 1;
 }
 
-/* ........................................................... */
-
 static int header_strdup (const char *header, void *closure)
 {
     *(char **) closure = g_strdup(header);
     return 1;
 }
 
-/* ........................................................... */
-
 static int skip_lws (const char *string)
 {
     const char *p = string;
@@ -566,8 +598,6 @@ static int skip_lws (const char *string)
     return p - string;
 }
 
-/* ........................................................... */
-
 static int header_process (const char *header, const char *name,
 			   int (*procfun) (const char *, void *),
 			   void *arg)
@@ -589,8 +619,6 @@ static int header_process (const char *header, const char *name,
 
 /* further functions from Wget's http.c */
 
-/* ........................................................... */
-
 static void rbuf_initialize (struct rbuf *rbuf, int fd)
 {
     rbuf->fd = fd;
@@ -598,8 +626,6 @@ static void rbuf_initialize (struct rbuf *rbuf, int fd)
     rbuf->buffer_left = 0;
 }
 
-/* ........................................................... */
-
 static int parse_http_status_line (const char *line, 
 				   const char **reason_phrase_ptr)
 /* Parse the HTTP status line, which is of format:
@@ -677,11 +703,10 @@ struct http_process_range_closure {
     long entity_length;
 };
 
-/* ........................................................... */
-
-static int http_process_range (const char *hdr, void *arg)
 /* Parse the `Content-Range' header and extract the information it
    contains.  Returns 1 if successful, -1 otherwise.  */
+
+static int http_process_range (const char *hdr, void *arg)
 {
     struct http_process_range_closure *closure
 	= (struct http_process_range_closure *)arg;
@@ -730,11 +755,10 @@ static int http_process_range (const char *hdr, void *arg)
     return 1;
 }
 
-/* ........................................................... */
-
-static int http_process_none (const char *hdr, void *arg)
 /* Place 1 to ARG if the HDR contains the word "none", 0 otherwise.
    Used for `Accept-Ranges'.  */
+
+static int http_process_none (const char *hdr, void *arg)
 {
     int *where = (int *) arg;
 
@@ -747,10 +771,9 @@ static int http_process_none (const char *hdr, void *arg)
     return 1;
 }
 
-/* ........................................................... */
+/* Place the malloc-ed copy of HDR hdr, to the first `;' to ARG */
 
 static int http_process_type (const char *hdr, void *arg)
-/* Place the malloc-ed copy of HDR hdr, to the first `;' to ARG */
 {
     char **result = (char **) arg;
     char *p;
@@ -777,8 +800,6 @@ static int http_process_type (const char *hdr, void *arg)
   (x).newloc = (x).remote_time = (x).error = NULL;	\
 } while (0)
 
-/* ........................................................... */
-
 static int numdigit (long a)
 {
     int res = 1;
@@ -787,8 +808,6 @@ static int numdigit (long a)
     return res;
 }
 
-/* ........................................................... */
-
 static char *herrmsg (int error)
 {
     if (error == HOST_NOT_FOUND
@@ -802,8 +821,6 @@ static char *herrmsg (int error)
     }
 }
 
-/* ........................................................... */
-
 static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs, 
 		       int *dt, struct urlinfo *proxy)
 {
@@ -814,7 +831,6 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
     int sock, hcount, num_written, all_length, statcode;
     long contlen, contrange;
     uerr_t err;
-    FILE *fp = NULL;
     struct rbuf rbuf;
 
     hs->len = 0L;
@@ -824,7 +840,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
     hs->remote_time = NULL;
     hs->error = NULL;
 
-    /* If we're using a proxy, we'll connect to the proxy server. */
+    /* If we're using a proxy, we'll connect to the proxy server */
     conn = (proxy != NULL)? proxy : u;
 
     err = make_connection(&sock, conn->host, conn->port);
@@ -856,25 +872,11 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	break;
     } 
 
-#ifdef WDEBUG
+#if WDEBUG
     fprintf(stderr, "connected to %s, port %d at socket %d\n",
 	    conn->host, conn->port, sock);
 #endif   
 
-    if (u->saveopt == SAVE_TO_FILE) { 
-#ifdef USE_G_FOPEN
-	fp = g_fopen(u->localfile, "wb");
-#else
-	fp = fopen(u->localfile, "wb");
-#endif
-	if (fp == NULL) {
-	    close(sock);
-	    free(all_headers);
-	    fprintf(stderr, "Couldn't open local file '%s'\n", u->localfile);
-	    return FOPENERR;
-	}
-    } 
-
     if (proxy) {
 	path = mymalloc(strlen(u->host) + strlen(u->path) + 8);
 	sprintf(path, "http://%s%s", u->host, u->path);
@@ -887,7 +889,6 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	free(all_headers);
 	return NOTENOUGHMEM;
     }
-	
 
     command = (*dt & HEAD_ONLY)? "HEAD" : "GET";
 
@@ -898,7 +899,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
     }
 
     range = NULL;
-    sprintf(useragent, "gretl-%s", version_string);
+    sprintf(useragent, "gretl-%s", GRETL_VERSION);
 #if defined(UPDATER) || defined (WIN32)
     /* the linux test updater program pretends to be Windows */
     strcat(useragent, "w");
@@ -925,9 +926,11 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	    command, path, useragent, u->host, u->port, HTTP_ACCEPT,
 	    pragma_h); 
 
-    if (proxy) free(path);
+    if (proxy) {
+	free(path);
+    }
 
-#ifdef WDEBUG
+#if WDEBUG
     fprintf(stderr, "Request:\n%s", request);
 #endif
 
@@ -936,7 +939,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
     free(request); /* moved from within following conditional, 03/25/01 */
     if (num_written < 0) {
 	close(sock);
-#ifdef WDEBUG
+#if WDEBUG
 	fprintf(stderr, "Failed to write to socket\n");
 #endif
 	return WRITEFAILED;
@@ -964,7 +967,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 			    (hcount == 1 ? HG_NO_CONTINUATIONS : HG_NONE));
 	/* Check for errors */
 	if (status == HG_EOF && *hdr) {
-#ifdef WDEBUG
+#if WDEBUG
 	    fprintf(stderr, "Got status = HG_EOF\n");
 #endif
 	    free(hdr);
@@ -974,7 +977,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	    close(sock);
 	    return HEOF;
 	} else if (status == HG_ERROR) {
-#ifdef WDEBUG
+#if WDEBUG
 	    fprintf(stderr, "Got status = HG_ERROR\n");
 #endif
 	    free(hdr);
@@ -1013,7 +1016,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	    goto done_header;
 	}
 
-#ifdef WDEBUG
+#if WDEBUG
 	fprintf(stderr, "hs->error: '%s'\n", hs->error);
 #endif
 
@@ -1022,28 +1025,33 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	    free(hdr);
 	    break;
 	}
+
 	/* Try getting content-length */
 	if (contlen == -1) {
 	    if (header_process(hdr, "Content-Length", header_extract_number,
 			       &contlen))
 		goto done_header;
 	}
+
 	/* Try getting content-type */
 	if (!type) {
 	    if (header_process (hdr, "Content-Type", http_process_type, &type))
 		goto done_header;
 	}
+
 	/* Try getting location */
 	if (!hs->newloc) {
 	    if (header_process (hdr, "Location", header_strdup, &hs->newloc))
 		goto done_header;
 	}
+
 	/* Try getting last-modified */
 	if (!hs->remote_time) {
 	    if (header_process (hdr, "Last-Modified", header_strdup,
 				&hs->remote_time))
 		goto done_header;
 	}
+
 	/* Check for accept-ranges header.  If it contains the word
 	   `none', disable the ranges */
 	if (*dt & ACCEPTRANGES) {
@@ -1057,6 +1065,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 		goto done_header;
 	    }
 	}
+
 	/* Try getting content-range */
 	if (contrange == -1) {
 	    struct http_process_range_closure closure;
@@ -1066,6 +1075,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 		goto done_header;
 	    }
 	}
+
     done_header:
 	free(hdr);
     }
@@ -1115,15 +1125,13 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
     }
 
     /* Get the contents of the document */
-    hs->res = get_contents(sock, fp, u->savebuf, &hs->len, 
+    hs->res = get_contents(sock, u->fp, u->savebuf, &hs->len, 
 			   (contlen != -1 ? contlen : 0), &rbuf);
 
-#ifdef WDEBUG
+#if WDEBUG
     fprintf(stderr, "get_contents returned %d\n", hs->res);
 #endif
 
-    if (fp != NULL) fclose(fp);
-
     free(all_headers);
     close(sock);
 
@@ -1136,14 +1144,12 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 
 #define MAXTRY 5
 
-/* ........................................................... */
-
 static uerr_t http_loop (struct urlinfo *u, int *dt, struct urlinfo *proxy)
 {
     int count = 0;
     char *tms;
     uerr_t err;
-    struct http_stat hstat;	/* HTTP status */
+    struct http_stat hstat; /* HTTP status */
 
     *dt = 0 | ACCEPTRANGES;
 
@@ -1160,7 +1166,7 @@ static uerr_t http_loop (struct urlinfo *u, int *dt, struct urlinfo *proxy)
 	/* Time? */
 	tms = time_str(NULL);
 
-#ifdef WDEBUG
+#if WDEBUG
 	fprintf(stderr, "http_loop: err (from gethttp) = %d, errbuf = '%s'\n", 
 		err, (proxy)? proxy->errbuf : u->errbuf);
 	if (err == RETRFINISHED) {
@@ -1215,12 +1221,12 @@ static uerr_t http_loop (struct urlinfo *u, int *dt, struct urlinfo *proxy)
 
 /* other utility functions from Wget */
 
-/* ........................................................... */
 
-static size_t rbuf_flush (struct rbuf *rbuf, char *where, int maxsize)
 /* Flush RBUF's buffer to WHERE.  Flush MAXSIZE bytes at most.
    Returns the number of bytes actually copied.  If the buffer is
    empty, 0 is returned.  */
+
+static size_t rbuf_flush (struct rbuf *rbuf, char *where, int maxsize)
 {
     if (!rbuf->buffer_left) {
 	return 0;
@@ -1236,8 +1242,6 @@ static size_t rbuf_flush (struct rbuf *rbuf, char *where, int maxsize)
     }
 }
 
-/* ........................................................... */
-
 static void url_init (struct urlinfo *u)
 {
     u->url = NULL;
@@ -1251,11 +1255,13 @@ static void url_init (struct urlinfo *u)
     u->host[0] = '\0';
     u->errbuf[0] = '\0';
     u->saveopt = 0;
+    u->fp = NULL;
 }
 
-static struct urlinfo *newurl (void)
 /* Allocate a new urlinfo structure, fill it with default values and
    return a pointer to it.  */
+
+static struct urlinfo *newurl (void)
 {
     struct urlinfo *u;
 
@@ -1267,28 +1273,32 @@ static struct urlinfo *newurl (void)
     return u;
 }
 
-/* ........................................................... */
-
-static void freeurl (struct urlinfo *u, int complete)
 /* Perform a "deep" free of the urlinfo structure.  The structure
    should have been created with newurl, but need not have been used.
-   If complete is non-0, free the pointer itself.  */
+   If defile is non-zero and there's a local file open, delete
+   that file.
+*/
+
+static void freeurl (struct urlinfo *u, int delfile)
 {
     if (u == NULL) return;
 
     free(u->url);
     free(u->path);
-    if (u->localfile) {
+
+    if (u->localfile != NULL) {
+	if (u->fp != NULL) {
+	    fclose(u->fp);
+	}
+	if (delfile) {
+	    remove(u->localfile);
+	}
 	free(u->localfile);
     }
 
-    if (complete) {
-	free(u);
-    }
+    free(u);
 }
 
-/* ........................................................... */
-
 static int get_contents (int fd, FILE *fp, char **getbuf, long *len, 
 			 long expected, struct rbuf *rbuf)
 {
@@ -1316,6 +1326,12 @@ static int get_contents (int fd, FILE *fp, char **getbuf, long *len,
 
     *len = 0L;
 
+#if 0
+    if (fp != NULL) {
+	rewind(fp);
+    }
+#endif
+
     if (rbuf && RBUF_FD(rbuf) == fd) {
 	while ((res = rbuf_flush(rbuf, cbuf, sizeof cbuf)) != 0) {
 	    if (fp == NULL) {
@@ -1385,13 +1401,11 @@ static int get_contents (int fd, FILE *fp, char **getbuf, long *len,
     return res;
 }
 
-/* ........................................................... */
-
 static int store_hostaddress (unsigned char *where, const char *hostname)
 {
     unsigned long addr = (unsigned long) inet_addr(hostname);
 
-#ifdef WDEBUG
+#if WDEBUG
     fprintf(stderr, "store_hostaddress: hostname='%s', addr=%lu\n",
 	    hostname, addr);
 #endif
@@ -1407,8 +1421,6 @@ static int store_hostaddress (unsigned char *where, const char *hostname)
 # define ECONNREFUSED WSAECONNREFUSED
 #endif
 
-/* ........................................................... */
-
 static uerr_t make_connection (int *sock, char *hostname, unsigned short port)
 /* Create an internet connection to HOSTNAME on PORT.  The created
    socket will be stored to *SOCK.  */
@@ -1437,10 +1449,9 @@ static uerr_t make_connection (int *sock, char *hostname, unsigned short port)
     return NOCONERROR;
 }
 
-/* ........................................................... */
+/* Read at most len bytes from FD, storing them to buf. */
 
 static int iread (int fd, char *buf, int len)
-/* Read at most len bytes from FD, storing them to buf. */
 {
     int res;
 
@@ -1451,14 +1462,13 @@ static int iread (int fd, char *buf, int len)
     return res;
 }
 
-/* ........................................................... */
-
-static int iwrite (int fd, char *buf, int len)
 /* Write len bytes from buf to fd.  This is similar to iread(), but
    doesn't bother with select().  Unlike iread(), it makes sure that
    all of BUF is actually written to FD, so callers needn't bother
    with checking that the return value equals to LEN.  Instead, you
    should simply check for -1.  */
+
+static int iwrite (int fd, char *buf, int len)
 {
     int res = 0;
 
@@ -1477,8 +1487,6 @@ static int iwrite (int fd, char *buf, int len)
     return res;
 }
 
-/* ........................................................... */
-
 static char *print_option (int opt)
 {
     switch (opt) {
@@ -1500,8 +1508,6 @@ static char *print_option (int opt)
     return NULL;
 } 
 
-/* ........................................................... */
-
 static int get_update_info (char **saver, char *errbuf, time_t filedate,
 			    int queryopt)
 {
@@ -1524,18 +1530,18 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
     u->path = mymalloc(strlen(cgi) + 64);
 
     if (u->path == NULL) {
-	freeurl(u, 1);
+	freeurl(u, 0);
 	return 1;
     }
 
 #ifdef UPDATER
-    err = get_db_host_ip(u->host, dbhost);
+    err = get_host_ip(u->host, dbhost);
 #else
-    err = get_db_host_ip(u->host, paths.dbhost);
+    err = get_host_ip(u->host, paths.dbhost);
 #endif
 
     if (err) {
-	freeurl(u, 1);
+	freeurl(u, 0);
 	return err;
     }
 
@@ -1554,7 +1560,7 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
 
     result = http_loop(u, &dt, proxy); 
 
-#ifdef WDEBUG
+#if WDEBUG
     fprintf(stderr, "http_loop returned %d, u->errbuf='%s'\n",
 	    (int) result, u->errbuf);
 #endif
@@ -1566,7 +1572,7 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
 	err = 1;
     }
 
-    freeurl(u, 1);
+    freeurl(u, 0);
 
     return err;
 }
@@ -1592,8 +1598,9 @@ static size_t get_size (char *buf)
 }
 # endif /* WIN32 */
 
+/* E.g. Sun Mar 16 13:50:52 EST 2003 */
+
 static time_t get_time_from_stamp_file (const char *fname)
-     /* E.g. Sun Mar 16 13:50:52 EST 2003 */
 {
     FILE *fp;
     struct tm stime;
@@ -1759,8 +1766,6 @@ int update_query (void)
 
 #endif /* ! UPDATER */
 
-/* ........................................................... */
-
 int proxy_init (const char *dbproxy)
 {
     char *p;
@@ -1799,17 +1804,35 @@ int proxy_init (const char *dbproxy)
     return 0;
 } 
 
-/* ........................................................... */
+static int open_local_file (struct urlinfo *u)
+{
+    int err = 0;
+
+    if (u->saveopt == SAVE_TO_FILE) { 
+#ifdef USE_G_FOPEN
+	u->fp = g_fopen(u->localfile, "wb");
+#else
+	u->fp = fopen(u->localfile, "wb");
+#endif
+	if (u->fp == NULL) {
+	    fprintf(stderr, "Couldn't open local file '%s'\n", u->localfile);
+	    err = 1;
+	}
+    }
+
+    return err;
+}
 
-static int 
-retrieve_url (int opt, const char *fname, const char *dbseries, 
-	      int saveopt, const char *savefile, char **savebuf,
-	      char *errbuf)
 /* grab data from URL.  If saveopt = SAVE_TO_FILE then data is stored to
    a local file whose name is given by "savefile".  If saveopt = SAVE_TO_BUFFER
    then "savebuf" is presumed to point to a char buffer to which the data
    should be written.
 */
+
+static int 
+retrieve_url (int opt, const char *fname, const char *dbseries, 
+	      int saveopt, const char *savefile, char **savebuf,
+	      char *errbuf)
 {
     uerr_t result;
     struct urlinfo *u;
@@ -1838,18 +1861,18 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
 
     u->path = mymalloc(strlen(cgi) + fnlen + 64);
     if (u->path == NULL) {
-	freeurl(u, 1);
+	freeurl(u, 0);
 	return 1;
     }
 
 #ifdef UPDATER
-    err = get_db_host_ip(u->host, dbhost);
+    err = get_host_ip(u->host, dbhost);
 #else
-    err = get_db_host_ip(u->host, paths.dbhost);
+    err = get_host_ip(u->host, paths.dbhost);
 #endif
 
     if (err) {
-	freeurl(u, 1);
+	freeurl(u, 0);
 	return err;
     }
 
@@ -1873,6 +1896,11 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
     if (saveopt == SAVE_TO_FILE) {
 	u->localfile = g_strdup(savefile);
 	u->savebuf = NULL;
+	err = open_local_file(u);
+	if (err) {
+	    freeurl(u, 0);
+	    return err;
+	}
     } else {
 	u->localfile = NULL;
 	u->savebuf = savebuf;
@@ -1880,7 +1908,7 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
 
     result = http_loop(u, &dt, proxy);
 
-#ifdef WDEBUG
+#if WDEBUG
     fprintf(stderr, "http_loop returned %d, u->errbuf='%s'\n",
 	    (int) result, u->errbuf);
 #endif
@@ -1892,11 +1920,75 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
 	err = 1;
     }
 
-    freeurl(u, 1);
+    freeurl(u, err);
 
     return err;
 }
 
+#ifndef UPDATER
+
+int retrieve_manfile (const char *fname, const char *savefile, char *errbuf)
+{
+    const char *manhost = "ricardo.ecn.wfu.edu";
+    uerr_t result;
+    struct urlinfo *u;
+    struct urlinfo *proxy = NULL; 
+    int dt, err = 0;
+
+    *errbuf = '\0';
+
+    if (use_proxy) {
+	proxy = &gretlproxy;
+    }
+
+    u = newurl();
+    if (u == NULL) {
+	return 1;
+    }
+
+    err = get_host_ip(u->host, manhost);
+    if (err) {
+	freeurl(u, 0);
+	return err;
+    }
+
+    u->path = g_strdup_printf("/pub/gretl/manual/PDF/%s", fname);
+    if (u->path == NULL) {
+	freeurl(u, 0);
+	return 1;
+    }
+
+    u->saveopt = SAVE_TO_FILE;
+    u->localfile = g_strdup(savefile);
+    u->savebuf = NULL;
+
+    err = open_local_file(u);
+    if (err) {
+	freeurl(u, 0);
+	return err;
+    }
+
+    result = http_loop(u, &dt, proxy);
+
+#if WDEBUG
+    fprintf(stderr, "http_loop returned %d, u->errbuf='%s'\n",
+	    (int) result, u->errbuf);
+#endif
+
+    if (result == RETROK) {
+	*errbuf = 0;
+    } else {
+	strcpy(errbuf, u->errbuf);
+	err = 1;
+    }
+
+    freeurl(u, err);
+
+    return err;
+}
+
+#endif /* !UPDATER */
+
 #ifdef WIN32
 
 #include <windows.h>
