@@ -217,6 +217,112 @@ void calendar_date_string (char *str, int t, const DATAINFO *pdinfo)
     }
 }
 
+/**
+ * MS_excel_date_string:
+ * @date: date string to be filled out.
+ * @mst: MS Excel-type date code: days since base.
+ * @pd: periodicity of data (or 0 if unknown).
+ * @d1904: set to 1 if the base is 1904/01/01; otherwise
+ * the base is assumed to be 1899/12/31.
+ * 
+ * Writes to @date the calendar representation of the date of
+ * observation @mst, in the form YYYY/MM/DD if @pd is 0, 5,
+ * 6, 7 or 52 (unknown, daily, or weekly frequency), otherwise 
+ * in the appropriate format for annual, quarterly or monthly data.
+ * 
+ * Returns: 0.
+ */
+
+int MS_excel_date_string (char *date, int mst, int pd, int d1904)
+{
+    int yr = (d1904)? 1904 : 1900;
+    int day = (d1904)? 2 : 1;
+    int mo = 1;
+    int leap, drem;
+
+    if (mst == 0) {
+	if (d1904) {
+	    day = 1;
+	} else {
+	    yr = 1899;
+	    mo = 12;
+	    day = 31;
+	}
+    } else if (mst > 0) {
+	drem = mst + d1904;
+
+	while (1) {
+	    int yd = 365 + leap_year(yr);
+
+	    /* MS tomfoolery */
+	    if (yr == 1900) yd++;
+
+	    if (drem > yd) {
+		drem -= yd;
+		yr++;
+	    } else {
+		break;
+	    }
+	}
+
+	leap = leap_year(yr) + (yr == 1900);
+
+	for (mo=1; mo<13; mo++) {
+	    int md = days_in_month[leap][mo];
+
+	    if (drem > md) {
+		drem -= md;
+	    } else {
+		day = drem;
+		break;
+	    }
+	}
+    } else {
+	/* mst < 0, date prior to base */
+	drem = - (mst + d1904);
+
+	yr = (d1904)? 1903 : 1899;
+
+	while (1) {
+	    int yd = 365 + leap_year(yr);
+
+	    if (drem > yd) {
+		drem -= yd;
+		yr--;
+	    } else {
+		break;
+	    }
+	}
+
+	leap = leap_year(yr);
+
+	for (mo=12; mo>0; mo--) {
+	    int md = days_in_month[leap][mo];
+
+	    if (drem >= md) {
+		drem -= md;
+	    } else {
+		day = md - drem;
+		break;
+	    }
+	}
+    }
+
+    if (pd == 1) {
+	sprintf(date, "%d", yr);
+    } else if (pd == 12) {
+	sprintf(date, "%d:%02d", yr, mo);
+    } else if (pd == 4) {
+	int qtr = 1 + mo / 3.25;
+
+	sprintf(date, "%d:%d", yr, qtr);
+    } else {
+	sprintf(date, "%04d/%02d/%02d", yr, mo, day);
+    }
+
+    return 0;
+}
+
 /**
  * get_dec_date:
  * @date: calendar representation of date.
