@@ -245,7 +245,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 		    cmp->dfn, cmp->dfd, cmp->F);
 	    pprintf(prn, _("with p-value = %g\n"), pval);
 	}
-	record_test_result(cmp->F, pval, (cmp->cmd == OMIT)? "omit" : "add");
+	record_test_result(cmp->F, pval, (cmp->cmd == OMIT)? _("omit") : _("add"));
 
 	if (test != NULL) {
 	    model_test_set_teststat(test, GRETL_STAT_F);
@@ -264,7 +264,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 		    _("Chi-square"), cmp->dfn, cmp->chisq);
 	    pprintf(prn, _("with p-value = %g\n\n"), pval);
 	}
-	record_test_result(cmp->chisq, pval, (cmp->cmd == OMIT)? "omit" : "add");
+	record_test_result(cmp->chisq, pval, (cmp->cmd == OMIT)? _("omit") : _("add"));
 
 	if (test != NULL) {
 	    model_test_set_teststat(test, (LIMDEP(cmp->ci))? 
@@ -348,7 +348,7 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int add,
 /* reconstitute full varlist for WLS, POISSON, AR and models */
 
 static int *
-full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
+full_model_list (const MODEL *pmod, const int *inlist)
 {
     int i, len, pos = 0;
     int *flist = NULL;
@@ -397,8 +397,6 @@ full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
 	}
     }
 
-    *ppos = pos;
-
     return flist;
 }
 
@@ -413,7 +411,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
     MODEL rep;
     double rho = 0.0;
     int *list = *plist;
-    int pos = 0, mc = get_model_count();
+    int mc = get_model_count();
     int repci = orig->ci;
 
     gretl_model_init(&rep);
@@ -432,7 +430,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	repci = POOLED;
     } else if (orig->ci == WLS || orig->ci == AR || 
 	       (orig->ci == POISSON && gretl_model_get_int(orig, "offset_var"))) {
-	int *full_list = full_model_list(orig, list, &pos);
+	int *full_list = full_model_list(orig, list);
 
 	free(list);
 	if (full_list == NULL) {
@@ -449,7 +447,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
     switch (orig->ci) {
 
     case AR:
-	rep = ar_func(list, pos, pZ, pdinfo, lsqopt, prn);
+	rep = ar_func(list, pZ, pdinfo, lsqopt, prn);
 	break;
     case ARCH:
 	rep = arch_model(list, orig->order, pZ, pdinfo, lsqopt, prn);
@@ -493,7 +491,11 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	if (gretl_model_get_int(orig, "hc_version") == 4) {
 	    repci = HCCM;
 	}
-	rep = lsq(list, pZ, pdinfo, repci, lsqopt, rho);
+	if (rho != 0.0) {
+	    rep = ar1_lsq(list, pZ, pdinfo, repci, lsqopt, rho);
+	} else {
+	    rep = lsq(list, pZ, pdinfo, repci, lsqopt);
+	}
 	break;
     }
 
@@ -544,7 +546,7 @@ real_nonlinearity_test (MODEL *pmod, int *list,
     /* replace the dependent var */
     list[1] = pdinfo->v - 1;
 
-    aux = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+    aux = lsq(list, pZ, pdinfo, OLS, OPT_A);
     if (aux.errcode) {
 	err = aux.errcode;
 	fprintf(stderr, "auxiliary regression failed\n");
@@ -574,7 +576,7 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 	    }
 	}
 
-	record_test_result(trsq, pval, "non-linearity");
+	record_test_result(trsq, pval, _("non-linearity"));
     } 
 
     clear_model(&aux);
@@ -763,7 +765,7 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 	    struct COMPARE cmp;
 	    int *addlist;
 
-	    addlist = gretl_list_diff_new(new->list, orig->list);
+	    addlist = gretl_list_diff_new(new->list, orig->list, 2);
 	    cmp = add_or_omit_compare(orig, new, 1, addlist);
 
 	    gretl_make_compare(&cmp, addlist, orig, pdinfo, opt, prn);
@@ -846,7 +848,7 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	/* special: just drop the last variable */
 	tmplist = gretl_list_omit_last(orig->list, &err);
     } else {
-	tmplist = gretl_list_omit(orig->list, omitvars, &err);
+	tmplist = gretl_list_omit(orig->list, omitvars, 2, &err);
     }
 
     if (tmplist == NULL) {
@@ -902,7 +904,7 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	    struct COMPARE cmp;
 	    int *omitlist;
 
-	    omitlist = gretl_list_diff_new(orig->list, new->list);
+	    omitlist = gretl_list_diff_new(orig->list, new->list, 2);
 	    cmp = add_or_omit_compare(orig, new, 0, omitlist);
 
 	    gretl_make_compare(&cmp, omitlist, orig, pdinfo, opt, prn); 
@@ -1029,7 +1031,7 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A, 0.0);
+	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A);
 	err = aux.errcode;
 	if (err) {
 	    errmsg(aux.errcode, prn);
@@ -1174,7 +1176,7 @@ int autocorr_test (MODEL *pmod, int order,
     if (!err) {
 	newlist[1] = v;
 	/* printlist(newlist); */
-	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A, 0.0);
+	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A);
 	err = aux.errcode;
 	if (err) {
 	    errmsg(aux.errcode, prn);
@@ -1203,7 +1205,7 @@ int autocorr_test (MODEL *pmod, int order,
 			lb, _("with p-value"), _("Chi-square"), order,
 			lb, chisq(lb, order));
 	    }
-	    record_test_result(LMF, pval, "autocorrelation");
+	    record_test_result(LMF, pval, _("autocorrelation"));
 	}
 
 	if (opt & OPT_S) {
@@ -1329,7 +1331,7 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	    chowlist[pmod->list[0] + 1 + i] = v + i;
 	}
 
-	chow_mod = lsq(chowlist, pZ, pdinfo, OLS, OPT_A, 0.0);
+	chow_mod = lsq(chowlist, pZ, pdinfo, OLS, OPT_A);
 
 	if (chow_mod.errcode) {
 	    err = chow_mod.errcode;
@@ -1461,7 +1463,7 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
     if (!err) {
 	for (j=0; j<n_est && !err; j++) {
-	    cum_mod = lsq(pmod->list, pZ, pdinfo, OLS, cmodopt, 0.0);
+	    cum_mod = lsq(pmod->list, pZ, pdinfo, OLS, cmodopt);
 	    err = cum_mod.errcode;
 	    if (err) {
 		errmsg(err, prn);
@@ -1879,7 +1881,8 @@ int mp_ols (const int *list, const char *pos,
     mplsq = get_plugin_function("mplsq", &handle);
     if (mplsq == NULL) return 1;
 
-    if (!err && *pos) { /* got a list of polynomial terms? */
+    if (!err && *pos) { 
+	/* got a list of polynomial terms? */
 	err = make_mp_lists(list, pos, &tmplist, &polylist);
 	if (err) {
 	    pputs(prn, _("Failed to parse mp_ols command\n"));
@@ -1892,9 +1895,12 @@ int mp_ols (const int *list, const char *pos,
     }
 
     nc = list[0] - 1;
-    if (polylist != NULL) nc--;
+    if (polylist != NULL) {
+	nc--;
+    }
 
     mpvals = gretl_mp_results_new(nc);
+
     if (mpvals == NULL || allocate_mp_varnames(mpvals)) {
 	pprintf(prn, "%s\n", _("Out of memory!"));
 	err = 1;
@@ -1923,7 +1929,9 @@ static int varmatch (const int *sumvars, int test)
     int j;
 
     for (j=1; j<=sumvars[0]; j++) {
-	if (sumvars[j] == test) return 1;
+	if (sumvars[j] == test) {
+	    return 1;
+	}
     }
 
     return 0;
@@ -2060,7 +2068,7 @@ int sum_test (const int *sumvars, MODEL *pmod,
 		pval = coeff_pval(&summod, tval, summod.dfd);
 		pprintf(prn, "   t(%d) = %g ", summod.dfd, tval);
 		pprintf(prn, _("with p-value = %g\n"), pval);
-		record_test_result(tval, pval, "sum");
+		record_test_result(tval, pval, _("sum"));
 	    }
 	}
     }
