@@ -638,7 +638,7 @@ int rhodiff (char *param, const int *list, double ***pZ, DATAINFO *pdinfo)
     p = 0;
     for (i=0; i<j; i++) {
 	if ((i == 0 || param[i] == ' ') && i < (j - 1)) {
-	    sscanf(param + i + (i? 1 : 0), "%8s", parmbit); 
+	    sscanf(param + i + (i? 1 : 0), "%15s", parmbit); 
 #ifdef RHODEBUG
 	    fprintf(stderr, "rhodiff: parmbit = '%s'\n", parmbit);
 #endif
@@ -666,7 +666,8 @@ int rhodiff (char *param, const int *list, double ***pZ, DATAINFO *pdinfo)
 
 	j = list[i];
 
-	strncat(pdinfo->varname[vr], pdinfo->varname[j], 7);
+	*pdinfo->varname[vr] = 0;
+	strncat(pdinfo->varname[vr], pdinfo->varname[j], VNAMELEN-2);
 	strcat(pdinfo->varname[vr], "#");
 
 	sprintf(VARLABEL(pdinfo, vr), _("= rho-differenced %s"),
@@ -699,230 +700,6 @@ int rhodiff (char *param, const int *list, double ***pZ, DATAINFO *pdinfo)
     return 0;
 }
 
-/* stuff to support "sim" command */
-
-/* ensure there's no gap betweeen a minus sign and the term
-   it qualifies, in a "sim" command */
-
-static char *close_minus (char *str)
-{
-    char *q, *p = str;
-
-    while ((p = strchr(p, '-'))) {
-	q = ++p;
-	while (*q == ' ') q++;
-	if (q != p) {
-	    memmove(p, q, strlen(q) + 1);
-	}
-    }
-
-    return str;
-}
-
-/* construct a descriptive label for a variable that is
-   modified via the "sim" command */
-
-static char *make_sim_label (char *label, const char *vname,
-			     char **parm, int nparm)
-{
-    int k, neg, started = 0;
-    char term[32];
-
-    sprintf(label, "%s(t)=", vname);
-
-    for (k=0; k<nparm; k++) {
-	if (isdigit(*parm[k]) && dot_atof(parm[k]) == 0.0) {
-	    continue;
-	}
-	if (k == 0) {
-	    strcpy(term, parm[k]);
-	} else {
-	    neg = (*parm[k] == '-');
-	    sprintf(term, "%s%s*%s(t-%d)", (neg)? "-" : 
-		    (started)? "+" : "",
-		    parm[k] + neg, vname, k);
-	}
-	if (strlen(label) + strlen(term) >= MAXLABEL - 4) {
-	    if (strlen(label) < MAXLABEL - 4) {
-		strcat(label, "...");
-	    }
-	    break;
-	} else {
-	    strcat(label, term);
-	}
-	started = 1;
-    }
-	
-    return label;
-}
-
-int simulate (const char *cmd, double **Z, DATAINFO *pdinfo)
-{
-    int f, i, t, t1, t2, tstart, m, nv = 0, pv;
-    char varname[VNAMELEN], parm[16], cmdcpy[MAXLEN];
-    char *isconst = NULL, **toks = NULL;
-    double xx, yy, *a = NULL;
-    int vtok = 0, err = 0;
-
-    *gretl_errmsg = '\0';
-
-    *cmdcpy = 0;
-    strncat(cmdcpy, cmd, MAXLEN - 1);
-
-    close_minus(cmdcpy);
-
-    f = count_fields(cmdcpy);
-    m = f - 2; /* default: allow for command word varname */
-
-    a = malloc(m * sizeof *a);
-    isconst = malloc(m * sizeof *isconst);
-    toks = malloc((f - 1) * sizeof *toks);
-
-    if (a == NULL || isconst == NULL || toks == NULL) {
-	err = E_ALLOC;
-	goto sim_bailout;
-    }
-
-    for (i=0; i<m; i++) {
-	isconst[i] = 1;
-    }
-
-    strtok(cmdcpy, " "); /* discard the "sim" command word */
-    for (i=0; i<f-1; i++) {
-	toks[i] = strtok(NULL, " ");
-    }
-
-    /* allow for implicit starting and ending dates */
-    if (isalpha(*toks[0])) {
-	t1 = pdinfo->t1;
-	t2 = pdinfo->t2;
-    } else {
-	m -= 2;
-	vtok = 2;
-	/* try getting valid obs from stobs and endobs */
-	t1 = dateton(toks[0], pdinfo);
-	t2 = dateton(toks[1], pdinfo);
-	if (*gretl_errmsg || t1 < 0 || t1 > t2 || t2 >= pdinfo->n) {
-
-	    if (t1 < 0 || t2 >= pdinfo->n) {
-		strcpy(gretl_errmsg, _("Observation number out of bounds"));
-	    } else if (t1 > t2 ) {
-		strcpy(gretl_errmsg, _("Invalid null sample"));
-	    }
-
-	    err = 1;
-	    goto sim_bailout;
-	}
-    }
-
-    /* name of var to simulate */
-    *varname = 0;
-    strncat(varname, toks[vtok], 8);
-    nv = varindex(pdinfo, varname);
-
-    if (nv > 0 && nv < pdinfo->v && pdinfo->vector[nv] == 0) {
-	sprintf(gretl_errmsg, _("variable %s is a scalar"), 
-		pdinfo->varname[nv]);
-	err = 1;
-	goto sim_bailout;
-    }
-		
-    if (nv == 0 || nv >= pdinfo->v) {
-	sprintf(gretl_errmsg, (nv)? _("For 'sim', the variable must already "
-				      "exist") :
-		_("You can't use the constant for this purpose"));
-	err = 1;
-	goto sim_bailout;
-    }
-
-    /* get the parameter terms */
-    for (i=0; i<m; i++) {
-	int neg = 0;
-	const char *p = parm;
-
-	*parm = '\0';
-	strncat(parm, toks[i + vtok + 1], sizeof parm - 1);
-
-	if (*parm == '-') {
-	    neg = 1;
-	    p++;
-	}
-
-	if (isalpha((unsigned char) *p)) {
-	    pv = varindex(pdinfo, p);
-	    if (pv == 0 || pv >= pdinfo->v) {
-		sprintf(gretl_errmsg, _("Bad varname '%s' in sim"), p);
-		err = 1;
-		goto sim_bailout;
-	    } else {
-		isconst[i] = !pdinfo->vector[pv];
-		a[i] = (isconst[i])? Z[pv][0] : (double) pv;
-	    }
-	} else {
-	    a[i] = dot_atof(p);
-	}
-
-	if (neg) {
-	    a[i] = -a[i];
-	}
-    }
-
-    tstart = t1;
-    if (tstart < m - 1) tstart = m - 1;
-
-    for (t=tstart; t<=t2; t++) {
-	xx = 0.0;
-	for (i=0; i<m; i++) {
-	    if (isconst[i]) {
-		if (i == 0) {
-		    xx += a[i];
-		} else {
-		    xx += a[i] * Z[nv][t-i];
-		}
-	    } else {
-		int neg = 0;
-
-		pv = (int) a[i];
-		if (pv < 0) {
-		    neg = 1;
-		    pv = -pv;
-		}
-		yy = Z[pv][t];
-		if (na(yy)) {
-		    xx = NADBL;
-		    break;
-		}
-		if (neg) {
-		    yy = -yy;
-		}
-		if (i == 0) {
-		    xx += yy;
-		} else {
-		    xx += yy * Z[nv][t-i];
-		}
-	    }
-	}
-	Z[nv][t] = xx;
-    }
-
- sim_bailout:
-
-    if (!err && nv > 0) {
-	sprintf(gretl_msg, "%s %s %s (ID %d)", 
-		_("Replaced"), _("vector"), pdinfo->varname[nv], nv);
-	make_sim_label(VARLABEL(pdinfo, nv), pdinfo->varname[nv],
-		       toks + vtok + 1, m);
-    }
-
-    free(a);
-    free(isconst);
-    free(toks);
-
-    return err;
-}
-
-/* .......................................................... */
-
 int gretl_multiply (char *s, int *list, char *sfx, double ***pZ,
 		    DATAINFO *pdinfo)
 {
