@@ -27,6 +27,10 @@
 #include <time.h>
 #include <errno.h>
 
+#ifdef USE_GTK2
+# include <glib.h>
+#endif
+
 #define QUOTE                  '\''
 #define CSVSTRLEN               72
 
@@ -36,7 +40,6 @@ static int writelbl (const char *lblfile, const int *list,
 		     const DATAINFO *pdinfo);
 static int writehdr (const char *hdrfile, const int *list, 
 		     const DATAINFO *pdinfo, int opt);
-static double obs_float (const DATAINFO *pdinfo, int end);
 static int xmlfile (const char *fname);
 static int csv_time_series_check (DATAINFO *pdinfo, PRN *prn);
 
@@ -142,11 +145,13 @@ static int readdata (FILE *fp, const DATAINFO *pdinfo, double **Z,
     int i, t, n = pdinfo->n;
     char c, marker[OBSLEN];
     int err = 0;
-    float x;
 
     gretl_errmsg[0] = '\0';
 
-    if (binary == 1) { /* single-precision binary data */
+    if (binary == 1) { 
+	/* single-precision binary data */
+	float x;
+
 	for (i=1; i<pdinfo->v; i++) {
 	    for (t=0; t<n; t++) {
 		if (!fread(&x, sizeof x, 1, fp)) {
@@ -154,15 +159,29 @@ static int readdata (FILE *fp, const DATAINFO *pdinfo, double **Z,
 			    "var %d"), i);
 		    return 1;
 		}
-		Z[i][t] = (double) x;
+		if (x == -999.0) {
+		    Z[i][t] = NADBL;
+		} else {
+		    Z[i][t] = (double) x;
+		}
 	    }
 	}
-    } else if (binary == 2) { /* double-precision binary data */
+    } else if (binary == 2) { 
+	/* double-precision binary data */
+	double x;
+
 	for (i=1; i<pdinfo->v; i++) {
-	    if (!fread(Z[i], sizeof(double), n, fp)) {
-		sprintf(gretl_errmsg, 
-			_("WARNING: binary data read error at var %d"), i);
-		return 1;
+	    for (t=0; t<n; t++) {
+		if (!fread(&x, sizeof x, 1, fp)) {
+		    sprintf(gretl_errmsg, 
+			    _("WARNING: binary data read error at var %d"), i);
+		    return 1;
+		}
+		if (x == -999.0) {
+		    Z[i][t] = NADBL;
+		} else {
+		    Z[i][t] = x;
+		}
 	    }
 	}
     } else if (old_byvar) {
@@ -327,11 +346,13 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
 
 /**
  * check_varname:
- * @varname: putative name for variable.
+ * @varname: putative name for variable (or object).
  * 
- * Check a variable name for legality.
+ * Check a variable/object name for legality: the name
+ * must start with a letter, and be composed of letters,
+ * numbers or the underscore character, and nothing else.
  * 
- * Returns: 0 if name is OK, 1 if not.
+ * Returns: 0 if name is OK, non-zero if not.
  */
 
 int check_varname (const char *varname)
@@ -1080,6 +1101,17 @@ format_from_opt_or_name (gretlopt opt, const char *fname)
     return fmt;
 }
 
+static void date_maj_min (const char *s, int *maj, int *min)
+{
+    *maj = atoi(s);
+    s = strchr(s, ':');
+    if (s != NULL && strlen(s) > 1) {
+	*min = atoi(s + 1);
+    } else {
+	*min = 1;
+    }
+}
+
 /**
  * write_data:
  * @fname: name of file to write.
@@ -1138,7 +1170,7 @@ int write_data (const char *fname, const int *list,
     strcpy(datfile, fname);
 
     if (fmt == GRETL_DATA_R && pdinfo->structure == TIME_SERIES) {
-	fmt = GRETL_DATA_R_ALT;
+	fmt = GRETL_DATA_R_TS;
     }
 
     if (fmt == GRETL_DATA_CSV && (opt & OPT_X)) {
@@ -1147,7 +1179,7 @@ int write_data (const char *fname, const int *list,
     }
 
     /* write header and label files if not exporting to other formats */
-    if (fmt != GRETL_DATA_R && fmt != GRETL_DATA_R_ALT && 
+    if (fmt != GRETL_DATA_R && fmt != GRETL_DATA_R_TS && 
 	fmt != GRETL_DATA_CSV && fmt != GRETL_DATA_OCTAVE &&
 	fmt != GRETL_DATA_DAT) {
 	if (!has_suffix(datfile, ".gz")) {
@@ -1176,20 +1208,30 @@ int write_data (const char *fname, const int *list,
 
     if (fp == NULL) return 1;
 
-    if (fmt == GRETL_DATA_FLOAT) { /* single-precision binary */
+    if (fmt == GRETL_DATA_FLOAT) { 
+	/* single-precision binary */
 	float x;
 
 	for (i=1; i<=l0; i++) {
 	    v = list[i];
-	    x = (float) Z[v][0];
+	    if (na(Z[v][0])) {
+		x = -999.0;
+	    } else {
+		x = (float) Z[v][0];
+	    }
 	    for (t=0; t<n; t++) {
 		if (pdinfo->vector[v]) {
-		    x = (float) Z[v][t];
+		    if (na(Z[v][0])) {
+			x = -999.0;
+		    } else {	
+			x = (float) Z[v][t];
+		    }
 		}
 		fwrite(&x, sizeof x, 1, fp);
 	    }
 	}
-    } else if (fmt == GRETL_DATA_DOUBLE) { /* double-precision binary */
+    } else if (fmt == GRETL_DATA_DOUBLE) { 
+	/* double-precision binary */
 	for (i=1; i<=l0; i++) {
 	    v = list[i];
 	    if (pdinfo->vector[v]) {
@@ -1223,7 +1265,8 @@ int write_data (const char *fname, const int *list,
     if (fmt == GRETL_DATA_CSV && pdinfo->decpoint == ',') ;
     else gretl_push_c_numeric_locale();
 
-    if (fmt == GRETL_DATA_TRAD) { /* plain ASCII */
+    if (fmt == GRETL_DATA_TRAD) { 
+	/* plain ASCII */
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    if (pdinfo->markers && pdinfo->S != NULL) {
 		fprintf(fp, "%s ", pdinfo->S[t]);
@@ -1299,8 +1342,7 @@ int write_data (const char *fname, const int *list,
 	    }
 	}
 	fputc('\n', fp);
-    } else if (fmt == GRETL_DATA_R_ALT && pdinfo->structure == TIME_SERIES) {
-	/* October, 2003: attempt at improved R time-series structure */
+    } else if (fmt == GRETL_DATA_R_TS) {
 	char *p, datestr[OBSLEN];
 	int subper = 1;
 
@@ -1344,34 +1386,6 @@ int write_data (const char *fname, const int *list,
 		fputs(")\n", fp);
 	    }
 	}
-    } else if (fmt == GRETL_DATA_R_ALT) { 
-	/* export GNU R (structure) */
-	for (i=1; i<=l0; i++) {
-	    v = list[i];
-	    fprintf(fp, "\"%s\" <-\n", pdinfo->varname[v]);
-	    fprintf(fp, "structure(c(");
-	    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-		xx = (pdinfo->vector[v])? Z[v][t] : Z[v][0];
-		if (na(xx)) {
-		    fprintf(fp, "NA");
-		} else {
-		    fprintf(fp, "%g", xx);
-		}
-		if (t < pdinfo->t2) {
-		    fprintf(fp, ", "); 
-		}
-		if (t > pdinfo->t1 && (t - pdinfo->t1) % 8 == 0 && t < pdinfo->t2) {
-		    fputc('\n', fp);
-		}
-	    }
-	    fputc(')', fp);
-	    if (pdinfo->structure == TIME_SERIES) { 
-		fprintf(fp, ",\n.Tsp = c(%f, %f, %d), class = \"ts\"",
-			obs_float(pdinfo, 0), obs_float(pdinfo, 1), 
-			pdinfo->pd);
-	    }
-	    fprintf(fp, ")\n");
-	}
     } else if (fmt == GRETL_DATA_OCTAVE) { 
 	/* GNU Octave: write out data as a matrix */
 	fprintf(fp, "# name: X\n# type: matrix\n# rows: %d\n# columns: %d\n", 
@@ -1398,12 +1412,12 @@ int write_data (const char *fname, const int *list,
 	    fprintf(fp, ">%s ", pdinfo->varname[list[i]]);
 	    if (pdinfo->structure == TIME_SERIES &&
 		(pd == 1 || pd == 4 || pd == 12)) {
-		double ts = obs_float(pdinfo, 0);
-		double te = obs_float(pdinfo, 1);
-			   
-		fprintf(fp, "%d %d %d %d %d\n",
-			(int) (floor(ts)), (int) (ts - floor(ts)) * pd + 1,
-			(int) (floor(te)), (int) (te - floor(te)) * pd + 1, pd);
+		int maj, min;
+
+		date_maj_min(pdinfo->stobs, &maj, &min);
+		fprintf(fp, "%d %d ", maj, min);
+		date_maj_min(pdinfo->endobs, &maj, &min);
+		fprintf(fp, "%d %d %d", maj, min, pd);
 	    } else {
 		fprintf(fp, "%d 1 %d 1 1", pdinfo->t1, pdinfo->t2);
 	    }
@@ -1472,9 +1486,11 @@ static void pd_string (char *str, const DATAINFO *pdinfo)
 	case 52:
 	    strcpy(str, _("weekly")); break;
 	case 5:
-	    strcpy(str, _("daily")); break;
+	case 6:
 	case 7:
 	    strcpy(str, _("daily")); break;
+	case 10:
+	    strcpy(str, _("decennial")); break;
 	default:
 	    strcpy(str, _("unknown")); break;
 	}
@@ -1534,31 +1550,6 @@ int data_report (const DATAINFO *pdinfo, PATHS *ppaths, PRN *prn)
     return 0;
 }
 
-static double obs_float (const DATAINFO *pdinfo, int end)
-{
-    double xx, xx2 = 0.0;
-    int i, x1, x2 = 0;
-
-    if (end) {
-	xx = obs_str_to_double(pdinfo->endobs);
-	if ((i = haschar(':', pdinfo->endobs)) > 0) {
-	   x2 = atoi(pdinfo->endobs + i + 1) - 1;
-	}
-    } else {
-	xx = obs_str_to_double(pdinfo->stobs);
-	if ((i = haschar(':', pdinfo->stobs)) > 0) {
-	   x2 = atoi(pdinfo->stobs + i + 1) - 1;
-	}
-    }
-
-    x1 = (int) xx;
-    if (x2 > 0) {
-	xx2 = (double) x2 / pdinfo->pd;
-    }
-    
-    return (double) x1 + xx2;
-}
-
 /* read data "labels" from file */
 
 static int readlbl (const char *lblfile, DATAINFO *pdinfo)
@@ -1942,7 +1933,7 @@ int open_nulldata (double ***pZ, DATAINFO *pdinfo,
     }
 
     /* print out basic info */
-    pprintf(prn, I_("periodicity: %d, maxobs: %d,\n"
+    pprintf(prn, M_("periodicity: %d, maxobs: %d,\n"
 	   "observations range: %s-%s\n"), pdinfo->pd, pdinfo->n,
 	   pdinfo->stobs, pdinfo->endobs);
 
@@ -2102,15 +2093,18 @@ static int compress_daily (DATAINFO *pdinfo, int pd)
 static int transform_daily_dates (DATAINFO *pdinfo, int dorder)
 {
     int t, yr, mon, day;
+    char sep1[2], sep2[2];
     int sret, err = 0;
 
     for (t=0; t<pdinfo->n && !err; t++) {
-	if (dorder == DDMMYYYY) {
-	    sret = sscanf(pdinfo->S[t], "%d/%d/%d", &day, &mon, &yr);
+	if (dorder == YYYYMMDD) {
+	    sret = sscanf(pdinfo->S[t], "%d%1[/-]%d%1[/-]%d", &yr, sep1, &mon, sep2, &day);
+	} else if (dorder == DDMMYYYY) {
+	    sret = sscanf(pdinfo->S[t], "%d%1[/-]%d%1[/-]%d", &day, sep1, &mon, sep2, &yr);
 	} else {
-	    sret = sscanf(pdinfo->S[t], "%d/%d/%d", &mon, &day, &yr);
+	    sret = sscanf(pdinfo->S[t], "%d%1[/-]%d%1[/-]%d", &mon, sep1, &day, sep2, &yr);
 	}
-	if (sret == 3) {
+	if (sret == 5) {
 	    sprintf(pdinfo->S[t], "%02d/%02d/%02d", yr, mon, day);
 	} else {
 	    err = 1;
@@ -2205,11 +2199,12 @@ static int
 csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 {
     int d1[3], d2[3];
+    char sep1[2], sep2[2];
     char *lbl1 = pdinfo->S[0];
     char *lbl2 = pdinfo->S[pdinfo->n - 1];
 
-    if (sscanf(lbl1, "%d/%d/%d", &d1[0], &d1[1], &d1[2]) == 3 &&
-	sscanf(lbl2, "%d/%d/%d", &d2[0], &d2[1], &d2[2]) == 3) {
+    if (sscanf(lbl1, "%d%1[/-]%d%1[/-]%d", &d1[0], sep1, &d1[1], sep2, &d1[2]) == 5 &&
+	sscanf(lbl2, "%d%1[/-]%d%1[/-]%d", &d2[0], sep1, &d2[1], sep2, &d2[2]) == 5) {
 	int yr1, mon1, day1;
 	int yr2, mon2, day2;
 	int dorder = get_date_order(d1[0], d2[0]);
@@ -2247,7 +2242,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 	    day1 > 0 && day1 < 32 &&
 	    day2 > 0 && day2 < 32) {
 	    /* looks promising for calendar dates */
-	    if (dorder != YYYYMMDD) {
+	    if (dorder != YYYYMMDD || *sep1 != '/' || *sep2 != '/') {
 		if (transform_daily_dates(pdinfo, dorder)) {
 		    return -1;
 		}
@@ -3053,7 +3048,18 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 
     csvinfo->delim = delim;
 
+#if USE_GTK2
+    if (!g_utf8_validate(fname, -1, NULL)) {
+	gchar *trfname = g_locale_to_utf8(fname, -1, NULL, NULL, NULL);
+
+	pprintf(prn, "%s %s...\n", M_("parsing"), trfname);
+	g_free(trfname);
+    } else {
+	pprintf(prn, "%s %s...\n", M_("parsing"), fname);
+    }
+#else
     pprintf(prn, "%s %s...\n", M_("parsing"), fname);
+#endif
 
     /* get line length, also check for binary data, etc. */
     maxlen = get_max_line_length(fp, delim, &gotdelim, &gottab, &trail, prn);    
@@ -3198,8 +3204,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 		goto csv_bailout;
 	    } else {
 		csvinfo->varname[nv][0] = 0;
-		/* was VNAMELEN below */
-		strncat(csvinfo->varname[nv], csvstr, USER_VLEN - 1);
+		strncat(csvinfo->varname[nv], csvstr, VNAMELEN - 1);
 		if (starts_number(*csvstr)) {
 		    numcount++;
 		} else {
@@ -3620,7 +3625,7 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
 	int j;
 
 	if (*s == '#') {
-	    if (sscanf(line, "# name: %8s", name) == 1) {
+	    if (sscanf(line, "# name: %15s", name) == 1) {
 		;
 	    } else if (sscanf(line, "# rows: %d", &brows) == 1) {
 		t = 0;
@@ -3720,7 +3725,7 @@ static char *unspace (char *s)
  * import_box:
  * @pZ: pointer to data set.
  * @ppdinfo: pointer to data information struct.
- * @fname: name of CSV file.
+ * @fname: name of BOX1 file.
  * @prn: gretl printing struct.
  * 
  * Open a BOX1 data file (as produced by the US Census Bureau's
@@ -3728,7 +3733,6 @@ static char *unspace (char *s)
  * the current work space.
  * 
  * Returns: 0 on successful completion, non-zero otherwise.
- *
  */
 
 int import_box (double ***pZ, DATAINFO **ppdinfo, 
@@ -3940,7 +3944,9 @@ int import_box (double ***pZ, DATAINFO **ppdinfo,
 
     if (*pZ == NULL) {
 	*pZ = boxZ;
-	if (*ppdinfo != NULL) free(*ppdinfo);
+	if (*ppdinfo != NULL) {
+	    free(*ppdinfo);
+	}
 	*ppdinfo = boxinfo;
     }
 
@@ -3951,6 +3957,68 @@ int import_box (double ***pZ, DATAINFO **ppdinfo,
     return err;
 }
 
+/**
+ * import_other:
+ * @pZ: pointer to data set.
+ * @ppdinfo: pointer to data information struct.
+ * @ftype: type of data file.
+ * @fname: name of file.
+ * @prn: gretl printing struct.
+ * 
+ * Open a ...
+ * 
+ * Returns: 0 on successful completion, non-zero otherwise.
+ */
+
+int import_other (double ***pZ, DATAINFO **ppdinfo, 
+		  int ftype, const char *fname, PRN *prn)
+{
+    void *handle;
+    FILE *fp;
+    int (*sheet_get_data)(const char*, double ***, DATAINFO *, PRN *);
+    int err = 0;
+
+    check_for_console(prn);
+
+    if (ftype == GRETL_GNUMERIC || ftype == GRETL_EXCEL) {
+	pputs(prn, "At present Gnumeric and Excel files can only be opened "
+	      "via the GUI menus.\n");
+	return E_DATA;
+    }
+
+    fp = gretl_fopen(fname, "r");
+    if (fp == NULL) {
+	pprintf(prn, M_("Couldn't open %s\n"), fname);
+	return E_FOPEN;
+    }
+
+    fclose(fp);
+
+    /* FIXME Gnumeric and XLS */
+    if (ftype == GRETL_GNUMERIC) {
+	sheet_get_data = get_plugin_function("wbook_get_data", &handle);
+    } else if (ftype == GRETL_EXCEL) {
+	sheet_get_data = get_plugin_function("excel_get_data", &handle);
+    } else if (ftype == GRETL_WF1) {
+	sheet_get_data = get_plugin_function("wf1_get_data", &handle);
+    } else if (ftype == GRETL_DTA) {
+	sheet_get_data = get_plugin_function("dta_get_data", &handle);
+    } else {
+	pprintf(prn, M_("Unrecognized data type"));
+	pputc(prn, '\n');
+	return E_DATA;
+    }
+
+    if (sheet_get_data == NULL) {
+        return 1;
+    }
+
+    err = (*sheet_get_data)(fname, pZ, *ppdinfo, prn);
+    close_plugin(handle);
+
+    return err;
+}
+
 static int xmlfile (const char *fname)
 {
     gzFile fz;
@@ -3975,10 +4043,9 @@ static int xmlfile (const char *fname)
  * @prn: gretl printing struct.
  * 
  * Attempt to determine the type of a file to be opened in gretl:
- * data file (native, CSV or BOX), or command script.
+ * data file (of various formats), or command script.
  * 
  * Returns: integer code indicating the type of file.
- *
  */
 
 GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn)
@@ -4139,7 +4206,7 @@ int transpose_data (double ***pZ, DATAINFO *pdinfo)
     for (t=0; t<pdinfo->n; t++) {
 	if (pdinfo->S != NULL && pdinfo->S[t][0] != '\0') {
 	    tinfo->varname[t+1][0] = '\0';
-	    strncat(tinfo->varname[t+1], pdinfo->S[t], 8);
+	    strncat(tinfo->varname[t+1], pdinfo->S[t], VNAMELEN - 1);
 	} else {
 	    sprintf(tinfo->varname[t+1], "v%d", t+1);
 	}
