@@ -29,7 +29,7 @@ typedef enum {
     GRETL_DATA_OCTAVE,    /* data in Gnu Octave format */
     GRETL_DATA_CSV,       /* data in Comma Separated Values format */
     GRETL_DATA_R,         /* data in Gnu R format */
-    GRETL_DATA_R_ALT,     /* data in alternate Gnu R format */
+    GRETL_DATA_R_TS,      /* data in Gnu R format (time series) */
     GRETL_DATA_GZIPPED,   /* gzipped data */
     GRETL_DATA_TRAD,      /* traditional (ESL-style) data */
     GRETL_DATA_DAT,       /* data in PcGive format */
@@ -69,6 +69,8 @@ typedef enum {
     VARNAME_BADCHAR       /* illegal character in second or subsequent place */
 } GretlVarnameError;
 
+#define WORKSHEET_IMPORT(f) (f == GRETL_GNUMERIC || f == GRETL_EXCEL || \
+                             f == GRETL_WF1 || f == GRETL_DTA)
 
 #define free_datainfo(p) do { if (p != NULL) { clear_datainfo(p, 0); free(p); } \
                             } while (0);
@@ -122,6 +124,9 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
 int import_box (double ***pZ, DATAINFO **ppdinfo, 
 		const char *fname, PRN *prn);
 
+int import_other (double ***pZ, DATAINFO **ppdinfo, 
+		  int ftype, const char *fname, PRN *prn);
+
 int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname);
 
 GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn);
