@@ -110,6 +110,8 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
 	pdinfo->submask = NULL;
     }
 
+    pdinfo->submode = 0;
+
     /* if this is not a sub-sample datainfo, free varnames, labels, etc. */
     if (code == CLEAR_FULL) {
 	if (pdinfo->varname != NULL) {
@@ -382,6 +384,8 @@ DATAINFO *datainfo_new (void)
     dinfo->descrip = NULL;
     dinfo->vector = NULL;
     dinfo->submask = NULL;
+    dinfo->submode = 0;
+
     dinfo->data = NULL;
 
     dinfo->structure = CROSS_SECTION;
@@ -529,6 +533,7 @@ int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample)
     pdinfo->descrip = NULL;
     pdinfo->data = NULL;
     pdinfo->submask = NULL;
+    pdinfo->submode = 0;
     
     return 0;
 }
@@ -1016,22 +1021,24 @@ dataset_add_allocated_series (double *x, double ***pZ, DATAINFO *pdinfo)
 }
 
 /**
- * dataset_add_scalar:
+ * dataset_add_scalars:
+ * @n: number of scalars to add.
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
  *
- * Allocates space for a new scalar member of the dataset.
+ * Allocates space for @n new scalar members of the dataset.
+ * The added variables are initialized to zero.
  *
  * Returns: 0 on success, %E_ALLOC on error.
  */
 
-int dataset_add_scalar (double ***pZ, DATAINFO *pdinfo)
+int dataset_add_scalars (int n, double ***pZ, DATAINFO *pdinfo)
 {
     double **newZ;
-    int n = pdinfo->n, v = pdinfo->v; 
-    int err = 0;
+    int v = pdinfo->v;
+    int i, err = 0;
 
-    newZ = realloc(*pZ, (v + 1) * sizeof *newZ);  
+    newZ = realloc(*pZ, (v + n) * sizeof *newZ);  
 
     if (newZ == NULL) {
 	err = E_ALLOC;
@@ -1040,23 +1047,48 @@ int dataset_add_scalar (double ***pZ, DATAINFO *pdinfo)
     }
 
     if (!err) {
-	newZ[v] = malloc(n * sizeof **newZ);
-	if (newZ[v] == NULL) {
-	    err = E_ALLOC;
+	for (i=0; i<n; i++) {
+	    newZ[v+i] = NULL;
+	}
+	for (i=0; i<n; i++) {
+	    newZ[v+i] = malloc(sizeof **newZ);
+	    if (newZ[v+i] == NULL) {
+		err = E_ALLOC;
+		break;
+	    }
+	    newZ[v+i][0] = 0.0;
 	}
     }
 
     if (!err) {
-	err = dataset_expand_varinfo(1, pdinfo);
+	err = dataset_expand_varinfo(n, pdinfo);
     }
 
     if (!err) {
-	pdinfo->vector[v] = 0;
+	for (i=0; i<n; i++) {
+	    pdinfo->vector[v+i] = 0;
+	}
     }
 
     return err;
 }
 
+/**
+ * dataset_add_scalar:
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
+ *
+ * Allocates space for a new scalar member of the dataset.
+ * The added variable is initialized to zero.
+ *
+ * Returns: 0 on success, %E_ALLOC on error.
+ */
+
+int dataset_add_scalar (double ***pZ, DATAINFO *pdinfo)
+{
+    return dataset_add_scalars(1, pZ, pdinfo);
+}
+
 /**
  * dataset_scalar_to_vector:
  * @v: index number of variable to process.
@@ -1110,7 +1142,7 @@ int dataset_scalar_to_vector (int v, double ***pZ, DATAINFO *pdinfo)
 int dataset_add_scalar_as (const char *numstr, const char *newname,
 			   double ***pZ, DATAINFO *pdinfo)
 {
-    int err = 0;
+    int vnew, err = 0;
 
     if (pdinfo->varinfo == NULL) {
 	strcpy(gretl_errmsg, _("Please open a data file first"));
@@ -1118,10 +1150,10 @@ int dataset_add_scalar_as (const char *numstr, const char *newname,
     }
 
     err = dataset_add_scalar(pZ, pdinfo);
-    if (!err) {
-	int vnew = pdinfo->v - 1;
 
-	(*pZ)[vnew][0] = atof(numstr);
+    if (!err) {
+	vnew = pdinfo->v - 1;
+	(*pZ)[vnew][0] = dot_atof(numstr);
 	strcpy(pdinfo->varname[vnew], newname);
 	STACK_LEVEL(pdinfo, vnew) += 1;
     }
@@ -1176,31 +1208,45 @@ int dataset_copy_variable_as (int v, const char *newname,
     return err;
 }
 
+enum {
+    DROP_NORMAL,
+    DROP_SPECIAL
+};
+
+/* DROP_SPECIAL is used when deleting variables from the "full" shadow
+   of a sub-sampled dataset: in that case we don't mess with the
+   pointer elements of the DATAINFO struct, because these are shared
+   between the full and sub-sampled datasets.
+*/
+
 static int 
-shrink_dataset_to_size (double ***pZ, DATAINFO *pdinfo, int nv)
+shrink_dataset_to_size (double ***pZ, DATAINFO *pdinfo, int nv, int drop)
 {
-    char **varname;
-    char *vector;
-    VARINFO **varinfo;
     double **newZ;
+
+    if (drop == DROP_NORMAL) {
+	char **varname;
+	char *vector;
+	VARINFO **varinfo;
     
-    varname = realloc(pdinfo->varname, nv * sizeof *varname);
-    if (varname == NULL) {
-	return E_ALLOC;
-    }
-    pdinfo->varname = varname;
+	varname = realloc(pdinfo->varname, nv * sizeof *varname);
+	if (varname == NULL) {
+	    return E_ALLOC;
+	}
+	pdinfo->varname = varname;
 
-    vector = realloc(pdinfo->vector, nv * sizeof *vector);
-    if (vector == NULL) {
-	return E_ALLOC;
-    }
-    pdinfo->vector = vector;
+	vector = realloc(pdinfo->vector, nv * sizeof *vector);
+	if (vector == NULL) {
+	    return E_ALLOC;
+	}
+	pdinfo->vector = vector;
 
-    varinfo = realloc(pdinfo->varinfo, nv * sizeof *varinfo);
-    if (varinfo == NULL) {
-	return E_ALLOC;
+	varinfo = realloc(pdinfo->varinfo, nv * sizeof *varinfo);
+	if (varinfo == NULL) {
+	    return E_ALLOC;
+	}
+	pdinfo->varinfo = varinfo;
     }
-    pdinfo->varinfo = varinfo;
 
     newZ = realloc(*pZ, nv * sizeof *newZ); 
     if (newZ == NULL) {
@@ -1213,7 +1259,95 @@ shrink_dataset_to_size (double ***pZ, DATAINFO *pdinfo, int nv)
     return 0;
 }
 
-#undef DROPDBG
+#define DROPDBG 0
+
+static int real_dataset_drop_listed_vars (const int *list, double ***pZ, 
+					  DATAINFO *pdinfo, int *renumber,
+					  int drop)
+{
+    double **Z = *pZ;
+    int oldv = pdinfo->v, vmax = pdinfo->v;
+    int i, v, ndel = 0; 
+
+    if (list == NULL) {
+	/* no-op */
+	return 0;
+    }
+
+    if (renumber != NULL) {
+	*renumber = 0;
+    }
+
+#if DROPDBG
+    fprintf(stderr, "dataset_drop_listed_variables: dropping %d vars:\n",
+	    list[0]);
+    if (drop == DROP_NORMAL) {
+	for (i=1; i<=list[0]; i++) {
+	    fprintf(stderr, " %d: %s\n", list[i], pdinfo->varname[list[i]]);
+	}
+    }
+#endif
+
+    /* free and set to NULL all the vars to be deleted */
+
+    for (i=1; i<=list[0]; i++) {
+	v = list[i];
+	if (v > 0 && v < oldv) {
+	    free(Z[v]);
+	    Z[v] = NULL;
+	    if (drop == DROP_NORMAL) {
+		free(pdinfo->varname[v]);
+		if (pdinfo->varinfo[v] != NULL) {
+		    free(pdinfo->varinfo[v]);
+		}
+	    }
+	    ndel++;
+	}
+    }
+
+    /* rearrange pointers if necessary */
+
+    for (v=1; v<vmax; v++) {
+	if (Z[v] == NULL) {
+	    int gap = 1;
+
+	    for (i=v+1; i<vmax; i++) {
+		if (Z[i] == NULL) {
+		    gap++;
+		} else {
+		    break;
+		}
+	    }
+
+	    if (i < vmax) {
+		vmax -= gap;
+		for (i=v; i<vmax; i++) {
+		    if (drop == DROP_NORMAL) {
+			if (renumber != NULL && 
+			    !is_hidden_variable(i + gap, pdinfo)) {
+			    *renumber = 1;
+			}
+			pdinfo->varname[i] = pdinfo->varname[i + gap];
+			pdinfo->varinfo[i] = pdinfo->varinfo[i + gap];
+			pdinfo->vector[i] = pdinfo->vector[i + gap];
+		    }
+		    Z[i] = Z[i + gap];
+		}		    
+	    } else {
+		/* deleting all subsequent vars */
+		break;
+	    }
+	}
+    }
+
+    return shrink_dataset_to_size(pZ, pdinfo, oldv - ndel, drop);
+}
+
+/* in subsample.c */
+
+extern double ***fetch_full_Z (void);
+extern void reset_full_Z (double ***pZ);
+extern DATAINFO *fetch_full_datainfo (void);
 
 /**
  * dataset_drop_listed_variables:
@@ -1235,65 +1369,39 @@ shrink_dataset_to_size (double ***pZ, DATAINFO *pdinfo, int nv)
 int dataset_drop_listed_variables (const int *list, double ***pZ, 
 				   DATAINFO *pdinfo, int *renumber)
 {
-    int oldv = pdinfo->v, vmax = pdinfo->v;
-    int i, v, ndel = 0; 
+    int err;
 
-    if (renumber != NULL) {
-	*renumber = 0;
+    err = real_dataset_drop_listed_vars(list, pZ, pdinfo, renumber,
+					DROP_NORMAL);
+
+    if (!err && complex_subsampled()) {
+	double ***fZ = fetch_full_Z();
+	DATAINFO *fdinfo = fetch_full_datainfo();
+
+	err = real_dataset_drop_listed_vars(list, fZ, fdinfo, NULL,
+					    DROP_SPECIAL);
+	reset_full_Z(fZ);
     }
 
-#if DROPDBG
-    printlist(list, "vars to be deleted");
-#endif
+    return err;
+}
 
-    /* free and set to NULL all the vars to be deleted */
+/**
+ * dataset_drop_variable:
+ * @v: ID number of variable to drop.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
+ *
+ * Deletes variable @v from the dataset.  
+ *
+ * Returns: 0 on success, %E_ALLOC on error.
+ */
 
-    for (i=1; i<=list[0]; i++) {
-	v = list[i];
-	if (v > 0 && v < oldv) {
-	    free((*pZ)[v]);
-	    (*pZ)[v] = NULL;
-	    free(pdinfo->varname[v]);
-	    if (pdinfo->varinfo[v] != NULL) {
-		free(pdinfo->varinfo[v]);
-	    }
-	    ndel++;
-	}
-    }
+int dataset_drop_variable (int v, double ***pZ, DATAINFO *pdinfo) 
+{
+    int list[2] = {1, v};
 
-    /* rearrange pointers if necessary */
-
-    for (v=1; v<vmax; v++) {
-	if ((*pZ)[v] == NULL) {
-	    int gap = 1;
-
-	    for (i=v+1; i<vmax; i++) {
-		if ((*pZ)[i] == NULL) {
-		    gap++;
-		} else {
-		    break;
-		}
-	    }
-
-	    if (i < vmax) {
-		vmax -= gap;
-		for (i=v; i<vmax; i++) {
-		    if (renumber != NULL && !is_hidden_variable(i + gap, pdinfo)) {
-			*renumber = 1;
-		    }
-		    pdinfo->varname[i] = pdinfo->varname[i + gap];
-		    pdinfo->varinfo[i] = pdinfo->varinfo[i + gap];
-		    pdinfo->vector[i] = pdinfo->vector[i + gap];
-		    (*pZ)[i] = (*pZ)[i + gap];
-		}		    
-	    } else {
-		/* deleting all subsequent vars */
-		break;
-	    }
-	}
-    }
-
-    return shrink_dataset_to_size(pZ, pdinfo, oldv - ndel);
+    return dataset_drop_listed_variables(list, pZ, pdinfo, NULL);
 }
 
 /**
@@ -1377,7 +1485,7 @@ int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo)
 	}
     }
 
-    return shrink_dataset_to_size(pZ, pdinfo, v - delvars);
+    return shrink_dataset_to_size(pZ, pdinfo, v - delvars, DROP_NORMAL);
 }
 
 static void make_stack_label (char *label, char *s)
@@ -1505,6 +1613,7 @@ static int missing_tail (const double *x, int n)
  * @pdinfo: dataset information.
  * @newvar: name for new variable, produced by stacking
  * @s: instructions for stacking existing variables.
+ * @prn: printing apparatus.
  *
  * Really for internal use.  Don't worry about it.
  *
@@ -1512,7 +1621,7 @@ static int missing_tail (const double *x, int n)
  */
 
 int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo, 
-			     char *newvar, char *s)
+			     char *newvar, char *s, PRN *prn)
 {
     char vn1[VNAMELEN], vn2[VNAMELEN];
     char format[16];
@@ -1709,7 +1818,7 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
     if (!err) {
 	strcpy(pdinfo->varname[genv], newvar);
 	make_stack_label(VARLABEL(pdinfo, genv), scpy);
-	sprintf(gretl_msg, "%s %s %s (ID %d)", 
+	pprintf(prn, "%s %s %s (ID %d)", 
 		(genv == pdinfo->v - 1)? _("Generated") : _("Replaced"),
 		_("vector"), newvar, genv);
     }
@@ -1735,8 +1844,7 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 
 int is_hidden_variable (int i, const DATAINFO *pdinfo)
 {
-    if (strcmp(pdinfo->varname[i], "subdum") == 0 ||
-	strcmp(pdinfo->varname[i], "annual") == 0 ||
+    if (strcmp(pdinfo->varname[i], "annual") == 0 ||
 	strcmp(pdinfo->varname[i], "qtrs") == 0 ||
 	strcmp(pdinfo->varname[i], "months") == 0 ||
 	strcmp(pdinfo->varname[i], "hrs") == 0 ||
