@@ -206,6 +206,8 @@ int dataset_add_series (int newvars, double ***pZ, DATAINFO *pdinfo);
 int dataset_add_allocated_series (double *x, double ***pZ, 
 				  DATAINFO *pdinfo);
 
+int dataset_add_scalars (int n, double ***pZ, DATAINFO *pdinfo);
+
 int dataset_add_scalar (double ***pZ, DATAINFO *pdinfo);
 
 int dataset_add_scalar_as (const char *numstr, const char *newname,
@@ -219,12 +221,14 @@ int dataset_scalar_to_vector (int v, double ***pZ, DATAINFO *pdinfo);
 int dataset_drop_listed_variables (const int *list, double ***pZ, 
 				   DATAINFO *pdinfo, int *renumber);
 
+int dataset_drop_variable (int v, double ***pZ, DATAINFO *pdinfo); 
+
 int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo);
 
 int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo);
 
 int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo, 
-			     char *newvar, char *s);
+			     char *newvar, char *s, PRN *prn);
 
 int is_hidden_variable (int i, const DATAINFO *pdinfo);
 
