@@ -224,7 +224,7 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 
 	if (*line1 == '#') continue;
 
-	if (sscanf(line1, "%8s", sername) != 1) break;
+	if (sscanf(line1, "%15s", sername) != 1) break;
 
 	if (!strcmp(series, sername)) {
 	    gotit = 1;
@@ -361,8 +361,10 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 
     sinfo->pd = dinfo->info;
     sinfo->nobs = n;
+
     *sinfo->varname = 0;
-    strncat(sinfo->varname, varname, 8);
+    strncat(sinfo->varname, varname, VNAMELEN - 1);
+
     *sinfo->descrip = 0;
     strncat(sinfo->descrip, comment, MAXLABEL - 1);
 
