@@ -463,12 +463,75 @@ double gretl_restricted_stddev (int t1, int t2, const double *x,
     return (na(xx))? xx : sqrt(xx);
 }
 
+/**
+* gretl_long_run_variance:
+* @t1: starting observation.
+* @t2: ending observation.
+* @x: data series.
+* @m: bandwidth.
+*
+* Returns: the long-run variance of the series @x from obs
+* @t1 to obs @t2, using Bartlett kernel weights, or #NADBL 
+* on failure (which includes encountering missing values). 
+*/
+
+double gretl_long_run_variance (int t1, int t2, const double *x, int m)
+{
+    double zt, wt, xbar, s2 = 0.0;
+    double *autocov;
+    int i, t, n;
+
+    if (array_adjust_t1t2(x, &t1, &t2)) {
+	return NADBL;
+    }
+
+    n = t2 - t1 + 1;
+
+    if (n < 2) {
+	return NADBL;
+    }
+
+    xbar = gretl_mean(t1, t2, x);
+
+    autocov = malloc(m * sizeof *autocov);
+    if (autocov == NULL) {
+	return NADBL;
+    }
+  
+    for (i=0; i<m; i++) {
+	autocov[i] = 0.0;
+    }
+
+    for (t=t1; t<=t2; t++) {
+	zt = x[t] - xbar;
+	s2 += zt * zt;
+	for (i=0; i<m; i++) {
+	    int s = i + 1;
+
+	    if (t - s >= t1) {
+		autocov[i] += zt * (x[t - s] - xbar);
+	    }
+	}
+    }
+
+    for (i=0; i<m; i++) {
+	wt = 1.0 - ((double) (i + 1)) / (m + 1.0);
+	s2 += 2.0 * wt * autocov[i];
+    }
+
+    s2 /= n;
+
+    free(autocov);
+
+    return s2;
+}
+
 /**
  * gretl_covar:
  * @t1: starting observation.
  * @t2: ending observation.
  * @x: data series.
- * @y: data seties.
+ * @y: data series.
  *
  * Returns: the covariance of the series @x and @y from obs
  * @t1 to obs @t2, skipping any missing values, or #NADBL 
@@ -525,7 +588,7 @@ double gretl_covar (int t1, int t2, const double *x, const double *y)
  * @t1: starting observation.
  * @t2: ending observation.
  * @x: data series.
- * @y: data seties.
+ * @y: data series.
  * @missing: location to receive information on the number
  * of missing observations that were skipped, or %NULL.
  *
@@ -600,7 +663,7 @@ double gretl_corr (int t1, int t2, const double *x, const double *y,
  * @t1: starting observation.
  * @t2: ending observation.
  * @x: data series.
- * @y: data seties.
+ * @y: data series.
  *
  * Returns: the square of the correlation coefficient for the series 
  * @x and @y from obs @t1 to obs @t2, skipping any missing values, 
@@ -953,8 +1016,8 @@ gretl_system_normality_test (const gretl_matrix *E, const gretl_matrix *Sigma,
 
     gretl_matrix_print_to_prn(C, "\nResidual correlation matrix, C", prn);
 
-    evals = gretl_symmetric_matrix_eigenvals(C, 1);
-    if (evals == NULL) {
+    evals = gretl_symmetric_matrix_eigenvals(C, 1, &err);
+    if (err) {
 	goto bailout;
     }
 
@@ -1049,6 +1112,24 @@ gretl_system_normality_test (const gretl_matrix *E, const gretl_matrix *Sigma,
     return err;
 }
 
+static freq_add_arrays (FreqDist *freq, int nbins)
+{
+    int err = 0;
+	
+    freq->endpt = malloc((nbins + 1) * sizeof *freq->endpt);
+    freq->midpt = malloc(nbins * sizeof *freq->midpt);
+    freq->f = malloc(nbins * sizeof *freq->f);
+	
+    if (freq->endpt == NULL || freq->midpt == NULL || freq->f == NULL) {
+	err = E_ALLOC;
+	strcpy(gretl_errmsg, _("Out of memory for frequency distribution"));
+    } else {
+	freq->numbins = nbins;
+    }
+
+    return err;
+}
+
 /**
  * get_freq:
  * @varno: ID number of variable to process.
@@ -1105,8 +1186,10 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
 	sprintf(gretl_errmsg, _("%s is a constant"), freq->varname);
 	free_freq(freq);
 	return NULL;
-    }    
-    
+    } 
+
+    freq->discrete = gretl_isdiscrete(pdinfo->t1, pdinfo->t2, x);
+
     gretl_moments(pdinfo->t1, pdinfo->t2, x, 
 		  &freq->xbar, &freq->sdx, 
 		  &skew, &kurt, params);
@@ -1134,73 +1217,128 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
 	return freq;
     }
     
-    if (n < 16) {
-	nbins = 5; 
-    } else if (n < 50) {
-	nbins = 7;
-    } else if (n > 850) {
-	nbins = 29;
+    if (freq->discrete) {
+	double *sorted = NULL;
+	int *ifreq = NULL, *ivals = NULL;
+	int i;
+
+	sorted = malloc(n * sizeof *sorted);
+	ifreq = malloc(n * sizeof *ifreq);
+	ivals = malloc(n * sizeof *ivals);
+
+	if (sorted == NULL || ifreq == NULL || ivals == NULL) {
+	    free(sorted);
+	    free(ifreq);
+	    free(ivals);
+	    free_freq(freq);
+	    return NULL;
+	}
+	
+	i = 0;
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    if (!na(x[t])) {
+		sorted[i++] = x[t];
+	    }
+	}
+	
+	qsort(sorted, n, sizeof *sorted, gretl_compare_doubles); 
+	nbins = 1;
+	xx = sorted[0];
+	ifreq[0] = 1;
+	ivals[0] = floor(xx);
+	
+	for (t=1; t<n; t++) {
+	    if (sorted[t] != xx) {
+		xx = sorted[t];
+		ifreq[nbins] = 1;
+		ivals[nbins++] = floor(xx);
+	    } else {
+		ifreq[nbins-1] += 1;
+	    }
+	}
+
+	if (freq_add_arrays(freq, nbins)) {
+	    gretl_errno = E_ALLOC;
+	    strcpy(gretl_errmsg, _("Out of memory for frequency distribution"));
+	    return freq;
+	}
+	
+	for (k=0; k<nbins; k++) {
+	    freq->endpt[k] = ivals[k];
+	    freq->midpt[k] = ivals[k];
+	    freq->f[k] = ifreq[k];
+	}
+	
+	freq->endpt[nbins] = xmax;
+
+	free(ifreq);
+	free(ivals);
+	free(sorted);
     } else {
-	nbins = (int) sqrt((double) n);
-	if (nbins % 2 == 0) {
-	    nbins++;
+	/* not discrete (integer) data */
+	if (n < 16) {
+	    nbins = 5; 
+	} else if (n < 50) {
+	    nbins = 7;
+	} else if (n > 850) {
+	    nbins = 29;
+	} else {
+	    nbins = (int) sqrt((double) n);
+	    if (nbins % 2 == 0) {
+		nbins++;
+	    }
 	}
-    }
-
-    freq->numbins = nbins;
-    binwidth = range / (nbins - 1);
-
-    freq->endpt = malloc((nbins + 1) * sizeof *freq->endpt);
-    freq->midpt = malloc(nbins * sizeof *freq->midpt);
-    freq->f = malloc(nbins * sizeof *freq->f);
-
-    if (freq->endpt == NULL || freq->midpt == NULL || freq->f == NULL) {
-	gretl_errno = E_ALLOC;
-	strcpy(gretl_errmsg, _("Out of memory for frequency distribution"));
-	return freq;
-    }
-    
-    freq->endpt[0] = xmin - .5 * binwidth;
-
-    if (xmin > 0.0 && freq->endpt[0] < 0.0) {
-	double rshift;
-
-	freq->endpt[0] = 0.0;
-	rshift = 1.0 - xmin / binwidth;
-	freq->endpt[freq->numbins] = xmax + rshift * binwidth;
-    } else {
-	freq->endpt[freq->numbins] = xmax + .5 * binwidth;
-    }
-    
-    for (k=0; k<freq->numbins; k++) {
-	freq->f[k] = 0;
-	if (k > 0) {
-	    freq->endpt[k] = freq->endpt[k-1] + binwidth;
+	
+	if (freq_add_arrays(freq, nbins)) {
+	    gretl_errno = E_ALLOC;
+	    strcpy(gretl_errmsg, _("Out of memory for frequency distribution"));
+	    return freq;
 	}
-	freq->midpt[k] = freq->endpt[k] + .5 * binwidth;
-    }
 
-    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	xx = x[t];
-	if (na(xx)) {
-	    continue;
+	binwidth = range / (nbins - 1);
+	
+	freq->endpt[0] = xmin - .5 * binwidth;
+	
+	if (xmin > 0.0 && freq->endpt[0] < 0.0) {
+	    double rshift;
+	    
+	    freq->endpt[0] = 0.0;
+	    rshift = 1.0 - xmin / binwidth;
+	    freq->endpt[freq->numbins] = xmax + rshift * binwidth;
+	} else {
+	    freq->endpt[freq->numbins] = xmax + .5 * binwidth;
 	}
-	if (xx < freq->endpt[1]) {
-	    freq->f[0] += 1;
-	    continue;
+	
+	for (k=0; k<freq->numbins; k++) {
+	    freq->f[k] = 0;
+	    if (k > 0) {
+		freq->endpt[k] = freq->endpt[k-1] + binwidth;
+	    }
+	    freq->midpt[k] = freq->endpt[k] + .5 * binwidth;
 	}
-	if (xx >= freq->endpt[freq->numbins]) {
-	    freq->f[freq->numbins-1] += 1;
-	    continue;
-	}
-	for (k=1; k<freq->numbins; k++) {
-	    if (freq->endpt[k] <= xx && xx < freq->endpt[k+1]) {
-		freq->f[k] += 1;
-		break;
+	
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    xx = x[t];
+	    if (na(xx)) {
+		continue;
+	    }
+	    if (xx < freq->endpt[1]) {
+		freq->f[0] += 1;
+		continue;
+	    }
+	    if (xx >= freq->endpt[freq->numbins]) {
+		freq->f[freq->numbins-1] += 1;
+		continue;
+	    }
+	    for (k=1; k<freq->numbins; k++) {
+		if (freq->endpt[k] <= xx && xx < freq->endpt[k+1]) {
+		    freq->f[k] += 1;
+		    break;
+		}
 	    }
 	}
     }
-
+	
     return freq;
 }
 
@@ -1360,8 +1498,10 @@ static char *corrgm_crit_string (void)
  * @order: integer order for autocorrelation function.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @batch: if = 1, use ASCII graphic rather than gnuplot graph.
  * @prn: gretl printing struct.
+ * @opt: if includes %OPT_A, use ASCII graphics; if includes
+ * %OPT_R, variable in question is a model residual generated
+ * "on the fly".
  *
  * Computes the autocorrelation function and plots the correlogram for
  * the variable specified by @varno.
@@ -1370,7 +1510,7 @@ static char *corrgm_crit_string (void)
  */
 
 int corrgram (int varno, int order, double ***pZ, 
-	      DATAINFO *pdinfo, int batch, PRN *prn)
+	      DATAINFO *pdinfo, PRN *prn, gretlopt opt)
 {
     double *acf, box, pm;
     double *pacf = NULL;
@@ -1420,9 +1560,13 @@ int corrgram (int varno, int order, double ***pZ,
 	acf[l-1] = gretl_acf(nobs, l, &(*pZ)[varno][t1]);
     }
 
-    sprintf(gretl_tmp_str, _("Autocorrelation function for %s"), 
-	    pdinfo->varname[varno]);
-    pprintf(prn, "\n%s\n\n", gretl_tmp_str);
+    if (opt & OPT_R) {
+	pprintf(prn, "\n%s\n\n", _("Residual autocorrelation function"));
+    } else {
+	sprintf(gretl_tmp_str, _("Autocorrelation function for %s"), 
+		pdinfo->varname[varno]);
+	pprintf(prn, "\n%s\n\n", gretl_tmp_str);
+    }
 
     /* add Ljung-Box statistic */
     box = 0;
@@ -1444,8 +1588,8 @@ int corrgram (int varno, int order, double ***pZ,
     }
     pputc(prn, '\n');
 
-    if (batch) { 
-	/* batch mode: use ASCII graphics, not gnuplot */
+    if (opt & OPT_A) { 
+	/* use ASCII graphics, not gnuplot */
 	double *xl = malloc(acf_m * sizeof *xl);
 
 	if (xl == NULL) {
@@ -1508,7 +1652,7 @@ int corrgram (int varno, int order, double ***pZ,
 	    corrgm_crit_string(),
 	    pm);
 
-    if (batch) {
+    if (opt & OPT_A) {
 	goto acf_getout;
     } else if (gnuplot_init(PLOT_CORRELOGRAM, &fq)) {
 	err = E_FOPEN;
@@ -1530,8 +1674,12 @@ int corrgram (int varno, int order, double ***pZ,
     if (!pacf_err) {
 	fputs("set origin 0.0,0.50\n", fq);
     }
-    fprintf(fq, "set title '%s %s'\n", I_("ACF for"), 
-	    pdinfo->varname[varno]);
+    if (opt & OPT_R) {
+	fprintf(fq, "set title '%s'\n", I_("Residual ACF"));
+    } else {
+	fprintf(fq, "set title '%s %s'\n", I_("ACF for"), 
+		pdinfo->varname[varno]);
+    }
     fprintf(fq, "set xrange [0:%d]\n", acf_m + 1);
     fprintf(fq, "plot \\\n"
 	    "'-' using 1:2 notitle w impulses, \\\n"
@@ -1545,8 +1693,12 @@ int corrgram (int varno, int order, double ***pZ,
     if (!pacf_err) {
 	/* lower plot: Partial Autocorrelation Function or PACF */
 	fputs("set origin 0.0,0.0\n", fq);
-	fprintf(fq, "set title '%s %s'\n", I_("PACF for"), 
-		pdinfo->varname[varno]);
+	if (opt & OPT_R) {
+	    fprintf(fq, "set title '%s'\n", I_("Residual PACF"));
+	} else {
+	    fprintf(fq, "set title '%s %s'\n", I_("PACF for"), 
+		    pdinfo->varname[varno]);
+	}
 	fprintf(fq, "set xrange [0:%d]\n", pacf_m + 1);
 	fprintf(fq, "plot \\\n"
 		"'-' using 1:2 notitle w impulses, \\\n"
@@ -1576,8 +1728,6 @@ int corrgram (int varno, int order, double ***pZ,
     return err;
 }
 
-/* ...................................................... */
-
 static int roundup_mod (int i, double x)
 {
     return (int) ceil((double) x * i);
@@ -1616,7 +1766,7 @@ static int fract_int_GPH (int n, double *hhat, double *omega, PRN *prn)
     list[2] = 0;
     list[3] = 2;
 
-    tmp = lsq(list, &tmpZ, tmpdinfo, OLS, OPT_A, 0);
+    tmp = lsq(list, &tmpZ, tmpdinfo, OLS, OPT_A);
 
     if (!tmp.errcode) {
 	tstat = -tmp.coeff[1] / tmp.sderr[1];
@@ -1722,7 +1872,7 @@ gretl_matrix *LWE_lambda (const gretl_matrix *I, int n, double *lcm)
     }
 
     llambda = gretl_matrix_copy(lambda);
-    gretl_matrix_log(llambda);
+    gretl_matrix_transform_elements(llambda, T_LOG);
     *lcm = gretl_vector_mean(llambda);
 
 #if LWE_DEBUG
@@ -1740,6 +1890,7 @@ double LWE_obj_func (const gretl_matrix *I, double d,
     gretl_matrix *lambda2, *Itmp;
     double dd = 2.0 * d;
     double ret;
+    int err = 0;
 
     lambda2 = gretl_matrix_copy(lambda);
     if (lambda2 == NULL) {
@@ -1748,7 +1899,7 @@ double LWE_obj_func (const gretl_matrix *I, double d,
 
     gretl_matrix_dot_pow(lambda2, dd);
 
-    Itmp = gretl_matrix_dot_multiply(I, lambda2);
+    Itmp = gretl_matrix_dot_multiply(I, lambda2, &err);
     if (Itmp == NULL) {
 	gretl_matrix_free(lambda2);
 	return NADBL;
@@ -1876,9 +2027,10 @@ int fract_int_LWE (const double **Z, int varno, int t1, int t2,
  * @varno: ID number of variable to process.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @batch: if non-zero, don't show gnuplot graph.
- * @opt: if non-zero, use Bartlett lag window for periodogram.
  * @prn: gretl printing struct.
+ * @opt: if includes %OPT_O, use Bartlett lag window for periodogram;
+ * if includes %OPT_N, don't display gnuplot graph; if includes
+ * %OPT_R, the variable is a model residual.
  *
  * Computes and displays the periodogram for the variable specified 
  * by @varno.
@@ -1888,15 +2040,21 @@ int fract_int_LWE (const double **Z, int varno, int t1, int t2,
  */
 
 int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo, 
-		 int batch, int opt, PRN *prn)
+		 PRN *prn, gretlopt opt)
 {
-    double *autocov, *omega, *hhat, *savexx = NULL;
-    double xx, yy, varx, w;
-    int err = 0, k, xmax, L, nT; 
-    int nobs, t, t1 = pdinfo->t1, t2 = pdinfo->t2;
+    double *autocov = NULL;
+    double *omega = NULL;
+    double *hhat = NULL;
+    double *savexx = NULL;
+    double *stdy = NULL;
+    double xx, yy, varx, stdx, w;
+    int k, xmax, L, nT, nobs; 
+    int t, t1 = pdinfo->t1, t2 = pdinfo->t2;
     int list[2];
-    int do_graph = !batch;
+    int do_graph = !(opt & OPT_N);
+    int window = (opt & OPT_O);
     FILE *fq = NULL;
+    int err = 0;
 
     *gretl_errmsg = 0;
 
@@ -1924,7 +2082,7 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
     }
 
     /* Chatfield (1996); Greene 4ed, p. 772 */
-    if (opt) {
+    if (window) {
 	L = (int) 2.0 * sqrt((double) nobs);
     } else {
 	L = nobs - 1; 
@@ -1940,19 +2098,29 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
     autocov = malloc((L + 1) * sizeof *autocov);
     omega = malloc(nT * sizeof *omega);
     hhat = malloc(nT * sizeof *hhat);
+    stdy = malloc(nobs * sizeof *stdy);
 
-    if (autocov == NULL || omega == NULL || hhat == NULL) {
+    if (autocov == NULL || omega == NULL || hhat == NULL || stdy == NULL) {
+	free(autocov);
+	free(omega);
+	free(hhat);
+	free(stdy);
 	return E_ALLOC;
     }
 
     xx = gretl_mean(t1, t2, (*pZ)[varno]);
+    varx = gretl_variance(t1, t2, &(*pZ)[varno][0]);
+    varx *= (double) (nobs - 1) / nobs;
+    stdx = sqrt(varx);
+    for (t=t1; t<=t2; t++) {
+	stdy[t-t1] = ((*pZ)[varno][t] - xx) / stdx;
+    }
 
     /* find autocovariances */
     for (k=1; k<=L; k++) {
-	autocov[k] = 0;
-	for (t=t1+k; t<=t2; t++) {
-	    autocov[k] += 
-		((*pZ)[varno][t] - xx) * ((*pZ)[varno][t-k] - xx);
+	autocov[k] = 0.0;
+	for (t=k; t<nobs; t++) {
+	    autocov[k] += stdy[t] * stdy[t-k];
 	}
 	autocov[k] /= nobs;
     }
@@ -1986,10 +2154,14 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 	fputs("set xzeroaxis\n", fq);
 	fputs("set nokey\n", fq);
 
-	sprintf(titlestr, I_("Spectrum of %s"), pdinfo->varname[varno]);
+	if (opt & OPT_R) {
+	    strcpy(titlestr, I_("Residual spectrum"));
+	} else {
+	    sprintf(titlestr, I_("Spectrum of %s"), pdinfo->varname[varno]);
+	}
 	fprintf(fq, "set title '%s", titlestr);
 
-	if (opt) {
+	if (window) {
 	    sprintf(titlestr, I_("Bartlett window, length %d"), L);
 	    fprintf(fq, " (%s)'\n", titlestr);
 	} else {
@@ -2005,9 +2177,14 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 	err = 1;
     }
 
-    pprintf(prn, _("\nPeriodogram for %s\n"), pdinfo->varname[varno]);
+    if (opt & OPT_R) {
+	pprintf(prn, "\n%s\n", _("Residual periodogram"));
+    } else {
+	pprintf(prn, _("\nPeriodogram for %s\n"), pdinfo->varname[varno]);
+    }
+
     pprintf(prn, _("Number of observations = %d\n"), nobs);
-    if (opt) {
+    if (window) {
 	pprintf(prn, _("Using Bartlett lag window, length %d\n\n"), L);
     }
     pputs(prn, _(" omega  scaled frequency  periods  spectral density\n\n"));
@@ -2021,21 +2198,18 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 	}
     }
 
-    varx = gretl_variance(t1, t2, &(*pZ)[varno][0]);
-    varx *= (double) (nobs - 1) / nobs;
-
     for (t=1; t<=nobs/2; t++) {
 	yy = 2 * M_PI * t / (double) nobs;
-	xx = varx; 
+	xx = 1.0; 
 	for (k=1; k<=L; k++) {
-	    if (opt) {
+	    if (window) {
 		w = 1 - (double) k/(L + 1);
 	    } else {
 		w = 1.0;
 	    }
 	    xx += 2.0 * w * autocov[k] * cos(yy * k);
 	}
-	xx /= 2 * M_PI;
+	xx *= varx /(2 * M_PI);
 	pprintf(prn, " %.4f%9d%16.2f%16.5f\n", yy, t, 
 		(double) nobs / t, xx);
 	if (savexx != NULL) {
@@ -2053,7 +2227,7 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 	gretl_push_c_numeric_locale();
 
 	for (t=1; t<=nobs/2; t++) {
-	    fprintf(fq, "%d %f\n", t, savexx[t]);
+	    fprintf(fq, "%d %g\n", t, savexx[t]);
 	}
 
 	gretl_pop_c_numeric_locale();
@@ -2065,7 +2239,7 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 	err = gnuplot_make_graph();
     }
 
-    if (opt == 0) {
+    if (!window) {
 	if (fract_int_GPH(nT, hhat, omega, prn)) {
 	    pprintf(prn, "\n%s\n", _("Fractional integration test failed"));
 	}
@@ -2075,12 +2249,11 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
     free(autocov);
     free(omega);
     free(hhat);
+    free(stdy);
 
     return err;
 }
 
-/* ............................................................. */
-
 static void printf15 (double zz, PRN *prn)
 {
     if (na(zz)) {
@@ -2116,8 +2289,6 @@ static void center_line (char *str, PRN *prn, int dblspc)
     }
 }
 
-/* ............................................................... */
-
 static void prhdr (const char *str, const DATAINFO *pdinfo, 
 		   int ci, int missing, PRN *prn)
 {
@@ -2137,7 +2308,7 @@ static void prhdr (const char *str, const DATAINFO *pdinfo,
     }
 }
 
-static void print_summary_single (const Summary *summ,
+static void print_summary_single (const Summary *summ, int j,
 				  const DATAINFO *pdinfo,
 				  PRN *prn)
 {
@@ -2160,17 +2331,17 @@ static void print_summary_single (const Summary *summ,
 
     prhdr(_("Summary Statistics"), pdinfo, SUMMARY, 0, prn);
     sprintf(tmp, _("for the variable '%s' (%d valid observations)"), 
-	    pdinfo->varname[summ->list[1]], summ->n);
+	    pdinfo->varname[summ->list[j+1]], summ->n);
     center_line(tmp, prn, 1);
 
-    vals[0] = summ->mean[0];
-    vals[1] = summ->median[0];
-    vals[2] = summ->low[0];
-    vals[3] = summ->high[0];
-    vals[4] = summ->sd[0];
-    vals[5] = summ->cv[0];
-    vals[6] = summ->skew[0];
-    vals[7] = summ->xkurt[0];
+    vals[0] = summ->mean[j];
+    vals[1] = summ->median[j];
+    vals[2] = summ->low[j];
+    vals[3] = summ->high[j];
+    vals[4] = summ->sd[j];
+    vals[5] = summ->cv[j];
+    vals[6] = summ->skew[j];
+    vals[7] = summ->xkurt[j];
 
     for (i=0; i<8; i++) {
 	if (strlen(_(labels[i])) > slen) {
@@ -2207,10 +2378,28 @@ void print_summary (const Summary *summ,
 		    PRN *prn)
 {
     int pause = gretl_get_text_pause();
+    int len, maxlen = 0;
     int i, vi, lineno;
 
     if (summ->list[0] == 1) {
-	print_summary_single(summ, pdinfo, prn);
+	print_summary_single(summ, 0, pdinfo, prn);
+	return;
+    }
+
+    for (i=1; i<=summ->list[0]; i++) {
+	vi = summ->list[i];
+	len = strlen(pdinfo->varname[vi]);
+	if (len > maxlen) {
+	    maxlen = len;
+	}
+    }
+
+    len = (maxlen <= 8)? 10 : maxlen + 1;
+
+    if (len > 13) {
+	for (i=0; i<summ->list[0]; i++) {
+	    print_summary_single(summ, i, pdinfo, prn);
+	}
 	return;
     }
 
@@ -2227,7 +2416,7 @@ void print_summary (const Summary *summ,
 	    scroll_pause();
 	    lineno = 1;
 	}
-	pprintf(prn, "%-10s", pdinfo->varname[vi]);
+	pprintf(prn, "%-*s", len, pdinfo->varname[vi]);
 	printf15(summ->mean[i], prn);
 	printf15(summ->median[i], prn);
 	printf15(summ->low[i], prn);
@@ -2256,7 +2445,7 @@ void print_summary (const Summary *summ,
 	    lineno = 1;
 	}
 
-	pprintf(prn, "%-10s", pdinfo->varname[vi]);
+	pprintf(prn, "%-*s", len, pdinfo->varname[vi]);
 
 	if (floateq(summ->mean[i], 0.0)) {
 	    cv = NADBL;
@@ -2623,7 +2812,9 @@ int means_test (const int *list, const double **Z, const DATAINFO *pdinfo,
     int vardiff = (opt & OPT_O);
     int df, n1, n2, n = pdinfo->n;
 
-    if (list[0] < 2) return E_ARGS;
+    if (list[0] < 2) {
+	return E_ARGS;
+    }
 
     x = malloc(n * sizeof *x);
     if (x == NULL) {
@@ -2688,7 +2879,7 @@ int means_test (const int *list, const double **Z, const DATAINFO *pdinfo,
     if (pval > .10)
 	pputs(prn, _("   The difference is not statistically significant.\n\n"));
 
-    record_test_result(t, pval, "difference of means");
+    record_test_result(t, pval, _("difference of means"));
 
     free(x);
     free(y);
@@ -2767,7 +2958,7 @@ int vars_test (const int *list, const double **Z, const DATAINFO *pdinfo,
     if (fdist(F, dfn, dfd) > .10)
 	pputs(prn, _("   The difference is not statistically significant.\n\n"));
 
-    record_test_result(F, pval, "difference of variances");
+    record_test_result(F, pval, _("difference of variances"));
 
     free(x);
     free(y);
@@ -3001,4 +3192,181 @@ MahalDist *get_mahal_distances (const int *list, double ***pZ,
     return md;
 }
 
+static double gini_coeff (const double *x, int t1, int t2, double **plz,
+			  int *pn, int *err)
+{
+    int m = t2 - t1 + 1;
+    double *sx = NULL;
+    double csx = 0.0, sumx = 0.0, sisx = 0.0;
+    double idx, gini;
+    int t, n = 0;
+
+    *gretl_errmsg = '\0';
+
+    sx = malloc(m * sizeof *sx);
+
+    if (sx == NULL) {
+	*err = E_ALLOC;
+	return NADBL;
+    }
+
+    n = 0;
+    for (t=t1; t<=t2; t++) {
+	if (na(x[t])) {
+	    continue;
+	} else if (x[t] < 0.0) {
+	    strcpy(gretl_errmsg, _("illegal negative value"));
+	    *err = E_DATA;
+	    break;
+	} else {
+	    sx[n++] = x[t];
+	    sumx += x[t];
+	}
+    }
+
+    if (*err) {
+	free(sx);
+	return NADBL;
+    } else if (n == 0) {
+	free(sx);
+	*err = E_DATA;
+	return NADBL;
+    }
+
+    qsort(sx, n, sizeof *sx, gretl_compare_doubles); 
+
+    for (t=0; t<n; t++) {
+	csx += sx[t];
+	idx = t + 1;
+	sisx += idx * sx[t];
+	sx[t] = csx / sumx; /* now = Lorenz curve */
+    }
+
+    gini = 2.0 * sisx / (n * sumx) - ((double) n + 1) / n;
+
+    if (plz != NULL) {
+	*plz = sx;
+	*pn = n;
+    } else {
+	free(sx);
+    }
+
+    return gini;
+}
+
+/**
+ * gretl_gini:
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @x: data series.
+ *
+ * Returns: the Gini coefficient for the series @x from obs
+ * @t1 to obs @t2, skipping any missing values, or #NADBL 
+ * on failure.
+ */
+
+double gretl_gini (int t1, int t2, const double *x)
+{
+    double g;
+    int err = 0;
+
+    g = gini_coeff(x, t1, t2, NULL, NULL, &err);
+
+    if (err) {
+	g = NADBL;
+    }
+
+    return g;
+}
+
+static int lorenz_graph (const char *vname, double *lz, int n)
+{
+    FILE *fp;
+    double idx;
+    int t, err = 0;
+
+    if (gnuplot_init(PLOT_REGULAR, &fp)) {
+	return E_FOPEN;
+    }
+
+    fputs("set key top left\n", fp);
+
+    fprintf(fp, "set title '%s'\n", vname);
+    fprintf(fp, "plot \\\n"
+	    "'-' using 1:2 title '%s' w lines, \\\n"
+	    "'-' using 1:2 notitle w lines\n",
+	    I_("Lorenz curve"));
+
+    gretl_push_c_numeric_locale();
+
+    for (t=0; t<n; t++) {
+	idx = t + 1;
+	fprintf(fp, "%g %g\n", idx / n, lz[t]);
+    }    
+    fputs("e\n", fp);
+
+    for (t=0; t<n; t++) {
+	idx = ((double) t + 1) / n;
+	fprintf(fp, "%g %g\n", idx, idx);
+    }    
+    fputs("e\n", fp);
+
+    gretl_pop_c_numeric_locale();
+
+    fclose(fp);
+
+    err = gnuplot_make_graph();
+
+    return err;
+}
+
+/**
+ * gini:
+ * @vnum: ID number of variable to examine.
+ * @Z: data matrix.
+ * @pdinfo: data information struct.
+ * @opt: not used yet.
+ * @prn: gretl printing struct.
+ *
+ * Graphs the Lorenz curve for variable @vnum and prints the
+ * Gini coefficient.
+ *
+ * Returns: 0 on successful completion, error code on error.
+ */
+
+int gini (int vnum, double ***pZ, DATAINFO *pdinfo, 
+	  gretlopt opt, PRN *prn)
+{
+    double *lz = NULL;
+    double gini;
+    int n, fulln;
+    int err = 0;
+
+    gini = gini_coeff((*pZ)[vnum], pdinfo->t1, pdinfo->t2, 
+		      &lz, &n, &err);
+
+    if (err) {
+	return err;
+    }
+
+    fulln = pdinfo->t2 - pdinfo->t1 - 1;
+    pprintf(prn, "%s\n", pdinfo->varname[vnum], n);
+    pprintf(prn, _("Number of observations = %d\n"), n);
+
+    if (n < fulln) {
+	pputs(prn, _("Warning: there were missing values\n"));
+    } 
+
+    pputc(prn, '\n');
+
+    pprintf(prn, "%s = %g\n", _("Sample Gini coefficient"), gini);
+    pprintf(prn, "%s = %g\n", _("Estimate of population value"), 
+	    gini * (double) n / (n - 1));
+
+    err = lorenz_graph(pdinfo->varname[vnum], lz, n);
+
+    free(lz);
+
+    return err;
+}
 
