@@ -17,6 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifndef DESCRIBE_H
+#define DESCRIBE_H
+
 typedef struct Summary_ Summary;
 typedef struct FreqDist_ FreqDist;
 typedef struct MahalDist_ MahalDist;
@@ -37,6 +40,7 @@ struct Summary_ {
 
 struct FreqDist_ {
     char varname[VNAMELEN];  /* for ID purposes */
+    int discrete;            /* 1 if variable contains integers */
     int dist;                /* code for theoretical distribution */
     int numbins;             /* number of bins or intervals */
     double xbar, sdx;        /* mean and std dev of variable */
@@ -79,6 +83,8 @@ double gretl_restricted_stddev (int t1, int t2, const double *x,
 				const double *y, GretlOp yop,
 				double yval);
 
+double gretl_long_run_variance (int t1, int t2, const double *x, int m);
+
 double gretl_covar (int t1, int t2, const double *x, const double *y);
 
 double gretl_corr (int t1, int t2, const double *x, const double *y,
@@ -105,11 +111,10 @@ int auto_acf_order (int pd, int nobs);
 
 int corrgram (int varno, int order, 
 	      double ***pZ, DATAINFO *pdinfo, 
-	      int batch, PRN *prn);
+	      PRN *prn, gretlopt opt);
 
-int periodogram (int varno, 
-		 double ***pZ, const DATAINFO *pdinfo, 
-		 int batch, int opt, PRN *prn);
+int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo, 
+		 PRN *prn, gretlopt opt);
 
 Summary *summary (const int *list, const double **Z, 
 		       const DATAINFO *pdinfo,
@@ -161,5 +166,14 @@ MahalDist *get_mahal_distances (const int *list, double ***pZ,
 void free_mahal_dist (MahalDist *md);
 
 const double *mahal_dist_get_distances (const MahalDist *md);
+
 int mahal_dist_get_n (const MahalDist *md);
+
 const int *mahal_dist_get_varlist(const MahalDist *md);
+
+double gretl_gini (int t1, int t2, const double *x);
+
+int gini (int vnum, double ***pZ, DATAINFO *pdinfo, 
+	  gretlopt opt, PRN *prn);
+
+#endif /* DESCRIBE_H */
