@@ -321,7 +321,7 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
  * errors and covariance matrix.
  *
  * Computes estimates of the discrete model specified by @list,
- * using an estimator determined by the value of @opt.  Uses the
+ * using an estimator determined by the value of @ci.  Uses the
  * BRMR auxiliary regression; see Davidson and MacKinnon.
  * 
  * Returns: a #MODEL struct, containing the estimates.
@@ -397,7 +397,7 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
     varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
 			  (const double **) *pZ);
 
-    dmod = lsq(list, pZ, pdinfo, OLS, OPT_A, 0);
+    dmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
     if (dmod.list[0] != list[0]) {
 	dmod.errcode = E_DATA;
 	goto bailout;
@@ -475,7 +475,7 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 
 	llbak = dmod.lnL;
 	clear_model(&dmod);
-	dmod = lsq(dmodlist, pZ, pdinfo, OLS, OPT_A, 0);
+	dmod = lsq(dmodlist, pZ, pdinfo, OLS, OPT_A);
 	if (dmod.errcode) {
 	    fprintf(stderr, "logit_probit: dmod errcode = %d\n", dmod.errcode);
 	    if (iters > 0) {
@@ -640,7 +640,7 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 			     4 * sizeof *act_pred);
     }
 
-    mle_aic_bic(&dmod, 0);
+    mle_criteria(&dmod, 0);
 
     dmod.ID = model_count_plus();
 
@@ -890,7 +890,7 @@ static int rewrite_logistic_stats (const double **Z, const DATAINFO *pdinfo,
     pmod->list[1] = dv;
     gretl_model_set_double(pmod, "lmax", lmax);
     pmod->ci = LOGISTIC;
-    ls_aic_bic(pmod);
+    ls_criteria(pmod);
 
     return 0;
 }
@@ -946,7 +946,7 @@ MODEL logistic_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     llist[1] = pdinfo->v - 1;
 
-    lmod = lsq(llist, pZ, pdinfo, OLS, OPT_A, 0.0);
+    lmod = lsq(llist, pZ, pdinfo, OLS, OPT_A);
     if (!lmod.errcode) {
 	rewrite_logistic_stats((const double **) *pZ, pdinfo, &lmod,
 			       dv, lmax);
