@@ -323,7 +323,7 @@ ldepvar_std_errors (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
     pdinfo->t1 = pmod->t1;
     pdinfo->t2 = pmod->t2;
 
-    emod = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+    emod = lsq(list, pZ, pdinfo, OLS, OPT_A);
     if (emod.errcode) {
 	err = emod.errcode;
     } else {
@@ -563,19 +563,6 @@ lsq_check_for_missing_obs (MODEL *pmod, gretlopt opts,
     return missv;
 }
 
-static int const_pos (const int *list)
-{
-    int i;
-
-    for (i=2; i<=list[0]; i++) {
-        if (list[i] == 0) {
-	    return i;
-	}
-    }
-
-    return 0;
-}
-
 static void 
 lagged_depvar_check (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
 {
@@ -590,7 +577,8 @@ lagged_depvar_check (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
 
 #define COLL_DEBUG 0
 
-int redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int trim)
+int 
+redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int **droplist)
 {
     MODEL cmod;
     int targ, ml0 = pmod->list[0];
@@ -616,10 +604,14 @@ int redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int trim)
     }
 
 #if COLL_DEBUG
-    fprintf(stderr, "\n*** redundant_var called (trim = %d) ***\n", trim);
+    fprintf(stderr, "\n*** redundant_var called ***\n");
     printlist(pmod->list, "original model list");
 #endif
 
+    /* back up along the list of regressors, trying to find a
+       variable such that, when it is deleted, the exact
+       collinearity error goes away. */
+
     while (err == E_SINGULAR && ml0 > 3) {
 
 	list[0] = ml0 - 1;
@@ -640,8 +632,7 @@ int redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int trim)
 	    fprintf(stderr, "target list position = %d\n", targ);
 	    printlist(list, "temp list for redundancy check");
 #endif
-
-	    cmod = lsq(list, pZ, pdinfo, OLS, OPT_A | OPT_Z, 0.0);
+	    cmod = lsq(list, pZ, pdinfo, OLS, OPT_A | OPT_Z);
 
 	    err = cmod.errcode;
 	    if (err == 0) {
@@ -665,27 +656,14 @@ int redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int trim)
     }
 
     if (ret == 1) {
-	static char msg[ERRLEN];
-	int rem, v = pmod->list[targ];
+	int v = pmod->list[targ];
 
 	/* remove var from list and reduce number of coeffs */
 	gretl_list_delete_at_pos(pmod->list, targ);
 	get_model_df(pmod);
 
-	/* compose a message */
-	if (trim == 0) {
-	    strcpy(msg, _("Omitted due to exact collinearity:"));
-	}
-
-	rem = ERRLEN - 1 - strlen(msg);
-
-	if (pdinfo->varname[v][0] != 0 && 
-	    rem > strlen(pdinfo->varname[v]) + 1) {
-	    strcat(msg, " ");
-	    strcat(msg, pdinfo->varname[v]);
-	}
-
-	strcpy(gretl_msg, msg);
+	/* add redundant var to list of drops */
+	gretl_list_append_term(droplist, v);
 
 	/* if there's a lagged dep var, it may have moved */
 	if (gretl_model_get_int(pmod, "ldepvar")) {
@@ -698,34 +676,13 @@ int redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int trim)
     return ret;
 }
 
-/**
- * lsq:
- * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- * @ci: one of the command indices in #LSQ_MODEL.
- * @opt: option flags: zero or more of the following --
- *   %OPT_R compute robust standard errors;
- *   %OPT_C force use of Cholesky decomp;
- *   %OPT_A treat as auxiliary regression (don't bother checking
- *     for presence of lagged dependent var, don't augment model count);
- *   %OPT_P use Prais-Winsten for first obs.
- *   %OPT_N don't use degrees of freedom correction for standard
- *      error of regression
- *   %OPT_M reject missing observations within sample range
- *   %OPT_Z (internal use) suppress the automatic elimination of 
- *      perfectly collinear variables
- * @rho: coefficient for rho-differencing the data (0.0 for no
- * differencing)
- *
- * Computes least squares estimates of the model specified by @list,
- * using an estimator determined by the value of @ci.
- * 
- * Returns: a #MODEL struct, containing the estimates.
- */
+/* as lsq() below, except that we allow for a non-zero value
+   of the first-order quasi-differencing coefficient, rho,
+   and there's no PRN.
+*/
 
-MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo, 
-	   GretlCmdIndex ci, gretlopt opt, double rho)
+MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo, 
+	       GretlCmdIndex ci, gretlopt opt, double rho)
 {
     int l0, yno, i;
     int effobs = 0;
@@ -844,12 +801,10 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     /* drop any vars that are all zero and repack the list */
     omitzero(&mdl, (const double **) *pZ, pdinfo);
 
-    /* if regressor list contains a constant, place it first */
-    i = const_pos(mdl.list);
-    mdl.ifc = (i > 1);
-    if (i > 2) {
-	rearrange_list(mdl.list);
-    }
+    /* if regressor list contains a constant, record this fact and 
+       place it first among the regressors */
+    mdl.ifc = reglist_check_for_const(mdl.list, (const double **) *pZ, 
+				      pdinfo);
 
     /* Check for presence of lagged dependent variable? 
        (Don't bother if this is an auxiliary regression.) */
@@ -895,12 +850,12 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	mdl.rho = rho;
 	gretl_qr_regress(&mdl, pZ, pdinfo, opt);
     } else {
-	int trim = 0;
+	int *droplist = NULL;
 	int l, nxpx;
 
     trim_var:
 
-	if (trim) {
+	if (droplist != NULL) {
 	    l0 = mdl.list[0];
 	    free(mdl.xpx);
 	    free(mdl.coeff);
@@ -911,6 +866,10 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	l = l0 - 1;
 	nxpx = l * (l + 1) / 2;
 
+	if (nxpx == 0) {
+	    fprintf(stderr, "problem: nxpx = 0 (l0 = %d)\n", l0);
+	}
+
 	xpy = malloc((l0 + 1) * sizeof *xpy);
 	mdl.xpx = malloc(nxpx * sizeof *mdl.xpx);
 	mdl.coeff = malloc(mdl.ncoeff * sizeof *mdl.coeff);
@@ -954,9 +913,13 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	free(xpy);
 
 	if (mdl.errcode == E_SINGULAR && !(opt & OPT_Z) &&
-	    redundant_var(&mdl, pZ, pdinfo, trim++)) {
+	    redundant_var(&mdl, pZ, pdinfo, &droplist)) {
 	    goto trim_var;
 	}
+
+	if (droplist != NULL) {
+	    gretl_model_set_list_as_data(&mdl, "droplist", droplist);
+	}
     }
 
     if (mdl.errcode) {
@@ -1004,7 +967,8 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     gretl_calculate_criteria((mdl.ci == WLS)? mdl.ess_wt : mdl.ess, 
 			     mdl.nobs, mdl.ncoeff, &mdl.lnL, 
 			     &mdl.criterion[C_AIC],
-			     &mdl.criterion[C_BIC]);
+			     &mdl.criterion[C_BIC],
+			     &mdl.criterion[C_HQC]);
 
     /* hccm command or HC3a */
     if (jackknife) {
@@ -1028,6 +992,36 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     return mdl;
 }
 
+/**
+ * lsq:
+ * @list: dependent variable plus list of regressors.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @ci: one of the command indices in #LSQ_MODEL.
+ * @opt: option flags: zero or more of the following --
+ *   %OPT_R compute robust standard errors;
+ *   %OPT_C force use of Cholesky decomp;
+ *   %OPT_A treat as auxiliary regression (don't bother checking
+ *     for presence of lagged dependent var, don't augment model count);
+ *   %OPT_P use Prais-Winsten for first obs;
+ *   %OPT_N don't use degrees of freedom correction for standard
+ *      error of regression;
+ *   %OPT_M reject missing observations within sample range;
+ *   %OPT_Z (internal use) suppress the automatic elimination of 
+ *      perfectly collinear variables.
+ *
+ * Computes least squares estimates of the model specified by @list,
+ * using an estimator determined by the value of @ci.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
+MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo, 
+	   GretlCmdIndex ci, gretlopt opt)
+{
+    return ar1_lsq(list, pZ, pdinfo, ci, opt, 0.0);
+}
+
 /*
   form_xpxxpy: form the X'X matrix and X'y vector
 
@@ -1508,10 +1502,10 @@ static void diaginv (double *xpx, double *xpy, double *diag, int nv)
  * makevcv:
  * @pmod: pointer to model.
  *
- * Inverts the Cholesky-decomposed X'X and computes the 
+ * Inverts the Cholesky-decomposed X'X matrix and computes the 
  * coefficient covariance matrix.
  * 
- * Returns: 0 on successful completion, error code on error.
+ * Returns: 0 on successful completion, non-zero code on error.
  */
 
 int makevcv (MODEL *pmod)
@@ -1595,9 +1589,16 @@ int makevcv (MODEL *pmod)
     return 0;
 }
 
-/*  dwstat: computes Durbin-Watson statistic
-    order is the order of autoregression, 0 for OLS.
-*/
+/**
+ * dwstat:
+ * @order: order of autoregression, 0 for OLS.
+ * @pmod: pointer to model.
+ * @Z: data array.
+ *
+ * Computes the Durbin-Watson statistic for @pmod.
+ * 
+ * Returns: the D-W value, or #NADBL on error.
+ */
 
 double dwstat (int order, MODEL *pmod, const double **Z)
 {
@@ -1665,9 +1666,18 @@ static double altrho (int order, int t1, int t2, const double *uhat)
     return rho;
 }
 
-/*  rhohat: computes first order serial correlation coefficient
-    order is the order of autoregression, 0 for OLS.
-*/
+/**
+ * rhohat:
+ * @order: order of autoregression, 0 for OLS residuals.
+ * @t1: start of sample range.
+ * @t2: end of sample range.
+ * @uhat: array of regression residuals.
+ *
+ * Computes the first order serial correlation coefficient
+ * for @uhat, over the range @t1 to @t2.
+ * 
+ * Returns: the \hat{rho} value, or #NADBL on error.
+ */
 
 double rhohat (int order, int t1, int t2, const double *uhat)
 {
@@ -1830,7 +1840,7 @@ static double autores (MODEL *pmod, const double **Z, int ci)
  * handling first-order serial correlation.  Print a trace of the
  * search for rho.
  * 
- * Returns: rho estimate on successful completion, %NADBL on error.
+ * Returns: rho estimate on successful completion, #NADBL on error.
  */
 
 double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
@@ -1869,7 +1879,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	/* Do Hildreth-Lu first */
 	for (rho = -0.990, iter = 0; rho < 1.0; rho += .01, iter++) {
 	    clear_model(&corc_model);
-	    corc_model = lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
+	    corc_model = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
 	    if ((*err = corc_model.errcode)) {
 		clear_model(&corc_model);
 		goto bailout;
@@ -1910,7 +1920,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	    /* try exploring this funny region? */
 	    for (rho = 0.99; rho <= 0.999; rho += .001) {
 		clear_model(&corc_model);
-		corc_model = lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
+		corc_model = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
 		if ((*err = corc_model.errcode)) {
 		    clear_model(&corc_model);
 		    goto bailout;
@@ -1927,7 +1937,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	    /* this even funnier one? */
 	    for (rho = 0.9991; rho <= 0.9999; rho += .0001) {
 		clear_model(&corc_model);
-		corc_model = lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
+		corc_model = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
 		if ((*err = corc_model.errcode)) {
 		    clear_model(&corc_model);
 		    goto bailout;
@@ -1950,7 +1960,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
     } else { 
 	/* Go straight to Cochrane-Orcutt (or Prais-Winsten) */
-	corc_model = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+	corc_model = lsq(list, pZ, pdinfo, OLS, OPT_A);
 	if (!corc_model.errcode && corc_model.dfd == 0) {
 	    corc_model.errcode = E_DF;
 	}
@@ -1988,7 +1998,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	while (diff > 0.001) {
 	    pprintf(prn, "          %10d %12.5f", ++iter, rho);
 	    clear_model(&corc_model);
-	    corc_model = lsq(list, pZ, pdinfo, OLS, lsqopt, rho);
+	    corc_model = ar1_lsq(list, pZ, pdinfo, OLS, lsqopt, rho);
 #if AR_DEBUG
 	    fprintf(stderr, "corc_model: t1=%d, first two uhats: %g, %g\n",
 		    corc_model.t1, 
@@ -2037,23 +2047,41 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
     return rho;
 }
 
-/* Given an original regression list, augment it by adding the squares
-   (and cross-products, if wanted) or logs of the original independent
-   variables.  Generate these variables if need be.  Return the
-   augmented list, or NULL on failure.
-*/
+/**
+ * augment_regression_list:
+ * @orig: list giving original regression specification.
+ * @aux: either %AUX_SQ, %AUX_LOG or %AUX_WHITE.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on the data set.
+ *
+ * Augment the regression list @orig with auxiliary terms.  If @aux 
+ * is %AUX_SQ add the squares of the original regressors; if @aux
+ * is %AUX_WHITE add squares and cross-products, or if @aux is
+ * %AUX_LOG add the natural logs of the original regressors.
+ * If theye are not already present, these variables are added
+ * to the data array.
+ * 
+ * Returns: the augmented list, or NULL on failure.
+ */
 
 int *augment_regression_list (const int *orig, int aux, 
 			      double ***pZ, DATAINFO *pdinfo)
 {
     int *list;
     int listlen;
+    int cnum = 0;
     int i, k;
 
     if (aux == AUX_WHITE) {
-	int trv = orig[0] - 1 - gretl_list_has_const(orig);
-	int nt = (trv * trv + trv) / 2;
+	int cpos = gretl_list_const_pos(orig, 2, (const double **) *pZ, 
+					pdinfo);
+	int nt, trv = orig[0] - 1;
 
+	if (cpos > 0) {
+	    trv--;
+	    cnum = orig[cpos];
+	}
+	nt = (trv * trv + trv) / 2;
 	listlen = orig[0] + nt + 1;
     } else {
 	listlen = 2 * orig[0];
@@ -2080,13 +2108,15 @@ int *augment_regression_list (const int *orig, int aux,
 
 	if (aux == AUX_SQ || aux == AUX_WHITE) {
 	    vnew = xpxgenr(vi, vi, pZ, pdinfo);
-	    if (vnew > 0) list[++k] = vnew;
+	    if (vnew > 0) {
+		list[++k] = vnew;
+	    }
 	    if (aux == AUX_WHITE) {
 		int j, vj;
 
 		for (j=i+1; j<=orig[0]; j++) {
 		    vj = orig[j];
-		    if (vj == 0) {
+		    if (vj == cnum) {
 			continue;
 		    }
 		    vnew = xpxgenr(vi, vj, pZ, pdinfo);
@@ -2095,7 +2125,9 @@ int *augment_regression_list (const int *orig, int aux,
 	    }
 	} else if (aux == AUX_LOG) {
 	    vnew = loggenr(vi, pZ, pdinfo);
-	    if (vnew > 0) list[++k] = vnew;
+	    if (vnew > 0) {
+		list[++k] = vnew;
+	    }
 	}
     }
 
@@ -2148,7 +2180,7 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
     pdinfo->t1 = pmod->t1;
     pdinfo->t2 = pmod->t2;
 
-    aux = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.);
+    aux = lsq(list, pZ, pdinfo, OLS, OPT_A);
     err = aux.errcode;
     if (err) {
 	shrink = pdinfo->v - oldv;
@@ -2182,7 +2214,7 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 /**
  * hsk_func:
  * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
+ * @pZ: pointer to data array.
  * @pdinfo: information on the data set.
  *
  * Estimate the model given in @list using a correction for
@@ -2201,7 +2233,7 @@ MODEL hsk_func (const int *list, double ***pZ, DATAINFO *pdinfo)
     *gretl_errmsg = '\0';
 
     /* run initial OLS */
-    hsk = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+    hsk = lsq(list, pZ, pdinfo, OLS, OPT_A);
     if (hsk.errcode) {
 	return hsk;
     }
@@ -2232,7 +2264,7 @@ MODEL hsk_func (const int *list, double ***pZ, DATAINFO *pdinfo)
     }
 
     clear_model(&hsk);
-    hsk = lsq(hsklist, pZ, pdinfo, WLS, OPT_NONE, 0.0);
+    hsk = lsq(hsklist, pZ, pdinfo, WLS, OPT_NONE);
     hsk.ci = HSK;
 
     dataset_drop_last_variables(pdinfo->v - orig_nvar, pZ, pdinfo);
@@ -2416,8 +2448,7 @@ static int jackknife_vcv (MODEL *pmod, const double **Z)
  * @opt: if flags include %OPT_S, save results to model.
  * @prn: gretl printing struct.
  *
- * Runs White's test for heteroskedasticity on the given model,
- * putting the results into @test.
+ * Runs White's test for heteroskedasticity on the given model.
  * 
  * Returns: 0 on successful completion, error code on error.
  */
@@ -2477,7 +2508,7 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
     if (!err) {
 	/* run auxiliary regression */
-	white = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.);
+	white = lsq(list, pZ, pdinfo, OLS, OPT_A);
 	err = white.errcode;
     }
 
@@ -2502,7 +2533,7 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	    }	  
 	}
 
-	record_test_result(TR2, pval, "White's");
+	record_test_result(TR2, pval, _("White's"));
     }
 
     clear_model(&white);
@@ -2530,8 +2561,6 @@ static int ar_list_max (const int *list)
 /**
  * ar_func:
  * @list: list of lags plus dependent variable and list of regressors.
- * @pos: position in list of separator (dummy element) between lag list
- * and specification of dependent and independent variables.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
  * @opt: may contain OPT_O to print covariance matrix.
@@ -2543,7 +2572,7 @@ static int ar_list_max (const int *list)
  * Returns: #MODEL struct containing the results.
  */
 
-MODEL ar_func (const int *list, int pos, double ***pZ, 
+MODEL ar_func (const int *list, double ***pZ, 
 	       DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
     double diff, ess, tss, xx;
@@ -2551,6 +2580,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
     int err, lag, maxlag, v = pdinfo->v;
     int *arlist = NULL, *rholist = NULL;
     int *reglist = NULL, *reglist2 = NULL;
+    int pos, cpos;
     MODEL ar, rhomod;
 
     *gretl_errmsg = '\0';
@@ -2558,6 +2588,8 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
     gretl_model_init(&ar);
     gretl_model_init(&rhomod);
 
+    pos = gretl_list_separator_position(list);
+
     arlist = malloc(pos * sizeof *arlist);
     reglist = malloc((list[0] - pos + 2) * sizeof *reglist);
     reglist2 = malloc((list[0] - pos + 2) * sizeof *reglist2);
@@ -2582,9 +2614,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
     rholist[0] = arlist[0] + 1;
     maxlag = ar_list_max(arlist);
 
-    if (const_pos(reglist) > 2) {
-	rearrange_list(reglist);
-    }
+    cpos = reglist_check_for_const(reglist, (const double **) *pZ, pdinfo);
 
     /* special case: ar 1 ; ... => use CORC */
     if (arlist[0] == 1 && arlist[1] == 1) {
@@ -2592,7 +2622,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
 	if (err) {
 	    ar.errcode = err;
 	} else {
-	    ar = lsq(reglist, pZ, pdinfo, CORC, OPT_NONE, xx);
+	    ar = ar1_lsq(reglist, pZ, pdinfo, CORC, OPT_NONE, xx);
 	    printmodel(&ar, pdinfo, opt, prn); 
 	}
 	goto bailout;
@@ -2601,7 +2631,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
     /* first pass: estimate model via OLS: use OPT_M to generate an
        error in case of missing values within sample range 
     */
-    ar = lsq(reglist, pZ, pdinfo, OLS, OPT_A | OPT_M, 0.0);
+    ar = lsq(reglist, pZ, pdinfo, OLS, OPT_A | OPT_M);
     if (ar.errcode) {
 	goto bailout;
     }
@@ -2654,7 +2684,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
 	if (iter > 1) {
 	    clear_model(&rhomod);
 	}
-	rhomod = lsq(rholist, pZ, pdinfo, OLS, OPT_A, 0.0);
+	rhomod = lsq(rholist, pZ, pdinfo, OLS, OPT_A);
 
 	/* and rho-transform the data */
 	ryno = vc = v + i;
@@ -2676,7 +2706,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
 
 	/* estimate the transformed model */
 	clear_model(&ar);
-	ar = lsq(reglist2, pZ, pdinfo, OLS, OPT_A, 0.0);
+	ar = lsq(reglist2, pZ, pdinfo, OLS, OPT_A);
 
         if (iter > 1) {
 	    diff = 100 * (ar.ess - ess) / ess;
@@ -2700,7 +2730,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
     for (i=0; i<=reglist[0]; i++) {
 	ar.list[i] = reglist[i];
     }
-    if (gretl_list_has_const(reglist)) {
+    if (cpos > 0) {
 	ar.ifc = 1;
     }
     if (ar.ifc) {
@@ -2737,7 +2767,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
 	tss += ((*pZ)[ryno][t] - xx) * ((*pZ)[ryno][t] - xx);
     }
     ar.fstt = ar.dfd * (tss - ar.ess) / (ar.dfn * ar.ess);
-    ls_aic_bic(&ar);
+    ls_criteria(&ar);
     ar.dw = dwstat(maxlag, &ar, (const double **) *pZ);
     ar.rho = rhohat(maxlag, ar.t1, ar.t2, ar.uhat);
 
@@ -2934,7 +2964,7 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 	arlist[2] = 0;
 
 	/* run OLS and get squared residuals */
-	archmod = lsq(pmod->list, pZ, pdinfo, OLS, OPT_A | OPT_M, 0.0);
+	archmod = lsq(pmod->list, pZ, pdinfo, OLS, OPT_A | OPT_M);
 	err = archmod.errcode;
     }
 
@@ -2963,7 +2993,7 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 
 	/* run aux. regression */
 	clear_model(&archmod);
-	archmod = lsq(arlist, pZ, pdinfo, OLS, OPT_A, 0.0);
+	archmod = lsq(arlist, pZ, pdinfo, OLS, OPT_A);
 	err = archmod.errcode;
     }
 
@@ -3029,7 +3059,7 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 		strcpy(pdinfo->varname[nwt], "1/sigma");
 
 		clear_model(&archmod);
-		archmod = lsq(wlist, pZ, pdinfo, WLS, OPT_NONE, 0.0);
+		archmod = lsq(wlist, pZ, pdinfo, WLS, OPT_NONE);
 
 		archmod.ci = ARCH;
 		archmod.order = order;
@@ -3137,7 +3167,7 @@ MODEL arch_model (const int *list, int order, double ***pZ, DATAINFO *pdinfo,
 /**
  * lad:
  * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
+ * @pZ: pointer to data array.
  * @pdinfo: information on the data set.
  *
  * Estimate the model given in @list using the method of Least
@@ -3156,7 +3186,7 @@ MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
        the lad_driver function will overwrite the coefficients etc.
     */
 
-    lad_model = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+    lad_model = lsq(list, pZ, pdinfo, OLS, OPT_A);
 
     if (lad_model.errcode) {
         return lad_model;
@@ -3185,7 +3215,8 @@ MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
  * @Z: data array.
  * @pdinfo: information on the data set.
  * @opt: options: may include %OPT_S to suppress intercept, %OPT_V
- * for verbose results, %OPT_X to use X-12-ARIMA.
+ * for verbose results, %OPT_X to use X-12-ARIMA, %OPT_C to put
+ * X-12-ARIMA into conditional maximum-likelihood mode.
  * @PRN: for printing details of iterations (or %NULL). 
  *
  * Calculate ARMA estimates, using either native gretl code or
@@ -3200,10 +3231,17 @@ MODEL arma (const int *list, const double **Z, const DATAINFO *pdinfo,
     MODEL armod;
     void *handle;
     MODEL (*arma_func) (const int *, const double **, const DATAINFO *, 
-			 gretlopt, PRN *);
+			gretlopt, PRN *);
 
     *gretl_errmsg = '\0';
 
+    if (opt & OPT_X && (pdinfo->t2 - pdinfo->t1) > 719) {
+	strcpy(gretl_errmsg, _("X-12-ARIMA can't handle more than 720 observations.\n"
+			       "Please select a smaller sample."));
+	armod.errcode = E_DATA;
+	return armod;
+    }	
+
     if (opt & OPT_X) {
 	arma_func = get_plugin_function("arma_x12_model", &handle);
     } else {
@@ -3228,12 +3266,12 @@ MODEL arma (const int *list, const double **Z, const DATAINFO *pdinfo,
 /**
  * tobit_model:
  * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
+ * @pZ: pointer to data array.
  * @pdinfo: information on the data set.
- * @prn: printing struct for iteration info (or NULL is this is not
+ * @prn: printing struct for iteration info (or %NULL is this is not
  * wanted).
  *
- * Estimate the model given in @list using Tobit.
+ * Produce Tobit estimates of the model given in @list.
  * 
  * Returns: a #MODEL struct, containing the estimates.
  */
@@ -3280,7 +3318,7 @@ static int get_offset_var (int *list)
  * @list: dependent variable plus list of regressors.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @prn: printing struct for iteration info (or NULL is this is not
+ * @prn: printing struct for iteration info (or %NULL is this is not
  * wanted).
  *
  * Estimate the Poisson regression model given in @list using ML.
@@ -3313,7 +3351,7 @@ MODEL poisson_model (const int *list, double ***pZ, DATAINFO *pdinfo, PRN *prn)
        coefficients etc.
     */
 
-    pmodel = lsq(listcpy, pZ, pdinfo, OLS, OPT_A, 0.0);
+    pmodel = lsq(listcpy, pZ, pdinfo, OLS, OPT_A);
     free(listcpy);
 
     if (pmodel.errcode) {
@@ -3339,7 +3377,7 @@ MODEL poisson_model (const int *list, double ***pZ, DATAINFO *pdinfo, PRN *prn)
 /**
  * garch:
  * @list: dependent variable plus arch and garch orders.
- * @pZ: pointer to data matrix.
+ * @pZ: pointer to data array.
  * @pdinfo: information on the data set.
  * @opt: can specify robust standard errors and VCV.
  * @prn: for printing details of iterations (or %NULL).
@@ -3423,7 +3461,7 @@ MODEL pooled (const int *list, double ***pZ, DATAINFO *pdinfo,
 
 	close_plugin(handle);
     } else {
-	wmod = lsq(list, pZ, pdinfo, POOLED, opt, 0.0);
+	wmod = lsq(list, pZ, pdinfo, POOLED, opt);
     }
 
     return wmod;
@@ -3432,7 +3470,7 @@ MODEL pooled (const int *list, double ***pZ, DATAINFO *pdinfo,
 /**
  * groupwise_hetero_test:
  * @pmod: pooled OLS model to be tested.
- * @pZ: pointer to data matrix.
+ * @pZ: pointer to data array.
  * @pdinfo: information on the (panel) data set.
  * @prn: for printing details of iterations (or %NULL).
  *
