@@ -21,6 +21,8 @@
 #include "forecast.h"
 #include "var.h"
 
+#define ARF_DEBUG 0
+
 #ifdef max
 # undef max
 #endif
@@ -46,7 +48,6 @@ struct Forecast_ {
     double *sderr;    /* array of forecast standard errors */
     double *eps;      /* array of estimated forecast errors */
     int *dvlags;      /* info on lagged dependent variable */
-    int offset;       /* start of yhat, etc. arrays relative to true 0 */
     int t1;           /* start of forecast range */
     int t2;           /* end of forecast range */
     int model_t2;     /* end of period over which model was estimated */
@@ -80,6 +81,8 @@ static int dummy_ar_info_init (MODEL *pmod)
 static int 
 allocate_basic_fit_resid_arrays (FITRESID *fr)
 {
+    int t;
+
     fr->actual = malloc(fr->nobs * sizeof *fr->actual);
     if (fr->actual == NULL) {
 	return E_ALLOC;
@@ -92,6 +95,10 @@ allocate_basic_fit_resid_arrays (FITRESID *fr)
 	return E_ALLOC;
     }
 
+    for (t=0; t<fr->nobs; t++) {
+	fr->actual[t] = fr->fitted[t] = NADBL;
+    }
+
     fr->sderr = NULL;
 
     return 0;
@@ -99,15 +106,19 @@ allocate_basic_fit_resid_arrays (FITRESID *fr)
 
 static int fit_resid_add_sderr (FITRESID *fr)
 {
-    int err = 0;
+    int t;
 
     fr->sderr = malloc(fr->nobs * sizeof *fr->sderr);
 
     if (fr->sderr == NULL) {
-	err = E_ALLOC;
-    } 
-    
-    return err;
+	return E_ALLOC;
+    }
+
+    for (t=0; t<fr->nobs; t++) {
+	fr->sderr[t] = NADBL;
+    }
+
+    return 0;
 }
 
 /**
@@ -134,11 +145,10 @@ static FITRESID *fit_resid_new (int n)
     fr->model_ID = 0;
     fr->model_ci = 0;
     fr->err = 0;
+    fr->t0 = 0;
     fr->t1 = 0;
     fr->t2 = 0;
     fr->nobs = 0;
-    fr->real_nobs = 0;
-    fr->pre_n = 0;
 
     if (n > 0) {
 	fr->nobs = n;
@@ -173,10 +183,12 @@ void free_fit_resid (FITRESID *fr)
 
 static void fit_resid_set_dec_places (FITRESID *fr)
 {
-    if (gretl_isdummy(0, fr->nobs - 1, fr->actual) > 0) {
-	fr->pmax = get_precision(fr->fitted, fr->nobs, 6);
+    int n = fr->t2 - fr->t0 + 1;
+
+    if (gretl_isdummy(fr->t0, fr->t2, fr->actual) > 0) {
+	fr->pmax = get_precision(fr->fitted + fr->t0, n, 6);
     } else {
-	fr->pmax = get_precision(fr->actual, fr->nobs, 6);
+	fr->pmax = get_precision(fr->actual + fr->t0, n, 6);
     }
 
     if (fr->pmax == 0) {
@@ -201,12 +213,12 @@ static void fit_resid_set_dec_places (FITRESID *fr)
 FITRESID *get_fit_resid (const MODEL *pmod, const double **Z, 
 			 const DATAINFO *pdinfo)
 {
-    int depvar, t, ft;
+    int depvar, t;
     FITRESID *fr;
 
     depvar = gretl_model_get_depvar(pmod);
 
-    fr = fit_resid_new(pmod->t2 - pmod->t1 + 1);
+    fr = fit_resid_new(pdinfo->n);
     if (fr == NULL) {
 	return NULL;
     }
@@ -217,14 +229,13 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
 	fr->sigma = pmod->sigma;
     }
 
+    fr->t0 = pmod->t1;
     fr->t1 = pmod->t1;
     fr->t2 = pmod->t2;
-    fr->real_nobs = pmod->nobs;
 
-    for (t=fr->t1; t<=fr->t2; t++) {
-	ft = t - fr->t1;
-	fr->actual[ft] = Z[depvar][t];
-	fr->fitted[ft] = pmod->yhat[t];
+    for (t=0; t<fr->nobs; t++) {
+	fr->actual[t] = Z[depvar][t];
+	fr->fitted[t] = pmod->yhat[t];
     }
 
     fit_resid_set_dec_places(fr);
@@ -243,8 +254,7 @@ static int *model_xlist (MODEL *pmod)
     if (xlist == NULL) {
 	xlist = gretl_model_get_x_list(pmod);
 	if (xlist != NULL) {
-	    gretl_model_set_data(pmod, "xlist", (void *) xlist,  
-				 (xlist[0] + 1) * sizeof *xlist);
+	    gretl_model_set_list_as_data(pmod, "xlist", xlist);
 	} 
     }
 
@@ -259,7 +269,7 @@ static int
 has_depvar_lags (MODEL *pmod, const DATAINFO *pdinfo)
 {
     int *xlist;
-    char xname[9], tmp[9];  
+    char xname[VNAMELEN], tmp[VNAMELEN];  
     const char *label;
     const char *yname;
     int i, lag;
@@ -274,8 +284,8 @@ has_depvar_lags (MODEL *pmod, const DATAINFO *pdinfo)
 
     for (i=1; i<=xlist[0]; i++) {
 	label = VARLABEL(pdinfo, xlist[i]);
-	if ((sscanf(label, "= %8[^(](t - %d)", xname, &lag) == 2 ||
-	     sscanf(label, "%8[^=]=%8[^(](-%d)", tmp, xname, &lag) == 3) &&
+	if ((sscanf(label, "= %15[^(](t - %d)", xname, &lag) == 2 ||
+	     sscanf(label, "%15[^=]=%15[^(](-%d)", tmp, xname, &lag) == 3) &&
 	    !strcmp(xname, yname)) {
 	    ret = 1;
 	    break;
@@ -316,7 +326,7 @@ static int process_lagged_depvar (MODEL *pmod,
     if (dvlags == NULL) {
 	err = 1;
     } else {
-	char xname[9], tmp[9];
+	char xname[VNAMELEN], tmp[VNAMELEN];
 	const char *label;
 	const char *yname;
 	int i, lag;
@@ -325,8 +335,8 @@ static int process_lagged_depvar (MODEL *pmod,
 
 	for (i=0; i<xlist[0]; i++) {
 	    label = VARLABEL(pdinfo, xlist[i+1]);
-	    if ((sscanf(label, "= %8[^(](t - %d)", xname, &lag) == 2 ||
-		 sscanf(label, "%8[^=]=%8[^(](-%d)", tmp, xname, &lag) == 3) &&
+	    if ((sscanf(label, "= %15[^(](t - %d)", xname, &lag) == 2 ||
+		 sscanf(label, "%15[^=]=%15[^(](-%d)", tmp, xname, &lag) == 3) &&
 		!strcmp(xname, yname)) {
 		dvlags[i] = lag;
 	    } else {
@@ -368,33 +378,29 @@ has_real_exog_regressors (MODEL *pmod, const int *dvlags,
 		}
 	    }
 	}
-    } else {
-	/* should not happen */
-	fprintf(stderr, "xlist is NULL\n");
-	ret = 1;
-    }
+    } 
 
     return ret;
 }
 
 static int 
-fit_resid_init (int t1, int t2, int pre_n, const MODEL *pmod, 
+fit_resid_init (int t0, int t1, int t2, const MODEL *pmod, 
 		const DATAINFO *pdinfo, FITRESID *fr)
 {
-    if (pre_n > t1) { /* is this right? */
-	pre_n = t1;
+    if (t0 > t1) { 
+	t0 = t1;
     }
 
+    fr->t0 = t0;
     fr->t1 = t1;
     fr->t2 = t2;
-    fr->pre_n = pre_n;
 
     if (fr->t1 < 0 || fr->t2 < 0 || fr->t2 < fr->t1) {
 	fr->err = E_OBS;
     }
 
     if (!fr->err) {
-	fr->nobs = fr->t2 - fr->t1 + 1 + fr->pre_n;
+	fr->nobs = pdinfo->n;
 	fr->err = allocate_basic_fit_resid_arrays(fr);
     }
 
@@ -433,7 +439,7 @@ static double fcast_get_ldv (Forecast *fc, int i, int t, int lag,
 #endif
 
     if (fc->method != FC_STATIC) {
-	int yht = t - fc->offset - lag;
+	int yht = t - lag;
 
 #if AR_DEBUG
 	fprintf(stderr, "fcast_get_ldv (non-static): yht = %d\n", yht);
@@ -473,7 +479,7 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 
     double s2 = pmod->sigma * pmod->sigma;
     int k = pmod->ncoeff;
-    int i, vi, s, t;
+    int i, vi, t;
     int err = 0;
 
     V = gretl_vcv_matrix_from_model(pmod, NULL);
@@ -498,8 +504,6 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 	int missing = 0;
 	double vyh;
 
-	s = t - fc->offset;
-
 	/* skip if we can't compute forecast */
 	if (t >= pmod->t1 && t <= pmod->t2) {
 	    missing = na(pmod->yhat[t]);
@@ -512,7 +516,7 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 	    vi = pmod->list[i + 2];
 	    xval = Z[vi][t];
 	    if (na(xval)) {
-		fc->sderr[s] = fc->yhat[s] = NADBL;
+		fc->sderr[t] = fc->yhat[t] = NADBL;
 		missing = 1;
 	    } else {
 		gretl_vector_set(Xs, i, xval);
@@ -520,12 +524,12 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 	}
 
 	if (missing) {
-	    fc->sderr[s] = fc->yhat[s] = NADBL;
+	    fc->sderr[t] = fc->yhat[t] = NADBL;
 	    continue;
 	}
 
 	/* forecast value */
-	fc->yhat[s] = gretl_matrix_dot_product(Xs, GRETL_MOD_NONE,
+	fc->yhat[t] = gretl_matrix_dot_product(Xs, GRETL_MOD_NONE,
 					       b, GRETL_MOD_NONE,
 					       NULL);
 
@@ -536,9 +540,9 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 	} else {
 	    vyh += s2;
 	    if (vyh >= 0.0) {
-		fc->sderr[s] = sqrt(vyh);
+		fc->sderr[t] = sqrt(vyh);
 	    } else {
-		fc->sderr[s] = NADBL;
+		fc->sderr[t] = NADBL;
 		err = 1;
 	    }
 	}
@@ -578,13 +582,13 @@ static int nls_fcast (Forecast *fc, const MODEL *pmod,
 	pdinfo->t1 = fc->t1;
 	pdinfo->t2 = fc->t2;
 	sprintf(formula, "$nl_y = %s", nlfunc);
-	err = generate(formula, pZ, pdinfo, OPT_P);
+	err = generate(formula, pZ, pdinfo, OPT_P, NULL);
     }
 
     if (!err) {
 	/* transcribe values from last generated var to target */
 	for (t=fc->t1; t<=fc->t2; t++) {
-	    fc->yhat[t - fc->offset] = (*pZ)[oldv][t];
+	    fc->yhat[t] = (*pZ)[oldv][t];
 	}
 	err = dataset_drop_last_variables(pdinfo->v - oldv, pZ, pdinfo);
     }
@@ -595,8 +599,6 @@ static int nls_fcast (Forecast *fc, const MODEL *pmod,
     return err;
 }
 
-#define ARF_DEBUG 0
-
 #if ARF_DEBUG
 # include <stdarg.h>
 static void dprintf (const char *format, ...)
@@ -848,7 +850,7 @@ static int garch_fcast (Forecast *fc, MODEL *pmod,
     double *mh = NULL;
     double *phi = NULL;
     double *psi = NULL;
-    int i, v, s, t;
+    int i, v, t;
 
     xlist = model_xlist(pmod);
     yno = gretl_model_get_depvar(pmod);
@@ -879,13 +881,11 @@ static int garch_fcast (Forecast *fc, MODEL *pmod,
 	int lag, miss = 0;
 	double yh = 0.0;
 
-	s = t - fc->offset;
-
 	if (fc->method != FC_DYNAMIC && 
 	    t >= pmod->t1 && t <= pmod->t2) {
-	    fc->yhat[s] = pmod->yhat[t];
+	    fc->yhat[t] = pmod->yhat[t];
 	    if (fc->sderr != NULL && mh != NULL) {
-		fc->sderr[s] = mh[t];
+		fc->sderr[t] = mh[t];
 	    }
 	    continue;
 	}
@@ -905,22 +905,22 @@ static int garch_fcast (Forecast *fc, MODEL *pmod,
 	}
 
 	if (miss) {
-	    fc->yhat[s] = NADBL;
+	    fc->yhat[t] = NADBL;
 	} else {
-	    fc->yhat[s] = yh;
+	    fc->yhat[t] = yh;
 	}
 
 	if (h != NULL) {
 	    if (t > pmod->t2) {
 		if (psi != NULL) {
 		    /* build in effect of lagged dependent var */
-		    fc->sderr[s] = garch_ldv_sderr(h, psi, t - pmod->t2 - 1);
+		    fc->sderr[t] = garch_ldv_sderr(h, psi, t - pmod->t2 - 1);
 		} else {
 		    /* no lagged dependent variable */
-		    fc->sderr[s] = sqrt(h[t - pmod->t2 - 1]);
+		    fc->sderr[t] = sqrt(h[t - pmod->t2 - 1]);
 		}
 	    } else {
-		fc->sderr[s] = NADBL;
+		fc->sderr[t] = NADBL;
 	    }
 	}
     }
@@ -936,60 +936,40 @@ static int garch_fcast (Forecast *fc, MODEL *pmod,
 }
 
 /* Compute ARMA forecast error variance (ignoring parameter
-   uncertainty, as is common), via recursion. 
+   uncertainty, as is common), via recursion.  Cf. Box and Jenkins,
+   1976, p. 508, "Program 4", V(l) algorithm (with the sign of theta
+   changed).
 */
 
-static double 
-arma_variance_machine (const double *phi, int p, 
-		       const double *theta, int q,
-		       double *psi, int s, double *ss_psi)
+static double arma_variance (const double *phi, int p, 
+			     const double *theta, int q,
+			     double *psi, int npsi, int l)
 {
-    int i, h = s - 1;
+    static double sspsi;
+    int i, j;
 
-    /* s = number of steps ahead of forecast baseline
-       h = index into "psi" array of infinite-order MA
-           coefficients
-    */
-
-    if (h > p) {
-	/* we don't retain more that (AR order + 1) 
-	   psi values (the last one being used as
-           workspace) */
-	h = p;
-    }
-
-    if (s == 1) {
-	/* one step ahead: initialize to unity */
-	psi[h] = 1.0;
-    } else {
-	/* init to zero prior to adding components */
-	psi[h] = 0.0;
-    }
-
-    /* add AR-derived psi[h] components */
-    for (i=1; i<=p && i<s; i++) {
-	psi[h] += phi[i-1] * psi[h-i];
-    }
-
-    /* add MA-derived psi[h] components */
-    if (s > 1 && s <= q+1) {
-	psi[h] += theta[s-2];
-    }
-
-    /* increment running sum of psi squared terms */
-    *ss_psi += psi[h] * psi[h];
-    
-    if (s > p) {
-	/* drop the oldest psi and make space for a new one */
-	for (i=0; i<p; i++) {
-	    psi[i] = psi[i+1];
+    if (l == 1) {
+	sspsi = 0.0;
+	psi[0] = 1.0;
+	for (j=1; j<npsi; j++) {
+	    psi[j] = 0.0;
+	    for (i=1; i<=j; i++) {
+		if (i <= p) {
+		    psi[j] += phi[i] * psi[j-i];
+		}
+		if (j <= q && theta != NULL) {
+		    psi[j] += theta[j];
+		}
+	    }
 	}
-    }
+    } 
 
-    return *ss_psi;
+    sspsi += psi[l-1] * psi[l-1];
+
+    return sspsi;
 }
 
-/* generate forecasts for ARMA (or ARMAX) models, including
+/* generate forecasts for AR(I)MA (or ARMAX) models, including
    forecast standard errors if we're doing out-of-sample
    forecasting
 */
@@ -997,19 +977,19 @@ arma_variance_machine (const double *phi, int p,
 static int arma_fcast (Forecast *fc, MODEL *pmod, 
 		       const double **Z, const DATAINFO *pdinfo)
 {
+    double *psi = NULL;
     double *phi = NULL;
     double *theta = NULL;
     const double *beta;
+    const double *y;
 
-    double *psi = NULL;
-    double ss_psi = 0.0;
-    double xval, yval;
+    double xval, yval, vl;
     int xvars, yno;
     int *xlist = NULL;
-    int p, q;
-    int tstart = fc->t1;
+    int p, q, npsi = 0;
+    int t1 = fc->t1;
     int ar_smax, ma_smax;
-    int i, s, t, tt;
+    int i, s, t;
     int err = 0;
 
     DPRINTF(("\n\n*** arma_fcast: METHOD = %d\n", fc->method));
@@ -1018,21 +998,20 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	/* use pre-calculated fitted values over model estimation range,
 	   and don't bother calculating forecast error variance */
 	for (t=fc->t1; t<=pmod->t2; t++) {
-	    tt = t - fc->offset;
-	    fc->yhat[tt] = pmod->yhat[t];
+	    fc->yhat[t] = pmod->yhat[t];
 	    if (fc->sderr != NULL) {
-		fc->sderr[tt] = NADBL;
+		fc->sderr[t] = NADBL;
 	    }
 	}
 	if (fc->t2 <= pmod->t2) {
 	    /* no "real" forecasts were called for, we're done */
 	    return 0;
 	}
-	tstart = pmod->t2 + 1;
+	t1 = pmod->t2 + 1;
     }
 
-    p = gretl_arma_model_get_max_AR_lag(pmod);
-    q = gretl_arma_model_get_max_MA_lag(pmod);
+    p = gretl_arma_model_max_AR_lag(pmod);
+    q = gretl_arma_model_max_MA_lag(pmod);
 
     xlist = model_xlist(pmod);
     yno = gretl_model_get_depvar(pmod);
@@ -1055,7 +1034,8 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 
     /* setup for forecast error variance */
     if (fc->sderr != NULL) {
-	psi = malloc((p + 1) * sizeof *psi);
+	npsi = fc->t2 - t1 + 1;
+	psi = malloc(npsi * sizeof *psi);
     }
 
     /* cut-off points for using actual rather than forecast
@@ -1073,13 +1053,13 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 
     DPRINTF(("ar_smax = %d, ma_smax = %d\n", ar_smax, ma_smax));
 
+    /* dependent variable */
+    y = Z[yno];
+
     /* do real forecast */
-    for (t=tstart; t<=fc->t2 && !err; t++) {
+    for (t=t1; t<=fc->t2 && !err; t++) {
 	int miss = 0;
 	double yh = 0.0;
-	int lag;
-
-	tt = t - fc->offset;
 
 	DPRINTF(("\n *** Doing forecast for obs %d\n", t));
 
@@ -1101,24 +1081,26 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 
 	DPRINTF((" x contribution = %g\n", yh));
 
-	/* AR contribution */
-	for (i=0; i<p && !miss; i++) {
-	    lag = i + 1;
-	    s = t - lag;
+	/* AR contribution (incorporating any differencing) */
+	for (i=1; i<=p && !miss; i++) {
+	    if (phi[i] == 0.0) {
+		continue;
+	    }
+	    s = t - i;
 	    if (s < 0) {
 		yval = NADBL;
 	    } else if (s <= ar_smax) {
-		yval = Z[yno][s];
-		DPRINTF(("  AR: lag %d, y[%d] = %g\n", lag, s, yval));
+		yval = y[s];
+		DPRINTF(("  AR: lag %d, y[%d] = %g\n", i, s, yval));
 	    } else {
-		yval = fc->yhat[s - fc->offset];
-		DPRINTF(("  AR: lag %d, yhat[%d] = %g\n", lag, s - fc->offset, yval));
+		yval = fc->yhat[s];
+		DPRINTF(("  AR: lag %d, yhat[%d] = %g\n", i, s, yval));
 	    }
 	    if (na(yval)) {
-		DPRINTF(("  AR: lag %d, s =%d, missing value\n", lag, s));
+		DPRINTF(("  AR: lag %d, missing value\n", i));
 		miss = 1;
 	    } else {
-		DPRINTF(("  AR: lag %d, s=%d, using coeff %g\n", lag, s, phi[i]));
+		DPRINTF(("  AR: lag %d, using coeff %g\n", i, phi[i]));
 		yh += phi[i] * yval;
 	    }
 	}
@@ -1126,84 +1108,73 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	DPRINTF((" with AR contribution: %g\n", yh));
 
 	/* MA contribution */
-	for (i=0; i<q && !miss; i++) {
-	    lag = i + 1;
-	    s = t - lag;
+	for (i=1; i<=q && !miss; i++) {
+	    if (theta[i] == 0.0) {
+		continue;
+	    }
+	    s = t - i;
 	    if (s >= pmod->t1 && s <= ma_smax) {
-		DPRINTF(("  MA: lag %d, e[%d] = %g, coeff %g\n", lag, s, 
-			 pmod->uhat[s], theta[i]));
+		DPRINTF(("  MA: lag %d, e[%d] = %g, theta[%d] = %g\n", i, s, 
+			 pmod->uhat[s], i, theta[i]));
 		yh += theta[i] * pmod->uhat[s];
 	    } else if (fc->eps != NULL) {
-		DPRINTF(("  MA: lag %d, ehat[%d] = %g, coeff %g\n", lag, s, 
-			 fc->eps[s - fc->offset], theta[i]));
-		yh += theta[i] * fc->eps[s - fc->offset];
+		DPRINTF(("  MA: lag %d, ehat[%d] = %g, theta[%d] = %g\n", i, s, 
+			 fc->eps[s], i, theta[i]));
+		yh += theta[i] * fc->eps[s];
 	    }
 	}
 
 	DPRINTF((" with MA contribution: %g\n", yh));
 
 	if (miss) {
-	    fc->yhat[tt] = NADBL;
+	    fc->yhat[t] = NADBL;
 	} else {
-	    fc->yhat[tt] = yh;
-	}
-
-	if (fc->eps != NULL && !miss) {
-	    /* form estimated error in case of static forecast */
-	    fc->eps[tt] = Z[yno][t] - fc->yhat[tt];
+	    fc->yhat[t] = yh;
+	    if (fc->eps != NULL) {
+		/* form estimated error: we do this only in
+		   the case of a "static" forecast */
+		fc->eps[t] = y[t] - yh;
+		DPRINTF(("\n setting ehat[%d] = %g - %g = %g\n", t, 
+			 y[t], yh, fc->eps[t]));
+	    }
 	}
 
 	/* forecast error variance */
 	if (psi != NULL) {
-	    arma_variance_machine(phi, p, theta, q,
-				  psi, t - tstart + 1, 
-				  &ss_psi);
-	    fc->sderr[tt] = pmod->sigma * sqrt(ss_psi);
+	    vl = arma_variance(phi, p, theta, q, psi, npsi, t - t1 + 1);
+	    fc->sderr[t] = pmod->sigma * sqrt(vl);
 	}
-
-#if 0
-	if (miss && t >= p) {
-	    DPRINTF(("aborting with NA at t=%d (p=%d)\n", t, p));
-	    err = 1;
-	}
-#endif
     }
 
  bailout:
 
-    if (psi != NULL) {
-	free(psi);
-    }
-    if (phi != NULL) {
-	free(phi);
-    }
-    if (theta != NULL) {
-	free(theta);
-    }
+    free(psi);
+    free(phi);
+    free(theta);
 
     return err;
 }
 
-/* construct the "phi" array of AR coefficients, based on the
-   ARINFO that was added to the model at estimation time.
-   The latter's rho member may be a compacted array, with
-   zero elements omitted, but here we need a full-length
-   array with zeros inserted as required */
+/* construct the "phi" array of AR coefficients, based on the ARINFO
+   that was added to the model at estimation time.  The latter's rho
+   member may be a compacted array, with zero elements omitted, but
+   here we need a full-length array with zeros inserted as required.
+*/
 
 static double *make_phi_from_arinfo (const ARINFO *arinfo, int pmax)
 {
-    double *phi = malloc(pmax * sizeof *phi);
+    double *phi = malloc((pmax + 1) * sizeof *phi);
 
     if (phi != NULL) {
 	int i, lag;
 
-	for (i=0; i<pmax; i++) {
+	for (i=0; i<=pmax; i++) {
 	    phi[i] = 0.0;
 	}
 
 	for (i=1; i<=arinfo->arlist[0]; i++) {
 	    lag = arinfo->arlist[i];
-	    phi[lag-1] = arinfo->rho[i-1];
+	    phi[lag] = arinfo->rho[i-1];
 	}
     }
 
@@ -1238,27 +1209,44 @@ static int max_ar_lag (Forecast *fc, const MODEL *pmod, int p)
    is quite right yet.
 */
 
-static void set_up_ar_fcast_variance (const MODEL *pmod, int pmax,
-				      double **phi, double **psi,
-				      double **errphi)
+static int
+set_up_ar_fcast_variance (Forecast *fc, const MODEL *pmod, 
+			  int pmax, int npsi, double **phi, double **psi,
+			  double **errphi)
 {
-    *errphi = make_phi_from_arinfo(pmod->arinfo, pmax);
+    int err = 0;
+    
+    *errphi = NULL;
+    *psi = NULL;
+    *phi = NULL;
 
-    if (*errphi != NULL) {
-	*psi = malloc((pmax + 1) * sizeof **psi);
+    *errphi = make_phi_from_arinfo(pmod->arinfo, pmax);
+    if (*errphi == NULL) {
+	err = E_ALLOC;
+    }
+
+    if (!err) {
+	*psi = malloc(npsi * sizeof **psi);
 	if (*psi == NULL) {
-	    free(*errphi);
-	    *errphi = NULL;
-	} else {
-	    *phi = malloc(pmax * sizeof **phi);
-	    if (*phi == NULL) {
-		free(*errphi);
-		*errphi = NULL;
-		free(*psi);
-		*psi = NULL;
-	    }
+	    err = E_ALLOC;
 	}
-    } 
+    }
+
+    if (!err) {
+	*phi = malloc((pmax + 1) * sizeof **phi);
+	if (*phi == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (err) {
+	free(*errphi);
+	*errphi = NULL;
+	free(*psi);
+	*psi = NULL;
+    }
+
+    return err;
 }
 
 /* 
@@ -1296,12 +1284,12 @@ static int ar_fcast (Forecast *fc, MODEL *pmod,
     double *phi = NULL;
     double *psi = NULL;
     double *errphi = NULL;
-    double ss_psi = 0.0;
-    double xval, yh;
+    double xval, yh, vl;
     double rk, ylag, xlag;
     int miss, yno;
-    int i, k, v, s, t, tk;
+    int i, k, v, t, tk;
     int p, dvlag, pmax = 0;
+    int npsi = 0;
     int err = 0;
 
 #if AR_DEBUG
@@ -1313,35 +1301,35 @@ static int ar_fcast (Forecast *fc, MODEL *pmod,
     p = arlist[arlist[0]]; /* AR order of error term */
 
     if (fc->t2 > pmod->t2 && fc->sderr != NULL) {
-	/* we compute variance only if we're forecasting
-	   out of sample */
+	/* we'll compute variance only if we're forecasting out of
+	   sample */
 	pmax = max_ar_lag(fc, pmod, p);
-	set_up_ar_fcast_variance(pmod, pmax, &phi, &psi, &errphi);
+	npsi = fc->t2 - fc->t1 + 1;
+	set_up_ar_fcast_variance(fc, pmod, pmax, npsi, &phi, &psi, &errphi);
     }
 
     for (t=fc->t1; t<=fc->t2; t++) {
 	miss = 0;
 	yh = 0.0;
-	s = t - fc->offset;
 
 	if (t < p) {
-	    fc->yhat[s] = NADBL;
+	    fc->yhat[t] = NADBL;
 	    if (fc->sderr != NULL) {
-		fc->sderr[s] = NADBL;
+		fc->sderr[t] = NADBL;
 	    }
 	    continue;
 	}
 
         if (pmod->ci == PWE && t == pmod->t1) {
             /* PWE first obs is special */
-            fc->yhat[s] = pmod->yhat[t];
+            fc->yhat[t] = pmod->yhat[t];
             continue;
         }
 
 	if (phi != NULL) {
 	    /* initialize the phi's based on the AR error process
 	       alone */
-	    for (i=0; i<pmax; i++) {
+	    for (i=1; i<=pmax; i++) {
 		phi[i] = errphi[i];
 	    }
 	}
@@ -1371,7 +1359,7 @@ static int ar_fcast (Forecast *fc, MODEL *pmod,
 	    } else {
 		if (dvlag > 0 && phi != NULL) {
 		    /* augment phi for computation of variance */
-		    phi[dvlag - 1] += pmod->coeff[i];
+		    phi[dvlag] += pmod->coeff[i];
 		}
 		for (k=1; k<=arlist[0]; k++) {
 		    rk = pmod->arinfo->rho[k-1];
@@ -1390,20 +1378,18 @@ static int ar_fcast (Forecast *fc, MODEL *pmod,
 	}
 
 	if (miss) {
-	    fc->yhat[s] = NADBL;
+	    fc->yhat[t] = NADBL;
 	} else {
-	    fc->yhat[s] = yh;
+	    fc->yhat[t] = yh;
 	}
 
 	/* forecast error variance */
 	if (phi != NULL && pmod->ci != GARCH) {
 	    if (t > pmod->t2) {
-		arma_variance_machine(phi, pmax, NULL, 0,
-				      psi, t - pmod->t2, 
-				      &ss_psi);
-		fc->sderr[s] = pmod->sigma * sqrt(ss_psi);
+		vl = arma_variance(phi, pmax, NULL, 0, psi, npsi, t - pmod->t2); 
+		fc->sderr[t] = pmod->sigma * sqrt(vl);
 	    } else {
-		fc->sderr[s] = NADBL;
+		fc->sderr[t] = NADBL;
 	    }
 	}
     }
@@ -1468,7 +1454,7 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod,
     int yno = pmod->list[1];
     double lmax = NADBL;
     double xval;
-    int i, vi, t, s;
+    int i, vi, t;
 
     if (pmod->ci == POISSON) {
 	/* special for poisson "offset" variable */
@@ -1485,8 +1471,6 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod,
 	int miss = 0;
 	double yh = 0.0;
 
-	s = t - fc->offset;
-
 	for (i=0; i<pmod->ncoeff && !miss; i++) {
 	    int lag;
 
@@ -1504,12 +1488,12 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod,
 	}
 
 	if (miss) {
-	    fc->yhat[s] = NADBL;
+	    fc->yhat[t] = NADBL;
 	} else if (FCAST_SPECIAL(pmod->ci)) {
 	    /* special handling for LOGIT and others */
-	    fc->yhat[s] = fcast_transform(yh, pmod->ci, t, offvar, lmax);
+	    fc->yhat[t] = fcast_transform(yh, pmod->ci, t, offvar, lmax);
 	} else {
-	    fc->yhat[s] = yh;
+	    fc->yhat[t] = yh;
 	}
     }
 
@@ -1572,6 +1556,48 @@ static int get_forecast_method (Forecast *fc,
     return err;
 }
 
+static void forecast_init (Forecast *fc)
+{
+    fc->method = FC_AUTO;
+
+    fc->t1 = 0;
+    fc->t2 = 0;
+    fc->model_t2 = 0;
+
+    fc->yhat = NULL;
+    fc->sderr = NULL;
+    fc->eps = NULL;
+    fc->dvlags = NULL;
+}
+
+static void forecast_free (Forecast *fc)
+{
+    if (fc->dvlags != NULL) {
+	free(fc->dvlags);
+    }
+
+    if (fc->eps != NULL) {
+	free(fc->eps);
+    }
+} 
+
+static int fc_add_eps (Forecast *fc, int n)
+{
+    int t, err = 0;
+
+    fc->eps = malloc(n * sizeof *fc->eps);
+
+    if (fc->eps == NULL) {
+	err = E_ALLOC;
+    } else {
+	for (t=0; t<n; t++) {
+	    fc->eps[t] = 0.0;
+	}
+    }
+
+    return err;
+}
+
 /* driver for various functions that compute forecasts
    for different sorts of models */
 
@@ -1586,13 +1612,13 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
     int DM_errs = 0;
     int dyn_errs = 0;
     int nf = 0;
-    int s, t, err = 0;
+    int t, err = 0;
+
+    forecast_init(&fc);
 
     fc.t1 = fr->t1;
     fc.t2 = fr->t2;
-    fc.offset = fr->t1;
     fc.model_t2 = pmod->t2;
-    fc.eps = NULL;
 
     get_forecast_method(&fc, pmod, pdinfo, opt);
 
@@ -1629,15 +1655,11 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	return err;
     }
 
-    fc.yhat = fr->fitted + fr->pre_n;
-    if (fr->sderr != NULL) {
-	fc.sderr = fr->sderr + fr->pre_n;
-    } else {
-	fc.sderr = NULL;
-    }
+    fc.yhat = fr->fitted;
+    fc.sderr = fr->sderr;
 
     if (pmod->ci == ARMA && fc.method == FC_STATIC) {
-	fc.eps = malloc((fr->nobs - fr->pre_n) * sizeof *fc.eps);
+	fc_add_eps(&fc, pdinfo->n);
     }
 
     if (DM_errs) {
@@ -1654,12 +1676,7 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	err = linear_fcast(&fc, pmod, Z, pdinfo);
     }
 
-    if (fc.dvlags != NULL) {
-	free(fc.dvlags);
-    }
-    if (fc.eps != NULL) {
-	free(fc.eps);
-    }
+    forecast_free(&fc);
 
     if (dummy_AR) {
 	free(pmod->arinfo->arlist);
@@ -1667,21 +1684,20 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	pmod->arinfo = NULL;
     }
 
-    for (s=0; s<fr->nobs; s++) {
-	t = s + fr->t1 - fr->pre_n;
-	if (s < fr->pre_n) {
+    for (t=0; t<fr->nobs; t++) {
+	if (t < fr->t1) {
 	    if (t >= pmod->t1 && t <= pmod->t2) {
-		fr->fitted[s] = pmod->yhat[t];
+		fr->fitted[t] = pmod->yhat[t];
 	    } else {
-		fr->fitted[s] = NADBL;
+		fr->fitted[t] = NADBL;
 	    }
 	    if (fr->sderr != NULL) {
-		fr->sderr[s] = NADBL;
+		fr->sderr[t] = NADBL;
 	    }
-	} else if (!na(fr->fitted[s])) {
+	} else if (!na(fr->fitted[t])) {
 	    nf++;
 	}
-	fr->actual[s] = (*pZ)[yno][t];
+	fr->actual[t] = (*pZ)[yno][t];
     }
 
     if (nf == 0) {
@@ -1700,9 +1716,6 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	fr->df = pmod->dfd;
     }
 
-    /* reset t1 to include "pre" period if required */
-    fr->t1 -= fr->pre_n;
-
     return err;
 }
 
@@ -1736,9 +1749,9 @@ static int parse_forecast_string (const char *s, const MODEL *pmod,
 /**
  * get_forecast:
  * @pmod: the model from which forecasts are wanted.
+ * @t0: start of data display range.
  * @t1: start of forecast range.
  * @t2: end of forecast range.
- * @pre_n: number of pre-forecast observations to display
  * @pZ: pointer to data array using which @pmod was estimated.
  * @pdinfo: dataset information.
  * @opt: if OPT_D, force a dynamic forecast; if OPT_S, force
@@ -1765,7 +1778,7 @@ static int parse_forecast_string (const char *s, const MODEL *pmod,
  * a non-zero value indicates an error condition.
  */
 
-FITRESID *get_forecast (MODEL *pmod, int t1, int t2, int pre_n,
+FITRESID *get_forecast (MODEL *pmod, int t0, int t1, int t2,
 			double ***pZ, DATAINFO *pdinfo,
 			gretlopt opt) 
 {
@@ -1783,7 +1796,7 @@ FITRESID *get_forecast (MODEL *pmod, int t1, int t2, int pre_n,
 	return fr;
     }
 
-    fit_resid_init(t1, t2, pre_n, pmod, pdinfo, fr); 
+    fit_resid_init(t0, t1, t2, pmod, pdinfo, fr); 
     if (fr->err) {
 	return fr;
     }
@@ -1793,6 +1806,45 @@ FITRESID *get_forecast (MODEL *pmod, int t1, int t2, int pre_n,
     return fr;
 }
 
+static int organize_fcast_vars (const char *yhname, const char *sdname,
+				int *vi, int *vj, double ***pZ,
+				DATAINFO *pdinfo)
+{
+    int nadd = 0;
+    int err = 0;
+
+    *vi = *vj = 0;
+
+    if (check_varname(yhname)) {
+	return 1;
+    }
+
+    if (*sdname && check_varname(sdname)) {
+	return 1;
+    }
+
+    *vi = varindex(pdinfo, yhname);
+    if (*vi == pdinfo->v) {
+	nadd++;
+    }
+
+    if (*sdname) {
+	*vj = varindex(pdinfo, sdname);
+	if (*vj == pdinfo->v) {
+	    if (nadd > 0) {
+		*vj += 1;
+	    }
+	    nadd++;
+	}
+    }
+
+    if (nadd > 0) {
+	err = dataset_add_series(nadd, pZ, pdinfo);
+    }
+
+    return err;
+}
+
 /**
  * add_forecast:
  * @str: command string, giving a starting observation, ending
@@ -1821,10 +1873,11 @@ int add_forecast (const char *str, MODEL *pmod, double ***pZ,
 		  DATAINFO *pdinfo, gretlopt opt)
 {
     int oldv = pdinfo->v;
-    int t, t1, t2, vi;
-    char t1str[OBSLEN], t2str[OBSLEN], varname[VNAMELEN];
-    int nf = 0;
-    int err = 0;
+    int t, t1, t2, vi, vj;
+    char t1str[OBSLEN], t2str[OBSLEN];
+    char yhname[VNAMELEN];
+    char sdname[VNAMELEN];
+    int nf, err = 0;
 
     *t1str = '\0'; *t2str = '\0';
 
@@ -1834,9 +1887,14 @@ int add_forecast (const char *str, MODEL *pmod, double ***pZ,
 	return E_DATA;
     }
 
-    /* the varname should either be in the 2nd or 4th position */
-    if (sscanf(str, "%*s %8s %8s %8s", t1str, t2str, varname) != 3) {
-	if (sscanf(str, "%*s" "%8s", varname) != 1) {
+    *yhname = *sdname = 0;
+
+    /* the name for forecast values should either be in the 2nd or 4th
+       position; it may be followed by a name for the standard errors
+    */
+    nf = sscanf(str, "%*s %10s %10s %15s %15s", t1str, t2str, yhname, sdname);
+    if (nf < 3) {
+	if (sscanf(str, "%*s" "%15s %15s", yhname, sdname) < 1) {
 	    return E_PARSE;
 	}
     }
@@ -1852,44 +1910,50 @@ int add_forecast (const char *str, MODEL *pmod, double ***pZ,
 	t2 = pdinfo->t2;
     }
 
-    if (check_varname(varname)) {
-	return 1;
+    err = organize_fcast_vars(yhname, sdname, &vi, &vj, pZ, pdinfo);
+    if (err) {
+	return err;
     }
 
-    vi = varindex(pdinfo, varname);
-    if (vi == pdinfo->v) {
-	err = dataset_add_series(1, pZ, pdinfo);
-    }
+    nf = 0;
 
     if (!err) {
 	const double **Z = (const double **) *pZ;
 	Forecast fc;
 
-	strcpy(pdinfo->varname[vi], varname);
+	forecast_init(&fc);
+
+	strcpy(pdinfo->varname[vi], yhname);
 	strcpy(VARLABEL(pdinfo, vi), _("predicted values"));
 
-	for (t=0; t<pdinfo->n; t++) {
-	    (*pZ)[vi][t] = NADBL;
+	if (vj > 0) {
+	    strcpy(pdinfo->varname[vj], sdname);
+	    strcpy(VARLABEL(pdinfo, vj), _("forecast standard errors"));
 	}
 
 	fc.yhat = (*pZ)[vi];
-	fc.sderr = NULL;
-	fc.eps = NULL;
+	if (vj > 0) {
+	    fc.sderr = (*pZ)[vj];
+	} 
+
+	for (t=0; t<pdinfo->n; t++) {
+	    fc.yhat[t] = NADBL;
+	    if (fc.sderr != NULL) {
+		fc.sderr[t] = NADBL;
+	    }	
+	}
+
 	fc.t1 = t1;
 	fc.t2 = t2;
-	fc.offset = 0;
 	fc.model_t2 = pmod->t2;
 
 	get_forecast_method(&fc, pmod, pdinfo, opt);
 
 	if (pmod->ci == ARMA && fc.method == FC_STATIC) {
-	    fc.eps = malloc(pdinfo->n * sizeof *fc.eps);
+	    fc_add_eps(&fc, pdinfo->n);
 	}
 
-	/* write forecast values into the newly added variable: note
-	   that in this case we're not interested in computing
-	   forecast standard errors
-	*/
+	/* write forecast values into the newly added variable(s) */
 	if (pmod->ci == NLS) {
 	    nls_fcast(&fc, pmod, pZ, pdinfo);
 	} else if (SIMPLE_AR_MODEL(pmod->ci)) {
@@ -1902,17 +1966,12 @@ int add_forecast (const char *str, MODEL *pmod, double ***pZ,
 	    linear_fcast(&fc, pmod, Z, pdinfo);
 	}
 
-	if (fc.dvlags != NULL) {
-	    free(fc.dvlags);
-	}
-	if (fc.eps != NULL) {
-	    free(fc.eps);
-	}
-    }
+	forecast_free(&fc);
 
-    for (t=0; t<pdinfo->n; t++) {
-	if (!na((*pZ)[vi][t])) {
-	    nf++;
+	for (t=0; t<pdinfo->n; t++) {
+	    if (!na(fc.yhat[t])) {
+		nf++;
+	    }
 	}
     }
 
@@ -1958,7 +2017,7 @@ int display_forecast (const char *str, MODEL *pmod,
 	return err;
     }
 
-    fr = get_forecast(pmod, t1, t2, 0, pZ, pdinfo, opt);
+    fr = get_forecast(pmod, t1, t1, t2, pZ, pdinfo, opt);
 
     if (fr == NULL) {
 	return E_ALLOC;
@@ -1986,7 +2045,9 @@ fcast_get_t2max (const int *list, const int *dvlags, const MODEL *pmod,
     int i, vi, t;
 
     if (pmod->ci == ARMA && ftype == FC_STATIC) {
-	ay = Z[gretl_model_get_depvar(pmod)];
+	int yno = gretl_model_get_depvar(pmod);
+
+	ay = Z[yno];
     }
 
     for (t=pmod->t2; t<pdinfo->n; t++) {
@@ -2031,9 +2092,9 @@ fcast_get_t2max (const int *list, const int *dvlags, const MODEL *pmod,
  * @i: 0-based index for the variable to forecast, within
  * the VAR system (the dependent variable in the ith equation,
  * counting from 0).
- * @t1: start of range.
- * @t2: end of range.
- * @pre_n: number of pre-forecast observations to display.
+ * @t0: start of data display range.
+ * @t1: start of forecast range.
+ * @t2: end of forecast range.
  * @Z: data array using which @var was estimated.
  * @pdinfo: dataset information.
  * @opt: if OPT_D, force a dynamic forecast; if OPT_S, force
@@ -2043,15 +2104,15 @@ fcast_get_t2max (const int *list, const int *dvlags, const MODEL *pmod,
  *
  * Allocates a #FITRESID structure and fills it out with forecasts
  * based on @var, over the specified range of observations.  
- * The first @pre_n observations, starting at @t1, will be not
- * contain forecasts (FIXME).
+ * If @t0 is less than @t1, the observations prior to @t1 will
+ * not contain forecasts.
  * 
  * Returns: pointer to allocated structure, or %NULL on failure.
  * The %err member of the returned object should be checked:
  * a non-zero value indicates an error condition.
  */
 
-FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t1, int t2, int pre_n,
+FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t0, int t1, int t2,
 			    const double **Z, DATAINFO *pdinfo,
 			    gretlopt opt)
 {
@@ -2072,13 +2133,13 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t1, int t2, int pre_n,
 	}
     }
 
-    F = gretl_VAR_get_forecast_matrix(var, t1, t2, pre_n, Z, pdinfo, opt);
+    F = gretl_VAR_get_forecast_matrix(var, t0, t1, t2, Z, pdinfo, opt);
     if (F == NULL) {
 	fprintf(stderr, "gretl_VAR_get_forecast_matrix() gave NULL\n");
 	return NULL;
     }
 
-    fr = fit_resid_new(nf);
+    fr = fit_resid_new(pdinfo->n);
     if (fr == NULL) {
 	return NULL;
     }
@@ -2091,7 +2152,8 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t1, int t2, int pre_n,
     }
 
     fr->model_ci = var->ci;
-    fr->pre_n = pre_n;
+
+    fr->t0 = t0;
     fr->t1 = t1;
     fr->t2 = t2;
 
@@ -2106,15 +2168,15 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t1, int t2, int pre_n,
     m = var->neqns;
 
     nf = 0;
-    for (s=0; s<fr->nobs; s++) {
-	t = s + fr->t1;
-	fr->actual[s] = Z[yno][t];
-	fr->fitted[s] = gretl_matrix_get(F, s, i);
-	if (!na(fr->fitted[s])) {
+
+    for (t=fr->t0, s=0; t<=fr->t2; t++, s++) {
+	fr->actual[t] = Z[yno][t];
+	fr->fitted[t] = gretl_matrix_get(F, s, i);
+	if (!na(fr->fitted[t])) {
 	    nf++;
 	}
 	if (fr->sderr != NULL) {
-	    fr->sderr[s] = gretl_matrix_get(F, s, i + m);
+	    fr->sderr[t] = gretl_matrix_get(F, s, i + m);
 	}
     }
 
