@@ -20,12 +20,19 @@
 #ifndef GENERATE_H
 #define GENERATE_H
 
+enum {
+    SORT_ASCENDING,
+    SORT_DESCENDING
+};
+
 typedef struct _GENERATOR GENERATOR;
 
-int generate (const char *line, double ***pZ, DATAINFO *pdinfo, gretlopt opt); 
+int generate (const char *line, double ***pZ, DATAINFO *pdinfo, 
+	      gretlopt opt, PRN *prn); 
 
 GENERATOR *
-genr_compile (const char *line, double ***pZ, DATAINFO *pdinfo, gretlopt opt);
+genr_compile (const char *line, double ***pZ, DATAINFO *pdinfo, 
+	      gretlopt opt, PRN *prn);
 
 int execute_genr (GENERATOR *genr, int oldv);
 
@@ -52,5 +59,21 @@ int genr_function_from_string (const char *s);
 int get_t_from_obs_string (char *s, const double **Z, 
 			   const DATAINFO *pdinfo);
 
+/* genrfuncs.c, public functions */
+
+int dummy (double ***pZ, DATAINFO *pdinfo, int center);
+
+int panel_unit_first_obs (int t, const DATAINFO *pdinfo);
+
+int panel_unit_dummies (double ***pZ, DATAINFO *pdinfo);
+
+int paneldum (double ***pZ, DATAINFO *pdinfo);
+
+int plotvar (double ***pZ, DATAINFO *pdinfo);
+
+int plotvar_from_varname (double ***pZ, DATAINFO *pdinfo, const char *vname);
+
+int genrtime (double ***pZ, DATAINFO *pdinfo, int tm);
+
 #endif /* GENERATE_H */
 
