@@ -22,6 +22,9 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "genstack.h"
+
+#include <errno.h>
 
 static double hp_lambda (const DATAINFO *pdinfo)
 {
@@ -464,7 +467,7 @@ make_dummy_name_and_label (int vi, const DATAINFO *pdinfo, int center,
 
 int dummy (double ***pZ, DATAINFO *pdinfo, int center)
 {
-    char vname[USER_VLEN];
+    char vname[VNAMELEN];
     char vlabel[MAXLABEL];
     int vi, t, yy, pp, mm;
     int ndums, nnew = 0;
@@ -819,7 +822,7 @@ make_panel_time_var (double *x, const DATAINFO *pdinfo)
  * otherwise just an index of observations.
  *
  * Generates (and adds to the dataset, if it's not already
- * present) a time-trend or index variable.  The function
+ * present) a time-trend or index variable.  This function
  * is panel-data aware: if the dataset is a panel and
  * @tm is non-zero, the trend will not simply run
  * consecutively over the entire range of the data, but
@@ -862,11 +865,280 @@ int genrtime (double ***pZ, DATAINFO *pdinfo, int tm)
 
 static int plotvar_is_full_size (int v, int n, const double *x)
 {
-    int t, ret = 1;
+    int t;
 
     for (t=0; t<n; t++) {
 	if (na(x[t])) {
-	    ret = 0;
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
+typedef enum {
+    PLOTVAR_INDEX,
+    PLOTVAR_TIME,
+    PLOTVAR_ANNUAL,
+    PLOTVAR_QUARTERS,
+    PLOTVAR_MONTHS,
+    PLOTVAR_CALENDAR,
+    PLOTVAR_DECADES,
+    PLOTVAR_HOURLY,
+    PLOTVAR_MAX
+} plotvar_type; 
+
+struct plotvar {
+    int ptype;
+    char *pname;
+};
+
+static struct plotvar pvars[] = {
+    { PLOTVAR_INDEX,    "index" },
+    { PLOTVAR_TIME,     "time" },
+    { PLOTVAR_ANNUAL,   "annual" },
+    { PLOTVAR_QUARTERS, "qtrs" },
+    { PLOTVAR_MONTHS,   "months" },
+    { PLOTVAR_CALENDAR, "caldate" },
+    { PLOTVAR_DECADES,  "decades" },
+    { PLOTVAR_HOURLY,   "hrs" }
+};
+
+static int ptype_from_name (const char *pname)
+{
+    int i;
+
+    for (i=0; i<PLOTVAR_MAX; i++) {
+	if (!strcmp(pvars[i].pname, pname)) {
+	    return pvars[i].ptype;
+	}
+    }
+
+    return -1;
+}
+
+int plotvar_code (const DATAINFO *pdinfo)
+{
+    if (!dataset_is_time_series(pdinfo)) {
+	return PLOTVAR_INDEX;
+    } else if (pdinfo->pd == 1) {
+	return PLOTVAR_ANNUAL;
+    } else if (pdinfo->pd == 4) {
+	return PLOTVAR_QUARTERS;
+    } else if (pdinfo->pd == 12) {
+	return PLOTVAR_MONTHS;
+    } else if (pdinfo->pd == 24) {
+	return PLOTVAR_HOURLY;
+    } else if ((dated_daily_data(pdinfo) && pdinfo->n > 365) ||
+	    (dated_weekly_data(pdinfo) && pdinfo->n > 52)) {
+	return PLOTVAR_CALENDAR;
+    } else if (dataset_is_decennial(pdinfo)) {
+	return PLOTVAR_DECADES;
+    } else {
+	return PLOTVAR_TIME;
+    }
+}
+
+/**
+ * plotvar_from_varname:
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ * @vname: name for the plotting variable.
+ *
+ * Adds to the dataset a special dummy variable named @vname,
+ * for use on the x-axis in plotting, or simply retrieves the 
+ * ID number of the variable if it already exists.  
+ *
+ * Returns: the ID number of the variable (> 0) or -1 on failure.
+ */
+
+int plotvar_from_varname (double ***pZ, DATAINFO *pdinfo, const char *vname)
+{
+    int t, v, y1, n = pdinfo->n;
+    const char *pname;
+    int ptype;
+    float rm;
+
+    if (vname != NULL) {
+	pname = vname;
+	ptype = ptype_from_name(pname);
+	if (ptype < 0) {
+	    return -1;
+	}
+    } else {
+	ptype = plotvar_code(pdinfo);
+	pname = pvars[ptype].pname;
+    }
+
+    v = varindex(pdinfo, pname);
+
+    if (v < pdinfo->v) {
+	if (plotvar_is_full_size(v, pdinfo->n, (*pZ)[v])) {
+	    return v;
+	} 
+    } else if (dataset_add_series(1, pZ, pdinfo)) {
+	return -1;
+    }
+
+    strcpy(pdinfo->varname[v], pname);
+
+    y1 = (int) pdinfo->sd0;
+    rm = pdinfo->sd0 - y1;
+
+    switch (ptype) {
+    case PLOTVAR_ANNUAL: 
+	strcpy(VARLABEL(pdinfo, v), _("annual plotting variable")); 
+	for (t=0; t<n; t++) {
+	    (*pZ)[v][t] = (double) (t + atoi(pdinfo->stobs));
+	}
+	break;
+    case PLOTVAR_QUARTERS:
+	strcpy(VARLABEL(pdinfo, v), _("quarterly plotting variable"));
+	(*pZ)[v][0] = y1 + (10.0 * rm - 1.0) / 4.0;
+	for (t=1; t<n; t++) {
+	    (*pZ)[v][t] = (*pZ)[v][t-1] + .25;
+	}
+	break;
+    case PLOTVAR_MONTHS:
+	strcpy(VARLABEL(pdinfo, v), _("monthly plotting variable"));
+	(*pZ)[v][0] = y1 + (100.0 * rm - 1.0) / 12.0;
+	for (t=1; t<n; t++) {
+	    (*pZ)[v][t] = (*pZ)[v][t-1] + (1.0 / 12.0);
+	}
+	break;
+    case PLOTVAR_HOURLY:
+	strcpy(VARLABEL(pdinfo, v), _("hourly plotting variable"));
+	(*pZ)[v][0] = y1 + (100.0 * rm - 1.0) / 24.0;
+	for (t=1; t<n; t++) {
+	    (*pZ)[v][t] = (*pZ)[v][t-1] + (1.0 / 24.0);
+	}
+	break;
+    case PLOTVAR_CALENDAR:
+	strcpy(VARLABEL(pdinfo, v), _("daily plotting variable"));
+	for (t=0; t<n; t++) {
+	    if (pdinfo->S != NULL) {
+		(*pZ)[v][t] = get_dec_date(pdinfo->S[t]);
+	    } else {
+		char datestr[OBSLEN];
+		    
+		calendar_date_string(datestr, t, pdinfo);
+		(*pZ)[v][t] = get_dec_date(datestr);
+	    }
+	}
+	break;
+    case PLOTVAR_DECADES:
+	strcpy(pdinfo->varname[v], "time");
+	strcpy(VARLABEL(pdinfo, v), _("time trend variable"));
+	for (t=0; t<n; t++) {
+	    (*pZ)[v][t] = pdinfo->sd0 + 10 * t;
+	}
+	break;
+    case PLOTVAR_INDEX:
+	strcpy(VARLABEL(pdinfo, v), _("index variable"));
+	for (t=0; t<n; t++) {
+	    (*pZ)[v][t] = (double) (t + 1);
+	}
+	break;
+    case PLOTVAR_TIME:
+	strcpy(VARLABEL(pdinfo, v), _("time trend variable"));
+	for (t=0; t<n; t++) {
+	    (*pZ)[v][t] = (double) (t + 1);
+	}
+	break;
+    default:
+	break;
+    }
+
+    return v;
+}
+
+/**
+ * plotvar:
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ *
+ * Adds to the dataset a special dummy variable for use on the
+ * x-axis in plotting, or simple retrieves the ID number of the
+ * plotting variable if it already exists.  The name of the variable 
+ * is assigned automatically based on the characteristics of the 
+ * dataset.
+ *
+ * Returns: the ID number of the variable (> 0) or -1 on failure.
+ */
+
+int plotvar (double ***pZ, DATAINFO *pdinfo)
+{
+    return plotvar_from_varname(pZ, pdinfo, NULL);
+}
+
+static int varnames_from_arg (const char *s, char *v1str, char *v2str)
+{
+    int i, p, n = strlen(s);
+
+    if (n > 2 * VNAMELEN + 1) {
+	return 1;
+    }
+
+    p = haschar(',', s);
+    if (p < 0 || p > VNAMELEN - 1) {
+	return 1;
+    }
+
+    /* get first var name */
+    for (i=0; i<p; i++) {
+	v1str[i] = s[i];
+    }
+    v1str[p] = '\0';
+
+    /* get second var name */
+    n = n - p - 1;
+    for (i=0; i<n; i++) {
+	v2str[i] = s[p+1+i];
+    }
+    v2str[i] = '\0';
+
+    return 0;
+}
+
+double genr_cov_corr (const char *s, double ***pZ, 
+		      const DATAINFO *pdinfo, int fn)
+{
+    char v1str[VNAMELEN], v2str[VNAMELEN];
+    int v1, v2;
+    double ret = NADBL;
+
+    if (varnames_from_arg(s, v1str, v2str)) {
+	return NADBL;
+    }
+
+    v1 = varindex(pdinfo, v1str);
+    v2 = varindex(pdinfo, v2str);
+
+    if (v1 >= pdinfo->v || v2 >= pdinfo->v) {
+	return NADBL;
+    }
+
+    if (fn == T_COV) {
+	ret = gretl_covar(pdinfo->t1, pdinfo->t2, (*pZ)[v1], (*pZ)[v2]);
+    } else if (fn == T_CORR) {
+	ret = gretl_corr(pdinfo->t1, pdinfo->t2, (*pZ)[v1], (*pZ)[v2], NULL);
+    }
+
+    return ret;
+}
+
+static int 
+get_model_param_number (const MODEL *pmod, const char *vname)
+{
+    int i, ret = 0;
+
+    if (pmod->params == NULL) {
+	return 0;
+    }
+
+    for (i=0; i<=pmod->ncoeff; i++) {
+	if (!strcmp(vname, pmod->params[i])) {
+	    ret = i + 1;
 	    break;
 	}
     }
@@ -874,110 +1146,411 @@ static int plotvar_is_full_size (int v, int n, const double *x)
     return ret;
 }
 
-/**
- * plotvar:
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
- * @period: string to identify periodicity: "annual", "qtrs",
- * "months", "decdate" (calendar data), "time" or "index".
- *
- * Adds to the data set a special dummy variable for use in plotting.
- *
- * Returns: the ID number of the variable (> 0) or -1 on failure
- */
-
-int plotvar (double ***pZ, DATAINFO *pdinfo, const char *period)
+double genr_vcv (const char *s, const DATAINFO *pdinfo, MODEL *pmod)
 {
-    int t, vi, y1, n = pdinfo->n;
-    float rm;
+    int v1 = 0, v2 = 0;
+    int i, j, k, v1l, v2l;
+    char v1str[VNAMELEN], v2str[VNAMELEN];
+    int gotit;
+    double ret = NADBL;
 
-    vi = varindex(pdinfo, period);
-
-    if (vi < pdinfo->v) {
-	if (plotvar_is_full_size(vi, pdinfo->n, (*pZ)[vi])) {
-	    return vi;
-	} 
-    } else if (dataset_add_series(1, pZ, pdinfo)) {
-	return -1;
+    if (pmod == NULL || pmod->list == NULL) {
+	return NADBL;
     }
 
-    strcpy(pdinfo->varname[vi], period);
+    if (varnames_from_arg(s, v1str, v2str)) {
+	return NADBL;
+    }
 
-    y1 = (int) pdinfo->sd0;
-    rm = pdinfo->sd0 - y1;
+    /* are the varnames valid? */
+    if (pmod->ci != NLS && pmod->ci != ARMA) {
+	v1 = varindex(pdinfo, v1str);
+	v2 = varindex(pdinfo, v2str);
+	if (v1 >= pdinfo->v || v2 >= pdinfo->v) {
+	    return NADBL;
+	}
+    }
 
-    switch (period[0]) {
-    case 'a':
-	strcpy(VARLABEL(pdinfo, vi), _("annual plotting variable")); 
-	for (t=0; t<n; t++) {
-	    (*pZ)[vi][t] = (double) (t + atoi(pdinfo->stobs));
-	}
-	break;
-    case 'q':
-	strcpy(VARLABEL(pdinfo, vi), _("quarterly plotting variable"));
-	(*pZ)[vi][0] = y1 + (10.0 * rm - 1.0) / 4.0;
-	for (t=1; t<n; t++) {
-	    (*pZ)[vi][t] = (*pZ)[vi][t-1] + .25;
-	}
-	break;
-    case 'm':
-	strcpy(VARLABEL(pdinfo, vi), _("monthly plotting variable"));
-	(*pZ)[vi][0] = y1 + (100.0 * rm - 1.0) / 12.0;
-	for (t=1; t<n; t++) {
-	    (*pZ)[vi][t] = (*pZ)[vi][t-1] + (1.0 / 12.0);
-	}
-	break;
-    case 'h':
-	strcpy(VARLABEL(pdinfo, vi), _("hourly plotting variable"));
-	(*pZ)[vi][0] = y1 + (100.0 * rm - 1.0) / 24.0;
-	for (t=1; t<n; t++) {
-	    (*pZ)[vi][t] = (*pZ)[vi][t-1] + (1.0 / 24.0);
-	}
-	break;
-    case 'd':
-	if ((dated_daily_data(pdinfo) && pdinfo->n > 365) ||
-	    (dated_weekly_data(pdinfo) && pdinfo->n > 52)) {
-	    strcpy(VARLABEL(pdinfo, vi), _("daily plotting variable"));
-	    for (t=0; t<n; t++) {
-		if (pdinfo->S != NULL) {
-		    (*pZ)[vi][t] = get_dec_date(pdinfo->S[t]);
-		} else {
-		    char datestr[OBSLEN];
-		    
-		    calendar_date_string(datestr, t, pdinfo);
-		    (*pZ)[vi][t] = get_dec_date(datestr);
-		}
-	    } 
-	} else if (dataset_is_decennial(pdinfo)) {
-	    strcpy(pdinfo->varname[vi], "time");
-	    strcpy(VARLABEL(pdinfo, vi), _("time trend variable"));
-	    for (t=0; t<n; t++) {
-		(*pZ)[vi][t] = pdinfo->sd0 + 10 * t;
-	    }	    
-	} else {
-	    strcpy(pdinfo->varname[vi], "time");
-	    strcpy(VARLABEL(pdinfo, vi), _("time trend variable"));
-	    for (t=0; t<n; t++) {
-		(*pZ)[vi][t] = (double) (t + 1);
+    /* check model list */
+    if (pmod->ci == NLS || pmod->ci == ARMA) {
+	v1l = get_model_param_number(pmod, v1str);
+	v2l = get_model_param_number(pmod, v2str);
+    } else {
+	v1l = gretl_list_position(v1, pmod->list);
+	v2l = gretl_list_position(v2, pmod->list);
+    }
+
+    if (v1l == 0 || v2l == 0) {
+	return NADBL;
+    }
+
+    v1l -= 2;
+    v2l -= 2;
+
+    /* make model vcv matrix if need be */
+    if (pmod->vcv == NULL && makevcv(pmod)) {
+	return NADBL;
+    }
+
+    /* now find the right entry */
+    if (v1l > v2l) {
+	k = v1l;
+	v1l = v2l;
+	v2l = k;
+    }
+
+    gotit = 0;
+    k = 0;
+    for (i=0; i<pmod->ncoeff && !gotit; i++) {
+	for (j=0; j<pmod->ncoeff; j++) {
+	    if (j < i) {
+		continue;
 	    }
+	    if (i == v1l && j == v2l) {
+		ret = pmod->vcv[k];
+		gotit = 1;
+		break;
+	    }
+	    k++;
 	}
-	break; 
-    case 'i':
-	strcpy(VARLABEL(pdinfo, vi), _("index variable"));
-	for (t=0; t<n; t++) {
-	    (*pZ)[vi][t] = (double) (t + 1);
-	}
-	break;
-    case 't':
-	strcpy(VARLABEL(pdinfo, vi), _("time trend variable"));
-	for (t=0; t<n; t++) {
-	    (*pZ)[vi][t] = (double) (t + 1);
-	}
-	break;
-    default:
-	break;
     }
 
-    return vi;
+    return ret;
 }
 
+/**
+ * genr_fit_resid:
+ * @pmod: pointer to model to be tested.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @code: GENR_RESID or GENR_FITTED or GENR_RESID2.
+ * @undo: if non-zero, don't bother labeling the variables
+ *
+ * Adds residuals or fitted values or squared residuals from a
+ * given model to the data set.
+ * 
+ * Returns: 0 on successful completion, error code on error.
+ */
+
+int genr_fit_resid (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+		    int code, int undo)
+{
+    char vname[VNAMELEN], vlabel[MAXLABEL];
+    int i, t;
+    double *h = NULL;
+
+    if (code == GENR_H) {
+	h = gretl_model_get_data(pmod, "garch_h");
+	if (h == NULL) return E_DATA;
+    }
+
+    if (dataset_add_series(1, pZ, pdinfo)) {
+	return E_ALLOC;
+    }
+
+    i = pdinfo->v - 1;
+
+    for (t=0; t<pdinfo->n; t++) {
+	(*pZ)[i][t] = NADBL;
+    }
+
+    if (code == GENR_RESID) {
+	sprintf(vname, "uhat%d", pmod->ID);
+	sprintf(vlabel, _("residual from model %d"), pmod->ID);
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    (*pZ)[i][t] = pmod->uhat[t];
+	}
+    } else if (code == GENR_FITTED) {
+	sprintf(vname, "yhat%d", pmod->ID);
+	sprintf(vlabel, _("fitted value from model %d"), pmod->ID);
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    (*pZ)[i][t] = pmod->yhat[t];
+	}
+    } else if (code == GENR_RESID2) { 
+	/* squared residuals */
+	sprintf(vname, "usq%d", pmod->ID);
+	sprintf(vlabel, _("squared residual from model %d"), pmod->ID);
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if (na(pmod->uhat[t])) {
+		(*pZ)[i][t] = NADBL;
+	    } else {
+		(*pZ)[i][t] = pmod->uhat[t] * pmod->uhat[t];
+	    }
+	}
+    } else if (code == GENR_H) { 
+	/* garch variance */
+	sprintf(vname, "h%d", pmod->ID);
+	sprintf(vlabel, _("fitted variance from model %d"), pmod->ID);
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    (*pZ)[i][t] = h[t];
+	}
+    }
+
+    strcpy(pdinfo->varname[i], vname);
+
+    if (!undo) {
+	strcpy(VARLABEL(pdinfo, i), vlabel);
+    }
+
+    return 0;
+}
+
+int get_observation_number (const char *s, const DATAINFO *pdinfo)
+{
+    char test[OBSLEN];
+    size_t n;
+    int t;
+
+    *test = 0;
+    strncat(test, (*s == '"')? s + 1 : s, OBSLEN - 1);
+
+    n = strlen(test);
+    if (test[n-1] == '"') {
+	test[n-1] = '\0';
+    }
+
+    if (pdinfo->markers && pdinfo->S != NULL) {
+	for (t=0; t<pdinfo->n; t++) {
+	    if (!strcmp(test, pdinfo->S[t])) return t + 1;
+	}
+	if (calendar_data(pdinfo)) {
+	    charsub(test, ':', '/');
+	    for (t=0; t<pdinfo->n; t++) {
+		if (!strcmp(test, pdinfo->S[t]) ||
+		    !strcmp(test, pdinfo->S[t] + 2)) {
+		    return t + 1;
+		}
+	    }
+	}
+    }
+
+    if (pdinfo->structure == TIME_SERIES) {
+	t = dateton(test, pdinfo);
+	if (t >= 0) return t + 1;
+    }
+
+    if (calendar_data(pdinfo)) {
+	char datestr[OBSLEN];
+
+	charsub(test, ':', '/');
+	for (t=0; t<pdinfo->n; t++) {
+	    calendar_date_string(datestr, t, pdinfo);
+	    if (!strcmp(test, datestr) ||
+		!strcmp(test, datestr + 2)) {
+		return t + 1;
+	    }
+	}
+    }
+
+    return 0;
+}
+
+#define OBS_DEBUG 0
+
+static int plain_obs_number (const char *obs, const DATAINFO *pdinfo)
+{
+    char *test;
+    int t = -1;
+
+#if OBS_DEBUG
+    fprintf(stderr, "plain_obs_number: looking at '%s'\n", obs);
+#endif
+
+    errno = 0;
+
+    strtol(obs, &test, 10);
+
+    if (*test != '\0' || !strcmp(obs, test) || errno == ERANGE) {
+#if OBS_DEBUG
+	fprintf(stderr, "plain_obs_number: failed on '%s'\n", obs);
+#endif
+    } else {
+	t = atoi(obs) - 1; /* convert from 1-based to 0-based */ 
+    } 
+
+    if (t >= pdinfo->n) {
+	t = -1;
+    }
+
+    return t;
+}
+
+static void fix_calendar_date (char *s)
+{
+    while (*s) {
+	if (*s == ':') *s = '/';
+	s++;
+    }
+}
+
+/* Given what looks like an observation number or date within "[" and
+   "]", try to determine the observation number.  This is quite tricky
+   since we try to handle both dates and plain observation numbers
+   (and in addition, variables representing the latter); and we may
+   have to deal with the translation from 1-based indexing in user
+   space to 0-based indexing for internal purposes.
+*/
+
+int get_t_from_obs_string (char *s, const double **Z, 
+			   const DATAINFO *pdinfo)
+{
+    int t;
+
+#if OBS_DEBUG
+    fprintf(stderr, "\nget_t_from_obs_string: s ='%s'\n", s);
+#endif
+
+    if (calendar_data(pdinfo)) {
+	fix_calendar_date(s);
+    } 
+
+    t = dateton(s, pdinfo);
+
+#if OBS_DEBUG
+    fprintf(stderr, " dateton gives t = %d\n", t);
+#endif
+
+    if (t < 0) {
+	if (isdigit((unsigned char) *s)) {
+	    t = plain_obs_number(s, pdinfo);
+#if OBS_DEBUG
+	    fprintf(stderr, " plain_obs_number gives t = %d\n", t);
+#endif
+	} else {
+	    int v = varindex(pdinfo, s);
+
+	    if (v < pdinfo->v) {
+		t = (int) Z[v][0];
+#if OBS_DEBUG
+		fprintf(stderr, " based on var %d: t = %d\n", v, t);
+#endif
+		if (t > pdinfo->n) {
+		    char try[16];
+
+		    sprintf(try, "%d", t);
+		    t = dateton(try, pdinfo);
+#if OBS_DEBUG
+		    fprintf(stderr, " revised via dateton: t = %d\n", t);
+#endif
+		} else {
+		    /* convert to 0-based */
+		    t--;
+		}
+	    }
+	}
+    }
+
+    if (t < 0) {
+	sprintf(gretl_errmsg, _("Observation number out of bounds"));
+    }
+
+#if OBS_DEBUG
+    fprintf(stderr, " return value: t = %d\n", t);
+#endif
+
+    return t;
+}
+
+static int arma_model_stat_pos (const char *s, const MODEL *pmod)
+{
+    int p = -1;
+
+    if (numeric_string(s)) {
+	p = atoi(s) - 1;
+	if (p >= pmod->ncoeff) p = -1;
+	return p;
+    } else if (pmod->params != NULL) {
+	int i;
+
+	for (i=1; i<=pmod->ncoeff; i++) {
+	    if (!strcmp(s, pmod->params[i])) {
+		p = i - 1;
+		break;
+	    }
+	}
+    }
+
+    return p;
+}
+
+/* retrieve a specific element from one of the arrays of data
+   on a model */
+
+double 
+get_model_data_element (MODEL *pmod, int idx, const char *key,
+			const DATAINFO *pdinfo, int *err)
+{
+    char s[32] = {0};
+    const char *p;
+    int lv, vi = 0;
+    double x = NADBL;
+
+    /* extract arg string in parentheses */
+    p = strchr(key, '(');
+    if (p != NULL) {
+	sscanf(p + 1, "%31[^) ]", s);
+    } else {
+	strncat(s, key, 31);
+    }
+
+    DPRINTF(("get_model_data_element: looking at key = '%s'\n", s));
+
+    if (idx == M_RHO_S) {
+	if (!(numeric_string(s))) {
+	    *err = E_INVARG;
+	} else if (dot_atof(s) == 1 && AR1_MODEL(pmod->ci)) {
+	    x = gretl_model_get_double(pmod, "rho_in");
+	} else if (pmod->ci != AR && dot_atof(s) == 1) {
+	    x = pmod->rho;
+	} else if (pmod->arinfo == NULL || 
+		   pmod->arinfo->arlist == NULL || 
+		   pmod->arinfo->rho == NULL) {
+	    *err = E_INVARG;
+	} else if (!(vi = gretl_list_position(atoi(s), pmod->arinfo->arlist))) {
+	    *err = E_INVARG;
+	} else {
+	    x = pmod->arinfo->rho[vi-1];
+	}
+    } else if (idx == M_VCV_S) {
+	x = genr_vcv(s, pdinfo, pmod);
+	if (na(x)) {
+	    *err = E_INVARG;
+	}
+    } else if (idx == M_COEFF_S || idx == M_SE_S) {
+	if (pmod == NULL || pmod->list == NULL) {
+	    *err = E_INVARG;
+	} else if (pmod->ci == ARMA) {
+	    vi = arma_model_stat_pos(s, pmod);
+	    if (vi < 0) {
+		*err = E_INVARG;
+	    }
+	} else {
+	    lv = numeric_string(s)? atoi(s) : varindex(pdinfo, s);
+	    vi = gretl_list_position(lv, pmod->list);
+
+	    if (vi < 2) {
+		*err = E_INVARG;
+	    } else {
+		vi -= 2;
+	    }
+	}
+
+	if (!*err) {
+	    if (idx == M_COEFF_S && pmod->coeff != NULL) { 
+		x = pmod->coeff[vi];
+	    } else if (pmod->sderr != NULL) {
+		x = pmod->sderr[vi];
+	    } else {
+		*err = E_INVARG;
+	    }
+	}
+    } 
+
+    if (*err) {
+	gretl_errno = *err;
+    }
+
+    DPRINTF(("get_model_data_element: err = %d\n", *err));
+
+    return x;
+}
