@@ -32,7 +32,7 @@
 # include <windows.h>
 #else
 # ifdef USE_GTK2
-#  define GLIB2
+#  define USE_GSPAWN
 #  include <glib.h>
 #  include <signal.h>
 #  if HAVE_SYS_WAIT_H
@@ -101,9 +101,9 @@ struct plot_type_info ptinfo[] = {
     { PLOT_TYPE_MAX,       NULL }
 };
     
-#ifdef GLIB2
+#ifdef USE_GSPAWN
 
-#undef SPAWN_DEBUG
+# define SPAWN_DEBUG 0  /* define != 0 for debugging statements */
 
 /**
  * gnuplot_test_command:
@@ -143,11 +143,11 @@ int gnuplot_test_command (const char *cmd)
 				   &serr,
 				   &error);
 
-#ifdef SPAWN_DEBUG
+# if SPAWN_DEBUG
     fprintf(stderr, "Testing gnuplot command '%s'\n", cmd);
     fprintf(stderr, "ok=%d, child_pid=%d, sinp=%d\n",
 	    ok, child_pid, sinp);
-#endif
+# endif
 
     if (ok) {
 	int test, status;
@@ -157,7 +157,7 @@ int gnuplot_test_command (const char *cmd)
 	write(sinp, "\n", 1);
 	close(sinp);
 	test = waitpid(child_pid, &status, 0);
-# ifdef SPAWN_DEBUG
+# if SPAWN_DEBUG
 	fprintf(stderr, "waitpid returned %d, WIFEXITED %d, "
 		"WEXITSTATUS %d\n", test, WIFEXITED(status),
 		WEXITSTATUS(status));
@@ -178,9 +178,9 @@ int gnuplot_test_command (const char *cmd)
 	g_error_free(error);
     }
 
-#ifdef SPAWN_DEBUG
+# if SPAWN_DEBUG
     fprintf(stderr, "gnuplot test: ret = %d\n", ret);
-#endif
+# endif
 
     return ret;
 }
@@ -203,7 +203,7 @@ int gnuplot_test_command (const char *cmd)
     return err;
 }
 
-#endif /* ! GLIB2, ! WIN32 */
+#endif /* ! USE_GSPAWN, ! WIN32 */
 
 static int printvars (FILE *fp, int t, const int *list, const double **Z,
 		      const char *label, double offset)
@@ -234,37 +234,6 @@ static int printvars (FILE *fp, int t, const int *list, const double **Z,
     return miss;
 }
 
-/**
- * get_timevar_name:
- * @pdinfo: information on dataset.
- *
- * Based on the properties and frequency of the dataset, 
- * figure out the appropriate (internal) name for the variable 
- * to be used on the x-axis in a time-series plot.
- *
- * Returns: the name.
- */
-
-const char *get_timevar_name (DATAINFO *pdinfo)
-{
-    if (!dataset_is_time_series(pdinfo)) {
-	return "index";
-    } else if (pdinfo->pd == 1) {
-	return "annual";
-    } else if (pdinfo->pd == 4) {
-	return "qtrs";
-    } else if (pdinfo->pd == 12) {
-	return "months";
-    } else if (pdinfo->pd == 24) {
-	return "hrs";
-    } else if (calendar_data(pdinfo) || 
-	       dataset_is_decennial(pdinfo)) {
-	return "decdate";
-    } else {
-	return "time";
-    }
-}
-
 static int factorized_vars (struct gnuplot_info *gpinfo,
 			    const double **Z, int ynum, int dum)
 {
@@ -649,7 +618,7 @@ static int write_plot_type_string (PlotType ptype, FILE *fp)
 int gnuplot_init (PlotType ptype, FILE **fpp)
 {
     int gui = gretl_in_gui_mode();
-    char plotfile[MAXLEN] = {0};
+    char plotfile[FILENAME_MAX] = {0};
 
     if (gretl_looping()) {
 	return E_OK;
@@ -764,7 +733,7 @@ int gnuplot_make_graph (void)
     char plotcmd[MAXLEN];
 
 #ifdef ENABLE_NLS  
-    if (use_latin_2()) {
+    if (use_latin_2() && gnuplot_has_ttf()) {
 # if GP_DEBUG
 	fprintf(stderr, "gnuplot_make_graph: calling recode_gnuplot_file()\n");
 # endif
@@ -813,7 +782,7 @@ static int auto_plot_var (const char *s)
 static void make_gtitle (FILE *fp, int code, const char *s1, const char *s2)
 {
     char title[128];
-    char depvar[9];
+    char depvar[VNAMELEN];
 
     switch (code) {
     case GTITLE_VLS:
@@ -821,7 +790,7 @@ static void make_gtitle (FILE *fp, int code, const char *s1, const char *s2)
 		s1, s2);
 	break;
     case GTITLE_RESID:
-	if (sscanf(s1, "residual for %8s", depvar) == 1) {
+	if (sscanf(s1, "residual for %15s", depvar) == 1) {
 	    sprintf(title, I_("Regression residuals (= observed - fitted %s)"), 
 		    depvar);
 	}
@@ -893,7 +862,7 @@ static const char *series_name (const DATAINFO *pdinfo, int v)
     if (pdinfo->varinfo != NULL && pdinfo->varinfo[v] != NULL) {
 	ret = DISPLAYNAME(pdinfo, v);
 	if (ret[0] == '\0') {
-	   ret = pdinfo->varname[v];
+	    ret = pdinfo->varname[v];
 	}
     } 
 
@@ -915,7 +884,7 @@ get_gnuplot_output_file (FILE **fpp, gnuplot_flags flags,
 	    err = E_FOPEN;
 	}
     } else if ((flags & GP_BATCH) && plot_count != NULL) {  
-	char fname[MAXLEN];
+	char fname[FILENAME_MAX];
 
 	if (*plotfile == '\0' || strstr(plotfile, "gpttmp") != NULL) {
 	    *plot_count += 1; 
@@ -954,7 +923,7 @@ get_ols_line (struct gnuplot_info *gpinfo, const int *list,
     olslist[2] = 0;
     olslist[3] = list[2];
 	
-    plotmod = lsq(olslist, pZ, pdinfo, OLS, OPT_A, 0.0);
+    plotmod = lsq(olslist, pZ, pdinfo, OLS, OPT_A);
     err = plotmod.errcode;
 
     if (!err) {
@@ -1359,7 +1328,7 @@ int gnuplot (int *list, const int *lines, const char *literal,
     if (!strcmp(pdinfo->varname[list[gpinfo.lo]], "time")) {
 	int pv;
 
-	pv = plotvar(pZ, pdinfo, get_timevar_name(pdinfo));
+	pv = plotvar(pZ, pdinfo);
 	if (pv > 0) {
 	    list[gpinfo.lo] = pv;
 	} else {
@@ -1574,7 +1543,6 @@ int gnuplot (int *list, const int *lines, const char *literal,
 /**
  * multi_scatters:
  * @list: list of variables to plot, by ID number.
- * @pos: 
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
  * @plot_count: count of graphs shown to date.
@@ -1586,15 +1554,17 @@ int gnuplot (int *list, const int *lines, const char *literal,
  * Returns: 0 on successful completion, error code on error.
  */
 
-int multi_scatters (const int *list, int pos, double ***pZ, 
+int multi_scatters (const int *list, double ***pZ, 
 		    const DATAINFO *pdinfo, int *plot_count, 
 		    gnuplot_flags flags)
 {
     int i, t, err = 0, xvar, yvar;
     int *plotlist = NULL;
-    int nplots = 0;
+    int pos, nplots = 0;
     FILE *fp = NULL;
 
+    pos = gretl_list_separator_position(list);
+
     if (pos > 2) { 
 	/* plot several yvars against one xvar */
 	yvar = 0;
@@ -1745,7 +1715,7 @@ maybe_add_surface (const int *list, double ***pZ, DATAINFO *pdinfo,
     gretl_minmax(pdinfo->t1, pdinfo->t2, (*pZ)[list[2]], &umin, &umax);
     gretl_minmax(pdinfo->t1, pdinfo->t2, (*pZ)[list[1]], &vmin, &vmax);
 
-    smod = lsq(olslist, pZ, pdinfo, OLS, OPT_A, 0.0);
+    smod = lsq(olslist, pZ, pdinfo, OLS, OPT_A);
 
     if (!smod.errcode && !na(smod.fstt) &&
 	(fdist(smod.fstt, smod.dfn, smod.dfd) < .10 || flags & GP_FA)) {
@@ -2080,7 +2050,7 @@ int plot_freq (FreqDist *freq, DistCode dist)
     return gnuplot_make_graph();
 }
 
-int plot_fcast_errs (int n, const double *obs, 
+int plot_fcast_errs (int t1, int t2, const double *obs, 
 		     const double *depvar, const double *yhat, 
 		     const double *maxerr, const char *varname, 
 		     int time_series)
@@ -2089,28 +2059,30 @@ int plot_fcast_errs (int n, const double *obs,
     double xmin, xmax, xrange;
     int depvar_present = 0;
     int do_errs = (maxerr != NULL);
-    int t, err;
-
-    if ((err = gnuplot_init(PLOT_FORECAST, &fp))) {
-	return err;
-    }
+    int t, n, err;
 
     /* don't graph empty portion of forecast */
-    for (t=n-1; t>0; t--) {
+    for (t=t2; t>=t1; t--) {
 	if (na(depvar[t]) && na(yhat[t])) {
-	    n--;
+	    t2--;
 	} else {
 	    break;
 	}
     }
 
+    n = t2 - t1 + 1;
+
     if (n < 3) {
 	/* won't draw a graph for 2 datapoints or less */
 	return 1;
     }
 
+    if ((err = gnuplot_init(PLOT_FORECAST, &fp))) {
+	return err;
+    }    
+
     /* check that we have any values for the actual var */
-    for (t=0; t<n; t++) {
+    for (t=t1; t<=t2; t++) {
 	if (!na(depvar[t])) {
 	    depvar_present = 1;
 	    break;
@@ -2119,7 +2091,7 @@ int plot_fcast_errs (int n, const double *obs,
 
     fputs("# forecasts with 95 pc conf. interval\n", fp);
 
-    gretl_minmax(0, n - 1, obs, &xmin, &xmax);
+    gretl_minmax(t1, t2, obs, &xmin, &xmax);
     xrange = xmax - xmin;
     xmin -= xrange * .025;
     xmax += xrange * .025;
@@ -2155,7 +2127,7 @@ int plot_fcast_errs (int n, const double *obs,
     gretl_push_c_numeric_locale();
 
     if (depvar_present) {
-	for (t=0; t<n; t++) {
+	for (t=t1; t<=t2; t++) {
 	    if (na(depvar[t])) {
 		fprintf(fp, "%.8g ?\n", obs[t]);
 	    } else {
@@ -2165,7 +2137,7 @@ int plot_fcast_errs (int n, const double *obs,
 	fputs("e\n", fp);
     }
 
-    for (t=0; t<n; t++) {
+    for (t=t1; t<=t2; t++) {
 	if (na(yhat[t])) {
 	    fprintf(fp, "%.8g ?\n", obs[t]);
 	} else {
@@ -2175,7 +2147,7 @@ int plot_fcast_errs (int n, const double *obs,
     fputs("e\n", fp);
 
     if (do_errs) {
-	for (t=0; t<n; t++) {
+	for (t=t1; t<=t2; t++) {
 	    if (na(yhat[t]) || na(maxerr[t])) {
 		fprintf(fp, "%.8g ? ?\n", obs[t]);
 	    } else {
@@ -2209,7 +2181,7 @@ int garch_resid_plot (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 	return err;
     }
 
-    pv = plotvar(pZ, pdinfo, get_timevar_name(pdinfo));
+    pv = plotvar(pZ, pdinfo);
     if (pv > 0) {
 	obs = (*pZ)[pv];
     } else {
@@ -2545,8 +2517,8 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 	fprintf(fp, "set mxtics %s\n", spec->mxtics);
     }
 
-    /* using two y axes? */
     if (spec->flags & GPTSPEC_Y2AXIS) {
+	/* using two y axes */
 	fputs("set ytics nomirror\n", fp);
 	fputs("set y2tics\n", fp);
     } else if (spec->flags & GPTSPEC_MINIMAL_BORDER) {
@@ -2559,7 +2531,7 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
     }
 
     /* in case of plots that are editable (see gui client), it is
-       important to re-write the comment string that identifies the
+       important to write out the comment string that identifies the
        sort of graph, so that it will be recognized by type when
        it is redisplayed */
 
@@ -2751,7 +2723,8 @@ int go_gnuplot (GPT_SPEC *spec, char *fname)
 #ifdef WIN32
 	int winshow = 0;
 
-	if (fname == NULL) { /* sending plot to screen */
+	if (fname == NULL) { 
+	    /* sending plot to screen */
 	    fputs("pause -1\n", fp);
 	    winshow = 1;
 	} 
@@ -2780,9 +2753,9 @@ int go_gnuplot (GPT_SPEC *spec, char *fname)
 int 
 rmplot (const int *list, const double **Z, DATAINFO *pdinfo, PRN *prn)
 {
-    int err;
-    void *handle;
     int (*range_mean_graph) (int, const double **, const DATAINFO *, PRN *);
+    void *handle = NULL;
+    int err;
 
     range_mean_graph = get_plugin_function("range_mean_graph", &handle);
     if (range_mean_graph == NULL) {
@@ -2803,9 +2776,9 @@ rmplot (const int *list, const double **Z, DATAINFO *pdinfo, PRN *prn)
 int 
 hurstplot (const int *list, const double **Z, DATAINFO *pdinfo, PRN *prn)
 {
-    int err;
-    void *handle;
     int (*hurst_exponent) (int, const double **, const DATAINFO *, PRN *);
+    void *handle = NULL;
+    int err;
 
     hurst_exponent = get_plugin_function("hurst_exponent", &handle);
     if (hurst_exponent == NULL) {
@@ -3158,7 +3131,7 @@ int gretl_VAR_residual_plot (const GRETL_VAR *var,
 	}
     }
 
-    pv = plotvar(pZ, pdinfo, get_timevar_name(pdinfo));
+    pv = plotvar(pZ, pdinfo);
 	
     gretl_push_c_numeric_locale();
 
@@ -3186,9 +3159,9 @@ int gretl_VAR_roots_plot (GRETL_VAR *var)
 {
     const gretl_matrix *lam;
     FILE *fp = NULL;
-    int n;
     double x, y;
-    int i, err;
+    double px, py;
+    int i, n, err;
 
     lam = gretl_VAR_get_roots(var);
     if (lam == NULL) {
@@ -3215,13 +3188,16 @@ int gretl_VAR_roots_plot (GRETL_VAR *var)
     fputs("set polar\n", fp);
     fputs("plot 1 w lines , \\\n"
 	  "'-' w points pt 7\n", fp);
-    	
+
     gretl_push_c_numeric_locale();
     
     for (i=0; i<n; i++) {
         x = gretl_matrix_get(lam, i, 0);
         y = gretl_matrix_get(lam, i, 1);
-	fprintf(fp, "%.8f %.8f\n", x, y);
+	/* in polar form */
+	px = atan2(y, x);
+	py = sqrt(x * x + y * y);
+	fprintf(fp, "%.8f %.8f # %.4f,%.4f\n", px, py, x, y);
     }
 
     gretl_pop_c_numeric_locale();
@@ -3240,7 +3216,7 @@ int confidence_ellipse_plot (gretl_matrix *V, double *b, double t, double c,
     double xcoeff[2];
     double ycoeff[2];
     double *e = NULL;
-    int err;
+    int err = 0;
 
     maxerr[0] = t * sqrt(gretl_matrix_get(V, 0, 0));
     maxerr[1] = t * sqrt(gretl_matrix_get(V, 1, 1));
@@ -3250,9 +3226,9 @@ int confidence_ellipse_plot (gretl_matrix *V, double *b, double t, double c,
 	return err;
     }
 
-    e = gretl_symmetric_matrix_eigenvals(V, 1);
-    if (e == NULL) {
-	return E_ALLOC;
+    e = gretl_symmetric_matrix_eigenvals(V, 1, &err);
+    if (err) {
+	return err;
     }
 
     e[0] = sqrt(1.0 / e[0] * c);
