@@ -148,8 +148,6 @@ const char *get_gretl_png_term_line (PlotType ptype);
 
 const char *get_gretl_emf_term_line (PlotType ptype, int color);
 
-const char *get_timevar_name (DATAINFO *pdinfo);
-
 const char *gp_justification_string (int j);
 
 int gnuplot_init (PlotType ptype, FILE **fpp);
@@ -162,7 +160,7 @@ int gnuplot (int *list, const int *lines, const char *literal,
 	     double ***pZ, DATAINFO *pdinfo, 
 	     int *plot_count, gnuplot_flags flags);
 
-int multi_scatters (const int *list, int pos, 
+int multi_scatters (const int *list, 
 		    double ***pZ, const DATAINFO *pdinfo, 
 		    int *plot_count, gnuplot_flags flags);
 
@@ -191,7 +189,7 @@ int hurstplot (const int *list, const double **Z, DATAINFO *pdinfo,
 int 
 gretl_panel_ts_plot (const int *list, const double **Z, DATAINFO *pdinfo);
 
-int plot_fcast_errs (int n, const double *obs, 
+int plot_fcast_errs (int t1, int t2, const double *obs, 
 		     const double *depvar, const double *yhat, 
 		     const double *maxerr, const char *varname, 
 		     int time_series);
