@@ -76,6 +76,8 @@ const char *gretl_error_messages[] = {
     N_("The operation was canceled"),                            /* E_CANCEL */
     N_("Missing values encountered"),                            /* E_MISSDATA */
     N_("Not a Number in calculation"),                           /* E_NAN */
+    N_("Matrices not conformable for operation"),                /* E_NONCONF */
+    N_("Data types not conformable for operation"),              /* E_TYPES */
     NULL,                                                        /* E_DB_DUP */
     NULL,                                                        /* E_OK */
     NULL                                                         /* E_MAX */
@@ -144,19 +146,6 @@ const char *get_gretl_errmsg (void)
     return gretl_errmsg;
 }
 
-const char *get_gretl_msg (void)
-{
-    const char *ret = NULL;
-
-    if (gretl_messages_on()) {
-	if (*gretl_msg != '\0') {
-	    ret = gretl_msg;
-	}
-    }
-
-    return ret;
-}
-
 int print_gretl_errmsg (PRN *prn)
 {
     int ret = 0;
@@ -171,23 +160,9 @@ int print_gretl_errmsg (PRN *prn)
     return ret;
 }
 
-int print_gretl_msg (PRN *prn)
-{
-    int ret = 0;
-
-    if (gretl_messages_on()) {
-	if (*gretl_msg != '\0') {
-	    pprintf(prn, "%s\n", gretl_msg);
-	    ret = 1;
-	}
-    }
-
-    return ret;
-}
-
 /**
  * gretl_errmsg_set:
- * @str: an error message
+ * @str: an error message.
  *
  * If gretl_errmsg is currently blank, copy the given string into
  * the message space.
@@ -205,3 +180,4 @@ void gretl_errmsg_clear (void)
 {
     *gretl_errmsg = '\0';
 }
+
