@@ -68,9 +68,11 @@ enum gretl_error_codes {
     E_CANCEL,      /* 41 */
     E_MISSDATA,    /* 42 */
     E_NAN,         /* 43 */
-    E_DB_DUP,      /* 44 : duplicate vars found when saving to database */
-    E_OK,          /* 45 : not really an error */
-    E_MAX          /* 46 */
+    E_NONCONF,     /* 44 */
+    E_TYPES,       /* 45 */
+    E_DB_DUP,      /* 46 : duplicate vars found when saving to database */
+    E_OK,          /* 47 : not really an error */
+    E_MAX          /* 48 */
 }; 
 
 /* functions follow */
@@ -83,10 +85,6 @@ int get_gretl_errno (void);
 
 const char *get_gretl_errmsg (void);
 
-const char *get_gretl_msg (void);
-
-int print_gretl_msg (PRN *prn);
-
 int print_gretl_errmsg (PRN *prn);
 
 void gretl_errmsg_set (const char *str);
