@@ -20,6 +20,7 @@
 #include "libgretl.h"
 #include "gretl_func.h"
 #include "libset.h"
+#include "usermat.h"
 
 #define CALLSTACK_DEPTH 8
 
@@ -31,9 +32,11 @@ typedef struct fncall_ fncall;
 #define FN_NAMELEN 32
 
 enum {
-    ARG_SCALAR = 1,
+    ARG_NONE = 0,
+    ARG_SCALAR,
     ARG_SERIES,
-    ARG_LIST
+    ARG_LIST,
+    ARG_MATRIX
 };
 
 struct ufunc_ {
@@ -145,7 +148,7 @@ static void callstack_destroy (void)
     callstack = NULL;
 }
 
-static int real_gretl_function_stack_depth (void)
+int gretl_function_stack_depth (void)
 {
     int i, n = 0;
 
@@ -163,11 +166,6 @@ static int real_gretl_function_stack_depth (void)
     return n;
 }
 
-int gretl_function_stack_depth (void)
-{
-    return real_gretl_function_stack_depth();
-}
-
 static int push_fncall (fncall *call, const DATAINFO *pdinfo)
 {
     int i, nc;
@@ -194,42 +192,134 @@ static int push_fncall (fncall *call, const DATAINFO *pdinfo)
     return 0;
 }
 
-static int on_returns_list (const char *vname, char **returns, int n_returns)
-{
-    int k, ret = 0;
-
-    for (k=0; k<n_returns; k++) {
-	if (!strcmp(vname, returns[k])) {
-#if FN_DEBUG
-	    fprintf(stderr, "%s: this var is wanted\n", vname);
-#endif
-	    ret = 1;
-	    break;
-	}
-    }
-
-    return ret;
-}
-
 static void copy_values_to_assignee (int targ, int src, double **Z, 
 				     const DATAINFO *pdinfo)
 {
     int t, n = (pdinfo->vector[targ])? pdinfo->n : 1;
 
+#if FN_DEBUG
+    fprintf(stderr, "copy_values_to_assignee: copying from Z[%d] (%s) to Z[%d] (%s)\n",
+	    src, pdinfo->varname[src], targ, pdinfo->varname[targ]);
+#endif
+
     for (t=0; t<n; t++) {
 	Z[targ][t] = Z[src][t];
     }
 }
 
-static int 
-destroy_or_assign_local_vars (fncall *call, double ***pZ, DATAINFO *pdinfo, 
-			      int nc)
+static int new_var_assignment (fncall *call, double ***pZ, DATAINFO *pdinfo, 
+			       int nc, int *locals)
+{
+    int i, j, src, targ;
+    int err = 0;
+
+#if FN_DEBUG
+    fprintf(stderr, "\n*** start assignment of return values ***\n");
+#endif
+
+    if (call->asslist == NULL) {
+#if FN_DEBUG
+	fprintf(stderr, "function call: no assigments\n");
+#endif
+	return 0;
+    }
+
+#if FN_DEBUG
+    printlist(call->asslist, "call->asslist");
+    fprintf(stderr, "number of assignments = %d, number of returns = %d\n",
+	    call->asslist[0], call->fun->n_returns);
+#endif
+
+    if (call->fun->n_returns < call->asslist[0]) {
+	fprintf(stderr, "bad function call: assignments > returns\n");
+	return 1;
+    }
+
+    for (i=0; i<call->asslist[0]; i++) {
+	targ = call->asslist[i+1];
+	src = -1;
+
+#if FN_DEBUG
+	fprintf(stderr, " assv[%d] = '%s'\n", i, call->assv[i]);
+	fprintf(stderr, " return[%d] = '%s'\n", i, call->fun->returns[i]);
+#endif
+	if (call->assv[i] == NULL || *call->assv[i] == '\0') {
+	    fprintf(stderr, " got a null assignment\n");
+	    continue;
+	}
+
+	/* find the source variable */
+	for (j=1; j<pdinfo->v; j++) {
+	    if (STACK_LEVEL(pdinfo, j) != nc) {
+		continue;
+	    }
+	    if (!strcmp(pdinfo->varname[j], call->fun->returns[i])) {
+#if FN_DEBUG
+		fprintf(stderr, " identified source as var %d (%s)\n",
+			j, pdinfo->varname[j]);
+#endif
+		src = j;
+		break;
+	    }
+	}
+
+	if (src > 0) {
+	    if (targ > 0) {
+		/* copy values to pre-existing var at caller level */
+		copy_values_to_assignee(targ, src, *pZ, pdinfo);
+	    } else if (locals != NULL) {
+		/* rename variable as caller desired and mark as global */
+		int pos;
+
+		strcpy(pdinfo->varname[src], call->assv[i]);
+		STACK_LEVEL(pdinfo, src) -= 1; 
+		if ((pos = in_gretl_list(locals, src))) {
+		    gretl_list_delete_at_pos(locals, pos);
+		}
+	    }
+	} else {
+	    /* assigning a matrix, not a variable? */
+	    gretl_matrix *S = get_matrix_by_name(call->fun->returns[i],
+						 pdinfo);
+
+
+	    if (S != NULL) {
+		gretl_matrix *T;
+#if FN_DEBUG
+		fprintf(stderr, " identified source as matrix '%s' (%p)\n",
+			call->fun->returns[i], (void *) S);
+#endif
+		T = get_matrix_by_name_at_level(call->assv[i], nc - 1, pdinfo); 
+		if (T != NULL) {
+#if FN_DEBUG
+		    fprintf(stderr, " identified target as matrix '%s' (%p)\n",
+			    call->assv[i], (void *) T);
+#endif
+		    /* copy values to pre-existing matrix at caller level */
+		    err = gretl_matrix_copy_values(T, S);
+		} else {
+#if FN_DEBUG
+		    fprintf(stderr, " renaming matrix '%s' (%p) as '%s'\n",
+			    call->fun->returns[i], (void *) S, call->assv[i]);
+#endif
+		    /* rename matrix as caller desired and mark as global */
+		    err = user_matrix_set_name_and_level(S, call->assv[i], nc - 1);
+		}
+	    }
+	}
+    }
+
+#if FN_DEBUG
+    fprintf(stderr, "*** done assignment of returns ***\n\n");
+#endif
+
+    return err;
+}
+
+static int *make_locals_list (const DATAINFO *pdinfo, int nc, int *err)
 {
     int *locals = NULL;
-    int nlocal = 0;
-    int i, j, v;
-    int anyerr = 0;
-    int err = 0;
+    int i, nlocal = 0;
 
     for (i=1; i<pdinfo->v; i++) {
 	if (STACK_LEVEL(pdinfo, i) == nc) {
@@ -240,77 +330,86 @@ destroy_or_assign_local_vars (fncall *call, double ***pZ, DATAINFO *pdinfo,
     if (nlocal > 0) {
 	locals = gretl_list_new(nlocal);
 	if (locals == NULL) {
-	    err = E_ALLOC;
+	    *err = E_ALLOC;
 	} else {
-	    locals[0] = 0;
-	}
-    } 
+	    int j = 1;
 
-    if (nlocal == 0 || err) {
+	    for (i=1; i<pdinfo->v; i++) {
+		if (STACK_LEVEL(pdinfo, i) == nc) {
+		    locals[j++] = i;
+		}
+	    }
+	}
+    }
+
+    return locals;
+}
+
+static int fn_offers_matrix (fncall *call)
+{
+    int i;
+
+    for (i=0; i< call->fun->n_returns; i++) {
+	if (call->fun->rtype[i] == ARG_MATRIX) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+static int 
+destroy_or_assign_local_vars (fncall *call, double ***pZ, DATAINFO *pdinfo, 
+			      int nc)
+{
+    int *locals = NULL;
+    int anyerr = 0;
+    int err = 0;
+
+#if FN_DEBUG
+    fprintf(stderr, "destroy_or_assign_local_vars: depth = %d\n", nc);
+#endif
+
+    locals = make_locals_list(pdinfo, nc, &err);
+
+    if (err || (locals == NULL && !fn_offers_matrix(call))) {
+#if FN_DEBUG
+	fprintf(stderr, "locals = %p, err = %d, returning\n", 
+		(void *) locals, err);
+#endif
 	destroy_saved_lists_at_level(nc);
 	return err;
-    }
+    }    
 
-    v = 0;
-    j = 1;
+    err = new_var_assignment(call, pZ, pdinfo, nc, locals);
 
-    /* now check for any local variables generated within the
-       function: if they are on the return list and wanted by the
-       caller, process them appropriately, otherwise tag them for
-       deletion then delete them in a batch
-    */
-
-    for (i=1; i<pdinfo->v; i++) {
-	int cv, saveit = 0;
-
-	if (STACK_LEVEL(pdinfo, i) != nc) {
-	    continue;
-	}
-
-	if (v < call->asslist[0]) {
-	    saveit = on_returns_list(pdinfo->varname[i], call->fun->returns,
-				     call->fun->n_returns);
-	} 
-
-	if (saveit) {
-	    /* does a variable of this name already exist at caller level? */
-	    cv = call->asslist[v+1];
+    if (locals != NULL) {
+	anyerr = dataset_drop_listed_variables(locals, pZ, pdinfo, NULL);
+	if (anyerr && !err) {
+	    err = anyerr;
 #if FN_DEBUG
-	    fprintf(stderr, " wanted by caller as '%s' (#%d)\n", call->assv[v], cv);
+	    fprintf(stderr, "dataset_drop_listed_variables: err = %d\n", err);
 #endif
-	    if (cv > 0) {
-		copy_values_to_assignee(cv, i, *pZ, pdinfo);
-		saveit = 0;
-	    } else {
-		/* rename variable as caller desired: mark for preservation */
-		strcpy(pdinfo->varname[i], call->assv[v]);
-		STACK_LEVEL(pdinfo, i) -= 1; 
-	    }
-	    v++;
-	}
-
-	if (!saveit) {
-#if FN_DEBUG
-	    fprintf(stderr, "local variable %d (%s) tagged for deletion\n", 
-		    i, pdinfo->varname[i]);
-#endif
-	    locals[j++] = i;
-	    locals[0] += 1;
 	}
+	free(locals);
     }
 
-    anyerr = dataset_drop_listed_variables(locals, pZ, pdinfo, NULL);
-    if (anyerr && !err) {
-	err = anyerr;
-    }
-
-    free(locals);
-
     anyerr = destroy_saved_lists_at_level(nc);
     if (anyerr && !err) {
 	err = anyerr;
+#if FN_DEBUG
+	fprintf(stderr, "destroy_saved_lists_at_level(%d): err = %d\n", nc, err);
+#endif
     }
 
+    anyerr = destroy_user_matrices_at_level(nc);
+    if (anyerr && !err) {
+	err = anyerr;
+#if FN_DEBUG
+	fprintf(stderr, "destroy_user_matrices_at_level(%d): err = %d\n", nc, err);
+#endif
+    }    
+
 #if FN_DEBUG
     fprintf(stderr, "destroy_or_assign_local_vars: returning err = %d\n", err);
 #endif
@@ -318,7 +417,7 @@ destroy_or_assign_local_vars (fncall *call, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
-static int unstack_fncall (double ***pZ, DATAINFO *pdinfo)
+static int unstack_fncall (double ***pZ, DATAINFO **ppdinfo)
 {
     fncall *call;   
     int i, nc;
@@ -338,11 +437,11 @@ static int unstack_fncall (double ***pZ, DATAINFO *pdinfo)
 	    call->fun->name, nc);
 #endif
 
-    err = destroy_or_assign_local_vars(call, pZ, pdinfo, nc);
+    pop_program_state(pZ, ppdinfo);
+
+    err = destroy_or_assign_local_vars(call, pZ, *ppdinfo, nc);
 
     set_executing_off(call);
-    pop_program_state(pdinfo);
-
     free_fncall(call);
 
     for (i=0; i<nc; i++) {
@@ -531,7 +630,8 @@ static char *function_name_from_line (const char *line, char *name)
 	    *q = 0;
 	}
 #if FN_DEBUG
-	fprintf(stderr, "function_name_from_line: got '%s'\n", name);
+	fprintf(stderr, "function_name_from_line: line='%s', got '%s'\n", 
+		line, name);
 #endif
     }
 
@@ -557,6 +657,41 @@ int gretl_is_user_function (const char *line)
     return ret;
 }
 
+int is_user_matrix_function (const char *word)
+{
+    ufunc *fun = get_ufunc_by_name(word);
+    if (fun != NULL) {
+	if (fun->n_returns == 1 && fun->rtype[0] == ARG_MATRIX) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+int gretl_get_user_function (const char *line, char **fnname)
+{
+    int ret = 0;
+
+    if (n_ufuns > 0 && !string_is_blank(line)) {
+	char name[FN_NAMELEN];
+
+#if FN_DEBUG > 1
+	fprintf(stderr, "gretl_is_user_function: testing '%s'\n", line);
+#endif
+	function_name_from_line(line, name);
+	if (get_ufunc_by_name(name) != NULL) {
+	    free(*fnname);
+	    *fnname = gretl_strdup(name);
+	    if (*fnname != NULL) {
+		ret = 1;
+	    }
+	}
+    }
+
+    return ret;
+}
+
 static void delete_ufunc_from_list (ufunc *fun)
 {
     if (n_ufuns == 0 || fun == NULL) {
@@ -587,7 +722,7 @@ static void delete_ufunc_from_list (ufunc *fun)
     }
 }
 
-static int check_func_name (const char *fname)
+static int check_func_name (const char *fname, PRN *prn)
 {
     int i, err = 0;
 
@@ -599,84 +734,187 @@ static int check_func_name (const char *fname)
 		fname);
 	err = 1;
     } else {
-	for (i=0; i<n_ufuns && !err; i++) {
-	    if (!strcmp(fname, (ufuns[i])->name)) {
-		sprintf(gretl_errmsg, "'%s': function is already defined",
-		    fname);
-		err = 1;
+	for (i=0; i<n_ufuns; i++) {
+	    if (!strcmp(fname, ufuns[i]->name)) {
+		pprintf(prn, "Redefining function '%s'\n", fname);
+		delete_ufunc_from_list(ufuns[i]);
+		break;
 	    }
 	}
     }
-
+    
     return err;
 }
 
+/* count fields that may be separated by spaces or commas,
+   including blank comma-delimited fields */
+
+static int special_count_fields (char *s)
+{
+    int n, nf = 0;
+
+    while (*s) {
+	while (*s == ' ') s++;
+	n = strcspn(s, " ,");
+	if (n > 0) {
+	    nf++;
+	    s += n;
+	    while (*s == ' ') s++;
+	    if (*s == ',') s++;
+	} else if (*s == ',') {
+	    nf++;
+	    s++;
+	}
+	while (*s == ' ') s++;
+    }
+
+    return nf;
+}
+
+static char *get_next_arg (char **parg, char *s, int maxlen)
+{
+    int m, n;
+
+    while (*s == ' ') s++;
+
+    n = strcspn(s, " ,");
+    
+    if (n > 0) {
+	m = (n > maxlen)? maxlen : n;
+	*parg = gretl_strndup(s, m);
+	s += n;
+	while (*s == ' ') s++;
+	if (*s == ',') s++;
+    } else if (*s == ',') {
+	*parg = gretl_strdup("");
+	s++;
+    }
+
+    while (*s == ' ') s++;
+
+    return s;
+}
+
+static int is_type_name (const char *s)
+{
+    int ret = ARG_NONE;
+
+    if (!strcmp(s, "scalar")) {
+	ret = ARG_SCALAR;
+    } else if (!strcmp(s, "series")) {
+	ret = ARG_SERIES;
+    } else if (!strcmp(s, "matrix")) {
+	ret = ARG_MATRIX;
+    }
+
+    return ret;
+}
+
 /* Parse line and return an allocated array of strings consisting of
    the space- or comma-separated fields in line, each one truncated if
    necessary to a maximum of maxlen characters.
 */ 
 
 static char **
-get_separated_fields (const char *line, int *nfields, int maxlen)
+get_separated_fields (const char *line, int *nfields, int maxlen, 
+		      char **passtypes, int *err)
 {
-    char fmt[8];
     char **fields = NULL;
+    char *asstypes = NULL;
     char *cpy, *s;
-    char str[9];
-    int i, j, nf, err = 0;
+    int realnf;
+    int i, n, nf;
 
     *nfields = 0;
 
     if (string_is_blank(line)) {
+	*err = E_PARSE;
 	return NULL;
     }
 
     cpy = gretl_strdup(line);
     if (cpy == NULL) {
+	*err = E_ALLOC;
 	return NULL;
     }
 
-    /* clean the string up */
+    /* clean up the copied string */
     s = cpy;
     s += strspn(s, " ");
     if (*s == '(') {
 	s++;
     }
     tailstrip(s);
-    j = strlen(s);
-    if (s[j-1] == ')') {
-	s[j-1] = '\0';
-    }    
+    n = strlen(s);
+    if (s[n-1] == ')') {
+	s[n-1] = '\0';
+    } 
 
-    charsub(s, ',', ' ');
+    realnf = nf = special_count_fields(s);
 
-    nf = count_fields(s);
-    fields = malloc(nf * sizeof *fields);
-    if (fields == NULL) {
-	free(s);
-	return NULL;
-    }
-
-    sprintf(fmt, "%%%ds", maxlen);
-
-    for (i=0; i<nf && !err; i++) {
-	sscanf(s, fmt, str);
-	fields[i] = gretl_strdup(str);
-	if (fields[i] == NULL) {
-	    for (j=0; j<i; j++) {
-		free(fields[j]);
+    if (nf > 0) {
+	if (passtypes != NULL) {
+	    asstypes = calloc(nf, 1);
+	    if (asstypes == NULL) {
+		*err = E_ALLOC;
 	    }
-	    free(fields);
-	    fields = NULL;
-	    err = 1;
-	} else {
-	    s += strcspn(s, " ");
-	    s += strspn(s, " ");
+	}
+	if (!*err) {
+	    fields = create_strings_array(nf);
+	    if (fields == NULL) {
+		*err = E_ALLOC;
+	    }
+	}
+	if (!*err) {
+	    int atype = 0;
+	    int j = 0;
+
+	    for (i=0; i<nf && !*err; i++) {
+		if (asstypes != NULL && atype) {
+		    asstypes[j] = atype;
+		}
+		s = get_next_arg(&fields[j], s, maxlen);
+		if (fields[j] == NULL) {
+		    *err = E_ALLOC;
+		} else if ((atype = is_type_name(fields[j]))) {
+		    if (asstypes == NULL || i == nf - 1) {
+			*err = E_PARSE;
+		    } else {
+			free(fields[j]);
+			fields[j] = NULL;
+		    } 
+		} else {
+		    j++;
+		}
+	    }
+	    realnf = j;
 	}
     }
 
-    if (!err) {
+    if (!*err && realnf < nf) {
+	if (realnf == 0) {
+	    *err = E_PARSE;
+	} else {
+	    char **realf = realloc(fields, realnf * sizeof *fields);
+
+	    if (realf == NULL) {
+		*err = E_ALLOC;
+	    } else {
+		fields = realf;
+		nf = realnf;
+	    }
+	}
+    }
+
+    if (*err) {
+	free_strings_array(fields, nf);
+	fields = NULL;
+	free(asstypes);
+    } else {
 	*nfields = nf;
+	if (passtypes != NULL) {
+	    *passtypes = asstypes;
+	}
     }
 
     free(cpy);
@@ -686,7 +924,7 @@ get_separated_fields (const char *line, int *nfields, int maxlen)
 
 static int 
 parse_function_args_etc (const char *s, int *argc, char ***pargv,
-			 int **asslist, char ***passv)
+			 int **asslist, char ***passv, char **passtypes)
 {
     char **argv = NULL;
     char **assign = NULL;
@@ -701,16 +939,18 @@ parse_function_args_etc (const char *s, int *argc, char ***pargv,
 	char *astr = gretl_strndup(s, p - s);
 
 	/* seems we have a left-hand side assignment */
-	assign = get_separated_fields(astr, &na, 8);
-	if (assign == NULL) {
-	    err = E_ALLOC;
-	} 
+	assign = get_separated_fields(astr, &na, VNAMELEN - 1, 
+				      passtypes, &err);
 	free(astr);
 	s = p + 1;
     } else {
 	s++;
     }
 
+    if (err) {
+	return err;
+    }
+
     if (na > 0) {
 	*asslist = gretl_list_new(na);
     } else {
@@ -742,10 +982,8 @@ parse_function_args_etc (const char *s, int *argc, char ***pargv,
 #if FN_DEBUG
 	    fprintf(stderr, "function_args: looking at '%s'\n", s);
 #endif
-	    argv = get_separated_fields(s, &na, 32);
-	    if (argv == NULL) {
-		err = 1;
-	    } else {
+	    argv = get_separated_fields(s, &na, 32, NULL, &err);
+	    if (!err) {
 		*pargv = argv;
 		*argc = na;
 	    }
@@ -782,6 +1020,8 @@ static int get_param_type (const char *s)
 	ret = ARG_SERIES;
     } else if (!strcmp(s, "list")) {
 	ret = ARG_LIST;
+    } else if (!strcmp(s, "matrix")) {
+	ret = ARG_MATRIX;
     }
 
     return ret;
@@ -896,7 +1136,8 @@ static int parse_fn_element (char *s, char **parmv, char *ptype, int i,
 
 static int 
 parse_fn_definition_or_returns (char *fname, char ***pparmv, int *pnp,
-				char **pptype, const char *str)
+				char **pptype, const char *str,
+				PRN *prn)
 {
     char **parmv = NULL;
     char *ptype = NULL;
@@ -928,7 +1169,7 @@ parse_fn_definition_or_returns (char *fname, char ***pparmv, int *pnp,
 	    }
 	}
 	if (!err) {
-	    err = check_func_name(fname);
+	    err = check_func_name(fname, prn);
 	}
 	if (!err) {
 	    str += len;
@@ -982,7 +1223,7 @@ parse_fn_definition_or_returns (char *fname, char ***pparmv, int *pnp,
     return err;
 }
 
-int gretl_start_compiling_function (const char *line)
+int gretl_start_compiling_function (const char *line, PRN *prn)
 {
     char **params = NULL;
     char *ptype = NULL;
@@ -1007,7 +1248,7 @@ int gretl_start_compiling_function (const char *line)
     }
 
     err = parse_fn_definition_or_returns(fname, &params, &n_params, 
-					 &ptype, line + 8);
+					 &ptype, line + 8, prn);
 
     if (!err) {
 	fun = add_ufunc();
@@ -1051,7 +1292,8 @@ static int create_function_return_list (ufunc *fun, const char *line)
     err = parse_fn_definition_or_returns(NULL, &fun->returns, 
 					 &fun->n_returns, 
 					 &fun->rtype, 
-					 line);
+					 line,
+					 NULL);
 
     return err;
 }
@@ -1225,19 +1467,25 @@ static int localize_list (const char *oldname, const char *newname,
     int *orig = NULL;
     int *new = NULL;
     int origv = pdinfo->v;
+    int orig0 = 0;
     int i, v, match, err = 0;
 
-    orig = get_list_by_name(oldname);
-    if (orig == NULL) {
-	return 1;
+    if (!strcmp(oldname, "null")) {
+	new = gretl_null_list();
+    } else {
+	orig = get_list_by_name(oldname);
+	if (orig == NULL) {
+	    return 1;
+	}
+	new = gretl_list_copy(orig);
+	orig0 = orig[0];
     }
 
-    new = gretl_list_copy(orig);
     if (new == NULL) {
 	return E_ALLOC;
     }
 
-    for (i=1; i<=orig[0] && !err; i++) {
+    for (i=1; i<=orig0 && !err; i++) {
 	v = orig[i];
 	if (v == pdinfo->v) {
 	    /* unknown variable */
@@ -1274,7 +1522,8 @@ static int check_and_allocate_function_args (ufunc *fun,
 					     double ***pZ,
 					     DATAINFO *pdinfo)
 {
-    int *outlist = NULL, *inlist = NULL;
+    int *outlist = NULL;
+    int *inlist = NULL;
     int i, v, err = 0;
 
     if (argc != fun->n_params) {
@@ -1287,8 +1536,8 @@ static int check_and_allocate_function_args (ufunc *fun,
 
     for (i=0; i<argc && !err; i++) {
 #if FN_DEBUG
-	fprintf(stderr, "fn args: argv[%d]='%s', parm[%d]='%s' ptype[%d]=%d\n", 
-		i, argv[i], i, fun->params[i], i, fun->ptype[i]);
+	fprintf(stderr, "fn argv[%d]: arg='%s', fun->param='%s' fun->ptype=%d\n", 
+		i, argv[i], fun->params[i], fun->ptype[i]);
 #endif
 	if (fun->ptype[i] == ARG_SCALAR) {
 	    if (numeric_string(argv[i])) {
@@ -1321,7 +1570,7 @@ static int check_and_allocate_function_args (ufunc *fun,
 		err = 1;
 	    } 
 	} else if (fun->ptype[i] == ARG_LIST) {
-	    if (get_list_by_name(argv[i]) != NULL) {
+	    if (get_list_by_name(argv[i]) != NULL || !strcmp(argv[i], "null")) {
 #ifdef LOCAL_COPY_LISTS
 		err = localize_list(argv[i], fun->params[i], pZ, pdinfo,
 				    &outlist, &inlist);
@@ -1332,20 +1581,32 @@ static int check_and_allocate_function_args (ufunc *fun,
 		sprintf(gretl_errmsg, "argument %d (%s): not a list", i+1, argv[i]);
 		err = 1;
 	    }
+	} else if (fun->ptype[i] == ARG_MATRIX) {
+	    if (get_matrix_by_name(argv[i], pdinfo) != NULL) {
+		err = copy_named_matrix_as(argv[i], fun->params[i]);
+#if FN_DEBUG
+		fprintf(stderr, "done copy_named_matrix_as, '%s' -> '%s', err = %d\n", 
+			argv[i], fun->params[i], err);
+#endif
+	    }
 	} else {
 	    /* impossible */
 	    err = 1;
 	}
     }
 
-    free(outlist);
-    free(inlist);
+    if (inlist != NULL) {
+	free(inlist);
+    }
+    if (outlist != NULL) {
+	free(outlist);
+    }
 
     return err;
 }
 
-static int check_function_assignments (ufunc *fun,
-				       int *asslist, char **assv, 
+static int check_function_assignments (ufunc *fun, int *asslist, 
+				       char **assv, char *asstypes,
 				       DATAINFO *pdinfo)
 {
     int i, v, err = 0;
@@ -1361,6 +1622,10 @@ static int check_function_assignments (ufunc *fun,
 #if FN_DEBUG
 	fprintf(stderr, "checking assignment %d to '%s'\n", i, assv[i]);
 #endif
+	if (assv[i] == NULL || *assv[i] == '\0') {
+	    /* placeholder, non-assignment */
+	    continue;
+	}
 	v = varindex(pdinfo, assv[i]);
 	if (v < pdinfo->v) {
 #if FN_DEBUG
@@ -1371,52 +1636,66 @@ static int check_function_assignments (ufunc *fun,
 		(fun->rtype[i] == ARG_SERIES && !pdinfo->vector[v])) {
 		sprintf(gretl_errmsg, "%s: wrong type for assignment", assv[i]);
 		err = 1;
+	    } else if (fun->rtype[i] == ARG_MATRIX) {
+		sprintf(gretl_errmsg, "%s: wrong type for assignment", assv[i]);
+		err = 1;
 	    } else {
 		asslist[i+1] = v;
 	    }
 	} else if (fun->rtype[i] == ARG_LIST) {
-	    /* FIXME? */
 	    fprintf(stderr, "requested return of list as '%s'\n", assv[i]);
+	    /* FIXME? */
 	} else {
-	    err = check_varname(assv[i]);
+	    /* new variable */
+	    if (asstypes != NULL && asstypes[i] && asstypes[i] != fun->rtype[i]) {
+		sprintf(gretl_errmsg, "%s: wrong type for assignment", assv[i]);
+		err = 1;
+	    }
+	    if (!err) {
+		err = check_varname(assv[i]);
+	    }
 	}
     }
 
     return err;
 }
 
-int gretl_function_start_exec (const char *line, double ***pZ,
-			       DATAINFO *pdinfo)
+int gretl_function_start_exec (const char *line, const char *fname, 
+			       double ***pZ, DATAINFO *pdinfo)
 {
     char **argv = NULL;
     char **assv = NULL;
+    char *asstypes = NULL;
     int *asslist = NULL;
-    char fname[FN_NAMELEN];
     ufunc *fun;
     fncall *call = NULL;
     int argc = 0;
     int err = 0;
 
-    function_name_from_line(line, fname);
+#if FN_DEBUG
+    fprintf(stderr, "gretl_function_start_exec: line='%s'\n"
+	    " fname='%s'\n", line, fname);
+#endif
+
     fun = get_ufunc_by_name(fname);
 
     if (fun == NULL) {
 	return 1;
     }
 
-    err = parse_function_args_etc(line, &argc, &argv, &asslist, &assv);
+    err = parse_function_args_etc(line, &argc, &argv, &asslist, &assv, &asstypes);
     if (err) {
-	return E_ALLOC;
+	return err;
     }
 
-    if (argc > 0) {
-	err = check_and_allocate_function_args(fun, argc, argv, pZ, pdinfo);
-    }
+    err = check_and_allocate_function_args(fun, argc, argv, pZ, pdinfo);
     
     if (!err && asslist[0] > 0) {
-	err = check_function_assignments(fun, asslist, assv, pdinfo);
+	err = check_function_assignments(fun, asslist, assv, asstypes, pdinfo);
     }
 
+    free(asstypes);
+
     if (err) {
 	free_strings_array(argv, argc);
 	free_strings_array(assv, asslist[0]);
@@ -1434,11 +1713,15 @@ int gretl_function_start_exec (const char *line, double ***pZ,
 	free_fncall(call);
     }
 
+#if FN_DEBUG
+    fprintf(stderr, "gretl_function_start_exec: returning err = %d\n", err);
+#endif
+
     return err;
 }
 
 char *gretl_function_get_line (char *line, int len,
-			       double ***pZ, DATAINFO *pdinfo,
+			       double ***pZ, DATAINFO **ppdinfo,
 			       int *err)
 {
     fncall *call = current_call();
@@ -1471,7 +1754,7 @@ char *gretl_function_get_line (char *line, int len,
     } 
 
     if (unstack) {
-	*err = unstack_fncall(pZ, pdinfo);
+	*err = unstack_fncall(pZ, ppdinfo);
 	return "";
     }	
 
@@ -1487,14 +1770,14 @@ void gretl_functions_cleanup (void)
     ufuncs_destroy();
 }
 
-void gretl_function_stop_on_error (DATAINFO *pdinfo, PRN *prn)
+void gretl_function_stop_on_error (double ***pZ, DATAINFO **ppdinfo, PRN *prn)
 {
     gretl_function_flagged_error(NULL, prn);
 
     callstack_destroy();
 
     if (fn_executing > 0) {
-	libset_restore_state_zero(pdinfo);
+	libset_restore_state_zero(pZ, ppdinfo);
     }
 
     fn_executing = 0;
