@@ -27,8 +27,8 @@
 typedef struct saved_list_ saved_list;
 
 struct saved_list_ {
+    char name[VNAMELEN];
     int *list;
-    char *name;
     int level;
 };
 
@@ -54,12 +54,8 @@ static saved_list *saved_list_new (const int *list, const char *name)
 	    free(sl);
 	    sl = NULL;
 	} else {
-	    sl->name = gretl_strndup(name, 31);
-	    if (sl->name == NULL) {
-		free(sl->list);
-		free(sl);
-		sl = NULL;
-	    }
+	    *sl->name = 0;
+	    strncat(sl->name, name, VNAMELEN - 1);
 	}
     }
 
@@ -69,7 +65,6 @@ static saved_list *saved_list_new (const int *list, const char *name)
 static void free_saved_list (saved_list *sl)
 {
     free(sl->list);
-    free(sl->name);
     free(sl);
 }
 
@@ -285,11 +280,16 @@ int destroy_saved_lists_at_level (int level)
 
     if (nl < n_lists) {
 	n_lists = nl;
-	lstack = realloc(list_stack, nl * sizeof *list_stack);
-	if (lstack == NULL) {
-	    err = E_ALLOC;
+	if (nl == 0) {
+	    free(list_stack);
+	    list_stack = NULL;
 	} else {
-	    list_stack = lstack;
+	    lstack = realloc(list_stack, nl * sizeof *list_stack);
+	    if (lstack == NULL) {
+		err = E_ALLOC;
+	    } else {
+		list_stack = lstack;
+	    }
 	}
     }
 
@@ -334,6 +334,10 @@ int *gretl_list_new (int nterms)
 {
     int *list = NULL;
     int i;
+    
+    if (nterms < 0) {
+	return NULL;
+    }
 
     list = malloc((nterms + 1) * sizeof *list);
 
@@ -347,6 +351,181 @@ int *gretl_list_new (int nterms)
     return list;
 }
 
+/**
+ * gretl_consecutive_list_new:
+ * @lmin: starting value for consecutive list elements.
+ * @lmax: ending value.
+ * 
+ * Creates a newly allocated list whose elements run from
+ * @lmin to @lmax consecutively.
+ *
+ * Returns: the newly allocated list, or %NULL on failure.
+ */
+
+int *gretl_consecutive_list_new (int lmin, int lmax)
+{
+    int *list = NULL;
+    int i, n;
+
+    n = lmax - lmin + 1;
+    if (n <= 0) {
+	return NULL;
+    }
+
+    list = malloc((n + 1) * sizeof *list);
+
+    if (list != NULL) {
+	list[0] = n;
+	for (i=0; i<n; i++) {
+	    list[i+1] = lmin + i;
+	}
+    }
+
+    return list;
+}
+
+/**
+ * gretl_list_resize:
+ * @oldlist: pointer to list to be resized.
+ * @nterms: the new maximum number of elements for the list.
+ * 
+ * Resizes the content of @oldlist to hold @nterms, and adjusts
+ * the first element to reflect the new size.  If the new
+ * list is longer than the old, the extra elements are initialized 
+ * to zero.
+ *
+ * Returns: the resized list, or %NULL on failure.
+ */
+
+int *gretl_list_resize (int **oldlist, int nterms)
+{
+    int *list = NULL;
+    int i, oldn = 0;
+
+    if (nterms < 0 || oldlist == NULL) {
+	return NULL;
+    }
+
+    if (*oldlist != NULL) {
+	oldn = (*oldlist)[0];
+	if (nterms == oldn) {
+	    /* no-op */
+	    return *oldlist;
+	}
+    }
+
+    list = realloc(*oldlist, (nterms + 1) * sizeof *list);
+
+    if (list != NULL) {
+	list[0] = nterms;
+	*oldlist = list;
+	for (i=oldn+1; i<=list[0]; i++) {
+	    list[i] = 0;
+	}
+    } else {
+	free(*oldlist);
+	*oldlist = NULL;
+    }
+
+    return list;
+}
+
+/**
+ * gretl_list_append_term:
+ * @plist: pointer to list to be augmented.
+ * @v: the term to be added.
+ * 
+ * Resizes (or allocates from scratch) the content of @plist,
+ * so that it can hold one extra element, and sets the last
+ * element to @v.
+ *
+ * Returns: the augmented list, or %NULL on failure.
+ */
+
+int *gretl_list_append_term (int **plist, int v)
+{
+    int *list = NULL;
+
+    if (*plist == NULL) {
+	list = gretl_list_new(1);
+	if (list != NULL) {
+	    list[1] = v;
+	}
+    } else {
+	int oldn = (*plist)[0];
+
+	list = realloc(*plist, (oldn + 2) * sizeof *list);
+	if (list != NULL) {
+	    list[0] += 1;
+	    list[list[0]] = v;
+	} else {
+	    free(*plist);
+	}
+    }
+
+    *plist = list;
+
+    return list;
+}
+
+int gretl_compare_ints (const void *a, const void *b)
+{
+    const int *ia = (const int *) a;
+    const int *ib = (const int *) b;
+     
+    return (*ia > *ib) - (*ia < *ib);
+}
+
+/**
+ * gretl_list_sort:
+ * @list: list to be sorted.
+ * 
+ * Sorts the elements from position 1 to the end of @list
+ * in ascending order.
+ *
+ * Returns: the sorted list.
+ */
+
+int *gretl_list_sort (int *list)
+{
+    int i, sorted = 1;
+
+    for (i=1; i<list[0]; i++) {
+	if (list[i] > list[i+1]) {
+	    sorted = 0;
+	    break;
+	}
+    }
+
+    if (!sorted) {
+	qsort(list + 1, list[0], sizeof *list, gretl_compare_ints);
+    }
+
+    return list;
+}
+
+/**
+ * gretl_list_cmp:
+ * @list1: gretl list.
+ * @list1: gretl list.
+ * 
+ * Returns: 0 if @list1 and @list2 have identical content,
+ * otherwise 1.
+ */
+
+int gretl_list_cmp (const int *list1, const int *list2)
+{
+    int i;
+
+    for (i=0; i<=list1[0]; i++) {
+	if (list2[i] != list1[i]) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 /**
  * gretl_null_list:
  * 
@@ -380,7 +559,7 @@ int *gretl_list_copy (const int *src)
     int *targ = NULL;
     int i;
 
-    if (src != NULL && src[0] != 0) {
+    if (src != NULL) {
 	targ = malloc((src[0] + 1) * sizeof *targ);
 	if (targ != NULL) {
 	    for (i=0; i<=src[0]; i++) {
@@ -396,11 +575,12 @@ int *gretl_list_copy (const int *src)
  * gretl_list_from_string:
  * @liststr: string representation of list of integers.
  *
- * Reads a string containing a list of integers and constructs
- * an array of these integers.  The first element is the number
- * of integers that follow.
+ * Reads a string containing a list of integers, separated by
+ * spaces and/or commas and possibly wrapped in parentheses,
+ * and constructs an array of these integers.  The first 
+ * element is the number of integers that follow.
  *
- * Returns: the allocated array.
+ * Returns: the allocated array, or %NULL on failure.
  */
 
 int *gretl_list_from_string (const char *liststr)
@@ -411,8 +591,8 @@ int *gretl_list_from_string (const char *liststr)
     int n = 0;
 
     while (*s) {
-	while (*s == ' ') s++;
-	if (sscanf(s, "%7s", numstr)) {
+	s += strspn(s, " ,()");
+	if (*s && sscanf(s, "%7[^ ,()]", numstr)) {
 	    n++;
 	    s += strlen(numstr);
 	}
@@ -422,18 +602,16 @@ int *gretl_list_from_string (const char *liststr)
 	return NULL;
     }
 
-    list = malloc((n + 1) * sizeof *list);
+    list = gretl_list_new(n);
     if (list == NULL) {
 	return NULL;
     }
 
-    list[0] = n;
-
     s = liststr;
     n = 1;
     while (*s) {
-	while (*s == ' ') s++;
-	if (sscanf(s, "%7s", numstr)) {
+	s += strspn(s, " ,()");
+	if (*s && sscanf(s, "%7[^ ,()]", numstr)) {
 	    list[n++] = atoi(numstr);
 	    s += strlen(numstr);
 
@@ -481,8 +659,12 @@ char *gretl_list_to_string (const int *list)
 	    err = 1;
 	    break;
 	}
-	sprintf(numstr, " %d", list[i]);
-	strcat(buf, numstr);
+	if (list[i] == LISTSEP) {
+	    strcat(buf, " ;");
+	} else {
+	    sprintf(numstr, " %d", list[i]);
+	    strcat(buf, numstr);
+	}
     }
 
     if (err) {
@@ -520,32 +702,46 @@ int in_gretl_list (const int *list, int k)
     return ret;
 }
 
+static void reglist_move_const (int *list, int from)
+{
+    int i, cnum = list[from];
+
+    for (i=from; i>2; i--) {
+	list[i] = list[i-1];
+    }
+
+    list[2] = cnum;
+}
+
 /**
- * rearrange_list:
- * @list: an array of integers, the first element of which holds
- * a count of the number of elements following.
+ * reglist_check_for_const:
+ * @list: regression list suitable for use with a gretl
+ * model (should not contain #LISTSEP).
+ * @Z: data array.
+ * @pdinfo: dataset information.
  *
- * Checks @list for the presence of a constant term (ID 0), and 
- * if present, moves it to position 2 in @list.  This is
- * designed for producing a canonical version of a regression
- * list, with the constant (if any) preceding any other
- * regressors (position 1 holds the dependent variable in such
- * a list).
+ * Checks @list for an intercept term (a variable all of
+ * whose valid values in sample are 1).  If such a variable
+ * is present, it is moved to position 2 in the list.
+ *
+ * Returns: 1 if the list contains an intercept, else 0.
  */
 
-void rearrange_list (int *list)
+int reglist_check_for_const (int *list, const double **Z,
+			     const DATAINFO *pdinfo)
 {
-    int i, v;
+    int cpos = gretl_list_const_pos(list, 2, Z, pdinfo);
+    int ret = 0;
 
-    for (v=list[0]; v>2; v--) {
-        if (list[v] == 0)  {
-	    for (i=v; i>2; i--) {
-		list[i] = list[i-1];
-	    }
-	    list[2] = 0;
-	    return;
-        }
+    if (cpos > 1) {
+	ret = 1;
     }
+
+    if (cpos > 2) {
+	reglist_move_const(list, cpos);
+    }
+
+    return ret;
 }
 
 /**
@@ -576,23 +772,26 @@ int gretl_list_delete_at_pos (int *list, int pos)
 	list[0] -= 1;
     }
 
-    return 0;
+    return err;
 }
 
 /**
  * gretl_list_purge_const:
- * @list: an array of integers, the first element of which holds
- * a count of the number of elements following.
+ * @list: list of variable ID numbers.
+ * @Z: data array.
+ * @pdinfo: dataset information.
  *
- * Checks @list from position 1 onward for the presence of a constant
- * (the variable with ID number 0).  If this is found, it is deleted
- * from list (that is, any following elements are moved forward by one
- * and list[0] is decremented by 1).
+ * Checks @list from position 1 onward for the presence of a 
+ * variable whose valid values in sample all equal 1.0.  If 
+ * such a variable is found, it is deleted from @list (that is, 
+ * any following elements are moved forward by one and list[0] 
+ * is decremented by 1).
  *
- * Returns: 1 if the constant was found and deleted, else 0.
+ * Returns: 1 if a constant was found and deleted, else 0.
  */
 
-int gretl_list_purge_const (int *list)
+int gretl_list_purge_const (int *list, const double **Z,
+			    const DATAINFO *pdinfo)
 {
     int i, gotc = 0;
     int l0 = list[0];
@@ -600,7 +799,8 @@ int gretl_list_purge_const (int *list)
     /* handle the case where the constant comes last; if it's
        the only element behind the list separator, remove both
        the constant and the separator */
-    if (list[l0] == 0) {
+
+    if (list[l0] == 0 || true_const(list[l0], Z, pdinfo)) {
 	gotc = 1;
 	list[0] -= 1;
 	if (list[l0 - 1] == LISTSEP) {
@@ -609,7 +809,7 @@ int gretl_list_purge_const (int *list)
 	}
     } else {
 	for (i=1; i<l0; i++) {
-	    if (list[i] == 0) {
+	    if (list[i] == 0 || true_const(list[i], Z, pdinfo)) {
 		for ( ; i<l0; i++) {
 		    list[i] = list[i+1];
 		}
@@ -755,15 +955,18 @@ static int list_count (const int *list)
  * @orig: an array of integers, the first element of which holds
  * a count of the number of elements following.
  * @omit: list of variables to drop.
+ * @minpos: minimum position to check.  This should be 2 for a regular
+ * regression list, to skip the dependent var in position 1; but in
+ * other contexts it may be 1 to start from the first element of @orig.
  * @err: pointer to receive error code.
  *
  * Creates a list containing the elements of @orig that are not
- * present in @omit. 
+ * present in @omit.
  *
  * Returns: new list on success, %NULL on error.
  */
 
-int *gretl_list_omit (const int *orig, const int *omit, int *err)
+int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
 {
     int i, j, k;
     int *smal;
@@ -776,7 +979,7 @@ int *gretl_list_omit (const int *orig, const int *omit, int *err)
     for (i=1; i<=nomit; i++) {
 	int pos = in_gretl_list(orig, omit[i]);
 
-	if (pos <= 1) {
+	if (pos < minpos) {
 	    sprintf(gretl_errmsg, _("Variable %d was not in the original list"),
 		    omit[i]);
 	    *err = 1;
@@ -784,23 +987,36 @@ int *gretl_list_omit (const int *orig, const int *omit, int *err)
 	}
     }
 
-    /* check for attempt to omit all vars */
-    if (nomit == norig - 1) {
-	*err = E_NOVARS;
-	return NULL;
+    if (minpos > 1) {
+	/* it's an error to attempt to omit all vars */
+	if (nomit == norig - 1) {
+	    *err = E_NOVARS;
+	    return NULL;
+	}
     }
 
-    smal = malloc((norig - nomit + 1) * sizeof *smal);
+    if (nomit == norig) {
+	/* omitting all */
+	smal = gretl_null_list();
+	if (smal == NULL) {
+	    *err = E_ALLOC;
+	}
+	return smal;
+    }
+
+    smal = gretl_list_new(norig - nomit);
     if (smal == NULL) {
 	*err = E_ALLOC;
 	return NULL;
     }
 
-    smal[0] = norig - nomit;
-    smal[1] = orig[1];
+    for (i=1; i<minpos; i++) {
+	/* copy unaltered portion */
+	smal[i] = orig[i];
+    }
 
-    k = 1;
-    for (i=2; i<=norig; i++) {
+    k = minpos;
+    for (i=minpos; i<=norig; i++) {
         int match = 0;
 
         for (j=1; j<=nomit; j++) {
@@ -810,7 +1026,7 @@ int *gretl_list_omit (const int *orig, const int *omit, int *err)
             }
         }
         if (!match) { /* var is not in omit list: keep it */
-            smal[++k] = orig[i];
+            smal[k++] = orig[i];
         }
     }
 
@@ -864,15 +1080,16 @@ int gretl_list_diff (int *targ, const int *biglist, const int *sublist)
  * gretl_list_diff_new:
  * @biglist: inclusive list.
  * @sublist: subset of biglist.
+ * @minpos: position in lists at which to start.
  *
  * Returns: a newly allocated list including the elements of @biglist,
- * from position 2 onwards, that are not present in @sublist, or %NULL on 
- * failure.  It is assumed that the variable ID number in position 1
- * (dependent variable) is the same in both lists.  It is an error
- * if, from position 2 on, @sublist is not a proper subset of @biglist.
+ * from position @minpos onwards, that are not present in @sublist, 
+ * or %NULL on failure.  It is an error if, from position @minpos on, 
+ * @sublist is not a proper subset of @biglist.
  */
 
-int *gretl_list_diff_new (const int *biglist, const int *sublist)
+int *gretl_list_diff_new (const int *biglist, const int *sublist,
+			  int minpos)
 {
     int *targ = NULL;
     int i, j, n, k = 0;
@@ -886,9 +1103,9 @@ int *gretl_list_diff_new (const int *biglist, const int *sublist)
 
     targ = gretl_list_new(n);
 
-    for (i=2; i<=biglist[0]; i++) {
+    for (i=minpos; i<=biglist[0]; i++) {
 	match = 0;
-	for (j=2; j<=sublist[0]; j++) {
+	for (j=minpos; j<=sublist[0]; j++) {
 	    if (sublist[j] == biglist[i]) {
 		match = 1;
 		break;
@@ -975,6 +1192,49 @@ int gretl_list_insert_list (int **targ, const int *src, int pos)
     return err;
 }
 
+/**
+ * gretl_list_insert_list_minus:
+ * @targ: location of list into which @src should be inserted.
+ * @src: list to be inserted.
+ * @pos: zero-based position at which @src should be inserted.
+ *
+ * Inserts @src into @targ at @pos.  The length of @targ becomes the
+ * sum of the lengths of the two original lists minus one.  This
+ * can be useful if we were expecting to insert a single variable
+ * but found we had to insert a list instead.  Insertion of @src
+ * overwrites any entries in @targ beyond @pos (the expectation is
+ * that this function will be called in the process of assembling
+ * @targ, in left-to-right mode).
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int gretl_list_insert_list_minus (int **targ, const int *src, int pos)
+{
+    int *big;
+    int n1 = (*targ)[0];
+    int n2 = src[0];
+    int bign = n1 - 1 + n2;
+    int i, err = 0;
+
+    if (pos > n1 + 1) {
+	return 1;
+    }
+
+    big = realloc(*targ, (bign + 1) * sizeof *big);
+    if (big == NULL) {
+	err = E_ALLOC;
+    } else {
+	big[0] = bign;
+	for (i=1; i<=src[0]; i++) {
+	    big[pos+i-1] = src[i];
+	}
+	*targ = big;
+    }
+
+    return err;
+}
+
 /**
  * list_members_replaced:
  * @list: an array of integer variable ID numbers, the first element
@@ -1023,22 +1283,40 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 }
 
 /**
- * gretl_list_has_const:
+ * gretl_list_const_pos:
  * @list: an array of integer variable ID numbers, the first element
  * of which holds a count of the number of elements following.
+ * @minpos: position in @list at which to start the search (>= 1).
  *
- * Returns: 1 if the constant (variable ID 0) is found in @list,
- * in position 2 or higher.  This corresponds to determining
- * whether of not a set of regressors contains an intercept.
+ * Checks @list for the presence, in position @minpos or higher, of
+ * a variable whose valid values in sample all equal 1.  This usually
+ * amounts to checking whether a list of regressors includes
+ * an intercept term.
+ * 
+ * Returns: The list position of the const, or 0 if none is
+ * found.
  */
 
-int gretl_list_has_const (const int *list)
+int gretl_list_const_pos (const int *list, int minpos, const double **Z, 
+			  const DATAINFO *pdinfo)
 {
     int i;
 
-    for (i=2; i<=list[0]; i++) {
+    if (minpos < 1) {
+	return 0;
+    }
+
+    /* we give preference to the "official" const... */
+    for (i=minpos; i<=list[0]; i++) {
         if (list[i] == 0) {
-	    return 1;
+	    return i;
+	}
+    }
+
+    /* ... but if it's not found */
+    for (i=minpos; i<=list[0]; i++) {
+        if (true_const(list[i], Z, pdinfo)) {
+	    return i;
 	}
     }
 
@@ -1236,3 +1514,58 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
 
     return list;
 }
+
+/**
+ * gretl_list_position:
+ * @v: ID number of variable to find.
+ * @list: list to search.
+ *
+ * Scans @list, from position 1 onward, for variable @v.
+ * In case the list contains a list separator, #LISTSEP,
+ * only the portion prior to the separator is considered.
+ *
+ * Returns: the 0-based position, or 0 if the variable is 
+ * not found.
+ */
+
+int gretl_list_position (int v, const int *list)
+{
+    int i, lmax = list[0];
+
+    /* handle lists with separator: search only first part */
+    for (i=1; i<=list[0]; i++) {
+	if (list[i] == LISTSEP) {
+	    lmax = i - 1;
+	    break;
+	}
+    }
+	    
+    for (i=lmax; i>=1; i--) {
+	if (v == list[i]) return i;
+    }
+
+    return 0;
+}
+
+/**
+ * gretl_list_is_consecutive:
+ * @list: list to check.
+ *
+ * Returns: 1 if the elements of @list, from position 1 onward,
+ * are consecutive integer values, else 0.
+ */
+
+int gretl_list_is_consecutive (const int *list)
+{
+    int i, ret = 1;
+
+    for (i=2; i<=list[0]; i++) {
+	if (list[i] != list[i-1] + 1) {
+	    ret = 0;
+	    break;
+	}
+    }
+
+    return ret;
+}
+
