@@ -22,8 +22,18 @@
 
 int *gretl_list_new (int nterms);
 
+int *gretl_list_resize (int **oldlist, int nterms);
+
+int *gretl_list_append_term (int **plist, int v);
+
+int *gretl_list_sort (int *list);
+
+int gretl_list_cmp (const int *list1, const int *list2);
+
 int *gretl_null_list (void);
 
+int *gretl_consecutive_list_new (int lmin, int lmax);
+
 int *gretl_list_copy (const int *src);
 
 int *gretl_list_from_string (const char *liststr);
@@ -34,33 +44,42 @@ int in_gretl_list (const int *list, int k);
 
 int gretl_list_delete_at_pos (int *list, int pos);
 
-int gretl_list_purge_const (int *list);
+int gretl_list_purge_const (int *list, const double **Z,
+			    const DATAINFO *pdinfo);
 
 int *gretl_list_add (const int *orig, const int *add, int *err);
 
-int *gretl_list_omit (const int *orig, const int *omit, int *err);
+int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err);
 
 int *gretl_list_omit_last (const int *orig, int *err);
 
 int gretl_list_diff (int *targ, const int *biglist, const int *sublist);
 
-int *gretl_list_diff_new (const int *biglist, const int *sublist);
+int *gretl_list_diff_new (const int *biglist, const int *sublist, int minpos);
 
 int gretl_list_add_list (int **targ, const int *src);
 
 int gretl_list_insert_list (int **targ, const int *src, int pos);
 
-void rearrange_list (int *list);
+int gretl_list_insert_list_minus (int **targ, const int *src, int pos);
+
+int reglist_check_for_const (int *list, const double **Z,
+			     const DATAINFO *pdinfo);
+
+int gretl_list_const_pos (const int *list, int minpos, const double **Z,
+			  const DATAINFO *pdinfo);
 
 int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 			   int ref_id);
 
-int gretl_list_has_const (const int *list);
+int gretl_list_position (int v, const int *list);
 
 int gretl_list_separator_position (const int *list);
 
 int gretl_list_has_separator (const int *list);
 
+int gretl_list_is_consecutive (const int *list);
+
 int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2);
 
 int gretl_list_duplicates (const int *list, GretlCmdIndex ci);
