@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
+ *  Copyright (c) by Allin Cottrell
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include "libgretl.h"
 #include "objstack.h"
+#include "modelspec.h"
 
 #include "glib.h"
 
@@ -495,15 +496,23 @@ gretl_model_get_coeff_intervals (const MODEL *pmod,
     return cf;
 }
 
+static int gretl_is_arima_model (const MODEL *pmod)
+{
+    int d = (int) gretl_model_get_data(pmod, "arima_d");
+    int D = (int) gretl_model_get_data(pmod, "arima_D");
+
+    return (d > 0 || D > 0);
+}
+
 /**
- * gretl_arma_model_get_nonseasonal_AR_order:
+ * gretl_arma_model_nonseasonal_AR_order:
  * @pmod: pointer to gretl model.
  *
  * Returns: the non-seasonal autoregressive order of @pmod, or 0 if
  * @pmod is not an ARMA model.
  */
 
-int gretl_arma_model_get_nonseasonal_AR_order (const MODEL *pmod)
+int gretl_arma_model_nonseasonal_AR_order (const MODEL *pmod)
 {
     int p = 0;
 
@@ -515,69 +524,72 @@ int gretl_arma_model_get_nonseasonal_AR_order (const MODEL *pmod)
 }
 
 /**
- * gretl_arma_model_get_nonseasonal_MA_order:
+ * gretl_arma_model_nonseasonal_MA_order:
  * @pmod: pointer to gretl model.
  *
  * Returns: the non-seasonal moving-average order of @pmod, or 0 if
  * @pmod is not an ARMA model.
  */
 
-int gretl_arma_model_get_nonseasonal_MA_order (const MODEL *pmod)
+int gretl_arma_model_nonseasonal_MA_order (const MODEL *pmod)
 {
     int q = 0;
 
     if (pmod->ci == ARMA) {
-	q = pmod->list[2];
+	if (gretl_is_arima_model(pmod)) {
+	    q = pmod->list[3];
+	} else {
+	    q = pmod->list[2];
+	}
     }
 
     return q;
 }
 
 /**
- * gretl_arma_model_get_max_AR_lag:
+ * gretl_arma_model_max_AR_lag:
  * @pmod: pointer to gretl model.
  *
  * Returns: the maximum autoregressive lag in @pmod, or 0 if
- * @pmod is not an ARMA model.
+ * @pmod is not an ARMA model. The maximum AR lag takes into
+ * account any differencing (seasonal and/or non-seasonal) in
+ * an ARIMA model.
  */
 
-int gretl_arma_model_get_max_AR_lag (const MODEL *pmod)
+int gretl_arma_model_max_AR_lag (const MODEL *pmod)
 {
     int pmax = 0;
 
     if (pmod->ci == ARMA) {
-	int p, P, pd;
+	int p = gretl_arma_model_nonseasonal_AR_order(pmod);
+	int P = gretl_model_get_int(pmod, "arma_P");
+	int s = gretl_model_get_int(pmod, "arma_pd");
+	int d = gretl_model_get_int(pmod, "arima_d");
+	int D = gretl_model_get_int(pmod, "arima_D");
 
-	p = pmod->list[1];
-	P = gretl_model_get_int(pmod, "arma_P");
-
-	if (P == 0) {
-	    pmax = p;
-	} else {
-	    pd = gretl_model_get_int(pmod, "arma_pd");
-	    pmax = P * pd + p;
-	}
+	pmax = p + s * P;
+	pmax += d + s * D;
     }
 
     return pmax;
 }
 
 /**
- * gretl_arma_model_get_max_MA_lag:
+ * gretl_arma_model_max_MA_lag:
  * @pmod: pointer to gretl model.
  *
  * Returns: the maximum moving-average lag in @pmod, or 0 if
  * @pmod is not an ARMA model.
  */
 
-int gretl_arma_model_get_max_MA_lag (const MODEL *pmod)
+int gretl_arma_model_max_MA_lag (const MODEL *pmod)
 {
     int qmax = 0;
 
     if (pmod->ci == ARMA) {
 	int q, Q, pd;
 
-	q = pmod->list[2];
+	q = gretl_arma_model_nonseasonal_MA_order(pmod);
 	Q = gretl_model_get_int(pmod, "arma_Q");
 	
 	if (Q == 0) {
@@ -591,6 +603,66 @@ int gretl_arma_model_get_max_MA_lag (const MODEL *pmod)
     return qmax;
 }
 
+/* From Box and Jenkins, 1976, pp 506-7, "Program 4": a clever
+   algorithm for "unscrambling the coefficients", or in effect
+   producing reduced-form AR coefficients that take into account any
+   differencing, seasonal and/or non-seasonal.
+*/
+
+static int ar_coeff_integrate (double *c0, int d, int D, int s, int pmax)
+{
+    int pstar = pmax + d + s * D;
+    double *c1;
+    int i, j, pp;
+
+    c1 = malloc((pstar + 1) * sizeof *c1);
+    if (c1 == NULL) {
+	return E_ALLOC;
+    }
+
+    for (j=0; j<=pstar; j++) {
+	c1[j] = 0.0;
+    }
+
+    pp = pmax;
+
+    for (i=0; D>0 && i<D; i++) {
+	for (j=0; j<=pstar; j++) {
+	    if (j < s) {
+		c1[j] = c0[j];
+	    } else if (j <= pp) {
+		c1[j] = c0[j] - c0[j-s];
+	    } else if (j <= pp + s) {
+		c1[j] = -c0[j-s];
+	    }
+	}
+	pp += s;
+	for (j=0; j<=pstar; j++) {
+	    c0[j] = c1[j];
+	} 
+    }
+
+    for (i=0; d>0 && i<d; i++) {
+	for (j=0; j<=pstar; j++) {
+	    if (j < 1) {
+		c1[j] = c0[j];
+	    } else if (j <= pp) {
+		c1[j] = c0[j] - c0[j-1];
+	    } else if (j <= pp + 1) {
+		c1[j] = -c0[j-1];
+	    }		
+	}
+	pp += 1;
+	for (j=0; j<=pstar; j++) {
+	    c0[j] = c1[j];
+	}
+    }
+
+    free(c1);
+
+    return 0;
+}
+
 /**
  * gretl_arma_model_get_AR_MA_coeffs:
  * @pmod: pointer to gretl model.
@@ -620,27 +692,31 @@ int gretl_arma_model_get_AR_MA_coeffs (const MODEL *pmod,
     } else {
 	const double *phi = NULL, *Phi = NULL;
 	const double *theta = NULL, *Theta = NULL;
+	double x, y;
 
-	int p = pmod->list[1];
-	int q = pmod->list[2];
+	int p = gretl_arma_model_nonseasonal_AR_order(pmod);
+	int q = gretl_arma_model_nonseasonal_MA_order(pmod);
 	int P = gretl_model_get_int(pmod, "arma_P");
 	int Q = gretl_model_get_int(pmod, "arma_Q");
-	int pd = gretl_model_get_int(pmod, "arma_pd");
-	int pmax, qmax;
+	int d = gretl_model_get_int(pmod, "arima_d");
+	int D = gretl_model_get_int(pmod, "arima_D");
+	int s = gretl_model_get_int(pmod, "arma_pd");
+	int pmax, pstar, qmax;
 	int i, j, k;
 
-	pmax = (P > 0)? P * pd + p : p;
-	qmax = (Q > 0)? Q * pd + q : q;
+	pmax = p + s * P;
+	pstar = pmax + d + s * D;
+	qmax = q + s * Q;
 	
-	if (pmax > 0) {
-	    ac = malloc(pmax * sizeof *ac);
+	if (pstar > 0) {
+	    ac = malloc((pstar + 1) * sizeof *ac);
 	    if (ac == NULL) {
 		err = E_ALLOC;
 	    }
 	}
 
 	if (!err && qmax > 0) {
-	    mc = malloc(qmax * sizeof *ac);
+	    mc = malloc((qmax + 1) * sizeof *ac);
 	    if (mc == NULL) {
 		free(ac);
 		ac = NULL;
@@ -656,39 +732,34 @@ int gretl_arma_model_get_AR_MA_coeffs (const MODEL *pmod,
 	}
 
 	if (ac != NULL) {
-	    for (i=0; i<p; i++) {
-		ac[i] = phi[i];
-	    }	    
-	    if (P > 0) {
-		for (i=p; i<pmax; i++) {
-		    ac[i] = 0.0;
+	    for (i=0; i<=pstar; i++) {
+		ac[i] = 0.0;
+	    }
+	    for (i=0; i<=P; i++) {
+		x = (i == 0)? -1 : Phi[i-1];
+		for (j=0; j<=p; j++) {
+		    y = (j == 0)? -1 : phi[j-1];
+		    k = j + s * i;
+		    ac[k] -= x * y;
 		}
-		for (i=0; i<P; i++) {
-		    k = pd * (i+1) - 1;
-		    ac[k] += Phi[i];
-		    for (j=0; j<p; j++) {
-			ac[k+j+1] += Phi[i] * phi[j];
-		    }
-		}
-	    }		
-	}
+	    }
+	    if (D > 0 || d > 0) {
+		ar_coeff_integrate(ac, d, D, s, pmax);
+	    }
+	}	
 
 	if (mc != NULL) {
-	    for (i=0; i<q; i++) {
-		mc[i] = theta[i];
-	    }	    
-	    if (Q > 0) {
-		for (i=p; i<qmax; i++) {
-		    mc[i] = 0.0;
+	    for (i=0; i<=qmax; i++) {
+		mc[i] = 0.0;
+	    }
+	    for (i=0; i<=Q; i++) {
+		x = (i == 0)? -1 : Theta[i-1];
+		for (j=0; j<=q; j++) {
+		    y = (j == 0)? -1 : theta[j-1];
+		    k = j + s * i;
+		    mc[k] -= x * y;
 		}
-		for (i=0; i<Q; i++) {
-		    k = pd * (i+1) - 1;
-		    mc[k] += Theta[i];
-		    for (j=0; j<q; j++) {
-			mc[k+j+1] += Theta[i] * theta[j];
-		    }
-		}
-	    }		
+	    }
 	}
     }
 
@@ -716,8 +787,8 @@ const double *gretl_arma_model_get_x_coeffs (const MODEL *pmod)
     if (pmod->ci == ARMA && gretl_model_get_int(pmod, "armax")) {
 	xc = pmod->coeff;
 	xc += pmod->ifc;
-	xc += pmod->list[1];
-	xc += pmod->list[2];
+	xc += gretl_arma_model_nonseasonal_AR_order(pmod);
+	xc += gretl_arma_model_nonseasonal_MA_order(pmod);
 	xc += gretl_model_get_int(pmod, "arma_P");
 	xc += gretl_model_get_int(pmod, "arma_Q");
     }
@@ -725,18 +796,32 @@ const double *gretl_arma_model_get_x_coeffs (const MODEL *pmod)
     return xc;
 }
 
-static int is_seasonal_arma (const MODEL *pmod)
+static int arma_depvar_pos (const MODEL *pmod)
 {
-    int ret = 0;
+    int seasonal = 0;
+    int arima = 0;
+    int dvpos;
 
     if (gretl_model_get_int(pmod, "arma_P") ||
+	gretl_model_get_int(pmod, "arima_D") ||
 	gretl_model_get_int(pmod, "arma_Q")) {
-	ret = 1;
+	seasonal = 1;
     }
 
-    return ret;
-}
+    if (gretl_model_get_int(pmod, "arima_d") ||
+	gretl_model_get_int(pmod, "arima_D")) {
+	arima = 1;
+    }
 
+    if (arima) {
+	dvpos = (seasonal)? 9 : 5;
+    } else {
+	dvpos = (seasonal)? 7 : 4;
+    }
+
+    return dvpos;
+}
+    
 /**
  * gretl_model_get_depvar:
  * @pmod: pointer to gretl model.
@@ -752,11 +837,7 @@ int gretl_model_get_depvar (const MODEL *pmod)
 	if (pmod->ci == GARCH) {
 	    dv = pmod->list[4];
 	} else if (pmod->ci == ARMA) {
-	    if (is_seasonal_arma(pmod)) {
-		dv = pmod->list[7];
-	    } else {
-		dv = pmod->list[4];
-	    }
+	    dv = pmod->list[arma_depvar_pos(pmod)];
 	} else {
 	    dv = pmod->list[1];
 	}
@@ -779,7 +860,7 @@ int *gretl_model_get_x_list (const MODEL *pmod)
     int i, nx;
 
     if (pmod->ci == ARMA) {
-	int start = (is_seasonal_arma(pmod))? 7 : 4;
+	int start = arma_depvar_pos(pmod);
 
 	nx = pmod->list[0] - start + pmod->ifc;
 	if (nx > 0) {
@@ -1117,7 +1198,7 @@ void gretl_model_init (MODEL *pmod)
 #endif
 
     pmod->ID = 0;
-    pmod->refcount = 1;
+    pmod->refcount = 0;
     pmod->full_n = 0;
 
     pmod->ntests = 0;
@@ -1157,19 +1238,20 @@ void gretl_model_smpl_init (MODEL *pmod, const DATAINFO *pdinfo)
     pmod->smpl.t2 = pdinfo->t2;
 }
 
-static MODEL *real_gretl_model_new (int protected)
+static MODEL *real_gretl_model_new (int protect)
 {
     MODEL *pmod = malloc(sizeof *pmod);
 
 #if MDEBUG
     fprintf(stderr, "real_gretl_model_new: model at %p\n", (void *) pmod);
+    if (pmod != NULL) fprintf(stderr, " calling gretl_model_init\n");
 #endif
 
     if (pmod != NULL) {
 	gretl_model_init(pmod);
     }
 
-    if (protected) {
+    if (protect) {
 #if MDEBUG
 	fprintf(stderr, " protecting this model\n");
 #endif
@@ -1400,22 +1482,14 @@ void clear_model (MODEL *pmod)
  * gretl_model_free:
  * @pmod: pointer to #MODEL.
  *
- * Free allocated content of @pmod then the pointer itself,
- * if the model's reference count has reached zero.
+ * Free allocated content of @pmod then the pointer itself.
  */
 
 void gretl_model_free (MODEL *pmod)
 {
     if (pmod != NULL) {
-#if MDEBUG
-	fprintf(stderr, "gretl_model_free: pmod at %p, incoming refcount = %d\n",
-		(void *) pmod, pmod->refcount);
-#endif
-	pmod->refcount -= 1;
-	if (pmod->refcount <= 0) {
-	    clear_model(pmod);
-	    free(pmod);
-	}
+	clear_model(pmod);
+	free(pmod);
     }
 }
 
@@ -1429,6 +1503,9 @@ void gretl_model_free (MODEL *pmod)
 
 void gretl_model_free_on_exit (MODEL *pmod)
 {
+#if MDEBUG
+    fprintf(stderr, "gretl_model_free_on_exit: pmod at %p\n", (void *) pmod);
+#endif
     if (pmod != NULL) {
 	remove_model_from_stack(pmod);
 	clear_model(pmod);
@@ -1990,7 +2067,7 @@ static int copy_model_data_items (MODEL *targ, const MODEL *src)
     }
 
     if (err) {
-	for (i=0; i<targ->n_data_items; i++) {
+	for (i=0; i<src->n_data_items; i++) {
 	    free(targ->data_items[i]);
 	}
 	free(targ->data_items);
@@ -2026,6 +2103,12 @@ static int copy_model (MODEL *targ, const MODEL *src)
     /* monolithic copy of structure */
     *targ = *src;
 
+    /* temporarily zero various count members just in case
+       copying of allocated info fails */
+    targ->ntests = 0;
+    targ->nparams = 0;
+    targ->n_data_items = 0;
+
     /* now work on pointer members */
     gretl_model_init_pointers(targ);
 
@@ -2059,13 +2142,11 @@ static int copy_model (MODEL *targ, const MODEL *src)
 	return 1;
     }
 
-    if (src->xpx != NULL && 
-	(targ->xpx = copyvec(src->xpx, m)) == NULL) {
+    if (src->xpx != NULL && (targ->xpx = copyvec(src->xpx, m)) == NULL) {
 	return 1;
     }
 
-    if (src->vcv != NULL && 
-	(targ->vcv = copyvec(src->vcv, m)) == NULL) {
+    if (src->vcv != NULL && (targ->vcv = copyvec(src->vcv, m)) == NULL) {
 	return 1;
     }
 
@@ -2079,6 +2160,7 @@ static int copy_model (MODEL *targ, const MODEL *src)
 	if (targ->tests == NULL) {
 	    return 1;
 	}
+	targ->ntests = src->ntests;
     }
 
     if (src->nparams > 0 && src->params != NULL) {
@@ -2086,6 +2168,7 @@ static int copy_model (MODEL *targ, const MODEL *src)
 	if (targ->params == NULL) {
 	    return 1;
 	}
+	targ->nparams = src->nparams;
     }    
 
     if (src->n_data_items > 0) {
@@ -2093,6 +2176,7 @@ static int copy_model (MODEL *targ, const MODEL *src)
 	if (targ->data_items == NULL) {
 	    return 1;
 	}
+	targ->n_data_items = src->n_data_items;
     }
 
     if (src->list != NULL && 
@@ -2100,6 +2184,8 @@ static int copy_model (MODEL *targ, const MODEL *src)
 	return 1;
     } 
 
+    /* src->dataset?? */
+
     return 0;
 }
 
@@ -2138,30 +2224,22 @@ MODEL *gretl_model_copy (const MODEL *pmod)
 
 /**
  * swap_models:
- * @targ: pointer to pointer to #MODEL.
- * @src: pointer to pointer to #MODEL.
+ * @targ: pointer to target #MODEL.
+ * @src: pointer to source #MODEL.
  *
- * Swaps the model pointers.
- *
- * Returns: 0 on success.
+ * Swaps the content of the two model pointers.
  */
 
-int swap_models (MODEL **targ, MODEL **src)
+void swap_models (MODEL *targ, MODEL *src)
 {
-    MODEL *tmp = *targ;
+    MODEL tmp = *targ;
 
 #if MDEBUG
-    fprintf(stderr, "swap_models: %p <-> %p\n", *targ, *src);
+    fprintf(stderr, "swap_models: %p <-> %p\n", targ, src);
 #endif
 
     *targ = *src;
     *src = tmp;
-
-    /* handle the case where we have a model that is currently
-       defined as the "last model" (objstack.c) */
-    maybe_swap_into_last_model(*targ, tmp);
-
-    return 0;
 }
 
 int is_model_cmd (const char *s)
@@ -2191,6 +2269,7 @@ int is_model_cmd (const char *s)
 	    !strncmp(s, "logistic", 8) ||
 	    !strncmp(s, "end nls", 7) ||
 	    !strncmp(s, "arma", 4) ||
+	    !strncmp(s, "arima", 5) ||
 	    !strncmp(s, "ar ", 3) ||
 	    !strcmp(s, "ar")) {
 	    ret = 1;
@@ -2237,17 +2316,23 @@ int command_ok_for_model (int test_ci, int model_ci)
     case OMIT:
     case OMITFROM:
 	if (model_ci == NLS || model_ci == ARMA || 
-	    model_ci == GARCH) ok = 0;
+	    model_ci == GARCH) {
+	    ok = 0;
+	}
 	break;
 
     case COEFFSUM:
     case VIF:
 	if (model_ci == NLS || model_ci == TSLS ||
-	    model_ci == ARMA || model_ci == GARCH) ok = 0;
+	    model_ci == ARMA || model_ci == GARCH) {
+	    ok = 0;
+	}
 	break;
 
     case EQNPRINT:
-	if (model_ci != OLS) ok = 0; /* FIXME: unduly restrictive? */
+	if (model_ci == ARMA || model_ci == NLS) {
+	    ok = 0; 
+	}
 	break;
 
     case LMTEST:
@@ -2313,8 +2398,8 @@ void set_model_id (MODEL *pmod)
 
 void model_list_to_string (int *list, char *buf)
 {
+    char numstr[8];
     int i;
-    char numstr[5];
 
     for (i=1; i<=list[0]; i++) {
 	if (list[i] == LISTSEP) {
@@ -2365,37 +2450,32 @@ int highest_numbered_var_in_model (const MODEL *pmod,
     return vmax;
 }
 
-int mle_aic_bic (MODEL *pmod, int addk)
+int mle_criteria (MODEL *pmod, int addk)
 {
     int err = 0;
 
     if (na(pmod->lnL)) {
 	pmod->criterion[C_AIC] = NADBL;
 	pmod->criterion[C_BIC] = NADBL;
+	pmod->criterion[C_HQC] = NADBL;
 	err = 1;
     } else {
 	int k = pmod->ncoeff + addk;
 
 	pmod->criterion[C_AIC] = -2.0 * pmod->lnL + 2.0 * k;
 	pmod->criterion[C_BIC] = -2.0 * pmod->lnL + k * log(pmod->nobs);
+	pmod->criterion[C_HQC] = -2.0 * pmod->lnL + 2 * k * log(log(pmod->nobs));
     }
 
     return err;
 }
 
-/* As of 2005-02-19, not ready for this yet: need to adjust
-   the text in the model printout.
-*/
-
 double coeff_pval (const MODEL *pmod, double x, int df)
 {
-    if (pmod->ci == MLE) {
-	return normal_pvalue_2(x);
-    } else if (0 && ML_ESTIMATOR(pmod->ci)) {
-	/* not ready */
-	return normal_pvalue_2(x);
+    if (ASYMPTOTIC_MODEL(pmod->ci)) {
+        return normal_pvalue_2(x);
     } else {
-	return t_pvalue_2(x, df);
+        return t_pvalue_2(x, df);
     }
 }
 
@@ -2420,14 +2500,9 @@ int gretl_model_add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 				   int yno, int p, int q, int P, int Q, 
 				   int r)
 {
-    int ptot = p + P;
-    int qtot = q + Q;
-    int np = ptot + qtot + r + pmod->ifc + 1;
+    int np = p + P + q + Q + r + pmod->ifc + 1;
     int xstart;
-    int i, j, s;
-
-    const char *depvar;
-    size_t n;
+    int i, j;
 
     pmod->params = malloc(np * sizeof pmod->params);
     if (pmod->params == NULL) {
@@ -2460,32 +2535,23 @@ int gretl_model_add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 	j = 1;
     }
 
-    depvar = pmod->params[0];
-    n = strlen(depvar);
-
-    for (i=0; i<ptot; i++) {
-	if (i < p) {
-	    s = i + 1;
-	} else {
-	    s = (i - p + 1) * pdinfo->pd;
-	}
-	if (n < VNAMELEN - 4) {
-	    sprintf(pmod->params[j++], "%s(-%d)", depvar, s);
-	} else {
-	    sprintf(pmod->params[j++], "y(-%d)", s);
-	}
+    for (i=0; i<p; i++) {
+	sprintf(pmod->params[j++], "phi_%d", i + 1);
     }
 
-    for (i=0; i<qtot; i++) {
-	if (i < q) {
-	    s = i + 1;
-	} else {
-	    s = (i - q + 1) * pdinfo->pd;
-	}    
-	sprintf(pmod->params[j++], "e(-%d)", s);
+    for (i=0; i<P; i++) {
+	sprintf(pmod->params[j++], "Phi_%d", i + 1);
+    }   
+
+    for (i=0; i<q; i++) {
+	sprintf(pmod->params[j++], "theta_%d", i + 1);
     }
 
-    xstart = (P || Q)? 8 : 5;
+    for (i=0; i<Q; i++) {
+	sprintf(pmod->params[j++], "Theta_%d", i + 1);
+    }       
+
+    xstart = arma_depvar_pos(pmod) + 1;
 
     for (i=0; i<r; i++) {
 	strcpy(pmod->params[j++], pdinfo->varname[pmod->list[xstart+i]]); 
@@ -2554,6 +2620,8 @@ char *gretl_model_get_fitted_formula (const MODEL *pmod, int xvar,
 	mdinfo = pdinfo;
     }
 
+    gretl_push_c_numeric_locale();
+
     if (pmod->ci == LOGISTIC) {
 	if (pmod->ifc && pmod->ncoeff == 2 && xvar == pmod->list[3]) {
 	    double lmax = gretl_model_get_double(pmod, "lmax");
@@ -2580,6 +2648,8 @@ char *gretl_model_get_fitted_formula (const MODEL *pmod, int xvar,
 				  pmod->coeff[2]);
 	}
     }
+
+    gretl_pop_c_numeric_locale();
 	
     return ret;
 }
@@ -2602,13 +2672,16 @@ const char *gretl_model_get_name (const MODEL *pmod)
     return NULL;
 }
 
-int gretl_model_stat_index (const char *s)
+int gretl_model_data_index (const char *s)
 {
     char test[8] = {0};
+    int ssel = 0;
+    int msel = 0;
 
     strncat(test, s, 7);
     lower(test);
 
+    /* scalar values */
     if (!strcmp(test, "$ess"))  
 	return M_ESS;
     if (!strcmp(test, "$t")) 
@@ -2627,14 +2700,61 @@ int gretl_model_stat_index (const char *s)
 	return M_AIC;
     if (!strcmp(test, "$bic"))   
 	return M_BIC;
+    if (!strcmp(test, "$hqc"))   
+	return M_HQC;
     if (!strcmp(test, "$nrsq") || 
 	!strcmp(test, "$trsq")) 
 	return M_TRSQ;
 
+    sscanf(s, "%7[^[( ]", test);
+    lower(test);
+
+    if (strchr(s, '[')) {
+	/* selecting submatrix? */
+	msel = 1;
+    }
+
+    /* series or matrices */
+    if (!strcmp(test, "$uhat")) 
+	return M_UHAT;
+    if (!strcmp(test, "$yhat"))
+	return M_YHAT;
+    if (!strcmp(test, "$h"))
+	return M_H;
+
+    if (!msel && strchr(s, '(')) {
+	/* selecting scalar element from array? */
+	ssel = 1;
+    } 
+
+    /* matrices, not series */
+    if (!strcmp(test, "$coeff"))  
+	return (ssel)? M_COEFF_S : M_COEFF;
+    if (!strcmp(test, "$stderr"))  
+	return (ssel)? M_SE_S : M_SE;
+    if (!strcmp(test, "$vcv"))   
+	return (ssel)? M_VCV_S : M_VCV;
+    if (!strcmp(test, "$rho"))   
+	return (ssel)? M_RHO_S : M_RHO;
+
     return 0;
 }
 
-double gretl_model_get_scalar (const MODEL *pmod, int idx, int *err)
+/**
+ * gretl_model_get_scalar:
+ * @pmod: pointer to target model.
+ * @idx: index for the scalar value that is wanted.
+ * @err: location to receive error code (required).
+ * 
+ * Retrieves a specified scalar statistic from @pmod:
+ * @idx must be less than %M_SCALAR_MAX.
+ * 
+ * Returns: the requested statistic, or #NADBL on failure,
+ * in which case @err will contain a non-zero error code.
+ */
+
+double gretl_model_get_scalar (const MODEL *pmod, ModelDataIndex idx, 
+			       int *err)
 {
     double x = NADBL;
 
@@ -2643,8 +2763,6 @@ double gretl_model_get_scalar (const MODEL *pmod, int idx, int *err)
 	return x;
     }
 
-    if (*err) return x;
-
     switch (idx) {  
     case M_ESS:
 	x = pmod->ess;
@@ -2661,6 +2779,9 @@ double gretl_model_get_scalar (const MODEL *pmod, int idx, int *err)
     case M_BIC:
 	x = pmod->criterion[C_BIC];
 	break;
+    case M_HQC:
+	x = pmod->criterion[C_HQC];
+	break;
     case M_SIGMA:
 	if (pmod->nwt) x = pmod->sigma_wt;
 	else x = pmod->sigma;
@@ -2679,6 +2800,8 @@ double gretl_model_get_scalar (const MODEL *pmod, int idx, int *err)
     case M_T:
 	x = (double) pmod->nobs;
 	break;	
+    default:
+	break;
     }
 
     if (na(x)) {
@@ -2687,3 +2810,238 @@ double gretl_model_get_scalar (const MODEL *pmod, int idx, int *err)
 
     return x;
 }
+
+/**
+ * gretl_model_get_series:
+ * @pmod: pointer to target model.
+ * @idx: index for the series that is wanted.
+ * @err: location to receive error code (required).
+ * 
+ * Retrieves from @pmod a copy of a specified series (for
+ * example, regression residuals); @idx must be greater than
+ * %M_ELEM_MAX and less than %M_SERIES_MAX.
+ * 
+ * Returns: the allocated series, or %NULL on failure,
+ * in which case @err will contain a non-zero error code.
+ */
+
+double *
+gretl_model_get_series (const MODEL *pmod, const DATAINFO *pdinfo, 
+			ModelDataIndex idx, int *err)
+{
+    double *x = NULL;
+    double *garch_h = NULL;
+    int t;
+
+    if (pmod->t2 - pmod->t1 + 1 > pdinfo->n || 
+	model_sample_issue(pmod, NULL, 0, pdinfo)) {
+	strcpy(gretl_errmsg, 
+	       (idx == M_UHAT)? 
+	       _("Can't retrieve uhat: data set has changed") :
+	       (idx == M_YHAT)?
+	       _("Can't retrieve yhat: data set has changed") :
+	       _("Can't retrieve ht: data set has changed"));
+	*err = 1;
+	return NULL;
+    }   
+
+    if ((idx == M_UHAT && pmod->uhat == NULL) ||
+	(idx == M_YHAT && pmod->yhat == NULL)) {
+	*err = 1;
+	return NULL;
+    }
+
+    if (idx == M_H) {
+	garch_h = gretl_model_get_data(pmod, "garch_h");
+	if (garch_h == NULL) {
+	    strcpy(gretl_errmsg, _("Can't retrieve error variance"));
+	    *err = 1;
+	    return NULL;
+	}
+    }
+
+    x = malloc(pdinfo->n * sizeof *x);
+    if (x == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (t=0; t<pdinfo->n; t++) {
+	if (t < pmod->t1 || t > pmod->t2) {
+	    x[t] = NADBL;
+	} else {
+	    if (idx == M_UHAT) {
+		x[t] = pmod->uhat[t];
+	    } else if (idx == M_YHAT) {
+		x[t] = pmod->yhat[t];
+	    } else if (idx == M_H) {
+		x[t] = garch_h[t];
+	    }
+	}
+    }
+	    
+    return x;
+}
+
+static gretl_matrix *
+model_get_estvec (const MODEL *pmod, int idx, int *err)
+{
+    gretl_matrix *v = NULL;
+    double x;
+    int i;
+
+    v = gretl_column_vector_alloc(pmod->ncoeff);
+    if (v == NULL) {
+	*err = E_ALLOC;
+    } else {
+	for (i=0; i<pmod->ncoeff; i++) {
+	    x = (idx == M_COEFF)? pmod->coeff[i] : pmod->sderr[i];
+	    gretl_vector_set(v, i, x);
+	}
+    }
+
+    return v;
+}
+
+static gretl_matrix *
+model_get_hatvec (const MODEL *pmod, int idx, int *err)
+{
+    gretl_matrix *v = NULL;
+    double x;
+    int t;
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (na(pmod->uhat[t])) {
+	    *err = E_MISSDATA;
+	    break;
+	}
+    }
+
+    if (!*err) {
+	v = gretl_column_vector_alloc(pmod->t2 - pmod->t1 + 1);
+	if (v == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    for (t=pmod->t1; t<=pmod->t2; t++) {
+		x = (idx == M_UHAT)? pmod->uhat[t] : pmod->yhat[t];
+		gretl_vector_set(v, t - pmod->t1, x);
+	    }
+	}
+    }
+
+    return v;
+}
+
+static gretl_matrix *model_get_hvec (const MODEL *pmod, int *err)
+{
+    double *garch_h = gretl_model_get_data(pmod, "garch_h");
+    gretl_matrix *v = NULL;
+    int t;
+
+    if (garch_h == NULL) {
+	*err = E_BADSTAT;
+    }
+
+    v = gretl_column_vector_alloc(pmod->t2 - pmod->t1 + 1);
+    if (v == NULL) {
+	*err = E_ALLOC;
+    } else {
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    /* FIXME: is indexation right? */
+	    gretl_vector_set(v, t - pmod->t1, garch_h[t]);
+	}
+    }
+
+    return v;
+}
+
+static gretl_matrix *model_get_rhovec (const MODEL *pmod, int *err)
+{
+    gretl_matrix *r = NULL;
+
+    if (AR1_MODEL(pmod->ci)) {
+	double x = gretl_model_get_double(pmod, "rho_in");
+
+	r = gretl_matrix_from_scalar(x);
+    } else if (pmod->ci != AR) {
+	r = gretl_matrix_from_scalar(pmod->rho);
+    } else if (pmod->arinfo == NULL || 
+	       pmod->arinfo->arlist == NULL || 
+	       pmod->arinfo->rho == NULL) {
+	*err = E_INVARG;
+    } else {
+	int i, l0 = pmod->arinfo->arlist[0];
+	int lmax = pmod->arinfo->arlist[l0];
+
+	r = gretl_vector_alloc(lmax);
+	if (r != NULL) {
+	    gretl_matrix_zero(r);
+	    for (i=1; i<=l0; i++) {
+		gretl_vector_set(r, pmod->arinfo->arlist[i] - 1,
+				 pmod->arinfo->rho[i-1]);
+	    }
+	}
+    }
+
+    return r;
+}
+
+/**
+ * gretl_model_get_matrix:
+ * @pmod: pointer to target model.
+ * @idx: index for the matrix that is wanted.
+ * @err: location to receive error code (required).
+ * 
+ * Retrieves from @pmod a copy of a specified matrix (for
+ * example, regression residuals); @idx must be greater than
+ * %M_ELEM_MAX and less than %M_MAX.
+ * 
+ * Returns: the allocated matrix, or %NULL on failure,
+ * in which case @err will contain a non-zero error code.
+ */
+
+gretl_matrix *gretl_model_get_matrix (MODEL *pmod, ModelDataIndex idx, 
+				      int *err)
+{
+    gretl_matrix *M = NULL;
+
+    if (pmod == NULL) {
+	*err = E_BADSTAT;
+	return M;
+    }
+
+    if (*err) return M;
+
+    switch (idx) {  
+    case M_UHAT:
+    case M_YHAT:
+	M = model_get_hatvec(pmod, idx, err);
+	break;
+    case M_COEFF:
+    case M_SE:
+	M = model_get_estvec(pmod, idx, err);
+	break;
+    case M_VCV:
+	M = gretl_vcv_matrix_from_model(pmod, NULL);
+	break;
+    case M_H:
+	if (pmod->ci != GARCH) {
+	    *err = E_BADSTAT;
+	} else {
+	    M = model_get_hvec(pmod, err);
+	}
+	break;
+    case M_RHO:
+	M = model_get_rhovec(pmod, err);
+	break;
+    default:
+	*err = E_BADSTAT;
+	break;
+    }
+
+    if (M == NULL && !*err) {
+	*err = E_ALLOC;
+    }
+
+    return M;
+}
