@@ -30,16 +30,12 @@
 # include <dirent.h>
 #endif
 
-#if defined(USE_GTK2) && !defined(WIN32)
+#ifdef USE_GTK2
 # include <glib.h>
-# if GLIB_CHECK_VERSION(2,0,0)
-#  define GLIB2
-# endif /* GLIB_CHECK_VERSION */
-#endif /* GTK2, !WIN32 */
-
-#if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
-# ifdef WIN32
-#  define USE_G_FOPEN
+# if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
+#  ifdef WIN32
+#   define USE_G_FOPEN
+#  endif
 # endif
 #endif
 
@@ -66,27 +62,51 @@ static int add_gdt_suffix (char *fname)
 
 FILE *gretl_fopen (const char *filename, const char *mode)
 {
-#ifdef USE_G_FOPEN
-    return g_fopen((const gchar *) filename, (const gchar *) mode);
-#else
-    return fopen(filename, mode);
+    FILE *fp = NULL;
+
+#if defined(USE_G_FOPEN)
+    fp = g_fopen((const gchar *) filename, (const gchar *) mode);
+#elif defined(WIN32)
+    fp = fopen(filename, mode);
+    if (fp == NULL) {
+	int save_errno = errno;
+	gchar *fconv;
+	gsize wrote;
+
+	fconv = g_locale_from_utf8(filename, -1, NULL, &wrote, NULL);
+	if (fconv != NULL) {
+	    fp = fopen(fconv, mode);
+	    g_free(fconv);
+	}
+	errno = save_errno;
+    }
+#else    
+    fp = fopen(filename, mode);
 #endif
+
+    return fp;
 }
 
 gzFile gretl_gzopen (const char *filename, const char *mode)
 {
-#ifdef USE_G_FOPEN
-    gchar *cp_filename = g_locale_from_utf8(filename, -1, NULL, NULL, NULL);
-    gzFile fz = gzopen(cp_filename, mode);
+    gzFile fz = NULL;
+
+#if defined(USE_G_FOPEN) || defined(WIN32)
     int save_errno = errno;
+    gchar *fconv;
+    gsize wrote;
 
-    g_free(cp_filename);
-
+    fconv = g_locale_from_utf8(filename, -1, NULL, &wrote, NULL);
+    if (fconv != NULL) {
+	fz = gzopen(fconv, mode);
+	g_free(fconv);
+    }    
     errno = save_errno;
-    return fz;
 #else
-    return gzopen(filename, mode);
+    fz = gzopen(filename, mode);
 #endif
+
+    return fz;
 }
 
 int gretl_path_prepend (char *file, const char *path)
@@ -561,7 +581,7 @@ static void set_gretl_libpath (const char *path)
 #ifdef WIN32
     strcpy(gretl_paths.libpath, path);
 #else
-# ifdef GLIB2 
+# ifdef USE_GTK2
     const char *sfx = "-gtk2/";
 # else
     const char *sfx = "-gtk1/";
