@@ -55,7 +55,7 @@ static int check_R_matrix (const gretl_matrix *R)
     int err = 0;
 
     m = gretl_matrix_alloc(k, k);
-    if (m == NULL) return GRETL_MATRIX_NOMEM;
+    if (m == NULL) return E_ALLOC;
 
     gretl_matrix_multiply_mod(R, GRETL_MOD_NONE,
 			      R, GRETL_MOD_TRANSPOSE,
@@ -792,9 +792,11 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
     gretl_vector *b = NULL;
     gretl_vector *br = NULL;
     gretl_matrix *Rv = NULL;
-    double F, pval;
+    double test_stat, pval;
     int err, robust, freeRv = 1;
 
+    int asym = ASYMPTOTIC_MODEL(rset->pmod->ci);
+
     *gretl_errmsg = '\0';
 
     err = restriction_set_form_matrices(rset, &R, &q);
@@ -808,7 +810,7 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 #endif
 
     if ((err = check_R_matrix(R))) {
-	if (err == GRETL_MATRIX_SINGULAR) {
+	if (err == E_SINGULAR) {
 	    pputs(prn, _("Matrix inversion failed:\n"
 			 " restrictions may be inconsistent or redundant\n"));
 	} else {
@@ -874,24 +876,31 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 	goto bailout;
     }
     
-    F = gretl_scalar_b_X_b(br, GRETL_MOD_TRANSPOSE, Rv, &err);
+    test_stat = gretl_scalar_b_X_b(br, GRETL_MOD_TRANSPOSE, Rv, &err);
     if (err) {
-	pputs(prn, _("Failed to compute F statistic for test\n"));
+	pputs(prn, _("Failed to compute test statistic\n"));
 	goto bailout;
     }
 
-    F /= rset->k;
-    pval = fdist(F, rset->k, rset->pmod->dfd);
-
     robust = gretl_model_get_int(rset->pmod, "robust");
 
-    pprintf(prn, "\n%s: %s(%d, %d) = %g, ", _("Test statistic"), 
-	    (robust)? _("Robust F"): "F",
-	    rset->k, rset->pmod->dfd, F);
+    if (asym) {
+	pval = chisq(test_stat, rset->k);
+	pprintf(prn, "\n%s: %s(%d) = %g, ", _("Test statistic"), 
+		(robust)? _("Robust chi^2"): "chi^2",
+		rset->k, test_stat);
+    } else {
+	test_stat /= rset->k;
+	pval = fdist(test_stat, rset->k, rset->pmod->dfd);
+	pprintf(prn, "\n%s: %s(%d, %d) = %g, ", _("Test statistic"), 
+		(robust)? _("Robust F"): "F",
+		rset->k, rset->pmod->dfd, test_stat);
+    }
+
     pprintf(prn, _("with p-value = %g\n"), pval);
     pputc(prn, '\n');
 
-    record_test_result(F, pval, "restriction");
+    record_test_result(test_stat, pval, _("restriction"));
 
  bailout:
 
@@ -941,7 +950,7 @@ gretl_restriction_set_finalize (gretl_restriction_set *rset,
 	err = restriction_set_form_matrices(rset, &R, &q);
 	if (!err) {
 	    err = check_R_matrix(R);
-	    if (err == GRETL_MATRIX_SINGULAR) {
+	    if (err == E_SINGULAR) {
 		pputs(prn, _("Matrix inversion failed:\n"
 			     " restrictions may be inconsistent or redundant\n"));
 	    } else if (err) {
