@@ -22,20 +22,18 @@
 #include "objstack.h"
 #include "cmd_private.h"
 #include "libset.h"
+#include "usermat.h"
 
 #include <errno.h>
 
 #ifndef WIN32
 # include <signal.h>
+# ifdef USE_GTK2
+#  include <glib.h>
+#  define USE_GSPAWN
+# endif
 #endif
 
-#if defined(USE_GTK2) && !defined(WIN32)
-# include <glib.h>
-# if GLIB_CHECK_VERSION(2,0,0)
-#  define GLIB2
-# endif /* GLIB_CHECK_VERSION */
-#endif /* GTK2, !WIN32 */
-
 /**
  * date:
  * @nt: observation number (zero-based).
@@ -236,6 +234,89 @@ int gretl_isconst (int t1, int t2, const double *x)
     return ret;
 }
 
+/**
+ * gretl_isunits:
+ * @x: data series to examine.
+ * @t1: starting observation.
+ * @t2: ending observation. 
+ * 
+ * Check whether variable @x equals 1 over the
+ * given sample range (aside from any missing values).
+ *
+ * Returns: 1 if so, otherwise 0.
+ */
+
+int gretl_isunits (int t1, int t2, const double *x)
+{
+    int t, ret = 1;
+
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t]) && x[t] != 1.0) {
+	    ret = 0;
+	    break;
+	}
+    }
+
+    return ret;
+}
+
+ /**
+ * gretl_isdiscrete:
+ * @x: data series to examine.
+ * @t1: starting observation.
+ * @t2: ending observation. 
+ *
+ * Returns: 1 if @x contains only values whose decimal part is 
+ * zero in the given sample range and 0 otherwise (missing 
+ * values are skipped).
+ */
+
+int gretl_isdiscrete (int t1, int t2, const double *x)
+{
+    int t, ret = 1;
+
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t]) && x[t] != floor(x[t])) {
+	    ret = 0;
+	    break;
+	}
+    }
+
+    return ret;
+}
+
+/**
+ * true_const:
+ * @v: index number of variable to test.
+ * @Z: data array.
+ * @pdinfo: dataset information. 
+ * 
+ * Check whether variable Z[v] equals 1 over the sample
+ * range given in @pdinfo, (aside from any missing values).
+ *
+ * Returns: 1 if so, otherwise 0.
+ */
+
+int true_const (int v, const double **Z, const DATAINFO *pdinfo)
+{
+    if (v < 0 || v >= pdinfo->v) {
+	return 0;
+    }
+
+    return gretl_isunits(pdinfo->t1, pdinfo->t2, Z[v]);
+}
+
+/**
+ * gretl_compare_doubles:
+ * @a: pointer to first element to compare.
+ * @b: pointer to second element to compare.
+ *
+ * Comparison function for use with %qsort.  Sorts doubles in
+ * ascending order.
+ * 
+ * Returns: appropriate value for %qsort.
+ */
+
 int gretl_compare_doubles (const void *a, const void *b)
 {
     const double *da = (const double *) a;
@@ -244,6 +325,25 @@ int gretl_compare_doubles (const void *a, const void *b)
     return (*da > *db) - (*da < *db);
 }
 
+/**
+ * gretl_inverse_compare_doubles:
+ * @a: pointer to first element to compare.
+ * @b: pointer to second element to compare.
+ * 
+ * Comparison function for use with %qsort.  Sorts doubles in
+ * descending order.
+ * 
+ * Returns: appropriate value for %qsort.
+ */
+
+int gretl_inverse_compare_doubles (const void *a, const void *b)
+{
+    const double *da = (const double *) a;
+    const double *db = (const double *) b;
+
+    return (*da < *db) - (*da > *db);
+}
+
 /**
  * printlist:
  * @list: array of integers.
@@ -276,65 +376,79 @@ void printlist (const int *list, const char *msg)
 
 /* Compute model selection criteria */
 
-int gretl_calculate_criteria (double ess, int nobs, int ncoeff,
-			      double *ll, double *aic, double *bic)
+int gretl_calculate_criteria (double ess, int n, int k,
+			      double *pll, double *aic, double *bic,
+			      double *hqc)
 {
+    double ll;
+    double c[3];
     int err = 0;
 
-    if (na(ess) || ess <= 0.0 || ncoeff < 1 || nobs <= ncoeff) {
-	*ll = NADBL;
-	*aic = NADBL;
-	*bic = NADBL;
+    if (na(ess) || ess <= 0.0 || k < 1 || n <= k) {
 	err = 1;
     } else {
 	const double ln2pi1 = 2.837877066409345;
 
 	errno = 0;
 
-	*ll = -.5 * nobs * log(ess);
+	ll = -.5 * n * log(ess);
 
 	if (errno == EDOM || errno == ERANGE) {
-	    *ll = NADBL;
-	    *aic = NADBL;
-	    *bic = NADBL;
+	    err = 1;
 	} else {
-	    *ll += -.5 * nobs * (ln2pi1 - log((double) nobs));
-	    *aic = -2.0 * *ll + 2 * ncoeff;
-	    *bic = -2.0 * *ll + ncoeff * log(nobs);
+	    ll += -.5 * n * (ln2pi1 - log((double) n));
+	    c[0] = -2.0 * ll + 2 * k;
+	    c[1] = -2.0 * ll + k * log(n);
+	    c[2] = -2.0 * ll + 2 * k * log(log(n));
 	}
     }
 
+    if (err) {
+	*pll = NADBL;
+	*aic = NADBL;
+	*bic = NADBL;
+	*hqc = NADBL;
+    } else {
+	*pll = ll;
+	*aic = c[0];
+	*bic = c[1];
+	*hqc = c[2];
+    }	
+
     return err;
 }
 
-int ls_aic_bic (MODEL *pmod)
+int ls_criteria (MODEL *pmod)
 {
-    double ll, aic, bic;
+    double ll, aic, bic, hqc;
     int err;
 
+    /* should this be ncoeff + 1 below? (1 for variance) */
+
     err = gretl_calculate_criteria(pmod->ess, pmod->nobs, pmod->ncoeff,
-				   &ll, &aic, &bic);
+				   &ll, &aic, &bic, &hqc);
 
     pmod->lnL = ll;
     pmod->criterion[C_AIC] = aic;
     pmod->criterion[C_BIC] = bic;
+    pmod->criterion[C_HQC] = hqc;
 
     return err;
 }
 
 int gretl_print_criteria (double ess, int nobs, int ncoeff, PRN *prn)
 {
-    double ll, aic, bic;
+    double ll, aic, bic, hqc;
     int err;
 
-    err = gretl_calculate_criteria(ess, nobs, ncoeff, &ll, &aic, &bic);
+    err = gretl_calculate_criteria(ess, nobs, ncoeff, &ll, &aic, &bic, &hqc);
 
     if (err) {
 	pputs(prn, _("Error calculating model selection criteria\n"));
     } else {
 	pprintf(prn, _("Using ess = %g, %d observations, %d coefficients\n"), 
 		ess, nobs, ncoeff);
-	pprintf(prn, "\nAIC = %g\nBIC = %g\n\n", aic, bic);
+	pprintf(prn, "\nAIC = %g\nBIC = %g\nHQC = %g\n\n", aic, bic, hqc);
     }
 
     return err;
@@ -404,6 +518,47 @@ static int get_stobs_maj_min (char *stobs, int *maj, int *min)
     return err;
 }
 
+static int 
+catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
+{
+    int err = 0;
+
+    if (pd == 1) {
+	if (min > 0) {
+	    strcpy(gretl_errmsg, _("no ':' allowed in starting obs with "
+				   "frequency 1"));
+	    err = 1;
+	} else if (opt == OPT_S || opt == OPT_C) {
+	    strcpy(gretl_errmsg, _("panel data must have frequency > 1"));
+	    err = 1;
+	}
+    } else {
+	if (min == 0) {
+	    strcpy(gretl_errmsg, _("starting obs must contain a ':' with "
+				   "frequency > 1"));
+	    err = 1;
+	} else if (min > pd) {
+	    sprintf(gretl_errmsg, 
+		    _("starting obs '%s' is incompatible with frequency"), 
+		    stobs);
+	    err = 1;
+	} else if (opt == OPT_X) {
+	    strcpy(gretl_errmsg, _("cross-sectional data: frequency must be 1"));
+	    err = 1;
+	} else if (n % pd != 0) {
+	    if (opt == OPT_S || opt == OPT_C) {
+		sprintf(gretl_errmsg, _("Panel datasets must be balanced.\n"
+					"The number of observations (%d) is not a multiple\n"
+					"of the number of %s (%d)."), 
+			n, ((opt == OPT_S)? _("periods") : _("units")), pd);
+		err = 1;
+	    }
+	}
+    }
+
+    return err;
+}
+
 #define recognized_ts_frequency(f) (f == 4 || f == 12 || f == 24)
 
 /**
@@ -498,51 +653,9 @@ int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt)
 	    && min == 0 && opt != OPT_X && opt != OPT_S && opt != OPT_C) {
 	    pdinfo->structure = TIME_SERIES;
 	} else {
-	    int balanced = 1;
-	    int err = 0;
-
-	    /* various pathologies */
-
-	    if (pd == 1) {
-		if (min > 0) {
-		    strcpy(gretl_errmsg, _("no ':' allowed in starting obs with "
-					   "frequency 1"));
-		    err = 1;
-		} else if (opt == OPT_S || opt == OPT_C) {
-		    strcpy(gretl_errmsg, _("panel data must have frequency > 1"));
-		    err = 1;
-		}
-	    } else {
-		if (min == 0) {
-		    strcpy(gretl_errmsg, _("starting obs must contain a ':' with "
-					   "frequency > 1"));
-		    err = 1;
-		} else if (min > pd) {
-		    sprintf(gretl_errmsg, 
-			    _("starting obs '%s' is incompatible with frequency"), 
-			    stobs);
-		    err = 1;
-		} else if (opt == OPT_X) {
-		    strcpy(gretl_errmsg, _("cross-sectional data: frequency must be 1"));
-		    err = 1;
-		} else if (pdinfo->n % pd != 0) {
-		    balanced = 0;
-		    if (opt == OPT_S || opt == OPT_C) {
-			sprintf(gretl_errmsg, _("Panel datasets must be balanced.\n"
-						"The number of observations (%d) is not a multiple\n"
-						"of the number of %s (%d)."), 
-				pdinfo->n, ((opt == OPT_S)? _("periods") : _("units")), pd);
-			err = 1;
-		    }
-		}
-	    }
-
-	    if (err) {
+	    if (catch_setobs_errors(stobs, pd, pdinfo->n, min, opt)) {
 		return 1;
 	    }
-
-	    /* OK? */
-		
 	    if (pd == 1) {
 		sprintf(stobs, "%d", maj);
 		if (structure == STRUCTURE_UNKNOWN) {
@@ -554,14 +667,9 @@ int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt)
 		}
 	    } else {
 		real_format_obs(stobs, maj, min, pd, '.');
-		if (structure == STRUCTURE_UNKNOWN) {
-		    if (maj > 1500 && recognized_ts_frequency(pd)) {
-			structure = TIME_SERIES;
-		    } else if (balanced) {
-			structure = STACKED_TIME_SERIES; /* panel? */
-		    } else {
-			structure = TIME_SERIES; /* ?? */
-		    }
+		if (structure == STRUCTURE_UNKNOWN && 
+		    recognized_ts_frequency(pd)) {
+		    structure = TIME_SERIES;
 		}
 	    }
 	}
@@ -579,7 +687,53 @@ int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt)
     return 0;
 }
 
-/* ......................................................  */
+/**
+ * gretl_int_from_string:
+ * @s: string to examine.
+ * @Z: data array.
+ * @pdinfo: data information struct.
+ * @err: location to receive error code.
+ * 
+ * If @s is a valid string representation of an integer,
+ * return that integer, otherwise if @s is the name of a
+ * scalar variable, return the value of that variable,
+ * otherwise set the content of @err to a non-zero value.
+ *
+ * Returns: integer value.
+ */
+
+int gretl_int_from_string (const char *s, const double **Z, 
+			   const DATAINFO *pdinfo, int *err)
+{
+    char *test;
+    int n = 0;
+
+    n = strtol(s, &test, 10);
+
+    if (*test == '\0') {
+	return n;
+    } else if (Z == NULL || pdinfo == NULL) {
+	*err = E_DATA;
+    } else {
+	int v = varindex(pdinfo, s);
+	double x;
+
+	if (v == pdinfo->v) {
+	    *err = E_UNKVAR;
+	} else if (pdinfo->vector[v]) {
+	    *err = E_TYPES;
+	} else {
+	    x = Z[v][0];
+	    if (na(x)) {
+		*err = E_MISSDATA;
+	    } else {
+		n = (int) x;
+	    }
+	} 
+    }
+
+    return n;    
+}
 
 int positive_int_from_string (const char *s)
 {
@@ -651,8 +805,6 @@ double *copyvec (const double *src, int n)
     return targ;
 }
 
-/* ........................................................... */
-
 int re_estimate (char *model_spec, MODEL *tmpmod, 
 		 double ***pZ, DATAINFO *pdinfo) 
 {
@@ -674,8 +826,7 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 
     switch (cmd.ci) {
     case AR:
-	*tmpmod = ar_func(cmd.list, atoi(cmd.param), pZ, 
-			  pdinfo, OPT_NONE, NULL);
+	*tmpmod = ar_func(cmd.list, pZ, pdinfo, OPT_NONE, NULL);
 	break;
     case CORC:
     case HILU:
@@ -683,7 +834,7 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 	rho = estimate_rho(cmd.list, pZ, pdinfo, cmd.ci, 
 			   &err, cmd.opt, NULL);
 	if (!err) {
-	    *tmpmod = lsq(cmd.list, pZ, pdinfo, cmd.ci, 0, rho);
+	    *tmpmod = ar1_lsq(cmd.list, pZ, pdinfo, cmd.ci, 0, rho);
 	}
 	break;
     case HSK:
@@ -703,7 +854,7 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
     case WLS:
     case HCCM:
     case POOLED:
-	*tmpmod = lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt, 0.0);
+	*tmpmod = lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt);
 	break;
     case TSLS:
 	break;
@@ -721,8 +872,6 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
     return err;
 }
 
-/* ........................................................... */
-
 int guess_panel_structure (double **Z, DATAINFO *pdinfo)
 {
     int v, panel;
@@ -852,7 +1001,7 @@ int allocate_mp_varnames (mp_results *mpvals)
     }
 
     for (i=0; i<n; i++) {
-	mpvals->varnames[i] = malloc(12);
+	mpvals->varnames[i] = malloc(VNAMELEN + 8);
 	if (mpvals->varnames[i] == NULL) {
 	    for (j=0; j<i; j++) {
 		free(mpvals->varnames[j]);
@@ -887,7 +1036,9 @@ mp_results *gretl_mp_results_new (int nc)
     int i;
 
     mpvals = malloc(sizeof *mpvals);
-    if (mpvals == NULL) return NULL;
+    if (mpvals == NULL) {
+	return NULL;
+    }
 
     mpvals->ncoeff = nc;
 
@@ -902,8 +1053,10 @@ mp_results *gretl_mp_results_new (int nc)
 	return NULL;
     }
 
-    for (i=0; i<nc; i++) mpvals->coeff[i] = NADBL;
-    for (i=0; i<nc; i++) mpvals->sderr[i] = NADBL;
+    for (i=0; i<nc; i++) {
+	mpvals->coeff[i] = NADBL;
+	mpvals->sderr[i] = NADBL;
+    }
 
     mpvals->sigma = mpvals->ess = NADBL;
     mpvals->rsq = mpvals->fstt = NADBL;
@@ -915,7 +1068,8 @@ mp_results *gretl_mp_results_new (int nc)
     return mpvals;
 }
 
-#if GRETL_GLIB
+#ifndef WIN32
+# ifdef USE_GSPAWN
 
 static int font_not_found (const char *s)
 {
@@ -928,7 +1082,7 @@ static int font_not_found (const char *s)
     }
 }
 
-int gretl_spawn (const char *cmdline)
+int gretl_spawn (char *cmdline)
 {
     GError *error = NULL;
     gchar *errout = NULL, *sout = NULL;
@@ -979,11 +1133,9 @@ int gretl_spawn (const char *cmdline)
     return ret;
 }
 
-#endif
+# else /* now non-glib2 version */
 
-#if !defined(WIN32) && !defined(GRETL_GLIB)
-
-int gretl_spawn (const char *cmdline)
+int gretl_spawn (char *cmdline)
 {
     int err;
 
@@ -1000,7 +1152,8 @@ int gretl_spawn (const char *cmdline)
     return err;
 }
 
-#endif
+# endif
+#endif /* !WIN32 */
 
 /* file copying */
 
@@ -1055,6 +1208,7 @@ void libgretl_cleanup (void)
     libset_cleanup();
     gretl_lists_cleanup();
     gretl_command_hash_cleanup();
+    destroy_user_matrices();
 
     p = strstr(gretl_plotfile(), "gpttmp");
     if (p != NULL) {
@@ -1075,25 +1229,30 @@ enum {
 };
 
 static double
-record_or_get_test_result (double teststat, double pval, char *blurb,
+record_or_get_test_result (double teststat, double pval, char *instr,
 			   int code)
 {
+    static char savestr[MAXLABEL] = {0};
     static double val = NADBL;
     static double pv = NADBL;
-    static char info[MAXLABEL] = {0};
 
     double ret = NADBL;
 
     if (code == SET_TEST_STAT) {
 	val = teststat;
 	pv = pval;
-	*info = '\0';
-	if (blurb != NULL) {
-	    strncat(info, blurb, MAXLABEL - 1);
+	*savestr = '\0';
+	if (instr != NULL) {
+	    strncat(savestr, instr, MAXLABEL - 1);
 	} 
     } else if (code == GET_TEST_STAT || code == GET_TEST_PVAL) {
-	if (blurb != NULL) {
-	    strncat(blurb, info, MAXLABEL - 1);
+	if (instr != NULL) {
+	    if (code == GET_TEST_STAT) {
+		sprintf(instr, _("%s test"), savestr);
+	    } else {
+		/* GET_TEST_PVAL */
+		sprintf(instr, _("p-value for %s test"), savestr);
+	    }
 	}
 	ret = (code == GET_TEST_STAT)? val : pv;
     } 
