@@ -25,6 +25,7 @@
 typedef enum {
     C_AIC,
     C_BIC,
+    C_HQC,
     C_MAX
 } model_selection_criteria;
 
@@ -51,6 +52,12 @@ int gretl_iszero (int t1, int t2, const double *x);
 
 int gretl_isconst (int t1, int t2, const double *x);
 
+int gretl_isunits (int t1, int t2, const double *x);
+
+int gretl_isdiscrete (int t1, int t2, const double *x);
+
+int true_const (int v, const double **Z, const DATAINFO *pdinfo);
+
 /* setting observations */
 
 char *format_obs (char *obs, int maj, int min, int pd);
@@ -61,6 +68,9 @@ int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt);
 
 void printlist (const int *list, const char *msg);
 
+int gretl_int_from_string (const char *s, const double **Z, 
+			   const DATAINFO *pdinfo, int *err);
+
 int positive_int_from_string (const char *s);
 
 int varnum_from_string (const char *str, DATAINFO *pdinfo);
@@ -81,20 +91,23 @@ double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo);
 
 int gretl_compare_doubles (const void *a, const void *b);
 
+int gretl_inverse_compare_doubles (const void *a, const void *b);
+
 int gretl_copy_file (const char *src, const char *dest);
 
 #ifndef WIN32
-int gretl_spawn (const char *cmdline);
+int gretl_spawn (char *cmdline);
 #endif
 
 /* model selection criteria */
 
 int gretl_calculate_criteria (double ess, int nobs, int ncoeff,
-			      double *ll, double *aic, double *bic);
+			      double *ll, double *aic, double *bic, 
+			      double *hqc);
 
 int gretl_print_criteria (double ess, int nobs, int ncoeff, PRN *prn);
 
-int ls_aic_bic (MODEL *pmod);
+int ls_criteria (MODEL *pmod);
 
 /* panel data utilities */
 
