@@ -37,8 +37,12 @@ void win_show_error (DWORD dw);
 int winfork (char *cmdline, const char *dir, int wshow,
 	     DWORD flags);
 
+int gretl_spawn (char *cmdline);
+
 char *desktop_path (void);
 
+int gretl_shell (const char *arg);
+
 #endif /* WIN32 */
 
 #endif /* GRETL_WIN32_H */
