@@ -31,6 +31,35 @@
 
 #undef XML_DEBUG
 
+#ifdef WIN32
+# include <glib.h>
+static xmlDocPtr gretl_xmlParseFile (const char *fname)
+{
+    xmlDocPtr ptr = NULL;
+    FILE *fp = fopen(fname, "r");
+
+    if (fp != NULL) {
+	fclose(fp);
+	ptr = xmlParseFile(fname);
+    } else {
+	int save_errno = errno;
+	gchar *fconv;
+	gsize wrote;
+
+	fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
+	if (fconv != NULL) {
+	    ptr = xmlParseFile(fconv);
+	    g_free(fconv);
+	}
+	errno = save_errno;
+    }
+
+    return ptr;
+}
+#else
+# define gretl_xmlParseFile(f) xmlParseFile(f)
+#endif
+
 static char *compact_method_to_string (int method)
 {
     if (method == COMPACT_SUM) return "COMPACT_SUM";
@@ -129,7 +158,7 @@ int gretl_write_gdt (const char *fname, const int *list,
     int tsamp = pdinfo->t2 - pdinfo->t1 + 1;
     int *pmax = NULL;
     char startdate[OBSLEN], enddate[OBSLEN];
-    char datname[MAXLEN], freqstr[16];
+    char datname[MAXLEN], freqstr[32];
     char numstr[128];
     char *xmlbuf = NULL;
     void *handle = NULL;
@@ -206,7 +235,7 @@ int gretl_write_gdt (const char *fname, const int *list,
     }
 
     if (custom_time_series(pdinfo)) {
-	strcpy(freqstr, "special");
+	sprintf(freqstr, "special:%d", pdinfo->pd);
     } else {
 	sprintf(freqstr, "%d", pdinfo->pd);
     }
@@ -743,8 +772,13 @@ static int xml_get_data_frequency (xmlNodePtr node, int *pd, int *dtype)
     *pd = 1;
 
     if (tmp != NULL) {
-	if (!strcmp((char *) tmp, "special")) {
+	if (!strncmp((char *) tmp, "special", 7)) {
 	    *dtype = SPECIAL_TIME_SERIES;
+	    if (sscanf((char *) tmp + 7, ":%d", pd) == 1) {
+		fprintf(stderr, "custom time series, frequency %d\n", *pd);
+	    } else {
+		fprintf(stderr, "custom time series, using frequency 1\n");
+	    }
 	} else if (sscanf((char *) tmp, "%d", pd) != 1) {
 	    strcpy(gretl_errmsg, _("Failed to parse data frequency"));
 	    err = 1;
@@ -893,7 +927,7 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
 	if (gui) progress = fsz;
     }
 
-    doc = xmlParseFile(fname);
+    doc = gretl_xmlParseFile(fname);
     if (doc == NULL) {
 	sprintf(gretl_errmsg, _("xmlParseFile failed on %s"), fname);
 	err = 1;
@@ -1057,7 +1091,7 @@ char *gretl_get_gdt_description (const char *fname)
     LIBXML_TEST_VERSION
 	xmlKeepBlanksDefault(0);
 
-    doc = xmlParseFile(fname);
+    doc = gretl_xmlParseFile(fname);
     if (doc == NULL) {
 	sprintf(gretl_errmsg, _("xmlParseFile failed on %s"), fname);
 	return NULL;
