@@ -73,9 +73,8 @@
 #define MAXLEN   512  /* max length of regular "long" strings */
 #define ERRLEN   256  /* max length of libgretl error messages */
 #define MAXDISP   32  /* max length of "display names" for variables */
-#define VNAMELEN  14  /* space allocated for var names (including termination) */
-#define USER_VLEN  9  /* max allocation for varname constructed by user */
-#define OBSLEN    11  /* space allocated for obs strings (including termination) */
+#define VNAMELEN  16  /* space allocated for var names (including termination) */
+#define OBSLEN    16  /* space allocated for obs strings (including termination) */
 
 #ifndef M_PI
 # define M_PI 3.14159265358979323846
@@ -156,7 +155,7 @@ enum gretl_opt_flags {
     OPT_W = 1 << 18,
     OPT_X = 1 << 19,
     OPT_Z = 1 << 20,
-    OPT_UNSET = ULONG_MAX
+    OPT_UNSET = 1 << 30
 };
 
 typedef enum {
@@ -165,9 +164,35 @@ typedef enum {
     OP_LT  = '<',
     OP_NEQ = 21,
     OP_GTE = 22,
-    OP_LTE = 23
+    OP_LTE = 23,
+    /* matrix operators */
+    OP_DOTMULT = 24,
+    OP_DOTDIV  = 25,
+    OP_DOTPOW  = 26,
+    OP_KRON    = 27,
 } GretlOp;
 
+typedef enum {
+    T_NONE,
+    T_LOG, 
+    T_EXP, 
+    T_SIN, 
+    T_COS,
+    T_TAN,
+    T_ATAN,
+    T_INT, 
+    T_ABS, 
+    T_SQRT, 
+    T_DNORM,
+    T_CNORM,
+    T_QNORM,
+    T_GAMMA,
+    T_LNGAMMA,
+    T_NORMAL, 
+    T_UNIFORM, 
+    T_MATHMAX
+} GretlMathFunc;
+
 typedef unsigned long gretlopt;
 
 typedef struct VARINFO_ VARINFO;
@@ -212,6 +237,7 @@ struct DATAINFO_ {
 			   observation markers */
     char delim;         /* default delimiter for "CSV" files */
     char decpoint;      /* character used to represent decimal point */
+    char submode;       /* mode of sub-sampling in force, if any */
     char **S;           /* to hold observation markers */
     char *descrip;      /* to hold info on data sources etc. */
     char *vector;       /* hold info on vars: vector versus scalar */
@@ -302,7 +328,7 @@ struct MODEL_ {
     double fstt;                 /* F-statistic */
     double lnL;                  /* log-likelihood */
     double ybar, sdy;            /* mean and std. dev. of dependent var. */
-    double criterion[2];         /* array of model selection statistics */
+    double criterion[3];         /* array of model selection statistics */
     double dw, rho;              /* Durbin-Watson stat. and estimated 1st
 				    order autocorrelation coefficient */
     ARINFO *arinfo;              /* pointer to struct to hold special info for 
@@ -353,7 +379,6 @@ struct mp_results_ {
 #include "dataset.h"
 #include "estimate.h"
 #include "generate.h"
-#include "genrfuncs.h"
 #include "compare.h"
 #include "gretl_intl.h"
 #include "gretl_list.h"
