@@ -22,6 +22,8 @@
 #include "libgretl.h"
 #include "libset.h"
 
+#define PDEBUG 0
+
 enum {
     AUTO_LAG_STOCK_WATSON,
     AUTO_LAG_WOOLDRIDGE
@@ -53,13 +55,20 @@ struct bkbp_opts {
 struct sample_info {
     int t1;
     int t2;
-    int subsampled;
+    char submode;
+    char *submask;
 };
     
+struct bhhh_opts {
+    double tolerance;
+    int maxiter;
+};
+
 struct set_vars_ {
     int use_qr;                 /* use QR decomposition? */
     unsigned int seed;          /* for PRNG */
     int halt_on_error;          /* halt cli program on script error? */
+    int shell_ok;               /* shell commands permitted? */
     double hp_lambda;           /* for Hodrick-Prescott filter */
     int horizon;                /* for VAR impulse responses */ 
     double nls_toler;           /* NLS convergence criterion */
@@ -70,13 +79,12 @@ struct set_vars_ {
     struct garch_opts gopts;    /* GARCH covariance matrix */
     struct bkbp_opts bkopts;    /* Baxter-King filter */
     struct sample_info sinfo;   /* record of dataset sample state */
+    struct bhhh_opts maxopts;   /* options for BHHH maximisation */
 };
 
 /* global state */
 set_vars *state;
 
-#define check_for_state() (state != NULL || libset_init())
-
 static void robust_opts_init (struct robust_opts *opts)
 {
     opts->auto_lag = AUTO_LAG_STOCK_WATSON;
@@ -119,20 +127,44 @@ static void bkbp_opts_copy (struct bkbp_opts *opts)
     opts->periods[1] = state->bkopts.periods[1];
 }
 
+static void bhhh_opts_init (struct bhhh_opts *opts)
+{
+    opts->tolerance = NADBL;
+    opts->maxiter = 500;
+}
+
+static void bhhh_opts_copy (struct bhhh_opts *opts)
+{
+    opts->tolerance = state->maxopts.tolerance;
+    opts->maxiter = state->maxopts.maxiter;
+}
+
 static void sample_info_init (struct sample_info *sinfo)
 {
     sinfo->t1 = UNSET_INT;
     sinfo->t2 = UNSET_INT;
-    sinfo->subsampled = 0;
+    sinfo->submask = NULL;
+    sinfo->submode = 0;
 }
 
 #define sinfo_is_set(s) (s.t1 != UNSET_INT && s.t2 != UNSET_INT)
 
+static void check_for_state (void) 
+{
+    if (state != NULL) {
+	libset_init();
+    }
+}
+
 static void state_vars_copy (set_vars *sv, const DATAINFO *pdinfo)
 {
+#if PDEBUG
+    fprintf(stderr, "state_vars_copy called: pdinfo=%p\n", (void *) pdinfo);
+#endif
     sv->use_qr = state->use_qr;
     sv->seed = state->seed;
     sv->halt_on_error = state->halt_on_error;
+    sv->shell_ok = state->shell_ok;
     sv->hp_lambda = state->hp_lambda;
     sv->horizon = state->horizon;
     sv->nls_toler = state->nls_toler;
@@ -143,11 +175,19 @@ static void state_vars_copy (set_vars *sv, const DATAINFO *pdinfo)
     robust_opts_copy(&sv->ropts);
     garch_opts_copy(&sv->gopts);
     bkbp_opts_copy(&sv->bkopts);
+    bhhh_opts_copy(&sv->maxopts);
 
     if (pdinfo != NULL) {
 	sv->sinfo.t1 = pdinfo->t1;
 	sv->sinfo.t2 = pdinfo->t2;
-	sv->sinfo.subsampled = complex_subsampled();
+	sv->sinfo.submode = pdinfo->submode;
+	sv->sinfo.submask = copy_datainfo_submask(pdinfo);
+#if PDEBUG
+	fprintf(stderr, " sinfo.t1=%d, sinfo.t2=%d, "
+		"sinfo.submode=%d, sinfo.submask=%p\n", sv->sinfo.t1,
+		sv->sinfo.t2, sv->sinfo.submode,
+		(void *) sv->sinfo.submask);
+#endif
     } else {
 	sample_info_init(&sv->sinfo);
     }
@@ -155,9 +195,13 @@ static void state_vars_copy (set_vars *sv, const DATAINFO *pdinfo)
 
 static void state_vars_init (set_vars *sv)
 {
+#if PDEBUG
+    fprintf(stderr, "state_vars_init called\n");
+#endif
     sv->use_qr = UNSET_INT; 
     sv->seed = 0;
     sv->halt_on_error = UNSET_INT;
+    sv->shell_ok = 0;
     sv->hp_lambda = NADBL;
     sv->horizon = UNSET_INT;
     sv->nls_toler = NADBL;
@@ -168,6 +212,7 @@ static void state_vars_init (set_vars *sv)
     robust_opts_init(&sv->ropts);
     garch_opts_init(&sv->gopts);
     bkbp_opts_init(&sv->bkopts);
+    bhhh_opts_init(&sv->maxopts);
     sample_info_init(&sv->sinfo);
 }
 
@@ -196,6 +241,49 @@ void get_bkbp_periods (int *periods)
     periods[1] = state->bkopts.periods[1];
 }
 
+double get_bhhh_toler (void)
+{
+    check_for_state();
+    return state->maxopts.tolerance;
+}
+
+int get_bhhh_maxiter (void)
+{
+    check_for_state();
+    return state->maxopts.maxiter;
+}
+
+int set_bhhh_toler (double tol)
+{
+    int err = 0;
+
+    check_for_state();
+
+    if (tol <= 0.0) {
+	err = 1;
+    } else {
+	state->maxopts.tolerance = tol;
+    }
+
+    return err;
+}
+
+int set_bhhh_maxiter (int n)
+{
+    int err = 0;
+
+    check_for_state();
+
+    if (n < 1) {
+	err = 1;
+    } else {
+	state->maxopts.maxiter = n;
+    }
+
+    return err;
+}
+
+
 int get_VAR_horizon (void)
 {
     check_for_state();
@@ -541,8 +629,25 @@ static char delim_from_arg (const char *s)
     return ret;
 }
 
+static const char *arg_from_delim (char c)
+{
+    const char *ret = "unset";
+
+    if (c == ',') {
+	ret = "comma";
+    } else if (c == ' ') {
+	ret = "space";
+    } else if (c == '\t') {
+	ret = "tab";
+    }
+
+    return ret;
+
+}
+
 static int display_settings (PRN *prn)
 {
+    double dval;
     unsigned int uval;
     int ival;
 
@@ -579,16 +684,33 @@ static int display_settings (PRN *prn)
 	pprintf(prn, " horizon = %d\n", state->horizon);
     }
 
-    /* undocumented! */
     pprintf(prn, " nls_toler = %g\n", get_nls_toler());
+
+    dval = get_bhhh_toler();
+    if (na(dval)) {
+	pputs(prn, " bhhh_toler = default\n");
+    } else {
+	pprintf(prn, " bhhh_toler = %g\n", dval);
+    }
+
+    pprintf(prn, " bhhh_maxiter = %d\n", get_bhhh_maxiter());
     pprintf(prn, " messages = %d\n", state->gretl_msgs);
 
     ival =  get_halt_on_error(); /* checks env */
     pprintf(prn, " halt_on_error = %d\n", state->halt_on_error);
 
+    pprintf(prn, " shell_ok = %d\n", state->shell_ok);
+    pprintf(prn, " csv_delim = %s\n", arg_from_delim(state->delim));
+
     return 0;
 }
 
+#define boolean_on(s) (!strcmp(s, "on") || !strcmp(s, "1") || \
+                       !strcmp(s, "true"))
+
+#define boolean_off(s) (!strcmp(s, "off") || !strcmp(s, "0") || \
+                        !strcmp(s, "false"))
+
 int execute_set_line (const char *line, PRN *prn)
 {
     char setobj[16], setarg[16], setarg2[16];
@@ -619,18 +741,18 @@ int execute_set_line (const char *line, PRN *prn)
 
 	/* set command echo on/off */
 	if (!strcmp(setobj, "echo")) {
-	    if (!strcmp(setarg, "off")) {
+	    if (boolean_off(setarg)) {
 		state->gretl_echo = 0;
 		err = 0;
-	    } else if (!strcmp(setarg, "on")) {
+	    } else if (boolean_on(setarg)) {
 		state->gretl_echo = 1;
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "messages")) {
-	    if (!strcmp(setarg, "off")) {
+	    if (boolean_off(setarg)) {
 		state->gretl_msgs = 0;
 		err = 0;
-	    } else if (!strcmp(setarg, "on")) {
+	    } else if (boolean_on(setarg)) {
 		state->gretl_msgs = 1;
 		err = 0;
 	    }
@@ -660,10 +782,10 @@ int execute_set_line (const char *line, PRN *prn)
 	    err = parse_hc_variant(setarg);
 	} else if (!strcmp(setobj, "force_hc")) {
 	    /* use HCCM, not HAC, even for time series */
-	    if (!strcmp(setarg, "on")) { 
+	    if (boolean_on(setarg)) { 
 		set_force_hc(1);
 		err = 0;
-	    } else if (!strcmp(setarg, "off")) { 
+	    } else if (boolean_off(setarg)) { 
 		set_force_hc(0);
 		err = 0;
 	    }
@@ -672,21 +794,23 @@ int execute_set_line (const char *line, PRN *prn)
 	    err = set_garch_vcv_variant(setarg);
 	} else if (!strcmp(setobj, "qr")) {
 	    /* switch QR vs Cholesky decomposition */
-	    if (!strcmp(setarg, "on")) {
+	    if (boolean_on(setarg)) {
 		state->use_qr = 1;
 		err = 0;
-	    } else if (!strcmp(setarg, "off")) {
+	    } else if (boolean_off(setarg)) {
 		state->use_qr = 0;
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "halt_on_error")) {
-	    if (!strcmp(setarg, "on")) {
+	    if (boolean_on(setarg)) {
 		state->halt_on_error = 1;
 		err = 0;
-	    } else if (!strcmp(setarg, "off")) {
+	    } else if (boolean_off(setarg)) {
 		state->halt_on_error = 0;
 		err = 0;
 	    }
+	} else if (!strcmp(setobj, "shell_ok")) {
+	    pprintf(prn, "You can only set this variable via the gretl GUI\n");
 	} else if (!strcmp(setobj, "seed")) {
 	    /* seed for PRNG */
 	    if (isdigit(*setarg)) {
@@ -735,6 +859,21 @@ int execute_set_line (const char *line, PRN *prn)
 	    if (sscanf(setarg, "%lf", &tol)) {
 		err = set_nls_toler(tol);
 	    }
+	} else if (!strcmp(setobj, "bhhh_toler")) {
+	    /* Tolerance for BHHH (ARMA, Tobit) */
+	    double tol;
+
+	    if (!strcmp(setarg, "default")) {
+		set_bhhh_toler(NADBL);
+		err = 0;
+	    } else if (sscanf(setarg, "%lf", &tol)) {
+		err = set_bhhh_toler(tol);
+	    }
+	} else if (!strcmp(setobj, "bhhh_maxiter")) {
+	    /* Maximum iterations for BHHH (ARMA, Tobit) */
+	    if (isdigit(*setarg)) {
+		err = set_bhhh_maxiter(atoi(setarg));
+	    }
 	}
     } else if (nw == 3) {
 	if (!strcmp(setobj, "bkbp_limits")) {
@@ -787,7 +926,55 @@ int get_halt_on_error (void)
     return state->halt_on_error;
 }
 
-/* Mechanism for pushing and popping program state for new-style
+#ifndef WIN32
+static int read_cli_shell_status (void)
+{
+    char shellstamp[FILENAME_MAX];
+    FILE *fp;
+    int ok = 0;
+
+    sprintf(shellstamp, "%s.gretl_shell_stamp", gretl_user_dir());
+    fp = fopen(shellstamp, "r");
+    if (fp != NULL) {
+	ok = 1;
+	fclose(fp);
+    }
+
+    return ok;
+}
+#endif
+
+void set_shell_ok (int set)
+{
+    check_for_state();
+    state->shell_ok = set;
+}
+
+int get_shell_ok (void)
+{
+    check_for_state();
+
+#ifndef WIN32
+    if (!gretl_in_gui_mode()) {
+	state->shell_ok = read_cli_shell_status();
+    }
+#endif
+
+    return state->shell_ok;
+}
+
+static void update_sample_info (set_vars *sv, const DATAINFO *pdinfo)
+{
+    if (pdinfo != NULL) {
+	sv->sinfo.t1 = pdinfo->t1;
+	sv->sinfo.t2 = pdinfo->t2;
+	sv->sinfo.submode = pdinfo->submode;
+	free(sv->sinfo.submask);
+	sv->sinfo.submask = copy_datainfo_submask(pdinfo);
+    }	 
+}
+
+/* Mechanism for pushing and popping program state for user-defined
    functions. push_program_state() is used when a function starts
    execution: the function gets a copy of the current program state,
    while that state is pushed onto the stack for restoration when the
@@ -799,17 +986,29 @@ static set_vars **state_stack;
 
 int push_program_state (const DATAINFO *pdinfo)
 {
-    set_vars *mystate, **sstack;
+    set_vars **sstack;
+    set_vars *newstate;
     int ns = n_states;
     int err = 0;
 
-    mystate = malloc(sizeof *mystate);
-    if (mystate == NULL) {
+#if PDEBUG
+    fprintf(stderr, "push_program_state: ns=%d\n", ns);
+    if (pdinfo != NULL) {
+	fprintf(stderr, " pdinfo=%p, t1=%d, t2=%d\n",
+		(void *) pdinfo, pdinfo->t1, pdinfo->t2);
+    } else {
+	fprintf(stderr, " pdinfo is NULL\n");
+    }
+#endif
+
+    newstate = malloc(sizeof *newstate);
+
+    if (newstate == NULL) {
 	err = 1;
     } else {
 	sstack = realloc(state_stack, (ns + 1) * sizeof *sstack);
 	if (sstack == NULL) {
-	    free(mystate);
+	    free(newstate);
 	    err = 1;
 	}
     }
@@ -817,19 +1016,55 @@ int push_program_state (const DATAINFO *pdinfo)
     if (!err) {
 	if (ns == 0) {
 	    /* set all defaults */
-	    state_vars_init(mystate);
+	    state_vars_init(newstate);
 	} else {
+	    /* set sample on existing state */
+	    update_sample_info(state, pdinfo);
 	    /* copy existing state */
-	    state_vars_copy(mystate, pdinfo);
+	    state_vars_copy(newstate, pdinfo);
 	}
 	state_stack = sstack;
-	state = state_stack[ns] = mystate;
+	state = state_stack[ns] = newstate;
 	n_states++;
     }
 
+#if PDEBUG
+    if (!err) {
+	fprintf(stderr, " state is now state_stack[%d]\n", ns);
+    }
+#endif
+
     return err;
 }
 
+static int 
+subsampled_at_this_level (const DATAINFO *pdinfo, set_vars *sv)
+{
+    int ret = 0;
+
+    if (!complex_subsampled()) {
+	return 0;
+    }
+
+    if (pdinfo->submask != NULL) {
+	if (sv->sinfo.submask == NULL) {
+	    ret = 1;
+	} else if (submask_cmp(pdinfo->submask, sv->sinfo.submask)) {
+	    ret = 1;
+	}
+    } 
+
+    return ret;
+}
+
+static void free_state (set_vars *sv)
+{
+    if (sv->sinfo.submask != NULL) {
+	free(sv->sinfo.submask);
+    }
+    free(sv);
+}
+
 /* Called when a new-style function exits: restores the program state
    that was in force when the function started executing.  This may
    involve putting the sample back the way it was.  But if the dataset
@@ -838,16 +1073,21 @@ int push_program_state (const DATAINFO *pdinfo)
    requires more careful thought.
 */
 
-int pop_program_state (DATAINFO *pdinfo)
+int pop_program_state (double ***pZ, DATAINFO **ppdinfo)
 {
     set_vars **sstack;
     int ns = n_states;
     int err = 0;
 
+#if PDEBUG
+    fprintf(stderr, "pop_program_state called: ns=%d, pdinfo=%p\n",
+	    ns, (void *) *ppdinfo);
+#endif
+
     if (ns < 2) {
 	err = 1;
     } else {
-	free(state_stack[ns - 1]);
+	free_state(state_stack[ns - 1]);
 	state_stack[ns - 1] = NULL;
 	sstack = realloc(state_stack, (ns - 1) * sizeof *sstack);
 	if (sstack == NULL) {
@@ -861,15 +1101,35 @@ int pop_program_state (DATAINFO *pdinfo)
 	n_states--;
     }
 
-    if (!err && pdinfo != NULL && sinfo_is_set(state->sinfo)) {
-	/* restore original t1, t2 if complex subsample in force
-	   neither before nor after the function call */
-	if (!complex_subsampled() && !state->sinfo.subsampled) {
-	    pdinfo->t1 = state->sinfo.t1;
-	    pdinfo->t2 = state->sinfo.t2;
+#if PDEBUG
+    fprintf(stderr, " state is now state_stack[%d]\n", ns - 2);
+#endif
+
+    if (!err && *ppdinfo != NULL && sinfo_is_set(state->sinfo)) {
+	if (subsampled_at_this_level(*ppdinfo, state)) {
+#if PDEBUG
+	    fprintf(stderr, " undoing current sub-sampling\n");
+#endif
+	    restore_full_sample(pZ, ppdinfo);
+	    if (state->sinfo.submask != NULL) {
+#if PDEBUG
+		fprintf(stderr, " reimposing outer sample restriction\n");
+#endif
+		restrict_sample_from_mask(state->sinfo.submask, state->sinfo.submode, 
+					  pZ, ppdinfo);
+	    }
 	}
+	(*ppdinfo)->t1 = state->sinfo.t1;
+	(*ppdinfo)->t2 = state->sinfo.t2;
     }
 
+#if PDEBUG
+    if (*ppdinfo != NULL) {
+	fprintf(stderr, " err=%d, pdinfo->t1=%d, pdinfo->t2=%d\n",
+		err, (*ppdinfo)->t1, (*ppdinfo)->t2);
+    } 
+#endif
+
     return err;
 }
 
@@ -880,6 +1140,10 @@ int libset_init (void)
     static int done;
     int err = 0;
 
+#if PDEBUG
+    fprintf(stderr, "libset_init called, done=%d\n", done);
+#endif
+
     if (!done) {
 	err = push_program_state(NULL);
 	done = 1;
@@ -893,7 +1157,7 @@ void libset_cleanup (void)
     int i;
 
     for (i=0; i<n_states; i++) {
-	free(state_stack[i]);
+	free_state(state_stack[i]);
     }
 
     free(state_stack);
@@ -901,13 +1165,13 @@ void libset_cleanup (void)
     n_states = 0;
 }
 
-int libset_restore_state_zero (DATAINFO *pdinfo)
+int libset_restore_state_zero (double ***pZ, DATAINFO **ppdinfo)
 {
     int i, ns = n_states;
     int err = 0;
 
     for (i=ns; i>=2 && !err; i--) {
-	err = pop_program_state(pdinfo);
+	err = pop_program_state(pZ, ppdinfo);
     }
 
     return err;
