@@ -31,14 +31,17 @@ enum vcv_codes {
 
 int libset_init (void);
 void libset_cleanup (void);
-int libset_restore_state_zero (DATAINFO *pdinfo);
+int libset_restore_state_zero (double ***pZ, DATAINFO **ppdinfo);
 
 int push_program_state (const DATAINFO *pdinfo);
-int pop_program_state (DATAINFO *pdinfo);
+int pop_program_state (double ***pZ, DATAINFO **ppdinfo);
 
 void set_use_qr (int set);
 int get_use_qr (void);
 
+void set_shell_ok (int set);
+int get_shell_ok (void);
+
 void set_xsect_hccme (const char *s);
 void set_tseries_hccme (const char *s);
 
@@ -59,6 +62,11 @@ void get_bkbp_periods (int *periods);
 
 int gretl_get_text_pause (void);
 
+double get_bhhh_toler (void);
+int get_bhhh_maxiter (void);
+int set_bhhh_toler (double tol);
+int set_bhhh_maxiter (int n);
+
 int get_VAR_horizon (void);
 
 double get_nls_toler (void);
