@@ -49,7 +49,6 @@ static void print_discrete_statistics (const MODEL *pmod,
 static void print_arma_stats (const MODEL *pmod, PRN *prn);
 static void print_arma_roots (const MODEL *pmod, PRN *prn);
 static void print_tobit_stats (const MODEL *pmod, PRN *prn);
-static void print_poisson_stats (const MODEL *pmod, PRN *prn);
 static void print_poisson_offset (const MODEL *pmod, const DATAINFO *pdinfo, 
 				  PRN *prn);
 static void print_ll (const MODEL *pmod, PRN *prn);
@@ -109,55 +108,32 @@ static void garch_variance_line (const MODEL *pmod, PRN *prn)
 
 static int essline (const MODEL *pmod, PRN *prn, int wt)
 {
+    double ess = (wt)? pmod->ess_wt : pmod->ess;
+    double sigma = (wt)? pmod->sigma_wt : pmod->sigma;
+
+    if (ess < 0) {
+	if (plain_format(prn)) {
+	    pprintf(prn, _("Error sum of squares (%g) is not > 0"), ess);
+	    pputs(prn, "\n\n");
+	}
+	return 1;
+    }
+
     if (plain_format(prn)) {    
-	if ((wt && pmod->ess_wt < 0) || (!wt && pmod->ess < 0)) {
-	    char tmp[128];
-
-	    sprintf(tmp, _("Error sum of squares (%g) is not > 0"), 
-		    (wt)? pmod->ess_wt : pmod->ess), 
-		pprintf(prn, "%s\n\n", tmp);
-	    return 1;
-	}
-
-	if (pmod->ci == ARMA) {
-	    pprintf(prn, "  %s = %.*g\n", _("Mean of residuals"), 
-		    GRETL_DIGITS, gretl_model_get_double(pmod, "mean_error"));
-	}
-
 	pprintf(prn, "  %s = %.*g\n", _("Sum of squared residuals"), 
-		GRETL_DIGITS, wt? pmod->ess_wt : pmod->ess);
+		GRETL_DIGITS, ess);
 	pprintf(prn, "  %s = %.*g\n", _("Standard error of residuals"), 
-		GRETL_DIGITS, wt? pmod->sigma_wt : pmod->sigma);
+		GRETL_DIGITS, sigma);
     } else if (rtf_format(prn)) {
-	if ((wt && pmod->ess_wt < 0) || (!wt && pmod->ess < 0)) {
-	    char tmp[128];
-
-	    sprintf(tmp, I_("Error sum of squares (%g) is not > 0"), 
-		    (wt)? pmod->ess_wt : pmod->ess); 
-	    pprintf(prn, "\\par \\ql %s\\par\n\n", tmp);
-	    return 1;
-	}
-
-	if (pmod->ci == ARMA) {
-	    pprintf(prn, RTFTAB "%s = %g\n", I_("Mean of residuals"), 
-		    gretl_model_get_double(pmod, "mean_error"));
-	}
-
 	pprintf(prn, RTFTAB "%s = %g\n", I_("Sum of squared residuals"), 
-		wt? pmod->ess_wt : pmod->ess);
+		ess);
 	pprintf(prn, RTFTAB "%s = %g\n", I_("Standard error of residuals"), 
-		wt? pmod->sigma_wt : pmod->sigma);
+		sigma);
     } else if (tex_format(prn)) {
 	char x1str[32], x2str[32];
 
-	if (pmod->ci == ARMA) {
-	    tex_dcolumn_double(gretl_model_get_double(pmod, "mean_error"), x1str);
-	    pprintf(prn, "%s & %s \\\\\n", I_("Mean of residuals"), 
-		    x1str);
-	}
-
-	tex_dcolumn_double(pmod->ess, x1str);
-	tex_dcolumn_double(pmod->sigma, x2str);
+	tex_dcolumn_double(ess, x1str);
+	tex_dcolumn_double(sigma, x2str);
 	pprintf(prn, "%s & %s \\\\\n%s ($\\hat{\\sigma}$) & %s \\\\\n",
 		I_("Sum of squared residuals"), x1str,
 		I_("Standard error of residuals"), x2str);
@@ -168,19 +144,19 @@ static int essline (const MODEL *pmod, PRN *prn, int wt)
 
 static void rsqline (const MODEL *pmod, PRN *prn)
 {
+    if (na(pmod->rsq)) {
+	return;
+    }
+
     if (plain_format(prn)) { 
-	if (!na(pmod->rsq)) {
-	    pprintf(prn, "  %s = %.*g\n", _("Unadjusted R-squared"), 
-		    GRETL_DIGITS, pmod->rsq);
-	}
+	pprintf(prn, "  %s = %.*g\n", _("Unadjusted R-squared"), 
+		GRETL_DIGITS, pmod->rsq);
 	if (!NO_RBAR_SQ(pmod->aux) && !na(pmod->adjrsq)) {
 	    pprintf(prn, "  %s = %.*g\n", _("Adjusted R-squared"),  
 		    GRETL_DIGITS, pmod->adjrsq);
 	}
     } else if (rtf_format(prn)) {
-	if (!na(pmod->rsq)) {
-	    pprintf(prn, RTFTAB "%s = %g\n", I_("Unadjusted R{\\super 2}"), pmod->rsq);
-	}
+	pprintf(prn, RTFTAB "%s = %g\n", I_("Unadjusted R{\\super 2}"), pmod->rsq);
 	if (!NO_RBAR_SQ(pmod->aux) && !na(pmod->adjrsq)) {
 	    pprintf(prn, RTFTAB "%s = %g\n", I_("Adjusted R{\\super 2}"),  
 		    pmod->adjrsq);
@@ -188,10 +164,8 @@ static void rsqline (const MODEL *pmod, PRN *prn)
     } else if (tex_format(prn)) {  
 	char r2[32];
 
-	if (!na(pmod->rsq)) {
-	    tex_dcolumn_double(pmod->rsq, r2);
-	    pprintf(prn, "%s & %s \\\\\n", I_("Unadjusted $R^2$"), r2);
-	}
+	tex_dcolumn_double(pmod->rsq, r2);
+	pprintf(prn, "%s & %s \\\\\n", I_("Unadjusted $R^2$"), r2);
 	if (!NO_RBAR_SQ(pmod->aux) && !na(pmod->adjrsq)) {
 	    tex_dcolumn_double(pmod->adjrsq, r2);
 	    pprintf(prn, "%s & %s \\\\\n", I_("Adjusted $\\bar{R}^2$"), r2);
@@ -201,8 +175,12 @@ static void rsqline (const MODEL *pmod, PRN *prn)
 
 static const char *aic_str = N_("Akaike information criterion");
 static const char *bic_str = N_("Schwarz Bayesian criterion");
+static const char *hqc_str = N_("Hannan-Quinn criterion");
+static const char *tex_hqc_str = N_("Hannan--Quinn criterion");
+
 static const char *aic_abbrev = N_("AIC");
 static const char *bic_abbrev = N_("BIC");
+static const char *hqc_abbrev = N_("HQC");
 
 static void info_stats_lines (const MODEL *pmod, PRN *prn)
 {
@@ -214,7 +192,7 @@ static void info_stats_lines (const MODEL *pmod, PRN *prn)
 	return;
     }
 
-    if (na(crit[C_AIC]) || na(crit[C_BIC])) {
+    if (na(crit[C_AIC]) || na(crit[C_BIC]) || na(crit[C_HQC])) {
 	return;
     }
 
@@ -223,9 +201,12 @@ static void info_stats_lines (const MODEL *pmod, PRN *prn)
 		GRETL_DIGITS, crit[C_AIC]);
 	pprintf(prn, "  %s (%s) = %.*g\n", _(bic_str), _(bic_abbrev),
 		GRETL_DIGITS, crit[C_BIC]);
+	pprintf(prn, "  %s (%s) = %.*g\n", _(hqc_str), _(hqc_abbrev),
+		GRETL_DIGITS, crit[C_HQC]);
     } else if (rtf_format(prn)) {
 	pprintf(prn, RTFTAB "%s = %g\n", I_(aic_str), crit[C_AIC]);
 	pprintf(prn, RTFTAB "%s = %g\n", I_(bic_str), crit[C_BIC]);
+	pprintf(prn, RTFTAB "%s = %g\n", I_(hqc_str), crit[C_HQC]);
     } else if (tex_format(prn)) {  
 	char cval[32];
 
@@ -233,6 +214,8 @@ static void info_stats_lines (const MODEL *pmod, PRN *prn)
 	pprintf(prn, "%s & %s \\\\\n", I_(aic_str), cval);
 	tex_dcolumn_double(crit[C_BIC], cval);
 	pprintf(prn, "%s & %s \\\\\n", I_(bic_str), cval);
+	tex_dcolumn_double(crit[C_HQC], cval);
+	pprintf(prn, "%s & %s \\\\\n", I_(tex_hqc_str), cval);
     }
 }
 
@@ -542,6 +525,9 @@ my_estimator_string (const MODEL *pmod, PRN *prn)
     if (pmod->ci == ARMA) {
 	if (gretl_model_get_int(pmod, "armax")) {
 	    return N_("ARMAX");
+	} else if (gretl_model_get_int(pmod, "arima_d") ||
+		   gretl_model_get_int(pmod, "arima_D")) {
+	    return N_("ARIMA");
 	} else {
 	    return N_("ARMA");
 	}
@@ -619,7 +605,7 @@ print_tsls_instruments (const int *list, const DATAINFO *pdinfo, PRN *prn)
 		}
 		pprintf(prn, "%s ", vname);
 		ccount += strlen(vname) + 1;
-		if (ccount >= 76) {
+		if (ccount >= 64) {
 		    if (tex) {
 			pputs(prn, "\\\\\n");
 		    } else if (rtf_format(prn)) {
@@ -765,11 +751,11 @@ void print_model_vcv_info (const MODEL *pmod, PRN *prn)
     }
 }
 
-static void print_tsls_droplist (const MODEL *pmod, 
-				 const DATAINFO *pdinfo,
-				 PRN *prn)
+static void print_model_droplist (const MODEL *pmod, 
+				  const DATAINFO *pdinfo,
+				  PRN *prn)
 {
-    const int *dlist = gretl_model_get_data(pmod, "tsls_droplist");
+    const int *dlist = gretl_model_get_data(pmod, "droplist");
     int i, v;
 
     pputs(prn, _("Omitted due to exact collinearity:"));
@@ -784,6 +770,90 @@ static void print_tsls_droplist (const MODEL *pmod,
     pputc(prn, '\n');
 }
 
+static void print_tsls_droplist (const MODEL *pmod, 
+				 const DATAINFO *pdinfo,
+				 PRN *prn)
+{
+    const int *dlist = gretl_model_get_data(pmod, "inst_droplist");
+    int i, v;
+
+    pputs(prn, _("Redundant instruments:"));
+    for (i=1; i<=dlist[0]; i++) {
+	v = dlist[i];
+	if (v < pdinfo->v) {
+	    pprintf(prn, " %s", pdinfo->varname[v]);
+	} else {
+	    pprintf(prn, " %d", v);
+	}
+    }
+    pputc(prn, '\n');
+}
+
+static void print_arma_depvar (const MODEL *pmod,
+			       const DATAINFO *pdinfo,
+			       PRN *prn)
+{
+    int tex = tex_format(prn);
+    int utf = plain_format(prn);
+    int yno = gretl_model_get_depvar(pmod);
+    int d = gretl_model_get_int(pmod, "arima_d");
+    int D = gretl_model_get_int(pmod, "arima_D");
+    char vname[64];
+
+    *vname = 0;
+
+    if (tex) {
+	char tmp[32];
+
+	if (d > 0 || D > 0) {
+	    strcat(vname, "$");
+	}
+	if (d == 1) {
+	    strcat(vname, "(1-L)");
+	} else if (d == 2) {
+	    strcat(vname, "(1-L)^2");
+	}
+	if (D == 1) {
+	    strcat(vname, "(1-L^s)");
+	} else if (d == 2) {
+	    strcat(vname, "(1-L^s)^2");
+	}
+	if (d > 0 || D > 0) {
+	    strcat(vname, "$");
+	}
+	tex_escape(tmp, pdinfo->varname[yno]);
+	strcat(vname, tmp);
+    } else {
+	if (d == 1) {
+	    strcat(vname, "(1-L)");
+	} else if (d == 2) {
+	    strcat(vname, "(1-L)^2");
+	}
+	if (D == 1) {
+	    strcat(vname, "(1-Ls)");
+	} else if (d == 2) {
+	    strcat(vname, "(1-Ls)^2");
+	}	
+	strcat(vname, pdinfo->varname[yno]);
+    }
+
+    pprintf(prn, "%s: %s", 
+	    (utf)? _("Dependent variable") : I_("Dependent variable"),
+	    vname);
+}
+
+static void arma_x12a_info (const MODEL *pmod, PRN *prn)
+{
+    int xcode = gretl_model_get_int(pmod, "arma_by_x12a");
+
+    if (xcode > 0) {
+	pputs(prn, _("Estimated using X-12-ARIMA"));
+	pputs(prn, " (");
+	pputs(prn, (xcode == 1)? _("exact ML") : _("conditional ML"));
+	pputs(prn, ")\n");
+    } 
+}
+
 static void print_model_heading (const MODEL *pmod, 
 				 const DATAINFO *pdinfo, 
 				 gretlopt opt, 
@@ -910,6 +980,10 @@ static void print_model_heading (const MODEL *pmod,
 	gretl_prn_newline(prn);
     }
 
+    if (pmod->ci == ARMA && plain_format(prn)) {
+	arma_x12a_info(pmod, prn);
+    }
+
     /* special formulations for dependent variable in various cases */
     if (pmod->aux == AUX_SQ || pmod->aux == AUX_LOG) {
 	pprintf(prn, "%s: %s", 
@@ -934,6 +1008,8 @@ static void print_model_heading (const MODEL *pmod,
 	} else {
 	    pputs(prn, pmod->params[0]);
 	}
+    } else if (pmod->ci == ARMA) {
+	print_arma_depvar(pmod, pdinfo, prn);
     } else { 
 	int v = gretl_model_get_depvar(pmod);
 
@@ -1014,17 +1090,16 @@ static void print_model_heading (const MODEL *pmod,
 	}
     }
 
-    /* message about collinear variables dropped */
-    if (plain_format(prn) && gretl_msg[0] != '\0' &&
-	strstr(gretl_msg, _("Replaced")) == NULL &&
-	strstr(gretl_msg, _("Generated")) == NULL) {
-	pprintf(prn, "%s\n", gretl_msg);
-    }
-
-    /* ditto, but special for TSLS */
+    /* TSLS: message about redundant instruments */
     if (plain_format(prn) && pmod->ci == TSLS &&
-	gretl_model_get_data(pmod, "tsls_droplist") != NULL) {
+	gretl_model_get_data(pmod, "inst_droplist") != NULL) {
 	print_tsls_droplist(pmod, pdinfo, prn);
+    }  
+
+    /* message about collinear regressors dropped */
+    if (plain_format(prn) && 
+	gretl_model_get_data(pmod, "droplist") != NULL) {
+	print_model_droplist(pmod, pdinfo, prn);
     }    
 
     if (pmod->missmask == NULL && gretl_model_get_int(pmod, "wt_dummy")) { 
@@ -1079,16 +1154,16 @@ static void print_coeff_table_start (const MODEL *pmod, PRN *prn, int discrete)
 
     if (plain_format(prn)) {
 	if (discrete) {
-	    pputs(prn, _("      VARIABLE      COEFFICIENT        STDERROR"
-			   "       T STAT       SLOPE\n"));
+	    pputs(prn, _("      VARIABLE       COEFFICIENT        STDERROR"
+			   "      T STAT       SLOPE\n"));
 	    pprintf(prn, "                                                 "
 		    "                 %s\n", _("(at mean)"));
 	} else if (use_param) {
-	    pputs(prn, _("      PARAMETER      ESTIMATE          STDERROR"
-			   "       T STAT   2Prob(t > |T|)\n\n"));
+	    pputs(prn, _("      PARAMETER       ESTIMATE          STDERROR"
+			   "      T STAT   P-VALUE\n\n"));
 	} else {
-	    pputs(prn, _("      VARIABLE      COEFFICIENT        STDERROR"
-			   "       T STAT   2Prob(t > |T|)\n\n"));
+	    pputs(prn, _("      VARIABLE       COEFFICIENT        STDERROR"
+			   "      T STAT   P-VALUE\n\n"));
 	}
 	return;
     } else {
@@ -1308,6 +1383,10 @@ static void print_whites_results (const MODEL *pmod, PRN *prn)
 
 static void print_ll (const MODEL *pmod, PRN *prn)
 {
+    if (na(pmod->lnL)) {
+	return;
+    }
+
     if (plain_format(prn)) {
 	pprintf(prn, "  %s = %.*g\n", _("Log-likelihood"), GRETL_DIGITS,
 		pmod->lnL);
@@ -1326,7 +1405,9 @@ static void print_ll (const MODEL *pmod, PRN *prn)
  * printmodel:
  * @pmod: pointer to gretl model.
  * @pdinfo: data information struct.
- * @opt: may contain OPT_O to print covariance matrix.
+ * @opt: may contain %OPT_O to print covariance matrix, %OPT_S
+ * to get a "simple" print (just coefficients and standard
+ * errors).
  * @prn: gretl printing struct.
  *
  * Print to @prn the estimates in @pmod plus associated statistics.
@@ -1395,7 +1476,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
     if (pmod->ci == POISSON) {
 	print_middle_table_start(prn);
 	depvarstats(pmod, prn);
-	print_poisson_stats(pmod, prn);
+	print_ll(pmod, prn);
 	info_stats_lines(pmod, prn);
 	print_middle_table_end(prn);
 	goto close_format;
@@ -1405,6 +1486,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	print_middle_table_start(prn);
 	depvarstats(pmod, prn);
 	print_tobit_stats(pmod, prn);
+	print_ll(pmod, prn);
 	info_stats_lines(pmod, prn);
 	print_middle_table_end(prn);
 	goto close_format;
@@ -1422,7 +1504,8 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	print_middle_table_start(prn);
 	depvarstats(pmod, prn);
 	garch_variance_line(pmod, prn);
-	print_arma_stats(pmod, prn);
+	print_ll(pmod, prn);
+	info_stats_lines(pmod, prn);
 	print_middle_table_end(prn);
 	goto close_format;
     }    
@@ -1462,7 +1545,8 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 
     if (!pmod->ifc && pmod->ci != NLS && pmod->aux != AUX_VAR
 	&& pmod->aux != AUX_JOHANSEN && pmod->aux != AUX_VECM
-	&& pmod->ci != MLE && plain_format(prn)) {
+	&& pmod->ci != ARMA && pmod->ci != MLE 
+	&& plain_format(prn)) {
 	noconst(pmod, prn);
     }
     
@@ -1472,14 +1556,25 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	goto close_format;
     }
 
+    if (pmod->ci == ARMA) {
+	print_middle_table_start(prn);
+	depvarstats(pmod, prn);
+	print_arma_stats(pmod, prn);
+	print_ll(pmod, prn);
+	info_stats_lines(pmod, prn);
+	print_middle_table_end(prn);
+	print_arma_roots(pmod, prn);
+	goto close_format;
+    }
+
     if (pmod->ci == LOGISTIC) {
 	original_stats_message(prn);
-    }
+    }    
 
     if (pmod->ci == OLS || pmod->ci == VAR || pmod->ci == TSLS 
 	|| pmod->ci == POOLED || pmod->ci == NLS
 	|| (pmod->ci == AR && pmod->arinfo->arlist[0] == 1)
-	|| pmod->ci == ARMA || pmod->ci == LOGISTIC || pmod->ci == TOBIT
+	|| pmod->ci == LOGISTIC || pmod->ci == TOBIT
 	|| (pmod->ci == WLS && gretl_model_get_int(pmod, "wt_dummy"))) {
 	print_middle_table_start(prn);
 	if (pmod->ci != VAR) {
@@ -1511,9 +1606,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	    }
 	}
 
-	if (pmod->ci == ARMA) {
-	    print_arma_stats(pmod, prn);
-	} else if (pmod->aux != AUX_VECM) {
+	if (pmod->aux != AUX_VECM) {
 	    if (pmod->ci == OLS) {
 		print_ll(pmod, prn);
 	    }
@@ -1522,10 +1615,6 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 
 	print_middle_table_end(prn);
 
-	if (pmod->ci == ARMA) {
-	    print_arma_roots(pmod, prn);
-	}
-
 	if (pmod->ci == TSLS && plain_format(prn)) {
 	    r_squared_message(prn);
 	}
@@ -1627,9 +1716,9 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 static void print_pval_str (double pval, char *str)
 {
     if (pval < .00001) {
-	sprintf(str, "< %.5f", 0.00001);
+	sprintf(str, "<%.5f", 0.00001);
     } else {
-	sprintf(str, "%f", pval);
+	sprintf(str, "%.5f", pval);
     }
 }
 
@@ -1642,10 +1731,10 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
     char varname[24];
 
     gretl_model_get_param_name(pmod, pdinfo, i, varname);
-    pprintf(prn, " %13s ", varname);
-
-    bufspace((strlen(varname) > 12)? 1 : 2, prn);
-
+    pputs(prn, "  ");
+    print_centered(varname, 15, prn);
+    pputc(prn, ' ');
+    
     /* print coeff value if well-defined */
     if (isnan(pmod->coeff[i]) || na(pmod->coeff[i])) {
 	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 17), _("undefined"));
@@ -1654,8 +1743,6 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	gretl_print_value(pmod->coeff[i], prn);
     }
 
-    bufspace(2, prn);
-
     /* get out if std error is undefined */
     if (isnan(pmod->sderr[i]) || na(pmod->sderr[i])) {
 	pprintf(prn, "%*s\n", UTF_WIDTH(_("undefined"), 16), _("undefined"));
@@ -1675,27 +1762,29 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	} else {
 	    pprintf(prn, " %7.3f", t);
 	}
+
 	if (pmod->aux == AUX_ADF || pmod->aux == AUX_DF) {
 	    if (i + 2 == gretl_model_get_int(pmod, "dfnum")) {
 		char pvalstr[16];
 
 		pvalue = gretl_model_get_double(pmod, "dfpval");
 		print_pval_str(pvalue, pvalstr);
-		pprintf(prn, "%*s", UTF_WIDTH(pvalstr, 12), pvalstr);
+		pprintf(prn, "%*s", UTF_WIDTH(pvalstr, 10), pvalstr);
 	    } 
 	    do_pval = 0;
 	}
+
 	if (do_pval) {
 	    char pvalstr[16];
 
 	    pvalue = coeff_pval(pmod, t, pmod->dfd);
 	    print_pval_str(pvalue, pvalstr);
-	    pprintf(prn, "%*s", UTF_WIDTH(pvalstr, 12), pvalstr);
+	    pprintf(prn, "%*s", UTF_WIDTH(pvalstr, 10), pvalstr);
 	}
     } else if (do_pval) { 
 	/* case of zero standard error */
 	do_pval = 0;
-	pprintf(prn, "     %*s", UTF_WIDTH(_("undefined"), 12), _("undefined"));
+	pprintf(prn, "     %*s", UTF_WIDTH(_("undefined"), 10), _("undefined"));
     }
 
     if (do_pval) {
@@ -1935,8 +2024,8 @@ static void print_arma_roots (const MODEL *pmod, PRN *prn)
     cmplx *roots = gretl_model_get_data(pmod, "roots");
 
     if (roots != NULL) {
-	int p = pmod->list[1];
-	int q = pmod->list[2];
+	int p = gretl_arma_model_nonseasonal_AR_order(pmod);
+	int q = gretl_arma_model_nonseasonal_MA_order(pmod);
 	int P = gretl_model_get_int(pmod, "arma_P");
 	int Q = gretl_model_get_int(pmod, "arma_Q");
 	int i, k, hline;
@@ -2057,11 +2146,9 @@ static void print_tobit_stats (const MODEL *pmod, PRN *prn)
     if (plain_format(prn)) {
 	pprintf(prn, "  %s: %d (%.1f%%)\n", _("Censored observations"), cenc, cenpc);
 	pprintf(prn, "  %s = %.*g\n", _("sigma"), GRETL_DIGITS, pmod->sigma);
-	pprintf(prn, "  %s = %.3f\n", _("Log-likelihood"), pmod->lnL);
     } else if (rtf_format(prn)) {
 	pprintf(prn, RTFTAB "%s: %d (%.1f%%)\n", I_("Censored observations"), cenc, cenpc);
 	pprintf(prn, RTFTAB "%s = %g\n", I_("sigma"), pmod->sigma);
-	pprintf(prn, RTFTAB "%s = %.3f\n", I_("Log-likelihood"), pmod->lnL);
     } else if (tex_format(prn)) {
 	char xstr[32];
 
@@ -2069,8 +2156,6 @@ static void print_tobit_stats (const MODEL *pmod, PRN *prn)
 		I_("Censored observations"), cenpc);
 	tex_dcolumn_double(pmod->sigma, xstr);
 	pprintf(prn, "$\\hat{\\sigma}$ & %s \\\\\n", xstr);
-	tex_dcolumn_double(pmod->lnL, xstr);
-	pprintf(prn, "%s & %s \\\\\n", I_("Log-likelihood"), xstr);
     }
 }
 
@@ -2080,9 +2165,9 @@ print_poisson_offset (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
     int offvar = gretl_model_get_int(pmod, "offset_var");
 
     if (offvar > 0) {
-	char namestr[14];
+	char namestr[24];
 
-	sprintf(namestr, "log(%.8s)", pdinfo->varname[offvar]);
+	sprintf(namestr, "log(%s)", pdinfo->varname[offvar]);
 	if (plain_format(prn)) {
 	    pprintf(prn, "\n %13s         1.0\n", namestr);
 	} else if (rtf_format(prn)) {
@@ -2098,40 +2183,26 @@ print_poisson_offset (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
     }
 }
 
-static void print_poisson_stats (const MODEL *pmod, PRN *prn)
-{
-
-    if (plain_format(prn)) {
-	pprintf(prn, "  %s = %.3f\n", _("Log-likelihood"), pmod->lnL);
-    } else if (rtf_format(prn)) {
-	pprintf(prn, RTFTAB "%s = %.3f\n", I_("Log-likelihood"), pmod->lnL);
-    } else if (tex_format(prn)) {
-	char xstr[32];
-
-	tex_dcolumn_double(pmod->lnL, xstr);
-	pprintf(prn, "%s & %s \\\\\n", I_("Log-likelihood"), xstr);
-    }
-}
-
 static void print_arma_stats (const MODEL *pmod, PRN *prn)
 {
+    double mu = gretl_model_get_double(pmod, "mean_error");
+    double var = pmod->sigma * pmod->sigma;
+
     if (plain_format(prn)) {
-	pprintf(prn, "  %s = %.3f\n", _("Log-likelihood"), pmod->lnL);
-	pprintf(prn, "  %s = %.3f\n", _("AIC"), pmod->criterion[C_AIC]);
-	pprintf(prn, "  %s = %.3f\n", _("BIC"), pmod->criterion[C_BIC]);
+	pprintf(prn, "  %s = %.*g\n", _("Mean of innovations"), 
+		GRETL_DIGITS, mu);
+	pprintf(prn, "  %s = %.*g\n", _("Variance of innovations"), 
+		GRETL_DIGITS, var);
     } else if (rtf_format(prn)) {
-	pprintf(prn, RTFTAB "%s = %.3f\n", I_("Log-likelihood"), pmod->lnL);
-	pprintf(prn, RTFTAB "%s = %.3f\n", I_("AIC"), pmod->criterion[C_AIC]);
-	pprintf(prn, RTFTAB "%s = %.3f\n", I_("BIC"), pmod->criterion[C_BIC]);
+	pprintf(prn, RTFTAB "%s = %g\n", I_("Mean of innovations"), mu);
+	pprintf(prn, RTFTAB "%s = %g\n", I_("Variance of innovations"), var);
     } else if (tex_format(prn)) {
 	char xstr[32];
 
-	tex_dcolumn_double(pmod->lnL, xstr);
-	pprintf(prn, "%s & %s \\\\\n", I_("Log-likelihood"), xstr);
-	tex_dcolumn_double(pmod->criterion[C_AIC], xstr);
-	pprintf(prn, "%s & %s \\\\\n", I_("AIC"), xstr);
-	tex_dcolumn_double(pmod->criterion[C_BIC], xstr);
-	pprintf(prn, "%s & %s \\\\\n", I_("BIC"), xstr);
+	tex_dcolumn_double(mu, xstr);
+	pprintf(prn, "%s & %s \\\\\n", I_("Mean of innovations"), xstr);
+	tex_dcolumn_double(var, xstr);
+	pprintf(prn, "%s & %s \\\\\n", I_("Variance of innovations"), xstr);
     }
 }
 
@@ -2204,6 +2275,8 @@ static void print_discrete_statistics (const MODEL *pmod,
 		crit[C_AIC]);
 	pprintf(prn, "  %s (%s) = %g\n", _(bic_str), _(bic_abbrev),
 		crit[C_BIC]);
+	pprintf(prn, "  %s (%s) = %g\n", _(hqc_str), _(hqc_abbrev),
+		crit[C_HQC]);
 	pprintf(prn, "  %s = %g\n", _("McFadden's pseudo-R-squared"), pmod->rsq);
 	pputc(prn, '\n');
 
@@ -2235,6 +2308,8 @@ static void print_discrete_statistics (const MODEL *pmod,
 		crit[C_AIC]);
 	pprintf(prn, "\\par %s (%s) = %g\\par\n", I_(bic_str), I_(bic_abbrev),
 		crit[C_BIC]);
+	pprintf(prn, "\\par %s (%s) = %g\\par\n", I_(hqc_str), I_(hqc_abbrev),
+		crit[C_HQC]);
 	pprintf(prn, "\\par %s = %g\\par\n", I_("McFadden's pseudo-R{\\super 2}"), 
 		pmod->rsq);
 	pputc(prn, '\n');
@@ -2268,6 +2343,8 @@ static void print_discrete_statistics (const MODEL *pmod,
 		crit[C_AIC]);
 	pprintf(prn, "%s (%s) = %g\\\\\n", I_(bic_str), I_(bic_abbrev),
 		crit[C_BIC]);
+	pprintf(prn, "%s (%s) = %g\\\\\n", I_(tex_hqc_str), I_(hqc_abbrev),
+		crit[C_HQC]);
 	pprintf(prn, "%s = %g\\\\\n", I_("McFadden's pseudo-$R^2$"), pmod->rsq);
 	pputs(prn, "\\end{raggedright}\n");
     }
@@ -2311,7 +2388,7 @@ static void mp_other_stats (const mp_results *mpvals, PRN *prn)
 static void print_mpvals_coeff (const mp_results *mpvals, 
 				int c, PRN *prn)
 {
-    pprintf(prn, " %3d) %8s ", mpvals->varlist[c+2], mpvals->varnames[c+1]);
+    pprintf(prn, " %*s ", VNAMELEN - 1, mpvals->varnames[c+1]);
 
     gretl_print_fullwidth_double(mpvals->coeff[c], 
 				 GRETL_MP_DIGITS, prn);
@@ -2344,6 +2421,7 @@ void print_mpols_results (const mp_results *mpvals, DATAINFO *pdinfo,
 	pprintf(prn, "%s: %s\n\n", _("Dependent variable"),
 		mpvals->varnames[0]);
 
+	bufspace(2, prn);
 	pputs(prn, _("      VARIABLE         COEFFICIENT          "
 		       "        STD. ERROR\n"));
     }
