@@ -87,9 +87,6 @@ static int genr_err;
 
 static void update_nls_param_values (const double *x);
 
-static int BFGS_min (int n, double *b, int maxit, double reltol,
-		     int *fncount, int *grcount);
-
 #if NEW_GENR
 
 static void destroy_genrs_array (GENERATOR **genrs, int n)
@@ -148,7 +145,7 @@ static int nls_genr_setup (void)
 	    sprintf(formula, "$nl_x%d = %s", i, pspec->params[j++].deriv);
 	}
 	
-	genrs[i] = genr_compile(formula, nZ, ndinfo, OPT_P);
+	genrs[i] = genr_compile(formula, nZ, ndinfo, OPT_P, NULL);
 	err = genr_get_err(genrs[i]);
 #if NLS_DEBUG
 	fprintf(stderr, "genrs[%d] = %p, err = %d\n", i, (void *) genrs[i], err);
@@ -406,12 +403,9 @@ get_params_from_nlfunc (nls_spec *spec, const double **Z,
 	p = s;
 	if (isalpha(*s) && *(s + 1)) { 
 	    /* find a variable name */
-	    p++;
-	    n = 1;
-	    while (isalnum(*p) || *p == '_') {
-		p++; n++;
-	    }
-	    if (n > 8) {
+	    n = gretl_varchar_spn(s);
+	    p += n;
+	    if (n > VNAMELEN - 1) {
 		/* variable name is too long */
 		return 1;
 	    }
@@ -431,44 +425,50 @@ get_params_from_nlfunc (nls_spec *spec, const double **Z,
     return err;
 }
 
-/* For case where analytical derivatives are not given, the user
-   may supply a line like:
-
-     params b0 b1 b2 b3
-
-   specifying the paramters to be estimated.  Here we parse
-   such a list and add the parameter info to spec.  The terms
-   in the list must be pre-existing scalar variables.
-*/
-
-static int 
-nls_spec_add_param_list (nls_spec *spec, const char *s,
-			 const double **Z, const DATAINFO *pdinfo)
+static int nls_spec_allocate_params (nls_spec *spec, int np)
 {
-    int i, nf = count_fields(s);
-    const char *p = s;
-    int err = 0;
-
-    if (spec->params != NULL) {
-	return E_DATA;
-    }
-
-    if (nf == 0) {
-	return E_DATA;
-    }
-
-    spec->params = malloc(nf * sizeof *spec->params);
+    spec->params = malloc(np * sizeof *spec->params);
     if (spec->params == NULL) {
 	return E_ALLOC;
     }
 
-    spec->coeff = malloc(nf * sizeof *spec->coeff);
+    spec->coeff = malloc(np * sizeof *spec->coeff);
     if (spec->coeff == NULL) {
 	free(spec->params);
 	spec->params = NULL;
 	return E_ALLOC;
     }    
 
+    return 0;
+}
+
+/* For case where analytical derivatives are not given, the user
+   may supply a line like:
+
+     params b0 b1 b2 b3
+
+   specifying the parameters to be estimated.  Here we parse
+   such a list and add the parameter info to spec.  The terms
+   in the list must be pre-existing scalar variables.
+*/
+
+static int 
+nls_spec_add_params_from_line (nls_spec *spec, const char *s,
+			       const double **Z, const DATAINFO *pdinfo)
+{
+    int i, nf = count_fields(s);
+    const char *p = s;
+    int err = 0;
+
+    if (spec->params != NULL || nf == 0) {
+	return E_DATA;
+    }
+
+    err = nls_spec_allocate_params(spec, nf);
+    if (err) {
+	return err;
+    }
+
     for (i=0; i<nf && !err; i++) {
 	char *pname = gretl_word_strdup(p, &p);
 	int v;
@@ -506,6 +506,61 @@ nls_spec_add_param_list (nls_spec *spec, const char *s,
     return err;
 }
 
+/**
+ * nls_spec_add_param_list:
+ * @spec: nls specification.
+ * @list: list of variables by ID number.
+ * @Z: data array.
+ * @pdinfo: information on dataset.
+ *
+ * Adds to @spec a list of (scalar) parameters to be estimated, as
+ * given in @list.
+ *
+ * Returns: 0 on success, non-zero error code on error.
+ */
+
+int nls_spec_add_param_list (nls_spec *spec, const int *list,
+			     const double **Z, const DATAINFO *pdinfo)
+{
+    int i, np = list[0];
+    int err = 0;
+
+    if (spec->params != NULL || np == 0) {
+	return E_DATA;
+    }
+
+    err = nls_spec_allocate_params(spec, np);
+    if (err) {
+	return err;
+    }
+
+    for (i=0; i<np && !err; i++) {
+	int v = list[i+1];
+
+	if (v >= pdinfo->v || pdinfo->vector[v]) {
+	    err = E_DATA;
+	} else {
+	    spec->params[i].varnum = v;
+	    spec->params[i].dernum = 0;
+	    strcpy(spec->params[i].name, pdinfo->varname[v]);
+	    spec->params[i].deriv = NULL;
+	    spec->coeff[i] = Z[v][0];
+	}
+    }
+
+    if (err) {
+	free(spec->params);
+	spec->params = NULL;
+	free(spec->coeff);
+	spec->coeff = NULL;
+	spec->nparam = 0;
+    } else {
+	spec->nparam = np;
+    }
+
+    return err;
+}
+
 /* Adjust starting and ending points of sample if need be, to avoid
    missing values; abort if there are missing values within the
    (possibly reduced) sample range.  For this purpose we generate
@@ -597,7 +652,7 @@ static double get_mle_ll (const double *b)
 	pspec->ll -= (*nZ)[v][t];
     }
 
-    return -pspec->ll;
+    return pspec->ll;
 }
 
 /* used in the context of BFGS */
@@ -619,7 +674,7 @@ static int get_mle_gradient (double *b, double *g)
 		return 1;
 	    }
 	    for (t=pspec->t1; t<=pspec->t2; t++) {
-		val1 += (*nZ)[pspec->uhatnum][t];
+		val1 -= (*nZ)[pspec->uhatnum][t];
 	    }
 	    b[i] += 2.0 * eps;
 	    update_nls_param_values(b);
@@ -627,7 +682,7 @@ static int get_mle_gradient (double *b, double *g)
 		return 1;
 	    }
 	    for (t=pspec->t1; t<=pspec->t2; t++) {
-		val2 += (*nZ)[pspec->uhatnum][t];
+		val2 -= (*nZ)[pspec->uhatnum][t];
 	    }
 	    b[i] -= eps;
 	    g[i] = (val2 - val1) / (2.0 * eps);
@@ -646,19 +701,18 @@ static int get_mle_gradient (double *b, double *g)
 	v = pspec->params[i].dernum;
 
 	g[i] = 0.0;
-
-	/* derivative may be vector or scalar */
 	if (ndinfo->vector[v]) {
+	    /* derivative may be vector or scalar */
 	    for (t=pspec->t1; t<=pspec->t2; t++) {
 		if (na((*nZ)[v][t])) {
 		    fprintf(stderr, "NA in gradient calculation\n");
 		    err = 1;
 		} else {
-		    g[i] -= (*nZ)[v][t];
+		    g[i] += (*nZ)[v][t];
 		}
 	    }
 	} else {
-	    g[i] -= (*nZ)[v][0];
+	    g[i] += (*nZ)[v][0];
 	}
 #if NLS_DEBUG > 1
 	fprintf(stderr, "g[%d] = %g, based on nZ[%d]\n", i, g[i], v);
@@ -670,28 +724,29 @@ static int get_mle_gradient (double *b, double *g)
 
 /* used in the context of BFGS */
 
-static void print_mle_iter_stats (double ll, const double *b, const double *g, 
-				  int iter, double sl)
+static void 
+print_mle_iter_stats (double ll, int nparam, const double *b, const double *g, 
+		      int iter, double sl, PRN *prn)
 {
     int i;
 
-    pprintf(nprn, "Iteration %d; log likelihood = %.8g", iter, -ll);	
+    pprintf(prn, "Iteration %d; log likelihood = %.8g", iter, ll);	
     if (iter > 1) {
-	pprintf(nprn, " (steplength = %.8g)", sl);
+	pprintf(prn, " (steplength = %.8g)", sl);
     }	
-    pputc(nprn, '\n');
+    pputc(prn, '\n');
 	
-    pputs(nprn, "Parameters: ");
-    for (i=0; i<pspec->nparam; i++) {
-	pprintf(nprn, "%#12.5g", b[i]);
+    pputs(prn, "Parameters: ");
+    for (i=0; i<nparam; i++) {
+	pprintf(prn, "%#12.5g", b[i]);
     }
-    pputc(nprn, '\n');
+    pputc(prn, '\n');
 
-    pputs(nprn, "Gradients:  ");
-    for (i=0; i<pspec->nparam; i++) {
-	pprintf(nprn, "%#12.5g", -g[i]);
+    pputs(prn, "Gradients:  ");
+    for (i=0; i<nparam; i++) {
+	pprintf(prn, "%#12.5g", -g[i]);
     }
-    pputs(nprn, "\n\n");
+    pputs(prn, "\n\n");
 }
 
 /* this function is used in the context of the minpack callback, and
@@ -1137,7 +1192,7 @@ static MODEL GNR (double *uhat, double *jac, nls_spec *spec,
     print_GNR_dataset(glist, gZ, gdinfo);
 #endif
 
-    gnr = lsq(glist, &gZ, gdinfo, OLS, OPT_A, 0.0);
+    gnr = lsq(glist, &gZ, gdinfo, OLS, OPT_A);
 
 #if NLS_DEBUG
     gnr.name = gretl_strdup("GNR for NLS");
@@ -1166,7 +1221,7 @@ static MODEL GNR (double *uhat, double *jac, nls_spec *spec,
     }
 
     if (gnr.errcode == 0) {
-	ls_aic_bic(&gnr);
+	ls_criteria(&gnr);
 	add_coeffs_to_model(&gnr, spec->coeff);
 	add_param_names_to_model(&gnr, spec, pdinfo);
 	add_fit_resid_to_model(&gnr, spec, uhat, Z, perfect);
@@ -1228,7 +1283,7 @@ static int make_mle_model (MODEL *pmod, nls_spec *spec,
 
     pmod->ci = MLE;
     pmod->lnL = pspec->ll;
-    mle_aic_bic(pmod, 0);
+    mle_criteria(pmod, 0);
 
     add_coeffs_to_model(pmod, spec->coeff);
 
@@ -1498,8 +1553,10 @@ static int mle_calculate (nls_spec *spec, double *fvec, double *jac, PRN *prn)
     }
 
     if (!err) {
-	err = BFGS_min(n, spec->coeff, maxit, pspec->tol, 
-		       &spec->fncount, &spec->grcount);
+	err = BFGS_max(n, spec->coeff, maxit, pspec->tol, 
+		       &spec->fncount, &spec->grcount,
+		       get_mle_ll, get_mle_gradient,
+		       pspec->opt, nprn);
     }
 
     return err;    
@@ -1743,7 +1800,7 @@ nls_spec_add_param_with_deriv (nls_spec *spec, const char *dstr,
     }
 
     *param->name = '\0';
-    strncat(param->name, vname, 8);
+    strncat(param->name, vname, VNAMELEN - 1);
     free(vname);
 
     v = varindex(pdinfo, param->name);
@@ -1853,10 +1910,8 @@ nls_spec_set_regression_function (nls_spec *spec, const char *fnstr,
 
     if (equation_get_lhs_and_rhs(p, &vname, &rhs)) { 
 	sprintf(gretl_errmsg, _("parse error in '%s'\n"), fnstr);
-	err = E_PARSE;
-    }
-
-    if (!err) {
+	err =  E_PARSE;
+    } else {
 	spec->depvar = varindex(pdinfo, vname);
 	if (spec->depvar == pdinfo->v) {
 	    if (spec->ci == NLS) {
@@ -1869,20 +1924,23 @@ nls_spec_set_regression_function (nls_spec *spec, const char *fnstr,
 	}
     }
 
-    if (spec->ci == MLE) {
-	flen = strlen(rhs) + 4;
-    } else {
-	flen = strlen(vname) + strlen(rhs) + 6;
-    }
-
-    spec->nlfunc = malloc(flen);
-    if (spec->nlfunc == NULL) {
-	err = E_ALLOC;
-    } else {
+    if (!err) {
 	if (spec->ci == MLE) {
-	    sprintf(spec->nlfunc, "-(%s)", rhs);
+	    flen = strlen(rhs) + 4;
 	} else {
-	    sprintf(spec->nlfunc, "%s - (%s)", vname, rhs);
+	    flen = strlen(vname) + strlen(rhs) + 6;
+	}
+
+	spec->nlfunc = malloc(flen);
+
+	if (spec->nlfunc == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    if (spec->ci == MLE) {
+		sprintf(spec->nlfunc, "-(%s)", rhs);
+	    } else {
+		sprintf(spec->nlfunc, "%s - (%s)", vname, rhs);
+	    }
 	}
     }
 
@@ -1970,7 +2028,7 @@ int nls_parse_line (int ci, const char *line, const double **Z,
 	    } else {
 		/* "params" */
 		if (pspec->mode != ANALYTIC_DERIVS) {
-		    err = nls_spec_add_param_list(pspec, line + 6, Z, pdinfo);
+		    err = nls_spec_add_params_from_line(pspec, line + 6, Z, pdinfo);
 		} else {
 		    pprintf(prn, _("Analytical derivatives supplied: "
 				   "\"params\" line will be ignored"));
@@ -2310,19 +2368,34 @@ static void free_Lmatrix (double **m, int n)
 #define acctol		0.0001 
 #define reltest		10.0
 
+/* FIXME: it would be nice to trash this, but that depends on
+   switching all the signs correctly below, which (to me!) is
+   more difficult than it looks -- AC */
+
+static void reverse_gradient (double *g, int n)
+{
+    int i;
+
+    for (i=0; i<n; i++) {
+	g[i] = -g[i];
+    }
+}
+
 /*  BFGS variable-metric method, based on Pascal code in J. C. Nash,
     "Compact Numerical Methods for Computers," 2nd edition, converted
     by p2c then re-crafted by B. D. Ripley.  Revised for gretl by
     Allin Cottrell.
 */
 
-static int BFGS_min (int n, double *b, int maxit, double reltol,
-		     int *fncount, int *grcount)
+int BFGS_max (int n, double *b, int maxit, double reltol,
+	      int *fncount, int *grcount, BFGS_LL_FUNC get_ll,
+	      BFGS_GRAD_FUNC get_gradient, gretlopt opt,
+	      PRN *prn)
 {
     int accpoint, enough;
     double *g = NULL, *t = NULL, *X = NULL, *c = NULL, **B = NULL;
     int count, funcount, gradcount;
-    double Fmin, f, gradproj;
+    double fmax, f, gradproj;
     int i, j, ilast, iter = 0;
     double s, steplength = 0.0;
     double D1, D2;
@@ -2339,23 +2412,24 @@ static int BFGS_min (int n, double *b, int maxit, double reltol,
 	goto bailout;
     }
 
-    f = get_mle_ll(b);
+    f = get_ll(b);
 
     if (na(f)) {
-	fprintf(stderr, "initial value in 'vmmin' is not finite\n");
+	fprintf(stderr, "initial value of f is not finite\n");
 	err = E_DATA;
 	goto bailout;
     }
 
-    Fmin = f;
+    fmax = f;
     funcount = gradcount = 1;
-    get_mle_gradient(b, g);
+    get_gradient(b, g);
+    reverse_gradient(g, n);
     iter++;
     ilast = gradcount;
 
     do {
-	if (pspec->opt & OPT_V) {
-	    print_mle_iter_stats(f, b, g, iter, steplength);
+	if (opt & OPT_V) {
+	    print_mle_iter_stats(f, n, b, g, iter, steplength, prn);
 	}
 	if (ilast == gradcount) {
 	    for (i=0; i<n; i++) {
@@ -2383,7 +2457,7 @@ static int BFGS_min (int n, double *b, int maxit, double reltol,
 	}
 
 	if (gradproj < 0.0) {	
-	    /* search direction is downhill */
+	    /* search direction is uphill */
 	    steplength = 1.0;
 	    accpoint = 0;
 	    do {
@@ -2396,35 +2470,36 @@ static int BFGS_min (int n, double *b, int maxit, double reltol,
 		    }
 		}
 		if (count < n) {
-		    f = get_mle_ll(b);
+		    f = get_ll(b);
 		    funcount++;
 		    accpoint = !na(f) &&
-			(f <= Fmin + gradproj * steplength * acctol);
+			(f >= fmax + gradproj * steplength * acctol);
 		    if (!accpoint) {
 			steplength *= stepredn;
 		    }
 		}
 	    } while (!(count == n || accpoint));
 
-	    enough = fabs(f - Fmin) > reltol * (fabs(Fmin) + reltol);
+	    enough = fabs(fmax - f) > reltol * (fabs(fmax) + reltol);
 #if BFGS_DEBUG
 	    fprintf(stderr, "enough = %d: f=%g, "
-		    "fabs(f - Fmin) = %g,\n reltol * "
-		    "(fabs(Fmin) + reltol) = %g\n",
-		    enough, f, fabs(f - Fmin), 
-		    reltol * (fabs(Fmin) + reltol));
+		    "fabs(fmax - f) = %g,\n reltol * "
+		    "(fabs(fmax) + reltol) = %g\n",
+		    enough, f, fabs(fmax - f), 
+		    reltol * (fabs(fmax) + reltol));
 #endif
 
 	    /* stop if value is small or if relative change is low */
 	    if (!enough) {
 		count = n;
-		Fmin = f;
+		fmax = f;
 	    }
 
 	    if (count < n) {
 		/* making progress */
-		Fmin = f;
-		get_mle_gradient(b, g);
+		fmax = f;
+		get_gradient(b, g);
+		reverse_gradient(g, n);
 		gradcount++;
 		iter++;
 		D1 = 0.0;
@@ -2465,7 +2540,7 @@ static int BFGS_min (int n, double *b, int maxit, double reltol,
 		}
 	    }
 	} else {
-	    /* uphill search */
+	    /* downhill search */
 	    count = 0;
 	    if (ilast == gradcount) {
 		count = n;
@@ -2495,9 +2570,9 @@ static int BFGS_min (int n, double *b, int maxit, double reltol,
     *fncount = funcount;
     *grcount = gradcount;
 
-    if (pspec->opt & OPT_V) {
+    if (opt & OPT_V) {
 	pputs(nprn, "\n--- FINAL VALUES: \n");	
-	print_mle_iter_stats(f, b, g, iter, steplength);
+	print_mle_iter_stats(f, n, b, g, iter, steplength, prn);
 	pputs(nprn, "\n\n");	
     }
 
