@@ -22,6 +22,9 @@
 
 typedef struct _nls_spec nls_spec;
 
+typedef double (*BFGS_LL_FUNC) (const double *);
+typedef int (*BFGS_GRAD_FUNC) (double *, double *);
+
 nls_spec *nls_spec_new (int ci, const DATAINFO *pdinfo);
 
 void nls_spec_destroy (nls_spec *spec);
@@ -32,6 +35,9 @@ nls_spec_add_param_with_deriv (nls_spec *spec,
 			       const double **Z, 
 			       const DATAINFO *pdinfo);
 
+int nls_spec_add_param_list (nls_spec *spec, const int *list,
+			     const double **Z, const DATAINFO *pdinfo);
+
 int 
 nls_spec_set_regression_function (nls_spec *spec, 
 				  const char *fnstr, 
@@ -50,4 +56,9 @@ MODEL model_from_nls_spec (nls_spec *spec, double ***pZ,
 
 double get_default_nls_toler (void);
 
+int BFGS_max (int n, double *b, int maxit, double reltol,
+	      int *fncount, int *grcount, BFGS_LL_FUNC get_ll,
+	      BFGS_GRAD_FUNC get_gradient, gretlopt opt,
+	      PRN *prn);
+
 #endif /* GRETL_NLS_H */
