@@ -27,14 +27,6 @@ enum {
     RUN_CLEANUP
 };
 
-static int inverse_compare_doubles (const void *a, const void *b)
-{
-    const double *da = (const double *) a;
-    const double *db = (const double *) b;
-     
-    return (*da < *db) - (*da > *db);
-}
-
 /* alpha = .001, .01, .025, .05, .1 */
 
 static double spearman_critical[18][5] = {      /* n = */
@@ -123,8 +115,8 @@ static int spearman_rho (const double *x, const double *y, int n,
     }
 
     /* get sorted series */
-    qsort(sx, nn, sizeof *sx, inverse_compare_doubles);
-    qsort(sy, nn, sizeof *sy, inverse_compare_doubles);
+    qsort(sx, nn, sizeof *sx, gretl_inverse_compare_doubles);
+    qsort(sy, nn, sizeof *sy, gretl_inverse_compare_doubles);
 
     for (t=0; t<n; t++) {
 	rx[t] = ry[t] = 0.0;
@@ -229,7 +221,7 @@ static int spearman_rho (const double *x, const double *y, int n,
 /**
  * spearman:
  * @list: list of (two) variables to process.
- * @Z: data matrix.
+ * @Z: data array.
  * @pdinfo: information on the data set.
  * @opt: if includes %OPT_V, print both the "raw" and the ranked data.
  * @prn: gretl printing struct.
@@ -375,14 +367,22 @@ static double *locke_shuffle (const double *x, int *n, int code)
 
 #define NREPEAT 100
 
-/* Charles Locke's nonparametric test for whether an empirical
-   distribution is gamma.  See C. Locke, "A Test for the Composite
-   Hypothesis that a Population has a Gamma Distribution,"
-   Commun. Statis.-Theor. Meth. A5(4), 351-364 (1976).
-
-   See also Shapiro and Chen, Journal of Quality Technology 33(1),
-   Jan 2001.
-*/
+/**
+ * lockes_test:
+ * @x: data series.
+ * @t1: start of sample range.
+ * @t2: end of sample range.
+ *
+ * Performs Charles Locke's nonparametric test for whether an
+ * empirical distribution (namely, that of @x over the range
+ * @t1 to @t2) is gamma.  See C. Locke, "A Test for the Composite
+ * Hypothesis that a Population has a Gamma Distribution,"
+ * Commun. Statis.-Theor. Meth. A5(4), 351-364 (1976).  Also
+ * see Shapiro and Chen, Journal of Quality Technology 33(1),
+ * Jan 2001.
+ * 
+ * Returns: the z value for test, or #NADBL on error.
+ */
 
 double lockes_test (const double *x, int t1, int t2)
 {
