@@ -152,7 +152,7 @@ void gui_script_logo (PRN *prn)
 static void 
 print_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 {
-    pprintf(prn, " %8s ", cf->names[i]);
+    pprintf(prn, " %*s ", VNAMELEN - 1, cf->names[i]);
 
     bufspace(3, prn);
 
@@ -175,6 +175,33 @@ print_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
     pputc(prn, '\n');
 }
 
+/**
+ * print_centered:
+ * @s: string to print.
+ * @width: width of field.
+ * @prn: gretl printing struct.
+ *
+ * If the string @s is shorter than width, print it centered
+ * in a field of the given width (otherwise just print it
+ * straight).
+ */
+
+void print_centered (const char *s, int width, PRN *prn)
+{
+    int rem = width - strlen(s);
+
+    if (rem <= 1) {
+	pprintf(prn, "%s", s);
+    } else {
+	int i, off = rem / 2;
+
+	for (i=0; i<off; i++) {
+	    pputs(prn, " ");
+	}
+	pprintf(prn, "%-*s", width - off, s);
+    }
+}
+
 /**
  * text_print_model_confints:
  * @cf: pointer to confidence intervals.
@@ -190,7 +217,7 @@ void text_print_model_confints (const CoeffIntervals *cf, PRN *prn)
 
     pprintf(prn, "t(%d, .025) = %.3f\n\n", cf->df, tcrit95(cf->df));
     /* xgettext:no-c-format */
-    pputs(prn, _("      VARIABLE      COEFFICIENT      95% CONFIDENCE "
+    pputs(prn, _("      VARIABLE         COEFFICIENT      95% CONFIDENCE "
 	    "INTERVAL\n\n"));      
 
     for (i=0; i<cf->ncoeff; i++) {
@@ -275,21 +302,25 @@ static void print_freq_test (const FreqDist *freq, PRN *prn)
  * @prn: gretl printing struct.
  *
  * Print frequency distribution to @prn.
- * 
  */
 
 void print_freq (const FreqDist *freq, PRN *prn)
 {
-    int i, k, nlw, K = freq->numbins - 1;
+    int i, k, nlw, K;
+    int total, valid, missing;
     char word[64];
+    double f, cumf = 0;
 
     if (freq == NULL) {
 	return;
     }
 
-    pprintf(prn, _("\nFrequency distribution for %s, obs %d-%d "
-		   "(%d valid observations)\n"),
-	    freq->varname, freq->t1 + 1, freq->t2 + 1, freq->n);
+    K = freq->numbins - 1;
+    valid = freq->n;
+    total = freq->t2 - freq->t1 + 1;
+
+    pprintf(prn, _("\nFrequency distribution for %s, obs %d-%d\n"),
+	    freq->varname, freq->t1 + 1, freq->t2 + 1);
 
     if (freq->numbins == 0) {
 	if (!na(freq->test)) {
@@ -298,44 +329,64 @@ void print_freq (const FreqDist *freq, PRN *prn)
 	return;
     } 
 
-    pprintf(prn, _("number of bins = %d, mean = %g, sd = %g\n"), 
-	    freq->numbins, freq->xbar, freq->sdx);
+    if (!freq->discrete) {
+	pprintf(prn, _("number of bins = %d, mean = %g, sd = %g\n"), 
+		freq->numbins, freq->xbar, freq->sdx);
 
-    pputs(prn, _("\n       interval          midpt      frequency\n\n"));
+	pputs(prn, 
+	      _("\n       interval          midpt   frequency    rel.     cum.\n\n"));
+    } else {
+	pputs(prn, _("\n          frequency    rel.     cum.\n\n"));
+    }
 
     for (k=0; k<=K; k++) {
 	*word = '\0';
-	if (k == 0) {
-	    pputs(prn, "          <  ");
-	} else if (k == K) {
-	    pputs(prn, "          >= ");
+	if (freq->discrete) {
+	    sprintf(word, "%4.0f", freq->midpt[k]);
 	} else {
-	    pprintf(prn, "%10g - ", freq->endpt[k]);
-	}
-	if (k == K) {
-	    sprintf(word, "%g", freq->endpt[k]);
-	} else {
-	    sprintf(word, "%g", freq->endpt[k+1]);
-	}
-	pprintf(prn, "%s", word);
+	    if (k == 0) {
+		pputs(prn, "          <  ");
+	    } else if (k == K) {
+		pputs(prn, "          >= ");
+	    } else {
+		pprintf(prn, "%10g - ", freq->endpt[k]);
+	    }
+	    if (k == K) {
+		sprintf(word, "%g", freq->endpt[k]);
+	    } else {
+		sprintf(word, "%g", freq->endpt[k+1]);
+	    }
 
-	nlw = 10 - strlen(word);
-	bufspace(nlw, prn);
+	    pprintf(prn, "%s", word);
+	    nlw = 10 - strlen(word);
+	    bufspace(nlw, prn);
+	    sprintf(word, " %g", freq->midpt[k]);
+	}
 
-	sprintf(word, " %g", freq->midpt[k]);
 	pputs(prn, word);
-
 	nlw = 10 - strlen(word);
 	bufspace(nlw, prn);
 
 	pprintf(prn, "%6d  ", freq->f[k]);
-	i = 36.0 * freq->f[k]/freq->n;
+	f = 100.0 * freq->f[k] / valid;
+	cumf += f;
+	pprintf(prn, "  %6.2f%% %7.2f%% ", f, cumf);
+	i = 0.36 * f;
 	while (i--) {
 	    pputc(prn, '*');
 	}
 	pputc(prn, '\n');
     }
 
+    missing = total - valid;
+
+    if (missing > 0) {
+	pputc(prn, '\n');
+	pprintf(prn, _("Invalid cases = %d (%5.2f%%)"), missing, 
+		100 * (double) missing / total);
+	pputc(prn, '\n');
+    }
+
     if (!na(freq->test)) {
 	print_freq_test(freq, prn);
     }
@@ -347,14 +398,17 @@ void print_freq (const FreqDist *freq, PRN *prn)
  * @fulln: full length of data series.
  * @prn: gretl printing struct.
  *
- * Print current sample information to @prn.
- * 
+ * Prints the current sample information to @prn.
  */
 
 void print_smpl (const DATAINFO *pdinfo, int fulln, PRN *prn)
 {
     char date1[OBSLEN], date2[OBSLEN];
 
+    if (!gretl_messages_on()) {
+	return;
+    }
+
     if (fulln && !dataset_is_panel(pdinfo)) {
 	pprintf(prn, _("Full data set: %d observations\n"),
 		fulln);
@@ -437,9 +491,10 @@ static void cut_extra_zero (char *numstr, int digits)
 void gretl_print_fullwidth_double (double x, int digits, PRN *prn)
 {
     char numstr[36], final[36];
-    char *p;
+    int totlen = 2 * digits + 5; /* try changing this? */
     int i, tmp, forept = 0;
     char decpoint = '.';
+    char *p;
 
 #ifdef ENABLE_NLS
     decpoint = get_local_decpoint();
@@ -476,7 +531,7 @@ void gretl_print_fullwidth_double (double x, int digits, PRN *prn)
 
     strcat(final, numstr);
 
-    tmp = 2 * digits + 5 - strlen(final);
+    tmp = totlen - strlen(final);
     for (i=0; i<tmp; i++) {
 	strcat(final, " ");
     }
@@ -565,17 +620,17 @@ int outcovmx (MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
-static void outxx (const double xx, int ci, PRN *prn)
+static void outxx (const double xx, int ci, int wid, PRN *prn)
 {
     if (isnan(xx) || na(xx)) { 
 	if (ci == CORR) {
-	    pprintf(prn, " %*s", UTF_WIDTH(_("undefined"), 13), 
+	    pprintf(prn, " %*s", UTF_WIDTH(_("undefined"), wid), 
 		    _("undefined"));
 	} else {
-	    pputs(prn, "              ");
+	    bufspace(wid, prn);
 	}
     } else if (ci == CORR) {
-	pprintf(prn, " %13.4f", xx);
+	pprintf(prn, " %*.4f", wid - 1, xx);
     } else {
 	char numstr[18];
 
@@ -585,12 +640,10 @@ static void outxx (const double xx, int ci, PRN *prn)
 	    sprintf(numstr, "%g", xx);
 	}
 	gretl_fix_exponent(numstr);
-	pprintf(prn, "%14s", numstr);
+	pprintf(prn, "%*s", wid, numstr);
     }
 }
 
-/* ......................................................... */ 
-
 static int takenotes (int quit_opt)
 {
     char resp[4];
@@ -637,6 +690,21 @@ int scroll_pause_or_quit (void)
     return takenotes(1);
 }
 
+static int vmat_maxlen (VMatrix *vmat)
+{
+    int len, maxlen = 0;
+    int i;
+
+    for (i=0; i<vmat->dim; i++) {
+	len = strlen(vmat->names[i]);
+	if (len > maxlen) {
+	    maxlen = len;
+	}
+    }
+
+    return maxlen;
+}
+
 /*  Given a one dimensional array which represents a symmetric
     matrix, prints out an upper triangular matrix of any size.
 
@@ -654,19 +722,27 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
     register int i, j;
     int nf, li2, p, k, m, idx, ij2, lineno = 0;
     int pause = gretl_get_text_pause();
+    int maxlen = 0;
+    int fwidth = 14;
+    int fields = 5;
     const char *s;
-    enum { FIELDS = 5 };
 
     if (vmat->ci != CORR) {
 	covhdr(prn);
     }
 
+    maxlen = vmat_maxlen(vmat);
+    if (maxlen > 10) {
+	fields = 4;
+	fwidth = 16;
+    }
+
     m = 1;
 
-    for (i=0; i<=vmat->dim/FIELDS; i++) {
-	nf = i * FIELDS;
+    for (i=0; i<=vmat->dim/fields; i++) {
+	nf = i * fields;
 	li2 = vmat->dim - nf;
-	p = (li2 > FIELDS) ? FIELDS : li2;
+	p = (li2 > fields) ? fields : li2;
 	if (p == 0) break;
 
 	if (pause && i > 0) {
@@ -676,7 +752,7 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 	/* print the varname headings */
 	for (j=1; j<=p; ++j)  {
 	    s = vmat->names[j + nf - 1];
-	    bufspace(14 - strlen(s), prn);
+	    bufspace(fwidth - strlen(s), prn);
 	    pputs(prn, s); 
 	}
 	pputc(prn, '\n');
@@ -692,9 +768,10 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 	    }
 	    for (k=0; k<p; k++) {
 		idx = ijton(j, nf+k, vmat->dim);
-		outxx(vmat->vec[idx], vmat->ci, prn);
+		outxx(vmat->vec[idx], vmat->ci, fwidth, prn);
 	    }
-	    pprintf(prn, "  %s\n", vmat->names[j]);
+	    if (fwidth < 15) pputc(prn, ' ');
+	    pprintf(prn, " %s\n", vmat->names[j]);
 	    lineno++;
 	}
 
@@ -706,12 +783,13 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 		lineno = 1;
 	    }
 	    ij2 = nf + j;
-	    bufspace(14 * j, prn);
+	    bufspace(fwidth * j, prn);
 	    for (k=j; k<p; k++) {
 		idx = ijton(ij2, nf+k, vmat->dim);
-		outxx(vmat->vec[idx], vmat->ci, prn);
+		outxx(vmat->vec[idx], vmat->ci, fwidth, prn);
 	    }
-	    pprintf(prn, "  %s\n", vmat->names[ij2]);
+	    if (fwidth < 15) pputc(prn, ' ');
+	    pprintf(prn, " %s\n", vmat->names[ij2]);
 	    lineno++;
 	}
 	pputc(prn, '\n');
@@ -730,7 +808,6 @@ static void fit_resid_head (const FITRESID *fr,
     ntodate(mdate2, fr->t2, pdinfo);
 
     pprintf(prn, _("Model estimation range: %s - %s"), mdate1, mdate2);
-    pprintf(prn, " (n = %d)\n", fr->real_nobs);
 
     if (!na(fr->sigma)) {
 	pprintf(prn, _("Standard error of residuals = %f\n"), fr->sigma);
@@ -750,7 +827,7 @@ static void fit_resid_head (const FITRESID *fr,
 
 /* prints names of variables in @list, positions v1 to v2 */
 
-static void varheading (const int *list, int v1, int v2, 
+static void varheading (const int *list, int v1, int v2, int wid,
 			const DATAINFO *pdinfo, PRN *prn)
 {
     int i;
@@ -768,7 +845,7 @@ static void varheading (const int *list, int v1, int v2,
     } else {
 	pputs(prn, "\n     Obs ");
 	for (i=v1; i<=v2; i++) { 
-	    pprintf(prn, "%13s", pdinfo->varname[list[i]]);
+	    pprintf(prn, "%*s", wid, pdinfo->varname[list[i]]);
 	}
 	pputs(prn, "\n\n");
     }
@@ -1156,20 +1233,25 @@ void print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn)
 
 void varlist (const DATAINFO *pdinfo, PRN *prn)
 {
-    int level = 0;
-    int i, j, n;
+    int level = gretl_function_stack_depth();
+    int len, maxlen = 0;
+    int nv = 4;
+    int i, j, n = 0;
 
-    if (gretl_executing_function()) {
-	level = gretl_function_stack_depth();
-
-	n = 0;
-	for (i=0; i<pdinfo->v; i++) {
-	    if (STACK_LEVEL(pdinfo, i) == level) {
-		n++;
+    for (i=0; i<pdinfo->v; i++) {
+	if (STACK_LEVEL(pdinfo, i) == level) {
+	    len = strlen(pdinfo->varname[i]);
+	    if (len > maxlen) {
+		maxlen = len;
 	    }
+	    n++;
 	}
-    } else {
-	n = pdinfo->v;
+    }
+
+    if (maxlen < 9) {
+	nv = 5;
+    } else if (maxlen > 13) {
+	nv = 3;
     }
 
     pprintf(prn, _("Listing %d variables:\n"), n);
@@ -1179,14 +1261,16 @@ void varlist (const DATAINFO *pdinfo, PRN *prn)
 	if (level > 0 && STACK_LEVEL(pdinfo, i) != level) {
 	    continue;
 	}
-	pprintf(prn, "%3d) %-10s", i, pdinfo->varname[i]);
-	if (j % 5 == 0) {
+	pprintf(prn, "%3d) %-*s", i, maxlen + 2, pdinfo->varname[i]);
+	if (j % nv == 0) {
 	    pputc(prn, '\n');
 	}
 	j++;
     }
 
-    if (n % 5) pputc(prn, '\n');
+    if (n % nv) {
+	pputc(prn, '\n');
+    }
 
     pputc(prn, '\n');
 }
@@ -1257,12 +1341,19 @@ static void print_scalar (double x, const char *vname,
 	pputc(prn, '\n');
     }
 
+    pprintf(prn, "%15s = ", vname);
+
     if (na(x)) {
-	pprintf(prn, "%8s = NA", vname);
-    } else if (opt & OPT_T) {
-	pprintf(prn, "%8s = %.10g", vname, x);
+	pputs(prn, "NA");
     } else {
-	pprintf(prn, "%8s = %10g", vname, x);
+	if (x >= 0.0) {
+	    pputc(prn, ' ');
+	}
+	if (opt & OPT_T) {
+	    pprintf(prn, "%.10e", x);
+	} else {
+	    pprintf(prn, "%#.6g", x);
+	}
     }
 
     if (allconst) {
@@ -1270,6 +1361,13 @@ static void print_scalar (double x, const char *vname,
     }
 }
 
+static int printdata_blocks;
+
+int get_printdata_blocks (void)
+{
+    return printdata_blocks;
+}
+
 /**
  * printdata:
  * @list: list of variables to print.
@@ -1289,9 +1387,10 @@ static void print_scalar (double x, const char *vname,
 int printdata (const int *list, const double **Z, const DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn)
 {
-    int j, v, v1, v2, j5, nvj5, lineno, ncol;
+    int j, v, v1, v2, jc, nvjc, lineno, ncol;
     int allconst, scalars = 0;
     int nvars = 0, sortvar = 0;
+    int maxlen = 0, bplen = 13;
     int *plist = NULL;
     int *pmax = NULL; 
     int t, nsamp;
@@ -1300,6 +1399,8 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 
     int pause = gretl_get_text_pause();
 
+    printdata_blocks = 0;
+
     if (list == NULL) {
 	plist = full_var_list(pdinfo, &nvars);
     } else {
@@ -1322,13 +1423,19 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 
     /* screen out any scalars and print them first */
     for (j=1; j<=plist[0]; j++) {
-	if (!pdinfo->vector[plist[j]]) {
-	    print_scalar(Z[plist[j]][0], pdinfo->varname[plist[j]],
-			 opt, 0, prn);
+	int len, v = plist[j];
+
+	if (!pdinfo->vector[v]) {
+	    print_scalar(Z[v][0], pdinfo->varname[v], opt, 0, prn);
 	    scalars = 1;
 	    gretl_list_delete_at_pos(plist, j);
 	    j--;
-	} 
+	} else {
+	    len = strlen(pdinfo->varname[v]);
+	    if (len > maxlen) {
+		maxlen = len;
+	    }
+	}
     }
 
     if (scalars) {
@@ -1357,7 +1464,8 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 	goto endprint;
     }
 
-    if (!(opt & OPT_O)) { /* not by observations, but by variable */
+    if (!(opt & OPT_O)) { 
+	/* not by observations, but by variable */
 	if (plist[0] > 0) {
 	    pputc(prn, '\n');
 	}
@@ -1385,19 +1493,26 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 
     sortvar = check_for_sorted_var(plist, pdinfo);
 
+    if (maxlen > 13) {
+	ncol = 4;
+	bplen = 16;
+    } else {
+	ncol = 5;
+    }
+
     /* print data by observations */
-    ncol = 5;
     for (j=0; j<=plist[0]/ncol; j++) {
 	char obs_string[OBSLEN];
 
-	j5 = j * ncol;
-	nvj5 = plist[0] - j5;
-	v1 = j5 +1;
-	if (nvj5) {
+	jc = j * ncol;
+	nvjc = plist[0] - jc;
+	v1 = jc +1;
+	if (nvjc) {
 	    /* starting a new block of variables */
-	    v2 = (ncol > nvj5)? nvj5 : ncol;
-	    v2 += j5;
-	    varheading(plist, v1, v2, pdinfo, prn);
+	    v2 = (ncol > nvjc)? nvjc : ncol;
+	    v2 += jc;
+	    varheading(plist, v1, v2, bplen, pdinfo, prn);
+	    printdata_blocks++;
 
 	    if (pause && j > 0 && takenotes(1)) {
 		goto endprint;
@@ -1422,8 +1537,11 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 
 		    if (na(xx)) {
 			strcat(line, "             ");
+			if (bplen == 16) {
+			    strcat(line, "   ");
+			}
 		    } else { 
-			bufprintnum(line, xx, pmax[v-1], 13);
+			bufprintnum(line, xx, pmax[v-1], bplen);
 		    }
 		}
 
@@ -1448,7 +1566,7 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 
 		lineno++;
 	    } /* end of printing obs (t) loop */
-	} /* end if nvj5 */
+	} /* end if nvjc */
     } /* end for j loop */
 
     pputc(prn, '\n');
@@ -1489,11 +1607,12 @@ int print_data_sorted (const int *list, const int *obsvec,
     double xx;
     char obs_string[OBSLEN];
     char line[128];
+    int bplen = 16;
     int T = obsvec[0];
     int i, s, t;
 
-    /* must have a list of up to 5 variables... */
-    if (list == NULL || list[0] > 5) {
+    /* must have a list of up to 4 variables... */
+    if (list == NULL || list[0] > 4) {
 	return E_DATA;
     }
 
@@ -1518,7 +1637,7 @@ int print_data_sorted (const int *list, const int *obsvec,
 	pmax[i-1] = get_signif(Z[list[i]] + pdinfo->t1, T);
     }
 
-    varheading(list, 1, list[0], pdinfo, prn);
+    varheading(list, 1, list[0], bplen, pdinfo, prn);
 
     if (csv) {
 	sdelim[0] = pdinfo->delim;
@@ -1543,13 +1662,13 @@ int print_data_sorted (const int *list, const int *obsvec,
 		if (csv) {
 		    strcat(line, "NA");
 		} else {
-		    strcat(line, "             ");
+		    strcat(line, "                ");
 		}
 	    } else { 
 		if (csv) {
 		    bufprintnum(line, xx, pmax[i-1], 0);
 		} else {
-		    bufprintnum(line, xx, pmax[i-1], 13);
+		    bufprintnum(line, xx, pmax[i-1], bplen);
 		}
 	    }
 	    if (csv && i < list[0]) {
@@ -1576,8 +1695,8 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 
     obs_marker_init(pdinfo);
 
-    for (t=0; t<fr->nobs; t++) {
-	print_obs_marker(t + fr->t1, pdinfo, prn);
+    for (t=fr->t1; t<=fr->t2; t++) {
+	print_obs_marker(t, pdinfo, prn);
 
 	if (na(fr->actual[t])) {
 	    pputc(prn, '\n');
@@ -1674,7 +1793,7 @@ int text_print_forecast (const FITRESID *fr,
     pputc(prn, '\n');
 
     if (do_errs) {
-	for (t=0; t<fr->pre_n; t++) {
+	for (t=0; t<fr->t1; t++) {
 	    maxerr[t] = NADBL;
 	}
 	if (pmax < 4) {
@@ -1684,8 +1803,8 @@ int text_print_forecast (const FITRESID *fr,
 
     obs_marker_init(pdinfo);
 
-    for (t=fr->pre_n; t<fr->nobs; t++) {
-	print_obs_marker(t + fr->t1, pdinfo, prn);
+    for (t=fr->t0; t<=fr->t2; t++) {
+	print_obs_marker(t, pdinfo, prn);
 	fcast_print_x(fr->actual[t], 15, pmax, prn);
 
 	if (na(fr->fitted[t])) {
@@ -1714,34 +1833,11 @@ int text_print_forecast (const FITRESID *fr,
     /* do we really want a plot for non-time series? */
 
     if (plot && fr->nobs > 0) {
-	if (time_series) {
-	    switch (pdinfo->pd) {
-	    case 1:
-		pv = plotvar(pZ, pdinfo, "annual");
-		break;
-	    case 4:
-		pv = plotvar(pZ, pdinfo, "qtrs");
-		break;
-	    case 12:
-		pv = plotvar(pZ, pdinfo, "months");
-		break;
-	    case 24:
-		pv = plotvar(pZ, pdinfo, "hrs");
-		break;
-	    case 10:
-		pv = plotvar(pZ, pdinfo, "decdate");
-		break;
-	    default:
-		pv = plotvar(pZ, pdinfo, "time");
-	    }
-	} else {
-	    pv = plotvar(pZ, pdinfo, "index");
-	}
-
+	pv = plotvar(pZ, pdinfo);
 	if (pv < 0) {
 	    err = 1;
 	} else {
-	    err = plot_fcast_errs(fr->nobs, &(*pZ)[pv][fr->t1], 
+	    err = plot_fcast_errs(fr->t0, fr->t2, (*pZ)[pv], 
 				  fr->actual, fr->fitted, maxerr, 
 				  fr->depvar, 
 				  (time_series)? pdinfo->pd : 0);
@@ -1957,6 +2053,18 @@ static char *varname_string (const char *s, const DATAINFO *pdinfo)
     return vname;
 }
 
+static char *literal_string (const char *s)
+{
+    char *ret = NULL;
+    int n = strlen(s);
+
+    if (*s == '"' && s[n - 1] == '"') {
+	ret = gretl_strndup(s + 1, n - 2);
+    }
+
+    return ret;
+}
+
 static int get_conversion (const char *s, int *skip)
 {
     *skip = strspn(s, "#0123456789.");
@@ -2086,12 +2194,11 @@ static int real_do_printf (const char *line, double ***pZ,
 	    svals[i] = special;
 	} else if ((special = date_string(argv, *pZ, pdinfo)) != NULL) {
 	    svals[i] = special;
+	} else if ((special = literal_string(argv)) != NULL) {
+	    svals[i] = special;
 	} else {
 	    int v = varindex(pdinfo, argv);
 
-#if PRINTF_DEBUG
-	    fprintf(stderr, "looked up '%s' as variable: not found\n", argv);
-#endif
 	    if (v < pdinfo->v) {
 #if PRINTF_DEBUG
 		fprintf(stderr, "'%s' is variable #%d (vector = %d)\n",
@@ -2104,6 +2211,9 @@ static int real_do_printf (const char *line, double ***pZ,
 		    xvals[i] = (*pZ)[v][0];
 		}
 	    } else {
+#if PRINTF_DEBUG
+		fprintf(stderr, "looked up '%s' as variable: not found\n", argv);
+#endif
 		err = get_generated_value(argv, &xvals[i], pZ, pdinfo, t);
 	    }
 	}
