@@ -59,6 +59,8 @@ void varlist (const DATAINFO *pdinfo, PRN *prn);
 
 void maybe_list_vars (const DATAINFO *pdinfo, PRN *prn);
 
+int get_printdata_blocks (void);
+
 int printdata (const int *list, const double **Z, const DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
 
@@ -88,6 +90,8 @@ char *gretl_fix_exponent (char *s);
 
 void bufspace (int n, PRN *prn);
 
+void print_centered (const char *s, int width, PRN *prn);
+
 void gretl_printxn (double x, int n, PRN *prn);
 
 int do_printf (const char *line, double ***pZ, 
