@@ -488,6 +488,30 @@ char *gretl_trunc (char *str, size_t n)
     return str;
 }
 
+/**
+ * gretl_varchar_spn:
+ * @s: the string to truncate.
+ *
+ * Returns: the length of the intial segment of @s which
+ * consists of characters that are valid in a gretl
+ * variable or object name, namely a-z, A-Z, 0-9 and _,
+ * starting with a letter.
+ */
+
+int gretl_varchar_spn (const char *s)
+{
+    const char *varchars = "abcdefghijklmnopqrstuvwxyz"
+	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+	"0123456789_";
+    int ret = 0;
+
+    if (isalpha(*s)) {
+	ret = strspn(s, varchars);
+    }
+
+    return ret;
+}
+
 /**
  * clear:
  * @str: the string to clear.
@@ -507,7 +531,6 @@ void clear (char *str, int len)
  * @s: the string to process.
  *
  * Returns: the number of space-separated fields in @s.
- *
  */
 
 int count_fields (const char *s)
@@ -689,11 +712,8 @@ int top_n_tail (char *str)
 
     if (*str != 0) {
 	/* drop any leading spaces, also possible questionmark */
-	i = 0;
-	while (isspace((unsigned char) str[i]) || str[i] == '?') {
-	    i++;
-	}
-	if (i) {
+	i = strspn(str, " \t?");
+	if (i > 0) {
 	    shift_string_left(str, i);
 	}
 
@@ -992,7 +1012,7 @@ char *get_obs_string (char *obs, int t, const DATAINFO *pdinfo)
 
 /**
  * get_full_obs_string:
- * @obs: char array big enough to hold the observation (OBSLEN).
+ * @obs: char array big enough to hold the observation (#OBSLEN).
  * @t: zero-based observation number.
  * @pdinfo: pointer to dataset information.
  *
@@ -1093,9 +1113,17 @@ void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
 	ntodate_full(tmp, t, pdinfo);
 	if (quarterly_or_monthly(pdinfo)) {
 	    modify_date_for_csv(tmp, pdinfo->pd);
-	    pprintf(prn, "\"%s\"%c", tmp, pdinfo->delim);
+	    if (pdinfo->delim == ',') {
+		pprintf(prn, "\"%s\"%c", tmp, pdinfo->delim);
+	    } else {
+		pprintf(prn, "%s%c", tmp, pdinfo->delim);
+	    }
 	} else {
-	    pprintf(prn, "\"'%s\"%c", tmp, pdinfo->delim);
+	    if (pdinfo->delim == ',') {
+		pprintf(prn, "\"'%s\"%c", tmp, pdinfo->delim);
+	    } else {
+		pprintf(prn, "%s%c", tmp, pdinfo->delim);
+	    }
 	}
     }
 }	
