@@ -20,8 +20,10 @@
 /* subsample.c for gretl */
 
 #include "libgretl.h"
+#include "libset.h"
+#include "gretl_func.h"
 
-#undef SUBDEBUG
+#define SUBDEBUG 0
 
 /*
   The purpose of the static pointers below: When the user subsamples
@@ -48,6 +50,38 @@ static const DATAINFO *peerinfo;
 
 #define SUBMASK_SENTINEL 127
 
+/* special functions shared with dataset.c */
+
+double ***fetch_full_Z (void)
+{
+    return &fullZ;
+}
+
+void reset_full_Z (double ***pZ)
+{
+    fullZ = *pZ;
+}
+
+DATAINFO *fetch_full_datainfo (void)
+{
+    return fullinfo;
+}
+
+/* end specials */
+
+static int full_data_length (const DATAINFO *pdinfo)
+{
+    int n = 0;
+
+    if (fullinfo != NULL) {
+	n = fullinfo->n;
+    } else if (pdinfo != NULL) {
+	n = pdinfo->n;
+    } 
+
+    return n;
+}
+
 static int get_submask_length (const char *s)
 {
     int n = 1;
@@ -75,6 +109,41 @@ char *copy_subsample_mask (const char *src)
     return ret;
 }
 
+char *copy_datainfo_submask (const DATAINFO *pdinfo)
+{
+    char *mask = NULL;
+
+    if (complex_subsampled()) {
+	mask = copy_subsample_mask(pdinfo->submask);
+    }
+
+    return mask;
+}
+
+int submask_cmp (const char *m1, const char *m2)
+{
+    while (*m1 != SUBMASK_SENTINEL && *m2 != SUBMASK_SENTINEL) {
+	if (*m1 != *m2) {
+	    return 1;
+	}
+	m1++;
+	m2++;
+    }
+
+    return 0;
+}
+
+static char *make_submask (int n)
+{
+    char *mask = calloc(n + 1, 1);
+    
+    if (mask != NULL) {
+	mask[n] = SUBMASK_SENTINEL;
+    }
+
+    return mask;
+}
+
 void maybe_free_full_dataset (const DATAINFO *pdinfo)
 {
     if (pdinfo == peerinfo) {
@@ -91,6 +160,17 @@ void maybe_free_full_dataset (const DATAINFO *pdinfo)
     }
 }
 
+static void relink_full_dataset (double ***pZ, DATAINFO **ppdinfo)
+{
+    *pZ = fullZ;
+    *ppdinfo = fullinfo;
+    (*ppdinfo)->submode = 0;
+
+    fullZ = NULL;
+    fullinfo = NULL;
+    peerinfo = NULL;
+}
+
 /* sync malloced elements of the fullinfo struct that might
    have been moved via realloc
 */
@@ -100,35 +180,7 @@ static void sync_dataset_elements (const DATAINFO *pdinfo)
     fullinfo->varname = pdinfo->varname;
     fullinfo->varinfo = pdinfo->varinfo;
     fullinfo->vector = pdinfo->vector;
-    fullinfo->descrip = pdinfo->descrip;      
-}
-
-static int 
-attach_subsample_to_datainfo (DATAINFO *subinfo, const double **Z, 
-			      const DATAINFO *pdinfo)
-{
-    int s, t;
-
-    subinfo->submask = malloc((pdinfo->n + 1) * sizeof *subinfo->submask);
-    if (subinfo->submask == NULL) {
-	return E_ALLOC;
-    }
-
-    s = varindex(pdinfo, "subdum");
-
-    if (s == pdinfo->v) { 
-	/* safety measure: should be impossible */
-	fprintf(stderr, "mystery failure in attach_subsample_to_dataset\n");
-	return 1;   
-    } 
-
-    for (t=0; t<pdinfo->n; t++) {
-	subinfo->submask[t] = Z[s][t];
-    }
-
-    subinfo->submask[pdinfo->n] = SUBMASK_SENTINEL;
-
-    return 0;
+    fullinfo->descrip = pdinfo->descrip;
 }
 
 /* attach_subsample_to_model:
@@ -159,24 +211,233 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     return err;
 }
 
-static double *get_old_mask (double **Z, const DATAINFO *pdinfo)
+/* apparatus for updating full dataset when restoring full sample
+   after sub-sampling */
+
+static void
+update_full_data_values (const double **subZ, const DATAINFO *pdinfo)
 {
-    double *mask = NULL;
-    int s = varindex(pdinfo, "subdum");
+    int i, t;
 
-    if (s < pdinfo->v && gretl_isdummy(0, pdinfo->n - 1, Z[s])) {
-	mask = Z[s];
-    }    
+    for (i=1; i<fullinfo->v; i++) {
+	int subt = 0;
 
-    return mask;
+	if (!pdinfo->vector[i]) {
+	    fullZ[i][0] = subZ[i][0];
+	} else {
+	    for (t=0; t<fullinfo->n; t++) {
+		if (pdinfo->submask[t]) {
+		    fullZ[i][t] = subZ[i][subt++];
+		}
+	    }
+	}
+    }
 }
 
-static int overlay_masks (char *targ, const double *src, int n)
+static int update_case_markers (const DATAINFO *pdinfo)
+{
+    int err = 0;
+
+    if (pdinfo->markers && !fullinfo->markers) {
+	dataset_allocate_obs_markers(fullinfo);
+	if (fullinfo->S == NULL) {
+	    err = 1;
+	} else {
+	    int t, subt = 0;
+
+	    for (t=0; t<fullinfo->n; t++) {
+		if (pdinfo->submask[t]) {
+		    strcpy(fullinfo->S[t], pdinfo->S[subt++]);
+		} else {
+		    sprintf(fullinfo->S[t], "%d", t + 1);
+		}
+	    }
+	}
+    }	
+	
+    return err;
+}
+
+static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
+{
+    int newvars = pdinfo->v - fullinfo->v;
+    int V = fullinfo->v;
+    int N = fullinfo->n;
+    double **newZ = NULL;
+    int i, t, subt;
+    int err = 0;
+
+    if (newvars <= 0) {
+	return 0;
+    }
+
+    if (pdinfo->submask == NULL) {
+	return E_NOMERGE;
+    }
+
+    /* allocate expanded data array */
+    newZ = realloc(fullZ, pdinfo->v * sizeof *fullZ);
+
+    if (newZ == NULL) {
+	return E_ALLOC;
+    } 
+
+    fullZ = newZ;
+
+    for (i=0; i<newvars && !err; i++) {
+	int vi = V + i;
+
+	if (pdinfo->vector[vi]) {
+	    fullZ[vi] = malloc(N * sizeof **newZ);
+	} else {
+	    fullZ[vi] = malloc(sizeof **newZ);
+	}
+	if (fullZ[vi] == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (err) {
+	return E_ALLOC;
+    }
+    
+    for (i=V; i<pdinfo->v; i++) {
+	if (!pdinfo->vector[i]) {
+	   fullZ[i][0] = Z[i][0]; 
+	}
+    }
+
+    subt = 0;
+
+    for (t=0; t<N; t++) {
+	if (pdinfo->submask[t]) {
+	    for (i=V; i<pdinfo->v; i++) {
+		if (pdinfo->vector[i]) {
+		    fullZ[i][t] = Z[i][subt];
+		}
+	    }
+	    subt++;
+	} else {
+	    for (i=V; i<pdinfo->v; i++) { 
+		if (pdinfo->vector[i]) {
+		    fullZ[i][t] = NADBL;
+		}
+	    }
+	}
+    }
+
+    fullinfo->v = pdinfo->v;
+
+    return 0;
+}
+
+static char *make_current_sample_mask (DATAINFO *pdinfo)
+{
+    int n = full_data_length(pdinfo);
+    char *currmask = NULL;
+    int s, t;
+
+    if (pdinfo->submask == NULL) {
+	/* no pre-existing mask: not sub-sampled */
+	currmask = make_submask(n);
+	if (currmask != NULL) {
+	    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+		currmask[t] = 1;
+	    }
+	}
+    } else {
+	/* there's a pre-existing mask */
+	currmask = copy_subsample_mask(pdinfo->submask);
+	if (currmask != NULL) {
+	    s = -1;
+	    for (t=0; t<n; t++) {
+		if (pdinfo->submask[t]) s++;
+		if (s < pdinfo->t1 || s > pdinfo->t2) {
+		    currmask[t] = 0;
+		} 
+	    }
+	}
+    }
+
+    return currmask;
+}
+
+/* restore_full_sample: 
+ * @pZ: pointer to data array.  
+ * @ppdinfo: address of data info pointer. 
+ *
+ * Restore the full data range, undoing any sub-sampling that was
+ * previously performed (either by shifting the endpoints of the
+ * sample range or by selecting cases on some criterion or other).
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int restore_full_sample (double ***pZ, DATAINFO **ppdinfo)
+{
+    int err = 0;
+
+    *gretl_errmsg = '\0';
+
+#if SUBDEBUG
+    fprintf(stderr, "real_restore_full_sample: pZ=%p, ppdinfo=%p cum=%d\n",
+	    (void *) pZ, (void *) ppdinfo, cumulate);
+    fprintf(stderr, "*pZ=%p, *ppdinfo=%p\n",
+	    (void *) *pZ, (void *) *ppdinfo);
+#endif
+
+    /* simple case: merely resetting the sample starting
+       and ending points to full range */
+    if (!complex_subsampled()) {
+	(*ppdinfo)->t1 = 0;
+	(*ppdinfo)->t2 = (*ppdinfo)->n - 1;
+#if SUBDEBUG
+	fprintf(stderr, " reset t1 and t2; all done\n");
+#endif
+	return 0;
+    }
+
+    /* beyond this point we are doing a non-trivial restoration
+       of a stored full dataset, which has previously been
+       subsampled, e.g., by some boolean criterion 
+    */
+
+    /* reattach malloc'd elements of subsampled dataset,
+       which may have moved */
+    sync_dataset_elements(*ppdinfo);
+
+    /* update values for pre-existing series, which may have been
+       modified via genr etc */
+    update_full_data_values((const double **) *pZ, *ppdinfo);
+
+    /* if case markers were added when subsampled, carry them back */
+    update_case_markers(*ppdinfo);
+
+    /* in case any new vars were added when subsampled, try to merge
+       them into the full dataset */
+    err = add_new_vars_to_full((const double **) *pZ, *ppdinfo);
+    if (err == E_ALLOC) {
+        sprintf(gretl_errmsg, _("Out of memory expanding data set\n"));
+    } else if (err == E_NOMERGE) {
+        sprintf(gretl_errmsg, 
+		_("Missing sub-sample information; can't merge data\n"));
+    }
+
+    free_Z(*pZ, *ppdinfo);
+    clear_datainfo(*ppdinfo, CLEAR_SUBSAMPLE);
+    free(*ppdinfo);
+
+    relink_full_dataset(pZ, ppdinfo);
+
+    return err;
+}
+
+static int overlay_masks (char *targ, const char *src, int n)
 {
     int i, sn = 0;
     
     for (i=0; i<n; i++) {
-	if (targ[i] == 1 && src[i] == 1.0) {
+	if (targ[i] == 1 && src[i] == 1) {
 	    targ[i] = 1;
 	    sn++;
 	} else {
@@ -188,94 +449,108 @@ static int overlay_masks (char *targ, const double *src, int n)
 }
 
 static int 
-make_missing_mask (const double **Z, const DATAINFO *pdinfo,
-		   const int *list, char *mask)
+make_missing_mask (const int *list, const double **Z, const DATAINFO *pdinfo,
+		   char *mask)
 {
-    int i, t, v, sn = 0;
-    int lmax;
+    int i, vi, t;
 
     if (list != NULL) {
-	/* examine only a specified list of variables */
-	lmax = list[0];
-    } else {
-	/* default: examine all variables */
-	lmax = pdinfo->v - 1;
-    }
-
-    for (t=0; t<pdinfo->n; t++) {
-	mask[t] = 1;
-	for (i=1; i<=lmax; i++) {
-	    if (list != NULL) {
-		v = list[i];
-	    } else {
-		v = i;
+	/* check specified list of variables */
+	for (t=0; t<pdinfo->n; t++) {
+	    mask[t] = 1;
+	    for (i=1; i<=list[0]; i++) {
+		vi = list[i];
+		if (pdinfo->vector[vi] && na(Z[vi][t])) {
+		    mask[t] = 0;
+		    break;
+		}
 	    }
-	    if (pdinfo->vector[v] && na(Z[v][t])) {
-		mask[t] = 0;
-		break;
+	}
+    } else {	
+	/* check all variables */
+	for (t=0; t<pdinfo->n; t++) {
+	    mask[t] = 1;
+	    for (i=1; i<pdinfo->v; i++) {
+		if (pdinfo->vector[i] && na(Z[i][t])) {
+		    mask[t] = 0;
+		    break;
+		}
 	    }
 	}
-	if (mask[t] == 1) sn++;
     }
 
-    return sn;
+    return 0;
 } 
 
-static void copy_to_mask (char *mask, const double *x, int n)
+static int copy_dummy_to_mask (char *mask, const double *x, int n)
 {
-    int t;
+    int t, err = 0;
 
-    for (t=0; t<n; t++) {
-	mask[t] = (x[t] != 0.0 && !na(x[t]));
+    for (t=0; t<n && !err; t++) {
+	if (x[t] == 1.0) {
+	    mask[t] = 1;
+	} else if (!na(x[t]) && x[t] != 0.0) { /* NA? */
+	    err = 1;
+	}
     }
+
+    return err;
 }
 
-static int sn_from_tmp_dummy (double ***pZ, DATAINFO *pdinfo, 
-			      const char *line, char *mask)
+static int mask_from_temp_dummy (const char *line,
+				 double ***pZ, DATAINFO *pdinfo, 
+				 char *mask)
 {
     char formula[MAXLEN];
-    int dnum, isdum, dlist[2];
-    int err;
+    int dnum, err;
 
     /* + 4 to skip the command word "smpl" */
     sprintf(formula, "__tmpmsk=%s", line + 4);
 
-    err = generate(formula, pZ, pdinfo, OPT_P);
-    *gretl_msg = '\0';
-
+    err = generate(formula, pZ, pdinfo, OPT_P, NULL);
     if (err) {
-	return -1;
+	return err;
     }
 
     dnum = varindex(pdinfo, "tmpmsk");
+    err = copy_dummy_to_mask(mask, (*pZ)[dnum], pdinfo->n);
+    if (err) {
+	sprintf(gretl_errmsg, _("'%s' is not a dummy variable"), "mask");
+    }
 
-    isdum = gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[dnum]);
+    dataset_drop_variable(dnum, pZ, pdinfo);
 
-    copy_to_mask(mask, (*pZ)[dnum], pdinfo->n);
-
-    dlist[0] = 1;
-    dlist[1] = dnum;
-    dataset_drop_listed_variables(dlist, pZ, pdinfo, NULL);
-
-    return isdum;
+    return err;
 }
 
-static int sn_from_dummy (const double **Z, const DATAINFO *pdinfo,
-			  const char *dname, char *mask)
+static int mask_from_dummy (const char *line,
+			    const double **Z, const DATAINFO *pdinfo,
+			    char *mask)
 {
-    int dnum = varindex(pdinfo, dname);
-    int isdum;
+    char dname[VNAMELEN] = {0};
+    int dnum, err = 0;
+
+    /* + 4 to skip the command word "smpl" */
+    sscanf(line + 4, "%15s", dname);
+
+    if (*dname == 0) {
+	strcpy(gretl_errmsg, "Unrecognized sample command");
+	return 1;
+    }
+
+    dnum = varindex(pdinfo, dname);
 
     if (dnum == pdinfo->v) {
 	sprintf(gretl_errmsg, _("Variable '%s' not defined"), dname);
-	return -1;
-    } 
+	err = 1;
+    } else {
+	err = copy_dummy_to_mask(mask, Z[dnum], pdinfo->n);
+	if (err) {
+	    sprintf(gretl_errmsg, _("'%s' is not a dummy variable"), dname);
+	}
+    }
 
-    isdum = gretl_isdummy(pdinfo->t1, pdinfo->t2, Z[dnum]);
-
-    copy_to_mask(mask, Z[dnum], pdinfo->n);
-
-    return isdum;
+    return err;
 } 
 
 static int count_selected_cases (const char *x, int n)
@@ -291,72 +566,67 @@ static int count_selected_cases (const char *x, int n)
     return count;
 }
 
-static int make_random_mask (double *oldmask, int fulln, int subn, 
+static int make_random_mask (const char *oldmask, const char *line, 
+			     const double **Z, const DATAINFO *pdinfo,
 			     char *mask)
 {
-    int i, cases = 0, err = 0;
+    char s[VNAMELEN] = {0};
     unsigned u;
+    int i, subn = 0;
+    int cases = 0;
+    int err = 0;
 
-    if (subn <= 0 || subn >= fulln) {
+    sscanf(line + 4, "%15s", s);
+    if (*s == 0) {
+	sprintf(gretl_errmsg, _("Invalid number of cases %d"), subn);
+	return 0;
+    } else if (isdigit(*s)) {
+	subn = atoi(s);
+    } else {
+	int v = varindex(pdinfo, s);
+
+	if (v < pdinfo->v && !na(Z[v][0])) {
+	    subn = Z[v][0];
+	}
+    }
+
+    if (subn <= 0 || subn >= pdinfo->n) {
 	err = 1;
     } else if (oldmask != NULL) {
 	int oldn = 0;
 
-	for (i=0; i<fulln; i++) {
-	    if (oldmask[i] != 0.0) oldn++;
+	for (i=0; i<pdinfo->n; i++) {
+	    if (oldmask[i]) {
+		oldn++;
+	    }
+	}
+	if (subn >= oldn) {
+	    err = 1;
 	}
-
-	if (subn >= oldn) err = 1;
     }	
 
     if (err) {
 	sprintf(gretl_errmsg, _("Invalid number of cases %d"), subn);
-	return 0;
+	return err;
     }	
 
-    for (i=0; i<fulln; i++) {
+    for (i=0; i<pdinfo->n; i++) {
 	mask[i] = 0;
     }
 
     for (i=0; (cases != subn); i++) {
-	u = gretl_rand_int_max(fulln);
-	if (oldmask == NULL || oldmask[u] == 1.0) {
+	u = gretl_rand_int_max(pdinfo->n);
+	if (oldmask == NULL || oldmask[u]) {
 	    mask[u] = 1;
 	}
 	if (i >= subn - 1) {
-	    cases = count_selected_cases(mask, fulln);
+	    cases = count_selected_cases(mask, pdinfo->n);
 	}
     }
 
-    return cases;
+    return err;
 }
 
-static int maybe_add_subdum (double ***pZ, DATAINFO *pdinfo, int *snum)
-{
-    int s = varindex(pdinfo, "subdum");
-
-    if (s == pdinfo->v) {
-	/* variable doesn't exist: create it */
-	if (dataset_add_series(1, pZ, pdinfo)) {
-	    return 1;
-	}
-	strcpy(pdinfo->varname[s], "subdum");
-	strcpy(VARLABEL(pdinfo, s), _("automatic sub-sampling dummy"));
-    } 
-
-    *snum = s;
-
-    return 0;
-}
-
-enum {
-    SUBSAMPLE_UNKNOWN,
-    SUBSAMPLE_DROP_MISSING,
-    SUBSAMPLE_USE_DUMMY,
-    SUBSAMPLE_BOOLEAN,
-    SUBSAMPLE_RANDOM
-} subsample_options;
-
 static void backup_full_dataset (double ***pZ, DATAINFO **ppdinfo,
 				 const DATAINFO *newinfo)
 {
@@ -365,16 +635,6 @@ static void backup_full_dataset (double ***pZ, DATAINFO **ppdinfo,
     peerinfo = newinfo;
 }
 
-static void relink_full_dataset (double ***pZ, DATAINFO **ppdinfo)
-{
-    *pZ = fullZ;
-    *ppdinfo = fullinfo;
-
-    fullZ = NULL;
-    fullinfo = NULL;
-    peerinfo = NULL;
-}
-
 int complex_subsampled (void)
 {
     return (fullZ != NULL);
@@ -618,88 +878,47 @@ copy_data_to_subsample (double **subZ, DATAINFO *subinfo,
     sprintf(subinfo->endobs, "%d", subinfo->n);
 }
 
-/* restrict_sample: 
- * @line: command line (or %NULL).  
- * @pZ: pointer to original data array.  
- * @ppdinfo: address of original data info pointer. 
- * @list: list of variables in case of OPT_M (or %NULL).  
- * @oflag: option flag.
- *
- * sub-sample the data set, based on the criterion of skipping all
- * observations with missing data values (OPT_M); or using as a mask a
- * specified dummy variable (OPT_O); or masking with a specified
- * boolean condition (OPT_R); or selecting at random (OPT_N).
- *
- * In case OPT_M a @list of variables may be supplied; in cases
- * OPT_O, OPT_R and OPT_N, @line must contain specifics.
- *
- * Returns: 0 on success, non-zero error code on failure.
- */
+int get_restriction_mode (gretlopt opt)
+{
+    int mode = SUBSAMPLE_UNKNOWN;
 
-int restrict_sample (const char *line, 
-		     double ***pZ, DATAINFO **ppdinfo, 
-		     const int *list, gretlopt oflag)
+    if (opt & OPT_M) {
+	mode = SUBSAMPLE_DROP_MISSING;
+    } else if (opt & OPT_R) {
+	mode = SUBSAMPLE_BOOLEAN;
+    } else if (opt & OPT_N) {
+	mode = SUBSAMPLE_RANDOM;
+    } else if (opt & OPT_O) {
+	mode = SUBSAMPLE_USE_DUMMY;
+    } 
+
+    return mode;
+}
+
+static int 
+make_restriction_mask (int mode, const char *line, const int *list, 
+		       double ***pZ, DATAINFO *pdinfo,
+		       PRN *prn, const char *oldmask, char **pmask)
 {
-    char dname[VNAMELEN] = {0};
-    int subnum = 0;
-    int t, sn = 0;
-    int opt = SUBSAMPLE_UNKNOWN;
-    double *oldmask = NULL;
-    double **subZ = NULL;
-    DATAINFO *subinfo = NULL;
     char *mask = NULL;
-    int err = 0;
+    int sn = 0, err = 0;
 
-    *gretl_errmsg = '\0';
-    *gretl_msg = '\0';
-
-    if (oflag & OPT_M) {
-	opt = SUBSAMPLE_DROP_MISSING;
-    } else if (oflag & OPT_R) {
-	opt = SUBSAMPLE_BOOLEAN;
-    } else if (oflag & OPT_N) {
-	opt = SUBSAMPLE_RANDOM;
-    } else if (oflag & OPT_O) {
-	if (line != NULL && sscanf(line, "%*s %s", dname)) {
-	    opt = SUBSAMPLE_USE_DUMMY;
-	} else {
-	    opt = SUBSAMPLE_DROP_MISSING;
-	}
-    } else {
-	strcpy(gretl_errmsg, "Unrecognized sample command");
-	return 1;
-    }
-
-    /* allocate temporary (char) mask series */
-    mask = calloc((*ppdinfo)->n, 1);
+    mask = make_submask(pdinfo->n);
     if (mask == NULL) {
 	return E_ALLOC;
     }
 
-    /* if there's no currently existing sub-sample mask, 
-       oldmask will be NULL (which is OK) */
-    oldmask = get_old_mask(*pZ, *ppdinfo);
-
     /* construct subsample mask in one of several possible ways */
-
-    if (opt == SUBSAMPLE_DROP_MISSING) {   
-	sn = make_missing_mask((const double **) *pZ, *ppdinfo, list, mask);
-    } else if (opt == SUBSAMPLE_RANDOM) {
-	sn = make_random_mask(oldmask, (*ppdinfo)->n, atoi(line + 4), mask);
-	if (sn == 0) {
-	    err = 1;
-	}
-    } else if (opt == SUBSAMPLE_USE_DUMMY || opt == SUBSAMPLE_BOOLEAN) {
-	if (opt == SUBSAMPLE_USE_DUMMY) {
-	    sn = sn_from_dummy((const double **) *pZ, *ppdinfo, dname, mask);
-	} else {
-	    sn = sn_from_tmp_dummy(pZ, *ppdinfo, line, mask);
-	}
-	if (sn < 0) {
-	    err = 1;
-	}
+    if (mode == SUBSAMPLE_DROP_MISSING) {   
+	err = make_missing_mask(list, (const double **) *pZ, pdinfo, mask);
+    } else if (mode == SUBSAMPLE_RANDOM) {
+	err = make_random_mask(oldmask, line, (const double **) *pZ,
+			       pdinfo, mask);
+    } else if (mode == SUBSAMPLE_USE_DUMMY) {
+	err = mask_from_dummy(line, (const double **) *pZ, pdinfo, mask);
+    } else if (mode == SUBSAMPLE_BOOLEAN) {
+	err = mask_from_temp_dummy(line, pZ, pdinfo, mask);
     } else {
-	/* impossible */
 	strcpy(gretl_errmsg, _("Sub-sample command failed mysteriously"));
 	err = 1;
     }
@@ -707,76 +926,65 @@ int restrict_sample (const char *line,
     /* exit now on unrecoverable error */
     if (err) {
 	free(mask);
-	return 1;
+	return err;
     }
 
     /* cumulate sample restrictions, if appropriate */
-    if (oldmask != NULL && opt != SUBSAMPLE_RANDOM) {
-	sn = overlay_masks(mask, oldmask, (*ppdinfo)->n);
+    if (oldmask != NULL && mode != SUBSAMPLE_RANDOM) {
+	sn = overlay_masks(mask, oldmask, pdinfo->n);
+    } else {
+	sn = count_selected_cases(mask, pdinfo->n);
     }
 
-    /* does this policy lead to an empty sample, or no change
-       in the sample, perchance? */
+    /* does this policy lead to an empty sample, or no change in the
+       sample, perchance? */
+
     if (sn == 0) {
-	if (opt == SUBSAMPLE_USE_DUMMY) {
-	    sprintf(gretl_errmsg, _("'%s' is not a dummy variable"), dname);
-	} else if (opt == SUBSAMPLE_DROP_MISSING) {
-	    strcpy(gretl_errmsg, _("No observations would be left!"));
-	} else if (opt == SUBSAMPLE_BOOLEAN) { 
-	    if (mask[(*ppdinfo)->t1] == 0) {
-		strcpy(gretl_errmsg, _("No observations would be left!"));
-	    } else {
-		/* this is not really an error, just a no-op */
-		strcpy(gretl_msg, _("No observations were dropped!"));
-		free(mask);
-		return 0;
-	    }
+	strcpy(gretl_errmsg, _("No observations would be left!"));
+	err = 1;
+    } else if (sn == pdinfo->n) {
+	/* not really an error, just a no-op */
+	if (gretl_messages_on()) {
+	    pputs(prn, _("No observations were dropped!"));
+	    pputc(prn, '\n');
 	}
-	err = 1;
-    } else if (sn == (*ppdinfo)->n) {
-	strcpy(gretl_errmsg, _("No observations were dropped!"));
-	err = 1;
+	free(mask);
+	mask = NULL;
     }
 
-    /* exit now on nonsense subsample */
     if (err) {
 	free(mask);
-	return 1;
+    } else {
+	*pmask = mask;
     }
 
-    /* now we start making the new dataset */
+    return err;
+}
+
+int 
+restrict_sample_from_mask (const char *mask, int mode, double ***pZ, 
+			   DATAINFO **ppdinfo)
+{
+    double **subZ = NULL;
+    DATAINFO *subinfo = NULL;
+    int err = 0;
 
     /* allocate new datainfo */
     subinfo = datainfo_new();
     if (subinfo == NULL) {
-	free(mask);
 	return E_ALLOC;
     }
 
-    /* create "hidden" dummy in full dataset to record sub-sample, 
-       if need be */
-    if (maybe_add_subdum(pZ, *ppdinfo, &subnum)) {
-	free(subinfo);
-	free(mask);
-	return E_ALLOC;
-    } 
-
-    /* write the new mask into the "subdum" variable */
-    for (t=0; t<(*ppdinfo)->n; t++) {
-	(*pZ)[subnum][t] = mask[t];
-    }
-
     /* set up the sub-sampled datainfo */
-    subinfo->n = sn;
+    subinfo->n = count_selected_cases(mask, (*ppdinfo)->n);
     subinfo->v = (*ppdinfo)->v;
     if (start_new_Z(&subZ, subinfo, 1)) {
 	free(subinfo);
-	free(mask);
 	return E_ALLOC;
     }
 
-    /* link (don't copy) varnames and descriptions, 
-       these are not dependent on the series length */
+    /* link (don't copy) varnames and descriptions, since these are
+       not dependent on the series length */
     subinfo->varname = (*ppdinfo)->varname;
     subinfo->varinfo = (*ppdinfo)->varinfo;
     subinfo->descrip = (*ppdinfo)->descrip;
@@ -788,24 +996,18 @@ int restrict_sample (const char *line,
 	if (err) {
 	    free_Z(subZ, subinfo);
 	    free(subinfo);
-	    free(mask);
 	    return E_ALLOC;
 	}
 	subinfo->markers = (*ppdinfo)->markers;
     }
 
     /* copy across data (and case markers, if any) */
-    copy_data_to_subsample(subZ, subinfo, 
-			   (const double **) *pZ, *ppdinfo,
-			   mask);
+    copy_data_to_subsample(subZ, subinfo, (const double **) *pZ, 
+			   *ppdinfo, mask);
 
-    attach_subsample_to_datainfo(subinfo, 
-				 (const double **) *pZ, 
-				 *ppdinfo);
-
-    if (opt == SUBSAMPLE_USE_DUMMY || 
-	opt == SUBSAMPLE_BOOLEAN ||
-	opt == SUBSAMPLE_DROP_MISSING) {
+    if (mode == SUBSAMPLE_USE_DUMMY || 
+	mode == SUBSAMPLE_BOOLEAN ||
+	mode == SUBSAMPLE_DROP_MISSING) {
 	if (dataset_is_panel(*ppdinfo)) {
 	    maybe_reconstitute_panel(*ppdinfo, mask, subinfo);
 	} else if (dataset_is_time_series(*ppdinfo)) {
@@ -815,16 +1017,86 @@ int restrict_sample (const char *line,
 
     /* save state */
     backup_full_dataset(pZ, ppdinfo, subinfo);
+    subinfo->submode = mode;
+    subinfo->submask = copy_subsample_mask(mask);
 
     /* and switch pointers */
     *pZ = subZ;
     *ppdinfo = subinfo;
 
-    free(mask);
-
     return 0;
 }
 
+/* restrict_sample: 
+ * @line: command line (or %NULL).  
+ * @pZ: pointer to original data array.  
+ * @ppdinfo: address of original data info pointer. 
+ * @list: list of variables in case of OPT_M (or %NULL).  
+ * @opt: option flags.
+ * @prn: printing apparatus.
+ *
+ * Sub-sample the data set, based on the criterion of skipping all
+ * observations with missing data values (OPT_M); or using as a mask a
+ * specified dummy variable (OPT_O); or masking with a specified
+ * boolean condition (OPT_R); or selecting at random (OPT_N).
+ *
+ * In case OPT_M a @list of variables may be supplied; in cases
+ * OPT_O, OPT_R and OPT_N, @line must contain specifics.
+ *
+ * In case OPT_P is included, the restriction will rePlace any
+ * existing sample restriction, otherwise the resulting restriction
+ * will be the logical product of the new restriction and any
+ * existing restriction.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int restrict_sample (const char *line, const int *list,
+		     double ***pZ, DATAINFO **ppdinfo, 
+		     gretlopt opt, PRN *prn)
+{
+    char *oldmask = NULL;
+    char *mask = NULL;
+    int mode, err = 0;
+
+    *gretl_errmsg = '\0';
+
+    mode = get_restriction_mode(opt);
+    if (mode == SUBSAMPLE_UNKNOWN) {
+	strcpy(gretl_errmsg, "Unrecognized sample command");
+	return 1;
+    }
+
+    if (!(opt & OPT_P)) {
+	/* not replacing but cumulating any existing restrictions */
+	oldmask = make_current_sample_mask(*ppdinfo);
+	if (oldmask == NULL) {
+	    return E_ALLOC;
+	}
+    }
+
+    /* We must first restore the full data range, for housekeeping
+       purposes */
+    err = restore_full_sample(pZ, ppdinfo);
+    if (err) {
+	return err;
+    }
+
+    if (!err) {
+	err = make_restriction_mask(mode, line, list, pZ, *ppdinfo, 
+				    prn, oldmask, &mask);
+    }
+
+    if (!err && mask != NULL) {
+	err = restrict_sample_from_mask(mask, mode, pZ, ppdinfo);
+    }
+
+    free(mask);
+    free(oldmask);
+
+    return err;
+}
+
 enum {
     SMPL_T1,
     SMPL_T2
@@ -870,7 +1142,6 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     char cmd[5], newstart[OBSLEN], newstop[OBSLEN];
 
     *gretl_errmsg = '\0';
-    *gretl_msg = '\0';
 
     nf = count_fields(line);
 
@@ -939,237 +1210,6 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     return 0;
 }
 
-/* apparatus for updating full dataset when restoring full sample
-   after sub-sampling */
-
-static void
-update_full_data_values (const double **subZ, const DATAINFO *pdinfo)
-{
-    int i, t;
-
-    for (i=1; i<fullinfo->v; i++) {
-	int subt = 0;
-
-	if (!pdinfo->vector[i]) {
-	    fullZ[i][0] = subZ[i][0];
-	} else {
-	    for (t=0; t<fullinfo->n; t++) {
-		if (pdinfo->submask[t]) {
-		    fullZ[i][t] = subZ[i][subt++];
-		}
-	    }
-	}
-    }
-}
-
-static int update_case_markers (const DATAINFO *pdinfo)
-{
-    int err = 0;
-
-    if (pdinfo->markers && !fullinfo->markers) {
-	dataset_allocate_obs_markers(fullinfo);
-	if (fullinfo->S == NULL) {
-	    err = 1;
-	} else {
-	    int t, subt = 0;
-
-	    for (t=0; t<fullinfo->n; t++) {
-		if (pdinfo->submask[t]) {
-		    strcpy(fullinfo->S[t], pdinfo->S[subt++]);
-		} else {
-		    sprintf(fullinfo->S[t], "%d", t + 1);
-		}
-	    }
-	}
-    }	
-	
-    return err;
-}
-
-static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
-{
-    int i, t, subt;
-    int newvars = pdinfo->v - fullinfo->v;
-    int n = fullinfo->n;
-    double **newZ = NULL;
-    int err = 0;
-
-    if (newvars <= 0) {
-	return 0;
-    }
-
-    if (pdinfo->submask == NULL) {
-	return E_NOMERGE;
-    }
-
-    /* allocate expanded data array */
-    newZ = realloc(fullZ, pdinfo->v * sizeof *fullZ);
-    if (newZ != NULL) {
-	for (i=0; i<newvars; i++) {
-	    if (pdinfo->vector[fullinfo->v+i]) {
-		newZ[fullinfo->v+i] = malloc(n * sizeof **newZ);
-	    } else {
-		newZ[fullinfo->v+i] = malloc(sizeof **newZ);
-	    }
-	    if (newZ[fullinfo->v+i] == NULL) {
-		err = 1;
-		break;
-	    }
-	}
-    } else {
-	err = 1;
-    }
-
-    if (err) {
-	return E_ALLOC;
-    }
-    
-    fullZ = newZ;
-
-    for (i=fullinfo->v; i<pdinfo->v; i++) {
-	if (!pdinfo->vector[i]) {
-	   fullZ[i][0] = Z[i][0]; 
-	}
-    }
-
-    subt = 0;
-
-    for (t=0; t<n; t++) {
-	if (pdinfo->submask[t]) {
-	    for (i=fullinfo->v; i<pdinfo->v; i++) {
-		if (pdinfo->vector[i]) {
-		    fullZ[i][t] = Z[i][subt];
-		}
-	    }
-	    subt++;
-	} else {
-	    for (i=fullinfo->v; i<pdinfo->v; i++) { 
-		if (pdinfo->vector[i]) {
-		    fullZ[i][t] = NADBL;
-		}
-	    }
-	}
-    }
-
-    fullinfo->v = pdinfo->v;
-
-    return 0;
-}
-
-static int make_smpl_mask (double ***pZ, DATAINFO *pdinfo)
-{
-    int old_vmax = pdinfo->v;
-    int v, t;
-    int err = 0;
-
-    /* add "subdum" variable if it's not already present */
-    if (maybe_add_subdum(pZ, pdinfo, &v)) {
-	err = 1;
-    } else if (v == old_vmax) {
-	/* no pre-existing mask: write it from scratch */
-	for (t=0; t<pdinfo->n; t++) {
-	    (*pZ)[v][t] = (t < pdinfo->t1 || t > pdinfo->t2)? 0.0 : 1.0;
-	}
-    } else {
-	/* there was a pre-existing mask: adjust it */
-	for (t=0; t<pdinfo->n; t++) {
-	    if (t < pdinfo->t1 || t > pdinfo->t2) {
-		(*pZ)[v][t] = 0.0;
-	    }
-	}	
-    }
-
-    return err;
-}
-
-/* restore_full_sample is called under various circumstances.
-   Complexities arise if it is being called as a prelude to
-   the imposition of a (new) sample restriction: then we have
-   to know whether the new restriction will replace any old
-   one, or whether the restrictions should be cumulated.
-
-   If the call is a prelude to _replacing_ an existing
-   sample restriction, the "opt" parameter should equal
-   OPT_C (think "replaCe"!).
-*/
-
-int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt)
-{
-    int i, t, err = 0;
-
-    *gretl_errmsg = '\0';
-    *gretl_msg = '\0';
-
-#if SUBDEBUG
-    fprintf(stderr, "restore_full_sample: pZ=%p, ppdinfo=%p opt=%lu\n",
-	    (void *) pZ, (void *) ppdinfo, opt);
-    fprintf(stderr, "*pZ=%p, *ppdinfo=%p\n",
-	    (void *) *pZ, (void *) *ppdinfo);
-#endif
-
-    if (*pZ != NULL && !(opt & OPT_C)) {  
-	/* cumulating, not replacing restrictions: we need to
-	   ensure that an appropriate mask is in place */
-	err = make_smpl_mask(pZ, *ppdinfo);
-	if (err) {
-	    return err;
-	}
-    }
-
-    /* simple case: merely resetting the sample starting
-       and ending points to full range */
-    if (!complex_subsampled()) {
-	(*ppdinfo)->t1 = 0;
-	(*ppdinfo)->t2 = (*ppdinfo)->n - 1;
-	return 0;
-    }
-
-    /* beyond this point we are doing a non-trivial restoration
-       of a stored full dataset, which has previously been
-       subsampled, e.g., by some boolean criterion 
-    */
-
-    /* reattach malloc'd elements of subsampled dataset,
-       which may have moved */
-    sync_dataset_elements(*ppdinfo);
-
-    /* update values for pre-existing series, which may have been
-       modified via genr */
-    update_full_data_values((const double **) *pZ, *ppdinfo);
-
-    /* if case markers were added when subsampled, carry them back */
-    update_case_markers(*ppdinfo);
-
-    /* in case any new vars were added when subsampled, try to merge
-       them into the full dataset */
-    err = add_new_vars_to_full((const double **) *pZ, *ppdinfo);
-    if (err == E_ALLOC) {
-        sprintf(gretl_errmsg, _("Out of memory expanding data set\n"));
-    } else if (err == E_NOMERGE) {
-        sprintf(gretl_errmsg, 
-		_("Missing sub-sample information; can't merge data\n"));
-    }
-
-    /* if we plan to replace existing sample restrictions,
-       zero out the "subdum" dummy variable */
-    if (opt & OPT_C) {
-	i = varindex(fullinfo, "subdum");
-	if (i < fullinfo->v) {
-	    for (t=0; t<fullinfo->n; t++) {
-		fullZ[i][t] = 0.0;
-	    }
-	}
-    }
-
-    free_Z(*pZ, *ppdinfo);
-    clear_datainfo(*ppdinfo, CLEAR_SUBSAMPLE);
-    free(*ppdinfo);
-
-    relink_full_dataset(pZ, ppdinfo);
-
-    return 0;
-}
-
 int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 {
     int i, t, tmiss;
