@@ -22,15 +22,30 @@
 #ifndef SUBSAMPLE_H
 #define SUBSAMPLE_H
 
-/* functions follow */
+typedef enum {
+    SUBSAMPLE_NONE,
+    SUBSAMPLE_DROP_MISSING,
+    SUBSAMPLE_USE_DUMMY,
+    SUBSAMPLE_BOOLEAN,
+    SUBSAMPLE_RANDOM,
+    SUBSAMPLE_UNKNOWN
+} SubsampleMode;
 
 char *copy_subsample_mask (const char *src);
 
+char *copy_datainfo_submask (const DATAINFO *pdinfo);
+
+int submask_cmp (const char *m1, const char *m2);
+
 int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo);
 
-int restrict_sample (const char *line, 
+int restrict_sample (const char *line, const int *list,  
 		     double ***pZ, DATAINFO **ppdinfo,
-		     const int *list, gretlopt oflag);
+		     gretlopt oflag, PRN *prn);
+
+int 
+restrict_sample_from_mask (const char *mask, int mode, 
+			   double ***pZ, DATAINFO **ppdinfo);
 
 int complex_subsampled (void);
 
@@ -38,7 +53,7 @@ int get_full_length_n (void);
 
 int set_sample (const char *line, const double **Z, DATAINFO *pdinfo);
 
-int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt); 
+int restore_full_sample (double ***pZ, DATAINFO **ppdinfo); 
 
 int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn);
 
