@@ -322,17 +322,30 @@ void gretl_equation_system_destroy (gretl_equation_system *sys)
     free(sys);
 }
 
-static void sur_rearrange_lists (gretl_equation_system *sys)
+static void sur_rearrange_lists (gretl_equation_system *sys,
+				 const double **Z,
+				 const DATAINFO *pdinfo)
 {
     if (sys->method == SYS_SUR) {
 	int i;
 
 	for (i=0; i<sys->n_equations; i++) {
-	    rearrange_list(sys->lists[i]);
+	    reglist_check_for_const(sys->lists[i], Z, pdinfo);
 	}
     }
 }
 
+/**
+ * gretl_equation_system_append:
+ * @sys: initialized equation system.
+ * @list: list containing dependent variable and regressors.
+ * 
+ * Adds an equation (as represented by @list) to @sys.
+ * 
+ * Returns: 0 on success, non-zero on failure, in which case
+ * @sys is destroyed.
+ */
+
 int gretl_equation_system_append (gretl_equation_system *sys, 
 				  const int *list)
 {
@@ -348,13 +361,10 @@ int gretl_equation_system_append (gretl_equation_system *sys,
     sys->lists = realloc(sys->lists, (neq + 1) * sizeof *sys->lists);
     if (sys->lists == NULL) return E_ALLOC;
 
-    sys->lists[neq] = malloc((list[0] + 1) * sizeof *list);
+    sys->lists[neq] = gretl_list_new(list[0]);
+
     if (sys->lists[neq] == NULL) {
-	for (i=0; i<neq; i++) {
-	    free(sys->lists[i]);
-	}
-	free(sys->lists);
-	sys->lists = NULL;
+	gretl_equation_system_destroy(sys);
 	return E_ALLOC;
     }
 
@@ -457,8 +467,12 @@ static int get_estimation_method_from_line (const char *s)
 }
 
 static void 
-system_set_save_flags (gretl_equation_system *sys, const char *s)
+system_set_flags (gretl_equation_system *sys, const char *s, gretlopt opt)
 {
+    if (opt & OPT_T) {
+	sys->flags |= GRETL_SYS_ITERATE;
+    }
+
     s = strstr(s, " save");
 
     if (s != NULL) {
@@ -475,17 +489,24 @@ system_set_save_flags (gretl_equation_system *sys, const char *s)
     }
 }
 
-/* Start compiling an equation system in response to gretl's "system"
-   command: the command must specify a "method" (estimation method)
-   and/or a name for the system.  If a method is given, the system
-   will be estimated as soon as its definition is complete.  If a name
-   is given, the system definition is saved on a stack, and it can
-   subsequently be estimated via various methods (the "estimate"
-   command).  If both a name and an estimation method are given, the
-   system is both estimated and saved.
-*/
+/**
+ * system_start:
+ * @line: command line.
+ * @opt: may include %OPT_T for iterative estimation, if the
+ * estimation method supports this.
+ * 
+ * Start compiling an equation system: @line must specify a "method" 
+ * (estimation method) and/or a name for the system.  If a method is
+ * given, the system will be estimated as soon as its definition is 
+ * complete.  If a name is given, the system definition is saved on a 
+ * stack, and it can subsequently be estimated via various methods
+ * If both a name and an estimation method are given, the system 
+ * is both estimated and saved.
+ * 
+ * Returns: pointer to a new equation system, or %NULL on error.
+ */
 
-gretl_equation_system *system_start (const char *line)
+gretl_equation_system *system_start (const char *line, gretlopt opt)
 {
     gretl_equation_system *sys = NULL;
     char *sysname = NULL;
@@ -512,7 +533,7 @@ gretl_equation_system *system_start (const char *line)
 	return NULL;
     }
 
-    system_set_save_flags(sys, line);
+    system_set_flags(sys, line, opt);
 
     if (sysname != NULL) {
 	free(sysname);
@@ -717,7 +738,19 @@ static int estimate_with_test (gretl_equation_system *sys,
     return err;
 }
 
-/* driver function for the routines in the "sysest" plugin */
+/**
+ * gretl_equation_system_estimate:
+ * @sys: pre-defined equation system.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
+ * @opt: may include %OPT_Q for relatively quiet operation.
+ * @prn: printing struct.
+ * 
+ * Estimate a pre-defined equation system and print the results
+ * to @prn.
+ * 
+ * Returns: 0 on success, non-zero on error.
+ */
 
 int 
 gretl_equation_system_estimate (gretl_equation_system *sys, 
@@ -739,7 +772,7 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
     if (err) goto system_bailout;
     
     if (sys->method == SYS_SUR) {
-	sur_rearrange_lists(sys);
+	sur_rearrange_lists(sys, (const double **) *pZ, pdinfo);
     }
 
     system_est = get_plugin_function("system_estimate", &handle);
@@ -775,11 +808,21 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
     return err;
 }
 
-/* Finalize an equation system in response to "end system".  If the
-   system has a specified name, we save it on a stack of defined
-   systems.  If it has a specified estimation method, we go ahead
-   and estimate it.  If it has both, we do both.
-*/
+/**
+ * gretl_equation_system_finalize:
+ * @sys: pre-defined equation system.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
+ * @prn: printing struct.
+ * 
+ * Finalize an equation system, e.g. in response to "end system".
+ * If the system has a specified name, we save it on a stack of 
+ * defined systems.  If it has a specified estimation method, 
+ * we go ahead and estimate it.  If it has both, we do both.
+ * 
+ * Returns: 0 on success, non-zero on error.  If the system is
+ * mal-formed, it is destroyed.
+ */
 
 int gretl_equation_system_finalize (gretl_equation_system *sys, 
 				    double ***pZ, DATAINFO *pdinfo,
@@ -807,12 +850,10 @@ int gretl_equation_system_finalize (gretl_equation_system *sys,
 	return 1;
     }    
 
-#if 1 /* for cli program */
     if (sys->name != NULL) {
 	/* save the system for subsequent estimation */
-	err = stack_system_as(sys, sys->name);
+	err = gretl_stack_object_as(sys, GRETL_OBJ_SYS, sys->name);
     }
-#endif
 
     if (!err && sys->method >= 0) {
 	err = gretl_equation_system_estimate(sys, pZ, pdinfo, opt, prn);
@@ -1586,7 +1627,7 @@ add_aux_list_to_sys (gretl_equation_system *sys, const char *line,
     const char *p;
     char vname[VNAMELEN];
     int *list;
-    int i, v, nf, len, cplen;
+    int i, j, v, nf, len, cplen;
     int err = 0;
 
     if (which == ENDOG_LIST) {
@@ -1598,7 +1639,8 @@ add_aux_list_to_sys (gretl_equation_system *sys, const char *line,
 	if (sys->instr_vars != NULL) {
 	    strcpy(gretl_errmsg, "Only one list of instruments may be given");
 	    return 1;
-	} else if (sys->method != SYS_3SLS && sys->method != SYS_TSLS) {
+	} else if (0 && sys->method != SYS_3SLS && sys->method != SYS_TSLS) {
+	    /* FIXME ?? */
 	    strcpy(gretl_errmsg, "Instruments may be specified only "
 		   "for 3SLS or TSLS");
 	    return 1;
@@ -1608,14 +1650,17 @@ add_aux_list_to_sys (gretl_equation_system *sys, const char *line,
     }
 
     nf = count_fields(line);
-    if (nf < 1) return 1;
+    if (nf < 1) {
+	return 1;
+    }
 
-    list = malloc((nf + 1) * sizeof *list);
-    if (list == NULL) return E_ALLOC;
+    list = gretl_list_new(nf);
+    if (list == NULL) {
+	return E_ALLOC;
+    }
 
-    list[0] = nf;
-    
     p = line;
+    j = 1;
     for (i=1; i<=nf && !err; i++) {
 	while (isspace(*p)) p++;
 	*vname = '\0';
@@ -1630,10 +1675,20 @@ add_aux_list_to_sys (gretl_equation_system *sys, const char *line,
 	    v = varindex(pdinfo, vname);
 	}
 	if (v < 0 || v >= pdinfo->v) {
-	    sprintf(gretl_errmsg, "Undefined variable '%s'.", vname);
-	    err = 1;
+	    /* maybe a named list */
+	    int *sublist = get_list_by_name(vname);
+
+	    if (sublist == NULL) {
+		sprintf(gretl_errmsg, "Undefined variable '%s'.", vname);
+		err = 1;
+	    } else {
+		err = gretl_list_insert_list_minus(&list, sublist, j);
+		if (!err) {
+		    j += sublist[0];
+		}
+	    }
 	} else {
-	    list[i] = v;
+	    list[j++] = v;
 	}
 	p += len;
     }
@@ -1652,6 +1707,22 @@ add_aux_list_to_sys (gretl_equation_system *sys, const char *line,
     return 0;
 }
 
+/**
+ * system_parse_line:
+ * @sys: initialized equation system.
+ * @line: command line.
+ * @pdinfo: dataset information.
+ * 
+ * Modifies @sys according to the command supplied in @line,
+ * which must start with "identity" (and supply an identity
+ * to be added to the system, or "endog" (and supply a list
+ * of endogenous variables), or "instr" (and supply a list of
+ * instrumental variables).
+ * 
+ * Returns: 0 on success, non-zero on failure, in which case
+ * @sys is destroyed.
+ */
+
 int 
 system_parse_line (gretl_equation_system *sys, const char *line,
 		   const DATAINFO *pdinfo)
@@ -1803,3 +1874,104 @@ system_set_restriction_matrices (gretl_equation_system *sys,
 
     sys->flags |= GRETL_SYS_RESTRICT;
 }
+
+double *
+gretl_equation_system_get_series (const gretl_equation_system *sys, 
+				  const DATAINFO *pdinfo,
+				  int idx, const char *key, int *err)
+{
+    double *x = NULL;
+    const char *msel;
+    int t, col = 0;
+
+    if (sys == NULL || idx != M_UHAT) { /* FIXME generalize this */
+	*err = E_BADSTAT;
+	return NULL;
+    }
+
+    msel = strchr(key, '[');
+    if (msel == NULL || sscanf(msel, "[,%d]", &col) != 1) {
+	*err = E_PARSE;
+    } else if (col <= 0 || col > sys->n_equations) {
+	*err = E_DATA;
+    }
+
+    if (!*err) {
+	x = malloc(pdinfo->n * sizeof *x);
+	if (x == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	if (sys->uhat == NULL) {
+	    *err = E_DATA;
+	    free(x);
+	    x = NULL;
+	} else {
+	    int s = 0;
+
+	    col--;
+	    for (t=0; t<pdinfo->n; t++) {
+		if (t < sys->t1 || t > sys->t2) {
+		    x[t] = NADBL;
+		} else {
+		    x[t] = gretl_matrix_get(sys->uhat, s++, col);
+		}
+	    }
+	}
+    }
+
+    return x;
+}
+
+gretl_matrix *
+gretl_equation_system_get_matrix (const gretl_equation_system *sys, int idx, 
+				  int *err)
+{
+    gretl_matrix *M = NULL;
+
+    if (sys == NULL) {
+	*err = E_BADSTAT;
+	return NULL;
+    }
+
+    switch (idx) {  
+    case M_UHAT:
+	M = gretl_matrix_copy(sys->uhat);
+	break;
+    case M_VCV:
+	M = gretl_matrix_copy(sys->sigma);
+	break;
+    default:
+	*err = E_BADSTAT;
+	break;
+    }
+
+    if (M == NULL && !*err) {
+	*err = E_ALLOC;
+    }
+
+    return M;
+}
+
+int highest_numbered_var_in_system (const gretl_equation_system *sys, 
+				    const DATAINFO *pdinfo)
+{
+    int i, j, v, vmax = 0;
+
+    for (i=0; i<sys->n_equations; i++) {
+	for (j=1; j<=sys->lists[i][0]; j++) {
+	    v = sys->lists[i][j];
+	    if (v == LISTSEP || v >= pdinfo->v) {
+		/* temporary variables, already gone? */
+		continue;
+	    }
+	    if (v > vmax) {
+		vmax = v;
+	    }
+	}
+    }
+
+    return vmax;
+}
