@@ -33,7 +33,7 @@ typedef enum {
     SYS_MAX
 } gretl_system_methods;
 
-gretl_equation_system *system_start (const char *line);
+gretl_equation_system *system_start (const char *line, gretlopt opt);
 
 char *get_system_name_from_line (const char *s);
 
@@ -156,4 +156,16 @@ system_normality_test (const gretl_equation_system *sys, PRN *prn);
 int gretl_system_add_resids_to_dataset (const char *sysname, int eqnum,
 					double ***pZ, DATAINFO *pdinfo);
 
+double *
+gretl_equation_system_get_series (const gretl_equation_system *sys, 
+				  const DATAINFO *pdinfo,
+				  int idx, const char *key, int *err);
+
+gretl_matrix *
+gretl_equation_system_get_matrix (const gretl_equation_system *sys, int idx, 
+				  int *err);
+
+int highest_numbered_var_in_system (const gretl_equation_system *sys, 
+				    const DATAINFO *pdinfo);
+
 #endif /* GRETL_EQUATION_SYSTEM_H */
