@@ -130,10 +130,10 @@ make_transform_label (char *label, const char *parent,
 int is_standard_lag (int v, const DATAINFO *pdinfo)
 {
     const char *test = VARLABEL(pdinfo, v);
-    char pm, vname[13];
+    char pm, vname[VNAMELEN];
     int lag, ret = 0;
 
-    if (sscanf(test, "= %12[^(](t %c %d)", vname, &pm, &lag) == 3) {
+    if (sscanf(test, "= %15[^(](t %c %d)", vname, &pm, &lag) == 3) {
 	ret = 1;
     }
 
@@ -201,9 +201,9 @@ void gretl_transforms_cleanup (void)
 static int get_lag (int v, int lag, double *lagvec, 
 		    const double **Z, const DATAINFO *pdinfo)
 {
-    int t, t1, lt;
-
-    t1 = (lag > pdinfo->t1)? lag : pdinfo->t1;
+    int t1 = (lag > 0)? lag : 0;
+    int t2 = pdinfo->n - 1;
+    int t, lt;
 
     for (t=0; t<pdinfo->n; t++) {
 	lagvec[t] = NADBL;
@@ -211,7 +211,7 @@ static int get_lag (int v, int lag, double *lagvec,
 
     if (pdinfo->structure == STACKED_CROSS_SECTION) {
 	/* needs rather special handling */
-	for (t=t1; t<=pdinfo->t2; t++) { 
+	for (t=t1; t<=t2; t++) { 
 	    lt = t - lag * pdinfo->pd;
 	    if (lt < 0 || lt >= pdinfo->n) {
 		continue;
@@ -219,7 +219,7 @@ static int get_lag (int v, int lag, double *lagvec,
 	    lagvec[t] = Z[v][lt];
 	}
     } else if (dated_daily_data(pdinfo)) {
-	for (t=t1; t<=pdinfo->t2; t++) {
+	for (t=t1; t<=t2; t++) {
 	    lt = t - lag;
 	    while (lt >= 0 && na(Z[v][lt])) {
 		lt--;
@@ -228,7 +228,7 @@ static int get_lag (int v, int lag, double *lagvec,
 	}
     } else { 
 	/* the "standard" time-series case */
-	for (t=t1; t<=pdinfo->t2; t++) {
+	for (t=t1; t<=t2; t++) {
 	    lt = t - lag;
 	    if (lt < 0 || lt >= pdinfo->n) {
 		continue;
@@ -242,7 +242,7 @@ static int get_lag (int v, int lag, double *lagvec,
 	char *p, obs[OBSLEN];
 	int j;
 
-	for (t=t1; t<=pdinfo->t2; t++) {
+	for (t=t1; t<=t2; t++) {
 	    ntodate(obs, t, pdinfo);
 	    p = strchr(obs, ':');
 	    j = atoi(p + 1);
@@ -498,7 +498,7 @@ int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
     } else if (lag == 0) {
 	lno = v;
     } else {
-	lno = get_transform(LAGS, v, lag, pZ, pdinfo, 8);
+	lno = get_transform(LAGS, v, lag, pZ, pdinfo, VNAMELEN - 3);
     }
 
     return lno;
@@ -518,7 +518,7 @@ int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
 
 int loggenr (int v, double ***pZ, DATAINFO *pdinfo)
 {
-    return get_transform(LOGS, v, 0, pZ, pdinfo, 8);
+    return get_transform(LOGS, v, 0, pZ, pdinfo, VNAMELEN - 3);
 }
 
 /**
@@ -550,7 +550,7 @@ int diffgenr (int v, int ci, double ***pZ, DATAINFO *pdinfo)
 	return -1;
     }
 
-    return get_transform(ci, v, 0, pZ, pdinfo, 8);
+    return get_transform(ci, v, 0, pZ, pdinfo, VNAMELEN - 3);
 }
 
 /**
@@ -575,13 +575,13 @@ int xpxgenr (int vi, int vj, double ***pZ, DATAINFO *pdinfo)
 	}
     }
 
-    return get_transform(SQUARE, vi, vj, pZ, pdinfo, 8);
+    return get_transform(SQUARE, vi, vj, pZ, pdinfo, VNAMELEN - 3);
 }
 
 static int 
 get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
 {
-    int width = 8 - trim;
+    int width = VNAMELEN - 3 - trim;
     int len, maxlen = 0;
     int i, j;
 
@@ -593,8 +593,8 @@ get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
     }
 
     if (maxlen <= width) {
-	/* no problem: generated names will fit in 8 chars */
-	return 8;
+	/* no problem: generated names will fit in VNAMELEN - 3 chars */
+	return VNAMELEN - 3;
     }
 
     for (len=width; len<=maxlen; len++) {
@@ -620,10 +620,10 @@ get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
 
     len += trim;
 
-    if (len < 8) {
-	len = 8;
-    } else if (len > VNAMELEN) {
-	len = VNAMELEN;
+    if (len < VNAMELEN - 3) {
+	len = VNAMELEN - 3;
+    } else if (len > VNAMELEN - 1) {
+	len = VNAMELEN - 1;
     }
 
     return len;
@@ -671,45 +671,86 @@ int list_loggenr (int *list, double ***pZ, DATAINFO *pdinfo)
     return (n_ok > 0)? 0 : E_LOGS;
 }
 
-/* used in var.c */
-
-int 
-real_list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo,
-		   int maxlag, int **lagnums)
+static int *make_lags_list (int *list, int order, DATAINFO *pdinfo)
 {
-    int lagnum, l, i, v;
-    int startlen;
-    int *record = NULL;
+    int i, v, nl = 0;
 
-    startlen = get_starting_length(list, pdinfo, (maxlag > 9)? 3 : 2);
-    
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
+	if (v > 0 && pdinfo->vector[v]) {
+	    nl += order;
+	}
+    }
+
+    return gretl_list_new(nl);
+}
+
+/**
+ * list_laggenr:
+ * @plist: pointer to list of variables to process.  On exit
+ * the list holds the ID numbers of the lag variables.
+ * @order: number of lags to generate (or 0 for automatic).
+ * @list: list of variables to process.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ *
+ * Generates and adds to the data set @order lagged values of the 
+ * variables given in @list.
+ *
+ * Returns: 0 on successful completion, 1 on error.
+ */
+
+int list_laggenr (int **plist, int order, double ***pZ, DATAINFO *pdinfo)
+{
+    int *list = *plist;
+    int *laglist = NULL;
+    int l, i, j;
+    int startlen;
+
+    if (order < 0) {
+	return E_DATA;
+    }
+
+    if (order == 0) {
+	order = default_lag_order(pdinfo);
+    } 
+
+    laglist = make_lags_list(list, order, pdinfo);
+    if (laglist == NULL) {
+	return E_ALLOC;
+    }
+
+    startlen = get_starting_length(list, pdinfo, (order > 9)? 3 : 2);
+
+    j = 1;
+    
+    for (i=1; i<=list[0]; i++) {
+	int lv, v = list[i];
+
 	if (v == 0 || !pdinfo->vector[v]) {
 	    continue;
 	}
-	if (lagnums != NULL) {
-	    record = lagnums[i-1];
-	}
-	for (l=1; l<=maxlag; l++) {
-	    lagnum = get_transform(LAGS, v, l, pZ, pdinfo, startlen);
+
+	for (l=1; l<=order; l++) {
+	    lv = get_transform(LAGS, v, l, pZ, pdinfo, startlen);
 #if TRDEBUG > 1
-	    fprintf(stderr, "base var '%s', lag %d: lagnum = %d\n",
-		    pdinfo->varname[v], l, lagnum);
+	    fprintf(stderr, "base var '%s', lag %d: lv = %d\n",
+		    pdinfo->varname[v], l, lv);
 #endif
-	    if (lagnum < 0) {
+	    if (lv < 0) {
 		return 1;
 	    }
 #if TRDEBUG > 1
 	    fprintf(stderr, "lag var name '%s', label '%s'\n",
-		    pdinfo->varname[lagnum], VARLABEL(pdinfo, lagnum));
+		    pdinfo->varname[lv], VARLABEL(pdinfo, lv));
 #endif
-	    if (record != NULL) {
-		record[l] = lagnum;
-	    }
+	    laglist[j++] = lv;
 	}
     }
 
+    free(*plist);
+    *plist = laglist;
+
     return 0;
 }
 
@@ -733,28 +774,6 @@ int default_lag_order (const DATAINFO *pdinfo)
     return order;
 }
 
-/**
- * list_laggenr:
- * @order: number of lags to generate (or 0 for automatic)
- * @list: list of variables to process.
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
- *
- * Generates and adds to the data set @order lagged values of the 
- * variables given in @list.
- *
- * Returns: 0 on successful completion, 1 on error.
- */
-
-int list_laggenr (int order, const int *list, double ***pZ, DATAINFO *pdinfo)
-{
-    if (order == 0) {
-	order = default_lag_order(pdinfo);
-    } 
-
-    return real_list_laggenr(list, pZ, pdinfo, order, NULL);  
-}
-
 /**
  * list_diffgenr:
  * @list: on entry, list of variables to process; on exit,
@@ -888,7 +907,7 @@ int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
     }
     
     if (dataset_add_series(1, pZ, pdinfo)) {
-	return TREND_FAILED;
+	return 0; /* error: valid value cannot == 0 */
     }
 
     for (t=0; t<pdinfo->n; t++) {
