@@ -20,8 +20,6 @@
 #ifndef TRANSFORMS_H
 #define TRANSFORMS_H
 
-#define TREND_FAILED 9999
-
 int default_lag_order (const DATAINFO *pdinfo);
 
 int is_standard_lag (int v, const DATAINFO *pdinfo);
@@ -36,7 +34,7 @@ int xpxgenr (int vi, int vj, double ***pZ, DATAINFO *pdinfo);
 
 int list_diffgenr (int *list, int ci, double ***pZ, DATAINFO *pdinfo);
 
-int list_laggenr (int order, const int *list, double ***pZ, DATAINFO *pdinfo);
+int list_laggenr (int **plist, int order, double ***pZ, DATAINFO *pdinfo);
 
 int list_loggenr (int *list, double ***pZ, DATAINFO *pdinfo);
 
