@@ -24,19 +24,6 @@
 #include "libset.h"
 #include "estim_private.h"
 
-static int translate_gretl_matrix_error (int gmerr)
-{
-    if (gmerr == GRETL_MATRIX_OK) {
-	return 0;
-    } else if (gmerr == GRETL_MATRIX_NOMEM) {
-	return E_ALLOC;
-    } else if (gmerr == GRETL_MATRIX_SINGULAR) {
-	return E_SINGULAR;
-    } else {
-	return E_UNSPEC;
-    }
-}
-
 static void tsls_omitzero (int *list, const double **Z, int t1, int t2)
 {
     int i, v;
@@ -67,14 +54,14 @@ void tsls_free_data (const MODEL *pmod)
 }
 
 static int 
-tsls_save_data (MODEL *pmod, const int *hatlist, const int *instlist,
+tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 		double **Z, DATAINFO *pdinfo)
 {
     double **X = NULL;
     char *endog = NULL;
     int i, j, v, err = 0;
-    size_t esize, Xsize;
-    size_t es_old, xs_old;
+    size_t Xsize, esize;
+    size_t xs_old, es_old;
     int recycle_X = 0;
     int recycle_e = 0;
 
@@ -84,32 +71,41 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *instlist,
     /* re-use old pointers if applicable */
 
     X = gretl_model_get_data_and_size(pmod, "tslsX", &xs_old);
-    if (X == NULL) {
+    if (X != NULL) {
+	if (Xsize == xs_old) {
+	    recycle_X = 1;
+	} else {
+	    tsls_free_data(pmod);
+	    gretl_model_detach_data_item(pmod, "tslsX");
+	    free(X);
+	    X = NULL;
+	} 
+    }
+
+    if (!recycle_X && Xsize > 0) {
 	X = malloc(Xsize);
-    } else if (Xsize != xs_old) {
-	tsls_free_data(pmod);
-	gretl_model_detach_data_item(pmod, "tslsX");
-	free(X);
-	X = malloc(Xsize);
-    } else {
-	recycle_X = 1;
+	if (X == NULL) {
+	    return E_ALLOC;
+	}
     }
 
     endog = gretl_model_get_data_and_size(pmod, "endog", &es_old);
-    if (endog == NULL) {
-	endog = malloc(esize);
-    } else if (esize != es_old) {
-	gretl_model_detach_data_item(pmod, "endog");
-	free(endog);
-	endog = malloc(esize);
-    } else {
-	recycle_e = 1;
+    if (endog != NULL) {
+	if (esize == es_old) {
+	    recycle_e = 1;
+	} else {
+	    gretl_model_detach_data_item(pmod, "endog");
+	    free(endog);
+	    endog = NULL;
+	}
     }
 
-    if (X == NULL || endog == NULL) {
-	free(X);
-	free(endog);
-	return E_ALLOC;
+    if (!recycle_e && esize > 0) {
+	endog = malloc(esize);
+	if (endog == NULL) {
+	    free(X);
+	    return E_ALLOC;
+	}
     }
 
     /* steal the appropriate columns from Z */
@@ -122,8 +118,8 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *instlist,
     for (i=0; i<pmod->ncoeff; i++) {
 	v = pmod->list[i+2];
 	endog[i] = 1;
-	for (j=1; j<=instlist[0]; j++) {
-	    if (instlist[j] == v) {
+	for (j=1; j<=exolist[0]; j++) {
+	    if (exolist[j] == v) {
 		endog[i] = 0;
 		break;
 	    }
@@ -143,17 +139,20 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *instlist,
 
 double *tsls_get_Xi (const MODEL *pmod, double **Z, int i)
 {
-    const char *endog;
+    const char *endog = gretl_model_get_data(pmod, "endog");
     double **X;
     double *ret;
 
-    endog = gretl_model_get_data(pmod, "endog");
-    X = gretl_model_get_data(pmod, "tslsX");
+    if (endog == NULL) {
+	return NULL;
+    }
 
-    if (endog == NULL || X == NULL) return NULL;
+    X = gretl_model_get_data(pmod, "tslsX");
 
     if (!endog[i]) {
 	ret = Z[pmod->list[i+2]];
+    } else if (X == NULL) {
+	ret = NULL;
     } else {
 	int j, k = 0;
 
@@ -365,11 +364,11 @@ tsls_list_add (const int *orig, const int *add, gretlopt opt, int *err)
 /*
   tsls_make_hatlist: determines which variables in reglist, when
   checked against the predetermined and exogenous vars in instlist,
-  need to be instrumented, and populates replist accordingly
+  need to be instrumented, and populates hatlist accordingly
 */
 
 static int 
-tsls_make_hatlist (const int *reglist, int *instlist, int *replist)
+tsls_make_hatlist (const int *reglist, int *instlist, int *hatlist)
 {
     int i, j, k = 0;
     int endog, addconst = 0;
@@ -382,15 +381,18 @@ tsls_make_hatlist (const int *reglist, int *instlist, int *replist)
 		break;
 	    }
 	}
-	if (reglist[i] == 0 && endog) {
-	    /* const is in reglist but not instlist: needs fixing */
-	    addconst = 1;
-	} else if (endog) {
-	    replist[++k] = reglist[i];
+	if (endog) {
+	    if (reglist[i] == 0) {
+		/* found const in reglist but not instlist: needs fixing
+		   FIXME non-official const? */
+		addconst = 1;
+	    } else {
+		hatlist[++k] = reglist[i];
+	    }
 	} 
     }
 
-    replist[0] = k;
+    hatlist[0] = k;
 
     if (addconst) {
 	/* add constant to list of instruments */
@@ -444,7 +446,7 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 	OT_list[i] = instlist[i-1];
     }
 
-    smod = lsq(OT_list, pZ, pdinfo, OLS, OPT_A, 0.0);
+    smod = lsq(OT_list, pZ, pdinfo, OLS, OPT_A);
     if (smod.errcode) {
 	err = smod.errcode;
     } else {
@@ -468,7 +470,7 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 }
 
 static int 
-tsls_hausman_test (MODEL *tsls_model, int *reglist, int *hatlist,
+tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 		   double ***pZ, DATAINFO *pdinfo)
 {
     double RRSS;
@@ -477,7 +479,7 @@ tsls_hausman_test (MODEL *tsls_model, int *reglist, int *hatlist,
     MODEL hmod;
     int *HT_list = NULL;
 
-    hmod = lsq(reglist, pZ, pdinfo, OLS, OPT_A, 0.0);
+    hmod = lsq(reglist, pZ, pdinfo, OLS, OPT_A);
     if (hmod.errcode) {
 	err = hmod.errcode;
 	goto bailout;
@@ -488,11 +490,11 @@ tsls_hausman_test (MODEL *tsls_model, int *reglist, int *hatlist,
 
     HT_list = gretl_list_add(reglist, hatlist, &err);
     if (err) {
-	fprintf(stderr, "gretl_add_list: %d\n", err);
+	fprintf(stderr, "tsls_hausman_test: gretl_list_add: err = %d\n", err);
 	goto bailout;
     } 
 
-    hmod = lsq(HT_list, pZ, pdinfo, OLS, OPT_A, 0.0);
+    hmod = lsq(HT_list, pZ, pdinfo, OLS, OPT_A);
 
     if (hmod.errcode) {
 	err = hmod.errcode;
@@ -511,7 +513,7 @@ tsls_hausman_test (MODEL *tsls_model, int *reglist, int *hatlist,
 	    model_test_set_dfn(test, df);
 	    model_test_set_value(test, HTest);
 	    model_test_set_pvalue(test, chisq(HTest, df));
-	    maybe_add_test_to_model(tsls_model, test);
+	    maybe_add_test_to_model(tmod, test);
 	}
     }
 
@@ -526,10 +528,9 @@ tsls_hausman_test (MODEL *tsls_model, int *reglist, int *hatlist,
 /* form matrix of instruments and perform QR decomposition
    of this matrix; return Q */
 
-static gretl_matrix * 
-tsls_Q (int *instlist, int *reglist, int **pdlist,
-	const double **Z, int t1, int t2, char **pmask,
-	int *err)
+static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
+			     const double **Z, int t1, int t2, char **pmask,
+			     int *err)
 {
     gretl_matrix *Q = NULL;
     gretl_matrix *R = NULL;
@@ -537,7 +538,7 @@ tsls_Q (int *instlist, int *reglist, int **pdlist,
     int rank, ndrop = 0;
     int *droplist = NULL;
     double test;
-    int i, k, v, pos;
+    int i, k, v;
 
     Q = gretl_matrix_data_subset(instlist, Z, t1, t2, &mask);
     if (Q == NULL) {
@@ -555,21 +556,14 @@ tsls_Q (int *instlist, int *reglist, int **pdlist,
 
     *err = gretl_matrix_QR_decomp(Q, R);
     if (*err) {
-	*err = translate_gretl_matrix_error(*err);
 	goto bailout;
     }
 
     rank = gretl_matrix_QR_rank(R, NULL, err);
     if (*err) {
-	*err = translate_gretl_matrix_error(*err);
 	goto bailout;
     } else if (rank < k) {
-	fprintf(stderr, "k = %d, rank = %d\n", k, rank);
-	if (pdlist == NULL) {
-	    /* disallowing dropping of redundant vars */
-	    *err = E_SINGULAR;
-	    goto bailout;
-	}
+	fprintf(stderr, "tsls_Q: k = %d, rank = %d\n", k, rank);
 	ndrop = k - rank;
     }
 
@@ -587,11 +581,8 @@ tsls_Q (int *instlist, int *reglist, int **pdlist,
 		    droplist[0] += 1;
 		    droplist[droplist[0]] = v;
 		}	    
-		fprintf(stderr, "Dropping redundant instrument %d\n", v);
+		fprintf(stderr, "tsls_Q: dropping redundant instrument %d\n", v);
 		gretl_list_delete_at_pos(instlist, i+1);
-		if ((pos = in_gretl_list(reglist, v))) {
-		    gretl_list_delete_at_pos(reglist, pos);
-		}
 	    }
 	}
 
@@ -601,9 +592,6 @@ tsls_Q (int *instlist, int *reglist, int **pdlist,
 	Q = gretl_matrix_data_subset(instlist, Z, t1, t2, &mask);
 	R = gretl_matrix_reuse(R, k, k);
 	*err = gretl_matrix_QR_decomp(Q, R);
-	if (*err) {
-	    *err = translate_gretl_matrix_error(*err);
-	}
     }
 
  bailout:
@@ -612,13 +600,12 @@ tsls_Q (int *instlist, int *reglist, int **pdlist,
 
     if (*err) {
 	free(mask);
+	free(droplist);
 	gretl_matrix_free(Q);
 	Q = NULL;
     } else {
 	*pmask = mask;
-	if (pdlist != NULL) {
-	    *pdlist = droplist;
-	}
+	*pdlist = droplist;
     }
 
     return Q;
@@ -718,7 +705,7 @@ static void tsls_extra_stats (MODEL *pmod, const double **Z)
 
     pmod->fstt = pmod->rsq * pmod->dfd / (pmod->dfn * r);
 
-    ls_aic_bic(pmod);
+    ls_criteria(pmod);
 
     if (pmod->missmask == NULL) {
 	/* no missing obs within sample range */
@@ -769,6 +756,26 @@ static void replace_list_element (int *list, int targ, int repl)
     }
 }
 
+static int 
+reglist_remove_redundant_vars (const MODEL *tmod, int *s2list, int *reglist)
+{
+    int *dlist = gretl_model_get_data(tmod, "droplist");
+    int i, pos;
+
+    if (dlist == NULL) {
+	return 1;
+    }
+
+    for (i=1; i<=dlist[0]; i++) {
+	pos = in_gretl_list(s2list, dlist[i]);
+	if (pos > 1) {
+	    gretl_list_delete_at_pos(reglist, pos);
+	}
+    }
+
+    return 0;
+}
+
 /**
  * tsls_func:
  * @list: dependent variable plus list of regressors.
@@ -795,6 +802,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     int orig_t1 = pdinfo->t1, orig_t2 = pdinfo->t2;
     int *reglist = NULL, *instlist = NULL;
     int *hatlist = NULL, *s2list = NULL;
+    int *exolist = NULL;
     int *droplist = NULL;
     int pos, orig_nvar = pdinfo->v;
     int rlen, ninst;
@@ -859,13 +867,23 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     instlist[0] = ninst;
     for (i=1; i<=instlist[0]; i++) {
 	instlist[i] = list[i + pos];
+	if (instlist[i] == list[1]) {
+	    strcpy(gretl_errmsg, "You can't use the dependent variable as an instrument");
+	    tsls.errcode = E_DATA;
+	    goto bailout;
+	}
     }	
 
+    /* in case we drop any instruments as redundant */
+    if (ci == SYSTEM) {
+	exolist = gretl_list_copy(instlist);
+    }
+
     /* drop any vars that are all zero, and reshuffle the constant
        into first position among the independent vars 
     */
     tsls_omitzero(reglist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
-    rearrange_list(reglist);
+    reglist_check_for_const(reglist, (const double **) *pZ, pdinfo);
     tsls_omitzero(instlist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
 
     /* determine the list of variables (hatlist) for which we need to
@@ -873,7 +891,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     */
     tsls_make_hatlist(reglist, instlist, hatlist);
 
-    Q = tsls_Q(instlist, reglist, (ci == TSLS)? &droplist : NULL, 
+    Q = tsls_Q(instlist, reglist, &droplist,
 	       (const double **) *pZ, pdinfo->t1, pdinfo->t2,
 	       &missmask, &tsls.errcode);
     if (tsls.errcode) {
@@ -889,7 +907,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
 		"varlist1."), -OverIdRank);
 	tsls.errcode = E_UNSPEC; 
 	goto bailout;
-    }	
+    }
 
     /* hatlist[0] holds the number of fitted vars to create */
     if (dataset_add_series(hatlist[0], pZ, pdinfo)) {
@@ -934,11 +952,17 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* second-stage regression */
-    tsls = lsq(s2list, pZ, pdinfo, OLS, (ci == TSLS)? OPT_NONE : OPT_Z, 0.0);
+    tsls = lsq(s2list, pZ, pdinfo, OLS, (ci == TSLS)? OPT_NONE : OPT_Z);
     if (tsls.errcode) {
 	goto bailout;
     }
 
+    if (tsls.list[0] < s2list[0]) {
+	/* collinear regressors dropped? If so, adjustments are needed */
+	OverIdRank += s2list[0] - tsls.list[0];
+	reglist_remove_redundant_vars(&tsls, s2list, reglist);
+    }
+
     /* special: we need to use the original RHS vars to compute
        residuals and associated statistics */
     tsls_residuals(&tsls, reglist, (const double **) *pZ, opt);
@@ -972,7 +996,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     tsls_recreate_full_list(&tsls, reglist, instlist);
 
     if (droplist != NULL) {
-	gretl_model_set_list_as_data(&tsls, "tsls_droplist", droplist); 
+	gretl_model_set_list_as_data(&tsls, "inst_droplist", droplist); 
     }
 
  bailout:
@@ -982,7 +1006,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
 
     /* save first-stage fitted values, if wanted */
     if (ci == SYSTEM && tsls.errcode == 0) {
-	tsls_save_data(&tsls, hatlist, instlist, *pZ, pdinfo);
+	tsls_save_data(&tsls, hatlist, exolist, *pZ, pdinfo);
     }
 
     /* delete first-stage fitted values from dataset */
@@ -991,6 +1015,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     free(hatlist);
     free(reglist); 
     free(instlist);
+    free(exolist);
     free(s2list);
 
     if (tsls.errcode) {
