@@ -28,11 +28,6 @@
 #define VAR_DEBUG 0
 #define BDEBUG    0  /* for debugging bootstrap IRFs */
 
-/* in transforms.c */
-extern int 
-real_list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo,
-		   int maxlag, int **lagnums);
-
 static gretl_matrix *irf_bootstrap (const GRETL_VAR *var, 
 				    int targ, int shock, int periods,
 				    const double **Z, 
@@ -47,6 +42,38 @@ struct var_lists {
     int **lagvlist;
 };
 
+static int VAR_list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo,
+			     int order, int **lagnums)
+{
+    int *tmplist = gretl_list_copy(list);
+    int i, j, k;
+    int err = 0;
+    
+    if (tmplist == NULL) {
+	return E_ALLOC;
+    }
+
+    err = list_laggenr(&tmplist, order, pZ, pdinfo);
+
+    if (!err && tmplist[0] < list[0] * order) {
+	/* we didn't get all the lags we wanted */
+	err = E_DATA;
+    }
+
+    if (!err) {
+	k = 1;
+	for (i=0; i<list[0]; i++) {
+	    for (j=1; j<=order; j++) {
+		lagnums[i][j] = tmplist[k++];
+	    }
+	}
+    }
+
+    free(tmplist);
+    
+    return err;
+}
+
 static int gretl_VAR_add_models (GRETL_VAR *var)
 {
     int n = var->neqns;
@@ -140,10 +167,11 @@ static void gretl_VAR_zero (GRETL_VAR *var)
 
     var->models = NULL;
     var->Fvals = NULL;
+    var->Ivals = NULL;
     var->name = NULL;
 
     var->ll = var->ldet = NADBL;
-    var->AIC = var->BIC = NADBL;
+    var->AIC = var->BIC = var->HQC = NADBL;
     var->LR = NADBL;
 }
 
@@ -190,6 +218,13 @@ static GRETL_VAR *gretl_VAR_new (int ci, int neqns, int order)
 	}
     }
 
+    if (!err) {
+	var->Ivals = malloc(3  * sizeof *var->Ivals);
+	if (var->Ivals == NULL) {
+	    err = 1;
+	}
+    }
+
     if (err) {
 	gretl_VAR_free(var);
 	var = NULL;
@@ -244,6 +279,7 @@ void gretl_VAR_free (GRETL_VAR *var)
     gretl_matrix_free(var->F);
 
     free(var->Fvals);
+    free(var->Ivals);
     free(var->name);
 
     if (var->models != NULL) {
@@ -297,14 +333,14 @@ static int add_VAR_fcast_variance (GRETL_VAR *var, gretl_matrix *F,
 }
 
 static int
-gretl_VAR_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
+gretl_VAR_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 			const double **Z, const DATAINFO *pdinfo, 
 			gretlopt opt)
 {
     const MODEL *pmod;
     gretl_matrix *F;
     double fti, xti;
-    int nf = t2 - t1 + 1;
+    int nf = t2 - t0 + 1;
     int staticfc = (opt & OPT_S);
     int i, j, k, s, t;
     int ns, lag, vj, m;
@@ -325,10 +361,9 @@ gretl_VAR_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
 
     ns = var->order * var->neqns;
 
-    for (t=t1; t<=t2; t++) {
+    for (t=t0, s=0; t<=t2; t++, s++) {
 	int miss = 0;
 
-	s = t - t1;
 	for (i=0; i<var->neqns; i++) {
 	    pmod = var->models[i];
 	    fti = 0.0;
@@ -338,7 +373,7 @@ gretl_VAR_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
 		vj = pmod->list[j + 2];
 		if (j < ns + pmod->ifc && vj > 0) {
 		    /* stochastic var */
-		    if (s < pre_obs || staticfc || s - lag < 0) {
+		    if (t < t1 || staticfc || s - lag < 0) {
 			/* pre-forecast value */
 			m = (j - pmod->ifc) / var->order;
 			vj = var->models[m]->list[1];
@@ -378,10 +413,10 @@ gretl_VAR_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
 
     /* now get variances, if not static */
     if (!staticfc) {
-	add_VAR_fcast_variance(var, F, nf, pre_obs);
+	add_VAR_fcast_variance(var, F, nf, t1 - t0);
     }
 
-    gretl_matrix_set_int(F, t1);
+    gretl_matrix_set_int(F, t0);
     var->F = F;
 
 #if 0
@@ -392,7 +427,7 @@ gretl_VAR_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
 }
 
 static int
-gretl_VECM_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
+gretl_VECM_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 			 const double **Z, DATAINFO *pdinfo, 
 			 gretlopt opt)
 {
@@ -402,7 +437,7 @@ gretl_VECM_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
     int order = var->order + 1;
     int nexo = (var->jinfo->exolist != NULL)? var->jinfo->exolist[0] : 0;
     int nseas = var->jinfo->seasonals;
-    int nf = t2 - t1 + 1;
+    int nf = t2 - t0 + 1;
     int staticfc = (opt & OPT_S);
     int i, j, k, vj, s, t;
     int fcols, d0 = 0;
@@ -429,9 +464,7 @@ gretl_VECM_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
 
     gretl_matrix_zero(F);
 
-    for (t=t1; t<=t2; t++) {
-
-	s = t - t1;
+    for (t=t0, s=0; t<=t2; t++, s++) {
 
 	for (i=0; i<var->neqns; i++) {
 	    double y, bij, fti = 0.0;
@@ -453,7 +486,7 @@ gretl_VECM_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
 		    }			
 		    bij = gretl_matrix_get(B, i, col++);
 		    ft = s - k - 1;
-		    if (s >= pre_obs && ft >= 0 && !staticfc) {
+		    if (t >= t1 && ft >= 0 && !staticfc) {
 			/* use prior forecast if available */
 			y = gretl_matrix_get(F, ft, j);
 		    } else {
@@ -515,10 +548,10 @@ gretl_VECM_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
 
     /* now get variances, if not static */
     if (!staticfc) {
-	add_VAR_fcast_variance(var, F, nf, pre_obs);
+	add_VAR_fcast_variance(var, F, nf, t1 - t0);
     }
 
-    gretl_matrix_set_int(F, t1);
+    gretl_matrix_set_int(F, t0);
     var->F = F;
 
 #if 0
@@ -529,12 +562,12 @@ gretl_VECM_add_forecast (GRETL_VAR *var, int t1, int t2, int pre_obs,
 }
 
 const gretl_matrix *
-gretl_VAR_get_forecast_matrix (GRETL_VAR *var, int t1, int t2, int pre_obs,
+gretl_VAR_get_forecast_matrix (GRETL_VAR *var, int t0, int t1, int t2,
 			       const double **Z, DATAINFO *pdinfo, 
 			       gretlopt opt)
 {
     if (var->F != NULL) {
-	int ncols, nf = t2 - t1 + 1;
+	int ncols, nf = t2 - t0 + 1;
 	int ft1 = gretl_matrix_get_int(var->F);
 
 	ncols = (opt & OPT_S)? var->neqns: 2 * var->neqns;
@@ -550,9 +583,9 @@ gretl_VAR_get_forecast_matrix (GRETL_VAR *var, int t1, int t2, int pre_obs,
 
     if (var->F == NULL) {
 	if (var->ecm) {
-	    gretl_VECM_add_forecast(var, t1, t2, pre_obs, Z, pdinfo, opt);
+	    gretl_VECM_add_forecast(var, t0, t1, t2, Z, pdinfo, opt);
 	} else {
-	    gretl_VAR_add_forecast(var, t1, t2, pre_obs, Z, pdinfo, opt);
+	    gretl_VAR_add_forecast(var, t0, t1, t2, Z, pdinfo, opt);
 	}
     }
 
@@ -638,10 +671,10 @@ int gretl_VAR_do_error_decomp (const gretl_matrix *S,
 	err = gretl_matrix_cholesky_decomp(tmp);
     }
 
-    /* write the decomposition into the C matrix */
+    /* write the decomposition (lower triangle) into the C matrix */
     if (!err) {
 	for (i=0; i<g; i++) {
-	    for (j=0; j<g; j++) {
+	    for (j=0; j<=i; j++) {
 		x = gretl_matrix_get(tmp, i, j);
 		gretl_matrix_set(C, i, j, x);
 	    }
@@ -1150,14 +1183,14 @@ static int organize_var_lists (const int *list, const double **Z,
 
 /* Compose a VAR regression list: it may be complete, or one variable
    may be omitted (to run an F-test), or the order may be one less
-   than the full VAR order (again, for an F-test).  If test = 1, also
-   compose a list containing the maximum lag of each stochastic
+   than the full VAR order (again, for an F-test).  If misstest = 1,
+   also compose a list containing the maximum lag of each stochastic
    variable, for use in testing for missing values.
 */
 
 static int
 compose_varlist (struct var_lists *vl, int depvar, int order, int omit, 
-		 int test, const DATAINFO *pdinfo)
+		 int misstest, const DATAINFO *pdinfo)
 {
     int l0 = 1 + vl->detvars[0] + order * vl->stochvars[0];
     int i, j, pos;
@@ -1184,7 +1217,7 @@ compose_varlist (struct var_lists *vl, int depvar, int order, int omit,
 	}
     }
 
-    /* append the deterministic vars */
+    /* append the exogenous vars */
     for (i=1; i<=vl->detvars[0]; i++) {
 	vl->reglist[pos++] = vl->detvars[i];
     }
@@ -1193,8 +1226,8 @@ compose_varlist (struct var_lists *vl, int depvar, int order, int omit,
     printlist(vl->reglist, "composed VAR list");
 #endif
 
-    if (test) {
-	/* now build the test list (to screen missing values) */
+    if (misstest) {
+	/* build a test list to screen missing values */
 	pos = 1;
 	for (i=1; i<=vl->stochvars[0]; i++) {
 	    vl->testlist[pos++] = vl->stochvars[i];
@@ -1279,21 +1312,19 @@ static int gretl_VAR_add_roots (GRETL_VAR *var)
 	}
     }
 
-    /* save eigenvalues of companion form matrix in polar form */
+    /* save eigenvalues of companion form matrix */
     if (!err) {
-        eigA = gretl_general_matrix_eigenvals(CompForm, NULL);
-	if (eigA == NULL) {
-	    err = E_ALLOC;
-	} else {
+        eigA = gretl_general_matrix_eigenvals(CompForm, 0, &err);
+	if (!err) {
 	    for (i=0; i<np; i++) {
 		x = eigA[i];
-		y = eigA[np+i];
-		gretl_matrix_set(var->lambda, i, 0, atan2(y, x));
-		gretl_matrix_set(var->lambda, i, 1, sqrt(x * x + y * y));
+		y = eigA[np + i];
+		gretl_matrix_set(var->lambda, i, 0, x);
+		gretl_matrix_set(var->lambda, i, 1, y);
 	    }
 #if 0
 	    gretl_matrix_print(var->A, "Companion form matrix");
-	    gretl_matrix_print(var->lambda, "Eigenvalues in polar form");
+	    gretl_matrix_print(var->lambda, "Eigenvalues");
 #endif
 	}
     }
@@ -1319,36 +1350,54 @@ const gretl_matrix *gretl_VAR_get_roots (GRETL_VAR *var)
     return var->lambda;
 }
 
-static int VAR_LR_lag_test (GRETL_VAR *var)
+static double gretl_VAR_ldet (GRETL_VAR *var, int *err)
 {
     gretl_matrix *S = NULL;
     double ldet = NADBL;
-    int err = 0;
 
     S = gretl_matrix_alloc(var->neqns, var->neqns);
-    if (S == NULL) {
-	err = 1;
-    }
 
-    if (!err) {
+    if (S == NULL) {
+	*err = E_ALLOC;
+    } else {
 	gretl_matrix_multiply_mod(var->F, GRETL_MOD_TRANSPOSE,
 				  var->F, GRETL_MOD_NONE,
 				  S);
 	gretl_matrix_divide_by_scalar(S, var->T);
-    }
-
-    if (!err) {
 	ldet = gretl_vcv_log_determinant(S);
-	if (na(var->ldet)) {
-	    err = 1;
+	if (na(ldet)) {
+	    *err = 1;
 	}
-    }    
-
-    if (!err) {
-	var->LR = var->T * (ldet - var->ldet);
+	gretl_matrix_free(S);
     }
 
-    gretl_matrix_free(S);
+    return ldet;
+}
+
+static int VAR_LR_lag_test (GRETL_VAR *var)
+{
+    double ldet;
+    int err = 0;
+
+    ldet = gretl_VAR_ldet(var, &err);
+
+    if (!err) {
+	double ll, AIC, BIC, HQC;
+	int T = var->T;
+	int g = var->neqns;
+	int m = var->ncoeff - g;
+	int k = g * m;
+
+	var->LR = T * (ldet - var->ldet);
+
+	ll = -(g * T / 2.0) * (LN_2_PI + 1) - (T / 2.0) * ldet;
+	AIC = (-2.0 * ll + 2.0 * k) / T;
+	BIC = (-2.0 * ll + k * log(T)) / T;
+	HQC = (-2.0 * ll + 2.0 * k * log(log(T))) / T;
+	var->Ivals[0] = AIC;
+	var->Ivals[1] = BIC;
+	var->Ivals[2] = HQC;
+    }
 
     /* we're done with this set of residuals */
     gretl_matrix_free(var->F);
@@ -1357,8 +1406,166 @@ static int VAR_LR_lag_test (GRETL_VAR *var)
     return err;
 }
 
-/* per-equation F-tests for excluding variables and maximum
-   lag */
+static void gretl_VAR_print_lagsel (gretl_matrix *lltab,
+				    gretl_matrix *crittab,
+				    int *best_row,
+				    PRN *prn)
+{
+    int maxlag = gretl_matrix_rows(crittab);
+    double x;
+    int i, j;
+
+    pprintf(prn, _("VAR system, maximum lag order %d"), maxlag);
+    pputs(prn, "\n\n");
+
+    pputs(prn, ("The asterisks below indicate the best (that is, minimized) values\n"
+	  "of the respective information criteria, AIC = Akaike criterion,\n"
+	  "BIC = Schwartz Bayesian criterion and HQC = Hannan-Quinn criterion."));
+    pputs(prn, "\n\n");
+
+    pputs(prn, _("lags        loglik    p(LR)       AIC          BIC          HQC"));
+    pputs(prn, "\n\n");
+
+    for (i=0; i<maxlag; i++) {
+	pprintf(prn, "%4d", i + 1);
+	x = gretl_matrix_get(lltab, i, 0);
+	pprintf(prn, "%14.5f", x);
+	if (i > 0) {
+	    x = gretl_matrix_get(lltab, i, 1);
+	    pprintf(prn, "%9.5f", x);
+	} else {
+	    pputs(prn, "         ");
+	}
+	for (j=0; j<3; j++) {
+	    x = gretl_matrix_get(crittab, i, j);
+	    pprintf(prn, "%12.6f", x);
+	    if (i == best_row[j]) {
+		pputc(prn, '*');
+	    } else {
+		pputc(prn, ' ');
+	    }
+	}
+	pputc(prn, '\n');
+    }
+}
+
+static int 
+gretl_VAR_do_lagsel (GRETL_VAR *var, struct var_lists *vl,
+		     double ***pZ, DATAINFO *pdinfo, PRN *prn)
+{
+    gretl_matrix *crittab = NULL;
+    gretl_matrix *lltab = NULL;
+    MODEL testmod;
+
+    /* initialize the "best" at the longest lag */
+    double best[3] = { var->AIC, var->BIC, var->HQC };
+    int r = var->order - 1;
+    int best_row[3] = { r, r, r };
+
+    double crit[3];
+    double LRtest;
+    int T = var->T;
+    int g = var->neqns;
+
+    double ldet = NADBL;
+    int depvar;
+    int i, j, t;
+    int tm, m = 0;
+    int err = 0;
+
+    if (var->order < 2) {
+	return 0;
+    }
+
+    if (var->F != NULL) {
+	gretl_matrix_free(var->F);
+    }
+
+    var->F = gretl_matrix_alloc(var->T, var->neqns);
+    if (var->F == NULL) {
+	return E_ALLOC;
+    }
+
+    crittab = gretl_matrix_alloc(var->order, 3);
+    lltab = gretl_matrix_alloc(var->order, 2);
+    if (crittab == NULL || lltab == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    for (j=1; j<var->order && !err; j++) {
+	for (i=0; i<var->neqns && !err; i++) {
+	    depvar = vl->stochvars[i + 1];
+	    compose_varlist(vl, depvar, j, 0, 0, pdinfo);
+	    testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A);
+	    err = testmod.errcode;
+	    if (!err) {
+		/* record residuals for equation i, order j */
+		tm = testmod.t1;
+		for (t=0; t<var->T; t++) {
+		    gretl_matrix_set(var->F, t, i, testmod.uhat[tm++]);
+		}
+	    }
+	    clear_model(&testmod);
+	}
+	if (!err) {
+	    /* resids matrix should now be complete for order j */
+	    ldet = gretl_VAR_ldet(var, &err);
+	}
+	if (!err) {
+	    double ll;
+	    int p = var->ncoeff - (g * (var->order - j));
+	    int c, k = g * p;
+
+	    ll = -(g * T / 2.0) * (LN_2_PI + 1) - (T / 2.0) * ldet;
+	    crit[0] = (-2.0 * ll + 2.0 * k) / T;               /* AIC */
+	    crit[1] = (-2.0 * ll + k * log(T)) / T;            /* BIC */
+	    crit[2] = (-2.0 * ll + 2.0 * k * log(log(T))) / T; /* HQC */
+
+	    gretl_matrix_set(lltab, m, 0, ll);
+	    if (j == 1) {
+		gretl_matrix_set(lltab, m, 1, 0);
+	    } else {
+		LRtest = 2.0 * (ll - gretl_matrix_get(lltab, m-1, 0));
+		gretl_matrix_set(lltab, m, 1, chisq(LRtest, g * g));
+	    }	
+	    
+	    for (c=0; c<3; c++) {
+		gretl_matrix_set(crittab, m, c, crit[c]);
+		if (crit[c] < best[c]) {
+		    best[c] = crit[c];
+		    best_row[c] = m;
+		}
+	    }
+	
+	    m++;
+	}
+    }
+
+    if (!err) {
+	gretl_matrix_set(lltab, m, 0, var->ll);
+	LRtest = 2.0 * (var->ll - gretl_matrix_get(lltab, m - 1, 0));
+	gretl_matrix_set(lltab, m, 1, chisq(LRtest, g * g));
+	gretl_matrix_set(crittab, m, 0, var->AIC);
+	gretl_matrix_set(crittab, m, 1, var->BIC);
+	gretl_matrix_set(crittab, m, 2, var->HQC);
+	gretl_VAR_print_lagsel(lltab, crittab, best_row, prn);
+    }
+
+    bailout:
+
+    gretl_matrix_free(crittab);
+    gretl_matrix_free(lltab);
+
+    gretl_matrix_free(var->F);
+    var->F = NULL;
+
+    return err;
+}
+
+/* Per-equation F-tests for excluding variables and for excluding the
+   last lag, plus AIC and BIC comparison for last lag.
+*/
 
 static int VAR_compute_tests (MODEL *varmod, GRETL_VAR *var,
 			      struct var_lists *vl,
@@ -1386,6 +1593,7 @@ static int VAR_compute_tests (MODEL *varmod, GRETL_VAR *var,
     }
 
     /* restrictions for all lags of specific variables */
+
     for (j=0; j<var->neqns && !err; j++) {
 
 	compose_varlist(vl, depvar, var->order, j + 1, 0, pdinfo);	
@@ -1397,7 +1605,7 @@ static int VAR_compute_tests (MODEL *varmod, GRETL_VAR *var,
 		err = 1;
 	    }
 	} else {
-	    testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A, 0.0);
+	    testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A);
 	    err = testmod.errcode;
 	    if (!err) {
 		F = ((testmod.ess - varmod->ess) / var->order) / 
@@ -1413,11 +1621,12 @@ static int VAR_compute_tests (MODEL *varmod, GRETL_VAR *var,
     }
     
     /* restrictions for last lag, all variables */
+
     if (!err && var->order > 1) {
 
 	compose_varlist(vl, depvar, var->order - 1, 0, 0, pdinfo);	
 
-	testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A, 0.0);
+	testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A);
 	err = testmod.errcode;
 
 	if (!err) {
@@ -1434,7 +1643,7 @@ static int VAR_compute_tests (MODEL *varmod, GRETL_VAR *var,
 	}
 
 	if (!err && var->F != NULL) {
-	    /* record residuals for LR test */
+	    /* record residuals for LR test, AIC, BIC, HQC */
 	    int j, t = testmod.t1;
 
 	    for (j=0; j<var->T; j++) {
@@ -1483,11 +1692,13 @@ static int VAR_add_stats (GRETL_VAR *var)
     if (!err) {
 	int T = var->T;
 	int g = var->neqns;
-	int k = var->ncoeff;
+	int p = var->ncoeff;
+	int k = g * p;
 
 	var->ll = -(g * T / 2.0) * (LN_2_PI + 1) - (T / 2.0) * var->ldet;
-	var->AIC = (-2.0 * var->ll + 2.0 * k * g) / T;
-	var->BIC = (-2.0 * var->ll + log(T) * k * g) / T;
+	var->AIC = (-2.0 * var->ll + 2.0 * k) / T;
+	var->BIC = (-2.0 * var->ll + k * log(T)) / T;
+	var->HQC = (-2.0 * var->ll + 2.0 * k * log(log(T))) / T;
     }
 
     if (!err && var->F != NULL) {
@@ -1516,14 +1727,13 @@ static int VAR_add_stats (GRETL_VAR *var)
 */
 
 static GRETL_VAR *real_var (int order, const int *inlist, 
+			    struct var_lists *vl,
 			    double ***pZ, DATAINFO *pdinfo,
 			    gretlopt opt, int *err)
 {
     GRETL_VAR *var = NULL;
-    int oldt1 = pdinfo->t1;
-    int oldt2 = pdinfo->t2;
-    struct var_lists vlists;
     gretlopt lsqopt = OPT_A | OPT_Z;
+    int lagtest = (opt & OPT_L);
     int i, k, neqns;
 
     if (order < 1) {
@@ -1537,23 +1747,22 @@ static GRETL_VAR *real_var (int order, const int *inlist,
     }
 
     *err = organize_var_lists(inlist, (const double **) *pZ, pdinfo, 
-			      order, &vlists);
+			      order, vl);
     if (*err) {
 	return NULL;
     }
 
     /* generate the required lags */
-    if (real_list_laggenr(vlists.stochvars, pZ, pdinfo, 
-			  order, vlists.lagvlist)) {
-	*err = E_ALLOC;
+    if ((*err = VAR_list_laggenr(vl->stochvars, pZ, pdinfo, 
+				 order, vl->lagvlist))) {
 	goto var_bailout;
     }
 
-    neqns = vlists.stochvars[0];    
+    neqns = vl->stochvars[0];    
 
     /* compose base VAR list (entry 1 will vary across equations);
        assemble test list for t1 and t2 while we're at it */
-    *err = compose_varlist(&vlists, vlists.stochvars[1], 
+    *err = compose_varlist(vl, vl->stochvars[1], 
 			   order, 0, 1, pdinfo);
     if (*err) {
 	*err = E_DATA;
@@ -1561,7 +1770,7 @@ static GRETL_VAR *real_var (int order, const int *inlist,
     }
 
     /* sort out sample range */
-    if (check_for_missing_obs(vlists.testlist, &pdinfo->t1, &pdinfo->t2,
+    if (check_for_missing_obs(vl->testlist, &pdinfo->t1, &pdinfo->t2,
 			      (const double **) *pZ, NULL)) {
 	*err = E_MISSDATA;
 	goto var_bailout;
@@ -1579,10 +1788,9 @@ static GRETL_VAR *real_var (int order, const int *inlist,
     for (i=0; i<neqns && !*err; i++) {
 	MODEL *pmod = var->models[i];
 
-	compose_varlist(&vlists, vlists.stochvars[i + 1], 
-			order, 0, 0, pdinfo);
+	compose_varlist(vl, vl->stochvars[i + 1], order, 0, 0, pdinfo);
 
-	*pmod = lsq(vlists.reglist, pZ, pdinfo, VAR, lsqopt, 0.0);
+	*pmod = lsq(vl->reglist, pZ, pdinfo, VAR, lsqopt);
 
 	if (pmod->errcode) {
 	    *err = pmod->errcode;
@@ -1596,18 +1804,12 @@ static GRETL_VAR *real_var (int order, const int *inlist,
 	}
 
 	if (!*err) {
-	    *err = VAR_compute_tests(pmod, var, &vlists, pZ, pdinfo, i, &k);
+	    *err = VAR_compute_tests(pmod, var, vl, pZ, pdinfo, i, &k);
 	}
     }
 
  var_bailout:
 
-    var_lists_free(&vlists);
-
-    /* reset sample range */
-    pdinfo->t1 = oldt1;
-    pdinfo->t2 = oldt2;
-
     if (!*err) {
 	var->ncoeff = var->models[0]->ncoeff;
 	var->t1 = var->models[0]->t1;
@@ -1616,7 +1818,7 @@ static GRETL_VAR *real_var (int order, const int *inlist,
 	*err = VAR_add_stats(var);
     }
 
-    if (!*err) {
+    if (!*err && !lagtest) {
 	*err = gretl_VAR_do_error_decomp(var->S, var->C);
     }
 
@@ -1638,22 +1840,38 @@ static int *
 maybe_expand_VAR_list (const int *list, double ***pZ, DATAINFO *pdinfo, 
 		       gretlopt opt, int *err)
 {
-    int needsep, addconst = 0, addseas = 0;
+    int needsep;
+    int addconst = 0;
+    int addtrend = 0;
+    int addseas = 0;
     int *vlist = NULL;
-    int i, l0, di0 = 0;
+    int vt = 0, di0 = 0;
+    int i, l0;
 
     if (!(opt & OPT_N)) {
 	addconst = 1;
     }
 
+    if (opt & OPT_T) {
+	addtrend = 1;
+    }    
+
     if (pdinfo->pd > 1 && (opt & OPT_D)) {
 	addseas = 1;
     }
 
-    if (!addconst && !addseas) {
+    if (!addconst && !addtrend && !addseas) {
 	return NULL;
     }
 
+    if (addtrend) {
+	vt = gettrend(pZ, pdinfo, 0);
+	if (vt == 0) {
+	    *err = E_ALLOC;
+	    return NULL;
+	}
+    }	
+
     if (addseas) {
 	di0 = dummy(pZ, pdinfo, 0);
 	if (di0 == 0) {
@@ -1663,11 +1881,9 @@ maybe_expand_VAR_list (const int *list, double ***pZ, DATAINFO *pdinfo,
     } 
 
     needsep = !gretl_list_has_separator(list);
-    l0 = list[0] + needsep;
 
-    if (addconst) {
-	l0 += 1;
-    }
+    l0 = list[0] + needsep + addconst + addtrend;
+
     if (addseas) {
 	l0 += pdinfo->pd - 1;
     }
@@ -1690,6 +1906,9 @@ maybe_expand_VAR_list (const int *list, double ***pZ, DATAINFO *pdinfo,
 		vlist[j++] = di0 + i;
 	    }
 	}
+	if (addtrend) {
+	    vlist[j++] = vt;
+	}
 	if (addconst) {
 	    vlist[j++] = 0;
 	}
@@ -1698,8 +1917,17 @@ maybe_expand_VAR_list (const int *list, double ***pZ, DATAINFO *pdinfo,
     return vlist;
 }
 
+static void var_lists_null (struct var_lists *vl)
+{
+    vl->detvars = NULL;
+    vl->stochvars = NULL;
+    vl->reglist = NULL;
+    vl->testlist = NULL;
+    vl->lagvlist = NULL;
+}
+
 /**
- * full_VAR:
+ * gretl_VAR:
  * @order: lag order for the VAR
  * @list: specification for the first model in the set.
  * @pZ: pointer to data matrix.
@@ -1710,7 +1938,10 @@ maybe_expand_VAR_list (const int *list, double ***pZ, DATAINFO *pdinfo,
  *       if includes %OPT_D, add seasonal dummies;
  *       if includes %OPT_N, do not include a constant.
  *       if includes %OPT_Q, do not show individual regressions.
+ *       if includes %OPT_T, include a linear trend.
+ *       if includes %OPT_L, test for optimal lag length (only).
  * @prn: gretl printing struct.
+ * @err: location to receive error code.
  *
  * Estimate a vector auto-regression (VAR), print and save
  * the results.
@@ -1718,29 +1949,47 @@ maybe_expand_VAR_list (const int *list, double ***pZ, DATAINFO *pdinfo,
  * Returns: pointer to VAR struct, which may be %NULL on error.
  */
 
-GRETL_VAR *full_VAR (int order, int *list, double ***pZ, DATAINFO *pdinfo,
-		     gretlopt opt, PRN *prn)
+GRETL_VAR *gretl_VAR (int order, int *list, double ***pZ, DATAINFO *pdinfo,
+		      gretlopt opt, PRN *prn, int *err)
 {
     GRETL_VAR *var = NULL;
+    struct var_lists vl;
     int *vlist = NULL;
-    int err = 0;
+    /* record original sample range */
+    int oldt1 = pdinfo->t1;
+    int oldt2 = pdinfo->t2;
 
-    gretl_list_purge_const(list);
-    vlist = maybe_expand_VAR_list(list, pZ, pdinfo, opt, &err);
+    var_lists_null(&vl);
 
-    if (!err) {
+    gretl_list_purge_const(list, (const double **) *pZ, pdinfo);
+
+    vlist = maybe_expand_VAR_list(list, pZ, pdinfo, opt, err);
+
+    if (!*err) {
 	var = real_var(order, (vlist != NULL)? vlist : list, 
-		       pZ, pdinfo, opt, &err);
+		       &vl, pZ, pdinfo, opt, err);
+    }
+    
+    if (var != NULL) {
+	if (opt & OPT_L) {
+	    gretl_VAR_do_lagsel(var, &vl, pZ, pdinfo, prn);
+	    gretl_VAR_free(var);
+	    var = NULL;
+	} else {
+	    gretl_VAR_print(var, pdinfo, opt, prn);
+	}
     }
 
-    if (var != NULL) {
-	gretl_VAR_print(var, pdinfo, opt, prn);
-    }
+    var_lists_free(&vl);
 
     if (vlist != NULL) {
 	free(vlist);
     }
 
+    /* reset original sample range */
+    pdinfo->t1 = oldt1;
+    pdinfo->t2 = oldt2; 
+
     return var;
 }
 
@@ -1961,6 +2210,7 @@ static int johansen_VAR (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 			 gretlopt opt, PRN *prn)
 {
     struct var_lists vlists;
+    struct var_lists *vl = &vlists;
     MODEL jmod;
     int i, err = 0;
 
@@ -1972,19 +2222,18 @@ static int johansen_VAR (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 
     /* generate the required lags, if any */
     if (jvar->order > 0) {
-	if (real_list_laggenr(vlists.stochvars, pZ, pdinfo, 
-			      jvar->order, vlists.lagvlist)) {
-	    err = E_ALLOC;
+	if ((err = VAR_list_laggenr(vl->stochvars, pZ, pdinfo, 
+				    jvar->order, vl->lagvlist))) {
 	    goto var_bailout;
 	}
     }
 
-    jvar->neqns = vlists.stochvars[0];    
+    jvar->neqns = vl->stochvars[0];    
 
     /* compose base VAR list (entry 1 will vary across equations);
        assemble test list for setting t1 and t2 while we're at it 
     */
-    err = compose_varlist(&vlists, vlists.stochvars[1], 
+    err = compose_varlist(&vlists, vl->stochvars[1], 
 			  jvar->order, 0, 1, pdinfo);
     if (err) {
 	err = E_DATA;
@@ -1993,7 +2242,7 @@ static int johansen_VAR (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 
     if (jvar->t2 == 0) {
 	/* sample hasn't been determined yet */
-	if (check_for_missing_obs(vlists.testlist, &pdinfo->t1, &pdinfo->t2,
+	if (check_for_missing_obs(vl->testlist, &pdinfo->t1, &pdinfo->t2,
 				  (const double **) *pZ, NULL)) {
 	    err = E_MISSDATA;
 	    goto var_bailout;
@@ -2010,7 +2259,7 @@ static int johansen_VAR (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (jrank(jvar) > 0) {
-	err = make_johansen_VECM_lists(jvar->jinfo, vlists.reglist, 
+	err = make_johansen_VECM_lists(jvar->jinfo, vl->reglist, 
 				       jvar->neqns);
 	if (err) {
 	    goto var_bailout;
@@ -2026,7 +2275,7 @@ static int johansen_VAR (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
     for (i=0; i<jvar->neqns; i++) {
 
 	/* insert the appropriate dependent variable */
-	vlists.reglist[1] = vlists.stochvars[i + 1];
+	vl->reglist[1] = vl->stochvars[i + 1];
 
 	if (jrank(jvar) > 0) {
 	    /* VECM: record ID number of first difference.  Note:
@@ -2034,20 +2283,20 @@ static int johansen_VAR (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 	       the final VECM models, even if the order of the present
 	       VAR is zero.
 	    */
-	    jvar->jinfo->difflist[i+1] = vlists.reglist[1];
+	    jvar->jinfo->difflist[i+1] = vl->reglist[1];
 	}
 
 	/* VAR in differences */
-	if (vlists.reglist[0] == 1) {
+	if (vl->reglist[0] == 1) {
 	    /* degenerate model (nothing to concentrate out) */
-	    transcribe_data_as_uhat(vlists.reglist[1], (const double **) *pZ,
+	    transcribe_data_as_uhat(vl->reglist[1], (const double **) *pZ,
 				    jvar->jinfo->u, i, jvar->t1);
 	} else {
-	    jmod = lsq(vlists.reglist, pZ, pdinfo, VAR, OPT_A | OPT_Z, 0.0);
+	    jmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A | OPT_Z);
 	    if ((err = jmod.errcode)) {
 		fprintf(stderr, "*** johansen_VAR: VAR in differences, eqn %d, "
 			"lsq err %d\n", i+1, err);
-		printlist(vlists.reglist, "list for this model");
+		printlist(vl->reglist, "list for this model");
 		goto var_bailout;
 	    }
 	    if (opt & OPT_V) {
@@ -2063,13 +2312,13 @@ static int johansen_VAR (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	/* y_{t-1} regressions */
-	vlists.reglist[1] = jvar->jinfo->levels_list[i + 1];
-	if (vlists.reglist[0] == 1) {
+	vl->reglist[1] = jvar->jinfo->levels_list[i + 1];
+	if (vl->reglist[0] == 1) {
 	    /* degenerate */
-	    transcribe_data_as_uhat(vlists.reglist[1], (const double **) *pZ,
+	    transcribe_data_as_uhat(vl->reglist[1], (const double **) *pZ,
 				    jvar->jinfo->v, i, jvar->t1);
 	} else {
-	    jmod = lsq(vlists.reglist, pZ, pdinfo, VAR, OPT_A | OPT_Z, 0.0);
+	    jmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A | OPT_Z);
 	    if ((err = jmod.errcode)) {
 		fprintf(stderr, "johansen_VAR: y_{t-1} regression, eqn %d, lsq err %d\n",
 			i+1, err);
@@ -2088,19 +2337,24 @@ static int johansen_VAR (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
     /* supplementary regressions for restricted cases */
     if (restricted(jvar)) {
 	if (jcode(jvar) == J_REST_CONST) {
-	    vlists.reglist[1] = 0;
+	    vl->reglist[1] = 0;
 	} else {
-	    vlists.reglist[1] = gettrend(pZ, pdinfo, 0);
+	    vl->reglist[1] = gettrend(pZ, pdinfo, 0);
+	    if (vl->reglist[1] == 0) {
+		err = E_ALLOC;
+		goto var_bailout;
+	    }
 	}
-	if (vlists.reglist[0] == 1) {
+
+	if (vl->reglist[0] == 1) {
 	    /* degenerate case */
-	    transcribe_data_as_uhat(vlists.reglist[1], (const double **) *pZ,
+	    transcribe_data_as_uhat(vl->reglist[1], (const double **) *pZ,
 				    jvar->jinfo->v, i, jvar->t1);
 	} else {	    
-	    jmod = lsq(vlists.reglist, pZ, pdinfo, VAR, OPT_A, 0.0);
+	    jmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A);
 	    if ((err = jmod.errcode)) {
-		fprintf(stderr, "johansen_VAR: restriction regression, eqn %d, lsq err %d\n",
-			i+1, err);
+		fprintf(stderr, "johansen_VAR: restriction regression, "
+			"eqn %d, lsq err %d\n", i+1, err);
 		goto var_bailout;
 	    }
 	    if (opt & OPT_V) {
@@ -2374,7 +2628,14 @@ johansen_VAR_prepare (int order, int rank, const int *list, const int *exolist,
 
     if (jcode(jvar) == J_UNREST_TREND) {
 	/* add trend to VAR list */
-	jvar->jinfo->varlist[k++] = gettrend(pZ, pdinfo, 0);
+	int vt = gettrend(pZ, pdinfo, 0);
+
+	if (vt == 0) {
+	    jvar->err = E_ALLOC;
+	    goto bailout;
+	} else {
+	    jvar->jinfo->varlist[k++] = vt;
+	}
     }	
 
     if (jcode(jvar) >= J_UNREST_CONST) {
@@ -2565,26 +2826,25 @@ int johansen_test_simple (int order, const int *list, double ***pZ, DATAINFO *pd
  * @pdinfo: dataset information.
  * @opt:
  * @prn: gretl printing struct.
- *
+ * @err: location to receive error code.
  *
  * Returns: pointer to struct containing information on 
- * the VECM system.
+ * the VECM system or %NULL on failure.
  */
 
 GRETL_VAR *vecm (int order, int rank, int *list, 
 		 double ***pZ, DATAINFO *pdinfo,
-		 gretlopt opt, PRN *prn)
+		 gretlopt opt, PRN *prn, int *err)
 {
     GRETL_VAR *jvar = NULL;
     int *endo_list = NULL, *exo_list = NULL;
     const int *vecm_list = list;
-    int err = 0;
 
-    gretl_list_purge_const(list);
+    gretl_list_purge_const(list, (const double **) *pZ, pdinfo);
 
     if (gretl_list_has_separator(list)) {
-	err = gretl_list_split_on_separator(list, &endo_list, &exo_list);
-	if (err) {
+	*err = gretl_list_split_on_separator(list, &endo_list, &exo_list);
+	if (*err) {
 	    return jvar;
 	}
 	vecm_list = endo_list;
@@ -2592,14 +2852,21 @@ GRETL_VAR *vecm (int order, int rank, int *list,
 
     if (rank <= 0 || rank > list[0]) {
 	sprintf(gretl_errmsg, _("vecm: rank %d is out of bounds"), rank);
+	*err = E_DATA;
 	return jvar;
     }
 
     jvar = johansen_wrapper(order, rank, vecm_list, exo_list,
 			    pZ, pdinfo, opt | OPT_S, prn);
     
-    if (jvar != NULL && !jvar->err) {
-	gretl_VAR_print(jvar, pdinfo, opt, prn);
+    if (jvar != NULL) {
+	if (!jvar->err) {
+	    gretl_VAR_print(jvar, pdinfo, opt, prn);
+	} else {
+	    *err = jvar->err;
+	}
+    } else {
+	*err = 1;
     }
 
     free(endo_list);
@@ -2765,4 +3032,80 @@ const int *gretl_VECM_list (const GRETL_VAR *vecm)
     return NULL;
 }
 
+double *gretl_VAR_get_series (const GRETL_VAR *var, const DATAINFO *pdinfo, 
+			      int idx, const char *key, int *err)
+{
+    double *x = NULL;
+    const char *msel;
+    int t, col = 0;
+
+    if (var == NULL || idx != M_UHAT) { /* FIXME generalize this */
+	*err = E_BADSTAT;
+	return NULL;
+    }
+
+    msel = strchr(key, '[');
+    if (msel == NULL || sscanf(msel, "[,%d]", &col) != 1) {
+	*err = E_PARSE;
+    } else if (col <= 0 || col > var->neqns) {
+	*err = E_DATA;
+    }
+
+    if (!*err) {
+	x = malloc(pdinfo->n * sizeof *x);
+	if (x == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	const MODEL *pmod = var->models[col-1];
+
+	if (pmod == NULL || pmod->full_n != pdinfo->n) {
+	    *err = E_DATA;
+	    free(x);
+	    x = NULL;
+	} else {
+	    for (t=0; t<pdinfo->n; t++) {
+		x[t] = pmod->uhat[t];
+	    }
+	}
+    }
+
+    return x;    
+}
+
+gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx, 
+				    int *err)
+{
+    gretl_matrix *M = NULL;
+
+    if (var == NULL) {
+	*err = E_BADSTAT;
+	return NULL;
+    }
+
+    switch (idx) {  
+    case M_UHAT:
+	M = gretl_matrix_copy(var->E);
+	break;
+    case M_COEFF:
+	M = gretl_matrix_copy(var->A);
+	break;
+    case M_VCV:
+	M = gretl_matrix_copy(var->S);
+	break;
+    default:
+	*err = E_BADSTAT;
+	break;
+    }
+
+    if (M == NULL && !*err) {
+	*err = E_ALLOC;
+    }
+
+    return M;
+}
+
 #include "irfboot.c"
+#include "varomit.c"
