@@ -24,6 +24,7 @@
 
 #include "libgretl.h"
 #include "bhhh_max.h"
+#include "libset.h"
 
 #include "../cephes/polrt.c"
 
@@ -34,6 +35,8 @@
 
 #include "arma_common.c"
 
+static PRN *errprn;
+
 /* check whether the MA estimates have gone out of bounds in the
    course of BHHH iterations */
 
@@ -64,7 +67,7 @@ ma_out_of_bounds (struct arma_info *ainfo, const double *ma_coeff,
 	return 0;
     }
 
-    if (ainfo->seasonal) {
+    if (arma_has_seasonal(ainfo)) {
 	qmax += ainfo->Q * ainfo->pd;
     }    
 
@@ -83,19 +86,13 @@ ma_out_of_bounds (struct arma_info *ainfo, const double *ma_coeff,
 
     temp[0] = 1.0;
 
-    /* Is this right, with seasonal MA?  Should we be 
-       calculating distinct seasonal and non-seasonal
-       roots?? 
-    */
-    
-
     /* initialize to non-seasonal MA or zero */
     for (i=0; i<qmax; i++) {
-	if (i < ainfo->q) {
-	    temp[i+1] = ma_coeff[i];
-	} else {
-	    temp[i+1] = 0.0;
-	}
+        if (i < ainfo->q) {
+            temp[i+1] = ma_coeff[i];
+        } else {
+            temp[i+1] = 0.0;
+        }
     }
 
     /* add seasonal MA and interaction */
@@ -116,6 +113,7 @@ ma_out_of_bounds (struct arma_info *ainfo, const double *ma_coeff,
 	im = roots[i].i;
 	rt = re * re + im * im;
 	if (rt > DBL_EPSILON && rt <= 1.0) {
+	    pprintf(errprn, "MA root %d = %g\n", i, rt);
 	    fprintf(stderr, "MA root %d = %g\n", i, rt);
 	    err = 1;
 	    break;
@@ -139,15 +137,21 @@ static void do_MA_partials (double *drv,
 
     for (i=0; i<ainfo->q; i++) {
 	s = t - (i + 1);
-	drv[t] -= ma_coeff[i] * drv[s];
+	if (s >= 0) {
+	    drv[t] -= ma_coeff[i] * drv[s];
+	}
     }
 
     for (i=0; i<ainfo->Q; i++) {
 	s = t - ainfo->pd * (i + 1);
-	drv[t] -= sma_coeff[i] * drv[s];
-	for (j=0; j<ainfo->q; j++) {
-	    p = s - (j + 1);
-	    drv[t] -= sma_coeff[i] * ma_coeff[j] * drv[p];
+	if (s >= 0) {
+	    drv[t] -= sma_coeff[i] * drv[s];
+	    for (j=0; j<ainfo->q; j++) {
+		p = s - (j + 1);
+		if (p >= 0) {
+		    drv[t] -= sma_coeff[i] * ma_coeff[j] * drv[p];
+		}
+	    }
 	}
     }
 }
@@ -184,14 +188,14 @@ static int arma_ll (double *coeff,
     ainfo = model_info_get_extra_info(arma);
 
     /* pointers to blocks of coefficients */
-    ar_coeff = coeff + ainfo->ifc;
+    ar_coeff = coeff + arma_has_const(ainfo);
     sar_coeff = ar_coeff + ainfo->p;
     ma_coeff = sar_coeff + ainfo->P;
     sma_coeff = ma_coeff + ainfo->q;
     reg_coeff = sma_coeff + ainfo->Q;
 
     /* pointers to blocks of derivatives */
-    de_a = de + ainfo->ifc;
+    de_a = de + arma_has_const(ainfo);
     de_sa = de_a + ainfo->p;
     de_m = de_sa + ainfo->P;
     de_sm = de_m + ainfo->q;
@@ -203,6 +207,7 @@ static int arma_ll (double *coeff,
 #endif
 
     if (ma_out_of_bounds(ainfo, ma_coeff, sma_coeff)) {
+	pputs(errprn, "arma: MA estimate(s) out of bounds\n");
 	fputs("arma: MA estimate(s) out of bounds\n", stderr);
 	return 1;
     }
@@ -215,7 +220,7 @@ static int arma_ll (double *coeff,
 	e[t] = y[t];
 
 	/* intercept */
-	if (ainfo->ifc) {
+	if (arma_has_const(ainfo)) {
 	    e[t] -= coeff[0];
 	} 
 
@@ -231,7 +236,7 @@ static int arma_ll (double *coeff,
 	    e[t] -= sar_coeff[i] * y[s];
 	    for (j=0; j<ainfo->p; j++) {
 		p = s - (j + 1);
-		e[t] -= sar_coeff[i] * ar_coeff[j] * y[p];
+		e[t] += sar_coeff[i] * ar_coeff[j] * y[p];
 	    }
 	}
 
@@ -279,7 +284,7 @@ static int arma_ll (double *coeff,
 	for (t=t1; t<=t2; t++) {
 
 	    /* the constant term (de_0) */
-	    if (ainfo->ifc) {
+	    if (arma_has_const(ainfo)) {
 		de[0][t] = -1.0;
 		do_MA_partials(de[0], ainfo, ma_coeff, sma_coeff, t);
 	    }
@@ -293,7 +298,7 @@ static int arma_ll (double *coeff,
 		    for (i=0; i<ainfo->P; i++) {
 			xlag = lag + ainfo->pd * (i + 1);
 			if (t >= xlag) {
-			    de_a[j][t] -= sar_coeff[i] * y[t-xlag];
+			    de_a[j][t] += sar_coeff[i] * y[t-xlag];
 			}
 		    }
 		    do_MA_partials(de_a[j], ainfo, ma_coeff, sma_coeff, t);
@@ -309,7 +314,7 @@ static int arma_ll (double *coeff,
 		    for (i=0; i<ainfo->p; i++) {
 			xlag = lag + (i + 1);
 			if (t >= xlag) {
-			    de_sa[j][t] -= ar_coeff[i] * y[t-xlag];
+			    de_sa[j][t] += ar_coeff[i] * y[t-xlag];
 			}
 		    }
 		    do_MA_partials(de_sa[j], ainfo, ma_coeff, sma_coeff, t);
@@ -385,7 +390,7 @@ static int arma_ll (double *coeff,
 
 static cmplx *arma_roots (struct arma_info *ainfo, const double *coeff) 
 {
-    const double *ar_coeff = coeff + ainfo->ifc;
+    const double *ar_coeff = coeff + arma_has_const(ainfo);
     const double *sar_coeff = ar_coeff + ainfo->p;
     const double *ma_coeff = sar_coeff + ainfo->P;
     const double *sma_coeff = ma_coeff + ainfo->q;
@@ -400,6 +405,10 @@ static cmplx *arma_roots (struct arma_info *ainfo, const double *coeff)
     qmax = (ainfo->q > ainfo->Q)? ainfo->q : ainfo->Q;
     lmax = (pmax > qmax)? pmax : qmax;
 
+    if (pmax == 0 && qmax == 0) {
+	return NULL;
+    }
+
     temp  = malloc((lmax + 1) * sizeof *temp);
     temp2 = malloc((lmax + 1) * sizeof *temp2);
     roots = malloc(nr * sizeof *roots);
@@ -465,46 +474,181 @@ static const double **
 make_armax_X (int *list, struct arma_info *ainfo, const double **Z)
 {
     const double **X;
-    int offset, nv;
+    int ypos, nx;
     int v, i;
 
-    offset = (ainfo->seasonal)? 7 : 4;
-    nv = list[0] - offset;
+    ypos = arma_list_y_position(ainfo);
+    nx = list[0] - ypos;
 
-    X = malloc((nv + 1) * sizeof *X);
+#if ARMA_DEBUG
+    fprintf(stderr, "make_armax_X: allocating %d series pointers\n",
+	    nx + 1);
+#endif    
+
+    X = malloc((nx + 1) * sizeof *X);
     if (X == NULL) {
 	return NULL;
     }
 
     /* the dependent variable */
-    X[0] = Z[list[offset]];
+    if (ainfo->dy != NULL) {
+	X[0] = ainfo->dy;
+    } else {
+	X[0] = Z[list[ypos]];
+    }
 
     /* the independent variables */
-    for (i=1; i<=nv; i++) {
-	v = list[i + offset];
+    for (i=1; i<=nx; i++) {
+	v = list[i + ypos];
 	X[i] = Z[v];
     }
 
     return X;
 }
 
-/* Run an initial OLS to get initial values for the AR coefficients */
+static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
+			       const int *alist, int axstart,
+			       double ***pZ, DATAINFO *pdinfo) 
+{
+#if ARMA_DEBUG
+    PRN *prn = gretl_print_new(GRETL_PRINT_STDERR);
+    gretlopt opt = OPT_V;
+#else
+    PRN *prn = NULL;
+    gretlopt opt = OPT_NONE;
+#endif
+    char fnstr[MAXLINE]; /* FIXME? */
+    char term[32];
+    nls_spec *spec;
+    int *plist = NULL;
+    int v = pdinfo->v;
+    int nparam;
+    int i, j, k, err = 0;
 
-static int ar_init_by_ols (const int *list, double *coeff,
-			   const double **Z, const DATAINFO *pdinfo,
-			   struct arma_info *ainfo)
+    spec = nls_spec_new(NLS, pdinfo);
+    if (spec == NULL) {
+	return E_ALLOC;
+    }
+
+    nparam = arma_has_const(ainfo) + ainfo->p + ainfo->P + ainfo->r;
+
+    plist = gretl_list_new(nparam);
+    if (plist == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    err = dataset_add_scalars(nparam, pZ, pdinfo); 
+    if (err) {
+	goto bailout;
+    }
+
+    strcpy(fnstr, "y=");
+    k = 1;
+
+    if (arma_has_const(ainfo)) {
+	double ybar = gretl_mean(0, pdinfo->n - 1, (*pZ)[1]);
+
+	strcat(fnstr, "b0+");
+	strcpy(pdinfo->varname[v], "b0");
+	(*pZ)[v][0] = ybar;
+	plist[k++] = v++;
+    }
+
+    if (!arma_has_const(ainfo)) {
+	(*pZ)[v][0] = 1.0; /* FIXME arbitrary */
+    }    
+
+    for (i=1; i<=ainfo->p; i++) {
+	if (i > 1) {
+	    strcat(fnstr, "+");
+	}
+	sprintf(term, "phi_%d*y_%d", i, i);
+	strcat(fnstr, term);
+	sprintf(pdinfo->varname[v], "phi_%d", i);
+	plist[k++] = v++;
+    }
+
+    if (ainfo->p > 0) {
+	strcat(fnstr, "+");
+    } else if (!arma_has_const(ainfo)) {
+	(*pZ)[v][0] = 1.0; /* FIXME arbitrary */
+    }      
+
+    for (i=1; i<=ainfo->P; i++) {
+	if (i > 1) {
+	    strcat(fnstr, "+");
+	}
+	sprintf(term, "Phi_%d*y_%d", i, i * ainfo->pd);
+	strcat(fnstr, term);
+	sprintf(pdinfo->varname[v], "Phi_%d", i);
+	plist[k++] = v++;
+    }
+
+    for (i=1; i<=ainfo->P; i++) {
+	for (j=1; j<=ainfo->p; j++) {
+	    sprintf(term, "-phi_%d*Phi_%d*y_%d", j, i, i * ainfo->pd + j);
+	    strcat(fnstr, term);
+	}
+    }  
+
+    for (i=0; i<ainfo->r; i++) {
+	j = alist[axstart + i];
+	sprintf(term, "+b%d*%s", i + 1, pdinfo->varname[j]);
+	strcat(fnstr, term);
+	sprintf(pdinfo->varname[v], "b%d", i + 1);
+	plist[k++] = v++;
+    }	
+
+    err = nls_spec_set_regression_function(spec, fnstr, pdinfo);
+
+    if (!err) {
+	err = nls_spec_add_param_list(spec, plist, (const double **) *pZ,
+				      pdinfo);
+    }
+
+    if (!err) {
+	*amod = model_from_nls_spec(spec, pZ, pdinfo, opt, prn);
+	err = amod->errcode;
+#if ARMA_DEBUG
+	if (!err) {
+	    printmodel(amod, pdinfo, OPT_NONE, prn);
+	}
+	gretl_print_destroy(prn);
+#endif
+    }
+
+    bailout:
+
+    nls_spec_destroy(spec);
+    free(plist);
+
+    return err;
+}
+
+/* Run a least squares model to get initial values for the AR
+   coefficients */
+
+static int ar_init_by_ls (const int *list, double *coeff,
+			  const double **Z, const DATAINFO *pdinfo,
+			  struct arma_info *ainfo)
 {
     int an = pdinfo->t2 - ainfo->t1 + 1;
-    int av = ainfo->p + ainfo->P + ainfo->r + 2;
     int np = ainfo->p, nq = ainfo->q;
+    int nmixed = ainfo->p * ainfo->P;
+    int ptotal = ainfo->p + ainfo->P + nmixed;
+    int av = ptotal + ainfo->r + 2;
+    int ifc = arma_has_const(ainfo);
+    const double *y;
     double **aZ = NULL;
     DATAINFO *adinfo = NULL;
     int *alist = NULL;
     MODEL armod;
-    int offset, xstart;
+    int xstart, axstart;
+    int lag, offset;
     int i, j, t, err = 0;
 
-    gretl_model_init(&armod);  
+    gretl_model_init(&armod); 
 
     alist = gretl_list_new(av);
     if (alist == NULL) {
@@ -513,7 +657,7 @@ static int ar_init_by_ols (const int *list, double *coeff,
 
     alist[1] = 1;
 
-    if (ainfo->ifc) {
+    if (ifc) {
 	alist[2] = 0;
 	offset = 3;
     } else {
@@ -522,15 +666,23 @@ static int ar_init_by_ols (const int *list, double *coeff,
     }
 
     for (i=0; i<ainfo->p; i++) {
-	alist[i + offset] = i + 2;
+	alist[i + offset] = 2 + i;
     }
 
     for (i=0; i<ainfo->P; i++) {
-	alist[i + offset + ainfo->p] = i + ainfo->p + 2;
-   }
+	alist[i + offset + ainfo->p] = ainfo->p + 2 + i;
+    }
 
+    for (i=0; i<ainfo->P; i++) {
+	for (j=0; j<ainfo->p; j++) {
+	    alist[i + offset + ainfo->p + ainfo->P] = 
+		ainfo->p + ainfo->P + 2 + i;
+	}
+    }
+
+    axstart = offset + ptotal;
     for (i=0; i<ainfo->r; i++) {
-	alist[i + offset + ainfo->p + ainfo->P] = i + ainfo->p + ainfo->P + 2;
+	alist[axstart + i] = ptotal + 2 + i;
     }
 
     adinfo = create_new_dataset(&aZ, av, an, 0);
@@ -539,66 +691,114 @@ static int ar_init_by_ols (const int *list, double *coeff,
 	return 1;
     }
 
-    xstart = (ainfo->seasonal)? 8 : 5;
+    if (ainfo->d > 0 || ainfo->D > 0) {
+	xstart = (arma_has_seasonal(ainfo))? 10 : 6;
+    } else {
+	xstart = (arma_has_seasonal(ainfo))? 8 : 5;
+    }
+
+    if (ainfo->dy != NULL) {
+	y = ainfo->dy;
+    } else {
+	y = Z[ainfo->yno];
+    }
 
     /* build temporary dataset including lagged vars */
     for (t=0; t<an; t++) {
 	int k, s;
 
 	s = t + ainfo->t1;
-	aZ[1][t] = Z[ainfo->yno][s];
+	aZ[1][t] = y[s];
+	strcpy(adinfo->varname[1], "y");
 
 	for (i=0; i<ainfo->p; i++) {
-	    s = t + ainfo->t1 - (i + 1);
-	    k = i + 2;
-	    aZ[k][t] = Z[ainfo->yno][s];
+	    lag = i + 1;
+	    s = t + ainfo->t1 - lag;
+	    k = 2 + i;
+	    aZ[k][t] = y[s];
+	    sprintf(adinfo->varname[k], "y_%d", lag);
 	}
 
 	for (i=0; i<ainfo->P; i++) {
-	    s = t + ainfo->t1 - ainfo->pd * (i + 1);
-	    k = i + ainfo->p + 2;
-	    aZ[k][t] = Z[ainfo->yno][s];
+	    lag = ainfo->pd * (i + 1);
+	    s = t + ainfo->t1 - lag;
+	    k = ainfo->p + 2 + i;
+	    aZ[k][t] = y[s];
+	    sprintf(adinfo->varname[k], "y_%d", lag);
+	    for (j=0; j<ainfo->p; j++) {
+		lag = ainfo->pd * (i + 1) + (j + 1);
+		s = t + ainfo->t1 - lag;
+		k = ainfo->p + ainfo->P + 2 + j;
+		aZ[k][t] = y[s];
+		sprintf(adinfo->varname[k], "y_%d", lag);
+	    }
 	}
 
 	s = t + ainfo->t1;
 
 	for (i=0; i<ainfo->r; i++) {
-	    k = i + ainfo->p + ainfo->P + 2;
+	    k = ptotal + 2 + i;
 	    j = list[xstart + i];
 	    aZ[k][t] = Z[j][s];
+	    strcpy(adinfo->varname[k], pdinfo->varname[j]);
 	}
     }
 
-    if (ainfo->seasonal) {
+    if (arma_has_seasonal(ainfo)) {
 	np += ainfo->P;
 	nq += ainfo->Q;
     }
 
-    /* run the OLS */
-    armod = lsq(alist, &aZ, adinfo, OLS, OPT_A | OPT_Z, 0.0);
-    err = armod.errcode;
+#if ARMA_DEBUG
+    printlist(alist, "'alist' in ar_init_by_ls");
+#endif
+
+    if (alist[0] == 1) {
+	/* arma 0, q model */
+	for (i=0; i<nq; i++) {
+	    /* insert zeros for MA coeffs */
+	    coeff[i + np + ifc] = 0.0;
+	} 
+	goto exit_init;
+    }
+
+    if (nmixed > 0) {
+	/* mixed: need to use nonlinear least squares */
+	err = arma_get_nls_model(&armod, ainfo, alist, axstart, &aZ, adinfo);
+    } else {
+	/* just use OLS */
+	armod = lsq(alist, &aZ, adinfo, OLS, OPT_A | OPT_Z);
+	err = armod.errcode;
+    }
+
     if (!err) {
 	j = 0;
 	for (i=0; i<armod.ncoeff; i++) {
-	    if (i == np + ainfo->ifc) {
+	    if (i == np + ifc) {
 		j += nq; /* reserve space for MA coeffs */
 	    }
 	    coeff[j++] = armod.coeff[i];
 	}
 	for (i=0; i<nq; i++) {
 	    /* insert zeros for MA coeffs */
-	    coeff[i + np + ainfo->ifc] = 0.0;
+	    coeff[i + np + ifc] = 0.0;
 	} 
     }
 
 #if ARMA_DEBUG
-    fprintf(stderr, "OLS init: ncoeff = %d, nobs = %d\n", 
-	    armod.ncoeff, armod.nobs);
-    for (i=0; i<armod.ncoeff; i++) {
-	fprintf(stderr, " coeff[%d] = %g\n", i, armod.coeff[i]);
+    if (!err) {
+	fprintf(stderr, "LS init: ncoeff = %d, nobs = %d\n", 
+		armod.ncoeff, armod.nobs);
+	for (i=0; i<armod.ncoeff; i++) {
+	    fprintf(stderr, " coeff[%d] = %g\n", i, armod.coeff[i]);
+	}
+    } else {
+	fprintf(stderr, "LS init: armod.errcode = %d\n", err);
     }
 #endif
 
+ exit_init:
+
     /* clear everything up */
     free(alist);
     destroy_dataset(aZ, adinfo);
@@ -612,9 +812,14 @@ static int ar_init_by_ols (const int *list, double *coeff,
 static model_info *
 set_up_arma_model_info (struct arma_info *ainfo)
 {
+    double tol = get_bhhh_toler();
     model_info *arma;
 
-    arma = model_info_new(ainfo->nc, ainfo->t1, ainfo->t2, 1.0e-6);
+    if (na(tol)) {
+	tol = 1.0e-6;
+    }
+
+    arma = model_info_new(ainfo->nc, ainfo->t1, ainfo->t2, ainfo->T, tol);
 
     if (arma == NULL) return NULL;
 
@@ -641,10 +846,12 @@ MODEL arma_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 
     if (opt & OPT_V) {
 	aprn = prn;
-    } 
-
-    ainfo.atype = ARMA_NATIVE;
+	errprn = prn;
+    } else {
+	errprn = NULL;
+    }
 
+    arma_info_init(&ainfo, 0, pdinfo);
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
@@ -654,16 +861,11 @@ MODEL arma_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	goto bailout;
     }
 
-    if (check_arma_list(alist, opt, &ainfo)) {
-	armod.errcode = E_UNSPEC;
+    err = arma_check_list(alist, opt, Z, pdinfo, &ainfo);
+    if (err) {
+	armod.errcode = err;
 	goto bailout;
-    }
-
-    /* dependent variable */
-    ainfo.yno = (ainfo.seasonal)? alist[7] : alist[4];
-
-    /* periodicity of data */
-    ainfo.pd = pdinfo->pd;
+    } 
 
     /* calculate maximum lag */
     calc_max_lag(&ainfo);
@@ -688,11 +890,16 @@ MODEL arma_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	goto bailout;
     }
 
-    /* initialize the coefficients: AR and regression part by OLS, 
-       MA at 0 */
-    err = ar_init_by_ols(alist, coeff, Z, pdinfo, &ainfo);
+    /* create differenced series if needed */
+    if (ainfo.d > 0 || ainfo.D > 0) {
+	err = arima_difference(Z[ainfo.yno], &ainfo);
+    }
+
+    /* initialize the coefficients: AR and regression part by least
+       squares, MA at 0 */
+    err = ar_init_by_ls(alist, coeff, Z, pdinfo, &ainfo);
     if (err) {
-	armod.errcode = E_ALLOC;
+	armod.errcode = err;
 	goto bailout;
     }	
 
@@ -714,8 +921,7 @@ MODEL arma_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	double *theta = model_info_get_theta(arma);
 	cmplx *roots;
 
-	write_arma_model_stats(pmod, arma, alist, Z[ainfo.yno], 
-			       theta, &ainfo);
+	write_arma_model_stats(pmod, arma, alist, Z, theta, &ainfo);
 
 	/* compute and save polynomial roots */
 	roots = arma_roots(&ainfo, theta);
@@ -736,7 +942,11 @@ MODEL arma_model (const int *list, const double **Z, const DATAINFO *pdinfo,
     free(alist);
     free(coeff);
     free(X);
+    free(ainfo.dy);
+
     model_info_free(arma);
 
+    errprn = NULL;
+
     return armod;
 }
