@@ -160,13 +160,20 @@ static int print_iterations (const char *path, PRN *prn)
 
 static int x12_date_to_n (const char *s, const DATAINFO *pdinfo)
 {
-    char date[12];
+    char date[12] = {0};
 
-    *date = 0;
-    strncat(date, s, 4);
     if (pdinfo->pd > 1) {
-	strcat(date, ":");
-	strncat(date, s + 4, 4);
+	if (strlen(s) <= 4) {
+	    strncat(date, s, 2);
+	    strcat(date, ":");
+	    strncat(date, s + 2, 4);
+	} else {
+	    strncat(date, s, 4);
+	    strcat(date, ":");
+	    strncat(date, s + 4, 4);
+	}
+    } else {
+	strncat(date, s, 4);
     }
 
     return dateton(date, pdinfo);
@@ -186,14 +193,18 @@ static int get_ll_stats (const char *fname, MODEL *pmod)
 	return 1;
     }
 
+    pmod->sigma = NADBL;
+
     gretl_push_c_numeric_locale();
 
     while (fgets(line, sizeof line, fp)) {
 	if (sscanf(line, "%11s %lf", statname, &x) == 2) {
 	    if (!strcmp(statname, "nobs")) pmod->nobs = (int) x;
+	    else if (!strcmp(statname, "var")) pmod->sigma = sqrt(x);
 	    else if (!strcmp(statname, "lnlkhd")) pmod->lnL = x;
 	    else if (!strcmp(statname, "aic")) pmod->criterion[C_AIC] = x;
 	    else if (!strcmp(statname, "bic")) pmod->criterion[C_BIC] = x;
+	    else if (!strcmp(statname, "hnquin")) pmod->criterion[C_HQC] = x;
 	}
     }
 
@@ -215,6 +226,9 @@ static int get_roots (const char *fname, MODEL *pmod,
     cmplx *roots;
 
     nr = ainfo->p + ainfo->q + ainfo->P + ainfo->Q;
+    if (nr == 0) {
+	return 0;
+    }
 
     roots = malloc(nr * sizeof *roots);
     if (roots == NULL) {
@@ -322,11 +336,11 @@ static int get_x12a_vcv (const char *fname, MODEL *pmod, int nc)
 static int 
 get_estimates (const char *fname, MODEL *pmod, struct arma_info *ainfo)
 {
-    double *ar_coeff = pmod->coeff + ainfo->ifc;
+    double *ar_coeff = pmod->coeff + arma_has_const(ainfo);
     double *ma_coeff = ar_coeff + ainfo->p + ainfo->P;
     double *x_coeff = ma_coeff + ainfo->q + ainfo->Q;
 
-    double *ar_sderr = pmod->sderr + ainfo->ifc;
+    double *ar_sderr = pmod->sderr + arma_has_const(ainfo);
     double *ma_sderr = ar_sderr + ainfo->p + ainfo->P;
     double *x_sderr = ma_sderr + ainfo->q + ainfo->Q;
 
@@ -464,7 +478,7 @@ get_uhat (const char *fname, MODEL *pmod, const DATAINFO *pdinfo)
 
 static void 
 populate_arma_model (MODEL *pmod, const int *list, const char *path, 
-		     const double *y, const DATAINFO *pdinfo, 
+		     const double **Z, const DATAINFO *pdinfo, 
 		     struct arma_info *ainfo)
 {
     char fname[MAXLEN];
@@ -481,6 +495,8 @@ populate_arma_model (MODEL *pmod, const int *list, const char *path,
 	return;
     }
 
+    pmod->full_n = pdinfo->n;
+
     for (t=0; t<pdinfo->n; t++) {
 	pmod->uhat[t] = pmod->yhat[t] = NADBL;
     }
@@ -515,7 +531,7 @@ populate_arma_model (MODEL *pmod, const int *list, const char *path,
 	fprintf(stderr, "problem getting model info\n");
 	pmod->errcode = E_FOPEN;
     } else {
-	write_arma_model_stats(pmod, NULL, list, y, NULL, ainfo);
+	write_arma_model_stats(pmod, NULL, list, Z, NULL, ainfo);
 	gretl_model_add_arma_varnames(pmod, pdinfo, ainfo->yno,
 				      ainfo->p, ainfo->q, 
 				      ainfo->P, ainfo->Q,
@@ -545,7 +561,7 @@ static int *
 arma_info_get_x_list (struct arma_info *ainfo, const int *alist)
 {
     int *xlist = NULL;
-    int start = (ainfo->seasonal)? 7 : 4;
+    int start = arma_list_y_position(ainfo);
     int i;
 
     xlist = gretl_list_new(ainfo->r);
@@ -559,17 +575,40 @@ arma_info_get_x_list (struct arma_info *ainfo, const int *alist)
     return xlist;
 }
 
+static void 
+make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
+{
+    double dx = date(t, pdinfo->pd, pdinfo->sd0);
+    int yr = (int) dx;
+    int subper = 0;
+    char *s;
+
+    sprintf(str, "%g", dx);
+    s = strchr(str, '.');
+
+    if (s != NULL) {
+	subper = atoi(s + 1);
+    } else if (pdinfo->pd > 1) {
+	subper = 1;
+    } 
+
+    if (subper > 0) {
+	sprintf(str, "%d.%d", yr, subper);
+    } else {
+	sprintf(str, "%d", yr);
+    }    
+}
+
 static int write_spc_file (const char *fname, 
 			   const double **Z, const DATAINFO *pdinfo,
 			   struct arma_info *ainfo, const int *alist,
-			   int verbose)
+			   gretlopt opt)
 {
     int ylist[2];
     int *xlist = NULL;
-    double dx;
     FILE *fp;
-    int startyr, startper;
-    char *s, tmp[8];
+    char datestr[12];
+    int tmax;
     int i;
 
     if (ainfo->r > 0) {
@@ -587,34 +626,33 @@ static int write_spc_file (const char *fname,
 
     gretl_push_c_numeric_locale();
 
-    dx = date(ainfo->t1, pdinfo->pd, pdinfo->sd0);
-    startyr = (int) dx;
-    sprintf(tmp, "%g", dx);
-    s = strchr(tmp, '.');
-    if (s != NULL) {
-	startper = atoi(s + 1);
-    } else if (pdinfo->pd > 1) {
-	startper = 1;
-    } else {
-	startper = 0;
-    }
+    make_x12a_date_string(ainfo->t1, pdinfo, datestr);
 
     fprintf(fp, "series {\n period = %d\n title = \"%s\"\n", pdinfo->pd, 
 	    pdinfo->varname[ainfo->yno]);
-    if (startper > 0) {
-	fprintf(fp, " start = %d.%d\n", startyr, startper);
-    } else {
-	fprintf(fp, " start = %d\n", startyr);
-    }
+    fprintf(fp, " start = %s\n", datestr);
 
     ylist[0] = 1;
     ylist[1] = ainfo->yno;
-    output_series_to_spc(ylist, Z, ainfo->t1, ainfo->t2, fp);
+
+    if ((opt & OPT_F) && ainfo->t2 < pdinfo->n - 1) {
+	tmax = pdinfo->n - 1;
+    } else {
+	tmax = ainfo->t2;
+    }
+
+    output_series_to_spc(ylist, Z, ainfo->t1, tmax, fp);
+
+    if (tmax > ainfo->t2) {
+	make_x12a_date_string(ainfo->t2, pdinfo, datestr);
+	fprintf(fp, " span = ( , %s)\n", datestr);
+    }
+
     fputs("}\n", fp);
 
     /* regression specification */
     fputs("Regression {\n", fp);
-    if (ainfo->ifc) {
+    if (arma_has_const(ainfo)) {
 	fputs(" variables = (const)\n", fp);
     }
     if (ainfo->r > 0) {
@@ -623,26 +661,41 @@ static int write_spc_file (const char *fname,
 	    fprintf(fp, "%s ", pdinfo->varname[xlist[i]]);
 	}
 	fputs(")\n", fp);
-	output_series_to_spc(xlist, Z, ainfo->t1, ainfo->t2, fp);
+	output_series_to_spc(xlist, Z, ainfo->t1, tmax, fp);
 	free(xlist);
     }
     fputs("\n}\n", fp);
 
     /* arima specification */
     if (ainfo->P > 0 || ainfo->Q > 0) {
-	fprintf(fp, "arima {\n model = (%d 0 %d)(%d 0 %d)\n}\n", 
-		ainfo->p, ainfo->q, ainfo->P, ainfo->Q);
+	fprintf(fp, "arima {\n model = (%d %d %d)(%d %d %d)\n}\n", 
+		ainfo->p, ainfo->d, ainfo->q, 
+		ainfo->P, ainfo->D, ainfo->Q);
     } else {
-	fprintf(fp, "arima {\n model = (%d 0 %d)\n}\n", ainfo->p, ainfo->q); 
+	fprintf(fp, "arima {\n model = (%d %d %d)\n}\n", 
+		ainfo->p, ainfo->d, ainfo->q); 
     }
 
-    if (verbose) {
-	fputs("estimate {\n print = (acm itr lkf lks mdl est rts rcm)\n", fp);
+    fputs("estimate {\n", fp);
+
+    if (opt & OPT_V) {
+	fputs(" print = (acm itr lkf lks mdl est rts rcm)\n", fp);
     } else {
-	fputs("estimate {\n print = (acm lkf lks mdl est rts rcm)\n", fp);
+	fputs(" print = (acm lkf lks mdl est rts rcm)\n", fp);
+    }
+    
+    fputs(" save = (rsd est lks acm rts rcm)\n", fp);
+
+    if (opt & OPT_C) {
+	fputs(" exact = none\n", fp);
     }
 
-    fputs(" save = (rsd est lks acm rts rcm)\n}\n", fp);
+    fputs("}\n", fp);
+
+    if (opt & OPT_F) {
+	fputs("forecast {\n save = (ftr)\n", fp);
+	fprintf(fp, " maxlead = %d\n}\n", tmax - ainfo->t2);
+    }
 
     gretl_pop_c_numeric_locale();
 
@@ -671,8 +724,12 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	aprn = prn;
     } 
 
-    ainfo.atype = ARMA_X12A;
+    if (pdinfo->t2 < pdinfo->n - 1) {
+	/* FIXME this is temporary */
+	opt |= OPT_F;
+    }
 
+    arma_info_init(&ainfo, ARMA_X12A, pdinfo);
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
@@ -682,17 +739,12 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	goto bailout;
     }
 
-    if (check_arma_list(alist, opt, &ainfo)) {
-	armod.errcode = E_UNSPEC;
+    err = arma_check_list(alist, opt, Z, pdinfo, &ainfo);
+    if (err) {
+	armod.errcode = err;
 	goto bailout;
     }
 
-    /* dependent variable */
-    ainfo.yno = (ainfo.seasonal)? alist[7] : alist[4];
-
-    /* periodicity of data */
-    ainfo.pd = pdinfo->pd;
-    
     /* calculate maximum lag */
     calc_max_lag(&ainfo);
 
@@ -700,13 +752,18 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
     if (arma_adjust_sample(pdinfo, Z, alist, &ainfo)) {
         armod.errcode = E_DATA;
 	goto bailout;
-    }	
+    }
+
+    /* create differenced series if needed */
+    if (ainfo.d > 0 || ainfo.D > 0) {
+	err = arima_difference(Z[ainfo.yno], &ainfo);
+    }    
 
     sprintf(yname, pdinfo->varname[ainfo.yno]);
 
     /* write out an .spc file */
     sprintf(path, "%s%c%s.spc", workdir, SLASH, yname);
-    write_spc_file(path, Z, pdinfo, &ainfo, alist, verbose);
+    write_spc_file(path, Z, pdinfo, &ainfo, alist, opt);
 
     /* run the program */
 #if defined(WIN32)
@@ -726,13 +783,17 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	armod.t1 = ainfo.t1;
 	armod.t2 = ainfo.t2;
 	armod.nobs = armod.t2 - armod.t1 + 1;
-	populate_arma_model(&armod, alist, path, Z[ainfo.yno], pdinfo, &ainfo);
+	populate_arma_model(&armod, alist, path, Z, pdinfo, &ainfo);
 	if (verbose && !armod.errcode) {
 	    print_iterations(path, aprn);
 	}
-	if (!armod.errcode && gretl_in_gui_mode()) {
-	    add_unique_output_file(&armod, path);
-	    gretl_model_set_int(&armod, "arma_by_x12a", 1);
+	if (!armod.errcode) {
+	    int xcode = (opt & OPT_C)? 2 : 1;
+
+	    if (gretl_in_gui_mode()) {
+		add_unique_output_file(&armod, path);
+	    }
+	    gretl_model_set_int(&armod, "arma_by_x12a", xcode);
 	}	
     } else {
 	armod.errcode = E_UNSPEC;
@@ -742,6 +803,7 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
  bailout:
 
     free(alist);
+    free(ainfo.dy);
 
     return armod;
 }
