@@ -68,7 +68,8 @@ enum {
     VARNAMES_OK = 0,
     VARNAMES_NULL,
     VARNAMES_NOTSTR,
-    VARNAMES_INVALID
+    VARNAMES_INVALID,
+    VARNAMES_NONE
 } varname_errors;
 
 char **sst = NULL;
@@ -279,24 +280,14 @@ static int is_date_format (int fmt)
 {
     int ret = 0;
 
-    switch (fmt) {
-    case 14:
-    case 15:
-    case 16:
-    case 17:
-    case 18:
-    case 19:
-    case 20:
-    case 21:
-    case 22:
-    case 45:
-    case 46:
-    case 47:
-    case 50:
+    fprintf(stderr, "is_date_format? fmt=%d\n", fmt);
+
+    if (fmt >= 14 && fmt <= 22) {
+	ret = 1;
+    } else if (fmt >= 45 && fmt <= 47) {
+	ret = 1;
+    } else if (fmt >= 50 && fmt <= 58) {
 	ret = 1;
-	break;
-    default:
-	break;
     }
 
     return ret;
@@ -343,7 +334,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		is_date_format(fmt)) {
 		fprintf(stderr, "Testing first obs cell (%d, %d): date format %d\n", 
 			i, j, fmt);
-		book->flags |= FIRST_COL_DATE_FORMAT;
+		book_set_numeric_dates(book);
 	    }
 	}
     }
@@ -805,11 +796,11 @@ static char *convert8to7 (const char *s, int count)
 {
     char *dest;
 
-    if (count > USER_VLEN - 1) {
-	count = USER_VLEN - 1;
+    if (count > VNAMELEN - 1) {
+	count = VNAMELEN - 1;
     }
 
-    dest = malloc(USER_VLEN);
+    dest = malloc(VNAMELEN);
     *dest = '\0';
     strncat(dest, s, count);
     iso_to_ascii(dest);
@@ -828,15 +819,15 @@ static char *convert16to7 (const unsigned char *s, int count)
     char *dest;
     int i, u, j = 0;
 
-    dest = malloc(USER_VLEN);
+    dest = malloc(VNAMELEN);
     if (dest == NULL) {
 	return NULL;
     }
 
-    memset(dest, 0, USER_VLEN);
+    memset(dest, 0, VNAMELEN);
 
 #if 1
-    for (i=0; i<count && j<USER_VLEN-1; i++) {
+    for (i=0; i<count && j<VNAMELEN-1; i++) {
 	u = MS_OLE_GET_GUINT16(s);
 	s += 2;
 	if ((isalnum(u) || ispunct(u)) && u < 128) {
@@ -850,7 +841,7 @@ static char *convert16to7 (const unsigned char *s, int count)
 	s += 2;
 	if ((isalnum(u) || ispunct(u) || u == ' ') && u < 128) {
 	    putchar(u);
-	    if (j<USER_VLEN-1) {
+	    if (j<VNAMELEN-1) {
 		dest[j++] = u;
 	    }
 	    if (i % 56 == 0) {
@@ -987,7 +978,7 @@ static int first_col_strings (wbook *book)
     int i, j = book->col_offset;
     int startrow = book->row_offset + 1;
     int ret = 1;
-    
+
     for (i=startrow; i<nrows; i++) {
 	dprintf("book->row_offset=%d, i=%d\n", book->row_offset, i);
 	dprintf("rows = %p\n", (void *) rows);
@@ -1001,6 +992,10 @@ static int first_col_strings (wbook *book)
 		rows[i].cells[j]);
     }
 
+    if (ret) {
+	book_set_obs_labels(book);
+    }
+
     return ret;
 }
 
@@ -1024,16 +1019,23 @@ static int fix_varname (char *vname)
 }   
 
 static int 
-check_all_varnames (wbook *book, int ncols, const char *blank_col, int skip)
+check_all_varnames (wbook *book, int ncols, const char *blank_col)
 {
     int j, i = book->row_offset;
-    int startcol = skip + book->col_offset;
-    char *test;
+    int startcol = book->col_offset;
+    int realcols = 0;
+    int vnames = 0;
+    int ret = VARNAMES_NONE;
+
+    if (book_obs_labels(book)) {
+	startcol++;
+    }
 
     for (j=startcol; j<ncols; j++) { 
 	if (blank_col[j]) {
 	    continue;
 	}
+
 	if (rows[i].cells[j] == NULL) {
 	    dprintf("got_varnames: rows[%d].cells[%d] is NULL\n", i, j);
 	    return VARNAMES_NULL;
@@ -1042,32 +1044,38 @@ check_all_varnames (wbook *book, int ncols, const char *blank_col, int skip)
 	dprintf("got_varnames: rows[%d].cells[%d] is '%s'\n", i, j, 
 		rows[i].cells[j]);
 
-	if (!IS_STRING(rows[i].cells[j])) {
-	    return VARNAMES_NOTSTR;
-	}
+	if (IS_STRING(rows[i].cells[j])) {
+	    /* skip beyond the quote */
+	    char *test = rows[i].cells[j] + 1;
 
-	/* skip beyond the quote */
-	test = rows[i].cells[j] + 1;
+	    /* "obs" or "id" is OK in the first col of the selection, 
+	       but not thereafter */
+	    if (j == startcol && obs_string(test)) {
+		/* pass along */
+		;
+	    } else {
+		int verr = check_varname(test);
 
-	/* "obs" or "id" is OK in the first col of the selection, 
-	   but not thereafter */
-	if (j == startcol && obs_string(test)) {
-	    /* pass along */
-	    ;
-	} else {
-	    int verr = check_varname(test);
-
-	    if (verr == VARNAME_BADCHAR) {
-		verr = fix_varname(test);
-	    }
+		if (verr == VARNAME_BADCHAR) {
+		    verr = fix_varname(test);
+		}
 	    
-	    if (verr) {
-		return VARNAMES_INVALID;
+		if (verr) {
+		    return VARNAMES_INVALID;
+		}
 	    }
+	    vnames++;
 	}
+	realcols++;
     }
 
-    return VARNAMES_OK;
+    if (vnames == realcols) {
+	ret = VARNAMES_OK;
+    } else if (vnames > 0) {
+	ret = VARNAMES_NOTSTR;
+    }
+
+    return ret;
 }
 
 static int missval_string (const char *s)
@@ -1101,12 +1109,16 @@ struct string_err {
 
 static int 
 check_data_block (wbook *book, int ncols, const char *blank_col,
-		  int skip, struct string_err *err)
+		  struct string_err *err)
 {
-    int startcol = skip + book->col_offset;
+    int startcol = book->col_offset;
     int startrow = book->row_offset + 1;
     int j, i, ret = 0;
 
+    if (book_obs_labels(book)) {
+	startcol++;
+    }
+
     err->row = 0;
     err->column = 0;
     err->str = NULL;
@@ -1152,11 +1164,12 @@ check_data_block (wbook *book, int ncols, const char *blank_col,
 */
 
 static int 
-n_vars_from_col (int totcols, char *blank_col, int offset, int first_special)
+n_vars_from_col (wbook *book, int totcols, char *blank_col)
 {
+    int offset = book->col_offset;
     int i, nv = 1;
 
-    if (first_special) {
+    if (book_time_series(book) || book_obs_labels(book)) {
 	offset++;
     }
 
@@ -1169,13 +1182,17 @@ n_vars_from_col (int totcols, char *blank_col, int offset, int first_special)
     return nv;
 }
 
-static int transcribe_data (double **Z, DATAINFO *pdinfo, wbook *book,
-			    int totcols, int startcol, 
-			    char *blank_col)
+static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo, 
+			    int totcols, char *blank_col)
 {
+    int startcol = book->col_offset;
     int roff = book->row_offset;
     int i, t, j = 1;
 
+    if (book_time_series(book) || book_obs_labels(book)) {
+	startcol++;
+    }    
+
     for (i=startcol; i<totcols; i++) {
 	int ts, missing = 0;
 
@@ -1188,10 +1205,14 @@ static int transcribe_data (double **Z, DATAINFO *pdinfo, wbook *book,
 	}
 
 	pdinfo->varname[j][0] = 0;
-	strncat(pdinfo->varname[j], rows[roff].cells[i] + 1, 
-		USER_VLEN - 1);
-	dprintf("accessing rows[%d].cells[%d] at %p\n",
-		roff, i, (void *) rows[roff].cells[i]);
+	if (book_auto_varnames(book)) {
+	    sprintf(pdinfo->varname[j], "v%d", j);
+	} else {
+	    strncat(pdinfo->varname[j], rows[roff].cells[i] + 1, 
+		    VNAMELEN - 1);
+	    dprintf("accessing rows[%d].cells[%d] at %p\n",
+		    roff, i, (void *) rows[roff].cells[i]);
+	}
 	dprintf("set varname[%d] = '%s'\n", j, pdinfo->varname[j]);
 
 	for (t=0; t<pdinfo->n; t++) {
@@ -1300,13 +1321,38 @@ get_sheet_dimensions (int *totcols, int *datacols, char **blank_col,
     return 0;
 }
 
+static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
+{
+    const char *s = rows[1 + book->row_offset].cells[book->col_offset];
+
+    if (book_numeric_dates(book)) {
+	int d0 = atoi(s);
+
+	MS_excel_date_string(newinfo->stobs, d0, pd, book_base_1904(book));
+    } else {
+	if (*s == '"' || *s == '\'') s++;
+	strcpy(newinfo->stobs, s);
+	colonize_obs(newinfo->stobs);
+    }
+
+    newinfo->pd = pd;
+    newinfo->structure = TIME_SERIES;
+
+    fprintf(stderr, "stobs='%s'\n", newinfo->stobs);
+    newinfo->sd0 = get_date_x(newinfo->pd, newinfo->stobs);
+    fprintf(stderr, "sd0=%g\n", newinfo->sd0);
+
+    book_set_time_series(book);
+    book_unset_obs_labels(book);
+}
+
 int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 		    PRN *prn)
 {
-    wbook book;
+    wbook xbook;
+    wbook *book = &xbook;
     double **newZ = NULL;
     DATAINFO *newinfo;
-    int label_strings, time_series = 0;
     int datacols, totcols;
     struct string_err strerr;
     char *blank_col = NULL;
@@ -1321,34 +1367,30 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 
     gretl_push_c_numeric_locale();
 
-    wbook_init(&book);
+    wbook_init(book);
 
-    if (excel_book_get_info(fname, &book)) {
+    if (excel_book_get_info(fname, book)) {
 	pputs(prn, _("Failed to get workbook info"));
 	err = 1;
-    } else if (book.nsheets == 0) {
+    } else if (book->nsheets == 0) {
 	pputs(prn, _("No worksheets found"));
 	err = 1;
     } else {
-	wbook_print_info(&book);
+	wbook_print_info(book);
     }
 
     if (!err) {
-	if (book.nsheets > 1) {
-	    wsheet_menu(&book, 1);
-	} else {
-	    wsheet_menu(&book, 0);
-	}
-	debug_print = book.debug;
-	if (debug_print) {
+	wsheet_menu(book, book->nsheets > 1);
+	if (book_debugging(book)) {
+	    debug_print = 1;
 	    print_version();
 	}
     }
 
     dprintf("sheet selected=%d; import offsets: col=%d, row=%d\n",
-	    book.selected, book.col_offset, book.row_offset);
+	    book->selected, book->col_offset, book->row_offset);
 
-    if (book.selected == -1) {
+    if (book->selected == -1) {
 	/* canceled */
 	err = -1; 
     }
@@ -1356,7 +1398,7 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     if (err) goto getout;
 
     /* processing for specific worksheet */
-    err = process_sheet(fname, &book, prn);
+    err = process_sheet(fname, book, prn);
 
     if (err) {
 	const char *buf = gretl_print_get_buffer(prn);
@@ -1375,37 +1417,41 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     if (err) goto getout;
 
     /* check feasibility of offsets */
-    if (book.row_offset > nrows) {
+    if (book->row_offset > nrows) {
 	pputs(prn, _("Starting row is out of bounds.\n"));
 	err = 1;
-    } else if (book.col_offset > totcols) {
+    } else if (book->col_offset > totcols) {
 	pputs(prn, _("Starting column is out of bounds.\n"));
 	err = 1;
     }
 
     if (err) goto getout;
 
-    label_strings = first_col_strings(&book);
-    if (label_strings) {
+    if (first_col_strings(book)) {
 	puts("found label strings in first imported column");
     } else {
 	puts("check for label strings in first imported column: not found");
     }
 
     /* any bad or missing variable names? */
-    err = check_all_varnames(&book, totcols, blank_col, label_strings);
+    err = check_all_varnames(book, totcols, blank_col);
 
     if (err == VARNAMES_NULL || err == VARNAMES_NOTSTR) {
 	pputs(prn, _("One or more variable names are missing.\n"));
 	pputs(prn, _(adjust_rc));
     } else if (err == VARNAMES_INVALID) {
 	invalid_varname(prn);
+    } else if (err == VARNAMES_NONE) {
+	pputs(prn, _("it seems there are no variable names\n"));
+	book_set_auto_varnames(book);
+	book->row_offset -= 1;
+	err = 0;
     }
 
     if (err) goto getout; 
 
     /* any bad data? */
-    err = check_data_block(&book, totcols, blank_col, label_strings, &strerr);
+    err = check_data_block(book, totcols, blank_col, &strerr);
 
     if (err == 1) {
 	pprintf(prn, _("Expected numeric data, found string:\n"
@@ -1420,24 +1466,25 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     }	    
 
     /* do we have a first column containing dates? */
-
     if (book_numeric_dates(book)) {
-	pd = pd_from_numeric_dates(nrows, book.row_offset, book.col_offset, NULL,
-				   &book);
-    } else if (obs_column_heading(rows[book.row_offset].cells[book.col_offset])) {
-	pd = consistent_date_labels(nrows, book.row_offset, book.col_offset, NULL); 
+	pd = pd_from_numeric_dates(nrows, book->row_offset, book->col_offset, NULL,
+				   book);
+    } else if (!book_auto_varnames(book)) {
+	int r0 = book->row_offset;
+	int c0 = book->col_offset;
+	
+	if (obs_column_heading(rows[r0].cells[c0])) {
+	    pd = consistent_date_labels(nrows, r0, c0, NULL);
+	}
     }
 
     if (pd) {
-	time_series_setup(rows[1 + book.row_offset].cells[book.col_offset],
-			  newinfo, pd, NULL, &time_series, &label_strings,
-			  book.flags);
+	book_time_series_setup(book, newinfo, pd);
     }    
 
     /* number of variables and observations for import dataset */
-    newinfo->v = n_vars_from_col(totcols, blank_col, book.col_offset,
-				 time_series || label_strings);
-    newinfo->n = nrows - 1 - book.row_offset + book.totmiss;
+    newinfo->v = n_vars_from_col(book, totcols, blank_col);
+    newinfo->n = nrows - 1 - book->row_offset + book->totmiss;
     fprintf(stderr, "newinfo->v = %d, newinfo->n = %d\n",
 	    newinfo->v, newinfo->n);
 
@@ -1449,7 +1496,7 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 
     set_all_missing(newZ, newinfo);
 
-    if (time_series) {
+    if (book_time_series(book)) {
 	ntodate_full(newinfo->endobs, newinfo->n - 1, newinfo);
     } else {
 	strcpy(newinfo->stobs, "1");
@@ -1460,20 +1507,18 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     } 
 
     /* OK: actually populate the dataset */
-    transcribe_data(newZ, newinfo, &book, totcols,
-		    book.col_offset + (time_series || label_strings),
-		    blank_col);
+    transcribe_data(book, newZ, newinfo, totcols, blank_col);
 
     if (fix_varname_duplicates(newinfo)) {
 	pputs(prn, _("warning: some variable names were duplicated\n"));
     }
 
-    if (label_strings) {
+    if (book_obs_labels(book)) {
 	dataset_allocate_obs_markers(newinfo);
 	if (newinfo->S != NULL) {
-	    i = book.col_offset;
+	    i = book->col_offset;
 	    for (t=0; t<newinfo->n; t++) {
-		int ts = t + 1 + book.row_offset;
+		int ts = t + 1 + book->row_offset;
 
 		strncat(newinfo->S[t], rows[ts].cells[i] + 1, OBSLEN - 1);
 	    }
@@ -1490,7 +1535,7 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
  getout:
     
     free(blank_col);
-    wbook_free(&book);
+    wbook_free(book);
     free_sheet();
 
     gretl_pop_c_numeric_locale();
