@@ -168,7 +168,7 @@ static int write_garch_stats (MODEL *pmod, const double **Z,
     pmod->adjrsq = NADBL; 
     pmod->fstt = NADBL;
 
-    mle_aic_bic(pmod, 1);
+    mle_criteria(pmod, 1);
 
     pmod->ci = GARCH;
     pmod->ifc = 1;
@@ -577,9 +577,10 @@ garch_init_by_arma (const MODEL *pmod, const int *garchlist,
 
 /* sanity/dimension check */
 
-static int *get_garch_list (const int *list, int *err)
+static int *get_garch_list (const int *list, const double **Z,
+			    const DATAINFO *pdinfo, int *err)
 {
-    int *ret = NULL;
+    int *glist = NULL;
     int i, p = list[1], q = list[2];
     int add0 = 1;
 
@@ -601,26 +602,25 @@ static int *get_garch_list (const int *list, int *err)
 
     /* insert constant if not present */
     for (i=4; i<=list[0]; i++) {
-	if (list[i] == 0) {
+	if (list[i] == 0 || true_const(list[i], Z, pdinfo)) {
 	    add0 = 0;
 	    break;
 	}
     }
 
-    ret = malloc((list[0] + 1 + add0) * sizeof *ret);
-    if (ret == NULL) {
+    glist = gretl_list_new(list[0] + add0);
+    if (glist == NULL) {
 	*err = E_ALLOC;
     } else {
-	ret[0] = list[0] + add0;
 	for (i=1; i<=list[0]; i++) {
-	    ret[i] = list[i];
+	    glist[i] = list[i];
 	}
 	if (add0) {
-	    ret[i] = 0;
+	    glist[i] = 0;
 	}
     }
 
-    return ret;
+    return glist;
 }
 
 /* make regresson list for initial OLS */
@@ -643,26 +643,39 @@ static int *make_ols_list (const int *list)
     return olist;
 }
 
-#if 0
+#define GARCH_AUTOCORR_TEST 1
+
+#if GARCH_AUTOCORR_TEST
+
 int garch_pretest (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
-		   PRN *prn)
+		   double *LMF, double *pvF)
 {
     int err;
 
     err = autocorr_test(pmod, pdinfo->pd, pZ, pdinfo,
-			OPT_S | OPT_Q, prn);
+			OPT_S | OPT_Q, NULL);
 
     if (!err) {
-	char teststat[64];
-	double LMF = get_last_test_statistic(teststat);
-	double pval = get_last_pvalue(teststat);
-
-	fprintf(stderr, "LMF = %g, pval = %g\n", LMF, pval);
-    }
+	*LMF = get_last_test_statistic(NULL);
+	*pvF = get_last_pvalue(NULL);
+    } 
 
     return err;
 }
-#endif
+
+static void autocorr_message (double LMF, double pvF, int order, PRN *prn)
+{
+    if (!na(LMF) && pvF < 0.05) {
+	pputs(prn, "\nConvergence was not reached.  One possible reason "
+	      "for this is\nautocorrelation in the error term.\n");
+	pprintf(prn, "After estimating the model by OLS, the following result "
+		"was\nobtained for a test of autocorrelation of order %d:\n",
+		order);
+	pprintf(prn, "LMF = %g, with p-value %g\n", LMF, pvF);
+    }
+}
+
+#endif /* GARCH_AUTOCORR_TEST */
 
 #define GARCH_SCALE_SIGMA 1
 
@@ -672,13 +685,17 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 		   PRN *prn, gretlopt opt) 
 {
     MODEL model;
-    int *list = NULL, *ols_list = NULL;
+    int *list = NULL;
+    int *ols_list = NULL;
+    double LMF = NADBL;
+    double pvF = NADBL;
     double scale = 1.0;
     int t, err, init_err, yno = 0;
 
     gretl_model_init(&model);
 
-    list = get_garch_list(cmdlist, &err);
+    list = get_garch_list(cmdlist, (const double **) *pZ,
+			  pdinfo, &err);
     if (err) {
 	model.errcode = err;
     }
@@ -692,35 +709,20 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 
     /* run initial OLS */
     if (!err) {
-	model = lsq(ols_list, pZ, pdinfo, OLS, OPT_A | OPT_M, 0.0);
+	model = lsq(ols_list, pZ, pdinfo, OLS, OPT_A | OPT_M);
 	if (model.errcode) {
 	    err = model.errcode;
 	}
     }
 
-#if 0
+#if GARCH_AUTOCORR_TEST
     /* pretest the residuals for autocorrelation */
-    if (!err) {
-	err = garch_pretest(&model, pZ, pdinfo, prn);
+    if (!err && prn != NULL) {
+	garch_pretest(&model, pZ, pdinfo, &LMF, &pvF);
     }
 #endif
 
-#if GARCH_SCALE_SDY
-    if (!err) {
-	yno = ols_list[1];
-	scale = model.sdy;
-	for (t=0; t<pdinfo->n; t++) {
-	    if (!na((*pZ)[yno][t])) {
-		(*pZ)[yno][t] /= scale;
-	    }
-	}
-	for (t=0; t<model.ncoeff; t++) {
-	    model.coeff[t] *= scale;
-	}
-	model.ess /= scale * scale;
-	model.sigma = sqrt(model.ess / model.dfd);
-    } 
-#elif GARCH_SCALE_SIGMA
+#if GARCH_SCALE_SIGMA
     if (!err) {
 	yno = ols_list[1];
 	scale = model.sigma;
@@ -735,7 +737,7 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 	model.ess /= scale * scale;
 	model.sigma = 1.0;
     } 
-#endif /* alternative scalings */
+#endif 
 
     /* default variance parameter initialization */
     vparm_init[0] = model.sigma * model.sigma;
@@ -765,6 +767,16 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
     free(ols_list);
     free(list);
 
+#if GARCH_AUTOCORR_TEST
+    if (!na(LMF)) {
+	if (model.errcode == E_NOCONV) {
+	    autocorr_message(LMF, pvF, pdinfo->pd, prn);
+	} else {
+	    gretl_model_destroy_tests(&model);
+	}
+    }
+#endif
+
     return model;
 }
 
