@@ -629,7 +629,7 @@ build_VECM_models (GRETL_VAR *vecm, double ***pZ, DATAINFO *pdinfo, int iter)
 #if JDEBUG
 	printlist(biglist, "build_VECM_models: biglist");
 #endif
-	*vecm->models[i] = lsq(biglist, pZ, pdinfo, OLS, lsqopt, 0.0);
+	*vecm->models[i] = lsq(biglist, pZ, pdinfo, OLS, lsqopt);
 	err = vecm->models[i]->errcode;
 	if (!err) {
 	    vecm->models[i]->ID = i + 1;
@@ -949,7 +949,7 @@ static int johansen_ll_calc (GRETL_VAR *jvar, const double *eigvals)
     if (Suu == NULL) {
 	err = E_ALLOC;
     } else {
-	ldet = gretl_matrix_log_determinant(Suu);
+	ldet = gretl_matrix_log_determinant(Suu, &err);
 	jvar->ll = - T_2 * n * (1.0 + LN_2_PI) - T_2 * ldet;
 	for (i=0; i<h; i++) {
 	    jvar->ll -= T_2 * log(1.0 - eigvals[i]); 
@@ -980,6 +980,7 @@ static int vecm_ll_stats (GRETL_VAR *vecm)
 
     vecm->AIC = (-2.0 * vecm->ll + 2.0 * k * g) / T;
     vecm->BIC = (-2.0 * vecm->ll + log(T) * k * g) / T;
+    vecm->HQC = (-2.0 * vecm->ll + 2.0 * log(log(T)) * k * g) / T;
 
     return 0;
 }
@@ -1092,21 +1093,15 @@ int johansen_analysis (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, PRN *prn
 	goto eigenvals_bailout;
     }
 
-    if (nv > n) {
-	/* "re-expand" this matrix */
-	gretl_matrix_reuse(TmpR, nv, nv);
-    }
-
-    eigvals = gretl_general_matrix_eigenvals(M, TmpR);
-    if (eigvals == NULL) {
+    eigvals = gretl_general_matrix_eigenvals(M, 1, &err);
+    if (err) {
 	pputs(prn, _("Failed to find eigenvalues\n"));
-	err = E_ALLOC;
     } else {
-	err = gretl_eigen_sort(eigvals, TmpR, rank);
+	err = gretl_eigen_sort(eigvals, M, rank);
     }
 
 #if JDEBUG
-    gretl_matrix_print(TmpR, "raw eigenvector(s)");
+    gretl_matrix_print(M, "raw eigenvector(s)");
 #endif
 
     if (!err) {
@@ -1117,15 +1112,15 @@ int johansen_analysis (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, PRN *prn
 	} 
 
 	/* normalize the eigenvectors */
-	johansen_normalize(jvar->jinfo, TmpR);
+	johansen_normalize(jvar->jinfo, M);
 #if JDEBUG
-	gretl_matrix_print(TmpR, "normalized tmpR");
+	gretl_matrix_print(M, "normalized tmpR");
 #endif
 
 	if (rank == 0) {
 	    /* just running cointegration test */
-	    jvar->jinfo->Beta = TmpR;
-	    TmpR = NULL;
+	    jvar->jinfo->Beta = M;
+	    M = NULL;
 	    err = compute_alpha(jvar->jinfo, n);
 	    if (!err) {
 		print_beta_and_alpha(jvar->jinfo, eigvals, n, pdinfo, prn);
@@ -1135,7 +1130,7 @@ int johansen_analysis (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, PRN *prn
 	    /* estimating VECM */
 	    int do_stderrs = rank < jvar->neqns;
 
-	    jvar->jinfo->Beta = gretl_matrix_copy(TmpR);
+	    jvar->jinfo->Beta = gretl_matrix_copy(M);
 	    if (jvar->jinfo->Beta == NULL) {
 		err = E_ALLOC;
 	    }
@@ -1236,33 +1231,26 @@ johansen_bootstrap_round (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 	goto eigenvals_bailout;
     }    
 
-    if (nv > n) {
-	/* "re-expand" this matrix */
-	gretl_matrix_reuse(TmpR, nv, nv);
-    }
-
     if (!err) {
-	eigvals = gretl_general_matrix_eigenvals(M, TmpR);
-	if (eigvals == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    err = gretl_eigen_sort(eigvals, TmpR, jrank(jvar));
+	eigvals = gretl_general_matrix_eigenvals(M, 1, &err);
+	if (!err) {
+	    err = gretl_eigen_sort(eigvals, M, jrank(jvar));
 	}
     }
 
 #if JDEBUG
-    gretl_matrix_print(TmpR, "raw eigenvector(s)");
+    gretl_matrix_print(M, "raw eigenvector(s)");
 #endif
 
     if (!err) {
-	johansen_normalize(jvar->jinfo, TmpR); 
+	johansen_normalize(jvar->jinfo, M); 
 #if JDEBUG
-	gretl_matrix_print(TmpR, "normalized tmpR");
+	gretl_matrix_print(M, "normalized M");
 #endif
 	if (jvar->jinfo->Beta == NULL) {
-	    jvar->jinfo->Beta = gretl_matrix_copy(TmpR);
+	    jvar->jinfo->Beta = gretl_matrix_copy(M);
 	} else {
-	    gretl_matrix_copy_values(jvar->jinfo->Beta, TmpR);
+	    gretl_matrix_copy_values(jvar->jinfo->Beta, M);
 	}
 	if (jvar->jinfo->Beta == NULL) {
 	    err = E_ALLOC;
@@ -1294,7 +1282,9 @@ static int
 johansen_LR_calc (GRETL_VAR *jvar, const double *eigvals, PRN *prn)
 {
     gretl_matrix *Suu;
-    double llr, ldet, T_2 = (double) jvar->T / 2.0;
+    double llr = 0.0;
+    double ldet = 0.0;
+    double T_2 = (double) jvar->T / 2.0;
     int n = jvar->neqns;
     int h, i, err = 0;
 
@@ -1305,13 +1295,19 @@ johansen_LR_calc (GRETL_VAR *jvar, const double *eigvals, PRN *prn)
     if (Suu == NULL) {
 	err = E_ALLOC;
     } else {
-	ldet = gretl_matrix_log_determinant(Suu);
+	ldet = gretl_matrix_log_determinant(Suu, &err);
+    }
+
+    if (!err) {
 	llr = - T_2 * n * (1.0 + LN_2_PI) - T_2 * ldet;
 	for (i=0; i<h; i++) {
 	    pprintf(prn, "eigenvalue %d = %g\n", i+1, eigvals[i]);
 	    llr -= T_2 * log(1.0 - eigvals[i]); 
 	}
 	pputc(prn, '\n');
+    }
+
+    if (Suu != NULL) {
 	gretl_matrix_free(Suu);
     }
 
@@ -1417,12 +1413,9 @@ int vecm_beta_test (GRETL_VAR *jvar, PRN *prn)
     gretl_matrix_print_to_prn(M, "M", prn);
 
     if (!err) {
-	gretl_matrix_reuse(TmpR, m, m);
-	eigvals = gretl_general_matrix_eigenvals(M, TmpR);
-	if (eigvals == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    err = gretl_eigen_sort(eigvals, TmpR, jrank(jvar));
+	eigvals = gretl_general_matrix_eigenvals(M, 1, &err);
+	if (!err) {
+	    err = gretl_eigen_sort(eigvals, M, jrank(jvar));
 	}
     }
 
