@@ -80,7 +80,7 @@ static int resids_to_E (gretl_matrix *E, MODEL *lmod, int *reglist,
 
 	/* regress the given endogenous var on the specified
 	   set of exogenous vars */
-	*lmod = lsq(reglist, pZ, pdinfo, OLS, OPT_NONE, 0.0);
+	*lmod = lsq(reglist, pZ, pdinfo, OLS, OPT_NONE);
 	if ((err = lmod->errcode)) {
 	    clear_model(lmod);
 	    break;
@@ -325,9 +325,8 @@ static int liml_do_equation (gretl_equation_system *sys, int eq,
     err = gretl_matrix_multiply(Inv, W0, W2);
     if (err) goto bailout;
     
-    lambda = gretl_general_matrix_eigenvals(W2, NULL);
-    if (lambda == NULL) {
-	err = 1;
+    lambda = gretl_general_matrix_eigenvals(W2, 0, &err);
+    if (err) {
 	goto bailout;
     }
 
@@ -346,11 +345,11 @@ static int liml_do_equation (gretl_equation_system *sys, int eq,
     /* compute and set log-likelihood, etc */
     ll = system_n_equations(sys) * LN_2_PI;
     ll += log(lmin);
-    ll += gretl_matrix_log_determinant(W1);
+    ll += gretl_matrix_log_determinant(W1, &err);
     ll *= -(T / 2.0);
     pmod->lnL = ll;
 
-    mle_aic_bic(pmod, 0); /* check the "0" (additional params) here */
+    mle_criteria(pmod, 0); /* check the "0" (additional params) here */
 
  bailout:
 
