@@ -122,7 +122,7 @@ transcribe_poisson_results (MODEL *targ, MODEL *src, const double *y,
     fprintf(stderr, "log-likelihood = %g\n", targ->lnL);
 #endif
 
-    mle_aic_bic(targ, 0); 
+    mle_criteria(targ, 0); 
 
     /* mask invalid statistics */
     targ->rsq = NADBL;
@@ -269,7 +269,7 @@ do_poisson (MODEL *pmod, int offvar, double ***pZ, DATAINFO *pdinfo, PRN *prn)
 
 	iter++;
 
-	tmpmod = lsq(local_list, pZ, pdinfo, WLS, OPT_A, 0.0);
+	tmpmod = lsq(local_list, pZ, pdinfo, WLS, OPT_A);
 
 	if (tmpmod.errcode) {
 	    fprintf(stderr, "poisson_estimate: lsq returned %d\n", 
