@@ -129,7 +129,7 @@ make_sys_X_block (gretl_matrix *X, const MODEL *pmod,
 	    Xi = Z[pmod->list[i+2]];
 	}
 	if (Xi == NULL) {
-	    return 1;
+	    return E_DATA;
 	}
 	for (t=0; t<X->rows; t++) {
 	    gretl_matrix_set(X, t, i, Xi[t+t1]);
@@ -361,7 +361,6 @@ calculate_sys_coefficients (gretl_equation_system *sys,
 
     /* are we saving the coefficient vector and covariance matrix
        (e.g. as the basis for testing restrictions)? */
-
     if (system_save_vcv(sys)) {
 	gretl_matrix *b = gretl_matrix_copy(y);
 
@@ -776,11 +775,9 @@ augment_X_with_restrictions (gretl_matrix *X, int mk,
     nc = sys->R->cols;
 
     /* place the R matrix */
-
     kronecker_place(X, sys->R, mk, 0, 1.0);
 
     /* place R-transpose */
-
     for (i=0; i<nr; i++) {
 	for (j=0; j<nc; j++) {
 	    rij = gretl_matrix_get(sys->R, i, j);
@@ -789,7 +786,6 @@ augment_X_with_restrictions (gretl_matrix *X, int mk,
     }
 
     /* zero the bottom right-hand block */
-
     for (i=mk; i<mk+nr; i++) {
 	for (j=mk; j<mk+nr; j++) {
 	    gretl_matrix_set(X, i, j, 0.0);
@@ -843,7 +839,6 @@ static int converged (gretl_equation_system *sys,
 #if SDEBUG
 	printf("SUR iteration %d, ll = %.8g\n", sys->iters, ll);
 #endif
-
 	if (crit <= tol) {
 	    met = 1;
 	} else if (sys->iters < SYS_MAX_ITER) {
@@ -856,7 +851,6 @@ static int converged (gretl_equation_system *sys,
 #if SDEBUG
 	printf("3SLS iteration %d, crit = %.8g\n", sys->iters, crit);
 #endif
-
 	if (crit <= tol) {
 	    met = 1;
 	} 
@@ -891,6 +885,35 @@ static void clean_up_models (gretl_equation_system *sys)
     sys->ess = ess;
 }
 
+static int drop_redundant_instruments (gretl_equation_system *sys,
+				       const int *droplist, int i)
+{
+    int j, k, pos, err = 0;
+
+    for (j=1; j<=droplist[0]; j++) {
+	pos = gretl_list_position(droplist[j], sys->instr_vars);
+	if (pos > 0) {
+	    gretl_list_delete_at_pos(sys->instr_vars, pos);
+	} else {
+	    err = 1;
+	}
+    }
+
+    pos = gretl_list_separator_position(sys->lists[i]);
+    if (pos > 0) {
+	for (j=1; j<=droplist[0]; j++) {
+	    for (k=pos+1; k<=sys->lists[i][0]; k++) {
+		if (sys->lists[i][k] == droplist[j]) {
+		    gretl_list_delete_at_pos(sys->lists[i], k);
+		    break;
+		}
+	    }
+	}
+    }
+
+    return err;
+}
+
 /* general function that forms the basis for all specific system
    estimators */
 
@@ -981,6 +1004,7 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     for (i=0; i<m; i++) {
 	int freeit = 0;
 	int *list = system_model_list(sys, i, &freeit);
+	const int *droplist = NULL;
 	MODEL *pmod = models[i];
 
 	if (list == NULL) {
@@ -988,8 +1012,11 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	    break;
 	}
 
+	/* FIXME: should we try to handle the case of redundant instruments
+	   in this context? */
+
 	if (method == SYS_SUR || method == SYS_OLS || method == SYS_WLS) {
-	    *pmod = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+	    *pmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
 	} else if (method == SYS_3SLS || method == SYS_FIML || 
 		   method == SYS_TSLS) {
 	    *pmod = tsls_func(list, SYSTEM, pZ, pdinfo, OPT_NONE);
@@ -1002,9 +1029,14 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	if ((err = pmod->errcode)) {
-	    fprintf(stderr, "model failed on lists[%d], code=%d\n",
-		    i, err);
+	    fprintf(stderr, "system_estimate: failed to estimate equation %d: "
+		    "err = %d\n", i+1, err);
 	    break;
+	} 
+
+	droplist = gretl_model_get_data(pmod, "inst_droplist");
+	if (droplist != NULL) {
+	    drop_redundant_instruments(sys, droplist, i);
 	}
 
 	pmod->ID = i;
@@ -1023,10 +1055,13 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (err) goto cleanup;
+    if (err) {
+	fprintf(stderr, "system_estimate: after initial tsls, err = %d", err);
+	goto cleanup;
+    }
 
     if (method == SYS_LIML) {
-	/* compute the minimum eigenvalues and generated the
+	/* compute the minimum eigenvalues and generate the
 	   suitably transformed data matrices */
 	err = liml_driver(sys, pZ, pdinfo, prn);
 	if (err) goto cleanup;
@@ -1127,7 +1162,10 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	krow += models[i]->ncoeff;
     }
 
-    if (err) goto cleanup;
+    if (err) {
+	fprintf(stderr, "after trying to make X matrix: err = %d\n", err);
+	goto cleanup;
+    }
 
     if (nr > 0) {
 	/* there are cross-equation restrictions to be imposed */
