@@ -23,10 +23,11 @@
 
 #include "libgretl.h"
 #include "bhhh_max.h"
+#include "libset.h"
 
 #undef TDEBUG 
 
-#define TOBIT_TOL 1.0e-10 /* was 1.0e-9: do more checking? */
+#define TOBIT_TOL 1.0e-10 /* calibrated against William Greene */
 
 /* Below: we are buying ourselves a considerable simplification when it comes
    to the tobit_ll function.  That function needs access to the original y
@@ -295,7 +296,7 @@ static int write_tobit_stats (MODEL *pmod, double *theta, int ncoeff,
 
     pmod->fstt = pmod->rsq = pmod->adjrsq = NADBL;
 
-    mle_aic_bic(pmod, 1);
+    mle_criteria(pmod, 1);
 
     make_vcv(pmod, VCV, scale);
 
@@ -308,11 +309,16 @@ static int write_tobit_stats (MODEL *pmod, double *theta, int ncoeff,
 }
 
 static model_info *
-tobit_model_info_init (int nobs, int k, int n_series)
+tobit_model_info_init (int model_obs, int bign, int k, int n_series)
 {
+    double tol = get_bhhh_toler();
     model_info *tobit;
 
-    tobit = model_info_new(k, 0, nobs - 1, TOBIT_TOL);
+    if (na(tol)) {
+	tol = TOBIT_TOL;
+    }
+
+    tobit = model_info_new(k, 0, model_obs - 1, bign, tol);
 
     if (tobit != NULL) {
 	model_info_set_opts(tobit, FULL_VCV_MATRIX);
@@ -361,7 +367,7 @@ static int do_tobit (const double **Z, DATAINFO *pdinfo, MODEL *pmod,
     /* initialization of variance */
     pmod->coeff[k-1] = 1.0;
 
-    tobit = tobit_model_info_init(pmod->nobs, k, n_series);
+    tobit = tobit_model_info_init(pmod->nobs, pdinfo->n, k, n_series);
     if (tobit == NULL) {
 	err = E_ALLOC;
 	goto bailout;
@@ -477,7 +483,7 @@ MODEL tobit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
     int t;
 
     /* run initial OLS: OPT_M bans missing obs */
-    model = lsq(list, pZ, pdinfo, OLS, OPT_A | OPT_M, 0.0);
+    model = lsq(list, pZ, pdinfo, OLS, OPT_A | OPT_M);
     if (model.errcode) {
 	return model;
     }
@@ -489,7 +495,7 @@ MODEL tobit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
 	    (*pZ)[model.list[1]][t] *= scale;
 	}
 	clear_model(&model);
-	model = lsq(list, pZ, pdinfo, OLS, OPT_A | OPT_M, 0.0);
+	model = lsq(list, pZ, pdinfo, OLS, OPT_A | OPT_M);
     }
 
 #ifdef TDEBUG
