@@ -781,7 +781,7 @@ int run_gretl_comparison (const char *datname,
 
     model = gretl_model_new();
 
-    *model = lsq(list, pZ, dinfo, OLS, OPT_D | OPT_Z, 0.0);
+    *model = lsq(list, pZ, dinfo, OLS, OPT_D | OPT_Z);
 
     if (model->errcode) {
 	if (verbose) pputc(prn, '\n');
